@@ -1,6 +1,6 @@
 # -*- sh-basic-offset: 2 -*-
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -80,6 +80,49 @@ find_header () {
 };
 
 
+has_valid_code_signature () {
+  local p="$1"; shift;
+  if [ -f ${p} ] && [ -x ${p} ]; then
+    if /usr/bin/codesign -v ${p} > /dev/null 2>&1; then
+      return 0;
+    else
+      return 1;
+    fi;
+  fi;
+};
+
+
+ad_hoc_sign_if_code_signature_is_invalid () {
+  local p="$1"; shift;
+  if [ -f ${p} ] && [ -x ${p} ]; then
+    if has_valid_code_signature ${p}; then
+      # If there is a valid code signature, we're done.
+      echo "${p} has a valid code signature.";
+      return 0;
+    else
+      # If codesign exits non-zero, that could mean either 'no signature' or
+      # 'invalid signature'. We need to determine which one.
+      # An unsigned binary has no LC_CODE_SIGNATURE load command.
+      if /usr/bin/otool -l ${p} | /usr/bin/grep LC_CODE_SIGNATURE > /dev/null 2>&1; then
+        # Has LC_CODE_SIGNATURE, but signature isn't valid. Ad-hoc sign it.
+        echo "${p} has an invalid code signature, attempting to repair..."
+        /usr/bin/codesign -f -s - ${p};
+        # Validate the ad-hoc signature
+        if has_valid_code_signature ${p}; then
+          echo "Invalid code signature replaced via ad-hoc signing.";
+          return 0;
+        fi;
+      else
+        # No code signature.
+        echo "${p} is not code signed. This is OK.";
+        return 0;
+      fi;
+    fi;
+  fi;
+  return 1;
+};
+
+
 # Initialize all the global state required to use this library.
 init_build () {
   cd "${wd}";
@@ -92,7 +135,7 @@ init_build () {
   conditional_set do_get "true";
   conditional_set do_setup "true";
   conditional_set force_setup "false";
-  conditional_set requirements "${wd}/requirements-dev.txt"
+  conditional_set virtualenv_opts "";
 
       dev_home="${wd}/.develop";
      dev_roots="${dev_home}/roots";
@@ -113,6 +156,9 @@ init_build () {
 
   project="$(setup_print name)" || project="<unknown>";
 
+  dev_patches="${dev_home}/patches";
+  patches="${wd}/lib-patches";
+
   # Find some hashing commands
   # sha1() = sha1 hash, if available
   # md5()  = md5 hash, if available
@@ -155,6 +201,25 @@ init_build () {
   else
     hash () { echo "INTERNAL ERROR: No hash function."; exit 1; }
   fi;
+
+  default_requirements="${wd}/requirements-default.txt";
+  use_openssl="true";
+
+  # Use SecureTransport instead of OpenSSL if possible, unless told otherwise
+  if [ -z "${USE_OPENSSL-}" ]; then 
+    if [ $(uname -s) = "Darwin" ]; then
+      # SecureTransport requires >= 10.11 (Darwin >= 15).
+      REV=$(uname -r)
+      if (( ${REV%%.*} >= 15 )); then
+        default_requirements="${wd}/requirements-osx.txt";
+        use_openssl="false";
+      else
+        # Needed to build OpenSSL 64-bit on OS X
+        export KERNEL_BITS=64;
+      fi;
+    fi;
+  fi;  
+  conditional_set requirements "${default_requirements}"
 }
 
 
@@ -169,6 +234,34 @@ EOF
 }
 
 
+# Apply patches from lib-patches to the given dependency codebase.
+apply_patches () {
+  local name="$1"; shift;
+  local path="$1"; shift;
+
+  if [ -d "${patches}/${name}" ]; then
+    echo "";
+    echo "Applying patches to ${name} in ${path}...";
+
+    cd "${path}";
+    find "${patches}/${name}"                  \
+        -type f                                \
+        -name '*.patch'                        \
+        -print                                 \
+        -exec patch -p0 --forward -i '{}' ';';
+    cd /;
+
+  fi;
+
+  echo "";
+  if [ -e "${path}/setup.py" ]; then
+    echo "Removing build directory ${path}/build...";
+    rm -rf "${path}/build";
+    echo "Removing pyc files from ${path}...";
+    find "${path}" -type f -name '*.pyc' -print0 | xargs -0 rm -f;
+  fi;
+}
+
 # If do_get is turned on, get an archive file containing a dependency via HTTP.
 www_get () {
   if ! "${do_get}"; then return 0; fi;
@@ -249,47 +342,21 @@ www_get () {
 
         echo "Downloading ${name}...";
 
-        local pkg_host="static.calendarserver.org";
-        local pkg_path="/pkg";
-
         #
-        # Try getting a copy from calendarserver.org.
+        # Try getting a copy from the upstream source.
         #
         local tmp="$(mktemp "/tmp/${cache_basename}.XXXXXX")";
-        curl -L "http://${pkg_host}${pkg_path}/${cache_basename}" -o "${tmp}" || true;
+        curl -L "${url}" -o "${tmp}";
         echo "";
+
         if [ ! -s "${tmp}" ] || grep '<title>404 Not Found</title>' "${tmp}" > /dev/null; then
           rm -f "${tmp}";
-          echo "${name} is not available from calendarserver.org; trying upstream source.";
+          echo "${name} is not available from upstream source: ${url}";
+          exit 1;
         elif ! check_hash "${tmp}"; then
           rm -f "${tmp}";
-          echo "${name} from calendarserver.org is invalid; trying upstream source.";
-        fi;
-
-        #
-        # That didn't work. Try getting a copy from the upstream source.
-        #
-        if [ ! -f "${tmp}" ]; then
-          curl -L "${url}" -o "${tmp}";
-          echo "";
-
-          if [ ! -s "${tmp}" ] || grep '<title>404 Not Found</title>' "${tmp}" > /dev/null; then
-            rm -f "${tmp}";
-            echo "${name} is not available from upstream source: ${url}";
-            exit 1;
-          elif ! check_hash "${tmp}"; then
-            rm -f "${tmp}";
-            echo "${name} from upstream source is invalid: ${url}";
-            exit 1;
-          fi;
-
-          if egrep "^${pkg_host}" "${HOME}/.ssh/known_hosts" > /dev/null 2>&1; then
-            echo "Copying cache file up to ${pkg_host}.";
-            if ! scp "${tmp}" "${pkg_host}:/var/www/static${pkg_path}/${cache_basename}"; then
-              echo "Failed to copy cache file up to ${pkg_host}.";
-            fi;
-            echo ""
-          fi;
+          echo "${name} from upstream source is invalid: ${url}";
+          exit 1;
         fi;
 
         #
@@ -315,6 +382,7 @@ www_get () {
     rm -rf "${path}";
     cd "$(dirname "${path}")";
     get | ${decompress} | ${unpack};
+    apply_patches "${name}" "${path}";
     cd "${wd}";
   fi;
 }
@@ -442,27 +510,29 @@ c_dependencies () {
 
   # The OpenSSL version number is special. Our strategy is to get the integer
   # value of OPENSSL_VERSION_NUBMER for use in inequality comparison.
-  ruler;
+  if [ "${use_openssl}" = "true" ]; then
+    ruler;
 
-  local min_ssl_version="9470463";  # OpenSSL 0.9.8zf
+    local min_ssl_version="268443791";  # OpenSSL 1.0.2h
 
-  local ssl_version="$(c_macro openssl/ssl.h OPENSSL_VERSION_NUMBER)";
-  if [ -z "${ssl_version}" ]; then ssl_version="0x0"; fi;
-  ssl_version="$("${bootstrap_python}" -c "print ${ssl_version}")";
+    local ssl_version="$(c_macro openssl/ssl.h OPENSSL_VERSION_NUMBER)";
+    if [ -z "${ssl_version}" ]; then ssl_version="0x0"; fi;
+    ssl_version="$("${bootstrap_python}" -c "print ${ssl_version}")";
 
-  if [ "${ssl_version}" -ge "${min_ssl_version}" ]; then
-    using_system "OpenSSL";
-  else
-    local v="0.9.8zf";
-    local n="openssl";
-    local p="${n}-${v}";
+    if [ "${ssl_version}" -ge "${min_ssl_version}" ]; then
+      using_system "OpenSSL";
+    else
+      local v="1.0.2h";
+      local n="openssl";
+      local p="${n}-${v}";
 
-    # use 'config' instead of 'configure'; 'make' instead of 'jmake'.
-    # also pass 'shared' to config to build shared libs.
-    c_dependency -c "config" -m "c69a4a679233f7df189e1ad6659511ec" \
-      -p "make depend" -b "make" \
-      "openssl" "${p}" \
-      "http://www.openssl.org/source/${p}.tar.gz" "shared";
+      # use 'config' instead of 'configure'; 'make' instead of 'jmake'.
+      # also pass 'shared' to config to build shared libs.
+      c_dependency -c "config" -s "577585f5f5d299c44dd3c993d3c0ac7a219e4949" \
+        -p "make depend" -b "make" \
+        "openssl" "${p}" \
+        "http://www.openssl.org/source/${p}.tar.gz" "shared";
+    fi;
   fi;
 
 
@@ -476,11 +546,11 @@ c_dependencies () {
       using_system "libffi";
     fi;
   else
-    local v="3.0.13";
+    local v="3.2.1";
     local n="libffi";
     local p="${n}-${v}";
 
-    c_dependency -m "45f3b6dbc9ee7c7dfbbbc5feba571529" \
+    c_dependency -m "83b89587607e3eb65c70d361f13bab43" \
       "libffi" "${p}" \
       "ftp://sourceware.org/pub/libffi/${p}.tar.gz"
   fi;
@@ -530,17 +600,24 @@ c_dependencies () {
     local n="libevent";
     local p="${n}-${v}";
 
+	local configure_openssl="--enable-openssl=yes";
+    if [ "${use_openssl}" = "false" ]; then
+	  local configure_openssl="--enable-openssl=no";
+	fi;
+
     c_dependency -m "b2405cc9ebf264aa47ff615d9de527a2" \
       "libevent" "${p}" \
-      "http://github.com/downloads/libevent/libevent/${p}.tar.gz";
+      "https://github.com/downloads/libevent/libevent/${p}.tar.gz" \
+      ${configure_openssl};
 
-    local v="1.4.20";
+    local v="1.4.24";
     local n="memcached";
     local p="${n}-${v}";
 
-    c_dependency -m "92f702bcb28d7bec8fdf9418360fc062" \
+    c_dependency -s "32a798a37ef782da10a09d74aa1e5be91f2861db" \
       "memcached" "${p}" \
-      "http://www.memcached.org/files/${p}.tar.gz";
+      "http://www.memcached.org/files/${p}.tar.gz" \
+      "--disable-docs";
   fi;
 
 
@@ -548,7 +625,7 @@ c_dependencies () {
   if command -v postgres > /dev/null; then
     using_system "Postgres";
   else
-    local v="9.3.1";
+    local v="9.5.3";
     local n="postgresql";
     local p="${n}-${v}";
 
@@ -558,14 +635,32 @@ c_dependencies () {
       local enable_dtrace="";
     fi;
 
-    c_dependency -m "c003d871f712d4d3895956b028a96e74" \
+    c_dependency -m "3f0c388566c688c82b01a0edf1e6b7a0" \
       "PostgreSQL" "${p}" \
       "http://ftp.postgresql.org/pub/source/v${v}/${p}.tar.bz2" \
-      --with-python ${enable_dtrace};
+      ${enable_dtrace};
   fi;
 
 }
 
+#
+# Special cx_Oracle patch handling
+#
+cx_Oracle_patch() {
+
+  local f_hash="-m 6a49e1aa0e5b48589f8edfe5884ff5a5";
+  local v="5.2";
+  local n="cx_Oracle";
+  local p="${n}-${v}";
+
+  local srcdir="${dev_patches}/${p}";
+
+  www_get ${f_hash} "${n}" "${srcdir}" "https://pypi.python.org/packages/source/c/${n}/${p}.tar.gz";
+  cd "${dev_patches}";
+  tar zcf "${p}.tar.gz" "${p}";
+  cd "${wd}";
+  rm -rf "${srcdir}";
+}
 
 #
 # Build Python dependencies
@@ -578,6 +673,7 @@ py_dependencies () {
   export PYTHON="${python}";
   export PYTHONPATH="${py_ve_tools}/lib:${wd}:${PYTHONPATH:-}";
 
+  mkdir -p "${dev_patches}";
 
   # Work around a change in Xcode tools that breaks Python modules in OS X
   # 10.9.2 and prior due to a hard error if the -mno-fused-madd is used, as
@@ -603,7 +699,21 @@ py_dependencies () {
     "${bootstrap_python}" -m virtualenv  \
       --system-site-packages             \
       --no-setuptools                    \
+      ${virtualenv_opts}                 \
       "${py_virtualenv}";
+    if [ "${use_openssl}" = "false" ]; then
+      # Interacting with keychain requires either a valid code signature, or no
+      # code signature. An *invalid* code signature won't work.
+      if ! ad_hoc_sign_if_code_signature_is_invalid ${python}; then
+        cat << EOF
+SecureTransport support is enabled, but we are unable to validate or fix the
+code signature for ${python}. Keychain interactions used with SecureTransport
+require either no code signature, or a valid code signature. To switch to
+OpenSSL, delete the .develop directory, export USE_OPENSSL=1, then run
+./bin/develop again.
+EOF
+      fi;
+    fi;
   fi;
 
   cd "${wd}";
@@ -622,51 +732,60 @@ py_dependencies () {
   echo "";
   "${pip_install}" --requirement="${requirements}";
 
-  for option in $("${bootstrap_python}" -c 'import setup; print "\n".join(setup.extras_requirements.keys())'); do
-    ruler "Preparing Python requirements for optional feature: ${option}";
+  for extra in $("${bootstrap_python}" -c 'import setup; print "\n".join(setup.extras_requirements.keys())'); do
+    ruler "Preparing Python requirements for optional feature: ${extra}";
     echo "";
-    if ! "${pip_install}" --editable="${wd}[${option}]"; then
-      echo "Feature ${option} is optional; continuing.";
+
+    if [ "${extra}" = "oracle" ]; then
+      cx_Oracle_patch;
+    fi;
+
+    if ! "${pip_install}" --editable="${wd}[${extra}]"; then
+      echo "Feature ${extra} is optional; continuing.";
     fi;
   done;
 
+  ruler "Preparing Python requirements for patching";
+  echo "";
+  "${pip_install}" --ignore-installed --no-deps --requirement="${wd}/requirements-ignore-installed.txt";
+
+  ruler "Patching Python requirements";
+  echo "";
+  twisted_version=$("${python}" -c 'from twisted import version; print version.base()');
+  if [ ! -e "${py_virtualenv}/lib/python2.7/site-packages/twisted/.patch_applied.${twisted_version}" ]; then
+    apply_patches "Twisted" "${py_virtualenv}/lib/python2.7/site-packages"
+    find "${py_virtualenv}/lib/python2.7/site-packages/twisted" -type f -name '.patch_applied*' -print0 | xargs -0 rm -f;
+    touch "${py_virtualenv}/lib/python2.7/site-packages/twisted/.patch_applied.${twisted_version}";
+  fi;
+
   echo "";
 }
 
 
+macos_oracle () {
+  if [ "${ORACLE_HOME-}" ]; then
+    case "$(uname -s)" in
+      Darwin)
+        echo "macOS Oracle init."
+        export   LD_LIBRARY_PATH="${LD_LIBRARY_PATH:-}:${ORACLE_HOME}";
+        export DYLD_LIBRARY_PATH="${DYLD_LIBRARY_PATH:-}:${ORACLE_HOME}";
+        ;;
+    esac;
+  fi;
+}
+
+
 bootstrap_virtualenv () {
   mkdir -p "${py_ve_tools}";
-  mkdir -p "${py_ve_tools}/lib";
-  mkdir -p "${py_ve_tools}/junk";
+  export PYTHONUSERBASE="${py_ve_tools}"
 
   for pkg in             \
-      setuptools-17.0    \
-      pip-7.0.3          \
-      virtualenv-13.0.3  \
+      setuptools==18.5    \
+      pip==8.1.2          \
+      virtualenv==15.0.2  \
   ; do
-      local    name="${pkg%-*}";
-      local version="${pkg#*-}";
-      local  first="$(echo "${name}" | sed 's|^\(.\).*$|\1|')";
-      local    url="https://pypi.python.org/packages/source/${first}/${name}/${pkg}.tar.gz";
-
-      ruler "Downloading ${pkg}";
-
-      local tmp="$(mktemp -d -t ccsXXXXX)";
-
-      curl -L "${url}" | tar -C "${tmp}" -xvzf -;
-
-      cd "${tmp}/$(basename "${pkg}")";
-      PYTHONPATH="${py_ve_tools}/lib"                \
-        "${bootstrap_python}" setup.py install       \
-            --install-base="${py_ve_tools}"          \
-            --install-lib="${py_ve_tools}/lib"       \
-            --install-headers="${py_ve_tools}/junk"  \
-            --install-scripts="${py_ve_tools}/junk"  \
-            --install-data="${py_ve_tools}/junk"     \
-            ;                                        \
-      cd "${wd}";
-
-      rm -rf "${tmp}";
+      ruler "Installing ${pkg}";
+      "${bootstrap_python}" -m pip install -I --user "${pkg}";
   done;
 }
 
@@ -674,10 +793,10 @@ bootstrap_virtualenv () {
 pip_download () {
   mkdir -p "${dev_home}/pip_downloads";
 
-  "${python}" -m pip install               \
+  "${python}" -m pip download               \
     --disable-pip-version-check            \
-    --download="${dev_home}/pip_downloads" \
-    --pre --allow-all-external             \
+    -d "${dev_home}/pip_downloads" \
+    --pre                                  \
     --no-cache-dir                         \
     --log-file="${dev_home}/pip.log"       \
     "$@";
@@ -687,9 +806,10 @@ pip_download () {
 pip_install_from_cache () {
   "${python}" -m pip install                 \
     --disable-pip-version-check              \
-    --pre --allow-all-external               \
+    --pre                                    \
     --no-index                               \
     --no-cache-dir                           \
+    --find-links="${dev_patches}"            \
     --find-links="${dev_home}/pip_downloads" \
     --log-file="${dev_home}/pip.log"         \
     "$@";
@@ -699,8 +819,9 @@ pip_install_from_cache () {
 pip_download_and_install () {
   "${python}" -m pip install                 \
     --disable-pip-version-check              \
-    --pre --allow-all-external               \
+    --pre                                    \
     --no-cache-dir                           \
+    --find-links="${dev_patches}"            \
     --log-file="${dev_home}/pip.log"         \
     "$@";
 }
@@ -713,6 +834,7 @@ develop () {
   init_build;
   c_dependencies;
   py_dependencies;
+  macos_oracle;
 }
 
 
