@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,24 +36,23 @@ except ImportError:
 from sys import platform
 import time
 
-from calendarserver.logAnalysis import getAdjustedMethodName, \
-    getAdjustedClientName
+from calendarserver.logAnalysis import getAdjustedMethodName
 
 from twext.python.log import Logger
-from twext.web2 import iweb
-from twext.web2.log import BaseCommonAccessLoggingObserver
-from twext.web2.log import LogWrapperResource
+from twext.who.idirectory import RecordType
+
+from txweb2 import iweb
+from txweb2.log import BaseCommonAccessLoggingObserver
+from txweb2.log import LogWrapperResource
 
 from twisted.internet import protocol, task
 from twisted.protocols import amp
 
 from twistedcaldav.config import config
-from twistedcaldav.directory.directory import DirectoryService
-
-from txdav.xml import element as davxml
 
 log = Logger()
 
+
 class DirectoryLogWrapperResource(LogWrapperResource):
 
     def __init__(self, resource, directory):
@@ -61,12 +60,10 @@ class DirectoryLogWrapperResource(LogWrapperResource):
 
         self.directory = directory
 
-
     def getDirectory(self):
         return self.directory
 
 
-
 class CommonAccessLoggingObserverExtensions(BaseCommonAccessLoggingObserver):
     """
     A base class for our extension to the L{BaseCommonAccessLoggingObserver}
@@ -84,33 +81,20 @@ class CommonAccessLoggingObserverExtensions(BaseCommonAccessLoggingObserver):
 
             # Try to determine authentication and authorization identifiers
             uid = "-"
-            if hasattr(request, "authnUser"):
-                if isinstance(request.authnUser.children[0], davxml.HRef):
-                    uidn = str(request.authnUser.children[0])
-                    uidz = None
-                    if hasattr(request, "authzUser") and str(request.authzUser.children[0]) != uidn:
-                        uidz = str(request.authzUser.children[0])
-
-                    def convertUIDtoShortName(uid):
-                        uid = uid.rstrip("/")
-                        uid = uid[uid.rfind("/") + 1:]
-                        record = request.site.resource.getDirectory().recordWithUID(uid)
-                        if record:
-                            if record.recordType == DirectoryService.recordType_users:
-                                return record.shortNames[0]
-                            else:
-                                return "(%s)%s" % (record.recordType, record.shortNames[0],)
-                        else:
-                            return uid
-
-                    uidn = convertUIDtoShortName(uidn)
-                    if uidz:
-                        uidz = convertUIDtoShortName(uidz)
-
-                    if uidn and uidz:
-                        uid = '"%s as %s"' % (uidn, uidz,)
+            if getattr(request, "authnUser", None) is not None:
+                def convertPrincipaltoShortName(principal):
+                    if principal.record.recordType == RecordType.user:
+                        return principal.record.shortNames[0]
                     else:
-                        uid = uidn
+                        return "({rtype}){name}".format(rtype=principal.record.recordType, name=principal.record.shortNames[0],)
+
+                uidn = convertPrincipaltoShortName(request.authnUser)
+                uidz = convertPrincipaltoShortName(request.authzUser)
+
+                if uidn != uidz:
+                    uid = '"{authn} as {authz}"'.format(authn=uidn, authz=uidz,)
+                else:
+                    uid = uidn
 
             #
             # For some methods which basically allow you to tunnel a
@@ -134,16 +118,16 @@ class CommonAccessLoggingObserverExtensions(BaseCommonAccessLoggingObserver):
             )
 
             formatArgs = {
-                "host"                : request.remoteAddr.host,
-                "uid"                 : uid,
-                "date"                : self.logDateString(response.headers.getHeader("date", 0)),
-                "method"              : method,
-                "uri"                 : request.uri.replace('"', "%22"),
-                "protocolVersion"     : ".".join(str(x) for x in request.clientproto),
-                "statusCode"          : response.code,
-                "bytesSent"           : loginfo.bytesSent,
-                "referer"             : request.headers.getHeader("referer", "-"),
-                "userAgent"           : request.headers.getHeader("user-agent", "-"),
+                "host": request.remoteAddr.host,
+                "uid": uid,
+                "date": self.logDateString(response.headers.getHeader("date", 0)),
+                "method": method,
+                "uri": request.uri.replace('"', "%22"),
+                "protocolVersion": ".".join(str(x) for x in request.clientproto),
+                "statusCode": response.code,
+                "bytesSent": loginfo.bytesSent,
+                "referer": request.headers.getHeader("referer", "-"),
+                "userAgent": request.headers.getHeader("user-agent", "-"),
             }
 
             # Add extended items to format and formatArgs
@@ -151,8 +135,9 @@ class CommonAccessLoggingObserverExtensions(BaseCommonAccessLoggingObserver):
                 format += ' i=%(serverInstance)s'
                 formatArgs["serverInstance"] = config.LogID if config.LogID else "0"
 
-                format += ' or=%(outstandingRequests)s'
-                formatArgs["outstandingRequests"] = request.chanRequest.channel.factory.outstandingRequests
+                if request.chanRequest:  # This can be None during tests
+                    format += ' or=%(outstandingRequests)s'
+                    formatArgs["outstandingRequests"] = request.chanRequest.channel.factory.outstandingRequests
 
                 # Tags for time stamps collected along the way - the first one in the list is the initial
                 # time for request creation - we use that to track the entire request/response time
@@ -195,6 +180,13 @@ class CommonAccessLoggingObserverExtensions(BaseCommonAccessLoggingObserver):
                     format += " fwd=%(fwd)s"
                     formatArgs["fwd"] = forwardedFor
 
+            if formatArgs["host"] == "0.0.0.0":
+                fwdHeaders = request.headers.getRawHeaders("x-forwarded-for", "")
+                if fwdHeaders:
+                    formatArgs["host"] = fwdHeaders[-1].split(",")[-1].strip()
+                    format += " unix=%(unix)s"
+                    formatArgs["unix"] = "true"
+
         elif "overloaded" in eventDict:
             overloaded = eventDict.get("overloaded")
 
@@ -206,16 +198,16 @@ class CommonAccessLoggingObserverExtensions(BaseCommonAccessLoggingObserver):
             )
 
             formatArgs = {
-                "host"                : overloaded.transport.hostname,
-                "uid"                 : "-",
-                "date"                : self.logDateString(time.time()),
-                "method"              : "???",
-                "uri"                 : "",
-                "protocolVersion"     : "",
-                "statusCode"          : 503,
-                "bytesSent"           : 0,
-                "referer"             : "-",
-                "userAgent"           : "-",
+                "host": overloaded.transport.hostname,
+                "uid": "-",
+                "date": self.logDateString(time.time()),
+                "method": "???",
+                "uri": "",
+                "protocolVersion": "",
+                "statusCode": 503,
+                "bytesSent": 0,
+                "referer": "-",
+                "userAgent": "-",
             }
 
             if config.EnableExtendedAccessLog:
@@ -241,7 +233,6 @@ class CommonAccessLoggingObserverExtensions(BaseCommonAccessLoggingObserver):
             self.logStats(formatArgs)
 
 
-
 class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
     """
     Class to do "apache" style access logging to a rotating log file. The log
@@ -255,7 +246,9 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
 
         self.systemStats = None
         self.statsByMinute = []
-
+        self.stats1m = None
+        self.stats5m = None
+        self.stats1h = None
 
     def accessLog(self, message, allowrotate=True):
         """
@@ -269,9 +262,10 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         if self.shouldRotate() and allowrotate:
             self.flush()
             self.rotate()
+        if isinstance(message, unicode):
+            message = message.encode("utf-8")
         self.f.write(message + "\n")
 
-
     def start(self):
         """
         Start logging. Open the log file and log an "open" message.
@@ -281,7 +275,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         self._open()
         self.accessLog("Log opened - server start: [%s]." % (datetime.datetime.now().ctime(),))
 
-
     def stop(self):
         """
         Stop logging. Close the log file and log an "open" message.
@@ -294,7 +287,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         if self.systemStats is not None:
             self.systemStats.stop()
 
-
     def _open(self):
         """
         Open the log file.
@@ -303,7 +295,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         self.f = open(self.logpath, "a", 1)
         self.lastDate = self.toDate(os.stat(self.logpath)[8])
 
-
     def _close(self):
         """
         Close the log file.
@@ -311,7 +302,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
 
         self.f.close()
 
-
     def flush(self):
         """
         Flush the log file.
@@ -319,7 +309,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
 
         self.f.flush()
 
-
     def shouldRotate(self):
         """
         Rotate when the date has changed since last write
@@ -330,7 +319,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         else:
             return False
 
-
     def toDate(self, *args):
         """
         Convert a unixtime to (year, month, day) localtime tuple,
@@ -343,7 +331,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         # primarily so this can be unit tested easily
         return time.localtime(*args)[:3]
 
-
     def suffix(self, tupledate):
         """
         Return the suffix given a (year, month, day) tuple or unixtime
@@ -355,7 +342,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
             # try taking a float unixtime
             return "_".join(map(str, self.toDate(tupledate)))
 
-
     def rotate(self):
         """
         Rotate the file and create a new one.
@@ -366,40 +352,47 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
 
         newpath = "%s.%s" % (self.logpath, self.suffix(self.lastDate))
         if os.path.exists(newpath):
-            log.info("Cannot rotate log file to %s because it already exists." % (newpath,))
+            log.info("Cannot rotate log file to '{path}' because it already exists.", path=newpath)
             return
         self.accessLog("Log closed - rotating: [%s]." % (datetime.datetime.now().ctime(),), False)
-        log.info("Rotating log file to: %s" % (newpath,), system="Logging")
+        log.info("Rotating log file to: '{path}'", path=newpath, system="Logging")
         self.f.close()
         os.rename(self.logpath, newpath)
         self._open()
         self.accessLog("Log opened - rotated: [%s]." % (datetime.datetime.now().ctime(),), False)
 
-
     def logStats(self, stats):
         """
         Update stats
         """
 
-        if self.systemStats is None:
-            self.systemStats = SystemMonitor()
+        # Only use the L{SystemMonitor} when stats socket is in use
+        if config.Stats.EnableUnixStatsSocket or config.Stats.EnableTCPStatsSocket:
 
-        # Currently only storing stats for access log type
-        if "type" not in stats or stats["type"] != "access-log":
-            return
+            # Initialize a L{SystemMonitor} on the first call
+            if self.systemStats is None:
+                self.systemStats = SystemMonitor()
 
-        currentStats = self.ensureSequentialStats()
-        self.updateStats(currentStats, stats)
+            # Currently only storing stats for access log type
+            if "type" not in stats or stats["type"] != "access-log":
+                return
+
+            currentStats = self.ensureSequentialStats()
+            self.updateStats(currentStats, stats)
 
         if stats["type"] == "access-log":
             self.accessLog(stats["log-format"] % stats)
 
-
     def getStats(self):
         """
         Return the stats
         """
 
+        # Only use the L{SystemMonitor} when stats socket is in use
+        if not config.Stats.EnableUnixStatsSocket and not config.Stats.EnableTCPStatsSocket:
+            return {}
+
+        # Initialize a L{SystemMonitor} on the first call
         if self.systemStats is None:
             self.systemStats = SystemMonitor()
 
@@ -407,35 +400,37 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         currentStats = self.ensureSequentialStats()
 
         # Get previous minute details
-        index = min(2, len(self.statsByMinute))
-        if index > 0:
-            previousMinute = self.statsByMinute[-index][1]
-        else:
-            previousMinute = self.initStats()
+        if self.stats1m is None:
+            index = min(2, len(self.statsByMinute))
+            if index > 0:
+                self.stats1m = self.statsByMinute[-index][1]
+            else:
+                self.stats1m = self.initStats()
 
         # Do five minute aggregate
-        fiveMinutes = self.initStats()
-        index = min(6, len(self.statsByMinute))
-        for i in range(-index, -1):
-            stat = self.statsByMinute[i][1]
-            self.mergeStats(fiveMinutes, stat)
+        if self.stats5m is None:
+            self.stats5m = self.initStats()
+            index = min(6, len(self.statsByMinute))
+            for i in range(-index, -1):
+                stat = self.statsByMinute[i][1]
+                self.mergeStats(self.stats5m, stat)
 
         # Do one hour aggregate
-        oneHour = self.initStats()
-        index = min(61, len(self.statsByMinute))
-        for i in range(-index, -1):
-            stat = self.statsByMinute[i][1]
-            self.mergeStats(oneHour, stat)
+        if self.stats1h is None:
+            self.stats1h = self.initStats()
+            index = min(61, len(self.statsByMinute))
+            for i in range(-index, -1):
+                stat = self.statsByMinute[i][1]
+                self.mergeStats(self.stats1h, stat)
 
         printStats = {
-            "System": self.systemStats.items,
-            "Current": currentStats,
-            "1 Minute": previousMinute,
-            "5 Minutes": fiveMinutes,
-            "1 Hour": oneHour,
+            "system": self.systemStats.items,
+            "current": currentStats,
+            "1m": self.stats1m,
+            "5m": self.stats5m,
+            "1h": self.stats1h,
         }
-        return json.dumps(printStats)
-
+        return printStats
 
     def ensureSequentialStats(self):
         """
@@ -446,63 +441,80 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         if len(self.statsByMinute) > 0:
             if self.statsByMinute[-1][0] != dtindex:
                 oldindex = self.statsByMinute[-1][0]
+                if oldindex != dtindex:
+                    # Adding a new minutes worth of data - clear out any cached
+                    # historical data
+                    self.stats1m = None
+                    self.stats5m = None
+                    self.stats1h = None
+
+                # Add enough new stats to account for any idle minutes between
+                # the last recorded stat and the current one
                 while oldindex != dtindex:
                     oldindex += 60
                     self.statsByMinute.append((oldindex, self.initStats(),))
         else:
             self.statsByMinute.append((dtindex, self.initStats(),))
-        return self.statsByMinute[-1][1]
+            self.stats1m = None
+            self.stats5m = None
+            self.stats1h = None
 
+        # We only need up to 1 hour's worth of data, so truncate the cached data
+        # to avoid filling memory
+        threshold = 65
+        if len(self.statsByMinute) > threshold:
+            self.statsByMinute = self.statsByMinute[-threshold:]
+
+        return self.statsByMinute[-1][1]
 
     def initStats(self):
 
         def initTimeHistogram():
             return {
                 "<10ms": 0,
-                "10ms<->100ms" : 0,
-                "100ms<->1s"   : 0,
-                "1s<->10s"     : 0,
-                "10s<->30s"    : 0,
-                "30s<->60s"    : 0,
-                ">60s"         : 0,
-                "Over 1s"      : 0,
-                "Over 10s"     : 0,
+                "10ms<->100ms": 0,
+                "100ms<->1s": 0,
+                "1s<->10s": 0,
+                "10s<->30s": 0,
+                "30s<->60s": 0,
+                ">60s": 0,
+                "Over 1s": 0,
+                "Over 10s": 0,
             }
 
         return {
-            "requests"   : 0,
-            "method"     : collections.defaultdict(int),
-            "method-t"   : collections.defaultdict(float),
-            "uid"        : collections.defaultdict(int),
-            "user-agent" : collections.defaultdict(int),
-            "500"        : 0,
-            "t"          : 0.0,
-            "t-resp-wr"  : 0.0,
-            "slots"      : 0,
-            "T"          : initTimeHistogram(),
-            "T-RESP-WR"  : initTimeHistogram(),
-            "T-MAX"      : 0.0,
-            "cpu"        : self.systemStats.items["cpu use"],
+            "requests": 0,
+            "method": collections.defaultdict(int),
+            "method-t": collections.defaultdict(float),
+            "500": 0,
+            "401": 0,
+            "t": 0.0,
+            "t-resp-wr": 0.0,
+            "slots": 0,
+            "max-slots": 0,
+            "T": initTimeHistogram(),
+            "T-RESP-WR": initTimeHistogram(),
+            "T-MAX": 0.0,
+            "cpu": self.systemStats.items["cpu use"],
         }
 
-
     def updateStats(self, current, stats):
         # Gather specific information and aggregate into our persistent stats
         adjustedMethod = getAdjustedMethodName(stats)
-        adjustedClient = getAdjustedClientName(stats)
 
         if current["requests"] == 0:
             current["cpu"] = 0.0
         current["requests"] += 1
         current["method"][adjustedMethod] += 1
         current["method-t"][adjustedMethod] += stats.get("t", 0.0)
-        current["uid"][stats["uid"]] += 1
-        current["user-agent"][adjustedClient] += 1
         if stats["statusCode"] >= 500:
             current["500"] += 1
+        elif stats["statusCode"] == 401:
+            current["401"] += 1
         current["t"] += stats.get("t", 0.0)
         current["t-resp-wr"] += stats.get("t-resp-wr", 0.0)
         current["slots"] += stats.get("outstandingRequests", 0)
+        current["max-slots"] = max(current["max-slots"], self.limiter.maxOutstandingRequests if hasattr(self, "limiter") else 0)
         current["cpu"] += self.systemStats.items["cpu use"]
 
         def histogramUpdate(t, key):
@@ -533,7 +545,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         if t is not None:
             histogramUpdate(t, "T-RESP-WR")
 
-
     def mergeStats(self, current, stats):
         # Gather specific information and aggregate into our persistent stats
         if current["requests"] == 0:
@@ -543,14 +554,12 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
             current["method"][method] += stats["method"][method]
         for method in stats["method-t"].keys():
             current["method-t"][method] += stats["method-t"][method]
-        for uid in stats["uid"].keys():
-            current["uid"][uid] += stats["uid"][uid]
-        for ua in stats["user-agent"].keys():
-            current["user-agent"][ua] += stats["user-agent"][ua]
         current["500"] += stats["500"]
+        current["401"] += stats["401"]
         current["t"] += stats["t"]
         current["t-resp-wr"] += stats["t-resp-wr"]
         current["slots"] += stats["slots"]
+        current["max-slots"] = max(current["max-slots"], stats["max-slots"])
         current["cpu"] += stats["cpu"]
 
         def histogramUpdate(t, key):
@@ -580,7 +589,6 @@ class RotatingFileAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
             current["T-RESP-WR"][bin] += stats["T-RESP-WR"][bin]
 
 
-
 class SystemMonitor(object):
     """
     Keeps track of system usage information. This installs a reactor task to
@@ -591,11 +599,11 @@ class SystemMonitor(object):
 
     def __init__(self):
         self.items = {
-            "cpu count"     : psutil.NUM_CPUS if psutil is not None else -1,
-            "cpu use"       : 0.0,
-            "memory used"   : 0,
+            "cpu count": psutil.cpu_count() if psutil is not None else -1,
+            "cpu use": 0.0,
+            "memory used": 0,
             "memory percent": 0.0,
-            "start time"    : time.time(),
+            "start time": time.time(),
         }
 
         if psutil is not None:
@@ -607,14 +615,12 @@ class SystemMonitor(object):
         self.task = task.LoopingCall(self.update)
         self.task.start(1.0)
 
-
     def stop(self):
         """
         Just stop the task
         """
         self.task.stop()
 
-
     def update(self):
 
         # CPU usage based on diff'ing CPU times
@@ -634,24 +640,21 @@ class SystemMonitor(object):
             self.items["memory percent"] = mem.percent
 
 
-
 class LogStats(amp.Command):
     arguments = [("message", amp.String())]
 
 
-
 class AMPCommonAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
+
     def __init__(self):
         self.protocol = None
         self._buffer = []
 
-
     def flushBuffer(self):
         if self._buffer:
             for msg in self._buffer:
                 self.logStats(msg)
 
-
     def addClient(self, connectedClient):
         """
         An AMP client connected; hook it up to this observer.
@@ -659,7 +662,6 @@ class AMPCommonAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
         self.protocol = connectedClient
         self.flushBuffer()
 
-
     def logStats(self, message):
         """
         Log server stats via the remote AMP Protocol
@@ -675,7 +677,6 @@ class AMPCommonAccessLoggingObserver(CommonAccessLoggingObserverExtensions):
             self._buffer.append(message)
 
 
-
 class AMPLoggingProtocol(amp.AMP):
     """
     A server side protocol for logging to the given observer.
@@ -686,7 +687,6 @@ class AMPLoggingProtocol(amp.AMP):
 
         super(AMPLoggingProtocol, self).__init__()
 
-
     def logStats(self, message):
         stats = json.loads(message)
         self.observer.logStats(stats)
@@ -695,19 +695,16 @@ class AMPLoggingProtocol(amp.AMP):
     LogStats.responder(logStats)
 
 
-
 class AMPLoggingFactory(protocol.ServerFactory):
+
     def __init__(self, observer):
         self.observer = observer
 
-
     def doStart(self):
         self.observer.start()
 
-
     def doStop(self):
         self.observer.stop()
 
-
     def buildProtocol(self, addr):
         return AMPLoggingProtocol(self.observer)
