@@ -1,6 +1,6 @@
 # -*- test-case-name: calendarserver.provision.test.test_root -*-
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -39,6 +39,7 @@ from twistedcaldav.extensions import DAVFile, CachingPropertyStore
 from twistedcaldav.extensions import DirectoryPrincipalPropertySearchMixIn
 from twistedcaldav.extensions import ReadOnlyResourceMixIn
 from twistedcaldav.resource import CalDAVComplianceMixIn
+from txdav.who.delegates import CachingDelegates
 from txdav.who.wiki import DirectoryService as WikiDirectoryService
 from txdav.who.wiki import uidForAuthToken
 from txweb2 import responsecode
@@ -93,9 +94,8 @@ class RootResource(
 
             # These class attributes need to be setup with our memcache\
             # notifier
-            DirectoryPrincipalResource.cacheNotifierFactory = (
-                MemcacheChangeNotifier
-            )
+            DirectoryPrincipalResource.cacheNotifierFactory = MemcacheChangeNotifier
+            CachingDelegates.cacheNotifier = MemcacheChangeNotifier(None, cacheHandle="PrincipalToken")
         else:
             self.responseCache = DisabledCache()
 
@@ -103,7 +103,6 @@ class RootResource(
             from txweb2.filter import gzip
             self.contentFilters.append((gzip.gzipfilter, True))
 
-
     def deadProperties(self):
         if not hasattr(self, "_dead_properties"):
             # Get the property store from super
@@ -119,11 +118,9 @@ class RootResource(
 
         return self._dead_properties
 
-
     def defaultAccessControlList(self):
         return succeed(config.RootResourceACL)
 
-
     @inlineCallbacks
     def checkSACL(self, request):
         """
@@ -190,7 +187,6 @@ class RootResource(
         )
         raise HTTPError(responsecode.FORBIDDEN)
 
-
     @inlineCallbacks
     def locateChild(self, request, segments):
 
@@ -249,9 +245,9 @@ class RootResource(
                             )
                         else:
                             log.error(
-                                "Failed to look up wiki token {token}: "
-                                "{message}",
-                                token=token, message=w.message
+                                "Failed to look up wiki token {token}: {msg}",
+                                token=token,
+                                msg=w.message
                             )
 
                     except Exception as e:
@@ -287,7 +283,7 @@ class RootResource(
                     "x-forwarded-host",
                     [config.ServerHostName]
                 )[-1].split(",")[-1].strip()
-                port = 443 if config.EnableSSL else 80
+                port = 443 if (config.EnableSSL or config.BehindTLSProxy) else 80
                 scheme = "https" if config.EnableSSL else "http"
 
                 response = RedirectResponse(
@@ -404,7 +400,6 @@ class RootResource(
         )
         returnValue(child)
 
-
     @inlineCallbacks
     def principalForUID(self, request, uid):
         principal = None
@@ -423,14 +418,11 @@ class RootResource(
 
         returnValue(principal)
 
-
     def http_COPY(self, request):
         return responsecode.FORBIDDEN
 
-
     def http_MOVE(self, request):
         return responsecode.FORBIDDEN
 
-
     def http_DELETE(self, request):
         return responsecode.FORBIDDEN
