@@ -1,6 +1,6 @@
 # -*- test-case-name: calendarserver.push.test.test_applepush -*-
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,6 +17,9 @@
 
 from twext.internet.ssl import ChainingOpenSSLContextFactory
 from twext.python.log import Logger
+from twext.enterprise.dal.record import fromTable
+from twext.enterprise.jobs.workitem import RegeneratingWorkItem
+from txdav.common.datastore.sql_tables import schema
 
 from txweb2 import responsecode
 from txdav.xml import element as davxml
@@ -28,7 +31,6 @@ from twisted.application import service
 from twisted.internet.protocol import Protocol
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed
 from twisted.internet.protocol import ClientFactory, ReconnectingClientFactory
-from twisted.internet.task import LoopingCall
 from twistedcaldav.extensions import DAVResource, DAVResourceWithoutChildrenMixin
 from twistedcaldav.resource import ReadOnlyNoCopyResourceMixIn
 import json
@@ -36,9 +38,8 @@ import OpenSSL
 import struct
 import time
 from txdav.common.icommondatastore import InvalidSubscriptionValues
-from calendarserver.push.util import (
-    validToken, TokenHistory, PushScheduler, PushPriority
-)
+from calendarserver.push.ipush import PushPriority
+from calendarserver.push.util import validToken, TokenHistory, PushScheduler
 from twext.internet.adaptendpoint import connect
 from twext.internet.gaiendpoint import GAIEndpoint
 from twisted.python.constants import Values, ValueConstant
@@ -46,7 +47,6 @@ from twisted.python.constants import Values, ValueConstant
 log = Logger()
 
 
-
 class ApplePushPriority(Values):
     """
     Maps calendarserver.push.util.PushPriority values to APNS-specific values
@@ -56,7 +56,6 @@ class ApplePushPriority(Values):
     high = ValueConstant(PushPriority.high.value)
 
 
-
 class ApplePushNotifierService(service.MultiService):
     """
     ApplePushNotifierService is a MultiService responsible for
@@ -108,7 +107,7 @@ class ApplePushNotifierService(service.MultiService):
 
         for protocol in ("CalDAV", "CardDAV"):
 
-            if settings[protocol]["CertificatePath"]:
+            if settings[protocol].Enabled:
 
                 providerTestConnector = None
                 feedbackTestConnector = None
@@ -124,6 +123,7 @@ class ApplePushNotifierService(service.MultiService):
                     settings[protocol]["PrivateKeyPath"],
                     chainPath=settings[protocol]["AuthorityChainPath"],
                     passphrase=settings[protocol]["Passphrase"],
+                    keychainIdentity=settings[protocol]["KeychainIdentity"],
                     staggerNotifications=settings["EnableStaggering"],
                     staggerSeconds=settings["StaggerSeconds"],
                     testConnector=providerTestConnector,
@@ -132,8 +132,9 @@ class ApplePushNotifierService(service.MultiService):
                 provider.setServiceParent(service)
                 service.providers[protocol] = provider
                 service.log.info(
-                    "APNS %s topic: %s" %
-                    (protocol, settings[protocol]["Topic"]))
+                    "APNS {proto} topic: {topic}",
+                    proto=protocol, topic=settings[protocol]["Topic"]
+                )
 
                 feedback = APNFeedbackService(
                     service.store,
@@ -144,6 +145,7 @@ class ApplePushNotifierService(service.MultiService):
                     settings[protocol]["PrivateKeyPath"],
                     chainPath=settings[protocol]["AuthorityChainPath"],
                     passphrase=settings[protocol]["Passphrase"],
+                    keychainIdentity=settings[protocol]["KeychainIdentity"],
                     testConnector=feedbackTestConnector,
                     reactor=reactor,
                 )
@@ -152,43 +154,24 @@ class ApplePushNotifierService(service.MultiService):
 
         return service
 
-
     def startService(self):
         """
         In addition to starting the provider and feedback sub-services, start a
         LoopingCall whose job it is to purge old subscriptions
         """
-        service.MultiService.startService(self)
         self.log.debug("ApplePushNotifierService startService")
-        self.purgeCall = LoopingCall(self.purgeOldSubscriptions, self.purgeSeconds)
-        self.purgeCall.start(self.purgeIntervalSeconds, now=False)
+        APNPurgingWork.purgeSeconds = self.purgeSeconds
+        APNPurgingWork.purgeIntervalSeconds = self.purgeIntervalSeconds
 
+        service.MultiService.startService(self)
 
     def stopService(self):
         """
         In addition to stopping the provider and feedback sub-services, stop the
         LoopingCall
         """
-        service.MultiService.stopService(self)
         self.log.debug("ApplePushNotifierService stopService")
-        if self.purgeCall is not None:
-            self.purgeCall.stop()
-            self.purgeCall = None
-
-
-    @inlineCallbacks
-    def purgeOldSubscriptions(self, purgeSeconds):
-        """
-        Remove any subscriptions that registered more than purgeSeconds ago
-
-        @param purgeSeconds: The cutoff given in seconds
-        @type purgeSeconds: C{int}
-        """
-        self.log.debug("ApplePushNotifierService purgeOldSubscriptions")
-        txn = self.store.newTransaction(label="ApplePushNotifierService.purgeOldSubscriptions")
-        yield txn.purgeOldAPNSubscriptions(int(time.time()) - purgeSeconds)
-        yield txn.commit()
-
+        service.MultiService.stopService(self)
 
     @inlineCallbacks
     def enqueue(
@@ -216,7 +199,7 @@ class ApplePushNotifierService(service.MultiService):
             protocol = pushKey.split("/")[1]
         except ValueError:
             # pushKey has no protocol, so we can't do anything with it
-            self.log.error("Push key '%s' is missing protocol" % (pushKey,))
+            self.log.error("Push key '{key}' is missing protocol", key=pushKey)
             return
 
         # Unit tests can pass this value in; otherwise it defaults to now
@@ -232,8 +215,9 @@ class ApplePushNotifierService(service.MultiService):
             numSubscriptions = len(subscriptions)
             if numSubscriptions > 0:
                 self.log.debug(
-                    "Sending %d APNS notifications for %s" %
-                    (numSubscriptions, pushKey))
+                    "Sending {num} APNS notifications for {key}",
+                    num=numSubscriptions, key=pushKey
+                )
                 tokens = [record.token for record in subscriptions if record.token and record.subscriberGUID]
                 if tokens:
                     provider.scheduleNotifications(
@@ -241,7 +225,6 @@ class ApplePushNotifierService(service.MultiService):
                         dataChangedTimestamp, priority)
 
 
-
 class APNProviderProtocol(Protocol):
     """
     Implements the Provider portion of APNS
@@ -256,16 +239,16 @@ class APNProviderProtocol(Protocol):
 
     # Returned only for an error.  Successful notifications get no response.
     STATUS_CODES = {
-        0   : "No errors encountered",
-        1   : "Processing error",
-        2   : "Missing device token",
-        3   : "Missing topic",
-        4   : "Missing payload",
-        5   : "Invalid token size",
-        6   : "Invalid topic size",
-        7   : "Invalid payload size",
-        8   : "Invalid token",
-        255 : "None (unknown)",
+        0: "No errors encountered",
+        1: "Processing error",
+        2: "Missing device token",
+        3: "Missing topic",
+        4: "Missing payload",
+        5: "Invalid token size",
+        6: "Invalid topic size",
+        7: "Invalid payload size",
+        8: "Invalid token",
+        255: "None (unknown)",
     }
 
     # If error code comes back as one of these, remove the associated device
@@ -279,7 +262,6 @@ class APNProviderProtocol(Protocol):
         self.log.debug("ProviderProtocol makeConnection")
         Protocol.makeConnection(self, transport)
 
-
     def connectionMade(self):
         self.log.debug("ProviderProtocol connectionMade")
         self.buffer = ""
@@ -288,13 +270,11 @@ class APNProviderProtocol(Protocol):
         self.factory.connection = self
         self.factory.clientConnectionMade()
 
-
     def connectionLost(self, reason=None):
-        # self.log.debug("ProviderProtocol connectionLost: %s" % (reason,))
+        # self.log.debug("ProviderProtocol connectionLost: {reason}", reason=reason)
         # Clear the reference to us from the factory
         self.factory.connection = None
 
-
     @inlineCallbacks
     def dataReceived(self, data, fn=None):
         """
@@ -305,7 +285,7 @@ class APNProviderProtocol(Protocol):
         if fn is None:
             fn = self.processError
 
-        self.log.debug("ProviderProtocol dataReceived %d bytes" % (len(data),))
+        self.log.debug("ProviderProtocol dataReceived {len} bytes", len=len(data))
         self.buffer += data
 
         while len(self.buffer) >= self.MESSAGE_LENGTH:
@@ -318,9 +298,9 @@ class APNProviderProtocol(Protocol):
                     yield fn(status, identifier)
             except Exception, e:
                 self.log.warn(
-                    "ProviderProtocol could not process error: %s (%s)" %
-                    (message.encode("hex"), e))
-
+                    "ProviderProtocol could not process error: {code} ({ex})",
+                    code=message.encode("hex"), ex=e
+                )
 
     @inlineCallbacks
     def processError(self, status, identifier):
@@ -337,23 +317,24 @@ class APNProviderProtocol(Protocol):
         @type status: C{int}
         """
         msg = self.STATUS_CODES.get(status, "Unknown status code")
-        self.log.info("Received APN error %d on identifier %d: %s" % (status, identifier, msg))
+        self.log.info("Received APN error {status} on identifier {id}: {msg}", status=status, id=identifier, msg=msg)
         if status in self.TOKEN_REMOVAL_CODES:
             token = self.history.extractIdentifier(identifier)
             if token is not None:
                 self.log.debug(
-                    "Removing subscriptions for bad token: %s" %
-                    (token,))
+                    "Removing subscriptions for bad token: {token}",
+                    token=token,
+                )
                 txn = self.factory.store.newTransaction(label="APNProviderProtocol.processError")
                 subscriptions = (yield txn.apnSubscriptionsByToken(token))
                 for record in subscriptions:
                     self.log.debug(
-                        "Removing subscription: %s %s" %
-                        (token, record.resourceKey))
+                        "Removing subscription: {token} {key}",
+                        token=token, key=record.resourceKey
+                    )
                     yield txn.removeAPNSubscription(token, record.resourceKey)
                 yield txn.commit()
 
-
     def sendNotification(self, token, key, dataChangedTimestamp, priority):
         """
         Sends a push notification message for the key to the device associated
@@ -374,16 +355,16 @@ class APNProviderProtocol(Protocol):
         try:
             binaryToken = token.replace(" ", "").decode("hex")
         except:
-            self.log.error("Invalid APN token in database: %s" % (token,))
+            self.log.error("Invalid APN token in database: {token}", token=token)
             return
 
         identifier = self.history.add(token)
         apnsPriority = ApplePushPriority.lookupByValue(priority.value).value
         payload = json.dumps(
             {
-                "key" : key,
-                "dataChangedTimestamp" : dataChangedTimestamp,
-                "pushRequestSubmittedTimestamp" : int(time.time()),
+                "key": key,
+                "dataChangedTimestamp": dataChangedTimestamp,
+                "pushRequestSubmittedTimestamp": int(time.time()),
             }
         )
         payloadLength = len(payload)
@@ -407,26 +388,26 @@ class APNProviderProtocol(Protocol):
 
         # Frame struct.pack format                ! Network byte order
         command = self.COMMAND_PROVIDER             # B
-        frameLength = (# I
+        frameLength = (  # I
             # Item 1 (Device token)
-            1 + # Item number                       # B
-            2 + # Item length                       # H
-            32 + # device token                     # 32s
+            1 +  # Item number                       # B
+            2 +  # Item length                       # H
+            32 +  # device token                     # 32s
             # Item 2 (Payload)
-            1 + # Item number                       # B
-            2 + # Item length                       # H
-            payloadLength + # the JSON payload      # %d s
+            1 +  # Item number                       # B
+            2 +  # Item length                       # H
+            payloadLength +  # the JSON payload      # %d s
             # Item 3 (Notification ID)
-            1 + # Item number                       # B
-            2 + # Item length                       # H
-            4 + # Notification ID                   # I
+            1 +  # Item number                       # B
+            2 +  # Item length                       # H
+            4 +  # Notification ID                   # I
             # Item 4 (Expiration)
-            1 + # Item number                       # B
-            2 + # Item length                       # H
-            4 + # Expiration seconds since epoch    # I
+            1 +  # Item number                       # B
+            2 +  # Item length                       # H
+            4 +  # Expiration seconds since epoch    # I
             # Item 5 (Priority)
-            1 + # Item number                       # B
-            2 + # Item length                       # H
+            1 +  # Item number                       # B
+            2 +  # Item length                       # H
             1    # Priority                         # B
         )
 
@@ -451,7 +432,7 @@ class APNProviderProtocol(Protocol):
 
                 4,                               # Item 4 (Expiration)
                 4,                               # Expiration length
-                int(time.time()) + 72 * 60 * 60, # Expires in 72 hours
+                int(time.time()) + 72 * 60 * 60,  # Expires in 72 hours
 
                 5,                               # Item 5 (Priority)
                 1,                               # Priority length
@@ -461,7 +442,6 @@ class APNProviderProtocol(Protocol):
         )
 
 
-
 class APNProviderFactory(ReconnectingClientFactory):
     log = Logger()
 
@@ -471,47 +451,45 @@ class APNProviderFactory(ReconnectingClientFactory):
         self.service = service
         self.store = store
         self.noisy = True
-        self.maxDelay = 30 # max seconds between connection attempts
+        self.maxDelay = 30  # max seconds between connection attempts
         self.shuttingDown = False
 
-
     def clientConnectionMade(self):
         self.log.info("Connection to APN server made")
         self.service.clientConnectionMade()
         self.delay = 1.0
 
-
     def clientConnectionLost(self, connector, reason):
         if not self.shuttingDown:
-            self.log.info("Connection to APN server lost: %s" % (reason,))
+            self.log.error("Connection to APN server lost: {reason}", reason=reason)
+            if reason.type == OpenSSL.SSL.Error:
+                # If we're failing due to a certificate issue, stop retrying.
+                self.log.error("Ensure APNS certificate is not expired")
+                ReconnectingClientFactory.stopTrying(self)
         ReconnectingClientFactory.clientConnectionLost(self, connector, reason)
 
-
     def clientConnectionFailed(self, connector, reason):
-        self.log.error("Unable to connect to APN server: %s" % (reason,))
+        self.log.error("Unable to connect to APN server: {reason}", reason=reason)
         self.connected = False
         ReconnectingClientFactory.clientConnectionFailed(
             self, connector,
             reason)
 
-
     def retry(self, connector=None):
         self.log.info("Reconnecting to APN server")
         ReconnectingClientFactory.retry(self, connector)
 
-
     def stopTrying(self):
         self.shuttingDown = True
         ReconnectingClientFactory.stopTrying(self)
 
 
-
 class APNConnectionService(service.Service):
     log = Logger()
 
     def __init__(
         self, host, port, certPath, keyPath, chainPath="",
-        passphrase="", sslMethod="TLSv1_METHOD", testConnector=None,
+        passphrase="", keychainIdentity="", sslMethod="TLSv1_METHOD", testConnector=None,
         reactor=None
     ):
 
@@ -521,6 +499,7 @@ class APNConnectionService(service.Service):
         self.keyPath = keyPath
         self.chainPath = chainPath
         self.passphrase = passphrase
+        self.keychainIdentity = keychainIdentity
         self.sslMethod = sslMethod
         self.testConnector = testConnector
 
@@ -528,14 +507,13 @@ class APNConnectionService(service.Service):
             from twisted.internet import reactor
         self.reactor = reactor
 
-
     def connect(self, factory):
         if self.testConnector is not None:
             # For testing purposes
             self.testConnector.connect(self, factory)
         else:
             if self.passphrase:
-                passwdCallback = lambda *ignored : self.passphrase
+                passwdCallback = lambda *ignored: self.passphrase
             else:
                 passwdCallback = None
             context = ChainingOpenSSLContextFactory(
@@ -543,25 +521,27 @@ class APNConnectionService(service.Service):
                 self.certPath,
                 certificateChainFile=self.chainPath,
                 passwdCallback=passwdCallback,
-                sslmethod=getattr(OpenSSL.SSL, self.sslMethod)
+                keychainIdentity=self.keychainIdentity,
+                sslmethod=getattr(OpenSSL.SSL, self.sslMethod),
+                peerName=self.host,
             )
             connect(GAIEndpoint(self.reactor, self.host, self.port, context),
                     factory)
 
 
-
 class APNProviderService(APNConnectionService):
 
     def __init__(
         self, store, host, port, certPath, keyPath, chainPath="",
-        passphrase="", sslMethod="TLSv1_METHOD",
+        passphrase="", keychainIdentity="", sslMethod="TLSv1_METHOD",
         staggerNotifications=False, staggerSeconds=3,
         testConnector=None, reactor=None
     ):
 
         APNConnectionService.__init__(
             self, host, port, certPath, keyPath,
-            chainPath=chainPath, passphrase=passphrase, sslMethod=sslMethod,
+            chainPath=chainPath, passphrase=passphrase,
+            keychainIdentity=keychainIdentity, sslMethod=sslMethod,
             testConnector=testConnector, reactor=reactor)
 
         self.store = store
@@ -574,12 +554,10 @@ class APNProviderService(APNConnectionService):
         else:
             self.scheduler = None
 
-
     def startService(self):
         self.log.debug("APNProviderService startService")
         self.factory = APNProviderFactory(self, self.store)
-        self.connect(self.factory)
-
+        self.reactor.callWhenRunning(self.connect, self.factory)
 
     def stopService(self):
         self.log.debug("APNProviderService stopService")
@@ -588,7 +566,6 @@ class APNProviderService(APNConnectionService):
         if self.scheduler is not None:
             self.scheduler.stop()
 
-
     def clientConnectionMade(self):
         # Service the queue
         if self.queue:
@@ -602,7 +579,6 @@ class APNProviderService(APNConnectionService):
                         token, key, dataChangedTimestamp,
                         priority)
 
-
     def scheduleNotifications(self, tokens, key, dataChangedTimestamp, priority):
         """
         The starting point for getting notifications to the APNS server.  If there is
@@ -629,7 +605,6 @@ class APNProviderService(APNConnectionService):
         else:
             self._saveForWhenConnected(tokens, key, dataChangedTimestamp, priority)
 
-
     def _saveForWhenConnected(self, tokens, key, dataChangedTimestamp, priority):
         """
         Called in order to save notifications that can't be sent now because there
@@ -648,13 +623,12 @@ class APNProviderService(APNConnectionService):
             tokenKeyPair = (token, key)
             for existingPair, _ignore_timstamp, priority in self.queue:
                 if tokenKeyPair == existingPair:
-                    self.log.debug("APNProviderService has no connection; skipping duplicate: %s %s" % (token, key))
-                    break # Already scheduled
+                    self.log.debug("APNProviderService has no connection; skipping duplicate: {token} {key}", token=token, key=key)
+                    break  # Already scheduled
             else:
-                self.log.debug("APNProviderService has no connection; queuing: %s %s" % (token, key))
+                self.log.debug("APNProviderService has no connection; queuing: {token} {key}", token=token, key=key)
                 self.queue.append(((token, key), dataChangedTimestamp, priority))
 
-
     def sendNotification(self, token, key, dataChangedTimestamp, priority):
         """
         If there is a connection the notification is sent right away, otherwise
@@ -679,7 +653,6 @@ class APNProviderService(APNConnectionService):
             connection.sendNotification(token, key, dataChangedTimestamp, priority)
 
 
-
 class APNFeedbackProtocol(Protocol):
     """
     Implements the Feedback portion of APNS
@@ -692,7 +665,6 @@ class APNFeedbackProtocol(Protocol):
         self.log.debug("FeedbackProtocol connectionMade")
         self.buffer = ""
 
-
     @inlineCallbacks
     def dataReceived(self, data, fn=None):
         """
@@ -703,7 +675,7 @@ class APNFeedbackProtocol(Protocol):
         if fn is None:
             fn = self.processFeedback
 
-        self.log.debug("FeedbackProtocol dataReceived %d bytes" % (len(data),))
+        self.log.debug("FeedbackProtocol dataReceived {len} bytes", len=len(data))
         self.buffer += data
 
         while len(self.buffer) >= self.MESSAGE_LENGTH:
@@ -718,9 +690,9 @@ class APNFeedbackProtocol(Protocol):
                 yield fn(timestamp, token)
             except Exception, e:
                 self.log.warn(
-                    "FeedbackProtocol could not process message: %s (%s)" %
-                    (message.encode("hex"), e))
-
+                    "FeedbackProtocol could not process message: {code} ({ex})",
+                    code=message.encode("hex"), ex=e
+                )
 
     @inlineCallbacks
     def processFeedback(self, timestamp, token):
@@ -738,21 +710,22 @@ class APNFeedbackProtocol(Protocol):
         """
 
         self.log.debug(
-            "FeedbackProtocol processFeedback time=%d token=%s" %
-            (timestamp, token))
+            "FeedbackProtocol processFeedback time={time} token={token}",
+            time=timestamp, token=token
+        )
         txn = self.factory.store.newTransaction(label="APNFeedbackProtocol.processFeedback")
         subscriptions = (yield txn.apnSubscriptionsByToken(token))
 
         for record in subscriptions:
             if timestamp > record.modified:
                 self.log.debug(
-                    "FeedbackProtocol removing subscription: %s %s" %
-                    (token, record.resourceKey))
+                    "FeedbackProtocol removing subscription: {token} {key}",
+                    token=token, key=record.resourceKey,
+                )
                 yield txn.removeAPNSubscription(token, record.resourceKey)
         yield txn.commit()
 
 
-
 class APNFeedbackFactory(ClientFactory):
     log = Logger()
 
@@ -761,45 +734,43 @@ class APNFeedbackFactory(ClientFactory):
     def __init__(self, store):
         self.store = store
 
-
     def clientConnectionFailed(self, connector, reason):
         self.log.error(
-            "Unable to connect to APN feedback server: %s" %
-            (reason,))
+            "Unable to connect to APN feedback server: {reason}",
+            reason=reason,
+        )
         self.connected = False
         ClientFactory.clientConnectionFailed(self, connector, reason)
 
 
-
 class APNFeedbackService(APNConnectionService):
 
     def __init__(
         self, store, updateSeconds, host, port,
-        certPath, keyPath, chainPath="", passphrase="", sslMethod="TLSv1_METHOD",
+        certPath, keyPath, chainPath="",
+        passphrase="", keychainIdentity="", sslMethod="TLSv1_METHOD",
         testConnector=None, reactor=None
     ):
 
         APNConnectionService.__init__(
             self, host, port, certPath, keyPath,
-            chainPath=chainPath, passphrase=passphrase, sslMethod=sslMethod,
+            chainPath=chainPath, passphrase=passphrase,
+            keychainIdentity=keychainIdentity, sslMethod=sslMethod,
             testConnector=testConnector, reactor=reactor)
 
         self.store = store
         self.updateSeconds = updateSeconds
 
-
     def startService(self):
         self.log.debug("APNFeedbackService startService")
         self.factory = APNFeedbackFactory(self.store)
-        self.checkForFeedback()
-
+        self.reactor.callWhenRunning(self.checkForFeedback)
 
     def stopService(self):
         self.log.debug("APNFeedbackService stopService")
         if self.nextCheck is not None:
             self.nextCheck.cancel()
 
-
     def checkForFeedback(self):
         self.nextCheck = None
         self.log.debug("APNFeedbackService checkForFeedback")
@@ -809,7 +780,6 @@ class APNFeedbackService(APNConnectionService):
             self.checkForFeedback)
 
 
-
 class APNSubscriptionResource(
     ReadOnlyNoCopyResourceMixIn,
     DAVResourceWithoutChildrenMixin, DAVResource
@@ -831,21 +801,17 @@ class APNSubscriptionResource(
         self.parent = parent
         self.store = store
 
-
     def deadProperties(self):
         if not hasattr(self, "_dead_properties"):
             self._dead_properties = NonePropertyStore(self)
         return self._dead_properties
 
-
     def etag(self):
         return succeed(None)
 
-
     def checkPreconditions(self, request):
         return None
 
-
     def defaultAccessControlList(self):
         return succeed(
             davxml.ACL(
@@ -868,27 +834,21 @@ class APNSubscriptionResource(
             )
         )
 
-
     def contentType(self):
         return MimeType.fromString("text/html; charset=utf-8")
 
-
     def resourceType(self):
         return None
 
-
     def isCollection(self):
         return False
 
-
     def isCalendarCollection(self):
         return False
 
-
     def isPseudoCalendarCollection(self):
         return False
 
-
     @inlineCallbacks
     def http_POST(self, request):
         yield self.authorize(request, (davxml.Write(),))
@@ -937,7 +897,6 @@ class APNSubscriptionResource(
 
         returnValue((code, msg))
 
-
     @inlineCallbacks
     def addSubscription(self, token, key, uid, userAgent, host):
         """
@@ -958,13 +917,37 @@ class APNSubscriptionResource(
         @param host: The host requesting the subscription
         @type key: C{str}
         """
-        now = int(time.time()) # epoch seconds
+        now = int(time.time())  # epoch seconds
         txn = self.store.newTransaction(label="APNSubscriptionResource.addSubscription")
         yield txn.addAPNSubscription(token, key, now, uid, userAgent, host)
         yield txn.commit()
 
-
     def renderResponse(self, code, body=None):
         response = Response(code, {}, body)
         response.headers.setHeader("content-type", MimeType("text", "html"))
         return response
+
+
+class APNPurgingWork(RegeneratingWorkItem, fromTable(schema.APN_PURGING_WORK)):
+
+    group = "apn_purging"
+    purgeIntervalSeconds = 12 * 60 * 60  # 12 hours by default
+    purgeSeconds = 14 * 24 * 60 * 60  # 14 days by default
+
+    @classmethod
+    def initialSchedule(cls, store, seconds):
+        def _enqueue(txn):
+            return APNPurgingWork.reschedule(txn, seconds)
+
+        return store.inTransaction("APNPurgingWork.initialSchedule", _enqueue)
+
+    def regenerateInterval(self):
+        """
+        Return the interval in seconds between regenerating instances.
+        """
+        return self.purgeIntervalSeconds
+
+    def doWork(self):
+        return self.transaction.purgeOldAPNSubscriptions(
+            int(time.time()) - self.purgeSeconds
+        )
