@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,12 +34,11 @@ from zope.interface.declarations import implements
 
 import datetime
 
-from calendarserver.push.util import PushPriority
+from calendarserver.push.ipush import PushPriority
 
 log = Logger()
 
 
-
 class PushNotificationWork(WorkItem, fromTable(schema.PUSH_NOTIFICATION_WORK)):
 
     group = property(lambda self: (self.table.PUSH_ID == self.pushID))
@@ -75,8 +74,8 @@ class PushNotificationWork(WorkItem, fromTable(schema.PUSH_NOTIFICATION_WORK)):
                 Where=self.table.WORK_ID.In(Parameter("workIDs", len(workIDs)))
             ).on(self.transaction, workIDs=workIDs)
             yield Delete(
-                From=JobItem.table, #@UndefinedVariable
-                Where=JobItem.jobID.In(Parameter("jobIDs", len(jobIDs))) #@UndefinedVariable
+                From=JobItem.table,  # @UndefinedVariable
+                Where=JobItem.jobID.In(Parameter("jobIDs", len(jobIDs)))  # @UndefinedVariable
             ).on(self.transaction, jobIDs=jobIDs)
 
         pushDistributor = self.transaction._pushDistributor
@@ -86,7 +85,6 @@ class PushNotificationWork(WorkItem, fromTable(schema.PUSH_NOTIFICATION_WORK)):
             yield pushDistributor.enqueue(self.transaction, self.pushID, priority=priority)
 
 
-
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Classes used within calendarserver itself
 #
@@ -105,17 +103,14 @@ class Notifier(object):
         self._storeObject = storeObject
         self._notify = True
 
-
     def enableNotify(self, arg):
-        self.log.debug("enableNotify: %s" % (self._ids['default'][1],))
+        self.log.debug("enableNotify: {id}", id=self._ids['default'][1])
         self._notify = True
 
-
     def disableNotify(self):
-        self.log.debug("disableNotify: %s" % (self._ids['default'][1],))
+        self.log.debug("disableNotify: {id}", id=self._ids['default'][1])
         self._notify = False
 
-
     @inlineCallbacks
     def notify(self, txn, priority=PushPriority.high):
         """
@@ -138,21 +133,21 @@ class Notifier(object):
         for prefix, id in ids:
             if self._notify:
                 self.log.debug(
-                    "Notifications are enabled: %s %s/%s priority=%d" %
-                    (self._storeObject, prefix, id, priority.value))
+                    "Notifications are enabled: {obj} {prefix}/{id} priority={priority}",
+                    obj=self._storeObject, prefix=prefix, id=id, priority=priority.value
+                )
                 yield self._notifierFactory.send(
                     prefix, id, txn,
                     priority=priority)
             else:
                 self.log.debug(
-                    "Skipping notification for: %s %s/%s" %
-                    (self._storeObject, prefix, id,))
-
+                    "Skipping notification for: %{obj} {prefix}/{id}",
+                    obj=self._storeObject, prefix=prefix, id=id,
+                )
 
     def clone(self, storeObject):
         return self.__class__(self._notifierFactory, storeObject)
 
-
     def nodeName(self):
         """
         The pushkey is the notifier id of the home collection for home and owned home child objects. For
@@ -168,7 +163,6 @@ class Notifier(object):
         return self._notifierFactory.pushKeyForId(prefix, id)
 
 
-
 class NotifierFactory(object):
     """
     Notifier Factory
@@ -189,7 +183,6 @@ class NotifierFactory(object):
             from twisted.internet import reactor
         self.reactor = reactor
 
-
     @inlineCallbacks
     def send(self, prefix, id, txn, priority=PushPriority.high):
         """
@@ -202,17 +195,14 @@ class NotifierFactory(object):
             pushPriority=priority.value
         )
 
-
     def newNotifier(self, storeObject):
         return Notifier(self, storeObject)
 
-
     def pushKeyForId(self, prefix, id):
         key = "/%s/%s/%s/" % (prefix, self.hostname, id)
         return key[:255]
 
 
-
 def getPubSubAPSConfiguration(notifierID, config):
     """
     Returns the Apple push notification settings specific to the pushKey
@@ -228,7 +218,7 @@ def getPubSubAPSConfiguration(notifierID, config):
     if applePushSettings.Enabled:
         settings = {}
         settings["APSBundleID"] = applePushSettings[protocol]["Topic"]
-        if config.EnableSSL:
+        if config.EnableSSL or config.BehindTLSProxy:
             url = "https://%s:%s/%s" % (
                 config.ServerHostName, config.SSLPort,
                 applePushSettings.SubscriptionURL)
@@ -244,7 +234,6 @@ def getPubSubAPSConfiguration(notifierID, config):
     return None
 
 
-
 class PushDistributor(object):
     """
     Distributes notifications to the protocol-specific subservices
@@ -258,7 +247,6 @@ class PushDistributor(object):
         # TODO: add an IPushObservers interface?
         self.observers = observers
 
-
     @inlineCallbacks
     def enqueue(self, transaction, pushKey, priority=PushPriority.high):
         """
