@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,8 @@ from calendarserver.push.amppush import AMPPushMaster, AMPPushNotifierProtocol
 from calendarserver.push.amppush import NotificationForID
 from twistedcaldav.test.util import StoreTestCase
 from twisted.internet.task import Clock
-from calendarserver.push.util import PushPriority
+from calendarserver.push.ipush import PushPriority
+
 
 class AMPPushMasterTests(StoreTestCase):
 
@@ -72,9 +73,9 @@ class AMPPushMasterTests(StoreTestCase):
                 (
                     NotificationForID,
                     {
-                        'id'                   : '/CalDAV/localhost/user01/',
-                        'dataChangedTimestamp' : 1354815999,
-                        'priority'             : PushPriority.high.value,
+                        'id': '/CalDAV/localhost/user01/',
+                        'dataChangedTimestamp': 1354815999,
+                        'priority': PushPriority.high.value,
                     }
                 )
             ]
@@ -88,9 +89,9 @@ class AMPPushMasterTests(StoreTestCase):
                 (
                     NotificationForID,
                     {
-                        'id'                   : '/CalDAV/localhost/user01/',
-                        'dataChangedTimestamp' : 1354815999,
-                        'priority'             : PushPriority.high.value,
+                        'id': '/CalDAV/localhost/user01/',
+                        'dataChangedTimestamp': 1354815999,
+                        'priority': PushPriority.high.value,
                     }
                 )
             ]
@@ -104,9 +105,9 @@ class AMPPushMasterTests(StoreTestCase):
                 (
                     NotificationForID,
                     {
-                        'id'                   : '/CalDAV/localhost/user01/',
-                        'dataChangedTimestamp' : 1354815999,
-                        'priority'             : PushPriority.high.value,
+                        'id': '/CalDAV/localhost/user01/',
+                        'dataChangedTimestamp': 1354815999,
+                        'priority': PushPriority.high.value,
                     }
                 )
             ]
@@ -114,7 +115,7 @@ class AMPPushMasterTests(StoreTestCase):
 
         client1.reset()
         client2.reset()
-        client2.unsubscribe("token2", "/CalDAV/localhost/user01/")
+        client2.unsubscribe("/CalDAV/localhost/user01/")
         service.enqueue(
             None, "/CalDAV/localhost/user01/",
             dataChangedTimestamp=dataChangedTimestamp,
@@ -127,9 +128,9 @@ class AMPPushMasterTests(StoreTestCase):
                 (
                     NotificationForID,
                     {
-                        'id'                   : '/CalDAV/localhost/user01/',
-                        'dataChangedTimestamp' : 1354815999,
-                        'priority'             : PushPriority.low.value,
+                        'id': '/CalDAV/localhost/user01/',
+                        'dataChangedTimestamp': 1354815999,
+                        'priority': PushPriority.low.value,
                     }
                 )
             ]
@@ -154,9 +155,9 @@ class AMPPushMasterTests(StoreTestCase):
                 (
                     NotificationForID,
                     {
-                        'id'                   : '/CalDAV/localhost/user01/',
-                        'dataChangedTimestamp' : 1354815999,
-                        'priority'             : PushPriority.medium.value,
+                        'id': '/CalDAV/localhost/user01/',
+                        'dataChangedTimestamp': 1354815999,
+                        'priority': PushPriority.medium.value,
                     }
                 )
             ]
@@ -167,26 +168,23 @@ class AMPPushMasterTests(StoreTestCase):
                 (
                     NotificationForID,
                     {
-                        'id'                   : '/CalDAV/localhost/user01/',
-                        'dataChangedTimestamp' : 1354815999,
-                        'priority'             : PushPriority.medium.value,
+                        'id': '/CalDAV/localhost/user01/',
+                        'dataChangedTimestamp': 1354815999,
+                        'priority': PushPriority.medium.value,
                     }
                 )
             ]
         )
 
 
-
 class TestProtocol(AMPPushNotifierProtocol):
 
     def __init__(self, service):
         super(TestProtocol, self).__init__(service)
         self.reset()
 
-
     def callRemote(self, cls, **kwds):
         self.history.append((cls, kwds))
 
-
     def reset(self):
         self.history = []
