@@ -1,6 +1,6 @@
 # -*- test-case-name: calendarserver.tap.test.test_caldav -*-
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ __all__ = [
     "getRootResource",
     "getSSLPassphrase",
     "MemoryLimitService",
-    "postAlert",
+    "AlertPoster",
     "preFlightChecks",
 ]
 
@@ -33,10 +33,10 @@ from calendarserver.accesslog import DirectoryLogWrapperResource
 from calendarserver.provision.root import RootResource
 from calendarserver.push.applepush import APNSubscriptionResource
 from calendarserver.push.notifier import NotifierFactory
-from calendarserver.push.util import getAPNTopicFromCertificate
+from calendarserver.push.util import getAPNTopicFromConfig
 from calendarserver.tools import diagnose
 from calendarserver.tools.util import checkDirectory
-from calendarserver.webadmin.landing import WebAdminLandingResource
+from calendarserver.webadmin.delegation import WebAdminResource
 from calendarserver.webcal.resource import WebCalendarResource
 
 from socket import fromfd, AF_UNIX, SOCK_STREAM, socketpair
@@ -44,8 +44,7 @@ from subprocess import Popen, PIPE
 
 from twext.enterprise.adbapi2 import ConnectionPool, ConnectionPoolConnection
 from twext.enterprise.adbapi2 import ConnectionPoolClient
-from twext.enterprise.ienterprise import ORACLE_DIALECT
-from twext.enterprise.ienterprise import POSTGRES_DIALECT
+from twext.enterprise.ienterprise import POSTGRES_DIALECT, ORACLE_DIALECT, DatabaseType
 from twext.internet.ssl import ChainingOpenSSLContextFactory
 from twext.python.filepath import CachingFilePath
 from twext.python.filepath import CachingFilePath as FilePath
@@ -59,7 +58,10 @@ from twisted.cred.portal import Portal
 from twisted.internet import reactor as _reactor
 from twisted.internet.defer import inlineCallbacks, returnValue, Deferred, succeed
 from twisted.internet.reactor import addSystemEventTrigger
+from twisted.internet.protocol import Factory
 from twisted.internet.tcp import Connection
+from twisted.protocols import amp
+from twisted.python.procutils import which
 from twisted.python.usage import UsageError
 
 from twistedcaldav.bind import doBind
@@ -72,23 +74,20 @@ from twistedcaldav.directory.digest import QopDigestCredentialFactory
 from twistedcaldav.directory.principal import DirectoryPrincipalProvisioningResource
 from twistedcaldav.directorybackedaddressbook import DirectoryBackedAddressBookResource
 from twistedcaldav.resource import AuthenticationWrapper
+from twistedcaldav.serverinfo import ServerInfoResource
 from twistedcaldav.simpleresource import SimpleResource, SimpleRedirectResource, \
     SimpleUnavailableResource
 from twistedcaldav.stdconfig import config
 from twistedcaldav.timezones import TimezoneCache
 from twistedcaldav.timezoneservice import TimezoneServiceResource
 from twistedcaldav.timezonestdservice import TimezoneStdServiceResource
-from twistedcaldav.util import getPasswordFromKeychain
-from twistedcaldav.util import KeychainAccessError, KeychainPasswordNotFound
 
 from txdav.base.datastore.dbapiclient import DBAPIConnector
 from txdav.base.datastore.subpostgres import PostgresService
 from txdav.caldav.datastore.scheduling.ischedule.dkim import DKIMUtils, DomainKeyResource
 from txdav.caldav.datastore.scheduling.ischedule.localservers import buildServersDB
 from txdav.caldav.datastore.scheduling.ischedule.resource import IScheduleInboxResource
-from txdav.common.datastore.file import CommonDataStore as CommonFileDataStore
 from txdav.common.datastore.podding.resource import ConduitResource
-from txdav.common.datastore.sql import CommonDataStore as CommonSQLDataStore
 from txdav.common.datastore.sql import current_sql_schema
 from txdav.common.datastore.upgrade.sql.upgrade import NotAllowedToUpgrade
 from txdav.dps.client import DirectoryService as DirectoryProxyClientService
@@ -104,12 +103,15 @@ from txweb2.http_headers import Headers
 from txweb2.resource import Resource
 from txweb2.static import File as FileResource
 
+from plistlib import readPlist
 from urllib import quote
 import OpenSSL
 import errno
 import os
 import psutil
 import sys
+import time
+
 
 try:
     from twistedcaldav.authkerb import NegotiateCredentialFactory
@@ -151,6 +153,7 @@ def pgServiceFromConfig(config, subServiceFactory, uid=None, gid=None):
         socketDir=config.Postgres.SocketDirectory,
         socketName=config.Postgres.SocketName,
         listenAddresses=config.Postgres.ListenAddresses,
+        txnTimeoutSeconds=config.Postgres.TxnTimeoutSeconds,
         sharedBuffers=config.Postgres.SharedBuffers,
         maxConnections=config.Postgres.MaxConnections,
         options=config.Postgres.Options,
@@ -161,7 +164,6 @@ def pgServiceFromConfig(config, subServiceFactory, uid=None, gid=None):
     )
 
 
-
 class ConnectionWithPeer(Connection):
 
     connected = True
@@ -169,27 +171,24 @@ class ConnectionWithPeer(Connection):
     def getPeer(self):
         return "<peer: %r %r>" % (self.socket.fileno(), id(self))
 
-
     def getHost(self):
         return "<host: %r %r>" % (self.socket.fileno(), id(self))
 
 
-
-def transactionFactoryFromFD(dbampfd, dialect, paramstyle):
+def transactionFactoryFromFD(dbampfd, dbtype):
     """
     Create a transaction factory from an inherited file descriptor, such as one
     created by L{ConnectionDispenser}.
     """
     skt = fromfd(dbampfd, AF_UNIX, SOCK_STREAM)
     os.close(dbampfd)
-    protocol = ConnectionPoolClient(dialect=dialect, paramstyle=paramstyle)
+    protocol = ConnectionPoolClient(dbtype=dbtype)
     transport = ConnectionWithPeer(skt, protocol)
     protocol.makeConnection(transport)
     transport.startReading()
     return protocol.newTransaction
 
 
-
 class ConnectionDispenser(object):
     """
     A L{ConnectionDispenser} can dispense already-connected file descriptors,
@@ -202,7 +201,6 @@ class ConnectionDispenser(object):
     def __init__(self, connectionPool):
         self.pool = connectionPool
 
-
     def dispense(self):
         """
         Dispense a socket object, already connected to a server, for a client
@@ -218,48 +216,36 @@ class ConnectionDispenser(object):
         return c
 
 
-
 def storeFromConfigWithoutDPS(config, txnFactory):
     store = storeFromConfig(config, txnFactory, None)
     directory = directoryFromConfig(config, store)
-    # if config.DirectoryProxy.InProcessCachingSeconds:
-    #     directory = CachingDirectoryService(
-    #         directory,
-    #         expireSeconds=config.DirectoryProxy.InProcessCachingSeconds
-    #     )
     store.setDirectoryService(directory)
     return store
 
 
-
 def storeFromConfigWithDPSClient(config, txnFactory):
     store = storeFromConfig(config, txnFactory, None)
     directory = DirectoryProxyClientService(config.DirectoryRealmName)
     if config.Servers.Enabled:
         directory.setServersDB(buildServersDB(config.Servers.MaxClients))
-    if config.DirectoryProxy.InProcessCachingSeconds:
+    if config.DirectoryCaching.CachingSeconds:
         directory = CachingDirectoryService(
             directory,
-            expireSeconds=config.DirectoryProxy.InProcessCachingSeconds
+            expireSeconds=config.DirectoryCaching.CachingSeconds,
+            lookupsBetweenPurges=config.DirectoryCaching.LookupsBetweenPurges,
+            negativeCaching=config.DirectoryCaching.NegativeCachingEnabled,
         )
     store.setDirectoryService(directory)
     return store
 
 
-
 def storeFromConfigWithDPSServer(config, txnFactory):
     store = storeFromConfig(config, txnFactory, None)
     directory = directoryFromConfig(config, store)
-    # if config.DirectoryProxy.InSidecarCachingSeconds:
-    #     directory = CachingDirectoryService(
-    #         directory,
-    #         expireSeconds=config.DirectoryProxy.InSidecarCachingSeconds
-    #     )
     store.setDirectoryService(directory)
     return store
 
 
-
 def storeFromConfig(config, txnFactory, directoryService):
     """
     Produce an L{IDataStore} from the given configuration, transaction factory,
@@ -282,11 +268,12 @@ def storeFromConfig(config, txnFactory, directoryService):
     if quota == 0:
         quota = None
     if txnFactory is not None:
-        if config.EnableSSL:
+        if config.EnableSSL or config.BehindTLSProxy:
             uri = "https://{config.ServerHostName}:{config.SSLPort}".format(config=config)
         else:
             uri = "https://{config.ServerHostName}:{config.HTTPPort}".format(config=config)
         attachments_uri = uri + "/calendars/__uids__/%(home)s/dropbox/%(dropbox_id)s/%(name)s"
+        from txdav.common.datastore.sql import CommonDataStore as CommonSQLDataStore
         store = CommonSQLDataStore(
             txnFactory, notifierFactories,
             directoryService,
@@ -305,6 +292,7 @@ def storeFromConfig(config, txnFactory, directoryService):
             cacheExpireSeconds=config.QueryCaching.ExpireSeconds
         )
     else:
+        from txdav.common.datastore.file import CommonDataStore as CommonFileDataStore
         store = CommonFileDataStore(
             FilePath(config.DocumentRoot),
             notifierFactories, directoryService,
@@ -319,7 +307,6 @@ def storeFromConfig(config, txnFactory, directoryService):
     return store
 
 
-
 # MOVE2WHO -- should we move this class somewhere else?
 class PrincipalCredentialChecker(object):
     credentialInterfaces = (IPrincipalCredentials,)
@@ -384,7 +371,6 @@ class PrincipalCredentialChecker(object):
                 )
 
 
-
 def getRootResource(config, newStore, resources=None):
     """
     Set up directory service and resource hierarchy based on config.
@@ -417,10 +403,11 @@ def getRootResource(config, newStore, resources=None):
     timezoneServiceResourceClass = TimezoneServiceResource
     timezoneStdServiceResourceClass = TimezoneStdServiceResource
     webCalendarResourceClass = WebCalendarResource
-    webAdminResourceClass = WebAdminLandingResource
+    webAdminResourceClass = WebAdminResource
     addressBookResourceClass = DirectoryAddressBookHomeProvisioningResource
     directoryBackedAddressBookResourceClass = DirectoryBackedAddressBookResource
     apnSubscriptionResourceClass = APNSubscriptionResource
+    serverInfoResourceClass = ServerInfoResource
     principalResourceClass = DirectoryPrincipalProvisioningResource
     controlResourceClass = ControlAPIResource
 
@@ -579,7 +566,7 @@ def getRootResource(config, newStore, resources=None):
             (config.Scheduling.iSchedule.Enabled, "ischedule", "/ischedule"),
         ):
             if enabled:
-                if config.EnableSSL:
+                if config.EnableSSL or config.BehindTLSProxy:
                     scheme = "https"
                     port = config.SSLPort
                 else:
@@ -743,6 +730,17 @@ def getRootResource(config, newStore, resources=None):
         apnResource = apnSubscriptionResourceClass(root, newStore)
         root.putChild(apnConfig["SubscriptionURL"], apnResource)
 
+    # Server info document
+    if config.EnableServerInfo:
+        log.info(
+            "Setting up server-info resource: {cls}",
+            cls=serverInfoResourceClass)
+
+        serverInfo = serverInfoResourceClass(
+            root,
+        )
+        root.putChild("server-info", serverInfo)
+
     #
     # Configure ancillary data
     #
@@ -796,7 +794,6 @@ def getRootResource(config, newStore, resources=None):
     return logWrapper
 
 
-
 def getDBPool(config):
     """
     Inspect configuration to determine what database connection pool
@@ -812,7 +809,7 @@ def getDBPool(config):
     pool = None
     if config.DBAMPFD:
         txnFactory = transactionFactoryFromFD(
-            int(config.DBAMPFD), dialect, paramstyle
+            int(config.DBAMPFD), DatabaseType(dialect, paramstyle, config.DBFeatures)
         )
     elif not config.UseDatabase:
         txnFactory = None
@@ -825,8 +822,7 @@ def getDBPool(config):
         else:
             connectionFactory = DBAPIConnector.connectorFor(config.DBType, **config.DatabaseConnection).connect
 
-        pool = ConnectionPool(connectionFactory, dialect=dialect,
-                              paramstyle=paramstyle,
+        pool = ConnectionPool(connectionFactory, dbtype=DatabaseType(dialect, paramstyle, config.DBFeatures),
                               maxConnections=config.MaxDBConnectionsPerPool)
         txnFactory = pool.connection
     else:
@@ -837,7 +833,6 @@ def getDBPool(config):
     return (pool, txnFactory)
 
 
-
 class FakeRequest(object):
 
     def __init__(self, rootResource, method, path, uri='/', transaction=None):
@@ -851,7 +846,6 @@ class FakeRequest(object):
         if transaction is not None:
             self._newStoreTransaction = transaction
 
-
     @inlineCallbacks
     def _getChild(self, resource, segments):
         if not segments:
@@ -860,7 +854,6 @@ class FakeRequest(object):
         child, remaining = (yield resource.locateChild(self, segments))
         returnValue((yield self._getChild(child, remaining)))
 
-
     @inlineCallbacks
     def locateResource(self, url):
         url = url.strip("/")
@@ -870,7 +863,6 @@ class FakeRequest(object):
             self._rememberResource(resource, url)
         returnValue(resource)
 
-
     @inlineCallbacks
     def locateChildResource(self, parent, childName):
         if parent is None or childName is None:
@@ -885,20 +877,17 @@ class FakeRequest(object):
             self._rememberResource(resource, url)
         returnValue(resource)
 
-
     def _rememberResource(self, resource, url):
         self._resourcesByURL[url] = resource
         self._urlsByResource[resource] = url
         return resource
 
-
     def _forgetResource(self, resource, url):
         if url in self._resourcesByURL:
             del self._resourcesByURL[url]
         if resource in self._urlsByResource:
             del self._urlsByResource[resource]
 
-
     def urlForResource(self, resource):
         url = self._urlsByResource.get(resource, None)
         if url is None:
@@ -907,12 +896,10 @@ class FakeRequest(object):
             raise NoURLForResourceError(resource)
         return url
 
-
     def addResponseFilter(self, *args, **kwds):
         pass
 
 
-
 def memoryForPID(pid, residentOnly=True):
     """
     Return the amount of memory in use for the given process.  If residentOnly is True,
@@ -924,11 +911,10 @@ def memoryForPID(pid, residentOnly=True):
     @return: Memory used by process in bytes
     @rtype: C{int}
     """
-    memoryInfo = psutil.Process(pid).get_memory_info()
+    memoryInfo = psutil.Process(pid).memory_info()
     return memoryInfo.rss if residentOnly else memoryInfo.vms
 
 
-
 class MemoryLimitService(Service, object):
     """
     A service which when paired with a DelayedStartupProcessMonitor will periodically
@@ -959,7 +945,6 @@ class MemoryLimitService(Service, object):
         # Unit tests can swap out _memoryForPID
         self._memoryForPID = memoryForPID
 
-
     def startService(self):
         """
         Start scheduling the memory checks
@@ -967,7 +952,6 @@ class MemoryLimitService(Service, object):
         super(MemoryLimitService, self).startService()
         self._delayedCall = self._reactor.callLater(self._seconds, self.checkMemory)
 
-
     def stopService(self):
         """
         Stop checking memory
@@ -977,7 +961,6 @@ class MemoryLimitService(Service, object):
             self._delayedCall.cancel()
             self._delayedCall = None
 
-
     def checkMemory(self):
         """
         Stop any processes monitored by our paired processMonitor whose resident
@@ -1010,7 +993,6 @@ class MemoryLimitService(Service, object):
             self._delayedCall = self._reactor.callLater(self._seconds, self.checkMemory)
 
 
-
 def checkDirectories(config):
     """
     Make sure that various key directories exist (and create if needed)
@@ -1078,7 +1060,6 @@ def checkDirectories(config):
     )
 
 
-
 class Stepper(object):
     """
     Manages the sequential, deferred execution of "steps" which are objects
@@ -1114,7 +1095,6 @@ class Stepper(object):
         self.result = None
         self.running = False
 
-
     def addStep(self, step):
         """
         Adds a step object to the ordered list of steps
@@ -1129,29 +1109,17 @@ class Stepper(object):
         self.steps.append(step)
         return self
 
-
     def defaultStepWithResult(self, result):
         return succeed(result)
 
-
-    def defaultStepWithFailure(self, failure):
+    def defaultStepWithFailure(self, failure, step):
         if failure.type not in (
             NotAllowedToUpgrade, ConfigurationError,
             OpenSSL.SSL.Error
         ):
-            log.failure("Step failure", failure=failure)
+            log.failure("Step failure: {name}", name=step.__class__.__name__, failure=failure)
         return failure
 
-    # def protectStep(self, callback):
-    #     def _protected(result):
-    #         try:
-    #             return callback(result)
-    #         except Exception, e:
-    #             # TODO: how to turn Exception into Failure
-    #             return Failure()
-    #     return _protected
-
-
     def start(self, result=None):
         """
         Begin executing the added steps in sequence.  If a step object
@@ -1174,11 +1142,13 @@ class Stepper(object):
                 callBack = self.defaultStepWithResult
             if hasattr(step, "stepWithFailure"):
                 errBack = step.stepWithFailure
+                errbackArgs = ()
             else:
                 errBack = self.defaultStepWithFailure
+                errbackArgs = (step,)
 
             # Add callbacks to the Deferred
-            self.deferred.addCallbacks(callBack, errBack)
+            self.deferred.addCallbacks(callBack, errBack, errbackArgs=errbackArgs)
 
         # Get things going
         self.deferred.callback(result)
@@ -1186,7 +1156,6 @@ class Stepper(object):
         return self.deferred
 
 
-
 def requestShutdown(programPath, reason):
     """
     Log the shutdown reason and call the shutdown-requesting program.
@@ -1212,7 +1181,6 @@ def requestShutdown(programPath, reason):
     ).communicate()
 
 
-
 def preFlightChecks(config):
     """
     Perform checks prior to spawning any processes.  Returns True if the checks
@@ -1253,7 +1221,6 @@ def preFlightChecks(config):
     return True
 
 
-
 def verifyConfig(config):
     """
     At least one of EnableCalDAV or EnableCardDAV must be True
@@ -1265,7 +1232,6 @@ def verifyConfig(config):
     return False, "Neither CalDAV nor CardDAV are enabled"
 
 
-
 def verifyServerRoot(config):
     """
     Ensure server root is not on a phantom volume
@@ -1281,33 +1247,50 @@ def verifyServerRoot(config):
     return True, "ServerRoot is ok"
 
 
-
 def verifyTLSCertificate(config):
     """
     If a TLS certificate is configured, make sure it exists, is non empty,
     and that it's valid.
     """
 
-    if config.SSLCertificate:
-        if not os.path.exists(config.SSLCertificate):
-            message = (
-                "The configured TLS certificate ({cert}) is missing".format(
-                    cert=config.SSLCertificate
-                )
-            )
-            postAlert("MissingCertificateAlert", ["path", config.SSLCertificate])
-            return False, message
-    else:
-        return True, "TLS disabled"
+    if hasattr(OpenSSL, "__SecureTransport__"):
+        if config.SSLKeychainIdentity:
+            # Fall through to see if we can load the identity from the keychain
+            certificate_title = "Keychain: {}".format(config.SSLKeychainIdentity)
 
-    length = os.stat(config.SSLCertificate).st_size
-    if length == 0:
-            message = (
-                "The configured TLS certificate ({cert}) is empty".format(
-                    cert=config.SSLCertificate
+            error = OpenSSL.crypto.check_keychain_identity(config.SSLKeychainIdentity)
+            if error:
+                message = (
+                    "The configured TLS Keychain Identity ({cert}) cannot be used: {reason}".format(
+                        cert=certificate_title,
+                        reason=error
+                    )
                 )
-            )
-            return False, message
+                return False, message
+        else:
+            return True, "TLS disabled"
+    else:
+        if config.SSLCertificate:
+            if not os.path.exists(config.SSLCertificate):
+                message = (
+                    "The configured TLS certificate ({cert}) is missing".format(
+                        cert=config.SSLCertificate
+                    )
+                )
+                AlertPoster.postAlert("MissingCertificateAlert", 0, ["path", config.SSLCertificate])
+                return False, message
+
+            length = os.stat(config.SSLCertificate).st_size
+            if length == 0:
+                message = (
+                    "The configured TLS certificate ({cert}) is empty".format(
+                        cert=config.SSLCertificate
+                    )
+                )
+                return False, message
+            certificate_title = config.SSLCertificate
+        else:
+            return True, "TLS disabled"
 
     try:
         ChainingOpenSSLContextFactory(
@@ -1315,22 +1298,30 @@ def verifyTLSCertificate(config):
             config.SSLCertificate,
             certificateChainFile=config.SSLAuthorityChain,
             passwdCallback=getSSLPassphrase,
+            keychainIdentity=config.SSLKeychainIdentity,
             sslmethod=getattr(OpenSSL.SSL, config.SSLMethod),
             ciphers=config.SSLCiphers.strip()
         )
     except Exception as e:
-        message = (
-            "The configured TLS certificate ({cert}) cannot be used: {reason}".format(
-                cert=config.SSLCertificate,
-                reason=str(e)
+        if hasattr(OpenSSL, "__SecureTransport__"):
+            message = (
+                "The configured TLS Keychain Identity ({cert}) cannot be used: {reason}".format(
+                    cert=certificate_title,
+                    reason=str(e)
+                )
+            )
+        else:
+            message = (
+                "The configured TLS certificate ({cert}) cannot be used: {reason}".format(
+                    cert=certificate_title,
+                    reason=str(e)
+                )
             )
-        )
         return False, message
 
     return True, "TLS enabled"
 
 
-
 def verifyAPNSCertificate(config):
     """
     If APNS certificates are configured, make sure they're valid.
@@ -1344,39 +1335,18 @@ def verifyAPNSCertificate(config):
         ):
             protoConfig = config.Notifications.Services.APNS[protocol]
 
-            # Verify the cert exists
-            if not os.path.exists(protoConfig.CertificatePath):
-                message = (
-                    "The {proto} APNS certificate ({cert}) is missing".format(
-                        proto=protocol,
-                        cert=protoConfig.CertificatePath
-                    )
-                )
-                postAlert("PushNotificationCertificateAlert", [])
-                return False, message
+            if not protoConfig.Enabled:
+                continue
 
-            # Verify we can extract the topic
-            if not protoConfig.Topic:
-                topic = getAPNTopicFromCertificate(protoConfig.CertificatePath)
-                protoConfig.Topic = topic
-            if not protoConfig.Topic:
-                postAlert("PushNotificationCertificateAlert", [])
-                message = "Cannot extract APN topic"
-                return False, message
+            if not hasattr(OpenSSL, "__SecureTransport__"):
+                if not checkCertExpiration(protoConfig.CertificatePath):
+                    return False, "APNS certificate expired {}".format(protoConfig.CertificatePath)
 
-            # Verify we can acquire the passphrase
-            if not protoConfig.Passphrase:
-                try:
-                    passphrase = getPasswordFromKeychain(accountName)
-                    protoConfig.Passphrase = passphrase
-                except KeychainAccessError:
-                    # The system doesn't support keychain
-                    pass
-                except KeychainPasswordNotFound:
-                    # The password doesn't exist in the keychain.
-                    postAlert("PushNotificationCertificateAlert", [])
-                    message = "Cannot retrieve APN passphrase from keychain"
-                    return False, message
+            try:
+                getAPNTopicFromConfig(protocol, accountName, protoConfig)
+            except ValueError as e:
+                AlertPoster.postAlert("PushNotificationCertificateAlert", 0, [])
+                return False, str(e)
 
             # Let OpenSSL try to use the cert
             try:
@@ -1390,17 +1360,27 @@ def verifyAPNSCertificate(config):
                     protoConfig.CertificatePath,
                     certificateChainFile=protoConfig.AuthorityChainPath,
                     passwdCallback=passwdCallback,
+                    keychainIdentity=protoConfig.KeychainIdentity,
                     sslmethod=getattr(OpenSSL.SSL, "TLSv1_METHOD"),
                 )
             except Exception as e:
-                message = (
-                    "The {proto} APNS certificate ({cert}) cannot be used: {reason}".format(
-                        proto=protocol,
-                        cert=protoConfig.CertificatePath,
-                        reason=str(e)
+                if hasattr(OpenSSL, "__SecureTransport__"):
+                    message = (
+                        "The {proto} APNS Keychain Identity ({cert}) cannot be used: {reason}".format(
+                            proto=protocol,
+                            cert=protoConfig.KeychainIdentity,
+                            reason=str(e)
+                        )
                     )
-                )
-                postAlert("PushNotificationCertificateAlert", [])
+                else:
+                    message = (
+                        "The {proto} APNS certificate ({cert}) cannot be used: {reason}".format(
+                            proto=protocol,
+                            cert=protoConfig.CertificatePath,
+                            reason=str(e)
+                        )
+                    )
+                AlertPoster.postAlert("PushNotificationCertificateAlert", 0, [])
                 return False, message
 
         return True, "APNS enabled"
@@ -1409,6 +1389,27 @@ def verifyAPNSCertificate(config):
         return True, "APNS disabled"
 
 
+def checkCertExpiration(certPath):
+    """
+    See if the given certificate is expired.
+
+    @param certPath: the path of the certificate
+    @type certPath: C{str}
+    @return: True if the cert has not expired (or we can't check because we
+        can't find the openssl command line utility); False otherwise
+    """
+
+    try:
+        opensslTool = which("openssl")[0]
+        args = [opensslTool, "x509", "-checkend", "0", "-noout", "-in", certPath]
+        child = Popen(args=args, stdout=PIPE, stderr=PIPE)
+        child.communicate()
+        exitStatus = child.returncode
+        return exitStatus == 0
+    except IndexError:
+        # We can't check
+        return True
+
 
 def getSSLPassphrase(*ignored):
 
@@ -1440,8 +1441,7 @@ def getSSLPassphrase(*ignored):
         config.SSLPassPhraseDialog and
         os.path.isfile(config.SSLPassPhraseDialog)
     ):
-        sslPrivKey = open(config.SSLPrivateKey)
-        try:
+        with open(config.SSLPrivateKey) as sslPrivKey:
             keyType = None
             for line in sslPrivKey.readlines():
                 if "-----BEGIN RSA PRIVATE KEY-----" in line:
@@ -1450,8 +1450,6 @@ def getSSLPassphrase(*ignored):
                 elif "-----BEGIN DSA PRIVATE KEY-----" in line:
                     keyType = "DSA"
                     break
-        finally:
-            sslPrivKey.close()
 
         if keyType is None:
             log.error(
@@ -1480,22 +1478,202 @@ def getSSLPassphrase(*ignored):
     return None
 
 
+#
+# Server Alert Posting
+#
 
-def postAlert(alertType, args):
-    if (
-        config.AlertPostingProgram and
-        os.path.exists(config.AlertPostingProgram)
-    ):
-        try:
-            commandLine = [config.AlertPostingProgram, alertType]
-            commandLine.extend(args)
-            Popen(
-                commandLine,
-                stdout=PIPE,
-                stderr=PIPE,
-            ).communicate()
-        except Exception, e:
-            log.error(
-                "Could not post alert: {alertType} {args} ({error})",
-                alertType=alertType, args=args, error=e
+class AlertPoster(object):
+    """
+    Encapsulates the posting of server alerts via a singleton which can be
+    configured differently depending on whether you want to directly spawn
+    the external alert program from this process, or send an AMP request to
+    another process to spawn.  Workers should call setupForWorker( ), and
+    then calls to postAlert( ) will send an AMP message to the master.  The
+    master should call setupForMaster( ), and then calls to postAlert( )
+    within the master will spawn the external alert program.
+    """
+
+    # Control socket message-routing constants
+    ALERT_ROUTE = "alert"
+
+    _alertPoster = None
+
+    @classmethod
+    def setupForMaster(cls, controlSocket):
+        cls._alertPoster = cls()
+        AMPAlertReceiver(controlSocket)
+
+    @classmethod
+    def setupForWorker(cls, controlSocket):
+        cls._alertPoster = cls(controlSocket)
+
+    @classmethod
+    def _getAlertPoster(cls):
+        if cls._alertPoster is None:
+            cls._alertPoster = cls()
+        return cls._alertPoster
+
+    def __init__(self, controlSocket=None):
+
+        if controlSocket is None:
+            self.sender = None
+        else:
+            self.sender = AMPAlertSender(controlSocket)
+
+    @classmethod
+    def postAlert(cls, alertType, ignoreWithinSeconds, args):
+
+        poster = cls._getAlertPoster()
+
+        if not config.AlertPostingProgram:
+            return
+
+        if not os.path.exists(config.AlertPostingProgram):
+            return
+
+        if ignoreWithinSeconds:
+            seconds = cls.secondsSinceLastPost(alertType)
+            if seconds < ignoreWithinSeconds:
+                return
+
+        if poster.sender is None:
+            # Just do it
+
+            cls.recordTimeStamp(alertType)
+
+            try:
+                commandLine = [config.AlertPostingProgram, alertType]
+                commandLine.extend(args)
+                Popen(
+                    commandLine,
+                    stdout=PIPE,
+                    stderr=PIPE,
+                ).communicate()
+            except Exception, e:
+                log.error(
+                    "Could not post alert: {alertType} {args} ({error})",
+                    alertType=alertType, args=args, error=e
+                )
+
+        else:
+            # Send request to master over AMP
+            poster.sender.sendAlert(alertType, args)
+
+    @classmethod
+    def secondsSinceLastPost(cls, alertType, timestampsDirectory=None, now=None):
+        if timestampsDirectory is None:
+            timestampsDirectory = config.DataRoot
+        if now is None:
+            now = int(time.time())
+
+        dirFP = FilePath(timestampsDirectory)
+        childFP = dirFP.child(".{}.timestamp".format(alertType))
+        if not childFP.exists():
+            timestamp = 0
+        else:
+            with childFP.open() as child:
+                try:
+                    line = child.readline().strip()
+                    timestamp = int(line)
+                except:
+                    timestamp = 0
+        return now - timestamp
+
+    @classmethod
+    def recordTimeStamp(cls, alertType, timestampsDirectory=None, now=None):
+        if timestampsDirectory is None:
+            timestampsDirectory = config.DataRoot
+        if now is None:
+            now = int(time.time())
+
+        dirFP = FilePath(timestampsDirectory)
+        childFP = dirFP.child(".{}.timestamp".format(alertType))
+        childFP.setContent(str(now))
+
+
+class AMPAlertSendingFactory(Factory):
+
+    def __init__(self, sender):
+        self.sender = sender
+
+    def buildProtocol(self, addr):
+        protocol = amp.AMP()
+        self.sender.protocol = protocol
+        return protocol
+
+
+class AMPAlertSender(object):
+    """
+    Runs in the workers, sends alerts to the master via AMP
+    """
+
+    def __init__(self, controlSocket=None, protocol=None):
+        self.protocol = protocol
+        if controlSocket is not None:
+            controlSocket.addFactory(AlertPoster.ALERT_ROUTE, AMPAlertSendingFactory(self))
+
+    def sendAlert(self, alertType, args):
+        return self.protocol.callRemote(
+            PostAlert, alertType=alertType, args=args
+        )
+
+
+class AMPAlertReceiverFactory(Factory):
+
+    def buildProtocol(self, addr):
+        return AMPAlertProtocol()
+
+
+class AMPAlertReceiver(object):
+    """
+    Runs in the master, receives alerts from workers, executes the alert posting program
+    """
+
+    def __init__(self, controlSocket):
+        if controlSocket is not None:
+            # Set up the listener which gets alerts from the slaves
+            controlSocket.addFactory(
+                AlertPoster.ALERT_ROUTE, AMPAlertReceiverFactory()
             )
+
+
+class PostAlert(amp.Command):
+    arguments = [
+        ('alertType', amp.String()),
+        ('args', amp.ListOf(amp.String())),
+    ]
+    response = [
+        ('status', amp.String()),
+    ]
+
+
+class AMPAlertProtocol(amp.AMP):
+    """
+    Defines the AMP protocol for sending alerts from worker to master
+    """
+
+    @PostAlert.responder
+    def postAlert(self, alertType, args):
+        """
+        The "PostAlert" handler in the master
+        """
+        AlertPoster.postAlert(alertType, 0, args)
+        return {
+            "status": "OK"
+        }
+
+
+def serverRootLocation():
+    """
+    Return the ServerRoot value from the OS X preferences plist.  If plist not
+    present, return empty string.
+
+    @rtype: C{unicode}
+    """
+    defaultPlistPath = "/Library/Server/Preferences/Calendar.plist"
+    serverRoot = u""
+    if os.path.exists(defaultPlistPath):
+        serverRoot = readPlist(defaultPlistPath).get("ServerRoot", serverRoot)
+    if isinstance(serverRoot, str):
+        serverRoot = serverRoot.decode("utf-8")
+    return serverRoot
