@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -51,6 +51,7 @@ WRITABLE_CONFIG_KEYS = [
     "Authentication.Kerberos.AllowedOverWireUnencrypted",
     "Authentication.Kerberos.Enabled",
     "Authentication.Wiki.Enabled",
+    "BehindTLSProxy",
     "DefaultLogLevel",
     "DirectoryAddressBook.params.queryPeopleRecords",
     "DirectoryAddressBook.params.queryUserRecords",
@@ -79,6 +80,7 @@ WRITABLE_CONFIG_KEYS = [
     "SSLCertificate",
     "SSLPort",
     "SSLPrivateKey",
+    "SSLKeychainIdentity",
 ]
 
 READONLY_CONFIG_KEYS = [
@@ -257,7 +259,7 @@ def setServiceState(service, state):
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE,
     )
-    output, error = child.communicate()
+    _ignore_output, error = child.communicate()
     if child.returncode:
         sys.stdout.write(
             "Error from serverctl: %d, %s" % (child.returncode, error)
@@ -291,7 +293,6 @@ def setReverseProxies():
     child.communicate()
 
 
-
 def processArgs(writable, args, restart=True):
     """
     Perform the read/write operations requested in the command line args.
@@ -334,7 +335,7 @@ def processArgs(writable, args, restart=True):
         try:
             plist = readPlistFromString(rawInput)
             # Note: values in plist will already be unicode
-        except xml.parsers.expat.ExpatError, e:
+        except xml.parsers.expat.ExpatError, e:  # @UndefinedVariable
             respondWithError(str(e))
             return
 
@@ -349,7 +350,6 @@ def processArgs(writable, args, restart=True):
         runner.run()
 
 
-
 class Runner(object):
 
     """
@@ -365,7 +365,6 @@ class Runner(object):
         self.commands = commands
         self.quiet = quiet
 
-
     def validate(self):
         """
         Validate all the commands by making sure this class implements
@@ -384,7 +383,6 @@ class Runner(object):
                 return False
         return True
 
-
     def run(self):
         """
         Find the appropriate method for each command and call them.
@@ -402,7 +400,6 @@ class Runner(object):
             respondWithError("Command failed: '%s'" % (str(e),))
             raise
 
-
     def command_readConfig(self, command):
         """
         Return current configuration
@@ -421,7 +418,6 @@ class Runner(object):
                 setKeyPath(result, keyPath, value)
         respond(command, result)
 
-
     def command_writeConfig(self, command):
         """
         Write config to secondary, writable plist
@@ -446,7 +442,6 @@ class Runner(object):
                 self.command_readConfig(command)
 
 
-
 def setKeyPath(parent, keyPath, value):
     """
     Allows the setting of arbitrary nested dictionary keys via a single
@@ -482,7 +477,6 @@ def setKeyPath(parent, keyPath, value):
     return original
 
 
-
 def getKeyPath(parent, keyPath):
     """
     Allows the getting of arbitrary nested dictionary keys via a single
@@ -506,7 +500,6 @@ def getKeyPath(parent, keyPath):
     return parent.get(parts[-1], None)
 
 
-
 def flattenDictionary(dictionary, current=""):
     """
     Returns a generator of (keyPath, value) tuples for the given dictionary,
@@ -527,7 +520,6 @@ def flattenDictionary(dictionary, current=""):
             yield (current + key, value)
 
 
-
 def restartService(pidFilename):
     """
     Given the path to a PID file, sends a HUP signal to the contained pid
@@ -537,9 +529,8 @@ def restartService(pidFilename):
     @type pidFilename: C{str}
     """
     if os.path.exists(pidFilename):
-        pidFile = open(pidFilename, "r")
-        pid = pidFile.read().strip()
-        pidFile.close()
+        with open(pidFilename, "r") as pidFile:
+            pid = pidFile.read().strip()
         try:
             pid = int(pid)
         except ValueError:
@@ -550,7 +541,6 @@ def restartService(pidFilename):
             pass
 
 
-
 class WritableConfig(object):
     """
     A wrapper around a Config object which allows writing of values.  The idea
@@ -572,7 +562,6 @@ class WritableConfig(object):
         self.currentConfigSubset = ConfigDict()
         self.dirty = False
 
-
     def set(self, data):
         """
         Merges data into a ConfigDict of changes intended to be saved to disk
@@ -586,7 +575,6 @@ class WritableConfig(object):
         mergeData(self.currentConfigSubset, data)
         self.dirty = True
 
-
     def delete(self, key):
         """
         Deletes the specified key
@@ -600,7 +588,6 @@ class WritableConfig(object):
         except:
             pass
 
-
     def read(self):
         """
         Reads in the data contained in the writable plist file.
@@ -612,11 +599,9 @@ class WritableConfig(object):
         else:
             self.currentConfigSubset = ConfigDict()
 
-
     def toString(self):
         return plistlib.writePlistToString(self.currentConfigSubset)
 
-
     def save(self, restart=False):
         """
         Writes any outstanding changes to the writable plist file.  Optionally
@@ -633,7 +618,6 @@ class WritableConfig(object):
             if restart:
                 restartService(self.config.PIDFile)
 
-
     @classmethod
     def convertToValue(cls, string):
         """
@@ -658,11 +642,9 @@ class WritableConfig(object):
         return value
 
 
-
 def respond(command, result):
     sys.stdout.write(writePlistToString({'command': command['command'], 'result': result}))
 
 
-
 def respondWithError(msg, status=1):
     sys.stdout.write(writePlistToString({'error': msg, }))
