@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- test-case-name: calendarserver.tools.test.test_calverify -*-
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ from calendarserver.tools.cmdline import utilityMain, WorkerService
 from pycalendar.datetime import DateTime
 
 from twext.enterprise.dal.syntax import Select, Parameter, Count, Delete, \
-    Constant
+    ALL_COLUMNS
 from twext.who.idirectory import RecordType
 
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed
@@ -49,6 +49,7 @@ import os
 import sys
 import traceback
 
+
 def usage(e=None):
     if e:
         print(e)
@@ -72,6 +73,7 @@ description = '\n'.join(
     )
 )
 
+
 class DBInspectOptions(Options):
     """
     Command-line options for 'calendarserver_dbinspect'
@@ -94,14 +96,12 @@ class DBInspectOptions(Options):
         self.outputName = '-'
 
 
-
 @inlineCallbacks
 def UserNameFromUID(txn, uid):
     record = yield txn.directoryService().recordWithUID(uid)
     returnValue(record.shortNames[0] if record else "(%s)" % (uid,))
 
 
-
 @inlineCallbacks
 def UIDFromInput(txn, value):
     try:
@@ -119,7 +119,6 @@ def UIDFromInput(txn, value):
     returnValue(record.uid if record else None)
 
 
-
 class Cmd(object):
 
     _name = None
@@ -128,12 +127,10 @@ class Cmd(object):
     def name(cls):
         return cls._name
 
-
     def doIt(self, txn):
         raise NotImplementedError
 
 
-
 class TableSizes(Cmd):
 
     _name = "Show Size of each Table"
@@ -142,7 +139,7 @@ class TableSizes(Cmd):
     def doIt(self, txn):
 
         results = []
-        for dbtable in schema.model.tables: #@UndefinedVariable
+        for dbtable in schema.model.tables:  # @UndefinedVariable
             dbtable = getattr(schema, dbtable.name)
             count = yield self.getTableSize(txn, dbtable)
             results.append((dbtable.model.name, count,))
@@ -160,17 +157,15 @@ class TableSizes(Cmd):
         print("Database Tables (total=%d):\n" % (len(results),))
         table.printTable()
 
-
     @inlineCallbacks
     def getTableSize(self, txn, dbtable):
         rows = (yield Select(
-            [Count(Constant(1)), ],
+            [Count(ALL_COLUMNS), ],
             From=dbtable,
         ).on(txn))
         returnValue(rows[0][0])
 
 
-
 class CalendarHomes(Cmd):
 
     _name = "List Calendar Homes"
@@ -197,7 +192,6 @@ class CalendarHomes(Cmd):
         print("Calendar Homes (total=%d, missing=%d):\n" % (len(uids), missing,))
         table.printTable()
 
-
     @inlineCallbacks
     def getAllHomeUIDs(self, txn):
         ch = schema.CALENDAR_HOME
@@ -208,7 +202,6 @@ class CalendarHomes(Cmd):
         returnValue(tuple([row[0] for row in rows]))
 
 
-
 class CalendarHomesSummary(Cmd):
 
     _name = "List Calendar Homes with summary information"
@@ -253,7 +246,6 @@ class CalendarHomesSummary(Cmd):
         print("Calendars with resource count (total=%d):\n" % (len(results),))
         table.printTable()
 
-
     @inlineCallbacks
     def getCalendars(self, txn):
         ch = schema.CALENDAR_HOME
@@ -274,7 +266,6 @@ class CalendarHomesSummary(Cmd):
         returnValue(tuple(rows))
 
 
-
 class Calendars(Cmd):
 
     _name = "List Calendars"
@@ -300,7 +291,6 @@ class Calendars(Cmd):
         print("Calendars with resource count (total=%d):\n" % (len(uids),))
         table.printTable()
 
-
     @inlineCallbacks
     def getCalendars(self, txn):
         ch = schema.CALENDAR_HOME
@@ -321,7 +311,6 @@ class Calendars(Cmd):
         returnValue(tuple(rows))
 
 
-
 class CalendarsByOwner(Cmd):
 
     _name = "List Calendars for Owner UID/Short Name"
@@ -354,7 +343,6 @@ class CalendarsByOwner(Cmd):
         print("Calendars with resource count (total=%d):\n" % (len(uids),))
         table.printTable()
 
-
     @inlineCallbacks
     def getCalendars(self, txn, uid):
         ch = schema.CALENDAR_HOME
@@ -377,7 +365,6 @@ class CalendarsByOwner(Cmd):
         returnValue(tuple(rows))
 
 
-
 class Events(Cmd):
 
     _name = "List Events"
@@ -405,7 +392,6 @@ class Events(Cmd):
         print("Calendar events (total=%d):\n" % (len(uids),))
         table.printTable()
 
-
     @inlineCallbacks
     def getEvents(self, txn):
         ch = schema.CALENDAR_HOME
@@ -427,7 +413,6 @@ class Events(Cmd):
         returnValue(tuple(rows))
 
 
-
 class EventsByCalendar(Cmd):
 
     _name = "List Events for a specific calendar"
@@ -458,7 +443,6 @@ class EventsByCalendar(Cmd):
         print("Calendar events (total=%d):\n" % (len(uids),))
         table.printTable()
 
-
     @inlineCallbacks
     def getEvents(self, txn, rid):
         ch = schema.CALENDAR_HOME
@@ -480,7 +464,6 @@ class EventsByCalendar(Cmd):
         returnValue(tuple(rows))
 
 
-
 class EventDetails(Cmd):
     """
     Base class for common event details commands.
@@ -501,7 +484,6 @@ class EventDetails(Cmd):
         table.printTable()
         print(data)
 
-
     @inlineCallbacks
     def getEventData(self, txn, whereClause, whereParams):
         ch = schema.CALENDAR_HOME
@@ -526,7 +508,6 @@ class EventDetails(Cmd):
         returnValue(tuple(rows))
 
 
-
 class Event(EventDetails):
 
     _name = "Get Event Data by Resource-ID"
@@ -546,13 +527,11 @@ class Event(EventDetails):
         else:
             print("Could not find resource")
 
-
     def getData(self, txn, rid):
         co = schema.CALENDAR_OBJECT
         return self.getEventData(txn, (co.RESOURCE_ID == Parameter("RID")), {"RID": rid})
 
 
-
 class EventsByUID(EventDetails):
 
     _name = "Get Event Data by iCalendar UID"
@@ -568,13 +547,11 @@ class EventsByUID(EventDetails):
         else:
             print("Could not find icalendar data")
 
-
     def getData(self, txn, uid):
         co = schema.CALENDAR_OBJECT
         return self.getEventData(txn, (co.ICALENDAR_UID == Parameter("UID")), {"UID": uid})
 
 
-
 class EventsByName(EventDetails):
 
     _name = "Get Event Data by resource name"
@@ -590,13 +567,11 @@ class EventsByName(EventDetails):
         else:
             print("Could not find icalendar data")
 
-
     def getData(self, txn, name):
         co = schema.CALENDAR_OBJECT
         return self.getEventData(txn, (co.RESOURCE_NAME == Parameter("NAME")), {"NAME": name})
 
 
-
 class EventsByOwner(EventDetails):
 
     _name = "Get Event Data by Owner UID/Short Name"
@@ -613,13 +588,11 @@ class EventsByOwner(EventDetails):
         else:
             print("Could not find icalendar data")
 
-
     def getData(self, txn, uid):
         ch = schema.CALENDAR_HOME
         return self.getEventData(txn, (ch.OWNER_UID == Parameter("UID")), {"UID": uid})
 
 
-
 class EventsByOwnerCalendar(EventDetails):
 
     _name = "Get Event Data by Owner UID/Short Name and calendar name"
@@ -637,14 +610,12 @@ class EventsByOwnerCalendar(EventDetails):
         else:
             print("Could not find icalendar data")
 
-
     def getData(self, txn, uid, name):
         ch = schema.CALENDAR_HOME
         cb = schema.CALENDAR_BIND
         return self.getEventData(txn, (ch.OWNER_UID == Parameter("UID")).And(cb.CALENDAR_RESOURCE_NAME == Parameter("NAME")), {"UID": uid, "NAME": name})
 
 
-
 class EventsByPath(EventDetails):
 
     _name = "Get Event Data by HTTP Path"
@@ -667,7 +638,6 @@ class EventsByPath(EventDetails):
         else:
             print("Could not find icalendar data")
 
-
     def getData(self, txn, homeName, calendarName, resourceName):
         ch = schema.CALENDAR_HOME
         cb = schema.CALENDAR_BIND
@@ -683,7 +653,6 @@ class EventsByPath(EventDetails):
         )
 
 
-
 class EventsByContent(EventDetails):
 
     _name = "Get Event Data by Searching its Text Data"
@@ -699,13 +668,11 @@ class EventsByContent(EventDetails):
         else:
             print("Could not find icalendar data")
 
-
     def getData(self, txn, text):
         co = schema.CALENDAR_OBJECT
         return self.getEventData(txn, (co.ICALENDAR_TEXT.Contains(Parameter("TEXT"))), {"TEXT": text})
 
 
-
 class EventsInTimerange(Cmd):
 
     _name = "Get Event Data within a specified time range"
@@ -740,7 +707,6 @@ class EventsInTimerange(Cmd):
 
         yield self.eventsForEachCalendar(home, uid, timerange)
 
-
     @inlineCallbacks
     def eventsForEachCalendar(self, home, uid, timerange):
 
@@ -750,7 +716,6 @@ class EventsInTimerange(Cmd):
                 continue
             yield self.eventsInTimeRange(calendar, uid, timerange)
 
-
     @inlineCallbacks
     def eventsInTimeRange(self, calendar, uid, timerange):
 
@@ -786,7 +751,6 @@ class EventsInTimerange(Cmd):
             print(ical_data.getTextWithoutTimezones())
 
 
-
 class Purge(Cmd):
 
     _name = "Purge all data from tables"
@@ -834,7 +798,7 @@ class Purge(Cmd):
             print("Removed rows in table %s" % (tableschema,))
 
         if calendaruserproxy.ProxyDBService is not None:
-            calendaruserproxy.ProxyDBService.clean() #@UndefinedVariable
+            calendaruserproxy.ProxyDBService.clean()  # @UndefinedVariable
             print("Removed all proxies")
         else:
             print("No proxy database to clean.")
@@ -847,7 +811,6 @@ class Purge(Cmd):
         else:
             print("No attachments path to delete.")
 
-
     @inlineCallbacks
     def removeTableData(self, txn, tableschema):
         yield Delete(
@@ -856,7 +819,6 @@ class Purge(Cmd):
         ).on(txn)
 
 
-
 class DBInspectService(WorkerService, object):
     """
     Service which runs, exports the appropriate records, then stops the reactor.
@@ -870,7 +832,6 @@ class DBInspectService(WorkerService, object):
         self.commands = []
         self.commandMap = {}
 
-
     def doWork(self):
         """
         Start the service.
@@ -896,19 +857,16 @@ class DBInspectService(WorkerService, object):
 
         return succeed(None)
 
-
     def postStartService(self):
         """
         Don't quit right away
         """
         pass
 
-
     def registerCommand(self, cmd):
         self.commands.append(cmd.name())
         self.commandMap[cmd.name()] = cmd
 
-
     @inlineCallbacks
     def runCommandByPosition(self, position):
         try:
@@ -917,7 +875,6 @@ class DBInspectService(WorkerService, object):
             print("Position %d not available" % (position,))
             returnValue(None)
 
-
     @inlineCallbacks
     def runCommandByName(self, name):
         try:
@@ -925,7 +882,6 @@ class DBInspectService(WorkerService, object):
         except IndexError:
             print("Unknown command: '%s'" % (name,))
 
-
     @inlineCallbacks
     def runCommand(self, cmd):
         txn = self.store.newTransaction()
@@ -937,7 +893,6 @@ class DBInspectService(WorkerService, object):
             print("Command '%s' failed because of: %s" % (cmd.name(), e,))
             yield txn.abort()
 
-
     def printCommands(self):
 
         print("\n<---- Commands ---->")
@@ -947,7 +902,6 @@ class DBInspectService(WorkerService, object):
             print("P. Purge\n")
         print("Q. Quit\n")
 
-
     @inlineCallbacks
     def doDBInspect(self):
         """
@@ -972,7 +926,6 @@ class DBInspectService(WorkerService, object):
 
         self.reactor.stop()
 
-
     def stopService(self):
         """
         Stop the service.  Nothing to do; everything should be finished by this
@@ -984,7 +937,6 @@ class DBInspectService(WorkerService, object):
         # anyway).
 
 
-
 def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
     """
     Do the export.
@@ -993,6 +945,7 @@ def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
         from twisted.internet import reactor
     options = DBInspectOptions()
     options.parseOptions(argv[1:])
+
     def makeService(store):
         return DBInspectService(store, options, reactor, config)
     utilityMain(options['config'], makeService, reactor, verbose=options['debug'])
