@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,9 +22,10 @@ from StringIO import StringIO
 
 from twisted.internet import reactor
 from twisted.internet.defer import inlineCallbacks
+from twisted.logger import LogLevel, STDLibLogObserver
 from twisted.python.usage import Options
 
-from twext.python.log import Logger, LogLevel, StandardIOObserver
+from twext.python.log import Logger
 from txweb2.http_headers import Headers
 
 from txdav.caldav.datastore.scheduling.ischedule.dkim import RSA256, DKIMRequest, \
@@ -33,21 +34,22 @@ from txdav.caldav.datastore.scheduling.ischedule.dkim import RSA256, DKIMRequest
 log = Logger()
 
 
-
 def _doKeyGeneration(options):
 
     key = RSA.generate(options["key-size"])
     output = key.exportKey()
     lineBreak = False
     if options["key"]:
-        open(options["key"], "w").write(output)
+        with open(options["key"], "w") as f:
+            f.write(output)
     else:
         print(output)
         lineBreak = True
 
     output = key.publickey().exportKey()
     if options["pub-key"]:
-        open(options["pub-key"], "w").write(output)
+        with open(options["pub-key"], "w") as f:
+            f.write(output)
     else:
         if lineBreak:
             print
@@ -62,15 +64,15 @@ def _doKeyGeneration(options):
         print(txt)
 
 
-
 @inlineCallbacks
 def _doRequest(options):
 
     if options["verbose"]:
-        log.publisher.levels.setLogLevelForNamespace("txdav.caldav.datastore.scheduling.ischedule.dkim", LogLevel.debug)
+        log.levels().setLogLevelForNamespace("txdav.caldav.datastore.scheduling.ischedule.dkim", LogLevel.debug)
 
     # Parse the HTTP file
-    request = open(options["request"]).read()
+    with open(options["request"]) as f:
+        request = f.read()
     method, uri, headers, stream = _parseRequest(request)
 
     # Setup signing headers
@@ -109,11 +111,11 @@ def _doRequest(options):
     print(s.getvalue())
 
 
-
 @inlineCallbacks
 def _doVerify(options):
     # Parse the HTTP file
-    verify = open(os.path.expanduser(options["verify"])).read()
+    with open(os.path.expanduser(options["verify"])) as f:
+        verify = f.read()
     _method, _uri, headers, body = _parseRequest(verify)
 
     # Check for local public key
@@ -135,7 +137,6 @@ def _doVerify(options):
         print("Verification Succeeded")
 
 
-
 def _parseRequest(request):
 
     lines = request.splitlines(True)
@@ -164,7 +165,6 @@ def _parseRequest(request):
     return method, uri, headers, stream
 
 
-
 def _writeRequest(request, f):
 
     f.write("%s %s HTTP/1.1\r\n" % (request.method, request.uri,))
@@ -175,7 +175,6 @@ def _writeRequest(request, f):
     f.write(request.stream.read())
 
 
-
 class PublicKeyLookup_File(PublicKeyLookup):
 
     method = "*"
@@ -185,8 +184,9 @@ class PublicKeyLookup_File(PublicKeyLookup):
         """
         Do the key lookup using the actual lookup method.
         """
-        return RSA.importKey(open(self.pubkeyfile).read())
-
+        with open(self.pubkeyfile) as f:
+            data = f.read()
+        return RSA.importKey(data)
 
 
 def usage(e=None):
@@ -274,7 +274,6 @@ class DKIMToolOptions(Options):
         self.outputName = '-'
 
 
-
 @inlineCallbacks
 def _runInReactor(fn, options):
 
@@ -286,7 +285,6 @@ def _runInReactor(fn, options):
         reactor.stop()
 
 
-
 def main(argv=sys.argv, stderr=sys.stderr):
     options = DKIMToolOptions()
     options.parseOptions(argv[1:])
@@ -294,11 +292,11 @@ def main(argv=sys.argv, stderr=sys.stderr):
     #
     # Send logging output to stdout
     #
-    observer = StandardIOObserver()
+    observer = STDLibLogObserver()
     observer.start()
 
     if options["verbose"]:
-        log.publisher.levels.setLogLevelForNamespace("txdav.caldav.datastore.scheduling.ischedule.dkim", LogLevel.debug)
+        log.levels().setLogLevelForNamespace("txdav.caldav.datastore.scheduling.ischedule.dkim", LogLevel.debug)
 
     if options["key-gen"]:
         _doKeyGeneration(options)
