@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- test-case-name: calendarserver.tools.test.test_export -*-
 ##
-# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -55,7 +55,6 @@ from txdav.xml import element as davxml
 log = Logger()
 
 
-
 def usage(e=None):
     if e:
         print(e)
@@ -105,7 +104,6 @@ class ExportOptions(Options):
         self.outputName = '-'
         self.outputDirectoryName = None
 
-
     def opt_uid(self, uid):
         """
         Add a calendar home directly by its UID (which is usually a directory
@@ -113,7 +111,6 @@ class ExportOptions(Options):
         """
         self.exporters.append(UIDExporter(uid))
 
-
     def opt_record(self, recordName):
         """
         Add a directory record's calendar home (format: 'recordType:shortName').
@@ -123,7 +120,6 @@ class ExportOptions(Options):
 
     opt_r = opt_record
 
-
     def opt_collection(self, collectionName):
         """
         Add a calendar collection.  This option must be passed after --record
@@ -135,7 +131,6 @@ class ExportOptions(Options):
 
     opt_c = opt_collection
 
-
     def opt_directory(self, dirname):
         """
         Specify output directory path.
@@ -144,7 +139,6 @@ class ExportOptions(Options):
 
     opt_d = opt_directory
 
-
     def opt_output(self, filename):
         """
         Specify output file path (default: '-', meaning stdout).
@@ -153,7 +147,6 @@ class ExportOptions(Options):
 
     opt_o = opt_output
 
-
     def opt_user(self, user):
         """
         Add a user's calendar home (shorthand for '-r users:shortName').
@@ -162,7 +155,6 @@ class ExportOptions(Options):
 
     opt_u = opt_user
 
-
     def openOutput(self):
         """
         Open the appropriate output file based on the '--output' option.
@@ -173,7 +165,6 @@ class ExportOptions(Options):
             return open(self.outputName, 'wb')
 
 
-
 class _ExporterBase(object):
     """
     Base exporter implementation that works from a home UID.
@@ -188,14 +179,12 @@ class _ExporterBase(object):
     def __init__(self):
         self.collections = []
 
-
     def getHomeUID(self, exportService):
         """
         Subclasses must implement this.
         """
         raise NotImplementedError()
 
-
     @inlineCallbacks
     def listCalendars(self, txn, exportService):
         """
@@ -215,7 +204,6 @@ class _ExporterBase(object):
         returnValue(result)
 
 
-
 class UIDExporter(_ExporterBase):
     """
     An exporter that constructs a list of calendars based on the UID of the
@@ -230,12 +218,10 @@ class UIDExporter(_ExporterBase):
         super(UIDExporter, self).__init__()
         self.uid = uid
 
-
     def getHomeUID(self, exportService):
         return succeed(self.uid)
 
 
-
 class DirectoryExporter(_ExporterBase):
     """
     An exporter that constructs a list of calendars based on the directory
@@ -257,7 +243,6 @@ class DirectoryExporter(_ExporterBase):
         self.recordType = recordType
         self.shortName = shortName
 
-
     @inlineCallbacks
     def getHomeUID(self, exportService):
         """
@@ -271,7 +256,6 @@ class DirectoryExporter(_ExporterBase):
         returnValue(record.uid)
 
 
-
 @inlineCallbacks
 def exportToFile(calendars, fileobj):
     """
@@ -303,7 +287,6 @@ def exportToFile(calendars, fileobj):
     fileobj.write(comp.getTextWithTimezones(True))
 
 
-
 @inlineCallbacks
 def exportToDirectory(calendars, dirname):
     """
@@ -346,10 +329,8 @@ def exportToDirectory(calendars, dirname):
                     comp.addComponent(sub)
 
         filename = os.path.join(dirname, "{}_{}.ics".format(homeUID, calendar.name()))
-        fileobj = open(filename, 'wb')
-        fileobj.write(comp.getTextWithTimezones(True))
-        fileobj.close()
-
+        with open(filename, 'wb') as fileobj:
+            fileobj.write(comp.getTextWithTimezones(True))
 
 
 class ExporterService(WorkerService, object):
@@ -365,7 +346,6 @@ class ExporterService(WorkerService, object):
         self.config = config
         self._directory = self.store.directoryService()
 
-
     @inlineCallbacks
     def doWork(self):
         """
@@ -396,14 +376,12 @@ class ExporterService(WorkerService, object):
         except:
             log.failure("doWork()")
 
-
     def directoryService(self):
         """
         Get an appropriate directory service.
         """
         return self._directory
 
-
     def stopService(self):
         """
         Stop the service.  Nothing to do; everything should be finished by this
@@ -415,7 +393,6 @@ class ExporterService(WorkerService, object):
         # anyway).
 
 
-
 def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
     """
     Do the export.
@@ -440,7 +417,6 @@ def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
             )
             sys.exit(1)
 
-
     def makeService(store):
         from twistedcaldav.config import config
         return ExporterService(store, options, output, reactor, config)
