@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- test-case-name: calendarserver.tools.test.test_calverify -*-
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -44,7 +44,6 @@ log = Logger()
 VERSION = "1"
 
 
-
 def usage(e=None):
     if e:
         print(e)
@@ -70,12 +69,10 @@ description = ''.join(
 description += "\nVersion: %s" % (VERSION,)
 
 
-
 class ConfigError(Exception):
     pass
 
 
-
 class MigrateVerifyOptions(Options):
     """
     Command-line options for 'calendarserver_migrate_verify'
@@ -92,12 +89,10 @@ class MigrateVerifyOptions(Options):
         ['data', 'd', "./paths.txt", "List of file paths for migrated data."],
     ]
 
-
     def __init__(self):
         super(MigrateVerifyOptions, self).__init__()
         self.outputName = '-'
 
-
     def opt_output(self, filename):
         """
         Specify output file path (default: '-', meaning stdout).
@@ -106,7 +101,6 @@ class MigrateVerifyOptions(Options):
 
     opt_o = opt_output
 
-
     def openOutput(self):
         """
         Open the appropriate output file based on the '--output' option.
@@ -117,7 +111,6 @@ class MigrateVerifyOptions(Options):
             return open(self.outputName, 'wb')
 
 
-
 class MigrateVerifyService(WorkerService, object):
     """
     Service which runs, does its stuff, then stops the reactor.
@@ -139,7 +132,6 @@ class MigrateVerifyService(WorkerService, object):
         self.missingCalendars = []
         self.missingResources = []
 
-
     @inlineCallbacks
     def doWork(self):
         """
@@ -156,41 +148,40 @@ class MigrateVerifyService(WorkerService, object):
         except:
             log.failure("doWork()")
 
-
     def readPaths(self):
 
         self.output.write("-- Reading data file: %s\n" % (self.options["data"]))
 
-        datafile = open(os.path.expanduser(self.options["data"]))
-        total = 0
-        invalidGUIDs = set()
-        for line in datafile:
-            line = line.strip()
-            total += 1
-            segments = line.split("/")
-            while segments and segments[0] != "__uids__":
-                segments.pop(0)
-            if segments and len(segments) >= 6:
-                guid = segments[3]
-                calendar = segments[4]
-                resource = segments[5]
+        with open(os.path.expanduser(self.options["data"])) as datafile:
+            total = 0
+            invalidGUIDs = set()
+            for line in datafile:
+                line = line.strip()
+                total += 1
+                segments = line.split("/")
+                while segments and segments[0] != "__uids__":
+                    segments.pop(0)
+                if segments and len(segments) >= 6:
+                    guid = segments[3]
+                    calendar = segments[4]
+                    resource = segments[5]
 
-                if calendar == "inbox":
-                    self.ignoreInbox += 1
-                    invalidGUIDs.add(guid)
-                elif calendar == "dropbox":
-                    self.ignoreDropbox += 1
-                    invalidGUIDs.add(guid)
-                elif len(segments) > 6:
-                    self.badPaths.append(line)
-                    invalidGUIDs.add(guid)
+                    if calendar == "inbox":
+                        self.ignoreInbox += 1
+                        invalidGUIDs.add(guid)
+                    elif calendar == "dropbox":
+                        self.ignoreDropbox += 1
+                        invalidGUIDs.add(guid)
+                    elif len(segments) > 6:
+                        self.badPaths.append(line)
+                        invalidGUIDs.add(guid)
+                    else:
+                        self.pathsByGUID.setdefault(guid, {}).setdefault(calendar, set()).add(resource)
+                        self.validPaths += 1
                 else:
-                    self.pathsByGUID.setdefault(guid, {}).setdefault(calendar, set()).add(resource)
-                    self.validPaths += 1
-            else:
-                if segments and len(segments) >= 4:
-                    invalidGUIDs.add(segments[3])
-                self.badPaths.append(line)
+                    if segments and len(segments) >= 4:
+                        invalidGUIDs.add(segments[3])
+                    self.badPaths.append(line)
 
         # Remove any invalid GUIDs that actuall were valid
         invalidGUIDs = [pguid for pguid in invalidGUIDs if pguid not in self.pathsByGUID]
@@ -215,7 +206,6 @@ class MigrateVerifyService(WorkerService, object):
         for badPath in sorted(self.badPaths):
             self.output.write("Bad path: %s\n" % (badPath,))
 
-
     @inlineCallbacks
     def doCheck(self):
         """
@@ -304,11 +294,10 @@ class MigrateVerifyService(WorkerService, object):
         for resource in sorted(self.missingResources):
             self.output.write("%s\n" % (resource,))
 
-
     @inlineCallbacks
     def guid2ResourceID(self, guid):
         ch = schema.CALENDAR_HOME
-        kwds = {"GUID" : guid}
+        kwds = {"GUID": guid}
         rows = (yield Select(
             [
                 ch.RESOURCE_ID,
@@ -321,11 +310,10 @@ class MigrateVerifyService(WorkerService, object):
 
         returnValue(rows[0][0] if rows else None)
 
-
     @inlineCallbacks
     def calendarsForUser(self, rid):
         cb = schema.CALENDAR_BIND
-        kwds = {"RID" : rid}
+        kwds = {"RID": rid}
         rows = (yield Select(
             [
                 cb.CALENDAR_RESOURCE_NAME,
@@ -339,11 +327,10 @@ class MigrateVerifyService(WorkerService, object):
 
         returnValue(rows)
 
-
     @inlineCallbacks
     def resourcesForCalendar(self, rid):
         co = schema.CALENDAR_OBJECT
-        kwds = {"RID" : rid}
+        kwds = {"RID": rid}
         rows = (yield Select(
             [
                 co.RESOURCE_NAME,
@@ -356,7 +343,6 @@ class MigrateVerifyService(WorkerService, object):
 
         returnValue(rows)
 
-
     def stopService(self):
         """
         Stop the service.  Nothing to do; everything should be finished by this
@@ -364,7 +350,6 @@ class MigrateVerifyService(WorkerService, object):
         """
 
 
-
 def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
     """
     Do the export.
@@ -379,7 +364,6 @@ def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
         stderr.write("Unable to open output file for writing: %s\n" % (e))
         sys.exit(1)
 
-
     def makeService(store):
         from twistedcaldav.config import config
         config.TransactionTimeoutSeconds = 0
