@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 ##
-# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,14 +32,15 @@ import uuid
 
 from calendarserver.tools.cmdline import utilityMain, WorkerService
 from calendarserver.tools.util import (
-    recordForPrincipalID, prettyRecord
+    recordForPrincipalID, prettyRecord, action_addProxy, action_removeProxy
 )
 from twext.who.directory import DirectoryRecord
 from twext.who.idirectory import RecordType, InvalidDirectoryRecordError
 from twisted.internet import reactor
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed
 from twistedcaldav.config import config
-from txdav.who.delegates import Delegates, RecordType as DelegateRecordType
+from twistedcaldav.cache import MemcacheChangeNotifier
+from txdav.who.delegates import CachingDelegates
 from txdav.who.idirectory import AutoScheduleMode
 from txdav.who.groups import GroupCacherPollingWork
 
@@ -110,7 +111,6 @@ def usage(e=None):
         sys.exit(0)
 
 
-
 class PrincipalService(WorkerService):
     """
     Executes principals-related functions in a context which has access to the store
@@ -125,12 +125,20 @@ class PrincipalService(WorkerService):
         Calls the function that's been assigned to "function" and passes the root
         resource, directory, store, and whatever has been assigned to "params".
         """
+        if (
+            config.EnableResponseCache and
+            config.Memcached.Pools.Default.ClientEnabled
+        ):
+            # These class attributes need to be setup with our memcache\
+            # notifier
+            CachingDelegates.cacheNotifier = MemcacheChangeNotifier(None, cacheHandle="PrincipalToken")
+
         if self.function is not None:
             yield self.function(self.store, *self.params)
 
 
-
 def main():
+
     try:
         (optargs, args) = getopt(
             sys.argv[1:], "a:hf:P:v", [
@@ -388,7 +396,6 @@ def main():
     utilityMain(configFileName, PrincipalService, verbose=verbose)
 
 
-
 def runListPrincipalTypes(service, store):
     directory = store.directoryService()
     for recordType in directory.recordTypes():
@@ -396,7 +403,6 @@ def runListPrincipalTypes(service, store):
     return succeed(None)
 
 
-
 @inlineCallbacks
 def runListPrincipals(service, store, listPrincipals):
     directory = store.directoryService()
@@ -412,7 +418,6 @@ def runListPrincipals(service, store, listPrincipals):
     returnValue(None)
 
 
-
 @inlineCallbacks
 def runPrincipalActions(service, store, principalIDs, actions):
     directory = store.directoryService()
@@ -433,7 +438,6 @@ def runPrincipalActions(service, store, principalIDs, actions):
             print("")
 
 
-
 @inlineCallbacks
 def runSearch(service, store, tokens, context=None):
     directory = store.directoryService()
@@ -481,7 +485,6 @@ def runSearch(service, store, tokens, context=None):
     print("")
 
 
-
 @inlineCallbacks
 def runAddPrincipal(service, store, addType, uid, shortNames, fullNames):
     directory = store.directoryService()
@@ -513,7 +516,6 @@ def runAddPrincipal(service, store, addType, uid, shortNames, fullNames):
     print("Added '{name}'".format(name=fullNames[0]))
 
 
-
 @inlineCallbacks
 def action_removePrincipal(store, record):
     directory = store.directoryService()
@@ -528,7 +530,6 @@ def action_removePrincipal(store, record):
     )
 
 
-
 @inlineCallbacks
 def action_listProxies(store, record, *proxyTypes):
     directory = store.directoryService()
@@ -555,7 +556,6 @@ def action_listProxies(store, record, *proxyTypes):
             print("No %s proxies for %s" % (proxyType, prettyRecord(record)))
 
 
-
 @inlineCallbacks
 def action_listProxyFor(store, record, *proxyTypes):
     directory = store.directoryService()
@@ -592,108 +592,6 @@ def action_listProxyFor(store, record, *proxyTypes):
             )
 
 
-
-@inlineCallbacks
-def _addRemoveProxy(msg, fn, store, record, proxyType, *proxyIDs):
-    directory = store.directoryService()
-    readWrite = (proxyType == "write")
-    for proxyID in proxyIDs:
-        proxyRecord = yield recordForPrincipalID(directory, proxyID)
-        if proxyRecord is None:
-            print("Invalid principal ID: %s" % (proxyID,))
-        else:
-            txn = store.newTransaction()
-            yield fn(txn, record, proxyRecord, readWrite)
-            yield txn.commit()
-            print(
-                "{msg} {proxy} as a {proxyType} proxy for {record}".format(
-                    msg=msg, proxy=prettyRecord(proxyRecord),
-                    proxyType=proxyType, record=prettyRecord(record)
-                )
-            )
-
-
-
-@inlineCallbacks
-def action_addProxy(store, record, proxyType, *proxyIDs):
-    if config.GroupCaching.Enabled and config.GroupCaching.UseDirectoryBasedDelegates:
-        if record.recordType in (
-            record.service.recordType.location,
-            record.service.recordType.resource,
-        ):
-            print("You are not allowed to add proxies for locations or resources via command line when their proxy assignments come from the directory service.")
-            returnValue(None)
-
-    yield _addRemoveProxy("Added", Delegates.addDelegate, store, record, proxyType, *proxyIDs)
-
-
-
-@inlineCallbacks
-def action_removeProxy(store, record, *proxyIDs):
-    if config.GroupCaching.Enabled and config.GroupCaching.UseDirectoryBasedDelegates:
-        if record.recordType in (
-            record.service.recordType.location,
-            record.service.recordType.resource,
-        ):
-            print("You are not allowed to remove proxies for locations or resources via command line when their proxy assignments come from the directory service.")
-            returnValue(None)
-
-    # Write
-    yield _addRemoveProxy("Removed", Delegates.removeDelegate, store, record, "write", *proxyIDs)
-    # Read
-    yield _addRemoveProxy("Removed", Delegates.removeDelegate, store, record, "read", *proxyIDs)
-
-
-
-@inlineCallbacks
-def setProxies(record, readProxyRecords, writeProxyRecords):
-    """
-    Set read/write proxies en masse for a record
-    @param record: L{IDirectoryRecord}
-    @param readProxyRecords: a list of records
-    @param writeProxyRecords: a list of records
-    """
-
-    proxyTypes = [
-        (DelegateRecordType.readDelegateGroup, readProxyRecords),
-        (DelegateRecordType.writeDelegateGroup, writeProxyRecords),
-    ]
-    for recordType, proxyRecords in proxyTypes:
-        if proxyRecords is None:
-            continue
-        proxyGroup = yield record.service.recordWithShortName(
-            recordType, record.uid
-        )
-        yield proxyGroup.setMembers(proxyRecords)
-
-
-
-@inlineCallbacks
-def getProxies(record):
-    """
-    Returns a tuple containing the records for read proxies and write proxies
-    of the given record
-    """
-
-    allProxies = {
-        DelegateRecordType.readDelegateGroup: [],
-        DelegateRecordType.writeDelegateGroup: [],
-    }
-    for recordType in allProxies.iterkeys():
-        proxyGroup = yield record.service.recordWithShortName(
-            recordType, record.uid
-        )
-        allProxies[recordType] = yield proxyGroup.members()
-
-    returnValue(
-        (
-            allProxies[DelegateRecordType.readDelegateGroup],
-            allProxies[DelegateRecordType.writeDelegateGroup]
-        )
-    )
-
-
-
 @inlineCallbacks
 def action_listGroupMembers(store, record):
     members = yield record.members()
@@ -707,7 +605,6 @@ def action_listGroupMembers(store, record):
         print("No group members for %s" % (prettyRecord(record),))
 
 
-
 @inlineCallbacks
 def action_addGroupMember(store, record, *memberIDs):
     directory = store.directoryService()
@@ -735,7 +632,6 @@ def action_addGroupMember(store, record, *memberIDs):
         yield record.service.updateRecords([record], create=False)
 
 
-
 @inlineCallbacks
 def action_removeGroupMember(store, record, *memberIDs):
     directory = store.directoryService()
@@ -763,7 +659,6 @@ def action_removeGroupMember(store, record, *memberIDs):
         yield record.service.updateRecords([record], create=False)
 
 
-
 @inlineCallbacks
 def printGroupCacherInfo(service, store, principalIDs):
     """
@@ -807,7 +702,6 @@ def printGroupCacherInfo(service, store, principalIDs):
     yield txn.commit()
 
 
-
 @inlineCallbacks
 def scheduleGroupRefreshJob(service, store):
     """
@@ -819,7 +713,6 @@ def scheduleGroupRefreshJob(service, store):
     yield txn.commit()
 
 
-
 def action_getAutoScheduleMode(store, record):
     print(
         "Auto-schedule mode for {record} is {mode}".format(
@@ -832,7 +725,6 @@ def action_getAutoScheduleMode(store, record):
     )
 
 
-
 @inlineCallbacks
 def action_setAutoScheduleMode(store, record, autoScheduleMode):
     if record.recordType == RecordType.group:
@@ -855,20 +747,12 @@ def action_setAutoScheduleMode(store, record, autoScheduleMode):
     else:
         print(
             "Setting auto-schedule-mode to {mode} for {record}".format(
-                mode=autoScheduleMode.description,
+                mode=("default" if autoScheduleMode is None else autoScheduleMode.description),
                 record=prettyRecord(record),
             )
         )
 
-        # Get original fields
-        newFields = record.fields.copy()
-
-        # Set new values
-        newFields[record.service.fieldName.autoScheduleMode] = autoScheduleMode
-
-        updatedRecord = DirectoryRecord(record.service, newFields)
-        yield record.service.updateRecords([updatedRecord], create=False)
-
+        yield record.setAutoScheduleMode(autoScheduleMode)
 
 
 @inlineCallbacks
@@ -910,7 +794,6 @@ def action_setAutoAcceptGroup(store, record, autoAcceptGroup):
             yield record.service.updateRecords([updatedRecord], create=False)
 
 
-
 @inlineCallbacks
 def action_getAutoAcceptGroup(store, record):
     if record.autoAcceptGroup:
@@ -938,7 +821,6 @@ def action_getAutoAcceptGroup(store, record):
         )
 
 
-
 @inlineCallbacks
 def action_setValue(store, record, name, value):
     print(
@@ -956,7 +838,6 @@ def action_setValue(store, record, name, value):
     yield record.service.updateRecords([updatedRecord], create=False)
 
 
-
 def action_getValue(store, record, name):
     try:
         value = record.fields[record.service.fieldName.lookupByName(name)]
@@ -973,7 +854,6 @@ def action_getValue(store, record, name):
         )
 
 
-
 def abort(msg, status=1):
     sys.stdout.write("%s\n" % (msg,))
     try:
@@ -983,7 +863,6 @@ def abort(msg, status=1):
     sys.exit(status)
 
 
-
 def parseCreationArgs(args):
     """
     Look at the command line arguments for --add; the first arg is required
@@ -1017,7 +896,6 @@ def parseCreationArgs(args):
     return fullName, shortName, uid
 
 
-
 def isUUID(value):
     try:
         uuid.UUID(value)
@@ -1026,7 +904,6 @@ def isUUID(value):
         return False
 
 
-
 def matchStrings(value, validValues):
     for validValue in validValues:
         if validValue.startswith(value):
@@ -1035,7 +912,6 @@ def matchStrings(value, validValues):
     raise ValueError("'%s' is not a recognized value" % (value,))
 
 
-
 def printRecordList(records):
     results = []
     for record in records:
@@ -1055,6 +931,5 @@ def printRecordList(records):
         print(format % (fullName, recordType, uid, u", ".join(shortNames)))
 
 
-
 if __name__ == "__main__":
     main()
