@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- test-case-name: calendarserver.tools.test.test_purge -*-
 ##
-# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -50,7 +50,6 @@ DEFAULT_BATCH_SIZE = 100
 DEFAULT_RETAIN_DAYS = 365
 
 
-
 class PrincipalPurgePollingWork(
     RegeneratingWorkItem,
     fromTable(schema.PRINCIPAL_PURGE_POLLING_WORK)
@@ -73,14 +72,12 @@ class PrincipalPurgePollingWork(
         else:
             return succeed(None)
 
-
     def regenerateInterval(self):
         """
         Return the interval in seconds between regenerating instances.
         """
         return config.AutomaticPurging.PollingIntervalSeconds if config.AutomaticPurging.Enabled else None
 
-
     @inlineCallbacks
     def doWork(self):
 
@@ -115,7 +112,6 @@ class PrincipalPurgePollingWork(
             )
 
 
-
 class PrincipalPurgeCheckWork(
     WorkItem,
     fromTable(schema.PRINCIPAL_PURGE_CHECK_WORK)
@@ -164,7 +160,6 @@ class PrincipalPurgeCheckWork(
             log.debug("{uid} is still in the directory", uid=self.uid)
 
 
-
 class PrincipalPurgeWork(
     WorkItem,
     fromTable(schema.PRINCIPAL_PURGE_WORK)
@@ -211,7 +206,6 @@ class PrincipalPurgeWork(
             log.debug("{uid} has re-appeared in the directory", uid=self.uid)
 
 
-
 class PrincipalPurgeHomeWork(
     WorkItem,
     fromTable(schema.PRINCIPAL_PURGE_HOME_WORK)
@@ -276,7 +270,6 @@ class PrincipalPurgeHomeWork(
                 yield home.remove()
 
 
-
 class PurgeOldEventsService(WorkerService):
 
     uuid = None
@@ -309,7 +302,6 @@ class PurgeOldEventsService(WorkerService):
         else:
             sys.exit(0)
 
-
     @classmethod
     def main(cls):
 
@@ -400,7 +392,6 @@ class PurgeOldEventsService(WorkerService):
             verbose=verbose,
         )
 
-
     @classmethod
     @inlineCallbacks
     def purgeOldEvents(cls, store, uuid, cutoff, batchSize, debug=False, dryrun=False):
@@ -414,13 +405,12 @@ class PurgeOldEventsService(WorkerService):
         result = yield service.doWork()
         returnValue(result)
 
-
     @inlineCallbacks
     def getMatchingHomeUIDs(self):
         """
         Find all the calendar homes that match the uuid cli argument.
         """
-        log.debug("Searching for calendar homes matching: '{}'".format(self.uuid))
+        log.debug("Searching for calendar homes matching: '{uid}'", uid=self.uuid)
         txn = self.store.newTransaction(label="Find matching homes")
         ch = schema.CALENDAR_HOME
         if self.uuid:
@@ -437,10 +427,9 @@ class PurgeOldEventsService(WorkerService):
             ).on(txn))
 
         yield txn.commit()
-        log.debug("  Found {} calendar homes".format(len(rows)))
+        log.debug("  Found {len} calendar homes", len=len(rows))
         returnValue(sorted(rows, key=lambda x: x[1]))
 
-
     @inlineCallbacks
     def getMatchingCalendarIDs(self, home_id, owner_uid):
         """
@@ -451,7 +440,7 @@ class PurgeOldEventsService(WorkerService):
         @param owner_uid: owner UUID of home to check
         @type owner_uid: L{str}
         """
-        log.debug("Checking calendar home: {} '{}'".format(home_id, owner_uid))
+        log.debug("Checking calendar home: {id} '{uid}'", id=home_id, uid=owner_uid)
         txn = self.store.newTransaction(label="Find matching calendars")
         cb = schema.CALENDAR_BIND
         kwds = {"home_id": home_id}
@@ -463,10 +452,9 @@ class PurgeOldEventsService(WorkerService):
             ),
         ).on(txn, **kwds))
         yield txn.commit()
-        log.debug("  Found {} calendars".format(len(rows)))
+        log.debug("  Found {len} calendars", len=len(rows))
         returnValue(rows)
 
-
     PurgeEvent = collections.namedtuple("PurgeEvent", ("home", "calendar", "resource",))
 
     @inlineCallbacks
@@ -483,7 +471,7 @@ class PurgeOldEventsService(WorkerService):
         @type calendar_name: L{str}
         """
 
-        log.debug("  Checking calendar: {} '{}'".format(calendar_id, calendar_name))
+        log.debug("  Checking calendar: {id} '{name}'", id=calendar_id, name=calendar_name)
         purge = set()
         txn = self.store.newTransaction(label="Find matching resources")
         co = schema.CALENDAR_OBJECT
@@ -503,7 +491,7 @@ class PurgeOldEventsService(WorkerService):
             ),
         ).on(txn, **kwds))
 
-        log.debug("    Found {} resources to check".format(len(rows)))
+        log.debug("    Found {len} resources to check", len=len(rows))
         for resource_id, recurrence_max, recurrence_min, max_end_date in rows:
 
             recurrence_max = parseSQLDateToPyCalendar(recurrence_max) if recurrence_max else None
@@ -530,10 +518,9 @@ class PurgeOldEventsService(WorkerService):
                     purge.add(self.PurgeEvent(home_id, calendar_id, resource_id,))
 
         yield txn.commit()
-        log.debug("    Found {} resources to purge".format(len(purge)))
+        log.debug("    Found {len} resources to purge", len=len(purge))
         returnValue(purge)
 
-
     @inlineCallbacks
     def getCalendar(self, txn, resid):
         """
@@ -543,7 +530,7 @@ class PurgeOldEventsService(WorkerService):
         @type resid: L{int}
         """
         co = schema.CALENDAR_OBJECT
-        kwds = {"ResourceID" : resid}
+        kwds = {"ResourceID": resid}
         rows = (yield Select(
             [co.ICALENDAR_TEXT],
             From=co,
@@ -558,7 +545,6 @@ class PurgeOldEventsService(WorkerService):
 
         returnValue(caldata)
 
-
     def checkLastInstance(self, calendar):
         """
         Determine the last instance of a calendar event. Try a "static" analysis of the data first,
@@ -599,7 +585,6 @@ class PurgeOldEventsService(WorkerService):
 
         return True
 
-
     @inlineCallbacks
     def getResourcesToPurge(self, home_id, owner_uid):
         """
@@ -618,7 +603,6 @@ class PurgeOldEventsService(WorkerService):
 
         returnValue(purge)
 
-
     @inlineCallbacks
     def purgeResources(self, events):
         """
@@ -639,18 +623,18 @@ class PurgeOldEventsService(WorkerService):
                 last_calendar = event.calendar
             resource = (yield calendar.objectResourceWithID(event.resource))
             yield resource.purge(implicitly=False)
-            log.debug("Removed resource {} '{}' from calendar {} '{}' of calendar home '{}'".format(
-                resource.id(),
-                resource.name(),
-                resource.parentCollection().id(),
-                resource.parentCollection().name(),
-                resource.parentCollection().ownerHome().uid()
-            ))
+            log.debug(
+                "Removed resource {id} '{name}' from calendar {pid} '{pname}' of calendar home '{uid}'",
+                id=resource.id(),
+                name=resource.name(),
+                pid=resource.parentCollection().id(),
+                pname=resource.parentCollection().name(),
+                uid=resource.parentCollection().ownerHome().uid()
+            )
             count += 1
         yield txn.commit()
         returnValue(count)
 
-
     @inlineCallbacks
     def doWork(self):
 
@@ -679,18 +663,18 @@ class PurgeOldEventsService(WorkerService):
         if self.dryrun:
             eventCount = len(purge)
             if eventCount == 0:
-                log.info("No events are older than %s" % (self.cutoff,))
+                log.info("No events are older than {cutoff}", cutoff=self.cutoff)
             elif eventCount == 1:
-                log.info("1 event is older than %s" % (self.cutoff,))
+                log.info("1 event is older than {cutoff}", cutoff=self.cutoff)
             else:
-                log.info("%d events are older than %s" % (eventCount, self.cutoff))
+                log.info("{count} events are older than {cutoff}", count=eventCount, cutoff=self.cutoff)
             returnValue(eventCount)
 
         purge = list(purge)
         purge.sort()
         totalEvents = len(purge)
 
-        log.info("Removing {} events older than {}...".format(len(purge), self.cutoff,))
+        log.info("Removing {len} events older than {cutoff}...", len=len(purge), cutoff=self.cutoff)
 
         numEventsRemoved = -1
         totalRemoved = 0
@@ -698,7 +682,7 @@ class PurgeOldEventsService(WorkerService):
             numEventsRemoved = (yield self.purgeResources(purge[:self.batchSize]))
             if numEventsRemoved:
                 totalRemoved += numEventsRemoved
-                log.debug("  Removed {} of {} events...".format(totalRemoved, totalEvents))
+                log.debug("  Removed {removed} of {total} events...", removed=totalRemoved, total=totalEvents)
                 purge = purge[numEventsRemoved:]
 
         if totalRemoved == 0:
@@ -706,12 +690,11 @@ class PurgeOldEventsService(WorkerService):
         elif totalRemoved == 1:
             log.info("1 event was removed in total")
         else:
-            log.info("%d events were removed in total" % (totalRemoved,))
+            log.info("{total} events were removed in total", total=totalRemoved)
 
         returnValue(totalRemoved)
 
 
-
 class PurgeAttachmentsService(WorkerService):
 
     uuid = None
@@ -745,7 +728,6 @@ class PurgeAttachmentsService(WorkerService):
         else:
             sys.exit(0)
 
-
     @classmethod
     def main(cls):
 
@@ -836,7 +818,6 @@ class PurgeAttachmentsService(WorkerService):
             verbose=debug,
         )
 
-
     @classmethod
     @inlineCallbacks
     def purgeAttachments(cls, store, uuid, days, limit, dryrun, verbose):
@@ -856,7 +837,6 @@ class PurgeAttachmentsService(WorkerService):
         result = yield service.doWork()
         returnValue(result)
 
-
     @inlineCallbacks
     def doWork(self):
 
@@ -877,7 +857,6 @@ class PurgeAttachmentsService(WorkerService):
                 total += yield self._managedPurge()
             returnValue(total)
 
-
     @inlineCallbacks
     def _orphansDryRun(self):
 
@@ -887,7 +866,6 @@ class PurgeAttachmentsService(WorkerService):
         orphans = yield txn.orphanedAttachments(self.uuid)
         returnValue(orphans)
 
-
     @inlineCallbacks
     def _dropboxDryRun(self):
 
@@ -899,7 +877,6 @@ class PurgeAttachmentsService(WorkerService):
 
         returnValue(cutoffs)
 
-
     @inlineCallbacks
     def _managedDryRun(self):
 
@@ -911,7 +888,6 @@ class PurgeAttachmentsService(WorkerService):
 
         returnValue(cutoffs)
 
-
     def _dryRunSummary(self, orphans, dropbox, managed):
 
         if self.verbose:
@@ -975,7 +951,6 @@ class PurgeAttachmentsService(WorkerService):
 
         return total
 
-
     @inlineCallbacks
     def _orphansPurge(self):
 
@@ -1006,7 +981,6 @@ class PurgeAttachmentsService(WorkerService):
 
         returnValue(totalRemoved)
 
-
     @inlineCallbacks
     def _dropboxPurge(self):
 
@@ -1037,7 +1011,6 @@ class PurgeAttachmentsService(WorkerService):
 
         returnValue(totalRemoved)
 
-
     @inlineCallbacks
     def _managedPurge(self):
 
@@ -1069,7 +1042,6 @@ class PurgeAttachmentsService(WorkerService):
         returnValue(totalRemoved)
 
 
-
 class PurgePrincipalService(WorkerService):
 
     root = None
@@ -1103,7 +1075,6 @@ class PurgePrincipalService(WorkerService):
         else:
             sys.exit(0)
 
-
     @classmethod
     def main(cls):
 
@@ -1158,7 +1129,6 @@ class PurgePrincipalService(WorkerService):
             verbose=debug,
         )
 
-
     @classmethod
     @inlineCallbacks
     def purgeUIDs(cls, store, directory, uids, verbose=False, dryrun=False,
@@ -1174,7 +1144,6 @@ class PurgePrincipalService(WorkerService):
         result = yield service.doWork()
         returnValue(result)
 
-
     @inlineCallbacks
     def doWork(self):
 
@@ -1196,7 +1165,6 @@ class PurgePrincipalService(WorkerService):
 
         returnValue(total)
 
-
     @inlineCallbacks
     def _purgeUID(self, uid):
 
@@ -1238,7 +1206,6 @@ class PurgePrincipalService(WorkerService):
 
         returnValue(count)
 
-
     @inlineCallbacks
     def _cleanHome(self, txn, storeCalHome):
 
@@ -1259,7 +1226,6 @@ class PurgePrincipalService(WorkerService):
             if notificationHome is not None:
                 yield notificationHome.purge()
 
-
     @inlineCallbacks
     def _cancelEvents(self, txn, uid, cuas):
 
@@ -1350,7 +1316,6 @@ class PurgePrincipalService(WorkerService):
 
         returnValue(count)
 
-
     @inlineCallbacks
     def _removeCalendarHome(self, uid):
 
@@ -1410,7 +1375,6 @@ class PurgePrincipalService(WorkerService):
             yield txn.abort()
             raise e
 
-
     @inlineCallbacks
     def _removeAddressbookHome(self, uid):
 
@@ -1464,7 +1428,6 @@ class PurgePrincipalService(WorkerService):
 
         returnValue(count)
 
-
     @inlineCallbacks
     def _purgeProxyAssignments(self, store, uid):
 
