@@ -1,6 +1,6 @@
 # -*- test-case-name: calendarserver.tools.shell.test.test_vfs -*-
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,25 +56,22 @@ from calendarserver.tools.shell.directory import recordInfo
 log = Logger()
 
 
-
 class ListEntry(object):
     """
     Information about a C{File} as returned by C{File.list()}.
     """
 
     def __init__(self, parent, Class, Name, **fields):
-        self.parent = parent # The class implementing list()
+        self.parent = parent  # The class implementing list()
         self.fileClass = Class
         self.fileName = Name
         self.fields = fields
 
         fields["Name"] = Name
 
-
     def __str__(self):
         return self.toString()
 
-
     def __repr__(self):
         fields = self.fields.copy()
         del fields["Name"]
@@ -91,18 +88,15 @@ class ListEntry(object):
             fields,
         )
 
-
     def isFolder(self):
         return issubclass(self.fileClass, Folder)
 
-
     def toString(self):
         if self.isFolder():
             return "%s/" % (self.fileName,)
         else:
             return self.fileName
 
-
     @property
     def fieldNames(self):
         if not hasattr(self, "_fieldNames"):
@@ -117,7 +111,6 @@ class ListEntry(object):
 
         return self._fieldNames
 
-
     def toFields(self):
         try:
             return tuple(self.fields[fieldName] for fieldName in self.fieldNames)
@@ -128,62 +121,55 @@ class ListEntry(object):
             )
 
 
-
 class File(object):
     """
     Object in virtual data hierarchy.
     """
+
     def __init__(self, service, path):
         assert type(path) is tuple
 
         self.service = service
         self.path = path
 
-
     def __str__(self):
         return "/" + "/".join(self.path)
 
-
     def __repr__(self):
         return "<%s: %s>" % (self.__class__.__name__, self)
 
-
     def __eq__(self, other):
         if isinstance(other, File):
             return self.path == other.path
         else:
             return NotImplemented
 
-
     def describe(self):
         return succeed("%s (%s)" % (self, self.__class__.__name__))
 
-
     def list(self):
         return succeed((
             ListEntry(self, self.__class__, self.path[-1]),
         ))
 
 
-
 class Folder(File):
     """
     Location in virtual data hierarchy.
     """
+
     def __init__(self, service, path):
         File.__init__(self, service, path)
 
         self._children = {}
         self._childClasses = {}
 
-
     def __str__(self):
         if self.path:
             return "/" + "/".join(self.path) + "/"
         else:
             return "/"
 
-
     @inlineCallbacks
     def locate(self, path):
         if path and path[-1] == "":
@@ -202,7 +188,6 @@ class Folder(File):
 
         returnValue(target)
 
-
     @inlineCallbacks
     def child(self, name):
         # FIXME: Move this logic to locate()
@@ -226,7 +211,6 @@ class Folder(File):
 
         raise NotFoundError("Folder %r has no child %r" % (str(self), name))
 
-
     def list(self):
         result = {}
         for name in self._children:
@@ -237,7 +221,6 @@ class Folder(File):
         return succeed(result.itervalues())
 
 
-
 class RootFolder(Folder):
     """
     Root of virtual data hierarchy.
@@ -263,6 +246,7 @@ class RootFolder(Folder):
           <name>/          PrincipalHomeFolder
             ...
     """
+
     def __init__(self, service):
         Folder.__init__(self, service, ())
 
@@ -273,15 +257,14 @@ class RootFolder(Folder):
         self._childClasses["groups"] = GroupsFolder
 
 
-
 class UIDsFolder(Folder):
     """
     Folder containing all principals by UID.
     """
+
     def child(self, name):
         return PrincipalHomeFolder(self.service, self.path + (name,), name)
 
-
     @inlineCallbacks
     def list(self):
         results = {}
@@ -298,8 +281,8 @@ class UIDsFolder(Folder):
             if record:
                 info = {
                     "Record Type": record.recordType,
-                    "Short Name" : record.shortNames[0],
-                    "Full Name"  : record.fullName,
+                    "Short Name": record.shortNames[0],
+                    "Full Name": record.fullName,
                 }
             else:
                 info = {}
@@ -310,14 +293,13 @@ class UIDsFolder(Folder):
         returnValue(results.itervalues())
 
 
-
 class RecordFolder(Folder):
+
     def _recordForName(self, name):
         recordTypeAttr = "recordType_" + self.recordType
         recordType = getattr(self.service.directory, recordTypeAttr)
         return self.service.directory.recordWithShortName(recordType, name)
 
-
     def child(self, name):
         record = self._recordForName(name)
 
@@ -331,7 +313,6 @@ class RecordFolder(Folder):
             record=record
         )
 
-
     @inlineCallbacks
     def list(self):
         names = set()
@@ -356,7 +337,6 @@ class RecordFolder(Folder):
     list.fieldNames = ("UID", "Full Name")
 
 
-
 class UsersFolder(RecordFolder):
     """
     Folder containing all user principals by name.
@@ -364,7 +344,6 @@ class UsersFolder(RecordFolder):
     recordType = "users"
 
 
-
 class LocationsFolder(RecordFolder):
     """
     Folder containing all location principals by name.
@@ -372,7 +351,6 @@ class LocationsFolder(RecordFolder):
     recordType = "locations"
 
 
-
 class ResourcesFolder(RecordFolder):
     """
     Folder containing all resource principals by name.
@@ -380,7 +358,6 @@ class ResourcesFolder(RecordFolder):
     recordType = "resources"
 
 
-
 class GroupsFolder(RecordFolder):
     """
     Folder containing all group principals by name.
@@ -388,11 +365,11 @@ class GroupsFolder(RecordFolder):
     recordType = "groups"
 
 
-
 class PrincipalHomeFolder(Folder):
     """
     Folder containing everything related to a given principal.
     """
+
     def __init__(self, service, path, uid, record=None):
         Folder.__init__(self, service, path)
 
@@ -406,7 +383,6 @@ class PrincipalHomeFolder(Folder):
         self.uid = uid
         self.record = record
 
-
     @inlineCallbacks
     def _initChildren(self):
         if not hasattr(self, "_didInitChildren"):
@@ -485,41 +461,36 @@ class PrincipalHomeFolder(Folder):
 
         self._didInitChildren = True
 
-
-    def _needsChildren(m): #@NoSelf
+    def _needsChildren(m):  # @NoSelf
         def decorate(self, *args, **kwargs):
             d = self._initChildren()
             d.addCallback(lambda _: m(self, *args, **kwargs))
             return d
         return decorate
 
-
     @_needsChildren
     def child(self, name):
         return Folder.child(self, name)
 
-
     @_needsChildren
     def list(self):
         return Folder.list(self)
 
-
     def describe(self):
         return recordInfo(self.service.directory, self.record)
 
 
-
 class CalendarHomeFolder(Folder):
     """
     Calendar home folder.
     """
+
     def __init__(self, service, path, home, record):
         Folder.__init__(self, service, path)
 
         self.home = home
         self.record = record
 
-
     @inlineCallbacks
     def child(self, name):
         calendar = (yield self.home.calendarWithName(name))
@@ -528,7 +499,6 @@ class CalendarHomeFolder(Folder):
         else:
             raise NotFoundError("Calendar home %r has no calendar %r" % (self, name))
 
-
     @inlineCallbacks
     def list(self):
         calendars = (yield self.home.calendars())
@@ -540,12 +510,11 @@ class CalendarHomeFolder(Folder):
 
             info = {
                 "Display Name": displayName,
-                "Sync Token"  : (yield calendar.syncToken()),
+                "Sync Token": (yield calendar.syncToken()),
             }
             result.append(ListEntry(self, CalendarFolder, calendar.name(), **info))
         returnValue(result)
 
-
     @inlineCallbacks
     def describe(self):
         description = ["Calendar home:\n"]
@@ -565,7 +534,7 @@ class CalendarHomeFolder(Folder):
         rows = []
         rows.append(("UID", uid))
         rows.append(("Owner", "(%s)%s" % (recordType, recordShortName)))
-        rows.append(("Created"      , timeString(created)))
+        rows.append(("Created", timeString(created)))
         rows.append(("Last modified", timeString(modified)))
         if quotaUsed is not None:
             rows.append((
@@ -587,23 +556,21 @@ class CalendarHomeFolder(Folder):
         returnValue("\n".join(description))
 
 
-
 class CalendarFolder(Folder):
     """
     Calendar.
     """
+
     def __init__(self, service, path, calendar):
         Folder.__init__(self, service, path)
 
         self.calendar = calendar
 
-
     @inlineCallbacks
     def _childWithObject(self, object):
         uid = (yield object.uid())
         returnValue(CalendarObject(self.service, self.path + (uid,), object, uid))
 
-
     @inlineCallbacks
     def child(self, name):
         object = (yield self.calendar.calendarObjectWithUID(name))
@@ -614,7 +581,6 @@ class CalendarFolder(Folder):
         child = (yield self._childWithObject(object))
         returnValue(child)
 
-
     @inlineCallbacks
     def list(self):
         result = []
@@ -626,7 +592,6 @@ class CalendarFolder(Folder):
 
         returnValue(result)
 
-
     @inlineCallbacks
     def describe(self):
         description = ["Calendar:\n"]
@@ -634,11 +599,11 @@ class CalendarFolder(Folder):
         #
         # Attributes
         #
-        ownerHome = (yield self.calendar.ownerCalendarHome()) # FIXME: Translate into human
+        ownerHome = (yield self.calendar.ownerCalendarHome())  # FIXME: Translate into human
         syncToken = (yield self.calendar.syncToken())
 
         rows = []
-        rows.append(("Owner"     , ownerHome))
+        rows.append(("Owner", ownerHome))
         rows.append(("Sync Token", syncToken))
 
         description.append("Attributes:")
@@ -653,7 +618,6 @@ class CalendarFolder(Folder):
 
         returnValue("\n".join(description))
 
-
     def delete(self, implicit=True):
 
         if implicit:
@@ -664,18 +628,17 @@ class CalendarFolder(Folder):
             self.calendarObject.remove()
 
 
-
 class CalendarObject(File):
     """
     Calendar object.
     """
+
     def __init__(self, service, path, calendarObject, uid):
         File.__init__(self, service, path)
 
         self.object = calendarObject
         self.uid = uid
 
-
     @inlineCallbacks
     def lookup(self):
         if not hasattr(self, "component"):
@@ -691,7 +654,7 @@ class CalendarObject(File):
                 self.mainComponent = mainComponent
 
             except InvalidICalendarDataError, e:
-                log.error("%s: %s" % (self.path, e))
+                log.error("{path}: {ex}", path=self.path, ex=e)
 
                 self.componentType = "?"
                 self.summary = "** Invalid data **"
@@ -699,7 +662,6 @@ class CalendarObject(File):
 
             self.component = component
 
-
     @inlineCallbacks
     def list(self):
         (yield self.lookup())
@@ -715,7 +677,6 @@ class CalendarObject(File):
         (yield self.lookup())
         returnValue(str(self.component))
 
-
     @inlineCallbacks
     def describe(self):
         (yield self.lookup())
@@ -736,12 +697,12 @@ class CalendarObject(File):
             organizerName = organizer.parameterValue("CN")
             organizerEmail = organizer.parameterValue("EMAIL")
 
-            name = " (%s)" % (organizerName ,) if organizerName else ""
+            name = " (%s)" % (organizerName,) if organizerName else ""
             email = " <%s>" % (organizerEmail,) if organizerEmail else ""
 
             rows.append(("Organizer", "%s%s%s" % (organizer.value(), name, email)))
 
-        rows.append(("Created" , timeString(self.object.created())))
+        rows.append(("Created", timeString(self.object.created())))
         rows.append(("Modified", timeString(self.object.modified())))
 
         description.append("Attributes:")
@@ -756,12 +717,12 @@ class CalendarObject(File):
             contentType = "%s/%s" % (contentType.mediaType, contentType.mediaSubtype)
 
             rows = []
-            rows.append(("Name"        , attachment.name()))
-            rows.append(("Size"        , "%s bytes" % (attachment.size(),)))
+            rows.append(("Name", attachment.name()))
+            rows.append(("Size", "%s bytes" % (attachment.size(),)))
             rows.append(("Content Type", contentType))
-            rows.append(("MD5 Sum"     , attachment.md5()))
-            rows.append(("Created"     , timeString(attachment.created())))
-            rows.append(("Modified"    , timeString(attachment.modified())))
+            rows.append(("MD5 Sum", attachment.md5()))
+            rows.append(("Created", timeString(attachment.created())))
+            rows.append(("Modified", timeString(attachment.modified())))
 
             description.append("Attachment:")
             description.append(tableString(rows))
@@ -776,11 +737,11 @@ class CalendarObject(File):
         returnValue("\n".join(description))
 
 
-
 class AddressBookHomeFolder(Folder):
     """
     Address book home folder.
     """
+
     def __init__(self, service, path, home, record):
         Folder.__init__(self, service, path)
 
@@ -790,7 +751,6 @@ class AddressBookHomeFolder(Folder):
     # FIXME
 
 
-
 def tableString(rows, header=None):
     table = Table()
     if header:
@@ -803,7 +763,6 @@ def tableString(rows, header=None):
     return output.getvalue()
 
 
-
 def tableStringForProperties(properties):
     return "Properties:\n%s" % (tableString((
         (name.toString(), truncateAtNewline(properties[name]))
@@ -811,7 +770,6 @@ def tableStringForProperties(properties):
     )))
 
 
-
 def timeString(time):
     if time is None:
         return "(unknown)"
@@ -819,7 +777,6 @@ def timeString(time):
     return strftime("%a, %d %b %Y %H:%M:%S %z(%Z)", localtime(time))
 
 
-
 def truncateAtNewline(text):
     text = str(text)
     try:
