@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2013-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2013-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,13 +28,14 @@ import plistlib
 PREAMBLE = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 """
+
+
 class WritableConfigTestCase(TestCase):
 
     def setUp(self):
         self.configFile = self.mktemp()
         self.fp = FilePath(self.configFile)
 
-
     def test_readSuccessful(self):
         content = """<plist version="1.0">
     <dict>
@@ -49,14 +50,12 @@ class WritableConfigTestCase(TestCase):
         writable.read()
         self.assertEquals(writable.currentConfigSubset, {"string": "foo"})
 
-
     def test_readInvalidXML(self):
         self.fp.setContent("invalid")
         config = ConfigDict()
         writable = WritableConfig(config, self.configFile)
         self.assertRaises(ExpatError, writable.read)
 
-
     def test_updates(self):
         content = """<plist version="1.0">
     <dict>
@@ -72,7 +71,7 @@ class WritableConfigTestCase(TestCase):
         writable.read()
         writable.set({"key1": "after"})
         writable.set({"key2": 15})
-        writable.set({"key2": 20}) # override previous set
+        writable.set({"key2": 20})  # override previous set
         writable.set({"key3": ["a", "b", "c"]})
         self.assertEquals(writable.currentConfigSubset, {"key1": "after", "key2": 20, "key3": ["a", "b", "c"]})
         writable.save()
@@ -81,7 +80,6 @@ class WritableConfigTestCase(TestCase):
         writable2.read()
         self.assertEquals(writable2.currentConfigSubset, {"key1": "after", "key2": 20, "key3": ["a", "b", "c"]})
 
-
     def test_convertToValue(self):
         self.assertEquals(True, WritableConfig.convertToValue("True"))
         self.assertEquals(False, WritableConfig.convertToValue("False"))
@@ -90,7 +88,6 @@ class WritableConfigTestCase(TestCase):
         self.assertEquals("xyzzy", WritableConfig.convertToValue("xyzzy"))
         self.assertEquals("xy.zzy", WritableConfig.convertToValue("xy.zzy"))
 
-
     def test_processArgs(self):
         """
         Ensure utf-8 encoded command line args are handled properly
@@ -111,7 +108,6 @@ class WritableConfigTestCase(TestCase):
         self.assertEquals(writable2.currentConfigSubset, {'key1': u'\U0001f4a3'})
 
 
-
 class ConfigTestCase(RunCommandTestCase):
 
     @inlineCallbacks
@@ -135,7 +131,6 @@ class ConfigTestCase(RunCommandTestCase):
         # Verify not all keys are present, such as umask which is not accessible
         self.assertFalse("umask" in results["result"])
 
-
     @inlineCallbacks
     def test_writeConfig(self):
         """
@@ -159,7 +154,6 @@ class ConfigTestCase(RunCommandTestCase):
         staticPlist = plistlib.readPlist(self.configFileName)
         self.assertTrue(staticPlist["EnableCalDAV"])
 
-
     @inlineCallbacks
     def test_error(self):
         """
@@ -170,7 +164,6 @@ class ConfigTestCase(RunCommandTestCase):
             script="calendarserver_config")
         self.assertEquals(results["error"], "Unknown command 'bogus'")
 
-
     @inlineCallbacks
     def test_commandLineArgs(self):
         """
@@ -218,7 +211,7 @@ class ConfigTestCase(RunCommandTestCase):
 
         self.assertEquals(
             results["result"]["LogLevels"],
-            {"testing": "debug", "testing2" : "warn"}
+            {"testing": "debug", "testing2": "warn"}
         )
 
         # Test that setting additional sub-keys retains previous sub-keys
@@ -236,7 +229,7 @@ class ConfigTestCase(RunCommandTestCase):
 
         self.assertEquals(
             results["result"]["LogLevels"],
-            {"testing": "debug", "testing2" : "warn", "testing3": "info", "testing4" : "warn"}
+            {"testing": "debug", "testing2": "warn", "testing3": "info", "testing4": "warn"}
         )
 
         # Test that an empty value deletes the key
@@ -253,18 +246,16 @@ class ConfigTestCase(RunCommandTestCase):
             script="calendarserver_config")
 
         self.assertEquals(
-            results["result"]["LogLevels"], # now an empty dict
+            results["result"]["LogLevels"],  # now an empty dict
             {}
         )
 
-
     @inlineCallbacks
     def test_loggingCategories(self):
         """
         Verify logging categories get mapped to the correct python module names
         """
 
-
         # Check existing values
 
         results = yield self.runCommand(
@@ -332,7 +323,6 @@ class ConfigTestCase(RunCommandTestCase):
             results["result"]["LogLevels"], {}
         )
 
-
     @inlineCallbacks
     def test_accountingCategories(self):
         """
@@ -351,8 +341,10 @@ class ConfigTestCase(RunCommandTestCase):
                 'HTTP': False,
                 'Implicit Errors': False,
                 'iSchedule': False,
+                'xPod': False,
                 'iTIP': False,
                 'iTIP-VFREEBUSY': False,
+                'Invalid Instance': False,
                 'migration': False,
             }
         )
@@ -377,8 +369,10 @@ class ConfigTestCase(RunCommandTestCase):
                 'HTTP': True,
                 'Implicit Errors': False,
                 'iSchedule': False,
+                'xPod': False,
                 'iTIP': True,
                 'iTIP-VFREEBUSY': True,
+                'Invalid Instance': False,
                 'migration': False,
             }
         )
@@ -403,15 +397,14 @@ class ConfigTestCase(RunCommandTestCase):
                 'HTTP': False,
                 'Implicit Errors': False,
                 'iSchedule': False,
+                'xPod': False,
                 'iTIP': False,
                 'iTIP-VFREEBUSY': False,
+                'Invalid Instance': False,
                 'migration': False,
             }
         )
 
-
-
-
     def test_keyPath(self):
         d = ConfigDict()
         setKeyPath(d, "one", "A")
@@ -434,18 +427,17 @@ class ConfigTestCase(RunCommandTestCase):
         self.assertEquals(getKeyPath(d, "three.one.one"), "F")
         self.assertEquals(getKeyPath(d, "three.one.two"), "G")
 
-
     def test_flattenDictionary(self):
         dictionary = {
-            "one" : "A",
-            "two" : {
-                "one" : "D",
-                "two" : "E",
+            "one": "A",
+            "two": {
+                "one": "D",
+                "two": "E",
             },
-            "three" : {
-                "one" : {
-                    "one" : "F",
-                    "two" : "G",
+            "three": {
+                "one": {
+                    "one": "F",
+                    "two": "G",
                 },
             },
         }
