@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- test-case-name: calendarserver.tools.test.test_upgrade -*-
 ##
-# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,16 +20,19 @@
 This tool allows any necessary upgrade to complete, then exits.
 """
 
-from __future__ import print_function
 import os
 import sys
 import time
 
+from txdav.common.datastore.sql import CommonDataStore
+
 from twisted.internet.defer import succeed
+from twisted.logger import LogLevel, formatEvent, formatTime
+from twisted.logger import FilteringLogObserver, LogLevelFilterPredicate
 from twisted.python.text import wordWrap
 from twisted.python.usage import Options, UsageError
 
-from twext.python.log import Logger, LogLevel, formatEvent, addObserver
+from twext.python.log import Logger
 
 from twistedcaldav.stdconfig import DEFAULT_CONFIG_FILE
 
@@ -39,7 +42,6 @@ from calendarserver.tap.caldav import CalDAVServiceMaker
 log = Logger()
 
 
-
 def usage(e=None):
     if e:
         print(e)
@@ -63,6 +65,7 @@ description = '\n'.join(
     )
 )
 
+
 class UpgradeOptions(Options):
     """
     Command-line options for 'calendarserver_upgrade'
@@ -76,6 +79,7 @@ class UpgradeOptions(Options):
 
     optFlags = [
         ['status', 's', "Check database status and exit."],
+        ['check', 'c', "Check current database schema and exit."],
         ['postprocess', 'p', "Perform post-database-import processing."],
         ['debug', 'D', "Print log messages to STDOUT."],
     ]
@@ -91,7 +95,6 @@ class UpgradeOptions(Options):
         self.outputName = '-'
         self.merge = False
 
-
     def opt_output(self, filename):
         """
         Specify output file path (default: '-', meaning stdout).
@@ -100,7 +103,6 @@ class UpgradeOptions(Options):
 
     opt_o = opt_output
 
-
     def opt_merge(self):
         """
         Rather than skipping homes that exist on the filesystem but not in the
@@ -110,7 +112,6 @@ class UpgradeOptions(Options):
 
     opt_m = opt_merge
 
-
     def openOutput(self):
         """
         Open the appropriate output file based on the '--output' option.
@@ -121,7 +122,6 @@ class UpgradeOptions(Options):
             return open(self.outputName, 'wb')
 
 
-
 class UpgraderService(WorkerService, object):
     """
     Service which runs, exports the appropriate records, then stops the reactor.
@@ -137,7 +137,6 @@ class UpgraderService(WorkerService, object):
         self.config = config
         self._directory = None
 
-
     def doWork(self):
         """
         Immediately stop.  The upgrade will have been run before this.
@@ -145,24 +144,40 @@ class UpgraderService(WorkerService, object):
         # If we get this far the database is OK
         if self.options["status"]:
             self.output.write("Database OK.\n")
+        elif self.options["check"]:
+            checkResult = getattr(CommonDataStore, "checkSchemaResults", None)
+            if checkResult is not None:
+                if checkResult:
+                    self.output.write("Database check FAILED:\n{}\n".format("\n".join(checkResult)))
+                else:
+                    self.output.write("Database check OK.\n")
+            else:
+                self.output.write("Database check FAILED.\n")
         else:
             self.output.write("Upgrade complete, shutting down.\n")
         UpgraderService.started = True
         return succeed(None)
 
-
     def doWorkWithoutStore(self):
         """
-        Immediately stop.  The upgrade will have been run before this.
+        Immediately stop.  The upgrade will have been run before this and failed.
         """
         if self.options["status"]:
             self.output.write("Upgrade needed.\n")
+        elif self.options["check"]:
+            checkResult = getattr(CommonDataStore, "checkSchemaResults", None)
+            if checkResult is not None:
+                if checkResult:
+                    self.output.write("Database check FAILED:\n{}\n".format("\n".join(checkResult)))
+                else:
+                    self.output.write("Database check OK.\n")
+            else:
+                self.output.write("Database check FAILED.\n")
         else:
             self.output.write("Upgrade failed.\n")
         UpgraderService.started = True
         return succeed(None)
 
-
     def postStartService(self):
         """
         Quit right away
@@ -175,7 +190,6 @@ class UpgraderService(WorkerService, object):
             # I don't care.
             pass
 
-
     def stopService(self):
         """
         Stop the service.  Nothing to do; everything should be finished by this
@@ -183,7 +197,6 @@ class UpgraderService(WorkerService, object):
         """
 
 
-
 def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
     """
     Do the export.
@@ -209,32 +222,33 @@ def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
             data.MergeUpgrades = True
         config.addPostUpdateHooks([setMerge])
 
-
     def makeService(store):
         return UpgraderService(store, options, output, reactor, config)
 
-
     def onlyUpgradeEvents(eventDict):
         text = formatEvent(eventDict)
-        output.write(logDateString() + " " + text + "\n")
+        output.write(formatTime(eventDict.get("log_time", time.time())) + " " + text + "\n")
         output.flush()
 
-    if not options["status"]:
-        log.publisher.levels.setLogLevelForNamespace(None, LogLevel.debug)
-        addObserver(onlyUpgradeEvents)
-
+    if not options["status"] and not options["check"]:
+        # When doing an upgrade always send L{LogLevel.warn} logging to the tool output
+        log.observer.addObserver(FilteringLogObserver(
+            onlyUpgradeEvents,
+            [LogLevelFilterPredicate(defaultLogLevel=LogLevel.warn), ]
+        ))
 
     def customServiceMaker():
         customService = CalDAVServiceMaker()
         customService.doPostImport = options["postprocess"]
         return customService
 
-
     def _patchConfig(config):
-        config.FailIfUpgradeNeeded = options["status"]
+        config.FailIfUpgradeNeeded = options["status"] or options["check"]
+        config.CheckExistingSchema = options["check"]
         if options["prefix"]:
             config.UpgradeHomePrefix = options["prefix"]
-
+        if not options["status"] and not options["check"]:
+            config.DefaultLogLevel = "debug"
 
     def _onShutdown():
         if not UpgraderService.started:
@@ -243,28 +257,5 @@ def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
     utilityMain(options["config"], makeService, reactor, customServiceMaker, patchConfig=_patchConfig, onShutdown=_onShutdown, verbose=options["debug"])
 
 
-
-def logDateString():
-    logtime = time.localtime()
-    Y, M, D, h, m, s = logtime[:6]
-    tz = computeTimezoneForLog(time.timezone)
-
-    return '%02d-%02d-%02d %02d:%02d:%02d%s' % (Y, M, D, h, m, s, tz)
-
-
-
-def computeTimezoneForLog(tz):
-    if tz > 0:
-        neg = 1
-    else:
-        neg = 0
-        tz = -tz
-    h, rem = divmod(tz, 3600)
-    m, rem = divmod(rem, 60)
-    if neg:
-        return '-%02d%02d' % (h, m)
-    else:
-        return '+%02d%02d' % (h, m)
-
 if __name__ == '__main__':
     main()
