@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -44,7 +44,6 @@ from twistedcaldav.timezones import hasTZ
 DEFAULT_TIMEZONE = "America/Los_Angeles"
 
 
-
 class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
 
     def defaultAccessControlList(self):
@@ -61,37 +60,29 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
             )
         )
 
-
     def etag(self):
         # Can't be calculated here
         return succeed(None)
 
-
     def contentLength(self):
         # Can't be calculated here
         return None
 
-
     def lastModified(self):
         return None
 
-
     def exists(self):
         return True
 
-
     def displayName(self):
         return "Web Calendar"
 
-
     def contentType(self):
         return MimeType.fromString("text/html; charset=utf-8")
 
-
     def contentEncoding(self):
         return None
 
-
     def createSimilarFile(self, path):
         return DAVFile(path, principalCollections=self.principalCollections())
 
@@ -131,11 +122,8 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
         # If we don't have a cached template, load it up.
         #
         if not hasattr(self, cacheAttr):
-            templateFile = open(templateFileName)
-            try:
+            with open(templateFileName) as templateFile:
                 htmlContent = templateFile.read()
-            finally:
-                templateFile.close()
 
             if debug:
                 # Don't cache
@@ -144,7 +132,6 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
 
         return getattr(self, cacheAttr)
 
-
     def render(self, request):
         if not self.fp.isdir():
             return responsecode.NOT_FOUND
@@ -172,7 +159,7 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
         tzid = queryValue("tzid")
         if not tzid:
             tzid = getLocalTimezone()
-            self.log.debug("Determined timezone to be %s" % (tzid,))
+            self.log.debug("Determined timezone to be {tzid}", tzid=tzid)
 
         #
         # Make some HTML
@@ -199,19 +186,20 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
         return response
 
 
-
 try:
-    from Foundation import NSTimeZone
+    from osx.utils import CFTimeZoneRef
 
     def lookupSystemTimezone():
-        return NSTimeZone.localTimeZone().name().encode("utf-8")
+        try:
+            return CFTimeZoneRef.defaultTimeZoneName()
+        except:
+            return ""
 
 except ImportError:
     def lookupSystemTimezone():
         return ""
 
 
-
 def getLocalTimezone():
     """
     Returns the default timezone for the server.  The order of precedence is:
