@@ -1,6 +1,6 @@
 # -*- test-case-name: contrib.performance.loadtest.test_population -*-
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -62,12 +62,10 @@ class ProfileType(object, FancyEqMixin):
         self.profileType = profileType
         self.params = params
 
-
     def __call__(self, reactor, simulator, client, number):
         return self.profileType(reactor, simulator, client, number, **self.params)
 
 
-
 class ClientType(object, FancyEqMixin):
     """
     @ivar clientType: An L{ICalendarClient} implementation
@@ -80,19 +78,17 @@ class ClientType(object, FancyEqMixin):
         self.clientParams = clientParams
         self.profileTypes = profileTypes
 
-
-    def new(self, reactor, serverAddress, principalPathTemplate, serializationPath, userRecord, authInfo):
+    def new(self, reactor, server, principalPathTemplate, serializationPath, userRecord, authInfo, instanceNumber):
         """
         Create a new instance of this client type.
         """
         return self.clientType(
-            reactor, serverAddress, principalPathTemplate,
-            serializationPath, userRecord, authInfo,
+            reactor, server, principalPathTemplate,
+            serializationPath, userRecord, authInfo, instanceNumber,
             **self.clientParams
         )
 
 
-
 class PopulationParameters(object, FancyEqMixin):
     """
     Descriptive statistics about a population of Calendar Server users.
@@ -102,7 +98,6 @@ class PopulationParameters(object, FancyEqMixin):
     def __init__(self):
         self.clients = []
 
-
     def addClient(self, weight, clientType):
         """
         Add another type of client to these parameters.
@@ -117,7 +112,6 @@ class PopulationParameters(object, FancyEqMixin):
         """
         self.clients.append((weight, clientType))
 
-
     def clientTypes(self):
         """
         Return a list of two-tuples giving the weights and types of
@@ -126,7 +120,6 @@ class PopulationParameters(object, FancyEqMixin):
         return self.clients
 
 
-
 class Populator(object):
     """
     @ivar userPattern: A C{str} giving a formatting pattern to use to
@@ -137,17 +130,16 @@ class Populator(object):
     @ivar passwordPattern: Similar to C{userPattern}, but for
         passwords.
     """
+
     def __init__(self, random):
         self._random = random
 
-
     def _cycle(self, elements):
         while True:
             for (weight, value) in elements:
                 for _ignore_i in range(weight):
                     yield value
 
-
     def populate(self, parameters):
         """
         Generate individuals such as might be randomly selected from a
@@ -160,14 +152,15 @@ class Populator(object):
             yield clientType
 
 
-
 class CalendarClientSimulator(object):
-    def __init__(self, records, populator, parameters, reactor, server,
+
+    def __init__(self, records, populator, random, parameters, reactor, servers,
                  principalPathTemplate, serializationPath, workerIndex=0, workerCount=1):
         self._records = records
         self.populator = populator
+        self._random = random
         self.reactor = reactor
-        self.server = server
+        self.servers = servers
         self.principalPathTemplate = principalPathTemplate
         self.serializationPath = serializationPath
         self._pop = self.populator.populate(parameters)
@@ -179,34 +172,52 @@ class CalendarClientSimulator(object):
 
         TimezoneCache.create()
 
-
     def getUserRecord(self, index):
         return self._records[index]
 
+    def getRandomUserRecord(self, besides=None):
+        count = len(self._records)
+
+        if count == 0:
+            # No records!
+            return None
+
+        if count == 1 and besides == 0:
+            # There is only one item and caller doesn't want it!
+            return None
+
+        for _i in xrange(100):
+            # Try to find one that is not "besides"
+            n = self._random.randint(0, count - 1)
+            if besides != n:
+                # Got it.
+                break
+        else:
+            # Give up
+            return None
+        return self._records[n]
 
     def _nextUserNumber(self):
         result = self._user
         self._user += 1
         return result
 
-
     def _createUser(self, number):
         record = self._records[number]
         user = record.uid
         authBasic = HTTPBasicAuthHandler(password_mgr=HTTPPasswordMgrWithDefaultRealm())
         authBasic.add_password(
             realm=None,
-            uri=self.server,
+            uri=self.servers[record.podID]["uri"],
             user=user.encode('utf-8'),
             passwd=record.password.encode('utf-8'))
         authDigest = HTTPDigestAuthHandler(passwd=HTTPPasswordMgrWithDefaultRealm())
         authDigest.add_password(
             realm=None,
-            uri=self.server,
+            uri=self.servers[record.podID]["uri"],
             user=user.encode('utf-8'),
             passwd=record.password.encode('utf-8'))
-        return user, {"basic": authBasic, "digest": authDigest, }
-
+        return record, user, {"basic": authBasic, "digest": authDigest, }
 
     def stop(self):
         """
@@ -223,8 +234,8 @@ class CalendarClientSimulator(object):
         self._stopped = True
         return DeferredList(deferreds)
 
-
     def add(self, numClients, clientsPerUser):
+        instanceNumber = 0
         for _ignore_n in range(numClients):
             number = self._nextUserNumber()
 
@@ -236,17 +247,19 @@ class CalendarClientSimulator(object):
                     # runs the same arrival policy).
                     continue
 
-                _ignore_user, auth = self._createUser(number)
+                record, _ignore_user, auth = self._createUser(number)
 
                 reactor = loggedReactor(self.reactor)
                 client = clientType.new(
                     reactor,
-                    self.server,
+                    self.servers[record.podID],
                     self.principalPathTemplate,
                     self.serializationPath,
-                    self.getUserRecord(number),
+                    record,
                     auth,
+                    instanceNumber
                 )
+                instanceNumber += 1
                 self.clients.append(client)
                 d = client.run()
                 d.addErrback(self._clientFailure, reactor)
@@ -255,6 +268,7 @@ class CalendarClientSimulator(object):
                     profile = profileType(reactor, self, client, number)
                     if profile.enabled:
                         d = profile.initialize()
+
                         def _run(result):
                             d2 = profile.run()
                             d2.addErrback(self._profileFailure, profileType, reactor)
@@ -265,7 +279,6 @@ class CalendarClientSimulator(object):
         # really used by much anyway.
         msg(type="status", clientCount=self._user - 1)
 
-
     def _dumpLogs(self, loggingReactor, reason):
         path = FilePath(mkdtemp())
         logstate = loggingReactor.getLogFiles()
@@ -277,7 +290,6 @@ class CalendarClientSimulator(object):
         path.child('reason.log').setContent(reason.getTraceback())
         return path
 
-
     def _clientFailure(self, reason, reactor):
         if not self._stopped:
             where = self._dumpLogs(reactor, reason)
@@ -287,21 +299,19 @@ class CalendarClientSimulator(object):
                 reason = Failure(reason)
             msg(type="client-failure", reason="%s: %s" % (reason.type, reason.value,))
 
-
     def _profileFailure(self, reason, profileType, reactor):
         if not self._stopped:
             where = self._dumpLogs(reactor, reason)
             err(reason, "Profile stopped with error; recent traffic in %r" % (
                 where.path,))
 
-
     def _simFailure(self, reason, reactor):
         if not self._stopped:
             msg(type="sim-failure", reason=reason)
 
 
-
 class SmoothRampUp(object):
+
     def __init__(self, reactor, groups, groupSize, interval, clientsPerUser):
         self.reactor = reactor
         self.groups = groups
@@ -309,15 +319,14 @@ class SmoothRampUp(object):
         self.interval = interval
         self.clientsPerUser = clientsPerUser
 
-
     def run(self, simulator):
         for i in range(self.groups):
             self.reactor.callLater(
                 self.interval * i, simulator.add, self.groupSize, self.clientsPerUser)
 
 
-
 class StatisticsBase(object):
+
     def observe(self, event):
         if event.get('type') == 'response':
             self.eventReceived(event)
@@ -326,23 +335,20 @@ class StatisticsBase(object):
         elif event.get('type') == 'sim-failure':
             self.simFailure(event)
 
-
     def report(self, output):
         pass
 
-
     def failures(self):
         return []
 
 
-
 class SimpleStatistics(StatisticsBase):
+
     def __init__(self):
         self._times = []
         self._failures = collections.defaultdict(int)
         self._simFailures = collections.defaultdict(int)
 
-
     def eventReceived(self, event):
         self._times.append(event['duration'])
         if len(self._times) == 200:
@@ -352,16 +358,13 @@ class SimpleStatistics(StatisticsBase):
             print('mad:', mad(self._times))
             del self._times[:100]
 
-
     def clientFailure(self, event):
         self._failures[event] += 1
 
-
     def simFailure(self, event):
         self._simFailures[event] += 1
 
 
-
 class ReportStatistics(StatisticsBase, SummarizingMixin):
     """
 
@@ -383,8 +386,9 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
     _fail_cut_off = 1.0     # % of total count at which failed requests will cause a failure
 
     _fields_init = [
-        ('request', -25, '%-25s'),
+        ('request', -30, '%-30s'),
         ('count', 8, '%8s'),
+        ('%', 6, '%6.1f'),
         ('failed', 8, '%8s'),
     ]
 
@@ -403,11 +407,12 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
         self._failed_clients = []
         self._failed_sim = collections.defaultdict(int)
         self._startTime = datetime.now()
-        self._expired_data = None
+        self._expired_data = {}
 
         # Load parameters from config
         if "thresholdsPath" in params:
-            jsondata = json.load(open(params["thresholdsPath"]))
+            with open(params["thresholdsPath"]) as f:
+                jsondata = json.load(f)
         elif "thresholds" in params:
             jsondata = params["thresholds"]
         else:
@@ -423,55 +428,46 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
         self._fields.extend(self._fields_extend)
 
         if "benchmarksPath" in params:
-            self.benchmarks = json.load(open(params["benchmarksPath"]))
+            with open(params["benchmarksPath"]) as f:
+                self.benchmarks = json.load(f)
         else:
             self.benchmarks = {}
 
         if "failCutoff" in params:
             self._fail_cut_off = params["failCutoff"]
 
-
     def observe(self, event):
         if event.get('type') == 'sim-expired':
             self.simExpired(event)
         else:
             super(ReportStatistics, self).observe(event)
 
-
     def countUsers(self):
         return len(self._users)
 
-
     def countClients(self):
         return len(self._clients)
 
-
     def countClientFailures(self):
         return len(self._failed_clients)
 
-
     def countSimFailures(self):
         return len(self._failed_sim)
 
-
     def eventReceived(self, event):
         dataset = self._perMethodTimes.setdefault(event['method'], [])
         dataset.append((event['success'], event['duration']))
         self._users.add(event['user'])
         self._clients.add(event['client_id'])
 
-
     def clientFailure(self, event):
         self._failed_clients.append(event['reason'])
 
-
     def simFailure(self, event):
         self._failed_sim[event['reason']] += 1
 
-
     def simExpired(self, event):
-        self._expired_data = event['reason']
-
+        self._expired_data[event['podID']] = event['reason']
 
     def printMiscellaneous(self, output, items):
         maxColumnWidth = str(len(max(items.iterkeys(), key=len)))
@@ -479,7 +475,6 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
         for k in sorted(items.iterkeys()):
             output.write(fmt % (k.title(), items[k],))
 
-
     def qos(self):
         """
         Determine a "quality of service" value that can be used for comparisons between runs. This value
@@ -490,7 +485,10 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
         # Get means for each type of method
         means = {}
         for method, results in self._perMethodTimes.items():
-            means[method] = mean([duration for success, duration in results if success])
+            try:
+                means[method] = mean([duration for success, duration in results if success])
+            except ZeroDivisionError:
+                pass  # Ignore the case where all samples were unsuccessful?
 
         # Determine percentage differences with weighting
         differences = []
@@ -501,7 +499,6 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
 
         return ("%-8.4f" % mean(differences)) if differences else "None"
 
-
     def qos_value(self, method, value):
         benchmark = self.benchmarks.get(method)
         if benchmark is None:
@@ -509,15 +506,13 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
         test_mean, weight = (benchmark["mean"], benchmark["weight"],)
         return ((value / test_mean) - 1.0) * weight + 1.0
 
-
-    def _summarizeData(self, operation, data):
-        data = SummarizingMixin._summarizeData(self, operation, data)
+    def _summarizeData(self, operation, data, total_count):
+        data = SummarizingMixin._summarizeData(self, operation, data, total_count)
         value = self.qos_value(operation, data[-4])
         if value is None:
             value = 0.0
         return data[:-1] + (value,) + data[-1:]
 
-
     def report(self, output):
         output.write("\n")
         output.write("** REPORT **\n")
@@ -550,14 +545,14 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
         self.printMiscellaneous(output, items)
         output.write("\n")
 
-        if self._expired_data is not None:
+        for podID, data in self._expired_data.items():
             items = {
-                "Req/sec" : "%.1f" % (self._expired_data[0],),
-                "Response": "%.1f (ms)" % (self._expired_data[1],),
-                "Slots": "%.2f" % (self._expired_data[2],),
-                "CPU": "%.1f%%" % (self._expired_data[3],),
+                "Req/sec": "%.1f" % (data[0],),
+                "Response": "%.1f (ms)" % (data[1],),
+                "Slots": "%.2f" % (data[2],),
+                "CPU": "%.1f%%" % (data[3],),
             }
-            output.write("* Server (Last 5 minutes)\n")
+            output.write("* Server {podID} (Last 5 minutes)\n".format(podID=podID))
             self.printMiscellaneous(output, items)
             output.write("\n")
         output.write("* Details\n")
@@ -616,7 +611,6 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
         return reasons
 
 
-
 def main():
     import random
 
@@ -638,7 +632,7 @@ def main():
     parameters.addClient(
         1, ClientType(OS_X_10_6, [Eventer, Inviter, Accepter]))
     simulator = CalendarClientSimulator(
-        populator, parameters, reactor, '127.0.0.1', 8008)
+        populator, r, parameters, reactor, '127.0.0.1', 8008)
 
     arrivalPolicy = SmoothRampUp(groups=10, groupSize=1, interval=3)
     arrivalPolicy.run(reactor, simulator)
