@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ from __future__ import division
 import json
 import random
 import sys
+from urlparse import urljoin, urlparse
 from uuid import uuid4
 
 from caldavclientlibrary.protocol.caldav.definitions import caldavxml
@@ -31,7 +32,7 @@ from caldavclientlibrary.protocol.caldav.definitions import caldavxml
 from twisted.python import context
 from twisted.python.log import msg
 from twisted.python.failure import Failure
-from twisted.internet.defer import Deferred, succeed, fail
+from twisted.internet.defer import Deferred, succeed, inlineCallbacks, returnValue
 from twisted.internet.task import LoopingCall
 from twisted.web.http import PRECONDITION_FAILED
 
@@ -40,11 +41,15 @@ from twistedcaldav.ical import Property, Component
 from contrib.performance.stats import NearFutureDistribution, NormalDistribution, UniformDiscreteDistribution, mean, median
 from contrib.performance.stats import LogNormalDistribution, RecurrenceDistribution
 from contrib.performance.loadtest.logger import SummarizingMixin
-from contrib.performance.loadtest.ical import IncorrectResponseCode
+from contrib.performance.loadtest.ical import Calendar, IncorrectResponseCode
 
 from pycalendar.datetime import DateTime
 from pycalendar.duration import Duration
 
+from datetime import datetime, timedelta
+import dateutil
+
+
 class ProfileBase(object):
     """
     Base class which provides some conveniences for profile
@@ -59,11 +64,9 @@ class ProfileBase(object):
         self._number = userNumber
         self.setParameters(**params)
 
-
     def setParameters(self):
         pass
 
-
     def initialize(self):
         """
         Called before the profile runs for real. Can be used to initialize client state.
@@ -72,14 +75,55 @@ class ProfileBase(object):
         """
         return succeed(None)
 
+    def _calendarsOfType(self, calendarType, componentType, justOwned=False):
+        results = []
 
-    def _calendarsOfType(self, calendarType, componentType):
-        return [
-            cal
-            for cal
-            in self._client._calendars.itervalues()
-            if cal.resourceType == calendarType and componentType in cal.componentTypes]
+        for cal in self._client._calendars.itervalues():
+            if cal.resourceType == calendarType and componentType in cal.componentTypes:
+                if justOwned:
+                    if (not cal.shared) or cal.sharedByMe:
+                        results.append(cal)
+                else:
+                    results.append(cal)
 
+        return results
+
+    def _getRandomCalendarOfType(self, componentType, justOwned=False):
+        """
+        Return a random L{Calendar} object from the current user
+        or C{None} if there are no calendars to work with
+        """
+        calendars = self._calendarsOfType(caldavxml.calendar, componentType, justOwned=justOwned)
+        if not calendars:
+            return None
+        # Choose a random calendar
+        calendar = self.random.choice(calendars)
+        return calendar
+
+    def _getRandomEventOfType(self, componentType, justOwned=True):
+        """
+        Return a random L{Event} object from the current user
+        or C{None} if there are no events to work with
+        """
+        calendars = self._calendarsOfType(caldavxml.calendar, componentType, justOwned=justOwned)
+        while calendars:
+            calendar = self.random.choice(calendars)
+            calendars.remove(calendar)
+            if not calendar.events:
+                continue
+
+            events = calendar.events.keys()
+            while events:
+                href = self.random.choice(events)
+                events.remove(href)
+                event = calendar.events[href]
+                if not event.component:
+                    continue
+                if event.scheduleTag:
+                    continue
+
+                return event
+        return None
 
     def _isSelfAttendee(self, attendee):
         """
@@ -89,7 +133,6 @@ class ProfileBase(object):
         """
         return attendee.parameterValue('EMAIL') == self._client.email[len('mailto:'):]
 
-
     def _newOperation(self, label, deferred):
         """
         Helper to emit a log event when a new operation is started and
@@ -131,7 +174,6 @@ class ProfileBase(object):
         deferred.addBoth(finished)
         return deferred
 
-
     def _failedOperation(self, label, reason):
         """
         Helper to emit a log event when an operation fails.
@@ -148,7 +190,6 @@ class ProfileBase(object):
         self._sim._simFailure("%s: %s" % (label, reason,), self._reactor)
 
 
-
 class CannotAddAttendee(Exception):
     """
     Indicates no new attendees can be invited to a particular event.
@@ -156,14 +197,12 @@ class CannotAddAttendee(Exception):
     pass
 
 
-
 def loopWithDistribution(reactor, distribution, function):
     result = Deferred()
 
     def repeat(ignored):
         reactor.callLater(distribution.sample(), iterate)
 
-
     def iterate():
         d = function()
         if d is not None:
@@ -175,125 +214,7 @@ def loopWithDistribution(reactor, distribution, function):
     return result
 
 
-
 class Inviter(ProfileBase):
-    """
-    A Calendar user who invites and de-invites other users to events.
-    """
-    def setParameters(
-        self,
-        enabled=True,
-        sendInvitationDistribution=NormalDistribution(600, 60),
-        inviteeDistribution=UniformDiscreteDistribution(range(-10, 11))
-    ):
-        self.enabled = enabled
-        self._sendInvitationDistribution = sendInvitationDistribution
-        self._inviteeDistribution = inviteeDistribution
-
-
-    def run(self):
-        return loopWithDistribution(
-            self._reactor, self._sendInvitationDistribution, self._invite)
-
-
-    def _addAttendee(self, event, attendees):
-        """
-        Create a new attendee to add to the list of attendees for the
-        given event.
-        """
-        selfRecord = self._sim.getUserRecord(self._number)
-        invitees = set([u'mailto:%s' % (selfRecord.email,)])
-        for att in attendees:
-            invitees.add(att.value())
-
-        for _ignore_i in range(10):
-            invitee = max(
-                0, self._number + self._inviteeDistribution.sample())
-            try:
-                record = self._sim.getUserRecord(invitee)
-            except IndexError:
-                continue
-            cuaddr = u'mailto:%s' % (record.email,)
-            uuidx = u'urn:x-uid:%s' % (record.guid,)
-            uuid = u'urn:uuid:%s' % (record.guid,)
-            if cuaddr not in invitees and uuidx not in invitees and uuid not in invitees:
-                break
-        else:
-            return fail(CannotAddAttendee("Can't find uninvited user to invite."))
-
-        attendee = Property(
-            name=u'ATTENDEE',
-            value=cuaddr.encode("utf-8"),
-            params={
-                'CN': record.commonName,
-                'CUTYPE': 'INDIVIDUAL',
-                'PARTSTAT': 'NEEDS-ACTION',
-                'ROLE': 'REQ-PARTICIPANT',
-                'RSVP': 'TRUE',
-            },
-        )
-
-        return succeed(attendee)
-
-
-    def _invite(self):
-        """
-        Try to add a new attendee to an event, or perhaps remove an
-        existing attendee from an event.
-
-        @return: C{None} if there are no events to play with,
-            otherwise a L{Deferred} which fires when the attendee
-            change has been made.
-        """
-
-        if not self._client.started:
-            return succeed(None)
-
-        # Find calendars which are eligible for invites
-        calendars = self._calendarsOfType(caldavxml.calendar, "VEVENT")
-
-        while calendars:
-            # Pick one at random from which to try to select an event
-            # to modify.
-            calendar = self.random.choice(calendars)
-            calendars.remove(calendar)
-
-            if not calendar.events:
-                continue
-
-            events = calendar.events.keys()
-            while events:
-                uuid = self.random.choice(events)
-                events.remove(uuid)
-                event = calendar.events[uuid].component
-                if event is None:
-                    continue
-
-                component = event.mainComponent()
-                organizer = component.getOrganizerProperty()
-                if organizer is not None and not self._isSelfAttendee(organizer):
-                    # This event was organized by someone else, don't try to invite someone to it.
-                    continue
-
-                href = calendar.url + uuid
-
-                # Find out who might attend
-                attendees = tuple(component.properties('ATTENDEE'))
-
-                d = self._addAttendee(event, attendees)
-                d.addCallbacks(
-                    lambda attendee:
-                        self._client.addEventAttendee(
-                            href, attendee),
-                    lambda reason: reason.trap(CannotAddAttendee))
-                return self._newOperation("invite", d)
-
-        # Oops, either no events or no calendars to play with.
-        return succeed(None)
-
-
-
-class RealisticInviter(ProfileBase):
     """
     A Calendar user who invites other users to new events.
     """
@@ -315,7 +236,6 @@ END:VEVENT
 END:VCALENDAR
 """.replace("\n", "\r\n"))
 
-
     def setParameters(
         self,
         enabled=True,
@@ -330,6 +250,9 @@ END:VCALENDAR
             120 * 60
         ]),
         recurrenceDistribution=RecurrenceDistribution(False),
+        fileAttachPercentage=30,
+        fileSizeDistribution=NormalDistribution(1024, 1),
+        inviteeLookupPercentage=50,
     ):
         self.enabled = enabled
         self._sendInvitationDistribution = sendInvitationDistribution
@@ -339,13 +262,14 @@ END:VCALENDAR
         self._eventStartDistribution = eventStartDistribution
         self._eventDurationDistribution = eventDurationDistribution
         self._recurrenceDistribution = recurrenceDistribution
-
+        self._fileAttachPercentage = fileAttachPercentage
+        self._fileSizeDistribution = fileSizeDistribution
+        self._inviteeLookupPercentage = inviteeLookupPercentage
 
     def run(self):
         return loopWithDistribution(
             self._reactor, self._sendInvitationDistribution, self._invite)
 
-
     def _addAttendee(self, event, attendees):
         """
         Create a new attendee to add to the list of attendees for the
@@ -388,7 +312,6 @@ END:VCALENDAR
         event.addProperty(attendee)
         attendees.append(attendee)
 
-
     def _invite(self):
         """
         Try to add a new event, or perhaps remove an
@@ -403,7 +326,7 @@ END:VCALENDAR
             return succeed(None)
 
         # Find calendars which are eligible for invites
-        calendars = self._calendarsOfType(caldavxml.calendar, "VEVENT")
+        calendars = self._calendarsOfType(caldavxml.calendar, "VEVENT", justOwned=True)
 
         while calendars:
             # Pick one at random from which to try to create an event
@@ -436,20 +359,28 @@ END:VCALENDAR
                 try:
                     self._addAttendee(vevent, attendees)
                 except CannotAddAttendee:
-                    self._failedOperation("invite", "Cannot add attendee")
-                    return succeed(None)
+                    continue
+
+            choice = self.random.randint(1, 100)
+            if choice <= self._fileAttachPercentage:
+                attachmentSize = int(self._fileSizeDistribution.sample())
+            else:
+                attachmentSize = 0  # no attachment
 
             href = '%s%s.ics' % (calendar.url, uid)
-            d = self._client.addInvite(href, vcalendar)
+            d = self._client.addInvite(
+                href, vcalendar, attachmentSize=attachmentSize,
+                lookupPercentage=self._inviteeLookupPercentage
+            )
             return self._newOperation("invite", d)
 
 
-
 class Accepter(ProfileBase):
     """
     A Calendar user who accepts invitations to events. As well as accepting requests, this
     will also remove cancels and replies.
     """
+
     def setParameters(
         self,
         enabled=True,
@@ -459,13 +390,11 @@ class Accepter(ProfileBase):
         self._accepting = set()
         self._acceptDelayDistribution = acceptDelayDistribution
 
-
     def run(self):
         self._subscription = self._client.catalog["eventChanged"].subscribe(self.eventChanged)
         # TODO: Propagate errors from eventChanged and _acceptInvitation to this Deferred
         return Deferred()
 
-
     def eventChanged(self, href):
         # Just respond to normal calendar events
         calendar = href.rsplit('/', 1)[0] + '/'
@@ -482,7 +411,6 @@ class Accepter(ProfileBase):
         else:
             return
 
-
     def calendarEventChanged(self, calendar, href):
         if href in self._accepting:
             return
@@ -499,7 +427,6 @@ class Accepter(ProfileBase):
                     self._reactor.callLater(
                         delay, self._acceptInvitation, href, attendee)
 
-
     def inboxEventChanged(self, calendar, href):
         if href in self._accepting:
             return
@@ -519,7 +446,6 @@ class Accepter(ProfileBase):
             self._reactor.callLater(
                 delay, self._handleCancel, href)
 
-
     def _acceptInvitation(self, href, attendee):
         def change():
             accepted = self._makeAcceptedAttendee(attendee)
@@ -534,6 +460,7 @@ class Accepter(ProfileBase):
             # Download the event again and attempt to make the change
             # to the attendee list again.
             d = self._client.updateEvent(href)
+
             def cbUpdated(ignored):
                 d = change()
                 d.addErrback(scheduleError)
@@ -551,19 +478,18 @@ class Accepter(ProfileBase):
                         if uid == event.getUID():
                             return self._client.deleteEvent(event.url)
         d.addCallback(accepted)
+
         def finished(passthrough):
             self._accepting.remove(href)
             return passthrough
         d.addBoth(finished)
         return self._newOperation("accept", d)
 
-
     def _handleReply(self, href):
         d = self._client.deleteEvent(href)
         d.addBoth(self._finishRemoveAccepting, href)
         return self._newOperation("reply done", d)
 
-
     def _finishRemoveAccepting(self, passthrough, href):
         self._accepting.remove(href)
         if isinstance(passthrough, Failure):
@@ -571,7 +497,6 @@ class Accepter(ProfileBase):
             passthrough = passthrough.value.response
         return passthrough
 
-
     def _handleCancel(self, href):
 
         uid = self._client._events[href].getUID()
@@ -588,7 +513,6 @@ class Accepter(ProfileBase):
         d.addBoth(self._finishRemoveAccepting, href)
         return self._newOperation("cancelled", d)
 
-
     def _makeAcceptedAttendee(self, attendee):
         accepted = attendee.duplicate()
         accepted.setParameter('PARTSTAT', 'ACCEPTED')
@@ -596,6 +520,48 @@ class Accepter(ProfileBase):
         return accepted
 
 
+class AttachmentDownloader(ProfileBase):
+    """
+    A Calendar user who downloads attachments.
+    """
+
+    def setParameters(
+        self,
+        enabled=True,
+    ):
+        self.enabled = enabled
+
+    def run(self):
+        self._subscription = self._client.catalog["eventChanged"].subscribe(self.eventChanged)
+        return Deferred()
+
+    def eventChanged(self, href):
+        # Just respond to normal calendar events
+        calendar = href.rsplit('/', 1)[0] + '/'
+        try:
+            calendar = self._client._calendars[calendar]
+        except KeyError:
+            return
+
+        if calendar.resourceType == caldavxml.calendar:
+            self.calendarEventChanged(calendar, href)
+
+    def calendarEventChanged(self, calendar, href):
+        component = self._client._events[href].component
+        attachments = tuple(component.mainComponent().properties('ATTACH'))
+        if attachments:
+            for attachment in attachments:
+                attachmentHref = attachment.value()
+                managedId = attachment.parameterValue('MANAGED-ID')
+                attachmentHref = self._normalizeHref(attachmentHref)
+
+                self._reactor.callLater(
+                    0, self._client.getAttachment, attachmentHref, managedId
+                )
+
+    def _normalizeHref(self, href):
+        return urljoin(self._client._managed_attachments_server_url, urlparse(href).path)
+
 
 class Eventer(ProfileBase):
     """
@@ -630,54 +596,250 @@ END:VCALENDAR
             120 * 60
         ]),
         recurrenceDistribution=RecurrenceDistribution(False),
+        fileAttachPercentage=30,
+        fileSizeDistribution=NormalDistribution(1024, 1),
     ):
         self.enabled = enabled
         self._interval = interval
         self._eventStartDistribution = eventStartDistribution
         self._eventDurationDistribution = eventDurationDistribution
         self._recurrenceDistribution = recurrenceDistribution
-
+        self._fileAttachPercentage = fileAttachPercentage
+        self._fileSizeDistribution = fileSizeDistribution
 
     def run(self):
         self._call = LoopingCall(self._addEvent)
         self._call.clock = self._reactor
-        return self._call.start(self._interval)
-
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
 
     def _addEvent(self):
+        # Don't perform any operations until the client is up and running
         if not self._client.started:
             return succeed(None)
 
-        calendars = self._calendarsOfType(caldavxml.calendar, "VEVENT")
+        calendar = self._getRandomCalendarOfType('VEVENT')
 
-        while calendars:
-            calendar = self.random.choice(calendars)
-            calendars.remove(calendar)
+        if not calendar:
+            # No VEVENT calendars, so no new event...
+            return succeed(None)
 
-            # Copy the template event and fill in some of its fields
-            # to make a new event to create on the calendar.
-            vcalendar = self._eventTemplate.duplicate()
-            vevent = vcalendar.mainComponent()
-            uid = str(uuid4())
-            dtstart = self._eventStartDistribution.sample()
-            dtend = dtstart + Duration(seconds=self._eventDurationDistribution.sample())
-            vevent.replaceProperty(Property("CREATED", DateTime.getNowUTC()))
+        # Copy the template event and fill in some of its fields
+        # to make a new event to create on the calendar.
+        vcalendar = self._eventTemplate.duplicate()
+        vevent = vcalendar.mainComponent()
+        uid = str(uuid4())
+        dtstart = self._eventStartDistribution.sample()
+        dtend = dtstart + Duration(seconds=self._eventDurationDistribution.sample())
+        vevent.replaceProperty(Property("CREATED", DateTime.getNowUTC()))
+        vevent.replaceProperty(Property("DTSTAMP", DateTime.getNowUTC()))
+        vevent.replaceProperty(Property("DTSTART", dtstart))
+        vevent.replaceProperty(Property("DTEND", dtend))
+        vevent.replaceProperty(Property("UID", uid))
+
+        rrule = self._recurrenceDistribution.sample()
+        if rrule is not None:
+            vevent.addProperty(Property(None, None, None, pycalendar=rrule))
+
+        choice = self.random.randint(1, 100)
+        if choice <= self._fileAttachPercentage:
+            attachmentSize = int(self._fileSizeDistribution.sample())
+        else:
+            attachmentSize = 0  # no attachment
+
+        href = '%s%s.ics' % (calendar.url, uid)
+        d = self._client.addEvent(href, vcalendar, attachmentSize=attachmentSize)
+        return self._newOperation("create", d)
+
+
+class EventUpdaterBase(ProfileBase):
+
+    @inlineCallbacks
+    def action(self):
+        # Don't perform any operations until the client is up and running
+        if not self._client.started:
+            returnValue(None)
+
+        event = self._getRandomEventOfType('VEVENT', justOwned=True)
+        if not event:
+            returnValue(None)
+        component = event.component
+        vevent = component.mainComponent()
+
+        label = yield self.modifyEvent(event.url, vevent)
+        if label:
             vevent.replaceProperty(Property("DTSTAMP", DateTime.getNowUTC()))
-            vevent.replaceProperty(Property("DTSTART", dtstart))
-            vevent.replaceProperty(Property("DTEND", dtend))
-            vevent.replaceProperty(Property("UID", uid))
 
-            rrule = self._recurrenceDistribution.sample()
-            if rrule is not None:
-                vevent.addProperty(Property(None, None, None, pycalendar=rrule))
+            event.component = component
+            yield self._newOperation(
+                label,
+                self._client.changeEvent(event.url)
+            )
 
-            href = '%s%s.ics' % (calendar.url, uid)
-            d = self._client.addEvent(href, vcalendar)
-            return self._newOperation("create", d)
+    def run(self):
+        self._call = LoopingCall(self.action)
+        self._call.clock = self._reactor
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
+
+    def modifyEvent(self, href, vevent):
+        """Overridden by subclasses"""
+        pass
 
 
+class TitleChanger(EventUpdaterBase):
 
-class EventUpdater(ProfileBase):
+    def setParameters(
+        self,
+        enabled=True,
+        interval=60,
+        titleLengthDistribution=NormalDistribution(10, 2)
+    ):
+        self.enabled = enabled
+        self._interval = interval
+        self._titleLength = titleLengthDistribution
+
+    def modifyEvent(self, _ignore_href, vevent):
+        length = max(5, int(self._titleLength.sample()))
+        vevent.replaceProperty(Property("SUMMARY", "Event" + "." * (length - 5)))
+        return succeed("update")
+
+
+class DescriptionChanger(EventUpdaterBase):
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=60,
+        descriptionLengthDistribution=NormalDistribution(10, 2)
+    ):
+        self.enabled = enabled
+        self._interval = interval
+        self._descriptionLength = descriptionLengthDistribution
+
+    def modifyEvent(self, _ignore_href, vevent):
+        length = int(self._descriptionLength.sample())
+        vevent.replaceProperty(Property("DESCRIPTION", "." * length))
+        return succeed("update")
+
+
+class Attacher(EventUpdaterBase):
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=60,
+        fileSizeDistribution=NormalDistribution(1024, 1),
+    ):
+        self.enabled = enabled
+        self._interval = interval
+        self._fileSize = fileSizeDistribution
+
+    @inlineCallbacks
+    def modifyEvent(self, href, vevent):
+        fileSize = int(self._fileSize.sample())
+        yield self._client.postAttachment(href, 'x' * fileSize)
+        returnValue(None)
+
+
+class EventCountLimiter(EventUpdaterBase):
+    """
+    Examines the number of events in each calendar collection, and when that
+    count exceeds eventCountLimit, events are randomly removed until the count
+    falls back to the limit.
+    """
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=60,
+        eventCountLimit=1000
+    ):
+        self.enabled = enabled
+        self._interval = interval
+        self._limit = eventCountLimit
+
+    @inlineCallbacks
+    def action(self):
+        # Don't perform any operations until the client is up and running
+        if not self._client.started:
+            returnValue(None)
+
+        for calendar in self._calendarsOfType(caldavxml.calendar, "VEVENT", justOwned=True):
+            while len(calendar.events) > self._limit:
+                event = calendar.events[self.random.choice(calendar.events.keys())]
+                yield self._client.deleteEvent(event.url)
+
+
+class CalendarSharer(ProfileBase):
+    """
+    A Calendar user who shares calendars to other random users.
+    """
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=60,
+        maxSharees=3
+    ):
+        self.enabled = enabled
+        self._interval = interval
+        self._maxSharees = maxSharees
+
+    def run(self):
+        self._call = LoopingCall(self.action)
+        self._call.clock = self._reactor
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
+
+    @inlineCallbacks
+    def action(self):
+        # Don't perform any operations until the client is up and running
+        if not self._client.started:
+            returnValue(None)
+
+        yield self.shareCalendar()
+
+    @inlineCallbacks
+    def shareCalendar(self):
+
+        # pick a calendar
+        calendar = self._getRandomCalendarOfType('VEVENT', justOwned=True)
+        if not calendar:
+            returnValue(None)
+
+        # don't exceed maxSharees
+        if len(calendar.invitees) >= self._maxSharees:
+            returnValue(None)
+
+        # pick a random sharee
+        shareeRecord = self._sim.getRandomUserRecord(besides=self._number)
+        if shareeRecord is None:
+            returnValue(None)
+
+        # POST the sharing invite
+        mailto = "mailto:{}".format(shareeRecord.email)
+        body = Calendar.addInviteeXML(mailto, calendar.name, readwrite=True)
+        yield self._client.postXML(
+            calendar.url,
+            body,
+            label="POST{share-calendar}"
+        )
+
+
+class AlarmAcknowledger(ProfileBase):
     """
     A Calendar user who creates a new event, and then updates its alarm.
     """
@@ -709,7 +871,8 @@ END:VCALENDAR
     def setParameters(
         self,
         enabled=True,
-        interval=25,
+        interval=5,
+        pastTheHour=[0, 15, 30, 45],
         eventStartDistribution=NearFutureDistribution(),
         eventDurationDistribution=UniformDiscreteDistribution([
             15 * 60, 30 * 60,
@@ -720,10 +883,11 @@ END:VCALENDAR
     ):
         self.enabled = enabled
         self._interval = interval
+        self._pastTheHour = pastTheHour
         self._eventStartDistribution = eventStartDistribution
         self._eventDurationDistribution = eventDurationDistribution
         self._recurrenceDistribution = recurrenceDistribution
-
+        self._lastMinuteChecked = -1
 
     def initialize(self):
         """
@@ -731,25 +895,30 @@ END:VCALENDAR
 
         @return: a L{Deferred} that fires when initialization is done
         """
+        self.myEventHref = None
         return self._initEvent()
 
-
     def run(self):
         self._call = LoopingCall(self._updateEvent)
         self._call.clock = self._reactor
-        return self._call.start(self._interval)
-
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
 
     def _initEvent(self):
+        # Don't perform any operations until the client is up and running
         if not self._client.started:
             return succeed(None)
+        try:
+            calendar = self._calendarsOfType(caldavxml.calendar, "VEVENT")[0]
+        except IndexError:
+            # There is no calendar
+            return succeed(None)
 
-        # If it already exists, don't re-create
-        calendar = self._calendarsOfType(caldavxml.calendar, "VEVENT")[0]
-        if calendar.events:
-            events = [event for event in calendar.events.values() if event.url.endswith("event_to_update.ics")]
-            if events:
-                return succeed(None)
+        self.myEventHref = '{}{}.ics'.format(calendar.url, str(uuid4()))
 
         # Copy the template event and fill in some of its fields
         # to make a new event to create on the calendar.
@@ -763,37 +932,50 @@ END:VCALENDAR
         vevent.replaceProperty(Property("DTSTART", dtstart))
         vevent.replaceProperty(Property("DTEND", dtend))
         vevent.replaceProperty(Property("UID", uid))
+        vevent.replaceProperty(Property("DESCRIPTION", "AlarmAcknowledger"))
 
         rrule = self._recurrenceDistribution.sample()
         if rrule is not None:
             vevent.addProperty(Property(None, None, None, pycalendar=rrule))
 
-        href = '%s%s' % (calendar.url, "event_to_update.ics")
-        d = self._client.addEvent(href, vcalendar)
+        d = self._client.addEvent(self.myEventHref, vcalendar)
         return self._newOperation("create", d)
 
+    def _shouldUpdate(self, minutePastTheHour):
+        """
+        We want to only acknowledge our alarm at the "past the hour" minutes
+        we've been configured for.
+        """
+        should = False
+        if minutePastTheHour in self._pastTheHour:
+            # This is one of the minutes we should update on, but only update
+            # as we pass into this minute, and not subsequent times
+            if minutePastTheHour != self._lastMinuteChecked:
+                should = True
+
+        self._lastMinuteChecked = minutePastTheHour
+        return should
 
     def _updateEvent(self):
         """
-        Try to add a new attendee to an event, or perhaps remove an
-        existing attendee from an event.
+        Set the ACKNOWLEDGED property on an event.
 
         @return: C{None} if there are no events to play with,
-            otherwise a L{Deferred} which fires when the attendee
+            otherwise a L{Deferred} which fires when the acknowledged
             change has been made.
         """
 
+        # Only do updates when we reach of the designated minutes past the hour
+        if not self._shouldUpdate(datetime.now().minute):
+            return succeed(None)
+
         if not self._client.started:
             return succeed(None)
 
-        # If it does not exist, try to create it
-        calendar = self._calendarsOfType(caldavxml.calendar, "VEVENT")[0]
-        if not calendar.events:
+        if self.myEventHref is None:
             return self._initEvent()
-        events = [event for event in calendar.events.values() if event.url.endswith("event_to_update.ics")]
-        if not events:
-            return self._initEvent()
-        event = events[0]
+
+        event = self._client.eventByHref(self.myEventHref)
 
         # Add/update the ACKNOWLEDGED property
         component = event.component.mainComponent()
@@ -802,7 +984,6 @@ END:VCALENDAR
         return self._newOperation("update", d)
 
 
-
 class Tasker(ProfileBase):
     """
     A Calendar user who creates new tasks.
@@ -832,14 +1013,18 @@ END:VCALENDAR
         self._interval = interval
         self._taskStartDistribution = taskDueDistribution
 
-
     def run(self):
         self._call = LoopingCall(self._addTask)
         self._call.clock = self._reactor
-        return self._call.start(self._interval)
-
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
 
     def _addTask(self):
+        # Don't perform any operations until the client is up and running
         if not self._client.started:
             return succeed(None)
 
@@ -865,6 +1050,134 @@ END:VCALENDAR
             return self._newOperation("create", d)
 
 
+class TimeRanger(ProfileBase):
+    """
+    A profile which does time queries
+    """
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=25,
+    ):
+        self.enabled = enabled
+        self._interval = interval
+
+    def run(self):
+        self._call = LoopingCall(self._runQuery)
+        self._call.clock = self._reactor
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
+
+    def _runQuery(self):
+        # Don't perform any operations until the client is up and running
+        if not self._client.started:
+            return succeed(None)
+
+        now = datetime.now(dateutil.tz.tzutc())
+        start = now.strftime("%Y%m%dT%H%M%SZ")
+        end = (now + timedelta(seconds=24 * 60 * 60)).strftime("%Y%m%dT%H%M%SZ")
+
+        calendar = self._getRandomCalendarOfType('VEVENT')
+        return self._client.timeRangeQuery(calendar.url, start, end)
+
+
+class DeepRefresher(ProfileBase):
+    """
+    A profile which PROPFINDs all collection in a home
+    """
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=25,
+    ):
+        self.enabled = enabled
+        self._interval = interval
+
+    def run(self):
+        self._call = LoopingCall(self._deepRefresh)
+        self._call.clock = self._reactor
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
+
+    def _deepRefresh(self):
+        # Don't perform any operations until the client is up and running
+        if not self._client.started:
+            return succeed(None)
+
+        return self._client.deepRefresh()
+
+
+class APNSSubscriber(ProfileBase):
+    """
+    A profile which POSTs to /apns
+    """
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=60,
+    ):
+        self.enabled = enabled
+        self._interval = interval
+
+    def run(self):
+        self._call = LoopingCall(self._apnsSubscribe)
+        self._call.clock = self._reactor
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
+
+    def _apnsSubscribe(self):
+        # Don't perform any operations until the client is up and running
+        if not self._client.started:
+            return succeed(None)
+
+        return self._client.apnsSubscribe()
+
+
+class Resetter(ProfileBase):
+    """
+    A Calendar user who resets their account and re-downloads everything.
+    """
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=600,
+    ):
+        self.enabled = enabled
+        self._interval = interval
+
+    def run(self):
+        self._call = LoopingCall(self._resetAccount)
+        self._call.clock = self._reactor
+        self._reactor.callLater(
+            self.random.randint(1, self._interval),
+            self._call.start,
+            self._interval
+        )
+        return Deferred()
+
+    def _resetAccount(self):
+        # Don't perform any operations until the client is up and running
+        if not self._client.started:
+            return succeed(None)
+
+        return self._client.reset()
+
 
 class OperationLogger(SummarizingMixin):
     """
@@ -873,8 +1186,8 @@ class OperationLogger(SummarizingMixin):
     logger.
     """
     formats = {
-        u"start" : u"%(user)s - - - - - - - - - - - %(label)8s BEGIN %(lag)s",
-        u"end"   : u"%(user)s - - - - - - - - - - - %(label)8s END [%(duration)5.2f s]",
+        u"start": u"%(user)s - - - - - - - - - - - %(label)8s BEGIN %(lag)s",
+        u"end": u"%(user)s - - - - - - - - - - - %(label)8s END [%(duration)5.2f s]",
         u"failed": u"%(user)s x x x x x x x x x x x %(label)8s FAILED %(reason)s",
     }
 
@@ -893,8 +1206,9 @@ class OperationLogger(SummarizingMixin):
     _fail_cut_off = 1.0     # % of total count at which failed requests will cause a failure
 
     _fields_init = [
-        ('operation', -25, '%-25s'),
+        ('operation', -30, '%-30s'),
         ('count', 8, '%8s'),
+        ('%', 6, '%6.1f'),
         ('failed', 8, '%8s'),
     ]
 
@@ -915,7 +1229,8 @@ class OperationLogger(SummarizingMixin):
 
         # Load parameters from config
         if "thresholdsPath" in params:
-            jsondata = json.load(open(params["thresholdsPath"]))
+            with open(params["thresholdsPath"]) as f:
+                jsondata = json.load(f)
         elif "thresholds" in params:
             jsondata = params["thresholds"]
         else:
@@ -936,6 +1251,7 @@ class OperationLogger(SummarizingMixin):
         if "failCutoff" in params:
             self._fail_cut_off = params["failCutoff"]
 
+        self._fail_if_no_push = params.get("failIfNoPush", False)
 
     def observe(self, event):
         if event.get("type") == "operation":
@@ -956,13 +1272,11 @@ class OperationLogger(SummarizingMixin):
                 dataset = self._perOperationLags.setdefault(event[u'label'], [])
                 dataset.append(lag)
 
-
-    def _summarizeData(self, operation, data):
+    def _summarizeData(self, operation, data, total_count):
         avglag = mean(self._perOperationLags.get(operation, [0.0])) * 1000.0
-        data = SummarizingMixin._summarizeData(self, operation, data)
+        data = SummarizingMixin._summarizeData(self, operation, data, total_count)
         return data[:-1] + (avglag,) + data[-1:]
 
-
     def report(self, output):
         output.write("\n")
         self.printHeader(output, [
@@ -977,6 +1291,9 @@ class OperationLogger(SummarizingMixin):
 
     _LATENCY_REASON = "Median %(operation)s scheduling lag greater than %(cutoff)sms"
     _FAILED_REASON = "Greater than %(cutoff).0f%% %(operation)s failed"
+    _PUSH_MISSING_REASON = "Push was configured but no pushes were received by clients"
+    _REASON_1 = "Greater than %(cutoff)g%% %(method)s exceeded "
+    _REASON_2 = "%g second total time"
 
     def failures(self):
         reasons = []
@@ -992,4 +1309,25 @@ class OperationLogger(SummarizingMixin):
                 reasons.append(self._FAILED_REASON % dict(
                     operation=operation.upper(), cutoff=self._fail_cut_off))
 
+        for operation, times in self._perOperationTimes.iteritems():
+            overDurations = [0] * len(self._thresholds)
+            for success, duration in times:
+                for ctr, item in enumerate(self._thresholds):
+                    threshold, _ignore_fail_at = item
+                    if duration > threshold:
+                        overDurations[ctr] += 1
+            checks = []
+            for ctr, item in enumerate(self._thresholds):
+                threshold, fail_at = item
+                fail_at = fail_at.get(operation, fail_at["default"])
+                checks.append(
+                    (overDurations[ctr], fail_at, self._REASON_1 + self._REASON_2 % (threshold,))
+                )
+            for count, cutoff, reason in checks:
+                if count * 100.0 / len(times) > cutoff:
+                    reasons.append(reason % dict(method=operation.upper(), cutoff=cutoff))
+
+        if self._fail_if_no_push and "push" not in self._perOperationTimes:
+            reasons.append(self._PUSH_MISSING_REASON)
+
         return reasons
