@@ -1,6 +1,6 @@
 # -*- test-case-name: contrib.performance.loadtest.test_sim -*-
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,17 @@
 ##
 from __future__ import print_function
 
-from collections import namedtuple
+import sys
+
+if "twisted.internet.reactor" not in sys.modules:
+    try:
+        from twisted.internet import kqreactor
+        kqreactor.install()
+    except ImportError:
+        from twisted.internet import epollreactor
+        epollreactor.install()
+
+from collections import namedtuple, defaultdict
 from os import environ, mkdir
 from os.path import isdir
 from plistlib import readPlist
@@ -25,6 +35,7 @@ from random import Random
 from sys import argv, stdout
 from urlparse import urlsplit
 from xml.parsers.expat import ExpatError
+import itertools
 import json
 import shutil
 import socket
@@ -52,27 +63,31 @@ from contrib.performance.loadtest.population import (
     CalendarClientSimulator)
 from contrib.performance.loadtest.webadmin import LoadSimAdminResource
 
+from contrib.performance.loadtest.amphub import AMPHub
 
 
 class _DirectoryRecord(object):
-    def __init__(self, uid, password, commonName, email, guid):
+
+    def __init__(self, uid, password, commonName, email, guid, podID="PodA"):
         self.uid = uid
         self.password = password
         self.commonName = commonName
         self.email = email
         self.guid = guid
+        self.podID = podID
 
+    def __repr__(self):
+        return "user='{}'".format(self.uid)
 
 
 def safeDivision(value, total, factor=1):
     return value * factor / total if total else 0
 
 
-
 def generateRecords(
     count, uidPattern="user%d", passwordPattern="user%d",
     namePattern="User %d", emailPattern="user%d@example.com",
-    guidPattern="user%d"
+    guidPattern="user%d", podID="PodA",
 ):
     for i in xrange(count):
         i += 1
@@ -81,20 +96,31 @@ def generateRecords(
         name = namePattern % (i,)
         email = emailPattern % (i,)
         guid = guidPattern % (i,)
-        yield _DirectoryRecord(uid, password, name, email, guid)
+        yield _DirectoryRecord(uid, password, name, email, guid, podID)
 
 
-
-def recordsFromCSVFile(path):
+def recordsFromCSVFile(path, interleavePods):
     if path:
         pathObj = FilePath(path)
     else:
         pathObj = FilePath(__file__).sibling("accounts.csv")
-    return [
+    records = [
         _DirectoryRecord(*line.decode('utf-8').split(u','))
         for line
-        in pathObj.getContent().splitlines()]
+        in pathObj.getContent().splitlines()
+    ]
 
+    if interleavePods:
+        # For Pods we re-order the record list so that we alternate between the pods
+        recordsByPod = defaultdict(list)
+        for record in records:
+            recordsByPod[record.podID].append(record)
+
+        records = []
+        for items in itertools.izip(*[recordsByPod[k] for k in sorted(recordsByPod.keys())]):
+            records.extend(items)
+
+    return records
 
 
 class LagTrackingReactor(object):
@@ -107,23 +133,22 @@ class LagTrackingReactor(object):
     the call was original scheduled and when the call actually took
     place.
     """
+
     def __init__(self, reactor):
         self._reactor = reactor
 
-
     def __getattr__(self, name):
         return getattr(self._reactor, name)
 
-
     def callLater(self, delay, function, *args, **kwargs):
         expected = self._reactor.seconds() + delay
+
         def modifyContext():
             now = self._reactor.seconds()
             context.call({'lag': now - expected}, function, *args, **kwargs)
         return self._reactor.callLater(delay, modifyContext)
 
 
-
 class SimOptions(Options):
     """
     Command line configuration options for the load simulator.
@@ -144,7 +169,6 @@ class SimOptions(Options):
          FilePath),
     ]
 
-
     def opt_logfile(self, filename):
         """
         Enable normal logging to some file.  - for stdout.
@@ -155,7 +179,6 @@ class SimOptions(Options):
             fObj = file(filename, "a")
         startLogging(fObj, setStdout=False)
 
-
     def opt_debug(self):
         """
         Enable Deferred and Failure debugging.
@@ -163,7 +186,6 @@ class SimOptions(Options):
         self.opt_debug_deferred()
         self.opt_debug_failure()
 
-
     def opt_debug_deferred(self):
         """
         Enable Deferred debugging.
@@ -171,7 +193,6 @@ class SimOptions(Options):
         from twisted.internet.defer import setDebugging
         setDebugging(True)
 
-
     def opt_debug_failure(self):
         """
         Enable Failure debugging.
@@ -179,7 +200,6 @@ class SimOptions(Options):
         from twisted.python.failure import startDebugMode
         startDebugMode()
 
-
     def postOptions(self):
         try:
             configFile = self['config'].open()
@@ -214,7 +234,6 @@ class SimOptions(Options):
 Arrival = namedtuple('Arrival', 'factory parameters')
 
 
-
 class LoadSimulator(object):
     """
     A L{LoadSimulator} simulates some configuration of calendar
@@ -228,15 +247,15 @@ class LoadSimulator(object):
         user information about the accounts on the server being put
         under load.
     """
-    def __init__(self, server, principalPathTemplate, webadminPort, serverStats, serializationPath, arrival, parameters, observers=None,
+
+    def __init__(self, servers, principalPathTemplate, webadminPort, serializationPath, arrival, parameters, observers=None,
                  records=None, reactor=None, runtime=None, workers=None,
                  configTemplate=None, workerID=None, workerCount=1):
         if reactor is None:
             from twisted.internet import reactor
-        self.server = server
+        self.servers = servers
         self.principalPathTemplate = principalPathTemplate
         self.webadminPort = webadminPort
-        self.serverStats = serverStats
         self.serializationPath = serializationPath
         self.arrival = arrival
         self.parameters = parameters
@@ -250,7 +269,6 @@ class LoadSimulator(object):
         self.workerID = workerID
         self.workerCount = workerCount
 
-
     @classmethod
     def fromCommandLine(cls, args=None, output=stdout):
         if args is None:
@@ -264,7 +282,6 @@ class LoadSimulator(object):
 
         return cls.fromConfig(options.config, options['runtime'], output)
 
-
     @classmethod
     def fromConfig(cls, config, runtime=None, output=stdout, reactor=None):
         """
@@ -272,20 +289,22 @@ class LoadSimulator(object):
         property list.
         """
 
+        servers = config.get('servers')
+
         workers = config.get("workers")
         if workers is None:
             # Client / place where the simulator actually runs configuration
             workerID = config.get("workerID", 0)
             workerCount = config.get("workerCount", 1)
             configTemplate = None
-            server = config.get('server', 'http://127.0.0.1:8008')
             principalPathTemplate = config.get('principalPathTemplate', '/principals/users/%s/')
             serializationPath = None
 
             if 'clientDataSerialization' in config:
                 serializationPath = config['clientDataSerialization']['Path']
                 if not config['clientDataSerialization']['UseOldData']:
-                    shutil.rmtree(serializationPath)
+                    if isdir(serializationPath):
+                        shutil.rmtree(serializationPath)
                 serializationPath = config['clientDataSerialization']['Path']
                 if not isdir(serializationPath):
                     try:
@@ -323,7 +342,6 @@ class LoadSimulator(object):
                                                 [Eventer, Inviter, Accepter]))
         else:
             # Manager / observer process.
-            server = ''
             principalPathTemplate = ''
             serializationPath = None
             arrival = None
@@ -337,12 +355,6 @@ class LoadSimulator(object):
             if config['webadmin']['enabled']:
                 webadminPort = config['webadmin']['HTTPPort']
 
-        serverStats = None
-        if 'serverStats' in config:
-            if config['serverStats']['enabled']:
-                serverStats = config['serverStats']
-                serverStats['server'] = config['server'] if 'server' in config else ''
-
         observers = []
         if 'observers' in config:
             for observer in config['observers']:
@@ -355,13 +367,13 @@ class LoadSimulator(object):
             loader = config['accounts']['loader']
             params = config['accounts']['params']
             records.extend(namedAny(loader)(**params))
+            records = cls.filterRecords(records, servers)
             output.write("Loaded {0} accounts.\n".format(len(records)))
 
         return cls(
-            server,
+            servers,
             principalPathTemplate,
             webadminPort,
-            serverStats,
             serializationPath,
             arrival,
             parameters,
@@ -375,7 +387,6 @@ class LoadSimulator(object):
             workerCount=workerCount,
         )
 
-
     @classmethod
     def _convertParams(cls, params):
         """
@@ -391,7 +402,6 @@ class LoadSimulator(object):
                 params[k] = cls._convertDistribution(v)
         return params
 
-
     @classmethod
     def _convertDistribution(cls, value):
         """
@@ -400,32 +410,46 @@ class LoadSimulator(object):
         """
         return namedAny(value['type'])(**value['params'])
 
+    @classmethod
+    def filterRecords(cls, records, servers):
+        """
+        Remove records that do not correspond to an enabled pod.
+
+        @param records: list of records to process
+        @type records: L{list}
+        @param servers: dictionary of servers
+        @type servers: L{dict}
+        """
+
+        # Get all enabled pod ids
+        enabled = set(filter(lambda podID: servers[podID]["enabled"], servers.keys()))
+        return filter(lambda record: record.podID in enabled, records)
 
     @classmethod
     def main(cls, args=None):
         simulator = cls.fromCommandLine(args)
-        raise SystemExit(simulator.run())
-
+        exitCode = simulator.run()
+        print("Exit code: {}".format(exitCode))
+        raise SystemExit(exitCode)
 
     def createSimulator(self):
         populator = Populator(Random())
         return CalendarClientSimulator(
             self.records,
             populator,
+            Random(),
             self.parameters,
             self.reactor,
-            self.server,
+            self.servers,
             self.principalPathTemplate,
             self.serializationPath,
             self.workerID,
             self.workerCount,
         )
 
-
     def createArrivalPolicy(self):
         return self.arrival.factory(self.reactor, **self.arrival.parameters)
 
-
     def serviceClasses(self):
         """
         Return a list of L{SimService} subclasses for C{attachServices} to
@@ -443,13 +467,21 @@ class LoadSimulator(object):
             ReporterService,
         ]
 
-
     def attachServices(self, output):
         self.ms = MultiService()
         for svcclass in self.serviceClasses():
             svcclass(self, output).setServiceParent(self.ms)
         attachService(self.reactor, self, self.ms)
+        self.startAmpHub()
 
+    def startAmpHub(self):
+        hostsAndPorts = []
+        for serverInfo in self.servers.itervalues():
+            if serverInfo["enabled"]:
+                for host in serverInfo.get("ampPushHosts", []):
+                    hostsAndPorts.append((host, serverInfo["ampPushPort"]))
+        if hostsAndPorts:
+            AMPHub.start(hostsAndPorts)
 
     def run(self, output=stdout):
         self.attachServices(output)
@@ -459,6 +491,8 @@ class LoadSimulator(object):
             self.reactor.listenTCP(self.webadminPort, Site(LoadSimAdminResource(self)))
         self.reactor.run()
 
+        # Return code to indicate pass or fail
+        return 0 if all([len(obs.failures()) == 0 for obs in self.observers]) else 1
 
     def stop(self):
         if self.ms.running:
@@ -466,31 +500,29 @@ class LoadSimulator(object):
             self.ms.stopService()
             self.reactor.callLater(5, self.stopAndReport)
 
-
     def shutdown(self):
         if self.ms.running:
             self.updateStats()
             return self.ms.stopService()
 
-
     def updateStats(self):
         """
         Capture server stats and stop.
         """
 
-        if self.serverStats is not None:
-            _ignore_scheme, hostname, _ignore_path, _ignore_query, _ignore_fragment = urlsplit(self.serverStats["server"])
-            data = self.readStatsSock((hostname.split(":")[0], self.serverStats["Port"],), True)
-            if "Failed" not in data:
-                data = data["stats"]["5m"] if "stats" in data else data["5 Minutes"]
-                result = (
-                    safeDivision(float(data["requests"]), 5 * 60),
-                    safeDivision(data["t"], data["requests"]),
-                    safeDivision(float(data["slots"]), data["requests"]),
-                    safeDivision(data["cpu"], data["requests"]),
-                )
-                msg(type="sim-expired", reason=result)
-
+        for podID, server in self.servers.items():
+            if server["enabled"] and server["stats"]["enabled"]:
+                _ignore_scheme, hostname, _ignore_path, _ignore_query, _ignore_fragment = urlsplit(server["uri"])
+                data = self.readStatsSock((hostname.split(":")[0], server["stats"]["Port"],), True)
+                if "Failed" not in data:
+                    data = data["stats"]["5m"] if "stats" in data else data["5 Minutes"]
+                    result = (
+                        safeDivision(float(data["requests"]), 5 * 60),
+                        safeDivision(data["t"], data["requests"]),
+                        safeDivision(float(data["slots"]), data["requests"]),
+                        safeDivision(data["cpu"], data["requests"]),
+                    )
+                    msg(type="sim-expired", podID=podID, reason=result)
 
     def stopAndReport(self):
         """
@@ -500,7 +532,6 @@ class LoadSimulator(object):
         self.updateStats()
         self.reactor.stop()
 
-
     def readStatsSock(self, sockname, useTCP):
         try:
             s = socket.socket(socket.AF_INET if useTCP else socket.AF_UNIX, socket.SOCK_STREAM)
@@ -517,11 +548,12 @@ class LoadSimulator(object):
             data = json.loads(data)
         except socket.error:
             data = {"Failed": "Unable to read statistics from server: %s" % (sockname,)}
+        except ValueError:
+            data = {"Failed": "Unable to parse statistics from server: %s" % (sockname,)}
         data["Server"] = sockname
         return data
 
 
-
 def attachService(reactor, loadsim, service):
     """
     Attach a given L{IService} provider to the given L{IReactorCore}; cause it
@@ -531,7 +563,6 @@ def attachService(reactor, loadsim, service):
     reactor.addSystemEventTrigger('before', 'shutdown', loadsim.shutdown)
 
 
-
 class SimService(Service, object):
     """
     Base class for services associated with the L{LoadSimulator}.
@@ -543,7 +574,6 @@ class SimService(Service, object):
         self.output = output
 
 
-
 class ObserverService(SimService):
     """
     A service that adds and removes a L{LoadSimulator}'s set of observers at
@@ -558,14 +588,12 @@ class ObserverService(SimService):
         for obs in self.loadsim.observers:
             addObserver(obs.observe)
 
-
     def stopService(self):
         super(ObserverService, self).stopService()
         for obs in self.loadsim.observers:
             removeObserver(obs.observe)
 
 
-
 class SimulatorService(SimService):
     """
     A service that starts the L{CalendarClientSimulator} associated with the
@@ -578,14 +606,12 @@ class SimulatorService(SimService):
         arrivalPolicy = self.loadsim.createArrivalPolicy()
         arrivalPolicy.run(self.clientsim)
 
-
     @inlineCallbacks
     def stopService(self):
         yield super(SimulatorService, self).stopService()
         yield self.clientsim.stop()
 
 
-
 class ReporterService(SimService):
     """
     A service which reports all the results from all the observers on a load
@@ -599,7 +625,6 @@ class ReporterService(SimService):
         super(ReporterService, self).startService()
         self.loadsim.reporter = self
 
-
     def stopService(self):
         """
         Emit the report to the specified output file.
@@ -607,7 +632,6 @@ class ReporterService(SimService):
         super(ReporterService, self).stopService()
         self.generateReport(self.output)
 
-
     def generateReport(self, output):
         """
         Emit the report to the specified output file.
@@ -624,7 +648,6 @@ class ReporterService(SimService):
             output.write('\n*** PASS\n')
 
 
-
 class ProcessProtocolBridge(ProcessProtocol):
 
     def __init__(self, spawner, proto):
@@ -632,37 +655,30 @@ class ProcessProtocolBridge(ProcessProtocol):
         self.proto = proto
         self.deferred = Deferred()
 
-
     def connectionMade(self):
         self.transport.getPeer = self.getPeer
         self.transport.getHost = self.getHost
         self.proto.makeConnection(self.transport)
 
-
     def getPeer(self):
         return "Peer:PID:" + str(self.transport.pid)
 
-
     def getHost(self):
         return "Host:PID:" + str(self.transport.pid)
 
-
     def outReceived(self, data):
         self.proto.dataReceived(data)
 
-
     def errReceived(self, error):
         msg("stderr received from " + str(self.transport.pid))
         msg("    " + repr(error))
 
-
     def processEnded(self, reason):
         self.proto.connectionLost(reason)
         self.deferred.callback(None)
         self.spawner.bridges.remove(self)
 
 
-
 class WorkerSpawnerService(SimService):
 
     def startService(self):
@@ -680,9 +696,9 @@ class WorkerSpawnerService(SimService):
                 bridge, sh, [sh, "-c", worker], env=environ
             )
 
-
     def stopService(self):
         TERMINATE_TIMEOUT = 30.0
+
         def killThemAll(name):
             for bridge in self.bridges:
                 bridge.transport.signalProcess(name)
@@ -691,7 +707,6 @@ class WorkerSpawnerService(SimService):
         return gatherResults([bridge.deferred for bridge in self.bridges])
 
 
-
 main = LoadSimulator.main
 
 if __name__ == '__main__':
