@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,18 +21,18 @@ from caldavclientlibrary.protocol.url import URL
 from caldavclientlibrary.protocol.webdav.definitions import davxml
 
 from contrib.performance.httpclient import MemoryConsumer, StringProducer
-from contrib.performance.loadtest.ical import XMPPPush, Event, Calendar, OS_X_10_6
+from contrib.performance.loadtest.ical import XMPPPush, Event, Calendar, OS_X_10_11, NotificationCollection
 from contrib.performance.loadtest.sim import _DirectoryRecord
 
 from pycalendar.datetime import DateTime
 from pycalendar.timezone import Timezone
 
-from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
+from twisted.internet.defer import Deferred, inlineCallbacks, returnValue, succeed
 from twisted.internet.protocol import ProtocolToConsumerAdapter
 from twisted.python.failure import Failure
 from twisted.trial.unittest import TestCase
 from twisted.web.client import ResponseDone
-from twisted.web.http import OK, NO_CONTENT, CREATED, MULTI_STATUS
+from twisted.web.http import OK, NO_CONTENT, CREATED, MULTI_STATUS, NOT_FOUND, FORBIDDEN
 from twisted.web.http_headers import Headers
 
 from twistedcaldav.ical import Component
@@ -404,11 +404,11 @@ END:VCALENDAR
 """.replace("\n", "\r\n") % {'UID': EVENT_UID}
 
 
-
 class EventTests(TestCase):
     """
     Tests for L{Event}.
     """
+
     def test_uid(self):
         """
         When the C{vevent} attribute of an L{Event} instance is set,
@@ -417,7 +417,6 @@ class EventTests(TestCase):
         event = Event(None, u'/foo/bar', u'etag', Component.fromString(EVENT))
         self.assertEquals(event.getUID(), EVENT_UID)
 
-
     def test_withoutUID(self):
         """
         When an L{Event} has a C{vevent} attribute set to C{None},
@@ -427,7 +426,6 @@ class EventTests(TestCase):
         self.assertIdentical(event.getUID(), None)
 
 
-
 PRINCIPAL_PROPFIND_RESPONSE = """\
 <?xml version='1.0' encoding='UTF-8'?>
 <multistatus xmlns='DAV:'>
@@ -581,6 +579,7 @@ _CALENDAR_HOME_PROPFIND_RESPONSE_TEMPLATE = """\
     <href>/calendars/__uids__/user01/notification/</href>
     <propstat>
       <prop>
+        <sync-token xmlns='DAV:'>SYNCTOKEN3</sync-token>
         <displayname>notification</displayname>
         <resourcetype>
           <collection/>
@@ -742,6 +741,8 @@ _CALENDAR_HOME_PROPFIND_RESPONSE_TEMPLATE = """\
     <propstat>
       <prop>
         <getctag xmlns='http://calendarserver.org/ns/'>c2696540-4c4c-4a31-adaf-c99630776828#3</getctag>
+        <sync-token xmlns='DAV:'>SYNCTOKEN1</sync-token>
+
         <displayname>calendar</displayname>
         <calendar-color xmlns='http://apple.com/ns/ical/'>#0252D4FF</calendar-color>
         <calendar-order xmlns='http://apple.com/ns/ical/'>1</calendar-order>
@@ -1027,6 +1028,7 @@ END:VCALENDAR
     <propstat>
       <prop>
         <getctag xmlns='http://calendarserver.org/ns/'>a483dab3-1391-445b-b1c3-5ae9dfc81c2f#0</getctag>
+        <sync-token xmlns='DAV:'>SYNCTOKEN2</sync-token>
         <displayname>inbox</displayname>
         <supported-calendar-component-set xmlns='urn:ietf:params:xml:ns:caldav'>
           <comp name='VEVENT'/>
@@ -1135,8 +1137,8 @@ CALENDAR_HOME_PROPFIND_RESPONSE_WITH_XMPP = _CALENDAR_HOME_PROPFIND_RESPONSE_TEM
 CALENDAR_HOME_PROPFIND_RESPONSE_XMPP_MISSING = _CALENDAR_HOME_PROPFIND_RESPONSE_TEMPLATE % {"xmpp": ""}
 
 
-
 class MemoryResponse(object):
+
     def __init__(self, version, code, phrase, headers, bodyProducer):
         self.version = version
         self.code = code
@@ -1145,18 +1147,17 @@ class MemoryResponse(object):
         self.length = bodyProducer.length
         self._bodyProducer = bodyProducer
 
-
     def deliverBody(self, protocol):
         protocol.makeConnection(self._bodyProducer)
         d = self._bodyProducer.startProducing(ProtocolToConsumerAdapter(protocol))
         d.addCallback(lambda ignored: protocol.connectionLost(Failure(ResponseDone())))
 
 
+class OS_X_10_11Mixin:
+    """
+    Mixin for L{TestCase}s for L{OS_X_10_11}.
+    """
 
-class OS_X_10_6Mixin:
-    """
-    Mixin for L{TestCase}s for L{OS_X_10_6}.
-    """
     def setUp(self):
         TimezoneCache.create()
         self.record = _DirectoryRecord(
@@ -1164,18 +1165,21 @@ class OS_X_10_6Mixin:
         )
         serializePath = self.mktemp()
         os.mkdir(serializePath)
-        self.client = OS_X_10_6(
+        self.client = OS_X_10_11(
             None,
-            "http://127.0.0.1",
+            {
+                "uri": "http://127.0.0.1",
+            },
             "/principals/users/%s/",
             serializePath,
             self.record,
             None,
+            1
         )
 
-
     def interceptRequests(self):
         requests = []
+
         def request(*args, **kwargs):
             result = Deferred()
             requests.append((result, args))
@@ -1184,11 +1188,11 @@ class OS_X_10_6Mixin:
         return requests
 
 
+class OS_X_10_11Tests(OS_X_10_11Mixin, TestCase):
+    """
+    Tests for L{OS_X_10_11}.
+    """
 
-class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
-    """
-    Tests for L{OS_X_10_6}.
-    """
     def test_parsePrincipalPROPFINDResponse(self):
         """
         L{Principal._parsePROPFINDResponse} accepts an XML document
@@ -1227,15 +1231,14 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
 #                     '{DAV:}expand-property',
 #                     )})
 
-
     def test_extractCalendars(self):
         """
-        L{OS_X_10_6._extractCalendars} accepts a calendar home
+        L{OS_X_10_11._extractCalendars} accepts a calendar home
         PROPFIND response body and returns a list of calendar objects
         constructed from the data extracted from the response.
         """
         home = "/calendars/__uids__/user01/"
-        calendars = self.client._extractCalendars(
+        calendars, notificationCollection, _ignore_homeToken = self.client._extractCalendars(
             self.client._parseMultiStatus(CALENDAR_HOME_PROPFIND_RESPONSE), home)
         calendars.sort(key=lambda cal: cal.resourceType)
         calendar, inbox = calendars
@@ -1243,20 +1246,21 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
         self.assertEquals(calendar.resourceType, caldavxml.calendar)
         self.assertEquals(calendar.name, "calendar")
         self.assertEquals(calendar.url, "/calendars/__uids__/user01/calendar/")
-        self.assertEquals(calendar.changeToken, "c2696540-4c4c-4a31-adaf-c99630776828#3")
+        self.assertEquals(calendar.changeToken, "SYNCTOKEN1")
 
         self.assertEquals(inbox.resourceType, caldavxml.schedule_inbox)
         self.assertEquals(inbox.name, "inbox")
         self.assertEquals(inbox.url, "/calendars/__uids__/user01/inbox/")
-        self.assertEquals(inbox.changeToken, "a483dab3-1391-445b-b1c3-5ae9dfc81c2f#0")
+        self.assertEquals(inbox.changeToken, "SYNCTOKEN2")
+
+        self.assertEquals(notificationCollection.changeToken, "SYNCTOKEN3")
 
         self.assertEqual({}, self.client.xmpp)
 
-
     def test_extractCalendarsXMPP(self):
         """
         If there is XMPP push information in a calendar home PROPFIND response,
-        L{OS_X_10_6._extractCalendars} finds it and records it.
+        L{OS_X_10_11._extractCalendars} finds it and records it.
         """
         home = "/calendars/__uids__/user01/"
         self.client._extractCalendars(
@@ -1272,18 +1276,16 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
             self.client.xmpp
         )
 
-
     def test_handleMissingXMPP(self):
         home = "/calendars/__uids__/user01/"
         self.client._extractCalendars(
             self.client._parseMultiStatus(CALENDAR_HOME_PROPFIND_RESPONSE_XMPP_MISSING), home)
         self.assertEqual({}, self.client.xmpp)
 
-
     @inlineCallbacks
     def test_changeEventAttendee(self):
         """
-        OS_X_10_6.changeEventAttendee removes one attendee from an
+        OS_X_10_11.changeEventAttendee removes one attendee from an
         existing event and appends another.
         """
         requests = self.interceptRequests()
@@ -1314,10 +1316,9 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
         self.assertEquals(attendees[0].parameterValue('CN'), 'User 01')
         self.assertEquals(attendees[1].parameterValue('CN'), 'Some Other Guy')
 
-
     def test_addEvent(self):
         """
-        L{OS_X_10_6.addEvent} PUTs the event passed to it to the
+        L{OS_X_10_11.addEvent} PUTs the event passed to it to the
         server and updates local state to reflect its existence.
         """
         requests = self.interceptRequests()
@@ -1340,6 +1341,7 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
 
         consumer = MemoryConsumer()
         finished = body.startProducing(consumer)
+
         def cbFinished(ignored):
             self.assertEqual(
                 Component.fromString(consumer.value()),
@@ -1348,18 +1350,17 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
 
         def requested(ignored):
             response = MemoryResponse(
-                ('HTTP', '1', '1'), CREATED, "Created", Headers({}),
+                ('HTTP', '1', '1'), CREATED, "Created", Headers({"etag": ["foo"]}),
                 StringProducer(""))
-            result.callback(response)
+            result.callback((response, ""))
         finished.addCallback(requested)
 
         return d
 
-
     @inlineCallbacks
     def test_addInvite(self):
         """
-        L{OS_X_10_6.addInvite} PUTs the event passed to it to the
+        L{OS_X_10_11.addInvite} PUTs the event passed to it to the
         server and updates local state to reflect its existence, but
         it also does attendee auto-complete and free-busy checks before
         the PUT.
@@ -1391,7 +1392,7 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
                 ('HTTP', '1', '1'), MULTI_STATUS, "MultiStatus", Headers({}),
                 StringProducer("<?xml version='1.0' encoding='UTF-8'?><multistatus xmlns='DAV:' />"))
 
-            returnValue(response)
+            returnValue((response, "<?xml version='1.0' encoding='UTF-8'?><multistatus xmlns='DAV:' />"))
 
         @inlineCallbacks
         def _testPost(*args, **kwargs):
@@ -1410,7 +1411,7 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
                 ('HTTP', '1', '1'), OK, "OK", Headers({}),
                 StringProducer(""))
 
-            returnValue(response)
+            returnValue((response, ""))
 
         def _testPost02(*args, **kwargs):
             return _testPost(*args, attendee="ATTENDEE:mailto:user02@example.com", **kwargs)
@@ -1437,9 +1438,22 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
                 ('HTTP', '1', '1'), CREATED, "Created", Headers({}),
                 StringProducer(""))
 
-            returnValue(response)
+            returnValue((response, ""))
 
-        requests = [_testReport, _testPost02, _testReport, _testPost03, _testPut, ]
+        def _testGet(*args, **kwargs):
+            expectedResponseCode, method, url = args
+            self.assertEqual(expectedResponseCode, OK)
+            self.assertEqual(method, 'GET')
+            self.assertEqual(url, 'http://127.0.0.1/mumble/frotz.ics')
+            self.assertIsInstance(url, str)
+
+            response = MemoryResponse(
+                ('HTTP', '1', '1'), OK, "OK", Headers({"etag": ["foo"]}),
+                StringProducer(EVENT_INVITE))
+
+            return succeed((response, EVENT_INVITE))
+
+        requests = [_testReport, _testPost02, _testReport, _testPost03, _testPut, _testGet]
 
         def _requestHandler(*args, **kwargs):
             handler = requests.pop(0)
@@ -1447,10 +1461,9 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
         self.client._request = _requestHandler
         yield self.client.addInvite('/mumble/frotz.ics', vcalendar)
 
-
     def test_deleteEvent(self):
         """
-        L{OS_X_10_6.deleteEvent} DELETEs the event at the relative
+        L{OS_X_10_11.deleteEvent} DELETEs the event at the relative
         URL passed to it and updates local state to reflect its
         removal.
         """
@@ -1467,7 +1480,7 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
 
         expectedResponseCode, method, url = req
 
-        self.assertEqual(expectedResponseCode, NO_CONTENT)
+        self.assertEqual(expectedResponseCode, (NO_CONTENT, NOT_FOUND))
         self.assertEqual(method, 'DELETE')
         self.assertEqual(url, 'http://127.0.0.1' + event.url)
         self.assertIsInstance(url, str)
@@ -1478,15 +1491,14 @@ class OS_X_10_6Tests(OS_X_10_6Mixin, TestCase):
         response = MemoryResponse(
             ('HTTP', '1', '1'), NO_CONTENT, "No Content", None,
             StringProducer(""))
-        result.callback(response)
+        result.callback((response, ""))
         return d
 
-
     def test_serialization(self):
         """
-        L{OS_X_10_6.serialize} properly generates a JSON document.
+        L{OS_X_10_11.serialize} properly generates a JSON document.
         """
-        clientPath = os.path.join(self.client.serializePath, "user91-OS_X_10.6")
+        clientPath = os.path.join(self.client.serializePath, "user91-OS_X_10.11")
         self.assertFalse(os.path.exists(clientPath))
         indexPath = os.path.join(clientPath, "index.json")
         self.assertFalse(os.path.exists(indexPath))
@@ -1527,7 +1539,7 @@ END:VCALENDAR
         self.client._events.update(dict([[event.url, event] for event in events]))
 
         calendars = (
-            Calendar(str(caldavxml.calendar), set(('VEVENT',)), u'calendar', u'/home/calendar/', "123"),
+            Calendar(str(caldavxml.calendar), set(('VEVENT',)), u'calendar', u'/home/calendar/', "123", invitees=["a", "b", "c"]),
             Calendar(str(caldavxml.calendar), set(('VTODO',)), u'tasks', u'/home/tasks/', "456"),
             Calendar(str(caldavxml.schedule_inbox), set(('VEVENT', "VTODO",)), u'calendar', u'/home/inbox/', "789"),
         )
@@ -1535,20 +1547,51 @@ END:VCALENDAR
         self.client._calendars["/home/calendar/"].events["1.ics"] = events[0]
         self.client._calendars["/home/inbox/"].events["i1.ics"] = events[1]
 
+        self.client._notificationCollection = NotificationCollection("/home/notification", "123")
+
+        self.client._managed_attachments_server_url = "attachmentsurl"
+        self.client.calendarHomeToken = "hometoken"
+
         self.client.serialize()
         self.assertTrue(os.path.exists(clientPath))
         self.assertTrue(os.path.exists(indexPath))
+
         def _normDict(d):
-            return dict([(k, sorted(v, key=lambda x: x["changeToken" if k == "calendars" else "url"]) if v else None,) for k, v in d.items()])
-        self.assertEqual(_normDict(json.loads(open(indexPath).read())), _normDict(json.loads("""{
+            return dict([
+                (
+                    k,
+                    sorted(
+                        v,
+                        key=lambda x:
+                            x["changeToken" if k == "calendars" else "url"]
+                    ) if isinstance(v, list) else v,
+                )
+                for k, v in d.items()
+            ])
+        with open(indexPath) as f:
+            jdata = f.read()
+
+        self.assertEqual(_normDict(json.loads(jdata)), _normDict(json.loads("""{
+  "notificationCollection": {
+      "url": "/home/notification",
+      "notifications": [],
+      "changeToken": "123"
+    },
   "calendars": [
     {
       "changeToken": "123",
       "name": "calendar",
+      "shared": false,
+      "sharedByMe": false,
       "resourceType": "{urn:ietf:params:xml:ns:caldav}calendar",
       "componentTypes": [
         "VEVENT"
       ],
+      "invitees": [
+        "a",
+        "b",
+        "c"
+      ],
       "url": "/home/calendar/",
       "events": [
         "1.ics"
@@ -1557,11 +1600,14 @@ END:VCALENDAR
     {
       "changeToken": "789",
       "name": "calendar",
+      "shared": false,
+      "sharedByMe": false,
       "resourceType": "{urn:ietf:params:xml:ns:caldav}schedule-inbox",
       "componentTypes": [
         "VEVENT",
         "VTODO"
       ],
+      "invitees": [],
       "url": "/home/inbox/",
       "events": [
         "i1.ics"
@@ -1570,15 +1616,20 @@ END:VCALENDAR
     {
       "changeToken": "456",
       "name": "tasks",
+      "shared": false,
+      "sharedByMe": false,
       "resourceType": "{urn:ietf:params:xml:ns:caldav}calendar",
       "componentTypes": [
         "VTODO"
       ],
+      "invitees": [],
       "url": "/home/tasks/",
       "events": []
     }
   ],
   "principalURL": null,
+  "homeToken": "hometoken",
+  "attachmentsUrl": "attachmentsurl",
   "events": [
     {
       "url": "/home/calendar/1.ics",
@@ -1592,21 +1643,25 @@ END:VCALENDAR
       "etag": "123.123",
       "uid": "00a79cad-857b-418e-a54a-340b5686d747"
     }
-  ]
+  ],
+  "attachments": {}
 }""")))
 
         event1Path = os.path.join(clientPath, "calendar", "1.ics")
         self.assertTrue(os.path.exists(event1Path))
-        self.assertEqual(open(event1Path).read(), cal1)
+        with open(event1Path) as f:
+            data = f.read()
+        self.assertEqual(data, cal1)
 
         event2Path = os.path.join(clientPath, "inbox", "i1.ics")
         self.assertTrue(os.path.exists(event2Path))
-        self.assertEqual(open(event2Path).read(), cal2)
-
+        with open(event2Path) as f:
+            data = f.read()
+        self.assertEqual(data, cal2)
 
     def test_deserialization(self):
         """
-        L{OS_X_10_6.deserailize} properly parses a JSON document.
+        L{OS_X_10_11.deserailize} properly parses a JSON document.
         """
 
         cal1 = """BEGIN:VCALENDAR
@@ -1639,18 +1694,27 @@ END:VEVENT
 END:VCALENDAR
 """.replace("\n", "\r\n")
 
-        clientPath = os.path.join(self.client.serializePath, "user91-OS_X_10.6")
+        clientPath = os.path.join(self.client.serializePath, "user91-OS_X_10.11")
         os.mkdir(clientPath)
         indexPath = os.path.join(clientPath, "index.json")
-        open(indexPath, "w").write("""{
+        with open(indexPath, "w") as f:
+            f.write("""{
   "calendars": [
     {
       "changeToken": "321",
+      "attachmentsUrl": "https://example.com/attachments/",
       "name": "calendar",
+      "shared": false,
+      "sharedByMe": false,
       "resourceType": "{urn:ietf:params:xml:ns:caldav}calendar",
       "componentTypes": [
         "VEVENT"
       ],
+      "invitees": [
+        "a",
+        "b",
+        "c"
+      ],
       "url": "/home/calendar/",
       "events": [
         "2.ics"
@@ -1659,11 +1723,18 @@ END:VCALENDAR
     {
       "changeToken": "987",
       "name": "calendar",
+      "shared": false,
+      "sharedByMe": false,
       "resourceType": "{urn:ietf:params:xml:ns:caldav}schedule-inbox",
       "componentTypes": [
         "VEVENT",
         "VTODO"
       ],
+      "invitees": [
+        "a",
+        "b",
+        "c"
+      ],
       "url": "/home/inbox/",
       "events": [
         "i2.ics"
@@ -1672,15 +1743,24 @@ END:VCALENDAR
     {
       "changeToken": "654",
       "name": "tasks",
+      "shared": false,
+      "sharedByMe": false,
       "resourceType": "{urn:ietf:params:xml:ns:caldav}calendar",
       "componentTypes": [
         "VTODO"
       ],
+      "invitees": [
+        "a",
+        "b",
+        "c"
+      ],
       "url": "/home/tasks/",
       "events": []
     }
   ],
   "principalURL": null,
+  "homeToken": "hometoken",
+  "attachmentsUrl": "attachmentsurl",
   "events": [
     {
       "url": "/home/calendar/2.ics",
@@ -1699,10 +1779,12 @@ END:VCALENDAR
 
         os.mkdir(os.path.join(clientPath, "calendar"))
         event1Path = os.path.join(clientPath, "calendar", "2.ics")
-        open(event1Path, "w").write(cal1)
+        with open(event1Path, "w") as f:
+            f.write(cal1)
         os.mkdir(os.path.join(clientPath, "inbox"))
         event1Path = os.path.join(clientPath, "inbox", "i2.ics")
-        open(event1Path, "w").write(cal2)
+        with open(event1Path, "w") as f:
+            f.write(cal2)
 
         self.client.deserialize()
 
@@ -1712,6 +1794,7 @@ END:VCALENDAR
         self.assertEqual(self.client._calendars["/home/calendar/"].name, "calendar")
         self.assertEqual(self.client._calendars["/home/calendar/"].resourceType, "{urn:ietf:params:xml:ns:caldav}calendar")
         self.assertEqual(self.client._calendars["/home/calendar/"].componentTypes, set(("VEVENT",)))
+        self.assertEqual(self.client._calendars["/home/calendar/"].invitees, ["a", "b", "c"])
         self.assertTrue("/home/tasks/" in self.client._calendars)
         self.assertTrue("/home/inbox/" in self.client._calendars)
         self.assertEqual(self.client._calendars["/home/inbox/"].componentTypes, set(("VEVENT", "VTODO",)))
@@ -1726,12 +1809,13 @@ END:VCALENDAR
         self.assertEqual(self.client._events["/home/inbox/i2.ics"].etag, "987.987")
         self.assertEqual(self.client._events["/home/inbox/i2.ics"].getUID(), "00a79cad-857b-418e-a54a-340b5686d747")
         self.assertEqual(str(self.client._events["/home/inbox/i2.ics"].component), cal2)
+        self.assertEqual(self.client.calendarHomeToken, "hometoken")
+        self.assertEqual(self.client._managed_attachments_server_url, "attachmentsurl")
 
 
-
-class UpdateCalendarTests(OS_X_10_6Mixin, TestCase):
+class UpdateCalendarTests(OS_X_10_11Mixin, TestCase):
     """
-    Tests for L{OS_X_10_6._updateCalendar}.
+    Tests for L{OS_X_10_11._updateCalendar}.
     """
 
     _CALENDAR_PROPFIND_RESPONSE_BODY = """\
@@ -1858,7 +1942,7 @@ END:VCALENDAR
 
     def test_eventMissing(self):
         """
-        If an event included in the calendar PROPFIND response no longer exists
+        If an event included in the calendar sync REPORT response no longer exists
         by the time a REPORT is issued for that event, the 404 is handled and
         the rest of the normal update logic for that event is skipped.
         """
@@ -1869,14 +1953,18 @@ END:VCALENDAR
         self.client._updateCalendar(calendar, "1234")
         result, req = requests.pop(0)
         expectedResponseCode, method, url, _ignore_headers, _ignore_body = req
-        self.assertEqual('PROPFIND', method)
+        self.assertEqual('REPORT', method)
         self.assertEqual('http://127.0.0.1/something/', url)
-        self.assertEqual((MULTI_STATUS,), expectedResponseCode)
+        self.assertEqual((MULTI_STATUS, FORBIDDEN), expectedResponseCode)
 
         result.callback(
-            MemoryResponse(
-                ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
-                StringProducer(self._CALENDAR_PROPFIND_RESPONSE_BODY)))
+            (
+                MemoryResponse(
+                    ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
+                    StringProducer(self._CALENDAR_PROPFIND_RESPONSE_BODY)),
+                self._CALENDAR_PROPFIND_RESPONSE_BODY
+            )
+        )
 
         result, req = requests.pop(0)
         expectedResponseCode, method, url, _ignore_headers, _ignore_body = req
@@ -1888,18 +1976,21 @@ END:VCALENDAR
         del self.client._events["/something/anotherthing.ics"]
 
         result.callback(
-            MemoryResponse(
-                ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
-                StringProducer(self._CALENDAR_REPORT_RESPONSE_BODY)))
+            (
+                MemoryResponse(
+                    ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
+                    StringProducer(self._CALENDAR_REPORT_RESPONSE_BODY)),
+                self._CALENDAR_REPORT_RESPONSE_BODY
+            )
+        )
 
         # Verify that processing proceeded to the response after the one with a
         # 404 status.
         self.assertIn('/something/else.ics', self.client._events)
 
-
     def test_multigetBatch(self):
         """
-        If an event included in the calendar PROPFIND response no longer exists
+        If an event included in the calendar sync REPORT response no longer exists
         by the time a REPORT is issued for that event, the 404 is handled and
         the rest of the normal update logic for that event is skipped.
         """
@@ -1912,14 +2003,18 @@ END:VCALENDAR
         self.client._updateCalendar(calendar, "1234")
         result, req = requests.pop(0)
         expectedResponseCode, method, url, _ignore_headers, _ignore_body = req
-        self.assertEqual('PROPFIND', method)
+        self.assertEqual('REPORT', method)
         self.assertEqual('http://127.0.0.1/something/', url)
-        self.assertEqual((MULTI_STATUS,), expectedResponseCode)
+        self.assertEqual((MULTI_STATUS, FORBIDDEN), expectedResponseCode)
 
         result.callback(
-            MemoryResponse(
-                ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
-                StringProducer(self._CALENDAR_PROPFIND_RESPONSE_BODY)))
+            (
+                MemoryResponse(
+                    ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
+                    StringProducer(self._CALENDAR_PROPFIND_RESPONSE_BODY)),
+                self._CALENDAR_PROPFIND_RESPONSE_BODY
+            )
+        )
 
         result, req = requests.pop(0)
         expectedResponseCode, method, url, _ignore_headers, _ignore_body = req
@@ -1928,9 +2023,13 @@ END:VCALENDAR
         self.assertEqual((MULTI_STATUS,), expectedResponseCode)
 
         result.callback(
-            MemoryResponse(
-                ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
-                StringProducer(self._CALENDAR_REPORT_RESPONSE_BODY_1)))
+            (
+                MemoryResponse(
+                    ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
+                    StringProducer(self._CALENDAR_REPORT_RESPONSE_BODY_1)),
+                self._CALENDAR_REPORT_RESPONSE_BODY_1
+            )
+        )
 
         self.assertTrue(self.client._events['/something/anotherthing.ics'].etag is not None)
         self.assertTrue(self.client._events['/something/else.ics'].etag is None)
@@ -1942,22 +2041,26 @@ END:VCALENDAR
         self.assertEqual((MULTI_STATUS,), expectedResponseCode)
 
         result.callback(
-            MemoryResponse(
-                ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
-                StringProducer(self._CALENDAR_REPORT_RESPONSE_BODY_2)))
+            (
+                MemoryResponse(
+                    ('HTTP', '1', '1'), MULTI_STATUS, "Multi-status", None,
+                    StringProducer(self._CALENDAR_REPORT_RESPONSE_BODY_2)),
+                self._CALENDAR_REPORT_RESPONSE_BODY_2
+            )
+        )
 
         self.assertTrue(self.client._events['/something/anotherthing.ics'].etag is not None)
         self.assertTrue(self.client._events['/something/else.ics'].etag is not None)
 
 
+class VFreeBusyTests(OS_X_10_11Mixin, TestCase):
+    """
+    Tests for L{OS_X_10_11.requestAvailability}.
+    """
 
-class VFreeBusyTests(OS_X_10_6Mixin, TestCase):
-    """
-    Tests for L{OS_X_10_6.requestAvailability}.
-    """
     def test_requestAvailability(self):
         """
-        L{OS_X_10_6.requestAvailability} accepts a date range and a set of
+        L{OS_X_10_11.requestAvailability} accepts a date range and a set of
         account uuids and issues a VFREEBUSY request.  It returns a Deferred
         which fires with a dict mapping account uuids to availability range
         information.
@@ -1987,6 +2090,7 @@ class VFreeBusyTests(OS_X_10_6Mixin, TestCase):
 
         consumer = MemoryConsumer()
         finished = body.startProducing(consumer)
+
         def cbFinished(ignored):
             vevent = Component.fromString(consumer.value())
             uid = vevent.resourceUID()
@@ -2016,7 +2120,7 @@ END:VCALENDAR
             response = MemoryResponse(
                 ('HTTP', '1', '1'), OK, "Ok", Headers({}),
                 StringProducer(""))
-            result.callback(response)
+            result.callback((response, ""))
         finished.addCallback(requested)
 
         return d
