@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,7 +38,13 @@ from contrib.performance.loadtest.sim import (
 
 
 VALID_CONFIG = {
-    'server': 'tcp:127.0.0.1:8008',
+    'servers': {
+        "PodA": {
+            "enabled": True,
+            "uri": 'http://example.org:1234/',
+            "stats": {"enabled": False},
+        },
+    },
     'webadmin': {
         'enabled': True,
         'HTTPPort': 8080,
@@ -56,8 +62,8 @@ VALID_CONFIG = {
 VALID_CONFIG_PLIST = writePlistToString(VALID_CONFIG)
 
 
-
 class SimOptionsTests(TestCase):
+
     def test_defaultConfig(self):
         """
         If the I{config} option is not specified, the default config.plist in
@@ -66,7 +72,6 @@ class SimOptionsTests(TestCase):
         options = SimOptions()
         self.assertEqual(options['config'], FilePath(__file__).sibling('config.plist'))
 
-
     def test_configFileNotFound(self):
         """
         If the filename given to the I{config} option is not found,
@@ -80,7 +85,6 @@ class SimOptionsTests(TestCase):
         self.assertEquals(
             str(exc), "--config %s: No such file or directory" % (name.path,))
 
-
     def test_configFileNotParseable(self):
         """
         If the contents of the file given to the I{config} option
@@ -98,7 +102,6 @@ class SimOptionsTests(TestCase):
             "--config %s: syntax error: line 1, column 0" % (config.path,))
 
 
-
 class CalendarClientSimulatorTests(TestCase):
     """
     Tests for L{CalendarClientSimulator} which adds running clients to
@@ -112,7 +115,6 @@ class CalendarClientSimulatorTests(TestCase):
         record = _DirectoryRecord(name, password, name, email, name)
         return record
 
-
     def test_createUser(self):
         """
         Subsequent calls to L{CalendarClientSimulator._createUser}
@@ -121,15 +123,22 @@ class CalendarClientSimulatorTests(TestCase):
         """
         calsim = CalendarClientSimulator(
             [self._user('alice'), self._user('bob'), self._user('carol')],
-            Populator(None), None, None, 'http://example.org:1234/', None, None)
+            Populator(None), None, None, None,
+            {
+                "PodA": {
+                    "enabled": True,
+                    "uri": 'http://example.org:1234/',
+                    "stats": {"enabled": False},
+                },
+            },
+            None, None)
         users = sorted([
-            calsim._createUser(0)[0],
-            calsim._createUser(1)[0],
-            calsim._createUser(2)[0],
+            calsim._createUser(0)[1],
+            calsim._createUser(1)[1],
+            calsim._createUser(2)[1],
         ])
         self.assertEqual(['alice', 'bob', 'carol'], users)
 
-
     def test_createUserAuthInfo(self):
         """
         The auth handler returned by L{CalendarClientSimulator._createUser}
@@ -137,8 +146,16 @@ class CalendarClientSimulatorTests(TestCase):
         """
         calsim = CalendarClientSimulator(
             [self._user('alice')],
-            Populator(None), None, None, 'http://example.org:1234/', None, None)
-        user, auth = calsim._createUser(0)
+            Populator(None), None, None, None,
+            {
+                "PodA": {
+                    "enabled": True,
+                    "uri": 'http://example.org:1234/',
+                    "stats": {"enabled": False},
+                },
+            },
+            None, None)
+        _ignore_record, user, auth = calsim._createUser(0)
         self.assertEqual(
             auth['basic'].passwd.find_user_password('Test Realm', 'http://example.org:1234/')[1],
             'password-' + user)
@@ -146,14 +163,14 @@ class CalendarClientSimulatorTests(TestCase):
             auth['digest'].passwd.find_user_password('Test Realm', 'http://example.org:1234/')[1],
             'password-' + user)
 
-
     def test_stop(self):
         """
         After L{CalendarClientSimulator.stop} is called, failed clients and
         profiles are not logged.
         """
         class BrokenClient(object):
-            def __init__(self, reactor, serverAddress, principalPathTemplate, serializationPath, userInfo, auth, runResult):
+
+            def __init__(self, reactor, serverAddress, principalPathTemplate, serializationPath, userInfo, auth, instanceNumber, runResult):
                 self._runResult = runResult
 
             def run(self):
@@ -163,6 +180,7 @@ class CalendarClientSimulatorTests(TestCase):
                 return succeed(None)
 
         class BrokenProfile(object):
+
             def __init__(self, reactor, simulator, client, userNumber, runResult):
                 self._runResult = runResult
                 self.enabled = True
@@ -182,7 +200,15 @@ class CalendarClientSimulatorTests(TestCase):
             [ProfileType(BrokenProfile, {'runResult': profileRunResult})])
         )
         sim = CalendarClientSimulator(
-            [self._user('alice')], Populator(None), params, None, 'http://example.com:1234/', None, None)
+            [self._user('alice')], Populator(None), None, params, None,
+            {
+                "PodA": {
+                    "enabled": True,
+                    "uri": 'http://example.org:1234/',
+                    "stats": {"enabled": False},
+                },
+            },
+            None, None)
         sim.add(1, 1)
         sim.stop()
         clientRunResult.errback(RuntimeError("Some fictional client problem"))
@@ -191,7 +217,6 @@ class CalendarClientSimulatorTests(TestCase):
         self.assertEqual([], self.flushLoggedErrors())
 
 
-
 class Reactor(object):
     message = "some event to be observed"
 
@@ -199,7 +224,6 @@ class Reactor(object):
         self._triggers = []
         self._whenRunning = []
 
-
     def run(self):
         for thunk in self._whenRunning:
             thunk()
@@ -208,48 +232,43 @@ class Reactor(object):
             if event == 'shutdown':
                 thunk()
 
-
     def callWhenRunning(self, thunk):
         self._whenRunning.append(thunk)
 
-
     def addSystemEventTrigger(self, phase, event, thunk):
         self._triggers.append((phase, event, thunk))
 
 
-
 class Observer(object):
+
     def __init__(self):
         self.reported = False
         self.events = []
 
-
     def observe(self, event):
         self.events.append(event)
 
-
     def report(self, output):
         self.reported = True
 
-
     def failures(self):
         return []
 
 
-
 class NullArrival(object):
+
     def run(self, sim):
         pass
 
 
-
 class StubSimulator(LoadSimulator):
+
     def run(self):
         return 3
 
 
-
 class LoadSimulatorTests(TestCase):
+
     def test_main(self):
         """
         L{LoadSimulator.main} raises L{SystemExit} with the result of
@@ -263,22 +282,26 @@ class LoadSimulatorTests(TestCase):
         self.assertEquals(
             exc.args, (StubSimulator(None, None, None, None, None, None, None).run(),))
 
-
     def test_createSimulator(self):
         """
         L{LoadSimulator.createSimulator} creates a L{CalendarClientSimulator}
         with its own reactor and host and port information from the
         configuration file.
         """
-        server = 'http://127.0.0.7:1243/'
+        servers = {
+            "PodA": {
+                "enabled": True,
+                "uri": 'http://example.org:1234/',
+                "stats": {"enabled": False},
+            },
+        }
         reactor = object()
-        sim = LoadSimulator(server, None, None, None, None, None, None, reactor=reactor)
+        sim = LoadSimulator(servers, None, None, None, None, None, None, reactor=reactor)
         calsim = sim.createSimulator()
         self.assertIsInstance(calsim, CalendarClientSimulator)
         self.assertIsInstance(calsim.reactor, LagTrackingReactor)
         self.assertIdentical(calsim.reactor._reactor, reactor)
-        self.assertEquals(calsim.server, server)
-
+        self.assertEquals(calsim.servers, servers)
 
     def test_loadAccountsFromFile(self):
         """
@@ -292,7 +315,8 @@ class LoadSimulatorTests(TestCase):
         config["accounts"] = {
             "loader": "contrib.performance.loadtest.sim.recordsFromCSVFile",
             "params": {
-                "path": accounts.path
+                "path": accounts.path,
+                "interleavePods": True,
             },
         }
         configpath = FilePath(self.mktemp())
@@ -310,7 +334,6 @@ class LoadSimulatorTests(TestCase):
         self.assertEqual(sim.records[1].commonName, 'baz2')
         self.assertEqual(sim.records[1].email, 'quux2')
 
-
     def test_loadDefaultAccountsFromFile(self):
         """
         L{LoadSimulator.fromCommandLine} takes an account loader (with
@@ -321,7 +344,8 @@ class LoadSimulatorTests(TestCase):
         config["accounts"] = {
             "loader": "contrib.performance.loadtest.sim.recordsFromCSVFile",
             "params": {
-                "path": ""
+                "path": "",
+                "interleavePods": True,
             },
         }
         configpath = FilePath(self.mktemp())
@@ -338,7 +362,6 @@ class LoadSimulatorTests(TestCase):
         self.assertEqual(sim.records[98].commonName, 'User 99')
         self.assertEqual(sim.records[98].email, 'user99@example.com')
 
-
     def test_generateRecordsDefaultPatterns(self):
         """
         L{LoadSimulator.fromCommandLine} takes an account loader from the
@@ -366,7 +389,6 @@ class LoadSimulatorTests(TestCase):
         self.assertEqual(sim.records[1].commonName, 'User 2')
         self.assertEqual(sim.records[1].email, 'user2@example.com')
 
-
     def test_generateRecordsNonDefaultPatterns(self):
         """
         L{LoadSimulator.fromCommandLine} takes an account loader from the
@@ -398,7 +420,6 @@ class LoadSimulatorTests(TestCase):
         self.assertEqual(sim.records[2].commonName, 'Test User 003')
         self.assertEqual(sim.records[2].email, 'USER003@example2.com')
 
-
     def test_specifyRuntime(self):
         """
         L{LoadSimulator.fromCommandLine} recognizes the I{--runtime} option to
@@ -409,7 +430,6 @@ class LoadSimulatorTests(TestCase):
         sim = LoadSimulator.fromCommandLine(['--config', config.path, '--runtime', '123'])
         self.assertEqual(123, sim.runtime)
 
-
     def test_loadServerConfig(self):
         """
         The Calendar Server host and port are loaded from the [server]
@@ -417,11 +437,16 @@ class LoadSimulatorTests(TestCase):
         """
         config = FilePath(self.mktemp())
         config.setContent(
-            writePlistToString({"server": "https://127.0.0.3:8432/"})
+            writePlistToString({"servers": {
+                "PodA": {
+                    "enabled": True,
+                    "uri": 'https://127.0.0.3:8432/',
+                    "stats": {"enabled": False},
+                },
+            }})
         )
         sim = LoadSimulator.fromCommandLine(['--config', config.path])
-        self.assertEquals(sim.server, "https://127.0.0.3:8432/")
-
+        self.assertEquals(sim.servers["PodA"]["uri"], "https://127.0.0.3:8432/")
 
     def test_loadArrivalConfig(self):
         """
@@ -446,13 +471,13 @@ class LoadSimulatorTests(TestCase):
             sim.arrival,
             Arrival(SmoothRampUp, dict(groups=10, groupSize=1, interval=3)))
 
-
     def test_createArrivalPolicy(self):
         """
         L{LoadSimulator.createArrivalPolicy} creates an arrival
         policy based on the L{Arrival} passed to its initializer.
         """
         class FakeArrival(object):
+
             def __init__(self, reactor, x, y):
                 self.reactor = reactor
                 self.x = x
@@ -460,14 +485,13 @@ class LoadSimulatorTests(TestCase):
 
         reactor = object()
         sim = LoadSimulator(
-            None, None, None, None, None, Arrival(FakeArrival, {'x': 3, 'y': 2}), None, reactor=reactor)
+            None, None, None, None, Arrival(FakeArrival, {'x': 3, 'y': 2}), None, reactor=reactor)
         arrival = sim.createArrivalPolicy()
         self.assertIsInstance(arrival, FakeArrival)
         self.assertIdentical(arrival.reactor, sim.reactor)
         self.assertEquals(arrival.x, 3)
         self.assertEquals(arrival.y, 2)
 
-
     def test_loadPopulationParameters(self):
         """
         Client weights and profiles are loaded from the [clients]
@@ -526,7 +550,6 @@ class LoadSimulatorTests(TestCase):
         )
         self.assertEquals(sim.parameters, expectedParameters)
 
-
     def test_requireClient(self):
         """
         At least one client is required, so if a configuration with an
@@ -543,7 +566,6 @@ class LoadSimulatorTests(TestCase):
             1, ClientType(OS_X_10_6, {}, [Eventer, Inviter, Accepter]))
         self.assertEquals(sim.parameters, expectedParameters)
 
-
     def test_loadLogObservers(self):
         """
         Log observers specified in the [observers] section of the
@@ -566,7 +588,6 @@ class LoadSimulatorTests(TestCase):
         self.assertEquals(len(sim.observers), 1)
         self.assertIsInstance(sim.observers[0], SimpleStatistics)
 
-
     def test_observeRunReport(self):
         """
         Each log observer is added to the log publisher before the
@@ -575,11 +596,16 @@ class LoadSimulatorTests(TestCase):
         """
         observers = [Observer()]
         sim = LoadSimulator(
-            "http://example.com:123/",
+            {
+                "PodA": {
+                    "enabled": True,
+                    "uri": 'http://example.org:1234/',
+                    "stats": {"enabled": False},
+                },
+            },
             "/principals/users/%s/",
             None,
             None,
-            None,
             Arrival(lambda reactor: NullArrival(), {}),
             None, observers, reactor=Reactor())
         io = StringIO()
