@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 Defines a set of HTTP requests to execute and return results.
 """
 
+
 class HTTPTestBase(object):
     """
     Base class for an HTTP request that executes and results are returned for.
@@ -30,7 +31,6 @@ class HTTPTestBase(object):
             self.rows = rows
             self.timing = timing
 
-
     def __init__(self, label, sessions, logFilePath, logFilePrefix):
         """
         @param label: label used to identify the test
@@ -42,7 +42,6 @@ class HTTPTestBase(object):
         self.logFilePrefix = logFilePrefix
         self.result = None
 
-
     def execute(self, count):
         """
         Execute the HTTP request and read the results.
@@ -55,20 +54,18 @@ class HTTPTestBase(object):
         self.cleanup()
         return self.result
 
-
     def prepare(self):
         """
         Do some setup prior to the real request.
         """
         pass
 
-
     def clearLog(self):
         """
         Clear the server's SQL log file.
         """
-        open(self.logFilePath, "w").write("")
-
+        with open(self.logFilePath, "w") as f:
+            f.write("")
 
     def doRequest(self):
         """
@@ -76,7 +73,6 @@ class HTTPTestBase(object):
         """
         raise NotImplementedError
 
-
     def collectResults(self, event_count):
         """
         Parse the server log file to extract the details we need.
@@ -91,7 +87,8 @@ class HTTPTestBase(object):
             return float(line[pos + 2:])
 
         # Need to skip over stats that are unlabeled
-        data = open(self.logFilePath).read()
+        with open(self.logFilePath) as f:
+            data = f.read()
         lines = data.splitlines()
         offset = 0
         while True:
@@ -110,7 +107,6 @@ class HTTPTestBase(object):
         with open("%s-%s-%d-%s" % (self.logFilePath, self.logFilePrefix, event_count, self.label), "w") as f:
             f.write(data)
 
-
     def cleanup(self):
         """
         Do some cleanup after the real request.
