@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,10 +31,13 @@ import errno
 This tool reads data from the server's statistics socket and prints a summary.
 """
 
+
 def safeDivision(value, total, factor=1):
     return value * factor / total if total else 0
 
 
+SummaryValues = collections.namedtuple("SummaryValues", ("requests", "response", "slots", "cpu", "errors"))
+
 
 def readSock(sockname, useTCP):
     try:
@@ -62,21 +65,21 @@ def readSock(sockname, useTCP):
     return data
 
 
-
 def printStats(stats, multimode, showMethods, topUsers, showAgents, dumpFile):
     if len(stats) == 1:
         if "Failed" in stats[0]:
             printFailedStats(stats[0]["Failed"])
         else:
             try:
-                printStat(stats[0], multimode[0], showMethods, topUsers, showAgents)
+                summary = printStat(stats[0], multimode[0], showMethods, topUsers, showAgents)
             except KeyError, e:
                 printFailedStats("Unable to find key '%s' in statistics from server socket" % (e,))
                 sys.exit(1)
 
     else:
-        printMultipleStats(stats, multimode, showMethods, topUsers, showAgents)
+        summary = printMultipleStats(stats, multimode, showMethods, topUsers, showAgents)
 
+    return summary
 
 
 def printStat(stats, index, showMethods, topUsers, showAgents):
@@ -99,7 +102,7 @@ def printStat(stats, index, showMethods, topUsers, showAgents):
         print("Current CPU: Unavailable")
         print("Current Memory Used: Unavailable")
     print
-    printRequestSummary(stats)
+    summary = printRequestSummary(stats)
     printHistogramSummary(stats[index], index)
     if showMethods:
         printMethodCounts(stats[index])
@@ -108,6 +111,7 @@ def printStat(stats, index, showMethods, topUsers, showAgents):
     if showAgents:
         printAgentCounts(stats[index])
 
+    return summary
 
 
 def printMultipleStats(stats, multimode, showMethods, topUsers, showAgents):
@@ -135,7 +139,7 @@ def printMultipleStats(stats, multimode, showMethods, topUsers, showAgents):
             cpus.append(-1)
             memories.append(-1)
 
-    printMultiRequestSummary(stats, cpus, memories, times, labels, multimode)
+    summary = printMultiRequestSummary(stats, cpus, memories, times, labels, multimode)
     printMultiHistogramSummary(stats, multimode[0])
     if showMethods:
         printMultiMethodCounts(stats, multimode[0])
@@ -144,6 +148,7 @@ def printMultipleStats(stats, multimode, showMethods, topUsers, showAgents):
     if showAgents:
         printMultiAgentCounts(stats, multimode[0])
 
+    return summary
 
 
 def serverLabels(stats):
@@ -169,7 +174,6 @@ def serverLabels(stats):
     return [".".join(item) for item in servers]
 
 
-
 def printFailedStats(message):
 
     print("- " * 40)
@@ -178,7 +182,6 @@ def printFailedStats(message):
     print("")
 
 
-
 def printRequestSummary(stats):
     table = tables.Table()
     table.addHeader(
@@ -220,6 +223,14 @@ def printRequestSummary(stats):
     table.printTable(os=os)
     print(os.getvalue())
 
+    stat = stats["1m"]
+    return SummaryValues(
+        requests=safeDivision(float(stat["requests"]), seconds),
+        response=safeDivision(stat["t"], stat["requests"]),
+        slots=safeDivision(float(stat["slots"]), stat["requests"]),
+        cpu=safeDivision(stat["cpu"], stat["requests"]),
+        errors=stat["500"],
+    )
 
 
 def printMultiRequestSummary(stats, cpus, memories, times, labels, index):
@@ -285,6 +296,13 @@ def printMultiRequestSummary(stats, cpus, memories, times, labels, index):
     table.printTable(os=os)
     print(os.getvalue())
 
+    return SummaryValues(
+        requests=totals[2],
+        response=totals[3],
+        slots=totals[6],
+        cpu=totals[7],
+        errors=totals[10],
+    )
 
 
 def printHistogramSummary(stat, index):
@@ -326,15 +344,14 @@ def printHistogramSummary(stat, index):
     print(os.getvalue())
 
 
-
 def printMultiHistogramSummary(stats, index):
 
     # Totals first
     keys = ("<10ms", "10ms<->100ms", "100ms<->1s", "1s<->10s", "10s<->30s", "30s<->60s", ">60s", "Over 1s", "Over 10s",)
     totals = {
-        "T"        : dict([(k, 0) for k in keys]),
+        "T": dict([(k, 0) for k in keys]),
         "T-RESP-WR": dict([(k, 0) for k in keys]),
-        "requests" : 0,
+        "requests": 0,
     }
 
     for stat in stats:
@@ -346,7 +363,6 @@ def printMultiHistogramSummary(stats, index):
     printHistogramSummary(totals, index)
 
 
-
 def printMethodCounts(stat):
 
     print("Method Counts")
@@ -390,7 +406,6 @@ def printMethodCounts(stat):
     print(os.getvalue())
 
 
-
 def printMultiMethodCounts(stats, index):
 
     methods = collections.defaultdict(int)
@@ -405,7 +420,6 @@ def printMultiMethodCounts(stats, index):
     printMethodCounts({"method": methods, "method-t": method_times})
 
 
-
 def printUserCounts(stat, topUsers):
 
     print("User Counts")
@@ -433,7 +447,6 @@ def printUserCounts(stat, topUsers):
     print(os.getvalue())
 
 
-
 def printMultiUserCounts(stats, index, topUsers):
 
     uids = collections.defaultdict(int)
@@ -444,7 +457,6 @@ def printMultiUserCounts(stats, index, topUsers):
     printUserCounts({"uid": uids}, topUsers)
 
 
-
 def printAgentCounts(stat):
 
     print("User-Agent Counts")
@@ -472,7 +484,6 @@ def printAgentCounts(stat):
     print(os.getvalue())
 
 
-
 def printMultiAgentCounts(stats, index):
 
     uas = collections.defaultdict(int)
@@ -483,6 +494,30 @@ def printMultiAgentCounts(stats, index):
     printAgentCounts({"user-agent": uas})
 
 
+def alert(summary, alerts):
+
+    alert = []
+    for attr, description in (
+        ("requests", "Request count",),
+        ("response", "Response time",),
+        ("slots", "Slot count",),
+        ("cpu", "CPU time",),
+        ("errors", "Error count",),
+    ):
+        threshold = getattr(alerts, attr)
+        value = getattr(summary, attr)
+        if threshold is not None and value > threshold:
+            if isinstance(value, float):
+                value = "{:.1f}".format(value)
+            alert.append("{} threshold exceeded: {}".format(description, value))
+
+    if alert:
+        for _ignore in range(10):
+            print('\a', end='')
+            sys.stdout.flush()
+            time.sleep(0.15)
+        print("\n".join(alert))
+
 
 def usage(error_msg=None):
     if error_msg:
@@ -503,6 +538,17 @@ Options:
     --agents      Include details about HTTP User-Agent usage
     --dump FILE   File to dump JSON data to
 
+    --alert-requests N     Generate an alert if the overall request
+                            count exceeds a threshold [INT]
+    --alert-response N     Generate an alert if the overall average
+                            response time exceeds a threshold [INT]
+    --alert-slots N        Generate an alert if the overall slot
+                            count exceeds a threshold [FLOAT]
+    --alert-cpu N          Generate an alert if the overall average
+                            cpu percent use exceeds a threshold [INT]
+    --alert-errors N       Generate an alert if the overall error
+                            count exceeds a threshold [INT]
+
 Description:
     This utility will print a summary of statistics read from a
     server continuously with the specified delay.
@@ -524,11 +570,21 @@ if __name__ == '__main__':
     topUsers = 0
     showAgents = False
     dumpFile = None
+    alerts = SummaryValues(
+        requests=None,
+        response=None,
+        slots=None,
+        cpu=None,
+        errors=None,
+    )
 
     multimodes = (("current", 60,), ("1m", 60,), ("5m", 5 * 60,), ("1h", 60 * 60,),)
     multimode = multimodes[2]
 
-    options, args = getopt.getopt(sys.argv[1:], "hs:t:", ["tcp=", "0", "1", "5", "60", "methods", "users=", "agents", "dump="])
+    options, args = getopt.getopt(sys.argv[1:], "hs:t:", [
+        "tcp=", "0", "1", "5", "60", "methods", "users=", "agents", "dump=",
+        "alert-requests=", "alert-response=", "alert-slots=", "alert-cpu=", "alert-errors=",
+    ])
 
     for option, value in options:
         if option == "-h":
@@ -556,14 +612,26 @@ if __name__ == '__main__':
             showAgents = True
         elif option == "--dump":
             dumpFile = value
+        elif option == "--alert-requests":
+            alerts = alerts._replace(requests=int(value))
+        elif option == "--alert-response":
+            alerts = alerts._replace(response=int(value))
+        elif option == "--alert-slots":
+            alerts = alerts._replace(slots=float(value))
+        elif option == "--alert-cpu":
+            alerts = alerts._replace(cpu=int(value))
+        elif option == "--alert-errors":
+            alerts = alerts._replace(errors=int(value))
 
     while True:
+        now = time.time()
         stats = [readSock(server, useTCP) for server in servers]
-        printStats(stats, multimode, showMethods, topUsers, showAgents, dumpFile)
+        summary = printStats(stats, multimode, showMethods, topUsers, showAgents, dumpFile)
         if dumpFile is not None:
             stats = dict([("{}:{}".format(*servers[ctr]), stat) for ctr, stat in enumerate(stats)])
             stats["timestamp"] = datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
             with open(dumpFile, "a") as f:
                 f.write(json.dumps(stats))
                 f.write("\n")
-        time.sleep(delay)
+        alert(summary, alerts)
+        time.sleep(delay - (time.time() - now))
