@@ -1,6 +1,6 @@
 ##
 # Copyright (c) 2001-2004 Twisted Matrix Laboratories.
-# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -78,12 +78,11 @@ class KerberosCredentialFactoryBase(object):
             try:
                 principal = kerberos.getServerPrincipalDetails(serviceType, hostname)
             except kerberos.KrbError, ex:
-                self.log.error("getServerPrincipalDetails: %s" % (ex[0],))
+                self.log.error("getServerPrincipalDetails: {ex}", ex=ex[0])
                 raise ValueError('Authentication System Failure: %s' % (ex[0],))
 
         self.service, self.realm = self._splitPrincipal(principal)
 
-
     def _splitPrincipal(self, principal):
 
         try:
@@ -93,7 +92,7 @@ class KerberosCredentialFactoryBase(object):
             service = splits[0].upper()
             realm = splits[1]
         except IndexError:
-            self.log.error("Invalid Kerberos principal: %s" % (principal,))
+            self.log.error("Invalid Kerberos principal: {principal}", principal=principal)
             raise ValueError('Authentication System Failure: Invalid Kerberos principal: %s' % (principal,))
 
         service = "%s@%s" % (servicetype, service,)
@@ -102,7 +101,6 @@ class KerberosCredentialFactoryBase(object):
         return (service, realm,)
 
 
-
 class BasicKerberosCredentials(credentials.UsernamePassword):
     """
     A set of user/password credentials that checks itself against Kerberos.
@@ -127,7 +125,6 @@ class BasicKerberosCredentials(credentials.UsernamePassword):
         self.default_realm = realm
 
 
-
 class BasicKerberosCredentialFactory(KerberosCredentialFactoryBase):
     """
     Authorizer for insecure Basic (base64-encoded plaintext) authentication.
@@ -152,11 +149,9 @@ class BasicKerberosCredentialFactory(KerberosCredentialFactoryBase):
 
         super(BasicKerberosCredentialFactory, self).__init__(principal, serviceType, hostname)
 
-
     def getChallenge(self, _ignore_peer):
         return succeed({'realm': self.realm})
 
-
     def decode(self, response, request):  # @UnusedVariable
         try:
             creds = (response + '===').decode('base64')
@@ -170,7 +165,6 @@ class BasicKerberosCredentialFactory(KerberosCredentialFactoryBase):
         raise error.LoginFailed('Invalid credentials')
 
 
-
 class BasicKerberosCredentialsChecker(object):
     log = Logger()
 
@@ -188,7 +182,7 @@ class BasicKerberosCredentialsChecker(object):
             try:
                 kerberos.checkPassword(creds.username, creds.password, creds.service, creds.default_realm)
             except kerberos.BasicAuthError, ex:
-                self.log.error("%s" % (ex[0],))
+                self.log.error("{ex}", ex=ex[0])
                 raise error.UnauthorizedLogin("Bad credentials for: %s (%s: %s)" % (pcreds.authnURI, ex[0], ex[1],))
             else:
                 return succeed((
@@ -199,7 +193,6 @@ class BasicKerberosCredentialsChecker(object):
         raise error.UnauthorizedLogin("Bad credentials for: %s" % (pcreds.authnURI,))
 
 
-
 class NegotiateCredentials(object):
     """
     A set of user/password credentials that checks itself against Kerberos.
@@ -213,7 +206,6 @@ class NegotiateCredentials(object):
         self.username = username
 
 
-
 class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
     """
     Authorizer for Negotiate authentication (http://www.ietf.org/rfc/rfc4559.txt).
@@ -235,11 +227,9 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
 
         super(NegotiateCredentialFactory, self).__init__(principal, serviceType, hostname)
 
-
     def getChallenge(self, _ignore_peer):
         return succeed({})
 
-
     def decode(self, base64data, request):
 
         # Init GSSAPI first - we won't specify the service now as we need to accept a target
@@ -247,18 +237,18 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
         try:
             _ignore_result, context = kerberos.authGSSServerInit("")
         except kerberos.GSSError, ex:
-            self.log.error("authGSSServerInit: %s(%s)" % (ex[0][0], ex[1][0],))
+            self.log.error("authGSSServerInit: {ex0}({ex1})", ex0=ex[0][0], ex1=ex[1][0])
             raise error.LoginFailed('Authentication System Failure: %s(%s)' % (ex[0][0], ex[1][0],))
 
         # Do the GSSAPI step and get response and username
         try:
             kerberos.authGSSServerStep(context, base64data)
         except kerberos.GSSError, ex:
-            self.log.error("authGSSServerStep: %s(%s)" % (ex[0][0], ex[1][0],))
+            self.log.error("authGSSServerStep: {ex0}(ex1)", ex0=ex[0][0], ex1=ex[1][0])
             kerberos.authGSSServerClean(context)
             raise error.UnauthorizedLogin('Bad credentials: %s(%s)' % (ex[0][0], ex[1][0],))
         except kerberos.KrbError, ex:
-            self.log.error("authGSSServerStep: %s" % (ex[0],))
+            self.log.error("authGSSServerStep: {ex}", ex=ex[0])
             kerberos.authGSSServerClean(context)
             raise error.UnauthorizedLogin('Bad credentials: %s' % (ex[0],))
 
@@ -266,11 +256,11 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
         try:
             service, _ignore_realm = self._splitPrincipal(targetname)
         except ValueError:
-            self.log.error("authGSSServerTargetName invalid target name: '%s'" % (targetname,))
+            self.log.error("authGSSServerTargetName invalid target name: '{target}'", target=targetname)
             kerberos.authGSSServerClean(context)
             raise error.UnauthorizedLogin('Bad credentials: bad target name %s' % (targetname,))
         if service.lower() != self.service.lower():
-            self.log.error("authGSSServerTargetName mismatch got: '%s' wanted: '%s'" % (service, self.service))
+            self.log.error("authGSSServerTargetName mismatch got: '{got}' wanted: '{wanted}'", got=service, wanted=self.service)
             kerberos.authGSSServerClean(context)
             raise error.UnauthorizedLogin('Bad credentials: wrong target name %s' % (targetname,))
 
@@ -294,7 +284,7 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
         try:
             kerberos.authGSSServerClean(context)
         except kerberos.GSSError, ex:
-            self.log.error("authGSSServerClean: %s" % (ex[0][0], ex[1][0],))
+            self.log.error("authGSSServerClean: {ex0}({ex1})", ex0=ex[0][0], ex1=ex[1][0])
             raise error.LoginFailed('Authentication System Failure %s(%s)' % (ex[0][0], ex[1][0],))
 
         # If we successfully decoded and verified the Kerberos credentials we need to add the Kerberos
@@ -314,7 +304,6 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
         return succeed(NegotiateCredentials(principal, username))
 
 
-
 class NegotiateCredentialsChecker(object):
 
     implements(checkers.ICredentialsChecker)
