@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,7 +22,7 @@ __all__ = [
     "AbstractADBAPIDatabase",
 ]
 
-import thread
+import threading
 
 try:
     from txdav.base.datastore.subpostgres import postgres
@@ -40,27 +40,38 @@ from twistedcaldav.config import ConfigurationError
 log = Logger()
 
 
+class ConnectionCloseThread(threading.Thread):
+    """
+    An L{Thread} that closes its DB connection when it has finished running.
+    """
+
+    def __init__(self, group=None, target=None, name=None,
+                 args=(), kwargs=None, verbose=None):
+
+        if target is not None:
+            self._realTarget = target
+            target = self.targetWithConnectionClose
+        super(ConnectionCloseThread, self).__init__(group=group, target=target, name=name, args=args, kwargs=kwargs, verbose=verbose)
+
+    def targetWithConnectionClose(self, *args, **kwargs):
+        self._realTarget(*args, **kwargs)
+        if hasattr(self, "_db_close"):
+            self._db_close()
+
 
 class ConnectionClosingThreadPool(ThreadPool):
     """
-    A ThreadPool that closes connections for each worker thread
+    A L{ThreadPool} that closes connections for each worker thread when stopped.
     """
 
-    def _worker(self):
-        log.debug("Starting ADBAPI thread: %s" % (thread.get_ident(),))
-        ThreadPool._worker(self)
-        self._closeConnection()
-
-
-    def _closeConnection(self):
-
-        tid = thread.get_ident()
-        log.debug("Closing ADBAPI thread: %s" % (tid,))
-
-        conn = self.pool.connections.get(tid)
-        self.pool._close(conn)
-        del self.pool.connections[tid]
+    threadFactory = ConnectionCloseThread
 
+    def stop(self):
+        for tid, conn in self.pool.connections.items():
+            for thread in self.threads:
+                if thread.ident == tid:
+                    thread._db_close = lambda: self.pool.disconnect(conn)
+        ThreadPool.stop(self)
 
 
 class AbstractADBAPIDatabase(object):
@@ -84,11 +95,9 @@ class AbstractADBAPIDatabase(object):
 
         self.initialized = False
 
-
     def __repr__(self):
         return "<%s %r>" % (self.__class__.__name__, self.pool)
 
-
     @inlineCallbacks
     def open(self):
         """
@@ -122,16 +131,20 @@ class AbstractADBAPIDatabase(object):
                     if (version != self._db_version()) or (dbtype != self._db_type()):
 
                         if dbtype != self._db_type():
-                            log.error("Database %s has different type (%s vs. %s)"
-                                      % (self.dbID, dbtype, self._db_type()))
+                            log.error(
+                                "Database {db} has different type ({t1} vs. {t2})",
+                                db=self.dbID, t1=dbtype, t2=self._db_type()
+                            )
 
                             # Delete this index and start over
                             yield self._db_remove()
                             yield self._db_init()
 
                         elif version != self._db_version():
-                            log.error("Database %s has different schema (v.%s vs. v.%s)"
-                                      % (self.dbID, version, self._db_version()))
+                            log.error(
+                                "Database {db} has different schema (v.{v1} vs. v.{v2})",
+                                db=self.dbID, v1=version, v2=self._db_version()
+                            )
 
                             # Upgrade the DB
                             yield self._db_upgrade(version)
@@ -145,18 +158,16 @@ class AbstractADBAPIDatabase(object):
                 self.pool = None
                 raise
 
-
     def close(self):
 
         if self.initialized:
             try:
                 self.pool.close()
             except Exception, e:
-                log.error("Error whilst closing connection pool: %s" % (e,))
+                log.error("Error whilst closing connection pool: {ex}", ex=e)
             self.pool = None
             self.initialized = False
 
-
     @inlineCallbacks
     def clean(self):
 
@@ -168,12 +179,11 @@ class AbstractADBAPIDatabase(object):
             try:
                 yield self._db_empty_data_tables()
             except Exception, e:
-                log.error("Error in database clean: %s" % (e,))
+                log.error("Error in database clean: {ex}", ex=e)
                 self.close()
             else:
                 break
 
-
     @inlineCallbacks
     def execute(self, sql, *query_params):
 
@@ -185,12 +195,11 @@ class AbstractADBAPIDatabase(object):
             try:
                 yield self._db_execute(sql, *query_params)
             except Exception, e:
-                log.error("Error in database execute: %s" % (e,))
+                log.error("Error in database execute: {ex}", ex=e)
                 self.close()
             else:
                 break
 
-
     @inlineCallbacks
     def executescript(self, script):
 
@@ -202,12 +211,11 @@ class AbstractADBAPIDatabase(object):
             try:
                 yield self._db_execute_script(script)
             except Exception, e:
-                log.error("Error in database executescript: %s" % (e,))
+                log.error("Error in database executescript: {ex}", ex=e)
                 self.close()
             else:
                 break
 
-
     @inlineCallbacks
     def query(self, sql, *query_params):
 
@@ -219,14 +227,13 @@ class AbstractADBAPIDatabase(object):
             try:
                 result = (yield self._db_all_values_for_sql(sql, *query_params))
             except Exception, e:
-                log.error("Error in database query: %s" % (e,))
+                log.error("Error in database query: {ex}", ex=e)
                 self.close()
             else:
                 break
 
         returnValue(result)
 
-
     @inlineCallbacks
     def queryList(self, sql, *query_params):
 
@@ -238,14 +245,13 @@ class AbstractADBAPIDatabase(object):
             try:
                 result = (yield self._db_values_for_sql(sql, *query_params))
             except Exception, e:
-                log.error("Error in database queryList: %s" % (e,))
+                log.error("Error in database queryList: {ex}", ex=e)
                 self.close()
             else:
                 break
 
         returnValue(result)
 
-
     @inlineCallbacks
     def queryOne(self, sql, *query_params):
 
@@ -257,38 +263,34 @@ class AbstractADBAPIDatabase(object):
             try:
                 result = (yield self._db_value_for_sql(sql, *query_params))
             except Exception, e:
-                log.error("Error in database queryOne: %s" % (e,))
+                log.error("Error in database queryOne: {ex}", ex=e)
                 self.close()
             else:
                 break
 
         returnValue(result)
 
-
     def _db_version(self):
         """
         @return: the schema version assigned to this DB.
         """
         raise NotImplementedError
 
-
     def _db_type(self):
         """
         @return: the collection type assigned to this DB.
         """
         raise NotImplementedError
 
-
     def _test_schema_table(self):
         return self._test_table("CALDAV")
 
-
     @inlineCallbacks
     def _db_init(self):
         """
         Initialise the underlying database tables.
         """
-        log.info("Initializing database %s" % (self.dbID,))
+        log.info("Initializing database {db}", db=self.dbID)
 
         # TODO we need an exclusive lock of some kind here to prevent a race condition
         # in which multiple processes try to create the tables.
@@ -297,7 +299,6 @@ class AbstractADBAPIDatabase(object):
         yield self._db_init_data_tables()
         yield self._db_recreate()
 
-
     @inlineCallbacks
     def _db_init_schema_table(self):
         """
@@ -327,14 +328,12 @@ class AbstractADBAPIDatabase(object):
             """, (self._db_type(),)
         )
 
-
     def _db_init_data_tables(self):
         """
         Initialise the underlying database tables.
         """
         raise NotImplementedError
 
-
     def _db_empty_data_tables(self):
         """
         Delete the database tables.
@@ -343,7 +342,6 @@ class AbstractADBAPIDatabase(object):
         # Implementations can override this to re-create data
         pass
 
-
     def _db_recreate(self):
         """
         Recreate the database tables.
@@ -352,7 +350,6 @@ class AbstractADBAPIDatabase(object):
         # Implementations can override this to re-create data
         pass
 
-
     @inlineCallbacks
     def _db_upgrade(self, old_version):
         """
@@ -368,7 +365,6 @@ class AbstractADBAPIDatabase(object):
             yield self._db_remove()
             yield self._db_init()
 
-
     def _db_upgrade_data_tables(self, old_version):
         """
         Upgrade the data from an older version of the DB.
@@ -377,7 +373,6 @@ class AbstractADBAPIDatabase(object):
         # cannot be thrown away.
         raise NotImplementedError("Persistent databases MUST support an upgrade method.")
 
-
     @inlineCallbacks
     def _db_upgrade_schema(self):
         """
@@ -385,7 +380,6 @@ class AbstractADBAPIDatabase(object):
         """
         yield self._db_execute("insert or replace into CALDAV (KEY, VALUE) values ('SCHEMA_VERSION', :1)", (self._db_version(),))
 
-
     @inlineCallbacks
     def _db_remove(self):
         """
@@ -394,14 +388,12 @@ class AbstractADBAPIDatabase(object):
         yield self._db_remove_data_tables()
         yield self._db_remove_schema()
 
-
     def _db_remove_data_tables(self):
         """
         Remove all the data from an older version of the DB.
         """
         raise NotImplementedError("Each database must remove its own tables.")
 
-
     @inlineCallbacks
     def _db_remove_schema(self):
         """
@@ -409,7 +401,6 @@ class AbstractADBAPIDatabase(object):
         """
         yield self._db_execute("drop table if exists CALDAV")
 
-
     @inlineCallbacks
     def _db_all_values_for_sql(self, sql, *query_params):
         """
@@ -425,7 +416,6 @@ class AbstractADBAPIDatabase(object):
         results = (yield self.pool.runQuery(sql, *query_params))
         returnValue(tuple(results))
 
-
     @inlineCallbacks
     def _db_values_for_sql(self, sql, *query_params):
         """
@@ -442,7 +432,6 @@ class AbstractADBAPIDatabase(object):
         results = (yield self.pool.runQuery(sql, *query_params))
         returnValue(tuple([row[0] for row in results]))
 
-
     @inlineCallbacks
     def _db_value_for_sql(self, sql, *query_params):
         """
@@ -460,7 +449,6 @@ class AbstractADBAPIDatabase(object):
             value = row
         returnValue(value)
 
-
     def _db_execute(self, sql, *query_params):
         """
         Execute an SQL operation that returns None.
@@ -489,27 +477,23 @@ class AbstractADBAPIDatabase(object):
 
     The " unique" modifier can be appended to any of those.
     """
+
     def _map_column_types(self, type):
         raise NotImplementedError
 
-
     def _create_table(self, name, columns, ifnotexists=False):
         raise NotImplementedError
 
-
     def _test_table(self, name):
         raise NotImplementedError
 
-
     def _create_index(self, name, ontable, columns, ifnotexists=False):
         raise NotImplementedError
 
-
     def _prepare_statement(self, sql):
         raise NotImplementedError
 
 
-
 class ADBAPISqliteMixin(object):
 
     @classmethod
@@ -533,7 +517,6 @@ class ADBAPISqliteMixin(object):
 
         return result
 
-
     @inlineCallbacks
     def _create_table(self, name, columns, ifnotexists=False):
 
@@ -545,7 +528,6 @@ class ADBAPISqliteMixin(object):
         )
         yield self._db_execute(statement)
 
-
     @inlineCallbacks
     def _test_table(self, name):
         result = (yield self._db_value_for_sql("""
@@ -554,7 +536,6 @@ class ADBAPISqliteMixin(object):
         """ % (name,)))
         returnValue(result)
 
-
     @inlineCallbacks
     def _create_index(self, name, ontable, columns, ifnotexists=False):
 
@@ -566,7 +547,6 @@ class ADBAPISqliteMixin(object):
         )
         yield self._db_execute(statement)
 
-
     def _prepare_statement(self, sql):
         # We are going to use the sqlite syntax of :1, :2 etc for our
         # internal statements so we do not need to remap those
@@ -597,7 +577,6 @@ if postgres:
 
             return result
 
-
         @inlineCallbacks
         def _create_table(self, name, columns, ifnotexists=False):
 
@@ -618,7 +597,6 @@ if postgres:
                 if not result:
                     raise
 
-
         @inlineCallbacks
         def _test_table(self, name):
             result = (yield self._db_value_for_sql("""
@@ -627,7 +605,6 @@ if postgres:
             """ % (name.lower(),)))
             returnValue(result)
 
-
         @inlineCallbacks
         def _create_index(self, name, ontable, columns, ifnotexists=False):
 
@@ -648,7 +625,6 @@ if postgres:
                 if not result:
                     raise
 
-
         @inlineCallbacks
         def _db_init_schema_table(self):
             """
@@ -681,7 +657,6 @@ if postgres:
             except postgres.DatabaseError:
                 pass
 
-
         def _prepare_statement(self, sql):
             # Convert :1, :2 etc format into %s
             ctr = 1
