@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -92,7 +92,6 @@ class AugmentDB(object):
 
         self.cachedRecords = {}
 
-
     @inlineCallbacks
     def normalizeUUIDs(self):
         """
@@ -118,7 +117,6 @@ class AugmentDB(object):
             # It's OK if we can't re-write the file.
             pass
 
-
     @inlineCallbacks
     def getAugmentRecord(self, uid, recordType):
         """
@@ -168,7 +166,6 @@ class AugmentDB(object):
         result.clonedFromDefault = True
         returnValue(result)
 
-
     @inlineCallbacks
     def getAllUIDs(self):
         """
@@ -179,7 +176,6 @@ class AugmentDB(object):
 
         raise NotImplementedError("Child class must define this.")
 
-
     def _lookupAugmentRecord(self, uid):
         """
         Get an AugmentRecord for the specified UID.
@@ -192,7 +188,6 @@ class AugmentDB(object):
 
         raise NotImplementedError("Child class must define this.")
 
-
     @inlineCallbacks
     def _cachedAugmentRecord(self, uid):
         """
@@ -209,7 +204,6 @@ class AugmentDB(object):
             self.cachedRecords[uid] = result
         returnValue(self.cachedRecords[uid])
 
-
     def addAugmentRecords(self, records):
         """
         Add an AugmentRecord to the DB.
@@ -222,7 +216,6 @@ class AugmentDB(object):
 
         raise NotImplementedError("Child class must define this.")
 
-
     def removeAugmentRecords(self, uids):
         """
         Remove AugmentRecords with the specified UIDs.
@@ -235,7 +228,6 @@ class AugmentDB(object):
 
         raise NotImplementedError("Child class must define this.")
 
-
     def refresh(self):
         """
         Refresh any cached data.
@@ -246,7 +238,6 @@ class AugmentDB(object):
         self.cachedRecords.clear()
         return None
 
-
     def clean(self):
         """
         Remove all records.
@@ -257,7 +248,6 @@ class AugmentDB(object):
         raise NotImplementedError("Child class must define this.")
 
 
-
 class AugmentXMLDB(AugmentDB):
     """
     XMLFile based augment database implementation.
@@ -284,7 +274,6 @@ class AugmentXMLDB(AugmentDB):
         self.lastCached = time.time()
         self.normalizeUUIDs()
 
-
     def getAllUIDs(self):
         """
         Get all AugmentRecord UIDs.
@@ -293,7 +282,6 @@ class AugmentXMLDB(AugmentDB):
         """
         return succeed(self.db.keys())
 
-
     def _lookupAugmentRecord(self, uid):
         """
         Get an AugmentRecord for the specified UID.
@@ -310,7 +298,6 @@ class AugmentXMLDB(AugmentDB):
 
         return succeed(self.db.get(uid))
 
-
     def addAugmentRecords(self, records):
         """
         Add an AugmentRecord to the DB.
@@ -343,7 +330,6 @@ class AugmentXMLDB(AugmentDB):
 
         return succeed(None)
 
-
     def _doAddToFile(self, xmlfile, records):
 
         if not os.path.exists(xmlfile):
@@ -363,13 +349,13 @@ class AugmentXMLDB(AugmentDB):
                 try:
                     uid = pwd.getpwnam(config.UserName).pw_uid
                 except KeyError:
-                    log.error("User not found: %s" % (config.UserName,))
+                    log.error("User not found: {user}", user=config.UserName)
             gid = -1
             if config.GroupName:
                 try:
                     gid = grp.getgrnam(config.GroupName).gr_gid
                 except KeyError:
-                    log.error("Group not found: %s" % (config.GroupName,))
+                    log.error("Group not found: {grp}", grp=config.GroupName)
             if uid != -1 and gid != -1:
                 os.chown(xmlfile, uid, gid)
 
@@ -382,7 +368,6 @@ class AugmentXMLDB(AugmentDB):
         # Modify xmlfile
         writeXML(xmlfile, augments_node)
 
-
     def _doModifyInFile(self, xmlfile, records):
 
         if not os.path.exists(xmlfile):
@@ -411,7 +396,6 @@ class AugmentXMLDB(AugmentDB):
         if changed:
             writeXML(xmlfile, augments_node)
 
-
     def removeAugmentRecords(self, uids):
         """
         Remove AugmentRecords with the specified UIDs.
@@ -435,7 +419,6 @@ class AugmentXMLDB(AugmentDB):
 
         return succeed(None)
 
-
     def _doRemoveFromFile(self, xmlfile, uids):
 
         try:
@@ -458,12 +441,10 @@ class AugmentXMLDB(AugmentDB):
         if changed:
             writeXML(xmlfile, augments_node)
 
-
     def _addRecordToXMLDB(self, record, parentNode):
         record_node = addSubElement(parentNode, xmlaugmentsparser.ELEMENT_RECORD)
         self._updateRecordInXMLDB(record, record_node)
 
-
     def _updateRecordInXMLDB(self, record, recordNode):
         del recordNode[:]
         addSubElement(recordNode, xmlaugmentsparser.ELEMENT_UID, record.uid)
@@ -477,7 +458,6 @@ class AugmentXMLDB(AugmentDB):
         if record.autoAcceptGroup:
             addSubElement(recordNode, xmlaugmentsparser.ELEMENT_AUTOACCEPTGROUP, record.autoAcceptGroup)
 
-
     def refresh(self):
         """
         Refresh any cached data.
@@ -494,7 +474,6 @@ class AugmentXMLDB(AugmentDB):
 
         return None
 
-
     def clean(self):
         """
         Remove all records.
@@ -502,7 +481,6 @@ class AugmentXMLDB(AugmentDB):
 
         return self.removeAugmentRecords(self.db.keys())
 
-
     def _shouldReparse(self, xmlFiles):
         """
         Check to see whether any of the given files have been modified since
@@ -517,7 +495,6 @@ class AugmentXMLDB(AugmentDB):
                     return True
         return False
 
-
     def _parseXML(self):
         """
         Parse self.xmlFiles into AugmentRecords.
