@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.directory.test.test_proxyprincipalmembers -*-
 ##
-# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,9 +32,13 @@ import uuid
 
 from twext.python.log import Logger
 from twext.who.idirectory import RecordType as BaseRecordType
+
 from twisted.internet.defer import succeed, inlineCallbacks, returnValue
 from twisted.python.modules import getModule
 from twisted.web.template import XMLFile, Element, renderer
+
+from twistedcaldav import customxml
+from twistedcaldav.customxml import calendarserver_namespace
 from twistedcaldav.config import config, fullServerPath
 from twistedcaldav.database import (
     AbstractADBAPIDatabase, ADBAPISqliteMixin, ADBAPIPostgreSQLMixin
@@ -43,7 +47,6 @@ from twistedcaldav.directory.util import normalizeUUID
 from twistedcaldav.directory.util import (
     formatLink, formatLinks, formatPrincipals
 )
-
 from twistedcaldav.extensions import (
     DAVPrincipalResource, DAVResourceWithChildrenMixin
 )
@@ -51,9 +54,11 @@ from twistedcaldav.extensions import DirectoryElement
 from twistedcaldav.extensions import ReadOnlyWritePropertiesResourceMixIn
 from twistedcaldav.memcacher import Memcacher
 from twistedcaldav.resource import CalDAVComplianceMixIn
+
 from txdav.who.delegates import RecordType as DelegateRecordType
 from txdav.xml import element as davxml
 from txdav.xml.base import dav_namespace
+
 from txweb2 import responsecode
 from txweb2.dav.noneprops import NonePropertyStore
 from txweb2.dav.util import joinURL
@@ -64,6 +69,7 @@ log = Logger()
 
 
 class PermissionsMixIn (ReadOnlyWritePropertiesResourceMixIn):
+
     def defaultAccessControlList(self):
         aces = (
             # DAV:read access for authenticated users.
@@ -91,14 +97,12 @@ class PermissionsMixIn (ReadOnlyWritePropertiesResourceMixIn):
 
         return succeed(davxml.ACL(*aces))
 
-
     def accessControlList(self, request, inheritance=True, expanding=False,
                           inherited_aces=None):
         # Permissions here are fixed, and are not subject to inheritance rules, etc.
         return self.defaultAccessControlList()
 
 
-
 class ProxyPrincipalDetailElement(Element):
     """
     A L{ProxyPrincipalDetailElement} is an L{Element} that can render the
@@ -113,7 +117,6 @@ class ProxyPrincipalDetailElement(Element):
         super(ProxyPrincipalDetailElement, self).__init__()
         self.resource = resource
 
-
     @renderer
     def principal(self, request, tag):
         """
@@ -152,7 +155,6 @@ class ProxyPrincipalDetailElement(Element):
         )
 
 
-
 class ProxyPrincipalElement(DirectoryElement):
     """
     L{ProxyPrincipalElement} is a renderer for a
@@ -167,7 +169,6 @@ class ProxyPrincipalElement(DirectoryElement):
         return ProxyPrincipalDetailElement(self.resource)
 
 
-
 class CalendarUserProxyPrincipalResource (
         CalDAVComplianceMixIn, PermissionsMixIn, DAVResourceWithChildrenMixin,
         DAVPrincipalResource):
@@ -207,15 +208,12 @@ class CalendarUserProxyPrincipalResource (
             if url.startswith("/")
         )
 
-
     def __repr__(self):
         return "<{}: {}>".format(self.__class__.__name__, str(self))
 
-
     def __str__(self):
         return "{} [{}]".format(self.parent, self.proxyType)
 
-
     def _index(self):
         """
         Return the SQL database for this group principal.
@@ -224,7 +222,6 @@ class CalendarUserProxyPrincipalResource (
         """
         return ProxyDBService
 
-
     def resourceType(self):
         if self.proxyType == "calendar-proxy-read":
             return davxml.ResourceType.calendarproxyread  # @UndefinedVariable
@@ -237,7 +234,6 @@ class CalendarUserProxyPrincipalResource (
         else:
             return super(CalendarUserProxyPrincipalResource, self).resourceType()
 
-
     def isProxyType(self, read_write):
         if (
             read_write and self.proxyType == "calendar-proxy-write" or
@@ -247,20 +243,42 @@ class CalendarUserProxyPrincipalResource (
         else:
             return False
 
-
     def isCollection(self):
         return True
 
-
     def etag(self):
         return succeed(None)
 
-
     def deadProperties(self):
         if not hasattr(self, "_dead_properties"):
             self._dead_properties = NonePropertyStore(self)
         return self._dead_properties
 
+    @inlineCallbacks
+    def readProperty(self, property, request):
+        if type(property) is tuple:
+            qname = property
+        else:
+            qname = property.qname()
+
+        namespace, name = qname
+
+        if namespace == calendarserver_namespace:
+            if name == "record-type":
+                if hasattr(self.parent, "record"):
+                    returnValue(
+                        customxml.RecordType(
+                            self._recordTypeFromProxyType().description
+                        )
+                    )
+                else:
+                    raise HTTPError(StatusResponse(
+                        responsecode.NOT_FOUND,
+                        "Property %s does not exist." % (qname,)
+                    ))
+
+        result = (yield super(CalendarUserProxyPrincipalResource, self).readProperty(property, request))
+        returnValue(result)
 
     def writeProperty(self, property, request):
         assert isinstance(property, davxml.WebDAVElement)
@@ -271,7 +289,6 @@ class CalendarUserProxyPrincipalResource (
         return super(CalendarUserProxyPrincipalResource, self).writeProperty(
             property, request)
 
-
     @inlineCallbacks
     def setGroupMemberSet(self, new_members, request):
         # FIXME: as defined right now it is not possible to specify a
@@ -332,7 +349,6 @@ class CalendarUserProxyPrincipalResource (
 
         returnValue(True)
 
-
     @inlineCallbacks
     def setGroupMemberSetPrincipals(self, principals):
 
@@ -345,7 +361,6 @@ class CalendarUserProxyPrincipalResource (
         memberRecords = [p.record for p in principals]
         yield record.setMembers(memberRecords)
 
-
     ##
     # HTTP
     ##
@@ -356,7 +371,6 @@ class CalendarUserProxyPrincipalResource (
         """
         return ProxyPrincipalElement(self)
 
-
     ##
     # DAV
     ##
@@ -364,7 +378,6 @@ class CalendarUserProxyPrincipalResource (
     def displayName(self):
         return self.proxyType
 
-
     ##
     # ACL
     ##
@@ -373,19 +386,15 @@ class CalendarUserProxyPrincipalResource (
         # FIXME: Add API to IDirectoryRecord for getting a record URI?
         return self._alternate_urls
 
-
     def principalURL(self):
         return self._url
 
-
     def principalUID(self):
         return self.uid
 
-
     def principalCollections(self):
         return self.parent.principalCollections()
 
-
     def _recordTypeFromProxyType(self):
         return {
             "calendar-proxy-read": DelegateRecordType.readDelegateGroup,
@@ -394,7 +403,6 @@ class CalendarUserProxyPrincipalResource (
             "calendar-proxy-write-for": DelegateRecordType.writeDelegatorGroup,
         }.get(self.proxyType)
 
-
     @inlineCallbacks
     def _directGroupMembers(self, expanded=False):
         """
@@ -419,11 +427,9 @@ class CalendarUserProxyPrincipalResource (
                         memberPrincipals.append(principal)
         returnValue(memberPrincipals)
 
-
     def groupMembers(self):
         return self._directGroupMembers(expanded=False)
 
-
     def expandedGroupMembers(self):
         """
         Return the complete, flattened set of principals belonging to this
@@ -431,12 +437,11 @@ class CalendarUserProxyPrincipalResource (
         """
         return self._directGroupMembers(expanded=True)
 
-
     def groupMemberships(self):
         # Unlikely to ever want to put a subprincipal into a group
         return succeed([])
 
-
+    @inlineCallbacks
     def containsPrincipal(self, principal):
         """
         Optimize this to ignore the owner principal when doing this test. This is a common occurrence as the owner
@@ -450,9 +455,18 @@ class CalendarUserProxyPrincipalResource (
         """
 
         if principal == self.parent:
-            return succeed(False)
-        return super(CalendarUserProxyPrincipalResource, self).containsPrincipal(principal)
+            returnValue(False)
 
+        # Find our pseudo-record
+        record = yield self.parent.record.service.recordWithShortName(
+            self._recordTypeFromProxyType(),
+            self.parent.principalUID()
+        )
+        if record is not None:
+            result = yield record.containsUID(principal.principalUID())
+        else:
+            result = yield super(CalendarUserProxyPrincipalResource, self).containsPrincipal(principal)
+        returnValue(result)
 
 
 class ProxyDB(AbstractADBAPIDatabase):
@@ -511,13 +525,11 @@ class ProxyDB(AbstractADBAPIDatabase):
         def deleteMembership(self, guid):
             return self.delete("memberships:%s" % (str(guid),))
 
-
     def __init__(self, dbID, dbapiName, dbapiArgs, **kwargs):
         AbstractADBAPIDatabase.__init__(self, dbID, dbapiName, dbapiArgs, True, **kwargs)
 
         self._memcacher = ProxyDB.ProxyDBMemcacher("ProxyDB")
 
-
     @inlineCallbacks
     def setGroupMembers(self, principalUID, members):
         """
@@ -545,7 +557,6 @@ class ProxyDB(AbstractADBAPIDatabase):
             yield self._memcacher.deleteMembership(member)
         yield self._memcacher.deleteMember(principalUID)
 
-
     @inlineCallbacks
     def setGroupMembersInDatabase(self, principalUID, members):
         """
@@ -558,7 +569,6 @@ class ProxyDB(AbstractADBAPIDatabase):
         yield self._delete_from_db(principalUID)
         yield self._add_to_db(principalUID, members)
 
-
     @inlineCallbacks
     def changeGroupMembersInDatabase(self, principalUID, addMembers, removeMembers):
         """
@@ -574,7 +584,6 @@ class ProxyDB(AbstractADBAPIDatabase):
         for member in addMembers:
             yield self._add_to_db_one(principalUID, member)
 
-
     @inlineCallbacks
     def removeGroup(self, principalUID):
         """
@@ -594,7 +603,6 @@ class ProxyDB(AbstractADBAPIDatabase):
                 yield self._memcacher.deleteMembership(member)
             yield self._memcacher.deleteMember(principalUID)
 
-
     def getMembers(self, principalUID):
         """
         Return the list of group member UIDs for the specified principal.
@@ -620,7 +628,6 @@ class ProxyDB(AbstractADBAPIDatabase):
         d.addCallback(gotCachedMembers)
         return d
 
-
     def getMemberships(self, principalUID):
         """
         Return the list of group principal UIDs the specified principal is a member of.
@@ -646,7 +653,6 @@ class ProxyDB(AbstractADBAPIDatabase):
         d.addCallback(gotCachedMemberships)
         return d
 
-
     @inlineCallbacks
     def _add_to_db(self, principalUID, members):
         """
@@ -663,7 +669,6 @@ class ProxyDB(AbstractADBAPIDatabase):
                 """, (principalUID.decode("utf-8"), member,)
             )
 
-
     def _add_to_db_one(self, principalUID, memberUID):
         """
         Insert the specified entry into the database.
@@ -678,7 +683,6 @@ class ProxyDB(AbstractADBAPIDatabase):
             """, (principalUID.decode("utf-8"), memberUID.decode("utf-8"),)
         )
 
-
     def _delete_from_db(self, principalUID):
         """
         Deletes the specified entry from the database.
@@ -687,7 +691,6 @@ class ProxyDB(AbstractADBAPIDatabase):
         """
         return self.execute("delete from GROUPS where GROUPNAME = :1", (principalUID.decode("utf-8"),))
 
-
     def _delete_from_db_one(self, principalUID, memberUID):
         """
         Deletes the specified entry from the database.
@@ -697,7 +700,6 @@ class ProxyDB(AbstractADBAPIDatabase):
         """
         return self.execute("delete from GROUPS where GROUPNAME = :1 and MEMBER = :2", (principalUID.decode("utf-8"), memberUID.decode("utf-8"),))
 
-
     def _delete_from_db_member(self, principalUID):
         """
         Deletes the specified member entry from the database.
@@ -706,21 +708,18 @@ class ProxyDB(AbstractADBAPIDatabase):
         """
         return self.execute("delete from GROUPS where MEMBER = :1", (principalUID.decode("utf-8"),))
 
-
     def _db_version(self):
         """
         @return: the schema version assigned to this index.
         """
         return ProxyDB.schema_version
 
-
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         return ProxyDB.schema_type
 
-
     @inlineCallbacks
     def _db_init_data_tables(self):
         """
@@ -753,7 +752,6 @@ class ProxyDB(AbstractADBAPIDatabase):
             ifnotexists=True,
         )
 
-
     @inlineCallbacks
     def open(self):
         """
@@ -763,7 +761,6 @@ class ProxyDB(AbstractADBAPIDatabase):
         yield self._maybeNormalizeUUIDs()
         returnValue(result)
 
-
     @inlineCallbacks
     def _maybeNormalizeUUIDs(self):
         """
@@ -795,7 +792,6 @@ class ProxyDB(AbstractADBAPIDatabase):
                 """
             )
 
-
     @inlineCallbacks
     def _db_upgrade_data_tables(self, old_version):
         """
@@ -819,7 +815,6 @@ class ProxyDB(AbstractADBAPIDatabase):
                 ifnotexists=True,
             )
 
-
     def _db_empty_data_tables(self):
         """
         Empty the underlying database tables.
@@ -831,7 +826,6 @@ class ProxyDB(AbstractADBAPIDatabase):
         #
         return self._db_execute("delete from GROUPS")
 
-
     @inlineCallbacks
     def clean(self):
 
@@ -843,7 +837,6 @@ class ProxyDB(AbstractADBAPIDatabase):
 
         yield super(ProxyDB, self).clean()
 
-
     @inlineCallbacks
     def getAllMembers(self):
         """
@@ -866,7 +859,6 @@ class ProxySqliteDB(ADBAPISqliteMixin, ProxyDB):
         ProxyDB.__init__(self, "Proxies", "sqlite3", (self.dbpath,))
 
 
-
 class ProxyPostgreSQLDB(ADBAPIPostgreSQLMixin, ProxyDB):
     """
     PostgreSQL based augment database implementation.
@@ -883,7 +875,6 @@ class ProxyPostgreSQLDB(ADBAPIPostgreSQLMixin, ProxyDB):
         if dbtype:
             ProxyDB.schema_type = dbtype
 
-
     def _maybeNormalizeUUIDs(self):
         """
         Don't bother normalizing UUIDs for postgres yet; users of postgres
@@ -893,7 +884,6 @@ class ProxyPostgreSQLDB(ADBAPIPostgreSQLMixin, ProxyDB):
         return succeed(None)
 
 
-
 ##
 # Utilities
 ##
