@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_wrapping,twistedcaldav.directory.test.test_calendar -*-
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -61,7 +61,6 @@ class CommonUIDProvisioningResource(object):
         self.directory = parent.directory
         self.parent = parent
 
-
     @inlineCallbacks
     def homeResourceForRecord(self, record, request):
 
@@ -69,12 +68,14 @@ class CommonUIDProvisioningResource(object):
         name = record.uid
 
         if record is None:
-            log.debug("No directory record with UID %r" % (name,))
+            log.debug("No directory record with UID {name}", name=name)
             returnValue(None)
 
         if not getattr(record, self.enabledAttribute, False):
-            log.debug("Directory record %r is not enabled for %s" % (
-                record, self.homeResourceTypeName))
+            log.debug(
+                "Directory record {rec!r} is not enabled for {type}",
+                rec=record, type=self.homeResourceTypeName
+            )
             returnValue(None)
 
         assert len(name) > 4, "Directory record has an invalid GUID: %r" % (
@@ -87,7 +88,6 @@ class CommonUIDProvisioningResource(object):
 
         returnValue(child)
 
-
     @inlineCallbacks
     def locateChild(self, request, segments):
         name = segments[0]
@@ -103,12 +103,10 @@ class CommonUIDProvisioningResource(object):
         child = yield self.homeResourceForRecord(record, request)
         returnValue((child, segments[1:]))
 
-
     def listChildren(self):
         # Not a listable collection
         raise HTTPError(responsecode.FORBIDDEN)
 
-
     ##
     # ACL
     ##
@@ -116,11 +114,9 @@ class CommonUIDProvisioningResource(object):
     def principalCollections(self):
         return self.parent.principalCollections()
 
-
     def principalForRecord(self, record):
         return self.parent.principalForRecord(record)
 
-
     ##
     # DAV
     ##
@@ -128,21 +124,17 @@ class CommonUIDProvisioningResource(object):
     def isCollection(self):
         return True
 
-
     def getChild(self, name, record=None):
         raise NotImplementedError(self.__class__.__name__ +
                                   ".getChild no longer exists.")
 
-
     def displayName(self):
         return uidsResourceName
 
-
     def url(self):
         return joinURL(self.parent.url(), uidsResourceName)
 
 
-
 class CommonHomeTypeProvisioningResource(object):
 
     @inlineCallbacks
