@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.directory.test.test_util -*-
 ##
-# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -54,7 +54,6 @@ def uuidFromName(namespace, name):
     return normalizeUUID(str(uuid5(UUID(namespace), name)))
 
 
-
 def normalizeUUID(value):
     """
     Convert strings which the uuid.UUID( ) method can parse into normalized
@@ -116,7 +115,6 @@ def transactionFromRequest(request, newStore):
     return transaction
 
 
-
 def splitIntoBatches(data, size):
     """
     Return a generator of sets consisting of the contents of the data set
@@ -130,7 +128,6 @@ def splitIntoBatches(data, size):
         del data[:size]
 
 
-
 class NotFoundResource(DAVResource):
     """
     In order to prevent unauthenticated discovery of existing users via 401/404
@@ -159,7 +156,6 @@ class NotFoundResource(DAVResource):
             returnValue(response)
 
 
-
 def formatLink(url):
     """
     Convert a URL string into some twisted.web.template DOM objects for
@@ -168,7 +164,6 @@ def formatLink(url):
     return tags.a(href=url)(url)
 
 
-
 def formatLinks(urls):
     """
     Format a list of URL strings as a list of twisted.web.template DOM links.
@@ -176,7 +171,6 @@ def formatLinks(urls):
     return formatList(formatLink(link) for link in urls)
 
 
-
 def formatPrincipals(principals):
     """
     Format a list of principals into some twisted.web.template DOM objects.
@@ -195,7 +189,6 @@ def formatPrincipals(principals):
             shortName = u""
         return (record.recordType, shortName)
 
-
     def describe(principal):
         if hasattr(principal, "record"):
             return " - %s" % (principal.record.displayName,)
@@ -210,7 +203,6 @@ def formatPrincipals(principals):
     )
 
 
-
 def formatList(iterable):
     """
     Format a list of stuff as an interable.
@@ -227,7 +219,7 @@ def formatList(iterable):
                 yield item
             yield "\n"
     except Exception, e:
-        log.error("Exception while rendering: %s" % (e,))
+        log.error("Exception while rendering: {ex}", ex=e)
         Failure().printTraceback()
         yield "  ** %s **: %s\n" % (e.__class__.__name__, e)
     if not thereAreAny:
