@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,6 +56,7 @@ log = Logger()
 
 MatchFlags_none = MatchFlags.NOT & ~MatchFlags.NOT  # can't import MatchFlags_none
 
+
 class DirectoryBackedAddressBookResource (CalDAVResource):
     """
     Directory-backed address book
@@ -69,24 +70,22 @@ class DirectoryBackedAddressBookResource (CalDAVResource):
         self.uri = uri
         self.directory = None
 
-
     def makeChild(self, name):
         from twistedcaldav.simpleresource import SimpleCalDAVResource
         return SimpleCalDAVResource(principalCollections=self.principalCollections())
         return self.directory
 
-
     def provisionDirectory(self):
         if self.directory is None:
             log.info(
                 "Setting search directory to {principalDirectory}",
-                principalDirectory=self.principalDirectory)
+                principalDirectory=self.principalDirectory
+            )
             self.directory = self.principalDirectory
             # future: instantiate another directory based on /Search/Contacts (?)
 
         return succeed(None)
 
-
     def defaultAccessControlList(self):
         if config.AnonymousDirectoryAddressBookAccess:
             # DAV:Read for all principals (includes anonymous)
@@ -109,7 +108,6 @@ class DirectoryBackedAddressBookResource (CalDAVResource):
             )
         )
 
-
     def supportedReports(self):
         result = super(DirectoryBackedAddressBookResource, self).supportedReports()
         if config.EnableSyncReport:
@@ -117,11 +115,9 @@ class DirectoryBackedAddressBookResource (CalDAVResource):
             result.remove(davxml.Report(davxml.SyncCollection(),))
         return result
 
-
     def resourceType(self):
         return davxml.ResourceType.directory
 
-
     def resourceID(self):
         if self.directory:
             resource_id = uuid.uuid5(uuid.UUID("5AAD67BF-86DD-42D7-9161-6AF977E4DAA3"), self.directory.guid).urn
@@ -129,24 +125,19 @@ class DirectoryBackedAddressBookResource (CalDAVResource):
             resource_id = "tag:unknown"
         return resource_id
 
-
     def isDirectoryBackedAddressBookCollection(self):
         return True
 
-
     def isAddressBookCollection(self):
         return True
 
-
     def isCollection(self):
         return True
 
-
     def accessControlList(self, request, inheritance=True, expanding=False, inherited_aces=None):
         # Permissions here are fixed, and are not subject to inheritance rules, etc.
         return self.defaultAccessControlList()
 
-
     @inlineCallbacks
     def renderHTTP(self, request):
         if not self.directory:
@@ -155,15 +146,16 @@ class DirectoryBackedAddressBookResource (CalDAVResource):
         response = (yield maybeDeferred(super(DirectoryBackedAddressBookResource, self).renderHTTP, request))
         returnValue(response)
 
-
     @inlineCallbacks
     def doAddressBookDirectoryQuery(self, addressBookFilter, addressBookQuery, maxResults, defaultKind="individual"):
         """
         Get vCards for a given addressBookFilter and addressBookQuery
         """
 
-        log.debug("doAddressBookDirectoryQuery: directory={directory} addressBookFilter={addressBookFilter}, addressBookQuery={addressBookQuery}, maxResults={maxResults}",
-                  directory=self.directory, addressBookFilter=addressBookFilter, addressBookQuery=addressBookQuery, maxResults=maxResults)
+        log.debug(
+            "doAddressBookDirectoryQuery: directory={directory} addressBookFilter={addressBookFilter}, addressBookQuery={addressBookQuery}, maxResults={maxResults}",
+            directory=self.directory, addressBookFilter=addressBookFilter, addressBookQuery=addressBookQuery, maxResults=maxResults
+        )
         results = []
         limited = False
         maxQueryRecords = 0
@@ -232,8 +224,10 @@ class DirectoryBackedAddressBookResource (CalDAVResource):
                 allRecords = set()
                 if fields:
                     records = yield self.directory.recordsMatchingFields(fields, operand, queryRecordType)
-                    log.debug("doAddressBookDirectoryQuery: recordsMatchingFields({f}, {o}): #records={n}, records={records!r}",
-                              f=fields, o=operand, n=len(records), records=records)
+                    log.debug(
+                        "doAddressBookDirectoryQuery: recordsMatchingFields({f}, {o}): #records={n}, records={records!r}",
+                        f=fields, o=operand, n=len(records), records=records
+                    )
                     allRecords = set(records)
                 else:
                     recordTypes = set([queryRecordType]) if queryRecordType else set(self.directory.recordTypes()) & set(recordTypeToVCardKindMap.keys())
@@ -280,7 +274,6 @@ class DirectoryBackedAddressBookResource (CalDAVResource):
         returnValue((results, limited,))
 
 
-
 def propertiesInAddressBookQuery(addressBookQuery):
     """
     Get the vCard properties requested by a given query
@@ -303,7 +296,6 @@ def propertiesInAddressBookQuery(addressBookQuery):
     return (etagRequested, propertyNames if len(propertyNames) else None)
 
 
-
 def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, constantProperties={}):
     """
     Convert the supplied addressbook-query into a ds expression tree.
@@ -315,7 +307,6 @@ def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, con
     """
 
     def propFilterListQuery(filterAllOf, propFilters):
-
         """
         Create an expression for a list of prop-filter elements.
 
@@ -356,7 +347,6 @@ def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, con
                         expressionList.extend(addedExpressions)
             return expressionList
 
-
         def propFilterExpression(filterAllOf, propFilter):
             """
             Create an expression for a single prop-filter element.
@@ -382,9 +372,8 @@ def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, con
 
                 return MatchExpression(fieldName, matchValue, matchType, matchFlags)
 
-
             def definedExpression(defined, allOf):
-                if constant or propFilter.filter_name in ("N" , "FN", "UID", "KIND",):
+                if constant or propFilter.filter_name in ("N", "FN", "UID", "KIND",):
                     return defined  # all records have this property so no records do not have it
                 else:
                     # FIXME: The startsWith expression below, which works with LDAP and OD. is not currently supported
@@ -404,8 +393,7 @@ def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, con
                 else:
                     return matchList
 
-
-            def paramFilterElementExpression(propFilterAllOf, paramFilterElement): #@UnusedVariable
+            def paramFilterElementExpression(propFilterAllOf, paramFilterElement):  # @UnusedVariable
 
                 params = vCardPropToParamMap.get(propFilter.filter_name.upper())
                 defined = params and paramFilterElement.filter_name.upper() in params
@@ -422,7 +410,6 @@ def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, con
 
                 return True
 
-
             def textMatchElementExpression(propFilterAllOf, textMatchElement):
 
                 # preprocess text match strings for ds query
@@ -437,7 +424,7 @@ def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, con
                     elif propFilter.filter_name == "GEO":
                         matchString = ",".join(matchString.split(";"))
 
-                    if propFilter.filter_name in ("N" , "ADR", "ORG",):
+                    if propFilter.filter_name in ("N", "ADR", "ORG",):
                         # for structured properties, change into multiple strings for ds query
                         if propFilter.filter_name == "ADR":
                             # split by newline and comma
@@ -476,7 +463,7 @@ def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, con
                         # use match_type where possible depending on property/attribute mapping
                         # FIXME: case-sensitive negate will not work.  This should return all all records in that case
                         matchType = MatchType.contains
-                        if propFilter.filter_name in ("NICKNAME" , "TITLE" , "NOTE" , "UID", "URL", "N", "ADR", "ORG", "REV", "LABEL",):
+                        if propFilter.filter_name in ("NICKNAME", "TITLE", "NOTE", "UID", "URL", "N", "ADR", "ORG", "REV", "LABEL",):
                             if textMatchElement.match_type == "equals":
                                 matchType = MatchType.equals
                             elif textMatchElement.match_type == "starts-with":
@@ -590,7 +577,6 @@ def expressionFromABFilter(addressBookFilter, vcardPropToSearchableFieldMap, con
     return((properties, expression))
 
 
-
 class ABDirectoryQueryResult(DAVPropertyMixIn):
     """
     Result from ab query report or multiget on directory
@@ -601,7 +587,6 @@ class ABDirectoryQueryResult(DAVPropertyMixIn):
         self._directoryBackedAddressBook = directoryBackedAddressBook
         # self._vCard = None
 
-
     def __repr__(self):
         return "<{self.__class__.__name__}[{rn}({uid})]>".format(
             self=self,
@@ -623,23 +608,18 @@ class ABDirectoryQueryResult(DAVPropertyMixIn):
         self._vCard = yield vCardFromRecord(record, forceKind, addProps, None)
         returnValue(self)
 
-
     def vCard(self):
         return self._vCard
 
-
     def vCardText(self):
         return str(self._vCard)
 
-
     def uri(self):
         return self.vCard().propertyValue("UID") + ".vcf"
 
-
     def hRef(self, parentURI=None):
         return davxml.HRef.fromString(joinURL(parentURI if parentURI else self._directoryBackedAddressBook.uri, self.uri()))
 
-
     def readProperty(self, property, request):
 
         if type(property) is tuple:
@@ -650,7 +630,7 @@ class ABDirectoryQueryResult(DAVPropertyMixIn):
 
         if namespace == dav_namespace:
             if name == "resourcetype":
-                result = davxml.ResourceType.empty #@UndefinedVariable
+                result = davxml.ResourceType.empty  # @UndefinedVariable
                 return result
             elif name == "getetag":
                 result = davxml.GETETag(ETag(hashlib.md5(self.vCardText()).hexdigest()).generate())
@@ -691,7 +671,6 @@ class ABDirectoryQueryResult(DAVPropertyMixIn):
 
         return self._directoryBackedAddressBook.readProperty(property, request)
 
-
     def listProperties(self, request):  # @UnusedVariable
         qnames = set(self.liveProperties())
 
