@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -69,11 +69,9 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
 
         self.parent = parent
 
-
     def __repr__(self):
         return "<%s (free-busy URL resource): %s>" % (self.__class__.__name__, joinURL(self.parent.url(), "freebusy"))
 
-
     def defaultAccessControlList(self):
         privs = (
             davxml.Privilege(davxml.Read()),
@@ -103,27 +101,21 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
             )
         return succeed(davxml.ACL(*aces))
 
-
     def resourceType(self):
         return davxml.ResourceType.freebusyurl
 
-
     def contentType(self):
         return MimeType("text", "calendar", charset="utf-8")
 
-
     def isCollection(self):
         return False
 
-
     def isCalendarCollection(self):
         return False
 
-
     def isPseudoCalendarCollection(self):
         return False
 
-
     def render(self, request):
         output = """<html>
 <head>
@@ -138,21 +130,18 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
         response.headers.setHeader("content-type", MimeType("text", "html"))
         return response
 
-
     def http_GET(self, request):
         """
         The free-busy URL POST method.
         """
         return self._processFBURL(request)
 
-
     def http_POST(self, request):
         """
         The free-busy URL POST method.
         """
         return self._processFBURL(request)
 
-
     @inlineCallbacks
     def _processFBURL(self, request):
         #
@@ -260,7 +249,7 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
         try:
             inbox = (yield request.locateResource(inboxURL))
         except:
-            log.error("No schedule inbox for principal: %s" % (principal,))
+            log.error("No schedule inbox for principal: {p}", p=principal)
             inbox = None
         if inbox is None:
             raise HTTPError(StatusResponse(responsecode.INTERNAL_SERVER_ERROR, "No schedule inbox for principal: %s" % (principal,)))
@@ -283,7 +272,6 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
 
         returnValue(response)
 
-
     ##
     # ACL
     ##
