@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,9 +27,8 @@ from twext.python.log import Logger
 from pycalendar.duration import Duration
 
 try:
-    from Foundation import (
-        NSPropertyListImmutable, NSPropertyListSerialization, NSData, NSLocale
-    )
+    from osx.corefoundation import CFError, CFDataRef
+    from osx.utils import CFLocaleRef, CFPropertyListRef
     foundationImported = True
 except ImportError:
     foundationImported = False
@@ -125,7 +124,6 @@ class translationTo(object):
                 localedir=localeDir, languages=[lang, 'en'], fallback=True)
             self.translations[key] = self.translation
 
-
     def __enter__(self):
         # Get the caller's globals so we can rebind their '_' to our translator
         caller_globals = inspect.stack()[1][0].f_globals
@@ -140,7 +138,6 @@ class translationTo(object):
         # What we return here is accessible to the caller via the 'as' clause
         return self
 
-
     def __exit__(self, type, value, traceback):
         # Restore '_' if it previously had a value
         if hasattr(self, 'prev'):
@@ -149,16 +146,13 @@ class translationTo(object):
         # Don't swallow exceptions
         return False
 
-
     def monthAbbreviation(self, monthNumber):
         return self.translation.ugettext(monthsAbbrev[monthNumber])
 
-
     def date(self, component):
         dtStart = component.propertyValue("DTSTART")
         return self.dtDate(dtStart)
 
-
     def time(self, component):
         """
         Examples:
@@ -209,16 +203,15 @@ class translationTo(object):
         return (
             _("%(startTime)s to %(endTime)s")
             % {
-                'startTime' : self.dtTime(
+                'startTime': self.dtTime(
                     dtStart,
                     includeTimezone=(tzStart != tzEnd)
                 ),
-                'endTime' : self.dtTime(dtEnd),
+                'endTime': self.dtTime(dtEnd),
             },
             self.dtDuration(duration)
         )
 
-
     def dtDate(self, val):
         # Bind to '_' so pygettext.py will pick this up for translation
         _ = self.translation.ugettext
@@ -226,14 +219,13 @@ class translationTo(object):
         return (
             _("%(dayName)s, %(monthName)s %(dayNumber)d, %(yearNumber)d")
             % {
-                'dayName'    : _(daysFull[(val.getDayOfWeek() + 6) % 7]),
-                'monthName'  : _(monthsFull[val.getMonth()]),
-                'dayNumber'  : val.getDay(),
-                'yearNumber' : val.getYear(),
+                'dayName': _(daysFull[(val.getDayOfWeek() + 6) % 7]),
+                'monthName': _(monthsFull[val.getMonth()]),
+                'dayNumber': val.getDay(),
+                'yearNumber': val.getYear(),
             }
         )
 
-
     def dtTime(self, val, includeTimezone=True):
         if val.isDateOnly():
             return ""
@@ -249,10 +241,10 @@ class translationTo(object):
         result = (
             _("%(hour12Number)d:%(minuteNumber)02d %(ampm)s")
             % {
-                'hour24Number' : val.getHours(), # 0-23
-                'hour12Number' : hour12, # 1-12
-                'minuteNumber' : val.getMinutes(), # 0-59
-                'ampm'         : ampm,
+                'hour24Number': val.getHours(),  # 0-23
+                'hour12Number': hour12,  # 1-12
+                'minuteNumber': val.getMinutes(),  # 0-59
+                'ampm': ampm,
             }
         )
 
@@ -261,7 +253,6 @@ class translationTo(object):
 
         return result
 
-
     def dtDuration(self, val):
 
         # Bind to '_' so pygettext.py will pick this up for translation
@@ -276,7 +267,7 @@ class translationTo(object):
         elif days > 1:
             parts.append(
                 _("%(dayCount)d days") %
-                {'dayCount' : days}
+                {'dayCount': days}
             )
 
         hours = divmod(total / 3600, 24)[1]
@@ -288,7 +279,7 @@ class translationTo(object):
         elif hours > 1:
             parts.append(
                 _("%(hourCount)d hours") %
-                {'hourCount' : hours}
+                {'hourCount': hours}
             )
 
         if minutes == 1:
@@ -296,7 +287,7 @@ class translationTo(object):
         elif minutes > 1:
             parts.append(
                 _("%(minuteCount)d minutes") %
-                {'minuteCount' : minutes}
+                {'minuteCount': minutes}
             )
 
         if seconds == 1:
@@ -304,7 +295,7 @@ class translationTo(object):
         elif seconds > 1:
             parts.append(
                 _("%(secondCount)d seconds") %
-                {'secondCount' : seconds}
+                {'secondCount': seconds}
             )
 
         return " ".join(parts)
@@ -368,7 +359,6 @@ monthsAbbrev = [
 ]
 
 
-
 ##
 # String file conversion routines
 ##
@@ -384,7 +374,7 @@ def processLocalizationFiles(settings):
     # Do we have an Apple translations directory?
     if lprojRoot and gnuRoot and os.path.exists(lprojRoot):
 
-        log.info("Looking for Apple .lproj directories in %s" % (lprojRoot,))
+        log.info("Looking for Apple .lproj directories in {r}", r=lprojRoot)
 
         # Make sure the gnutext translations directory exists
         if not os.path.exists(gnuRoot):
@@ -392,8 +382,9 @@ def processLocalizationFiles(settings):
                 os.mkdir(gnuRoot)
             except OSError:
                 log.warn(
-                    "Could not create gnuttext translation directory: %s"
-                    % (gnuRoot,))
+                    "Could not create gnuttext translation directory: {r}",
+                    r=gnuRoot,
+                )
                 return
 
         # Scan for Apple translations (directories ending in .lproj)
@@ -414,24 +405,22 @@ def processLocalizationFiles(settings):
                         os.stat(stringsFile).st_mtime >
                         os.stat(moFile).st_mtime
                     ):
-                        log.info("Converting %s to %s" % (stringsFile, moFile))
+                        log.info("Converting {s} to {m}", s=stringsFile, m=moFile)
                         try:
                             convertStringsFile(stringsFile, moFile)
                         except Exception, e:
                             log.error(
-                                "Failed to convert %s to %s: %s" %
-                                (stringsFile, moFile, e)
+                                "Failed to convert {s} to {m}: {ex}",
+                                s=stringsFile, m=moFile, ex=e,
                             )
                     else:
-                        log.info("%s is up to date" % (moFile,))
-
+                        log.info("{m} is up to date", m=moFile)
 
 
 class ParseError(Exception):
     pass
 
 
-
 def convertStringsFile(src, dest):
 
     dir = os.path.dirname(dest)
@@ -444,16 +433,18 @@ def convertStringsFile(src, dest):
             return
 
     # Parse the binary plist .strings file:
-    parser = NSPropertyListSerialization.propertyListFromData_mutabilityOption_format_errorDescription_
-    data = NSData.dataWithContentsOfMappedFile_(src)
-    strings, format, error = parser(data, NSPropertyListImmutable, None, None)
-    if error:
+    with open(src) as f:
+        data = f.read()
+    data = CFDataRef.fromString(data)
+    try:
+        parsed = CFPropertyListRef.createFromData(data)
+        strings = parsed.toDict()
+    except CFError as error:
         raise ParseError(error)
 
     # The format of GNUtext MO files is described here:
     # http://www.gnu.org/software/autoconf/manual/gettext/MO-Files.html
 
-    strings = dict(strings)
     originals = strings.keys()
     originals.sort()
 
@@ -464,17 +455,19 @@ def convertStringsFile(src, dest):
     for original in originals:
         translation = strings[original]
 
-        origStr = original.encode("UTF-8")
-        transStr = translation.encode("UTF-8")
+        if isinstance(original, unicode):
+            original = original.encode("UTF-8")
+        if isinstance(translation, unicode):
+            translation = translation.encode("UTF-8")
 
         descriptors.append(
             (
-                len(keys), len(origStr), len(values),
-                len(transStr)
+                len(keys), len(original), len(values),
+                len(translation)
             )
         )
-        keys += origStr + '\0' # <NUL> terminated
-        values += transStr + '\0'
+        keys += original + '\0'  # <NUL> terminated
+        values += translation + '\0'
 
     # The header is 28 bytes, each descriptor is 8 bytes, with two descriptors
     # per string (one pointing at original, one pointing at translation)
@@ -495,7 +488,7 @@ def convertStringsFile(src, dest):
         0,                     # file format revision
         len(originals),        # number of strings
         28,                    # offset of table with original strings
-        28 + len(originals) * 8, # offset of table with translation strings
+        28 + len(originals) * 8,  # offset of table with translation strings
         0,                     # size of hashing table
         0                      # offset of hashing table
     )
@@ -508,7 +501,6 @@ def convertStringsFile(src, dest):
         outFile.write(result)
 
 
-
 def _remapLanguageCode(code):
     """
     Remap certain language codes to others, per the localization team
@@ -522,7 +514,6 @@ def _remapLanguageCode(code):
     return code
 
 
-
 def getLanguage(config):
     """
     If the language has been specified explicitly in the config, return it.  Otherwise
@@ -538,7 +529,7 @@ def getLanguage(config):
         return config.Localization.Language
 
     try:
-        language = NSLocale.preferredLanguages()[0]
+        language = CFLocaleRef.preferredLanguages()[0]
         language = _remapLanguageCode(language)
     except:
         language = "en"
