@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@ from twisted.internet.defer import inlineCallbacks, Deferred, returnValue, \
 from twisted.internet import reactor
 import time
 
+
 class MemcacheLock(Memcacher):
 
     def __init__(self, namespace, locktoken, timeout=5.0, retry_interval=0.1, expire_time=0):
@@ -44,7 +45,6 @@ class MemcacheLock(Memcacher):
         self._expire_time = expire_time
         self._hasLock = False
 
-
     def _getMemcacheProtocol(self):
 
         result = super(MemcacheLock, self)._getMemcacheProtocol()
@@ -54,7 +54,6 @@ class MemcacheLock(Memcacher):
 
         return result
 
-
     @inlineCallbacks
     def acquire(self):
 
@@ -68,24 +67,24 @@ class MemcacheLock(Memcacher):
             if result:
                 self._hasLock = True
                 if waiting:
-                    self.log.debug("Got lock after waiting on %s" % (self._locktoken,))
+                    self.log.debug("Got lock after waiting on {t}", t=self._locktoken)
                 break
 
             if self._timeout and time.time() < timeout_at:
                 waiting = True
-                self.log.debug("Waiting for lock on %s" % (self._locktoken,))
+                self.log.debug("Waiting for lock on {t}", t=self._locktoken)
                 pause = Deferred()
+
                 def _timedDeferred():
                     pause.callback(True)
                 reactor.callLater(self._retry_interval, _timedDeferred)
                 yield pause
             else:
-                self.log.debug("Timed out lock after waiting on %s" % (self._locktoken,))
+                self.log.debug("Timed out lock after waiting on {t}", t=self._locktoken)
                 raise MemcacheLockTimeoutError()
 
         returnValue(True)
 
-
     def release(self):
 
         assert self._hasLock, "Lock not acquired."
@@ -98,7 +97,6 @@ class MemcacheLock(Memcacher):
         d.addCallback(_done)
         return d
 
-
     def clean(self):
 
         if self._hasLock:
@@ -106,7 +104,6 @@ class MemcacheLock(Memcacher):
         else:
             return succeed(True)
 
-
     def locked(self):
         """
         Test if the lock is currently being held.
@@ -120,6 +117,5 @@ class MemcacheLock(Memcacher):
         return d
 
 
-
 class MemcacheLockTimeoutError(Exception):
     pass
