@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,7 +26,6 @@ from twext.internet.adaptendpoint import connect
 from twisted.internet.endpoints import UNIXClientEndpoint
 
 
-
 class PooledMemCacheProtocol(MemCacheProtocol):
     """
     A MemCacheProtocol that will notify a connectionPool that it is ready
@@ -47,7 +46,6 @@ class PooledMemCacheProtocol(MemCacheProtocol):
             self.factory.deferred = None
 
 
-
 class MemCacheClientFactory(ReconnectingClientFactory):
     """
     A client factory for MemCache that reconnects and notifies a pool of it's
@@ -64,11 +62,9 @@ class MemCacheClientFactory(ReconnectingClientFactory):
     connectionPool = None
     _protocolInstance = None
 
-
     def __init__(self):
         self.deferred = Deferred()
 
-
     def clientConnectionLost(self, connector, reason):
         """
         Notify the connectionPool that we've lost our connection.
@@ -78,7 +74,7 @@ class MemCacheClientFactory(ReconnectingClientFactory):
             # The reactor is stopping; don't reconnect
             return
 
-        self.log.error("MemCache connection lost: %s" % (reason,))
+        self.log.error("MemCache connection lost: {r}", r=reason)
         if self._protocolInstance is not None:
             self.connectionPool.clientBusy(self._protocolInstance)
 
@@ -87,12 +83,11 @@ class MemCacheClientFactory(ReconnectingClientFactory):
             connector,
             reason)
 
-
     def clientConnectionFailed(self, connector, reason):
         """
         Notify the connectionPool that we're unable to connect
         """
-        self.log.error("MemCache connection failed: %s" % (reason,))
+        self.log.error("MemCache connection failed: {r}", r=reason)
         if self._protocolInstance is not None:
             self.connectionPool.clientBusy(self._protocolInstance)
 
@@ -101,7 +96,6 @@ class MemCacheClientFactory(ReconnectingClientFactory):
             connector,
             reason)
 
-
     def buildProtocol(self, addr):
         """
         Attach the C{self.connectionPool} to the protocol so it can tell it,
@@ -115,7 +109,6 @@ class MemCacheClientFactory(ReconnectingClientFactory):
         return self._protocolInstance
 
 
-
 class MemCachePool(object):
     """
     A connection pool for MemCacheProtocol instances.
@@ -172,7 +165,6 @@ class MemCachePool(object):
         self._pendingConnects = 0
         self._commands = []
 
-
     def _isIdle(self):
         return (
             len(self._busyClients) == 0 and
@@ -180,7 +172,6 @@ class MemCachePool(object):
             self._pendingConnects == 0
         )
 
-
     def _shutdownCallback(self):
         self.shutdown_requested = True
         if self._isIdle():
@@ -188,7 +179,6 @@ class MemCachePool(object):
         self.shutdown_deferred = Deferred()
         return self.shutdown_deferred
 
-
     def _newClientConnection(self):
         """
         Create a new client connection.
@@ -196,7 +186,7 @@ class MemCachePool(object):
         @return: A L{Deferred} that fires with the L{IProtocol} instance.
         """
         self.log.debug(
-            "Initiating new client connection to: %r" % (self._endpoint,)
+            "Initiating new client connection to: {r!r}", r=self._endpoint
         )
         self._logClientStats()
 
@@ -218,7 +208,6 @@ class MemCachePool(object):
         d.addCallback(_connected)
         return d
 
-
     def _performRequestOnClient(self, client, command, *args, **kwargs):
         """
         Perform the given request on the given client.
@@ -245,10 +234,10 @@ class MemCachePool(object):
             message and free the client.
             """
             self.log.error(
-                "Memcache error: %s; request: %s %s" % (
-                    failure.value, command,
-                    " ".join(args)[:self.REQUEST_LOGGING_SIZE],
-                )
+                "Memcache error: {ex}; request: {cmd} {args}",
+                ex=failure.value,
+                cmd=command,
+                args=" ".join(args)[:self.REQUEST_LOGGING_SIZE],
             )
             self.clientFree(client)
 
@@ -263,7 +252,6 @@ class MemCachePool(object):
 
         return d
 
-
     def performRequest(self, command, *args, **kwargs):
         """
         Select an available client and perform the given request on it.
@@ -290,7 +278,7 @@ class MemCachePool(object):
             d = Deferred()
             self._commands.append((d, command, args, kwargs))
             self.log.debug(
-                "Command queued: %s, %r, %r" % (command, args, kwargs)
+                "Command queued: {c}, {a!r}, {k!r}", c=command, a=args, k=kwargs
             )
             self._logClientStats()
 
@@ -301,18 +289,15 @@ class MemCachePool(object):
 
         return d
 
-
     def _logClientStats(self):
         self.log.debug(
-            "Clients #free: %d, #busy: %d, #pending: %d, #queued: %d" % (
-                len(self._freeClients),
-                len(self._busyClients),
-                self._pendingConnects,
-                len(self._commands),
-            )
+            "Clients #free: {f}, #busy: {b}, #pending: {p}, #queued: {q}",
+            f=len(self._freeClients),
+            b=len(self._busyClients),
+            p=self._pendingConnects,
+            q=len(self._commands),
         )
 
-
     def clientGone(self, client):
         """
         Notify that the given client is to be removed from the pool completely.
@@ -325,10 +310,9 @@ class MemCachePool(object):
         elif client in self._freeClients:
             self._freeClients.remove(client)
 
-        self.log.debug("Removed client: %r" % (client,))
+        self.log.debug("Removed client: {c!r}", c=client)
         self._logClientStats()
 
-
     def clientBusy(self, client):
         """
         Notify that the given client is being used to complete a request.
@@ -341,10 +325,9 @@ class MemCachePool(object):
 
         self._busyClients.add(client)
 
-        self.log.debug("Busied client: %r" % (client,))
+        self.log.debug("Busied client: {c!r}", c=client)
         self._logClientStats()
 
-
     def clientFree(self, client):
         """
         Notify that the given client is free to handle more requests.
@@ -363,8 +346,8 @@ class MemCachePool(object):
             d, command, args, kwargs = self._commands.pop(0)
 
             self.log.debug(
-                "Performing Queued Command: %s, %r, %r"
-                % (command, args, kwargs)
+                "Performing Queued Command: {c}, {a}, {k}",
+                c=command, a=args, k=kwargs,
             )
             self._logClientStats()
 
@@ -373,10 +356,9 @@ class MemCachePool(object):
 
             _ign_d.addCallback(d.callback)
 
-        self.log.debug("Freed client: %r" % (client,))
+        self.log.debug("Freed client: {c!r}", c=client)
         self._logClientStats()
 
-
     def suggestMaxClients(self, maxClients):
         """
         Suggest the maximum number of concurrently connected clients.
@@ -386,40 +368,31 @@ class MemCachePool(object):
         """
         self._maxClients = maxClients
 
-
     def get(self, *args, **kwargs):
         return self.performRequest('get', *args, **kwargs)
 
-
     def set(self, *args, **kwargs):
         return self.performRequest('set', *args, **kwargs)
 
-
     def checkAndSet(self, *args, **kwargs):
         return self.performRequest('checkAndSet', *args, **kwargs)
 
-
     def delete(self, *args, **kwargs):
         return self.performRequest('delete', *args, **kwargs)
 
-
     def add(self, *args, **kwargs):
         return self.performRequest('add', *args, **kwargs)
 
-
     def incr(self, *args, **kwargs):
         return self.performRequest('increment', *args, **kwargs)
 
-
     def decr(self, *args, **kwargs):
         return self.performRequest('decrement', *args, **kwargs)
 
-
     def flushAll(self, *args, **kwargs):
         return self.performRequest('flushAll', *args, **kwargs)
 
 
-
 class CachePoolUserMixIn(object):
     """
     A mixin that returns a saved cache pool or fetches the default cache pool.
@@ -436,12 +409,10 @@ class CachePoolUserMixIn(object):
         return self._cachePool
 
 
-
 _memCachePools = {}         # Maps a name to a pool object
 _memCachePoolHandler = {}   # Maps a handler id to a named pool
 
 
-
 def installPools(pools, maxClients=5, reactor=None):
     if reactor is None:
         from twisted.internet import reactor
@@ -461,7 +432,6 @@ def installPools(pools, maxClients=5, reactor=None):
             )
 
 
-
 def _installPool(
     name, handleTypes, serverEndpoint, maxClients=5, reactor=None
 ):
@@ -472,7 +442,6 @@ def _installPool(
         _memCachePoolHandler[handle] = pool
 
 
-
 def defaultCachePool(name):
     if name not in _memCachePoolHandler:
         name = "Default"
