@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2017 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -44,11 +44,9 @@ from twistedcaldav.memcacheclient import ClientFactory
 from twistedcaldav.memcacheclient import MemcacheError, TokenMismatchError
 
 
-
 NoValue = ""
 
 
-
 class MemcachePropertyCollection (object):
     """
     Manages a single property store for all resources in a collection.
@@ -59,7 +57,6 @@ class MemcachePropertyCollection (object):
         self.collection = collection
         self.cacheTimeout = cacheTimeout
 
-
     @classmethod
     def memcacheClient(cls, refresh=False):
         if not hasattr(MemcachePropertyCollection, "_memcacheClient"):
@@ -77,7 +74,6 @@ class MemcachePropertyCollection (object):
 
         return MemcachePropertyCollection._memcacheClient
 
-
     def propertyCache(self):
         # The property cache has this format:
         #  {
@@ -95,7 +91,6 @@ class MemcachePropertyCollection (object):
             self._propertyCache = self._loadCache()
         return self._propertyCache
 
-
     def childCache(self, child):
         path = child.fp.path
         key = self._keyForPath(path)
@@ -104,7 +99,7 @@ class MemcachePropertyCollection (object):
         try:
             childCache, token = propertyCache[key]
         except KeyError:
-            self.log.debug("No child property cache for %s" % (child,))
+            self.log.debug("No child property cache for {c!r}", c=child)
             childCache, token = ({}, None)
 
             # message = "No child property cache for %s" % (child,)
@@ -113,7 +108,6 @@ class MemcachePropertyCollection (object):
 
         return propertyCache, key, childCache, token
 
-
     def _keyForPath(self, path):
         key = "|".join((
             self.__class__.__name__,
@@ -121,7 +115,6 @@ class MemcachePropertyCollection (object):
         ))
         return md5(key).hexdigest()
 
-
     def _loadCache(self, childNames=None):
         if childNames is None:
             abortIfMissing = False
@@ -132,7 +125,7 @@ class MemcachePropertyCollection (object):
             else:
                 return {}
 
-        self.log.debug("Loading cache for %s" % (self.collection,))
+        self.log.debug("Loading cache for {c}", c=self.collection)
 
         client = self.memcacheClient()
         assert client is not None, "OMG no cache!"
@@ -175,7 +168,6 @@ class MemcachePropertyCollection (object):
 
         return result
 
-
     def _split_gets_multi(self, keys, func, chunksize=250):
         """
         Splits gets_multi into chunks to avoid a memcacheclient timeout due
@@ -198,7 +190,6 @@ class MemcachePropertyCollection (object):
             results.update(func(subset))
         return results
 
-
     def _split_set_multi(self, values, func, time=0, chunksize=250):
         """
         Splits set_multi into chunks to avoid a memcacheclient timeout due
@@ -218,9 +209,8 @@ class MemcachePropertyCollection (object):
         if count:
             func(subset, time=time)
 
-
     def _storeCache(self, cache):
-        self.log.debug("Storing cache for %s" % (self.collection,))
+        self.log.debug("Storing cache for {c}", c=self.collection)
 
         values = dict((
             (self._keyForPath(path), props)
@@ -235,14 +225,13 @@ class MemcachePropertyCollection (object):
                 time=self.cacheTimeout
             )
 
-
     def _buildCache(self, childNames=None):
         if childNames is None:
             childNames = self.collection.listChildren()
         elif not childNames:
             return {}
 
-        self.log.debug("Building cache for %s" % (self.collection,))
+        self.log.debug("Building cache for {c}", c=self.collection)
 
         cache = {}
 
@@ -262,7 +251,6 @@ class MemcachePropertyCollection (object):
 
         return cache
 
-
     def setProperty(self, child, property, uid, delete=False):
         propertyCache, key, childCache, token = self.childCache(child)
 
@@ -319,11 +307,9 @@ class MemcachePropertyCollection (object):
                     child
                 ))
 
-
     def deleteProperty(self, child, qname, uid):
         return self.setProperty(child, qname, uid, delete=True)
 
-
     def flushCache(self, child):
         path = child.fp.path
         key = self._keyForPath(path)
@@ -338,11 +324,9 @@ class MemcachePropertyCollection (object):
             if not result:
                 raise MemcacheError("Unable to flush cache on %s" % (child,))
 
-
     def propertyStoreForChild(self, child, childPropertyStore):
         return self.ChildPropertyStore(self, child, childPropertyStore)
 
-
     class ChildPropertyStore (object):
         log = Logger()
 
@@ -372,29 +356,32 @@ class MemcachePropertyCollection (object):
                         "No such property: %s%s" % (uid if uid else "", encodeXMLName(*qname))
                     ))
 
-            self.log.debug("Read for %s%s on %s" % (
-                ("{%s}:" % (uid,)) if uid else "",
-                qname,
-                self.childPropertyStore.resource.fp.path
-            ))
+            self.log.debug(
+                "Read for {u}{q} on {p}",
+                u=("{%s}:" % (uid,)) if uid else "",
+                q=qname,
+                p=self.childPropertyStore.resource.fp.path
+            )
             return self.childPropertyStore.get(qname, uid=uid)
 
         def set(self, property, uid=None):
-            self.log.debug("Write for %s%s on %s" % (
-                ("{%s}:" % (uid,)) if uid else "",
-                property.qname(),
-                self.childPropertyStore.resource.fp.path
-            ))
+            self.log.debug(
+                "Write for {u}{q} on {p}",
+                u=("{%s}:" % (uid,)) if uid else "",
+                q=property.qname(),
+                p=self.childPropertyStore.resource.fp.path
+            )
 
             self.parentPropertyCollection.setProperty(self.child, property, uid)
             self.childPropertyStore.set(property, uid=uid)
 
         def delete(self, qname, uid=None):
-            self.log.debug("Delete for %s%s on %s" % (
-                ("{%s}:" % (uid,)) if uid else "",
-                qname,
-                self.childPropertyStore.resource.fp.path
-            ))
+            self.log.debug(
+                "Delete for {u}{q} on {p}",
+                u=("{%s}:" % (uid,)) if uid else "",
+                q=qname,
+                p=self.childPropertyStore.resource.fp.path,
+            )
 
             self.parentPropertyCollection.deleteProperty(self.child, qname, uid)
             self.childPropertyStore.delete(qname, uid=uid)
@@ -405,11 +392,12 @@ class MemcachePropertyCollection (object):
                 qnameuid = qname + (uid,)
                 return qnameuid in propertyCache
 
-            self.log.debug("Contains for %s%s on %s" % (
-                ("{%s}:" % (uid,)) if uid else "",
-                qname,
-                self.childPropertyStore.resource.fp.path,
-            ))
+            self.log.debug(
+                "Contains for {u}{q} on {p}",
+                u=("{%s}:" % (uid,)) if uid else "",
+                q=qname,
+                p=self.childPropertyStore.resource.fp.path,
+            )
             return self.childPropertyStore.contains(qname, uid=uid)
 
         def list(self, uid=None, filterByUID=True, cache=True):
@@ -425,6 +413,7 @@ class MemcachePropertyCollection (object):
                 else:
                     return results
 
-            self.log.debug("List for %s"
-                           % (self.childPropertyStore.resource.fp.path,))
+            self.log.debug(
+                "List for{p}", p=self.childPropertyStore.resource.fp.path
+            )
             return self.childPropertyStore.list(uid=uid, filterByUID=filterByUID)
