@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_DAV.MKCOL -*-
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,14 +25,14 @@ from twisted.python.failure import Failure
 from twisted.internet.defer import inlineCallbacks, returnValue
 
 from twext.python.log import Logger
-from twext.web2 import responsecode
+from txweb2 import responsecode
 from txdav.xml import element as davxml
-from twext.web2.http import Response
-from twext.web2.dav.http import ErrorResponse, PropertyStatusResponseQueue
-from twext.web2.dav.util import davXMLFromStream
-from twext.web2.dav.util import parentForURL
-from twext.web2.http import HTTPError
-from twext.web2.http import StatusResponse
+from txweb2.http import Response
+from txweb2.dav.http import ErrorResponse, PropertyStatusResponseQueue
+from txweb2.dav.util import davXMLFromStream
+from txweb2.dav.util import parentForURL
+from txweb2.http import HTTPError
+from txweb2.http import StatusResponse
 
 from twistedcaldav import caldavxml, carddavxml, mkcolxml
 from twistedcaldav.config import config
@@ -54,8 +54,7 @@ def http_MKCOL(self, request):
     yield parent.authorize(request, (davxml.Bind(),))
 
     if self.exists():
-        log.error("Attempt to create collection where resource exists: %s"
-                  % (self,))
+        log.error("Attempt to create collection where resource exists: {s!r}", s=self)
         raise HTTPError(ErrorResponse(
             responsecode.FORBIDDEN,
             (davxml.dav_namespace, "resource-must-be-null"),
@@ -63,8 +62,7 @@ def http_MKCOL(self, request):
         ))
 
     if not parent.isCollection():
-        log.error("Attempt to create collection with non-collection parent: %s"
-                  % (self,))
+        log.error("Attempt to create collection with non-collection parent: {s!r}", s=self)
         raise HTTPError(ErrorResponse(
             responsecode.CONFLICT,
             (davxml.dav_namespace, "collection-location-ok"),
@@ -99,8 +97,8 @@ def http_MKCOL(self, request):
     try:
         doc = (yield davXMLFromStream(request.stream))
     except ValueError, e:
-        log.error("Error while handling MKCOL: %s" % (e,))
-        # TODO: twext.web2.dav 'MKCOL' tests demand this particular response
+        log.error("Error while handling MKCOL: {ex}", ex=e)
+        # TODO: txweb2.dav 'MKCOL' tests demand this particular response
         # code, but should we really be looking at the XML content or the
         # content-type header?  It seems to me like this ought to be considered
         # a BAD_REQUEST if it claims to be XML but isn't, but an
@@ -154,8 +152,10 @@ def http_MKCOL(self, request):
                 raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, error))
 
             # Make sure feature is enabled
-            if (rtype == "calendar" and not config.EnableCalDAV or
-                rtype == "addressbook" and not config.EnableCardDAV):
+            if (
+                rtype == "calendar" and not config.EnableCalDAV or
+                rtype == "addressbook" and not config.EnableCardDAV
+            ):
                 error = "{DAV:}resourcetype property in MKCOL request body not supported: %s" % (mkcol,)
                 log.error("Error: {err}", err=error)
                 raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, error))
@@ -184,8 +184,8 @@ def http_MKCOL(self, request):
             # Clean up
             errors.error()
             raise HTTPError(Response(
-                    code=responsecode.FORBIDDEN,
-                    stream=mkcolxml.MakeCollectionResponse(errors.response()).toxml()
+                code=responsecode.FORBIDDEN,
+                stream=mkcolxml.MakeCollectionResponse(errors.response()).toxml()
             ))
 
         # When calendar collections are single component only, default MKCALENDAR is VEVENT only
