@@ -1,6 +1,6 @@
 # -*- test-case-name: txweb2.dav.test.test_prop.PROP.test_PROPFIND -*-
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -46,6 +46,7 @@ This is a direct copy of the twisted implementation of PROPFIND except that it u
 findChildrenFaster method to optimize child privilege checking.
 """
 
+
 @inlineCallbacks
 def http_PROPFIND(self, request):
     """
@@ -57,7 +58,7 @@ def http_PROPFIND(self, request):
         parent = (yield request.locateResource(parentURL))
         yield parent.authorize(request, (davxml.Bind(),))
 
-        log.error("Resource not found: %s" % (self,))
+        log.error("Resource not found: {s!r}", s=self)
         raise HTTPError(responsecode.NOT_FOUND)
 
     #
@@ -71,7 +72,7 @@ def http_PROPFIND(self, request):
     try:
         doc = (yield davXMLFromStream(request.stream))
     except ValueError, e:
-        log.error("Error while handling PROPFIND body: %s" % (e,))
+        log.error("Error while handling PROPFIND body: {ex}", ex=e)
         raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, str(e)))
 
     if doc is None:
@@ -159,7 +160,7 @@ def http_PROPFIND(self, request):
                 try:
                     resource_properties = (yield resource.listProperties(request))
                 except:
-                    log.error("Unable to get properties for resource %r" % (resource,))
+                    log.error("Unable to get properties for resource {r!r}", r=resource)
                     raise
 
                 properties_by_status = {
@@ -167,8 +168,8 @@ def http_PROPFIND(self, request):
                 }
             else:
                 properties_by_status = {
-                    responsecode.OK        : [],
-                    responsecode.NOT_FOUND : [],
+                    responsecode.OK: [],
+                    responsecode.NOT_FOUND: [],
                 }
 
                 if search_properties == "all":
@@ -240,7 +241,6 @@ def http_PROPFIND(self, request):
     returnValue(MultiStatusResponse(xml_responses))
 
 
-
 ##
 # Utilities
 ##
