@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_sharing -*-
 # #
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -79,7 +79,7 @@ class SharedResourceMixin(object):
                 ))
 
             # See if this property is on the shared calendar
-            if self.isShared():
+            if self.isSharedByOwner():
                 invitations = yield self.validateInvites(request)
                 returnValue(customxml.Invite(
                     *[(yield invitePropertyElement(invitation)) for invitation in invitations]
@@ -114,7 +114,6 @@ class SharedResourceMixin(object):
 
         returnValue(None)
 
-
     @inlineCallbacks
     def upgradeToShare(self):
         """
@@ -124,7 +123,6 @@ class SharedResourceMixin(object):
         # Change status on store object
         yield self._newStoreObject.setShared(True)
 
-
     @inlineCallbacks
     def downgradeFromShare(self, request):
 
@@ -137,7 +135,6 @@ class SharedResourceMixin(object):
 
         returnValue(True)
 
-
     @inlineCallbacks
     def directShare(self, request):
         """
@@ -219,17 +216,15 @@ class SharedResourceMixin(object):
             )
         ))
 
-
-    def isShared(self):
+    def isSharedByOwner(self):
         """
         Return True if this is an owner shared calendar collection.
         """
         try:
-            return self._newStoreObject.isShared() if self._newStoreObject else False
+            return self._newStoreObject.isSharedByOwner() if self._newStoreObject else False
         except AttributeError:
             return False
 
-
     def setShare(self, share_url):
         """
         Set the URL associated with this L{SharedResourceMixin}.  (This
@@ -238,7 +233,6 @@ class SharedResourceMixin(object):
         self._isShareeResource = True
         self._share_url = share_url
 
-
     def isShareeResource(self):
         """
         Return True if this is a sharee view of a shared collection.
@@ -250,14 +244,12 @@ class SharedResourceMixin(object):
             getattr(self._newStoreObject, "_bindMode", None) is not None
         )
 
-
     def removeShareeResource(self, request):
         """
         Called when the sharee DELETEs a shared collection.
         """
         return self._newStoreObject.deleteShare()
 
-
     @inlineCallbacks
     def _checkAccessControl(self):
         """
@@ -301,7 +293,6 @@ class SharedResourceMixin(object):
             bindMode = yield self._newStoreObject.effectiveShareMode()
             returnValue(invitationAccessFromBindModeMap.get(bindMode))
 
-
     @inlineCallbacks
     def shareeAccessControlList(self, request, *args, **kwargs):
         """
@@ -405,7 +396,6 @@ class SharedResourceMixin(object):
 
         returnValue(element.ACL(*aces))
 
-
     @inlineCallbacks
     def validUserIDForShare(self, userid, request=None):
         """
@@ -438,7 +428,6 @@ class SharedResourceMixin(object):
         else:
             returnValue(None)
 
-
     @inlineCallbacks
     def principalForCalendarGroupAddress(self, groupid):
         """
@@ -460,7 +449,6 @@ class SharedResourceMixin(object):
 
         returnValue(None)
 
-
     @inlineCallbacks
     def validateInvites(self, request, invitations=None):
         """
@@ -487,7 +475,6 @@ class SharedResourceMixin(object):
 
         returnValue(adjusted_invitations)
 
-
     def inviteUIDToShare(self, userid, cn, ace, summary, request):
         """ Send out in invite first, and then add this user to the share list
             @param userid:
@@ -505,7 +492,6 @@ class SharedResourceMixin(object):
         dl = [self.inviteSingleUserToShare(_user, _cn, ace, summary, request) for _user, _cn in zip(userid, cn)]
         return self._processShareActionList(dl, resultIsList)
 
-
     def uninviteUIDFromShare(self, userid, ace, request):
         """
         Send out in uninvite first, and then remove this user from the share list.
@@ -523,7 +509,6 @@ class SharedResourceMixin(object):
         dl = [self.uninviteSingleUserFromShare(user, ace, request) for user in userid]
         return self._processShareActionList(dl, resultIsList)
 
-
     def inviteUserUpdateToShare(self, userid, cn, aceOLD, aceNEW, summary, request):
 
         resultIsList = True
@@ -536,16 +521,14 @@ class SharedResourceMixin(object):
         dl = [self.inviteSingleUserUpdateToShare(_user, _cn, aceOLD, aceNEW, summary, request) for _user, _cn in zip(userid, cn)]
         return self._processShareActionList(dl, resultIsList)
 
-
     def _processShareActionList(self, dl, resultIsList):
         def _defer(resultset):
             results = [result if success else False for success, result in resultset]
             return results if resultIsList else results[0]
         return DeferredList(dl).addCallback(_defer)
 
-
     @inlineCallbacks
-    def inviteSingleUserToShare(self, userid, cn, ace, summary, request): #@UnusedVariable
+    def inviteSingleUserToShare(self, userid, cn, ace, summary, request):  # @UnusedVariable
 
         # We currently only handle local users
         sharee = yield self.principalForCalendarUserAddress(userid)
@@ -554,17 +537,20 @@ class SharedResourceMixin(object):
             if sharee is None:
                 returnValue(False)
 
-        result = (yield self._newStoreObject.inviteUIDToShare(
-            sharee.principalUID(),
-            invitationBindModeFromXMLMap[type(ace)],
-            summary,
-        ))
+        try:
+            result = (yield self._newStoreObject.inviteUIDToShare(
+                sharee.principalUID(),
+                invitationBindModeFromXMLMap[type(ace)],
+                summary,
+            ))
+        except Exception as e:
+            self.log.error("Could not send sharing invite '{userid}': {ex}", userid=userid, ex=e)
+            result = None
 
         returnValue(result)
 
-
     @inlineCallbacks
-    def uninviteSingleUserFromShare(self, userid, aces, request): #@UnusedVariable
+    def uninviteSingleUserFromShare(self, userid, aces, request):  # @UnusedVariable
 
         # Cancel invites - we'll just use whatever userid we are given. However, if we
         # cannot find a matching principal, try to extract the uid from the userid
@@ -577,24 +563,25 @@ class SharedResourceMixin(object):
         else:
             returnValue(False)
 
-        result = (yield self._newStoreObject.uninviteUIDFromShare(uid))
+        try:
+            result = (yield self._newStoreObject.uninviteUIDFromShare(uid))
+        except Exception as e:
+            self.log.error("Could not send sharing uninvite '{userid}': {ex}", userid=userid, ex=e)
+            result = None
 
         returnValue(result)
 
-
     @inlineCallbacks
     def uninviteFromShare(self, invitation, request):
 
         yield self._newStoreObject.uninviteFromShare(invitation)
         returnValue(True)
 
-
-    def inviteSingleUserUpdateToShare(self, userid, commonName, acesOLD, aceNEW, summary, request): #@UnusedVariable
+    def inviteSingleUserUpdateToShare(self, userid, commonName, acesOLD, aceNEW, summary, request):  # @UnusedVariable
 
         # Just update existing
         return self.inviteSingleUserToShare(userid, commonName, aceNEW, summary, request)
 
-
     @inlineCallbacks
     def _xmlHandleInvite(self, request, docroot):
         # Sharing must be enabled for this collection
@@ -608,7 +595,6 @@ class SharedResourceMixin(object):
         result = (yield self._handleInvite(request, docroot))
         returnValue(result)
 
-
     @inlineCallbacks
     def _handleInvite(self, request, invitedoc):
         def _handleInviteSet(inviteset):
@@ -645,7 +631,6 @@ class SharedResourceMixin(object):
                     "%s: %s" % (", ".join(error_text), inviteset,),
                 ))
 
-
         def _handleInviteRemove(inviteremove):
             userid = None
             access = []
@@ -723,7 +708,7 @@ class SharedResourceMixin(object):
         numRecords = len(invites)
 
         # Set the sharing state on the collection
-        shared = self.isShared()
+        shared = self.isSharedByOwner()
         if shared and numRecords == 0:
             yield self.downgradeFromShare(request)
         elif not shared and numRecords != 0:
@@ -748,7 +733,6 @@ class SharedResourceMixin(object):
         else:
             returnValue(responsecode.OK)
 
-
     @inlineCallbacks
     def _xmlHandleInviteReply(self, request, docroot):
         # Sharing must be enabled for this collection
@@ -762,11 +746,9 @@ class SharedResourceMixin(object):
         result = (yield self._handleInviteReply(request, docroot))
         returnValue(result)
 
-
     def _handleInviteReply(self, request, docroot):
         raise NotImplementedError
 
-
     @inlineCallbacks
     def xmlRequestHandler(self, request):
 
@@ -775,7 +757,7 @@ class SharedResourceMixin(object):
         try:
             doc = element.WebDAVDocument.fromString(xmldata)
         except ValueError, e:
-            self.log.error("Error parsing doc (%s) Doc:\n %s" % (str(e), xmldata,))
+            self.log.error("Error parsing doc ({ex}) Doc:\n {x}", ex=str(e), x=xmldata)
             raise HTTPError(ErrorResponse(
                 responsecode.FORBIDDEN,
                 (customxml.calendarserver_namespace, "valid-request"),
@@ -787,7 +769,7 @@ class SharedResourceMixin(object):
             result = (yield self.xmlDocHandlers[type(root)](self, request, root))
             returnValue(result)
         else:
-            self.log.error("Unsupported XML (%s)" % (root,))
+            self.log.error("Unsupported XML ({r})", r=root)
             raise HTTPError(ErrorResponse(
                 responsecode.FORBIDDEN,
                 (customxml.calendarserver_namespace, "valid-request"),
@@ -799,43 +781,41 @@ class SharedResourceMixin(object):
         customxml.InviteReply: _xmlHandleInviteReply,
     }
 
-
     def isGroup(self):
         try:
             return self._newStoreObject._kind == _ABO_KIND_GROUP
         except AttributeError:
             return False
 
-
     def POST_handler_content_type(self, request, contentType):
         if self.isCollection() or self.isGroup():
             if contentType:
                 if contentType in self._postHandlers:
                     return self._postHandlers[contentType](self, request)
                 else:
-                    self.log.info("Got a POST on collection or group with an unsupported content type: %s" % (contentType,))
+                    self.log.info("Got a POST on collection or group with an unsupported content type: {t}", t=contentType)
             else:
                 self.log.info("Got a POST on collection or group with no content type")
         return succeed(responsecode.FORBIDDEN)
 
     _postHandlers = {
-        ("application", "xml") : xmlRequestHandler,
-        ("text", "xml") : xmlRequestHandler,
+        ("application", "xml"): xmlRequestHandler,
+        ("text", "xml"): xmlRequestHandler,
     }
 
 
 invitationBindStatusToXMLMap = {
-    _BIND_STATUS_INVITED      : customxml.InviteStatusNoResponse,
-    _BIND_STATUS_ACCEPTED     : customxml.InviteStatusAccepted,
-    _BIND_STATUS_DECLINED     : customxml.InviteStatusDeclined,
-    _BIND_STATUS_INVALID      : customxml.InviteStatusInvalid,
-    _BIND_STATUS_DELETED      : customxml.InviteStatusDeleted,
+    _BIND_STATUS_INVITED: customxml.InviteStatusNoResponse,
+    _BIND_STATUS_ACCEPTED: customxml.InviteStatusAccepted,
+    _BIND_STATUS_DECLINED: customxml.InviteStatusDeclined,
+    _BIND_STATUS_INVALID: customxml.InviteStatusInvalid,
+    _BIND_STATUS_DELETED: customxml.InviteStatusDeleted,
 }
 invitationBindStatusFromXMLMap = dict((v, k) for k, v in invitationBindStatusToXMLMap.iteritems())
 
 invitationBindModeToXMLMap = {
-    _BIND_MODE_READ           : customxml.ReadAccess,
-    _BIND_MODE_WRITE          : customxml.ReadWriteAccess,
+    _BIND_MODE_READ: customxml.ReadAccess,
+    _BIND_MODE_WRITE: customxml.ReadWriteAccess,
 }
 invitationBindModeFromXMLMap = dict((v, k) for k, v in invitationBindModeToXMLMap.iteritems())
 
@@ -868,12 +848,10 @@ class SharedHomeMixin(LinkFollowerMixIn):
                 returnValue(None)
         returnValue(child)
 
-
     def _otherPrincipalHomeURL(self, otherUID):
         # Is this only meant to be overridden?
         pass
 
-
     @inlineCallbacks
     def acceptShare(self, request, inviteUID, summary):
 
@@ -903,7 +881,6 @@ class SharedHomeMixin(LinkFollowerMixIn):
             )
         ))
 
-
     @inlineCallbacks
     def declineShare(self, request, inviteUID):
 
@@ -921,7 +898,6 @@ class SharedHomeMixin(LinkFollowerMixIn):
             ))
         returnValue(Response(code=responsecode.NO_CONTENT))
 
-
     def _handleInviteReply(self, request, invitereplydoc):
         """
         Handle a user accepting or declining a sharing invite
