@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,9 @@
 
 import socket
 
-from plistlib import writePlist #@UnresolvedImport
-from twext.python.log import LogLevel
-from twext.python.test.test_log import defaultLogLevel, logLevelForNamespace
+from plistlib import writePlist  # @UnresolvedImport
+
+from twext.python.log import Logger
 
 from twistedcaldav.config import config, ConfigDict, mergeData
 from twistedcaldav.resource import CalDAVResource
@@ -26,6 +26,7 @@ from twistedcaldav.stdconfig import DEFAULT_CONFIG, PListConfigProvider, \
     RELATIVE_PATHS
 from twistedcaldav.test.util import TestCase
 
+from twisted.logger import LogLevel
 
 
 testConfig = """<?xml version="1.0" encoding="UTF-8"?>
@@ -125,25 +126,23 @@ testConfig = """<?xml version="1.0" encoding="UTF-8"?>
 """
 
 
-
 def _testResponseCompression(testCase):
     testCase.assertEquals(config.ResponseCompression, True)
 
 
-
 class ConfigTests(TestCase):
+
     def setUp(self):
         TestCase.setUp(self)
         config.setProvider(PListConfigProvider(DEFAULT_CONFIG))
         self.testConfig = self.mktemp()
-        open(self.testConfig, "w").write(testConfig)
-
+        with open(self.testConfig, "w") as f:
+            f.write(testConfig)
 
     def tearDown(self):
         config.setDefaults(DEFAULT_CONFIG)
         config.reset()
 
-
     def testDefaults(self):
         for key, value in DEFAULT_CONFIG.iteritems():
             if key in (
@@ -167,7 +166,6 @@ class ConfigTests(TestCase):
                 % (key, getattr(config, key), value)
             )
 
-
     def testLoadConfig(self):
         self.assertEquals(config.ResponseCompression, False)
 
@@ -175,7 +173,6 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.ResponseCompression, True)
 
-
     def testScoping(self):
         self.assertEquals(config.ResponseCompression, False)
 
@@ -185,12 +182,10 @@ class ConfigTests(TestCase):
 
         _testResponseCompression(self)
 
-
     def _myUpdateHook(self, data, reloading=False):
         # A stub hook to record the value of reloading=
         self._reloadingValue = reloading
 
-
     def testReloading(self):
         self.assertEquals(config.HTTPPort, 0)
 
@@ -209,7 +204,6 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.HTTPPort, 0)
 
-
     def testUpdateAndReload(self):
         self.assertEquals(config.HTTPPort, 0)
 
@@ -225,7 +219,6 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.HTTPPort, 8008)
 
-
     def testPreserveAcrossReload(self):
         self.assertEquals(config.Scheduling.iMIP.Sending.Password, "")
         self.assertEquals(config.Scheduling.iMIP.Receiving.Password, "")
@@ -242,7 +235,6 @@ class ConfigTests(TestCase):
         self.assertEquals(config.Scheduling.iMIP.Sending.Password, "sending")
         self.assertEquals(config.Scheduling.iMIP.Receiving.Password, "receiving")
 
-
     def testSetAttr(self):
         self.assertNotIn("BindAddresses", config.__dict__)
 
@@ -252,7 +244,6 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.BindAddresses, ["127.0.0.1"])
 
-
     def testDirty(self):
         config.__dict__["_dirty"] = False
         self.assertEquals(config.__dict__["_dirty"], False)
@@ -266,7 +257,6 @@ class ConfigTests(TestCase):
         config._foo = "bar"
         self.assertEquals(config.__dict__["_dirty"], False)
 
-
     def testUpdating(self):
         self.assertEquals(config.SSLPort, 0)
 
@@ -274,7 +264,6 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.SSLPort, 8443)
 
-
     def testMerge(self):
         self.assertEquals(config.MultiProcess.StaggeredStartup.Enabled, False)
 
@@ -282,7 +271,6 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.MultiProcess.StaggeredStartup.Enabled, False)
 
-
     def testDirectoryService_noChange(self):
         self.assertEquals(config.DirectoryService.type, "xml")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -292,7 +280,6 @@ class ConfigTests(TestCase):
         self.assertEquals(config.DirectoryService.type, "xml")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
 
-
     def testDirectoryService_sameType(self):
         self.assertEquals(config.DirectoryService.type, "xml")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -302,7 +289,6 @@ class ConfigTests(TestCase):
         self.assertEquals(config.DirectoryService.type, "xml")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
 
-
     def testDirectoryService_newType(self):
         self.assertEquals(config.DirectoryService.type, "xml")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -313,7 +299,6 @@ class ConfigTests(TestCase):
         self.assertNotIn("xmlFile", config.DirectoryService.params)
         self.assertEquals(config.DirectoryService.params.node, "/Search")
 
-
     def testDirectoryService_newParam(self):
         self.assertEquals(config.DirectoryService.type, "xml")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -323,7 +308,6 @@ class ConfigTests(TestCase):
         self.assertEquals(config.DirectoryService.type, "opendirectory")
         self.assertEquals(config.DirectoryService.params.node, "/Search")
 
-
     def testDirectoryService_unknownType(self):
         self.assertEquals(config.DirectoryService.type, "xml")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -352,13 +336,11 @@ class ConfigTests(TestCase):
 
         config.updateDefaults({"SSLPort": 0})
 
-
     def testMergeDefaults(self):
         config.updateDefaults({"MultiProcess": {}})
 
         self.assertEquals(config._provider.getDefaults().MultiProcess.StaggeredStartup.Enabled, False)
 
-
     def testSetDefaults(self):
         config.updateDefaults({"SSLPort": 8443})
 
@@ -368,13 +350,11 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.SSLPort, 0)
 
-
     def testCopiesDefaults(self):
         config.updateDefaults({"Foo": "bar"})
 
         self.assertNotIn("Foo", DEFAULT_CONFIG)
 
-
     def testComplianceClasses(self):
         resource = CalDAVResource()
 
@@ -398,11 +378,11 @@ class ConfigTests(TestCase):
         config.GroupAttendees.Enabled = True
         config.update()
 
-
     def test_logging(self):
         """
         Logging module configures properly.
         """
+        defaultLogLevel = Logger.filterPredicate.logLevelForNamespace(None)
         self.assertNotEqual(
             defaultLogLevel, LogLevel.error,
             "This test assumes the default log level is not error."
@@ -411,20 +391,19 @@ class ConfigTests(TestCase):
         config.setDefaults(DEFAULT_CONFIG)
         config.reload()
 
-        self.assertEquals(logLevelForNamespace(None), defaultLogLevel)
-        self.assertEquals(logLevelForNamespace("some.namespace"), defaultLogLevel)
+        self.assertEquals(Logger.filterPredicate.logLevelForNamespace(None), defaultLogLevel)
+        self.assertEquals(Logger.filterPredicate.logLevelForNamespace("some.namespace"), defaultLogLevel)
 
         config.load(self.testConfig)
 
-        self.assertEquals(logLevelForNamespace(None), LogLevel.error)
-        self.assertEquals(logLevelForNamespace("some.namespace"), LogLevel.debug)
+        self.assertEquals(Logger.filterPredicate.logLevelForNamespace(None), LogLevel.error)
+        self.assertEquals(Logger.filterPredicate.logLevelForNamespace("some.namespace"), LogLevel.debug)
 
         writePlist({}, self.testConfig)
         config.reload()
 
-        self.assertEquals(logLevelForNamespace(None), defaultLogLevel)
-        self.assertEquals(logLevelForNamespace("some.namespace"), defaultLogLevel)
-
+        self.assertEquals(Logger.filterPredicate.logLevelForNamespace(None), defaultLogLevel)
+        self.assertEquals(Logger.filterPredicate.logLevelForNamespace("some.namespace"), defaultLogLevel)
 
     def test_ConfigDict(self):
         configDict = ConfigDict({
@@ -475,33 +454,32 @@ class ConfigTests(TestCase):
         configDict._x = "X"
         self.assertEquals(configDict._x, "X")
 
-
     def test_mergeData(self):
         """
         Verify we don't lose keys which are present in the old but not
         replaced in the new.
         """
         old = ConfigDict({
-            "Scheduling" : ConfigDict({
-                "iMIP" : ConfigDict({
-                    "Enabled" : True,
-                    "Receiving" : ConfigDict({
-                        "Username" : "xyzzy",
-                        "Server" : "example.com",
+            "Scheduling": ConfigDict({
+                "iMIP": ConfigDict({
+                    "Enabled": True,
+                    "Receiving": ConfigDict({
+                        "Username": "xyzzy",
+                        "Server": "example.com",
                     }),
-                    "Sending" : ConfigDict({
-                        "Username" : "plugh",
+                    "Sending": ConfigDict({
+                        "Username": "plugh",
                     }),
-                    "AddressPatterns" : ["mailto:.*"],
+                    "AddressPatterns": ["mailto:.*"],
                 }),
             }),
         })
         new = ConfigDict({
-            "Scheduling" : ConfigDict({
-                "iMIP" : ConfigDict({
-                    "Enabled" : False,
-                    "Receiving" : ConfigDict({
-                        "Username" : "changed",
+            "Scheduling": ConfigDict({
+                "iMIP": ConfigDict({
+                    "Enabled": False,
+                    "Receiving": ConfigDict({
+                        "Username": "changed",
                     }),
                 }),
             }),
@@ -510,7 +488,6 @@ class ConfigTests(TestCase):
         self.assertEquals(old.Scheduling.iMIP.Receiving.Server, "example.com")
         self.assertEquals(old.Scheduling.iMIP.Sending.Username, "plugh")
 
-
     def test_SimpleInclude(self):
 
         testConfigMaster = """<?xml version="1.0" encoding="UTF-8"?>
@@ -565,16 +542,17 @@ class ConfigTests(TestCase):
         config.setProvider(PListConfigProvider(DEFAULT_CONFIG))
 
         self.testInclude = self.mktemp()
-        open(self.testInclude, "w").write(testConfigInclude)
+        with open(self.testInclude, "w") as f:
+            f.write(testConfigInclude)
 
         self.testMaster = self.mktemp()
-        open(self.testMaster, "w").write(testConfigMaster % (self.testInclude,))
+        with open(self.testMaster, "w") as f:
+            f.write(testConfigMaster % (self.testInclude,))
 
         config.load(self.testMaster)
         self.assertEquals(config.HTTPPort, 9008)
         self.assertEquals(config.SSLPort, 8443)
 
-
     def test_FQDNInclude(self):
 
         testConfigMaster = """<?xml version="1.0" encoding="UTF-8"?>
@@ -630,16 +608,17 @@ class ConfigTests(TestCase):
 
         self.testIncludeRoot = self.mktemp()
         self.testInclude = self.testIncludeRoot + "." + socket.getfqdn()
-        open(self.testInclude, "w").write(testConfigInclude)
+        with open(self.testInclude, "w") as f:
+            f.write(testConfigInclude)
 
         self.testMaster = self.mktemp()
-        open(self.testMaster, "w").write(testConfigMaster % (self.testIncludeRoot,))
+        with open(self.testMaster, "w") as f:
+            f.write(testConfigMaster % (self.testIncludeRoot,))
 
         config.load(self.testMaster)
         self.assertEquals(config.HTTPPort, 9008)
         self.assertEquals(config.SSLPort, 8443)
 
-
     def test_HostnameInclude(self):
 
         testConfigMaster = """<?xml version="1.0" encoding="UTF-8"?>
@@ -695,19 +674,28 @@ class ConfigTests(TestCase):
 
         self.testIncludeRoot = self.mktemp()
         self.testInclude = self.testIncludeRoot + "." + socket.gethostbyname(socket.getfqdn())
-        open(self.testInclude, "w").write(testConfigInclude)
+        with open(self.testInclude, "w") as f:
+            f.write(testConfigInclude)
 
         self.testMaster = self.mktemp()
-        open(self.testMaster, "w").write(testConfigMaster % (self.testIncludeRoot,))
+        with open(self.testMaster, "w") as f:
+            f.write(testConfigMaster % (self.testIncludeRoot,))
 
         config.load(self.testMaster)
         self.assertEquals(config.HTTPPort, 9008)
         self.assertEquals(config.SSLPort, 8443)
 
-
     def testSyncToken(self):
         config.load(self.testConfig)
 
+        self.assertEquals(
+            sorted(config._syncTokenKeys),
+            ['Notifications.Services.APNS.CalDAV.Topic',
+             'Notifications.Services.APNS.CardDAV.Topic',
+             'Notifications.Services.APNS.Enabled',
+             'ServerInfoToken']
+        )
+
         # no sync token keys specified; need to empty this array here because
         # stdconfig is registering keys automatically
         config._syncTokenKeys = []
@@ -734,4 +722,4 @@ class ConfigTests(TestCase):
 
         # reload automatically invalidates
         config.reload()
-        self.assertEquals("a1c46c5aff1899658dac033ee8520b07", config.syncToken())
+        self.assertEquals(config._cachedSyncToken, None)
