@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -44,7 +44,6 @@ sharedOwnerType = davxml.ResourceType.sharedownercalendar  # @UndefinedVariable
 regularCalendarType = davxml.ResourceType.calendar  # @UndefinedVariable
 
 
-
 def normalize(x):
     """
     Normalize some XML by parsing it, collapsing whitespace, and
@@ -53,7 +52,6 @@ def normalize(x):
     return WebDAVDocument.fromString(x).toxml()
 
 
-
 class BaseSharingTests(StoreTestCase):
 
     def configure(self):
@@ -65,13 +63,11 @@ class BaseSharingTests(StoreTestCase):
         self.patch(config.Sharing.Calendars, "Enabled", True)
         self.patch(config.Authentication.Wiki, "Enabled", True)
 
-
     @inlineCallbacks
     def setUp(self):
         yield super(BaseSharingTests, self).setUp()
         self.resource = yield self._getResource()
 
-
     @inlineCallbacks
     def _refreshRoot(self, request=None):
         if request is None:
@@ -84,7 +80,6 @@ class BaseSharingTests(StoreTestCase):
         self.site.resource.putChild("calendars", self.homeProvisioner)
         returnValue(result)
 
-
     @inlineCallbacks
     def _doPOST(self, body, resultcode=responsecode.OK):
         authPrincipal = yield self.actualRoot.findPrincipalForAuthID("user01")
@@ -103,7 +98,6 @@ class BaseSharingTests(StoreTestCase):
         else:
             returnValue(None)
 
-
     @inlineCallbacks
     def _doPROPFINDHome(self, resultcode=responsecode.MULTI_STATUS):
         body = """<?xml version="1.0" encoding="UTF-8"?>
@@ -163,14 +157,12 @@ class BaseSharingTests(StoreTestCase):
         else:
             returnValue(None)
 
-
     @inlineCallbacks
     def _getResource(self):
         request = SimpleStoreRequest(self, "GET", "/calendars/__uids__/user01/calendar/")
         resource = yield request.locateResource("/calendars/__uids__/user01/calendar/")
         returnValue(resource)
 
-
     @inlineCallbacks
     def _doPOSTSharerAccept(self, body, resultcode=responsecode.OK, sharer="user02"):
         authPrincipal = yield self.actualRoot.findPrincipalForAuthID(sharer)
@@ -187,14 +179,12 @@ class BaseSharingTests(StoreTestCase):
         else:
             returnValue(None)
 
-
     @inlineCallbacks
     def _getResourceSharer(self, name):
         request = SimpleStoreRequest(self, "GET", "%s" % (name,))
         resource = yield request.locateResource("%s" % (name,))
         returnValue(resource)
 
-
     def _getUIDElementValue(self, xml):
 
         for user in xml.children:
@@ -203,7 +193,6 @@ class BaseSharingTests(StoreTestCase):
                     return element.children[0].data
         return None
 
-
     def _getUIDElementValues(self, xml):
 
         results = {}
@@ -213,7 +202,6 @@ class BaseSharingTests(StoreTestCase):
             results[href] = uid
         return results
 
-
     def _clearUIDElementValue(self, xml):
 
         for user in xml.children:
@@ -222,7 +210,6 @@ class BaseSharingTests(StoreTestCase):
                 uid.children[0].data = ""
         return xml
 
-
     def _getHRefElementValue(self, xml):
 
         for href in xml.root_element.children:
@@ -231,7 +218,6 @@ class BaseSharingTests(StoreTestCase):
         return None
 
 
-
 class SharingTests(BaseSharingTests):
 
     @inlineCallbacks
@@ -239,7 +225,7 @@ class SharingTests(BaseSharingTests):
 
         rtype = self.resource.resourceType()
         self.assertEquals(rtype, regularCalendarType)
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertFalse(isShared)
         isShareeResource = self.resource.isShareeResource()
         self.assertFalse(isShareeResource)
@@ -248,12 +234,11 @@ class SharingTests(BaseSharingTests):
 
         rtype = self.resource.resourceType()
         self.assertEquals(rtype, sharedOwnerType)
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertTrue(isShared)
         isShareeResource = self.resource.isShareeResource()
         self.assertFalse(isShareeResource)
 
-
     @inlineCallbacks
     def test_downgradeFromShare(self):
 
@@ -261,7 +246,7 @@ class SharingTests(BaseSharingTests):
 
         rtype = self.resource.resourceType()
         self.assertEquals(rtype, sharedOwnerType)
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertTrue(isShared)
         isShareeResource = self.resource.isShareeResource()
         self.assertFalse(isShareeResource)
@@ -270,12 +255,11 @@ class SharingTests(BaseSharingTests):
 
         rtype = self.resource.resourceType()
         self.assertEquals(rtype, regularCalendarType)
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertFalse(isShared)
         isShareeResource = self.resource.isShareeResource()
         self.assertFalse(isShareeResource)
 
-
     @inlineCallbacks
     def test_POSTaddInviteeAlreadyShared(self):
 
@@ -302,12 +286,11 @@ class SharingTests(BaseSharingTests):
             )
         ))
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertTrue(isShared)
         isShareeResource = self.resource.isShareeResource()
         self.assertFalse(isShareeResource)
 
-
     @inlineCallbacks
     def test_POSTaddInviteeNotAlreadyShared(self):
 
@@ -332,16 +315,15 @@ class SharingTests(BaseSharingTests):
             )
         ))
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertTrue(isShared)
         isShareeResource = (yield self.resource.isShareeResource())
         self.assertFalse(isShareeResource)
 
-
     @inlineCallbacks
     def test_POSTupdateInvitee(self):
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertFalse(isShared)
 
         yield self._doPOST("""<?xml version="1.0" encoding="utf-8" ?>
@@ -354,7 +336,7 @@ class SharingTests(BaseSharingTests):
             </CS:share>
             """)
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertTrue(isShared)
 
         yield self._doPOST("""<?xml version="1.0" encoding="utf-8" ?>
@@ -367,7 +349,7 @@ class SharingTests(BaseSharingTests):
             </CS:share>
             """)
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertTrue(isShared)
 
         propInvite = (yield self.resource.readProperty(customxml.Invite, None))
@@ -381,11 +363,10 @@ class SharingTests(BaseSharingTests):
             )
         ))
 
-
     @inlineCallbacks
     def test_POSTremoveInvitee(self):
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertFalse(isShared)
 
         yield self._doPOST("""<?xml version="1.0" encoding="utf-8" ?>
@@ -398,7 +379,7 @@ class SharingTests(BaseSharingTests):
             </CS:share>
             """)
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertTrue(isShared)
 
         yield self._doPOST("""<?xml version="1.0" encoding="utf-8" ?>
@@ -409,13 +390,12 @@ class SharingTests(BaseSharingTests):
             </CS:share>
             """)
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertFalse(isShared)
 
         propInvite = (yield self.resource.readProperty(customxml.Invite, None))
         self.assertEquals(propInvite, None)
 
-
     @inlineCallbacks
     def test_POSTaddMoreInvitees(self):
 
@@ -470,7 +450,6 @@ class SharingTests(BaseSharingTests):
             ),
         ))
 
-
     @inlineCallbacks
     def test_POSTaddRemoveInvitees(self):
 
@@ -521,7 +500,6 @@ class SharingTests(BaseSharingTests):
             ),
         ))
 
-
     @inlineCallbacks
     def test_POSTaddRemoveSameInvitee(self):
 
@@ -572,7 +550,6 @@ class SharingTests(BaseSharingTests):
             ),
         ))
 
-
     @inlineCallbacks
     def test_POSTremoveNonInvitee(self):
         """
@@ -618,7 +595,6 @@ class SharingTests(BaseSharingTests):
         propInvite = (yield self.resource.readProperty(customxml.Invite, None))
         self.assertEquals(propInvite, None)
 
-
     @inlineCallbacks
     def test_POSTaddInvalidInvitee(self):
         yield self.resource.upgradeToShare()
@@ -648,14 +624,12 @@ class SharingTests(BaseSharingTests):
         propInvite = (yield self.resource.readProperty(customxml.Invite, None))
         self.assertEquals(propInvite, None)
 
-
     def assertXMLEquals(self, a, b):
         """
         Assert two strings are equivalent as XML.
         """
         self.assertEquals(normalize(a), normalize(b))
 
-
     @inlineCallbacks
     def test_POSTremoveInvalidInvitee(self):
 
@@ -707,7 +681,6 @@ class SharingTests(BaseSharingTests):
         propInvite = (yield self.resource.readProperty(customxml.Invite, None))
         self.assertEquals(propInvite, None)
 
-
     @inlineCallbacks
     def wikiSetup(self):
         """
@@ -732,7 +705,6 @@ class SharingTests(BaseSharingTests):
         sharedName = yield cal.shareWith(sharee, _BIND_MODE_DIRECT)
         returnValue(sharedName)
 
-
     @inlineCallbacks
     def test_wikiACL(self):
         """
@@ -760,7 +732,6 @@ class SharingTests(BaseSharingTests):
         acl = (yield collection.shareeAccessControlList(request))
         self.assertTrue("<write/>" in acl.toxml())
 
-
     @inlineCallbacks
     def test_noWikiAccess(self):
         """
@@ -798,7 +769,6 @@ class SharingTests(BaseSharingTests):
         childNames = yield listChildrenViaPropfind()
         self.assertNotIn(sharedName, childNames)
 
-
     @inlineCallbacks
     def test_POSTDowngradeWithMissingInvitee(self):
 
@@ -830,7 +800,6 @@ class SharingTests(BaseSharingTests):
 
         yield self.resource.downgradeFromShare(norequest())
 
-
     @inlineCallbacks
     def test_POSTRemoveWithMissingInvitee(self):
 
@@ -868,13 +837,12 @@ class SharingTests(BaseSharingTests):
             </CS:share>
             """)
 
-        isShared = self.resource.isShared()
+        isShared = self.resource.isSharedByOwner()
         self.assertFalse(isShared)
 
         propInvite = (yield self.resource.readProperty(customxml.Invite, None))
         self.assertEquals(propInvite, None)
 
-
     @inlineCallbacks
     def test_POSTShareeRemoveWithDisabledSharer(self):
 
@@ -928,7 +896,6 @@ class SharingTests(BaseSharingTests):
         resource = (yield self._getResourceSharer(href))
         self.assertFalse(resource.exists())
 
-
     @inlineCallbacks
     def test_POSTShareeRemoveWithMissingSharer(self):
 
@@ -982,7 +949,6 @@ class SharingTests(BaseSharingTests):
         resource = (yield self._getResourceSharer(href))
         self.assertFalse(resource.exists())
 
-
     @inlineCallbacks
     def test_POSTShareeAcceptNewWithMissingSharer(self):
 
@@ -1030,7 +996,6 @@ class SharingTests(BaseSharingTests):
             resultcode=responsecode.FORBIDDEN,
         )
 
-
     @inlineCallbacks
     def test_POSTShareeAcceptExistingWithMissingSharer(self):
 
@@ -1117,7 +1082,6 @@ class SharingTests(BaseSharingTests):
             resultcode=responsecode.OK,
         )
 
-
     @inlineCallbacks
     def test_POSTShareeDeclineNewWithMissingSharer(self):
 
@@ -1165,7 +1129,6 @@ class SharingTests(BaseSharingTests):
             resultcode=responsecode.NO_CONTENT,
         )
 
-
     @inlineCallbacks
     def test_POSTShareeDeclineExistingWithMissingSharer(self):
 
@@ -1252,7 +1215,6 @@ class SharingTests(BaseSharingTests):
             resultcode=responsecode.NO_CONTENT,
         )
 
-
     @inlineCallbacks
     def test_shareeInviteWithDisabledSharer(self):
 
@@ -1318,7 +1280,6 @@ class SharingTests(BaseSharingTests):
             ),
         ))
 
-
     @inlineCallbacks
     def test_shareeInviteWithMissingSharer(self):
 
@@ -1384,7 +1345,6 @@ class SharingTests(BaseSharingTests):
             ),
         ))
 
-
     @inlineCallbacks
     def test_shareeNotificationWithMissingSharer(self):
 
@@ -1411,6 +1371,48 @@ class SharingTests(BaseSharingTests):
         note = yield note_child.text()
         self.assertTrue(isinstance(note, str))
 
+    @inlineCallbacks
+    def test_shareeReplyWithMissingSharee(self):
+
+        yield self.resource.upgradeToShare()
+
+        yield self._doPOST("""<?xml version="1.0" encoding="utf-8" ?>
+            <CS:share xmlns:D="DAV:" xmlns:CS="http://calendarserver.org/ns/">
+                <CS:set>
+                    <D:href>mailto:user02@example.com</D:href>
+                    <CS:summary>My Shared Calendar</CS:summary>
+                    <CS:read-write/>
+                </CS:set>
+            </CS:share>
+            """)
+        propInvite = (yield self.resource.readProperty(customxml.Invite, None))
+        uids = self._getUIDElementValues(propInvite)
+
+        yield self._doPOSTSharerAccept("""<?xml version='1.0' encoding='UTF-8'?>
+            <invite-reply xmlns='http://calendarserver.org/ns/'>
+              <href xmlns='DAV:'>mailto:user01@example.com</href>
+              <invite-accepted/>
+              <hosturl>
+                <href xmlns='DAV:'>/calendars/__uids__/user01/calendar/</href>
+              </hosturl>
+              <in-reply-to>%s</in-reply-to>
+              <summary>The Shared Calendar</summary>
+              <common-name>User 02</common-name>
+              <first-name>user</first-name>
+              <last-name>02</last-name>
+            </invite-reply>
+            """ % (uids["urn:x-uid:user02"],))
+
+        yield self.directory.removeRecords(((yield self.userUIDFromShortName("user02")),))
+        self.assertTrue((yield self.userUIDFromShortName("user02")) is None)
+
+        request = SimpleStoreRequest(self, "GET", "/calendars/__uids__/user01/notification/")
+        notification = yield request.locateResource("/calendars/__uids__/user01/notification/")
+        names = yield notification.listChildren()
+        self.assertEqual(len(names), 1)
+        note_child = yield notification.getChild(names[0])
+        note = yield note_child.text()
+        self.assertTrue(isinstance(note, str))
 
     @inlineCallbacks
     def test_hideInvalidSharers(self):
@@ -1507,7 +1509,6 @@ class SharingTests(BaseSharingTests):
         ))
 
 
-
 class DropboxSharingTests(BaseSharingTests):
 
     def configure(self):
@@ -1518,7 +1519,6 @@ class DropboxSharingTests(BaseSharingTests):
         self.patch(config, "EnableDropBox", True)
         self.patch(config, "EnableManagedAttachments", False)
 
-
     @inlineCallbacks
     def test_dropboxWithMissingInvitee(self):
 
@@ -1583,7 +1583,6 @@ END:VCALENDAR
         self.assertTrue(acl is not None)
 
 
-
 class MamnagedAttachmentSharingTests(BaseSharingTests):
 
     def configure(self):
@@ -1594,7 +1593,6 @@ class MamnagedAttachmentSharingTests(BaseSharingTests):
         self.patch(config, "EnableDropBox", False)
         self.patch(config, "EnableManagedAttachments", True)
 
-
     @inlineCallbacks
     def test_attachmentWithMissingInvitee(self):
 
