@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,8 +22,10 @@ from twisted.trial.unittest import TestCase
 
 from twistedcaldav.config import Config, ConfigDict
 from twistedcaldav.stdconfig import NoUnicodePlistParser, PListConfigProvider,\
-    _updateDataStore, _updateMultiProcess
+    _updateDataStore, _updateMultiProcess, _updateUtilityLog
 import twistedcaldav.stdconfig
+import sys
+import os
 
 nonASCIIValue = "→←"
 nonASCIIPlist = "<plist version='1.0'><string>%s</string></plist>" % (
@@ -39,6 +41,7 @@ nonASCIIConfigPList = """
 </plist>
 """ % (nonASCIIValue,)
 
+
 class ConfigParsingTests(TestCase):
     """
     Tests to verify the behavior of the configuration parser.
@@ -54,7 +57,6 @@ class ConfigParsingTests(TestCase):
         self.assertEquals(parser.parse(StringIO(nonASCIIPlist)),
                           nonASCIIValue)
 
-
     def test_parseNonASCIIConfig(self):
         """
         Non-ASCII <string>s found as part of a configuration file will be
@@ -67,7 +69,6 @@ class ConfigParsingTests(TestCase):
         cfg.load(tempfile.path)
         self.assertEquals(cfg.DataRoot, nonASCIIValue)
 
-
     def test_relativeDefaultPaths(self):
         """
         The paths specified in the default configuration should be interpreted
@@ -89,7 +90,6 @@ class ConfigParsingTests(TestCase):
         cfg.load(tempfile.path)
         self.assertEquals(cfg.AccountingLogRoot, "/other/root/some-path")
 
-
     def test_includes(self):
 
         plist1 = """
@@ -144,33 +144,44 @@ class ConfigParsingTests(TestCase):
         self.assertEquals(cfg.DocumentRoot, "/root/overridedata/defaultdoc")
         self.assertEquals(cfg.DataRoot, "/root/overridedata")
 
-
     def test_updateDataStore(self):
         configDict = {
-            "ServerRoot" : "/a/b/c/",
+            "ServerRoot": "/a/b/c/",
         }
         _updateDataStore(configDict)
         self.assertEquals(configDict["ServerRoot"], "/a/b/c")
 
-
     def test_updateMultiProcess(self):
         def stubProcessCount(*args):
             return 3
         self.patch(twistedcaldav.stdconfig, "computeProcessCount", stubProcessCount)
         configDict = ConfigDict({
-            "MultiProcess" : {
-                "ProcessCount" : 0,
-                "MinProcessCount" : 2,
-                "PerCPU" : 1,
-                "PerGB" : 1,
+            "MultiProcess": {
+                "ProcessCount": 0,
+                "MinProcessCount": 2,
+                "PerCPU": 1,
+                "PerGB": 1,
             },
-            "Postgres" : {
-                "ExtraConnections" : 5,
-                "BuffersToConnectionsRatio" : 1.5,
+            "Postgres": {
+                "ExtraConnections": 5,
+                "BuffersToConnectionsRatio": 1.5,
             },
-            "SharedConnectionPool" : False,
-            "MaxDBConnectionsPerPool" : 10,
+            "SharedConnectionPool": False,
+            "MaxDBConnectionsPerPool": 10,
         })
         _updateMultiProcess(configDict)
         self.assertEquals(45, configDict.Postgres.MaxConnections)
         self.assertEquals(67, configDict.Postgres.SharedBuffers)
+
+    def test_updateUtilityLog(self):
+        configDict = {
+            "ServerRoot": "/a/b/c/",
+            "LogRoot": "Logs",
+            "UtilityLogFile": "util.txt",
+        }
+        _updateUtilityLog(configDict)
+        self.assertEquals(configDict["UtilityLogFile"], "{}.log".format(os.path.basename(sys.argv[0])))
+
+        _updateDataStore(configDict)
+        _updateUtilityLog(configDict)
+        self.assertEquals(configDict["UtilityLogFile"], "/a/b/c/Logs/{}.log".format(os.path.basename(sys.argv[0])))
