@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_upgrade -*-
 ##
-# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -49,7 +49,6 @@ from twistedcaldav.xmlutil import readXML, writeXML, addSubElement
 
 from txdav.caldav.datastore.index_file import db_basename
 from txdav.caldav.datastore.scheduling.cuaddress import LocalCalendarUser
-from txdav.caldav.datastore.scheduling.imip.mailgateway import MailGatewayTokensDatabase
 from txdav.caldav.datastore.scheduling.imip.mailgateway import migrateTokensToStore
 from txdav.caldav.datastore.scheduling.scheduler import DirectScheduler
 from txdav.caldav.datastore.util import normalizationLookup
@@ -84,7 +83,6 @@ def xattrname(n):
     return deadPropertyXattrPrefix + n
 
 
-
 def getCalendarServerIDs(config):
 
     # Determine uid/gid for ownership of directories we create here
@@ -93,19 +91,18 @@ def getCalendarServerIDs(config):
         try:
             uid = pwd.getpwnam(config.UserName).pw_uid
         except KeyError:
-            log.error("User not found: %s" % (config.UserName,))
+            log.error("User not found: {user}", user=config.UserName)
 
     gid = -1
     if config.GroupName:
         try:
             gid = grp.getgrnam(config.GroupName).gr_gid
         except KeyError:
-            log.error("Group not found: %s" % (config.GroupName,))
+            log.error("Group not found: {grp}", grp=config.GroupName)
 
     return uid, gid
 
 
-
 def fixBadQuotes(data):
     if (
         data.find('\\"') != -1 or
@@ -126,7 +123,6 @@ def fixBadQuotes(data):
         return data, False
 
 
-
 @inlineCallbacks
 def upgradeCalendarCollection(calPath, directory, cuaCache):
     errorOccurred = False
@@ -143,7 +139,7 @@ def upgradeCalendarCollection(calPath, directory, cuaCache):
             # Skip directories
             continue
 
-        log.debug("Processing: %s" % (resPath,))
+        log.debug("Processing: {path}", path=resPath)
         needsRewrite = False
         with open(resPath) as res:
             data = res.read()
@@ -151,12 +147,12 @@ def upgradeCalendarCollection(calPath, directory, cuaCache):
             try:
                 data, fixed = fixBadQuotes(data)
                 if fixed:
-                    log.warn("Fixing bad quotes in %s" % (resPath,))
+                    log.warn("Fixing bad quotes in {path}", path=resPath)
                     needsRewrite = True
             except Exception, e:
                 log.error(
-                    "Error while fixing bad quotes in %s: %s" %
-                    (resPath, e)
+                    "Error while fixing bad quotes in {path}: {ex}",
+                    path=resPath, ex=e,
                 )
                 errorOccurred = True
                 continue
@@ -164,12 +160,12 @@ def upgradeCalendarCollection(calPath, directory, cuaCache):
             try:
                 data, fixed = removeIllegalCharacters(data)
                 if fixed:
-                    log.warn("Removing illegal characters in %s" % (resPath,))
+                    log.warn("Removing illegal characters in {path}", path=resPath)
                     needsRewrite = True
             except Exception, e:
                 log.error(
-                    "Error while removing illegal characters in %s: %s" %
-                    (resPath, e)
+                    "Error while removing illegal characters in {path}: {ex}",
+                    path=resPath, ex=e,
                 )
                 errorOccurred = True
                 continue
@@ -177,12 +173,12 @@ def upgradeCalendarCollection(calPath, directory, cuaCache):
             try:
                 data, fixed = (yield normalizeCUAddrs(data, directory, cuaCache))
                 if fixed:
-                    log.debug("Normalized CUAddrs in %s" % (resPath,))
+                    log.debug("Normalized CUAddrs in {path}", path=resPath)
                     needsRewrite = True
             except Exception, e:
                 log.error(
-                    "Error while normalizing %s: %s" %
-                    (resPath, e)
+                    "Error while normalizing {path}: {ex}",
+                    path=resPath, ex=e,
                 )
                 errorOccurred = True
                 continue
@@ -223,13 +219,12 @@ def upgradeCalendarCollection(calPath, directory, cuaCache):
     returnValue(errorOccurred)
 
 
-
 @inlineCallbacks
 def upgradeCalendarHome(homePath, directory, cuaCache):
 
     errorOccurred = False
 
-    log.debug("Upgrading calendar home: %s" % (homePath,))
+    log.debug("Upgrading calendar home: {path}", path=homePath)
 
     try:
         for cal in os.listdir(homePath):
@@ -242,7 +237,7 @@ def upgradeCalendarHome(homePath, directory, cuaCache):
                 # Delete the old, now obsolete, notifications directory.
                 rmdir(calPath)
                 continue
-            log.debug("Upgrading calendar: %s" % (calPath,))
+            log.debug("Upgrading calendar: {path}", path=calPath)
             if not (yield upgradeCalendarCollection(calPath, directory, cuaCache)):
                 errorOccurred = True
 
@@ -265,13 +260,12 @@ def upgradeCalendarHome(homePath, directory, cuaCache):
                 except:
                     raise
     except Exception, e:
-        log.error("Failed to upgrade calendar home %s: %s" % (homePath, e))
+        log.error("Failed to upgrade calendar home {path}: {ex}", path=homePath, ex=e)
         raise
 
     returnValue(errorOccurred)
 
 
-
 @inlineCallbacks
 def upgrade_to_1(config, directory):
     """
@@ -284,7 +278,6 @@ def upgrade_to_1(config, directory):
             log.error(f)
         errorOccurred.append(True)
 
-
     def doProxyDatabaseMoveUpgrade(config, uid=-1, gid=-1):
         # See if the new one is already present
         oldFilename = ".db.calendaruserproxy"
@@ -319,11 +312,10 @@ def upgrade_to_1(config, directory):
             )
 
         log.debug(
-            "Moved the calendar user proxy database from '%s' to '%s'."
-            % (oldDbPath, newDbPath,)
+            "Moved the calendar user proxy database from '{old}' to '{new}'.",
+            old=oldDbPath, new=newDbPath
         )
 
-
     def moveCalendarHome(oldHome, newHome, uid=-1, gid=-1):
         if os.path.exists(newHome):
             # Both old and new homes exist; stop immediately to let the
@@ -338,20 +330,6 @@ def upgrade_to_1(config, directory):
         )
         os.rename(oldHome, newHome)
 
-
-    def createMailTokensDatabase(config, uid, gid):
-        # Cause the tokens db to be created on disk so we can set the
-        # permissions on it now
-        MailGatewayTokensDatabase(config.DataRoot).lookupByToken("")
-
-        dbPath = os.path.join(config.DataRoot, MailGatewayTokensDatabase.dbFilename)
-        if os.path.exists(dbPath):
-            os.chown(dbPath, uid, gid)
-
-        journalPath = "%s-journal" % (dbPath,)
-        if os.path.exists(journalPath):
-            os.chown(journalPath, uid, gid)
-
     cuaCache = {}
 
     docRoot = config.DocumentRoot
@@ -372,8 +350,8 @@ def upgrade_to_1(config, directory):
             # Now delete the on disk representation of principals
             rmdir(oldPrincipals)
             log.debug(
-                "Removed the old principal directory at '%s'."
-                % (oldPrincipals,)
+                "Removed the old principal directory at '{path}'.",
+                path=oldPrincipals,
             )
 
         calRoot = os.path.join(docRoot, "calendars")
@@ -383,7 +361,7 @@ def upgrade_to_1(config, directory):
 
             # Move calendar homes to new location:
 
-            log.warn("Moving calendar homes to %s" % (uidHomes,))
+            log.warn("Moving calendar homes to {path}", path=uidHomes)
 
             if os.path.exists(uidHomes):
                 for home in os.listdir(uidHomes):
@@ -467,7 +445,7 @@ def upgrade_to_1(config, directory):
                 os.chown(inboxItemsFile, uid, gid)
 
             if total:
-                log.warn("Processing %d calendar homes in %s" % (total, uidHomes))
+                log.warn("Processing {total} calendar homes in {path}", total=total, path=uidHomes)
 
                 # Upgrade calendar homes in the new location:
                 count = 0
@@ -492,18 +470,15 @@ def upgrade_to_1(config, directory):
                                     count += 1
                                     if count % 10 == 0:
                                         log.warn(
-                                            "Processed calendar home %d of %d"
-                                            % (count, total)
+                                            "Processed calendar home {count} of {total}",
+                                            count=count, total=total,
                                         )
                 log.warn("Done processing calendar homes")
 
-    createMailTokensDatabase(config, uid, gid)
-
     if errorOccurred:
         log.warn("Data upgrade encountered errors but will proceed; see error.log for details")
 
 
-
 @inlineCallbacks
 def normalizeCUAddrs(data, directory, cuaCache):
     """
@@ -547,7 +522,6 @@ def normalizeCUAddrs(data, directory, cuaCache):
     returnValue((newData, not newData == data))
 
 
-
 @inlineCallbacks
 def upgrade_to_2(config, directory):
 
@@ -563,10 +537,9 @@ def upgrade_to_2(config, directory):
         if os.path.exists(oldDbPath) and not os.path.exists(newDbPath):
             os.rename(oldDbPath, newDbPath)
 
-
     def flattenHome(calHome):
 
-        log.debug("Flattening calendar home: %s" % (calHome,))
+        log.debug("Flattening calendar home: {path}", path=calHome)
 
         try:
             for cal in os.listdir(calHome):
@@ -595,24 +568,23 @@ def upgrade_to_2(config, directory):
 #                                newPath = os.path.join(calHome, child)
 #                                if os.path.exists(newPath):
 #                                    newPath = os.path.join(calHome, str(uuid.uuid4()))
-#                                log.debug("Moving up calendar: %s" % (childCollection,))
+#                                log.debug("Moving up calendar: {path}", path=childCollection)
 #                                os.rename(childCollection, newPath)
 #                            else:
 #                                scanCollection(childCollection)
 
                 if os.path.isdir(calPath):
-                    # log.debug("Regular collection scan: %s" % (calPath,))
+                    # log.debug("Regular collection scan: {path}", path=calPath)
                     # scanCollection(calPath)
-                    log.warn("Regular collection hidden: %s" % (calPath,))
+                    log.warn("Regular collection hidden: {path}", path=calPath)
                     os.rename(calPath, os.path.join(calHome, ".collection." + os.path.basename(calPath)))
 
         except Exception, e:
-            log.error("Failed to upgrade calendar home %s: %s" % (calHome, e))
+            log.error("Failed to upgrade calendar home {path}: {ex}", path=calHome, ex=e)
             return succeed(False)
 
         return succeed(True)
 
-
     def flattenHomes():
         """
         Make sure calendars inside regular collections are all moved to the top level.
@@ -653,7 +625,6 @@ def upgrade_to_2(config, directory):
         raise UpgradeError("Data upgrade failed, see error.log for details")
 
 
-
 def upgradeResourcesXML(resourcesFilePath):
     """
     Convert the old XML format to the twext.who.xml format
@@ -708,7 +679,6 @@ def upgradeResourcesXML(resourcesFilePath):
     resourcesFilePath.setContent(etreeToString(directoryNode, "utf-8"))
 
 
-
 def upgradeAugmentsXML(augmentsFilePath):
     """
     Convert the old augments XML auto-schedule related elements to the twext.who.xml format
@@ -777,7 +747,7 @@ def upgradeData(config, directory):
     docRoot = config.DocumentRoot
 
     if not os.path.exists(docRoot):
-        log.info("DocumentRoot (%s) doesn't exist; skipping migration" % (docRoot,))
+        log.info("DocumentRoot ({path}) doesn't exist; skipping migration", path=docRoot)
         return
 
     versionFilePath = os.path.join(docRoot, ".calendarserver_version")
@@ -789,22 +759,22 @@ def upgradeData(config, directory):
                 onDiskVersion = int(versionFile.read().strip())
         except IOError:
             log.error(
-                "Cannot open %s; skipping migration" %
-                (versionFilePath,)
+                "Cannot open {path}; skipping migration",
+                path=versionFilePath,
             )
         except ValueError:
             log.error(
-                "Invalid version number in %s; skipping migration" %
-                (versionFilePath,)
+                "Invalid version number in {path}; skipping migration",
+                path=versionFilePath,
             )
 
     uid, gid = getCalendarServerIDs(config)
 
     for version, method in upgradeMethods:
         if onDiskVersion < version:
-            log.warn("Upgrading to version %d" % (version,))
+            log.warn("Upgrading to version {vers}", vers=version)
             (yield method(config, directory))
-            log.warn("Upgraded to version %d" % (version,))
+            log.warn("Upgraded to version {vers}", vers=version)
             with open(versionFilePath, "w") as verFile:
                 verFile.write(str(version))
             os.chown(versionFilePath, uid, gid)
@@ -814,7 +784,6 @@ def upgradeData(config, directory):
         os.remove(triggerPath)
 
 
-
 class UpgradeError(RuntimeError):
     """
     Generic upgrade error.
@@ -822,7 +791,6 @@ class UpgradeError(RuntimeError):
     pass
 
 
-
 #
 # Utility functions
 #
@@ -838,7 +806,7 @@ def updateFreeBusyHref(href, directory):
     record = yield directory.recordWithShortName(directory.oldNameToRecordType(recordType), shortName)
     if record is None:
         # We will simply ignore this and not write out an fb-set entry
-        log.error("Can't update free-busy href; %s is not in the directory" % shortName)
+        log.error("Can't update free-busy href; {user} is not in the directory", user=shortName)
         returnValue("")
 
     uid = record.uid
@@ -846,7 +814,6 @@ def updateFreeBusyHref(href, directory):
     returnValue(newHref)
 
 
-
 @inlineCallbacks
 def updateFreeBusySet(value, directory):
 
@@ -888,7 +855,6 @@ def updateFreeBusySet(value, directory):
     returnValue(None)  # no update required
 
 
-
 def makeDirsUserGroup(path, uid=-1, gid=-1):
     parts = path.split("/")
     if parts[0] == "":  # absolute path
@@ -904,7 +870,6 @@ def makeDirsUserGroup(path, uid=-1, gid=-1):
             os.chown(path, uid, gid)
 
 
-
 def archive(config, srcPath, uid, gid):
     """
     Move srcPath into dataroot/archived, giving the destination a unique
@@ -934,7 +899,6 @@ def archive(config, srcPath, uid, gid):
         os.remove(srcPath)
 
 
-
 DELETECHARS = ''.join(chr(i) for i in xrange(32) if i not in (9, 10, 13))
 
 
@@ -954,7 +918,6 @@ def removeIllegalCharacters(data):
         return data, False
 
 
-
 # Deferred
 def migrateFromOD(directory):
     #
@@ -982,18 +945,19 @@ def migrateFromOD(directory):
     return migrateResources(EnhancedDirectoryService(), directory)
 
 
-
 @inlineCallbacks
 def migrateAutoSchedule(config, directory):
     # Fetch the autoSchedule assignments from resourceinfo.sqlite and store
     # the values in augments
     augmentService = None
-    if config.AugmentService.type:
-        augmentClass = namedClass(config.AugmentService.type)
-        try:
-            augmentService = augmentClass(**config.AugmentService.params)
-        except:
-            log.error("Could not start augment service")
+    serviceClass = {
+        "xml": "twistedcaldav.directory.augment.AugmentXMLDB",
+    }
+    augmentClass = namedClass(serviceClass[config.AugmentService.type])
+    try:
+        augmentService = augmentClass(**config.AugmentService.params)
+    except:
+        log.error("Could not start augment service")
 
     if augmentService:
         augmentRecords = []
@@ -1005,23 +969,23 @@ def migrateAutoSchedule(config, directory):
                 "select GUID, AUTOSCHEDULE from RESOURCEINFO"
             )
             for uid, autoSchedule in results:
-                record = yield directory.recordWithUID(uid)
-                if record is not None:
-                    augmentRecord = (
-                        yield augmentService.getAugmentRecord(
-                            uid,
-                            directory.recordTypeToOldName(record.recordType)
+                if uid is not None:
+                    record = yield directory.recordWithUID(uid)
+                    if record is not None:
+                        augmentRecord = (
+                            yield augmentService.getAugmentRecord(
+                                uid,
+                                directory.recordTypeToOldName(record.recordType)
+                            )
                         )
-                    )
-                    augmentRecord.autoScheduleMode = (
-                        "automatic" if autoSchedule else "default"
-                    )
-                    augmentRecords.append(augmentRecord)
+                        augmentRecord.autoScheduleMode = (
+                            "automatic" if autoSchedule else "default"
+                        )
+                        augmentRecords.append(augmentRecord)
 
             if augmentRecords:
                 yield augmentService.addAugmentRecords(augmentRecords)
-            log.warn("Migrated %d auto-schedule settings" % (len(augmentRecords),))
-
+            log.warn("Migrated {len} auto-schedule settings", len=len(augmentRecords))
 
 
 def loadDelegatesFromXMLintoProxyDB(xmlFile, service):
@@ -1029,13 +993,11 @@ def loadDelegatesFromXMLintoProxyDB(xmlFile, service):
     return loader.updateProxyDB(service)
 
 
-
 def loadDelegatesFromXMLintoStore(xmlFile, store):
     loader = XMLCalendarUserProxyLoader(xmlFile)
     return loader.updateProxyStore(store)
 
 
-
 @inlineCallbacks
 def migrateDelegatesToStore(store):
     """
@@ -1074,13 +1036,13 @@ def migrateDelegatesToStore(store):
     yield txn.commit()
 
     # Remove the old file
+    service.close()
     os.remove(service.dbpath)
     journalPath = service.dbpath + "-journal"
     if os.path.exists(journalPath):
         os.remove(journalPath)
 
 
-
 class UpgradeFileSystemFormatStep(object):
     """
     Upgrade filesystem from previous versions.
@@ -1093,7 +1055,6 @@ class UpgradeFileSystemFormatStep(object):
         self.config = config
         self.store = store
 
-
     @inlineCallbacks
     def doUpgrade(self):
         """
@@ -1115,7 +1076,6 @@ class UpgradeFileSystemFormatStep(object):
 
         returnValue(None)
 
-
     def stepWithResult(self, result):
         """
         Execute the step.
@@ -1123,7 +1083,6 @@ class UpgradeFileSystemFormatStep(object):
         return self.doUpgrade()
 
 
-
 class PostDBImportStep(object):
     """
     Step which runs after database import but before workers are spawned
@@ -1145,7 +1104,6 @@ class PostDBImportStep(object):
         self.config = config
         self.doPostImport = doPostImport
 
-
     @inlineCallbacks
     def stepWithResult(self, result):
         if self.doPostImport:
@@ -1194,7 +1152,6 @@ class PostDBImportStep(object):
             # Migrate mail tokens from sqlite to store
             yield migrateTokensToStore(self.config.DataRoot, self.store)
 
-
     @inlineCallbacks
     def processInboxItems(self):
         """
@@ -1223,24 +1180,24 @@ class PostDBImportStep(object):
                 totalItems = len(itemsToProcess)
                 ignoreUUIDs = set()
                 for ctr, inboxItem in enumerate(itemsToProcess):
-                    log.info("Processing %d/%d inbox item: %s" % (ctr + 1, totalItems, inboxItem,))
+                    log.info("Processing {ctr}/{total} inbox item: {item}", ctr=ctr + 1, total=totalItems, item=inboxItem)
                     ignore, uuid, ignore, fileName = inboxItem.rsplit("/", 3)
 
                     if uuid in ignoreUUIDs:
-                        log.debug("Ignored inbox item - uuid ignored: %s" % (inboxItem,))
+                        log.debug("Ignored inbox item - uuid ignored: {item}", item=inboxItem)
                         inboxItems.remove(inboxItem)
                         continue
 
                     record = yield directory.recordWithUID(uuid)
                     if record is None:
-                        log.debug("Ignored inbox item - no record: %s" % (inboxItem,))
+                        log.debug("Ignored inbox item - no record: {item}", item=inboxItem)
                         inboxItems.remove(inboxItem)
                         ignoreUUIDs.add(uuid)
                         continue
 
                     principal = yield principalCollection.principalForRecord(record)
                     if principal is None or not isinstance(principal, DirectoryCalendarPrincipalResource):
-                        log.debug("Ignored inbox item - no principal: %s" % (inboxItem,))
+                        log.debug("Ignored inbox item - no principal: {item}", item=inboxItem)
                         inboxItems.remove(inboxItem)
                         ignoreUUIDs.add(uuid)
                         continue
@@ -1256,7 +1213,7 @@ class PostDBImportStep(object):
                     try:
                         calendarHome = yield principal.calendarHome(request)
                         if calendarHome is None:
-                            log.debug("Ignored inbox item - no calendar home: %s" % (inboxItem,))
+                            log.debug("Ignored inbox item - no calendar home: {item}", item=inboxItem)
                             inboxItems.remove(inboxItem)
                             ignoreUUIDs.add(uuid)
                             continue
@@ -1284,13 +1241,13 @@ class PostDBImportStep(object):
                                     )
                                 except Exception, e:
                                     log.error(
-                                        "Error processing inbox item: %s (%s)"
-                                        % (inboxItem, e)
+                                        "Error processing inbox item: {item} ({ex})",
+                                        item=inboxItem, ex=e,
                                     )
                             else:
-                                log.debug("Ignored inbox item - no resource: %s" % (inboxItem,))
+                                log.debug("Ignored inbox item - no resource: {item}", item=inboxItem)
                         else:
-                            log.debug("Ignored inbox item - no inbox: %s" % (inboxItem,))
+                            log.debug("Ignored inbox item - no inbox: {item}", item=inboxItem)
 
                         inboxItems.remove(inboxItem)
 
@@ -1301,7 +1258,7 @@ class PostDBImportStep(object):
             # FIXME: Some generic exception handlers to deal with unexpected errors that for some reason
             # we are not logging properly.
             except Exception, e:
-                log.error("Exception during inbox item processing: %s" % (e,))
+                log.error("Exception during inbox item processing: {ex}", ex=e)
 
             except:
                 log.error("Unknown exception during inbox item processing.")
@@ -1321,7 +1278,6 @@ class PostDBImportStep(object):
                     os.remove(inboxItemsList)
                     log.info("Completed inbox item processing.")
 
-
     @inlineCallbacks
     def processInboxItem(
         self, root, directory, principal, request, inbox,
@@ -1332,7 +1288,7 @@ class PostDBImportStep(object):
         the inbox item.
         """
 
-        log.debug("Processing inbox item %s" % (inboxItem,))
+        log.debug("Processing inbox item {item}", item=inboxItem)
 
         txn = request._newStoreTransaction
 
@@ -1347,7 +1303,7 @@ class PostDBImportStep(object):
         if calendar.mainType() is not None:
             try:
                 method = calendar.propertyValue("METHOD")
-                log.info("Inbox item method is %s" % (method,))
+                log.info("Inbox item method is {method}", method=method)
             except ValueError:
                 returnValue(None)
 
@@ -1370,7 +1326,7 @@ class PostDBImportStep(object):
                 internal_request=False, noAttendeeRefresh=True
             )
         else:
-            log.warn("Removing invalid inbox item: %s" % (uri,))
+            log.warn("Removing invalid inbox item: {uri}", uri=uri)
 
         #
         # Remove item
