@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.caldav.datastore.test.test_file -*-
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -79,7 +79,6 @@ CalendarStoreTransaction = CommonStoreTransaction
 IGNORE_NAMES = ('dropbox', 'notification', 'freebusy')
 
 
-
 class CalendarHome(CommonHome):
     implements(ICalendarHome)
 
@@ -102,7 +101,6 @@ class CalendarHome(CommonHome):
     createCalendarWithName = CommonHome.createChildWithName
     removeCalendarWithName = CommonHome.removeChildWithName
 
-
     def childWithName(self, name):
         if name in IGNORE_NAMES:
             # "dropbox" is a file storage area, not a calendar.
@@ -112,7 +110,6 @@ class CalendarHome(CommonHome):
 
     calendarWithName = childWithName
 
-
     def children(self):
         """
         Return a generator of the child resource objects.
@@ -134,7 +131,6 @@ class CalendarHome(CommonHome):
     listCalendars = listChildren
     loadCalendars = CommonHome.loadChildren
 
-
     @inlineCallbacks
     def hasCalendarResourceUIDSomewhereElse(self, uid, ok_object, type):
 
@@ -148,14 +144,12 @@ class CalendarHome(CommonHome):
 
         returnValue(None)
 
-
     @inlineCallbacks
     def getCalendarResourcesForUID(self, uid):
 
         results = (yield self.objectResourcesWithUID(uid, ("inbox",)))
         returnValue(results)
 
-
     @inlineCallbacks
     def calendarObjectWithDropboxID(self, dropboxID):
         """
@@ -166,7 +160,6 @@ class CalendarHome(CommonHome):
                 if dropboxID == (yield calendarObject.dropboxID()):
                     returnValue(calendarObject)
 
-
     @inlineCallbacks
     def getAllDropboxIDs(self):
         dropboxIDs = []
@@ -181,12 +174,10 @@ class CalendarHome(CommonHome):
                     dropboxIDs.append(dropboxID)
         returnValue(dropboxIDs)
 
-
     @property
     def _calendarStore(self):
         return self._dataStore
 
-
     def createdHome(self):
 
         # Check whether components type must be separate
@@ -204,7 +195,6 @@ class CalendarHome(CommonHome):
 
         self.createCalendarWithName("inbox")
 
-
     def ensureDefaultCalendarsExist(self):
         """
         Double check that we have calendars supporting at least VEVENT and VTODO,
@@ -234,7 +224,6 @@ class CalendarHome(CommonHome):
             _requireCalendarWithType("VTODO", "tasks")
 
 
-
 class Calendar(CommonHomeChild):
     """
     File-based implementation of L{ICalendar}.
@@ -261,7 +250,6 @@ class Calendar(CommonHomeChild):
         self._index = Index(self)
         self._objectResourceClass = CalendarObject
 
-
     @property
     def _calendarHome(self):
         return self._home
@@ -275,15 +263,12 @@ class Calendar(CommonHomeChild):
     createCalendarObjectWithName = CommonHomeChild.createObjectResourceWithName
     calendarObjectsSinceToken = CommonHomeChild.objectResourcesSinceToken
 
-
     def _createCalendarObjectWithNameInternal(self, name, component, internal_state, options=None):
         return self.createCalendarObjectWithName(name, component, options)
 
-
     def calendarObjectsInTimeRange(self, start, end, timeZone):
         raise NotImplementedError()
 
-
     def setSupportedComponents(self, supported_components):
         """
         Update the private property with the supported components. Technically this should only happen once
@@ -296,12 +281,10 @@ class Calendar(CommonHomeChild):
         elif pname in self.properties():
             del self.properties()[pname]
 
-
     def getSupportedComponents(self):
         result = str(self.properties().get(PropertyName.fromElement(customxml.TwistedCalendarSupportedComponents), ""))
         return result if result else None
 
-
     def isSupportedComponent(self, componentType):
         supported = self.getSupportedComponents()
         if supported:
@@ -309,7 +292,6 @@ class Calendar(CommonHomeChild):
         else:
             return True
 
-
     def initPropertyStore(self, props):
         # Setup peruser special properties
         props.setSpecialProperties(
@@ -324,14 +306,12 @@ class Calendar(CommonHomeChild):
             (),
         )
 
-
     def contentType(self):
         """
         The content type of Calendar objects is text/calendar.
         """
         return MimeType.fromString("text/calendar; charset=utf-8")
 
-
     def splitCollectionByComponentTypes(self):
         """
         If the calendar contains iCalendar data with different component types, then split it into separate collections
@@ -342,7 +322,6 @@ class Calendar(CommonHomeChild):
         # TODO: implement this for filestore
         pass
 
-
     def _countComponentTypes(self):
         """
         Count each component type in this calendar.
@@ -354,7 +333,6 @@ class Calendar(CommonHomeChild):
         result = tuple([(componentType, componentCount) for componentType, componentCount in sorted(rows, key=lambda x:x[0])])
         return result
 
-
     def _splitComponentType(self, component):
         """
         Create a new calendar and move all components of the specified component type into the new one.
@@ -367,7 +345,6 @@ class Calendar(CommonHomeChild):
         # TODO: implement this for filestore
         pass
 
-
     def _transferSharingDetails(self, newcalendar, component):
         """
         If the current calendar is shared, make the new calendar shared in the same way, but tweak the name.
@@ -376,7 +353,6 @@ class Calendar(CommonHomeChild):
         # TODO: implement this for filestore
         pass
 
-
     def _transferCalendarObjects(self, newcalendar, component):
         """
         Move all calendar components of the specified type to the specified calendar.
@@ -386,7 +362,6 @@ class Calendar(CommonHomeChild):
         pass
 
 
-
 class CalendarObject(CommonObjectResource, CalendarObjectBase):
     """
     @ivar _path: The path of the .ics file on disk
@@ -406,16 +381,13 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
             self.scheduleEtags = metadata.get("scheduleEtags", "")
             self.hasPrivateComment = metadata.get("hasPrivateComment", False)
 
-
     @property
     def _calendar(self):
         return self._parentCollection
 
-
     def calendar(self):
         return self._calendar
 
-
     @writeOperation
     def setComponent(self, component, inserting=False):
 
@@ -462,7 +434,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
 
         self._calendar.notifyChanged()
 
-
     def component(self):
         """
         Read calendar data and validate/fix it. Do not raise a store error here if there are unfixable
@@ -484,14 +455,13 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
         fixed, unfixed = component.validCalendarData(doFix=True, doRaise=False)
 
         if unfixed:
-            self.log.error("Calendar data at %s had unfixable problems:\n  %s" % (self._path.path, "\n  ".join(unfixed),))
+            self.log.error("Calendar data at {path} had unfixable problems:\n  {problems}", path=self._path.path, problems="\n  ".join(unfixed))
 
         if fixed:
-            self.log.error("Calendar data at %s had fixable problems:\n  %s" % (self._path.path, "\n  ".join(fixed),))
+            self.log.error("Calendar data at {path} had fixable problems:\n  {problems}", path=self._path.path, problems="\n  ".join(fixed))
 
         return component
 
-
     def componentForUser(self, user_uuid=None):
         """
         Return the iCalendar component filtered for the specified user's per-user data.
@@ -507,7 +477,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
             user_uuid = self._parentCollection.viewerHome().uid()
         return PerUserDataFilter(user_uuid).filter(self.component().duplicate())
 
-
     def _text(self):
         if self._objectText is not None:
             return self._objectText
@@ -545,23 +514,19 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
         self._objectText = text
         return text
 
-
     def uid(self):
         if not hasattr(self, "_uid"):
             self._uid = self.component().resourceUID()
         return self._uid
 
-
     def componentType(self):
         if not hasattr(self, "_componentType"):
             self._componentType = self.component().mainType()
         return self._componentType
 
-
     def organizer(self):
         return self.component().getOrganizer()
 
-
     def getMetadata(self):
         metadata = {}
         metadata["accessMode"] = self.accessMode
@@ -571,11 +536,9 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
         metadata["hasPrivateComment"] = self.hasPrivateComment
         return metadata
 
-
     def _get_accessMode(self):
         return str(self.properties().get(PropertyName.fromElement(customxml.TwistedCalendarAccessProperty), ""))
 
-
     def _set_accessMode(self, value):
         pname = PropertyName.fromElement(customxml.TwistedCalendarAccessProperty)
         if value:
@@ -595,7 +558,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
             prop = str(prop) == "true"
         return prop
 
-
     def _set_isScheduleObject(self, value):
         pname = PropertyName.fromElement(customxml.TwistedSchedulingObjectResource)
         if value is not None:
@@ -608,7 +570,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
     def _get_scheduleTag(self):
         return str(self.properties().get(PropertyName.fromElement(caldavxml.ScheduleTag), ""))
 
-
     def _set_scheduleTag(self, value):
         pname = PropertyName.fromElement(caldavxml.ScheduleTag)
         if value:
@@ -621,7 +582,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
     def _get_scheduleEtags(self):
         return tuple([str(etag) for etag in self.properties().get(PropertyName.fromElement(customxml.TwistedScheduleMatchETags), customxml.TwistedScheduleMatchETags()).children])
 
-
     def _set_scheduleEtags(self, value):
         if value:
             etags = [davxml.GETETag.fromString(etag) for etag in value]
@@ -637,7 +597,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
     def _get_hasPrivateComment(self):
         return PropertyName.fromElement(customxml.TwistedCalendarHasPrivateCommentsProperty) in self.properties()
 
-
     def _set_hasPrivateComment(self, value):
         pname = PropertyName.fromElement(customxml.TwistedCalendarHasPrivateCommentsProperty)
         if value:
@@ -647,19 +606,15 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
 
     hasPrivateComment = property(_get_hasPrivateComment, _set_hasPrivateComment)
 
-
     def addAttachment(self, pathpattern, rids, content_type, filename, stream):
         raise NotImplementedError
 
-
     def updateAttachment(self, pathpattern, managed_id, content_type, filename, stream):
         raise NotImplementedError
 
-
     def removeAttachment(self, rids, managed_id):
         raise NotImplementedError
 
-
     @inlineCallbacks
     def createAttachmentWithName(self, name):
         """
@@ -671,7 +626,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
         self._attachments[name] = attachment
         returnValue(attachment)
 
-
     @inlineCallbacks
     def removeAttachmentWithName(self, name):
         """
@@ -689,7 +643,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
         # Adjust quota
         self._calendar._home.adjustQuotaUsedBytes(-oldSize)
 
-
     @inlineCallbacks
     def attachmentWithName(self, name):
         # Attachments can be local or remote, but right now we only care about
@@ -707,16 +660,13 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
             # FIXME: test for non-existent attachment.
             returnValue(None)
 
-
     def attendeesCanManageAttachments(self):
         return self.component().hasPropertyInAnyComponent("X-APPLE-DROPBOX")
 
-
     def dropboxID(self):
         # NB: Deferred
         return dropboxIDFromCalendarObject(self)
 
-
     @inlineCallbacks
     def _dropboxPath(self):
         dropboxPath = self._parentCollection._home._path.child(
@@ -726,7 +676,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
             dropboxPath.makedirs()
         returnValue(dropboxPath)
 
-
     @inlineCallbacks
     def attachments(self):
         # See comment on attachmentWithName.
@@ -737,7 +686,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
              if not name.startswith(".")]
         )
 
-
     def initPropertyStore(self, props):
         # Setup peruser special properties
         props.setSpecialProperties(
@@ -754,7 +702,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
             (),
         )
 
-
     # IDataStoreObject
     def contentType(self):
         """
@@ -763,7 +710,6 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
         return MimeType.fromString("text/calendar; charset=utf-8")
 
 
-
 class AttachmentStorageTransport(StorageTransportBase):
 
     def __init__(self, attachment, contentType, dispositionName):
@@ -781,12 +727,10 @@ class AttachmentStorageTransport(StorageTransportBase):
 
         self._txn.postAbort(self.aborted)
 
-
     @property
     def _txn(self):
         return self._attachment._txn
 
-
     def aborted(self):
         """
         Transaction aborted - clean up temp files.
@@ -794,13 +738,11 @@ class AttachmentStorageTransport(StorageTransportBase):
         if self._path.exists():
             self._path.remove()
 
-
     def write(self, data):
         # FIXME: multiple chunks
         self._file.write(data)
         return super(AttachmentStorageTransport, self).write(data)
 
-
     def loseConnection(self):
         home = self._attachment._calendarObject._calendar._home
         oldSize = self._attachment.size()
@@ -815,8 +757,7 @@ class AttachmentStorageTransport(StorageTransportBase):
 
         # Check overall user quota
         allowed = home.quotaAllowedBytes()
-        if allowed is not None and allowed < (home.quotaUsedBytes()
-                                              + (newSize - oldSize)):
+        if allowed is not None and allowed < (home.quotaUsedBytes() + (newSize - oldSize)):
             self._path.remove()
             return fail(QuotaExceeded())
 
@@ -835,7 +776,6 @@ class AttachmentStorageTransport(StorageTransportBase):
         return succeed(None)
 
 
-
 class Attachment(FileMetaDataMixin):
     """
     An L{Attachment} is a container for the data associated with a I{locally-
@@ -853,37 +793,30 @@ class Attachment(FileMetaDataMixin):
         self._name = name
         self._dropboxPath = dropboxPath
 
-
     @property
     def _txn(self):
         return self._calendarObject._txn
 
-
     def name(self):
         return self._name
 
-
     def properties(self):
         home = self._calendarObject._parentCollection._home
         uid = home.uid()
         propStoreClass = home._dataStore._propertyStoreClass
         return propStoreClass(uid, lambda: self._path)
 
-
     def store(self, contentType, dispositionName=None):
         return AttachmentStorageTransport(self, contentType, dispositionName)
 
-
     def retrieve(self, protocol):
         return AttachmentRetrievalTransport(self._path).start(protocol)
 
-
     @property
     def _path(self):
         return self._dropboxPath.child(self.name())
 
 
-
 class CalendarStubResource(CommonStubResource):
     """
     Just enough resource to keep the calendar's sql DB classes going.
@@ -892,11 +825,11 @@ class CalendarStubResource(CommonStubResource):
     def isCalendarCollection(self):
         return True
 
-
     def getChild(self, name):
         calendarObject = self.resource.calendarObjectWithName(name)
         if calendarObject:
             class ChildResource(object):
+
                 def __init__(self, calendarObject):
                     self.calendarObject = calendarObject
 
@@ -908,12 +841,12 @@ class CalendarStubResource(CommonStubResource):
             return None
 
 
-
 class Index(object):
     #
     # OK, here's where we get ugly.
     # The index code needs to be rewritten also, but in the meantime...
     #
+
     def __init__(self, calendar):
         self.calendar = calendar
         stubResource = CalendarStubResource(calendar)
@@ -923,7 +856,6 @@ class Index(object):
             indexClass = OldIndex
         self._oldIndex = indexClass(stubResource)
 
-
     def calendarObjects(self):
         calendar = self.calendar
         for name, uid, componentType in self._oldIndex.bruteForceSearch():
