@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_index -*-
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -69,10 +69,10 @@ schema_version = "10"
 collection_types = {"Calendar": "Regular Calendar Collection", "iTIP": "iTIP Calendar Collection"}
 
 icalfbtype_to_indexfbtype = {
-    "FREE"            : 'F',
-    "BUSY"            : 'B',
+    "FREE": 'F',
+    "BUSY": 'B',
     "BUSY-UNAVAILABLE": 'U',
-    "BUSY-TENTATIVE"  : 'T',
+    "BUSY-TENTATIVE": 'T',
 }
 indexfbtype_to_icalfbtype = dict([(v, k) for k, v in icalfbtype_to_indexfbtype.iteritems()])
 
@@ -95,6 +95,8 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         db_filename = self.resource.fp.child(db_basename).path
         super(AbstractCalendarIndex, self).__init__(db_filename, False)
 
+        self.resource._txn.postCommit(self._db_close)
+        self.resource._txn.postAbort(self._db_close)
 
     def create(self):
         """
@@ -102,7 +104,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         """
         self._db()
 
-
     def reserveUID(self, uid):
         """
         Reserve a UID for this index's resource.
@@ -111,7 +112,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         """
         raise NotImplementedError
 
-
     def unreserveUID(self, uid):
         """
         Unreserve a UID for this index's resource.
@@ -120,7 +120,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         """
         raise NotImplementedError
 
-
     def isReservedUID(self, uid):
         """
         Check to see whether a UID is reserved.
@@ -129,7 +128,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         """
         raise NotImplementedError
 
-
     def isAllowedUID(self, uid, *names):
         """
         Checks to see whether to allow an operation with adds the the specified
@@ -144,7 +142,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         """
         raise NotImplementedError
 
-
     def resourceNamesForUID(self, uid):
         """
         Looks up the names of the resources with the given UID.
@@ -162,7 +159,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
             name_utf8 = name.encode("utf-8")
             if name is not None and self.resource.getChild(name_utf8) is None:
                 # Clean up
-                log.error("Stale resource record found for child %s with UID %s in %s" % (name, uid, self.resource))
+                log.error("Stale resource record found for child {name} with UID {uid} in {rsrc!r}", name=name, uid=uid, resource=self.resource)
                 self._delete_from_db(name, uid, False)
                 self._db_commit()
             else:
@@ -170,7 +167,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
 
         return resources
 
-
     def resourceNameForUID(self, uid):
         """
         Looks up the name of the resource with the given UID.
@@ -185,7 +181,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
 
         return result
 
-
     def resourceUIDForName(self, name):
         """
         Looks up the UID of the resource with the given name.
@@ -197,14 +192,12 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
 
         return uid
 
-
     def componentTypeCounts(self):
         """
         Count each type of component.
         """
         return self._db_execute("select TYPE, COUNT(TYPE) from RESOURCE group by TYPE")
 
-
     def addResource(self, name, calendar, fast=False, reCreate=False):
         """
         Adding or updating an existing resource.
@@ -223,7 +216,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         if not fast:
             self._db_commit()
 
-
     def deleteResource(self, name):
         """
         Remove this resource from the index.
@@ -235,7 +227,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
             self._delete_from_db(name, uid)
             self._db_commit()
 
-
     def resourceExists(self, name):
         """
         Determines whether the specified resource name exists in the index.
@@ -245,7 +236,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         uid = self._db_value_for_sql("select UID from RESOURCE where NAME = :1", name)
         return uid is not None
 
-
     def resourcesExist(self, names):
         """
         Determines whether the specified resource name exists in the index.
@@ -261,16 +251,14 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         results = self._db_values_for_sql(statement, *names)
         return results
 
-
     def testAndUpdateIndex(self, minDate):
         # Find out if the index is expanded far enough
         names = self.notExpandedBeyond(minDate)
         # Actually expand recurrence max
         for name in names:
-            self.log.info("Search falls outside range of index for %s %s" % (name, minDate))
+            self.log.info("Search falls outside range of index for {name} {date}", name=name, date=minDate)
             self.reExpandResource(name, minDate)
 
-
     def whatchanged(self, revision):
 
         results = [(name.encode("utf-8"), deleted) for name, deleted in self._db_execute("select NAME, DELETED from REVISIONS where REVISION > :1", revision)]
@@ -291,13 +279,11 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
 
         return (changed, deleted, invalid)
 
-
     def lastRevision(self):
         return self._db_value_for_sql(
             "select REVISION from REVISION_SEQUENCE"
         )
 
-
     def bumpRevision(self, fast=False):
         self._db_execute(
             """
@@ -311,7 +297,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
             """,
         )
 
-
     def indexedSearch(self, filter, useruid="", fbtype=False):
         """
         Finds resources matching the given qualifiers.
@@ -380,13 +365,14 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
                     del row[9]
                 results.append(row)
             else:
-                log.error("Calendar resource %s is missing from %s. Removing from index."
-                          % (name, self.resource))
+                log.error(
+                    "Calendar resource {name} is missing from {rsrc!r}. Removing from index.",
+                    name=name, rsrc=self.resource,
+                )
                 self.deleteResource(name)
 
         return results
 
-
     def bruteForceSearch(self):
         """
         List the whole index and tests for existence, updating the index
@@ -403,20 +389,19 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
             if self.resource.getChild(name.encode("utf-8")):
                 results.append(row)
             else:
-                log.error("Calendar resource %s is missing from %s. Removing from index."
-                          % (name, self.resource))
-                self.deleteResource(name)
+                log.error(
+                    "Calendar resource {name} is missing from {rsrc!r}. Removing from index.",
+                    name=name, rsrc=self.resource,
+                )
 
         return results
 
-
     def _db_version(self):
         """
         @return: the schema version assigned to this index.
         """
         return schema_version
 
-
     def _add_to_db(self, name, calendar, cursor=None, expand_until=None, reCreate=False):
         """
         Records the given calendar resource in the index with the given name.
@@ -430,7 +415,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         """
         raise NotImplementedError
 
-
     def _delete_from_db(self, name, uid, dorevision=True):
         """
         Deletes the specified entry from all dbs.
@@ -440,7 +424,6 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
         raise NotImplementedError
 
 
-
 def sqlcalendarquery(filter, calendarid=None, userid=None, freebusy=False):
     """
     Convert the supplied calendar-query into a partial SQL statement.
@@ -458,7 +441,6 @@ def sqlcalendarquery(filter, calendarid=None, userid=None, freebusy=False):
         return None
 
 
-
 class CalendarIndex (AbstractCalendarIndex):
     """
     Calendar index - abstract class for indexer that indexes calendar objects in a collection.
@@ -471,7 +453,6 @@ class CalendarIndex (AbstractCalendarIndex):
         """
         super(CalendarIndex, self).__init__(resource)
 
-
     def _db_init_data_tables_base(self, q, uidunique):
         """
         Initialise the underlying database tables.
@@ -634,7 +615,6 @@ class CalendarIndex (AbstractCalendarIndex):
             """
         )
 
-
     def _db_can_upgrade(self, old_version):
         """
         Can we do an in-place upgrade
@@ -643,7 +623,6 @@ class CalendarIndex (AbstractCalendarIndex):
         # v10 is a big change - no upgrade possible
         return False
 
-
     def _db_upgrade_data_tables(self, q, old_version):
         """
         Upgrade the data from an older version of the DB.
@@ -652,7 +631,6 @@ class CalendarIndex (AbstractCalendarIndex):
         # v10 is a big change - no upgrade possible
         pass
 
-
     def notExpandedBeyond(self, minDate):
         """
         Gives all resources which have not been expanded beyond a given date
@@ -660,7 +638,6 @@ class CalendarIndex (AbstractCalendarIndex):
         """
         return self._db_values_for_sql("select NAME from RESOURCE where RECURRANCE_MAX < :1", pyCalendarToSQLTimestamp(minDate))
 
-
     def reExpandResource(self, name, expand_until):
         """
         Given a resource name, remove it from the database and re-add it
@@ -670,7 +647,6 @@ class CalendarIndex (AbstractCalendarIndex):
         self._add_to_db(name, calendar, expand_until=expand_until, reCreate=True)
         self._db_commit()
 
-
     def _add_to_db(self, name, calendar, cursor=None, expand_until=None, reCreate=False):
         """
         Records the given calendar resource in the index with the given name.
@@ -732,7 +708,7 @@ class CalendarIndex (AbstractCalendarIndex):
             instances = calendar.expandTimeRanges(expand, ignoreInvalidInstances=reCreate)
             recurrenceLimit = instances.limit
         except InvalidOverriddenInstanceError, e:
-            log.error("Invalid instance %s when indexing %s in %s" % (e.rid, name, self.resource,))
+            log.error("Invalid instance {rid} when indexing {name} in {rsrc!r}", rid=e.rid, name=name, rsrc=self.resource)
             raise
 
         # Now coerce indexing to off if needed
@@ -831,7 +807,6 @@ class CalendarIndex (AbstractCalendarIndex):
             """, name, self.bumpRevision(fast=True), 'N',
         )
 
-
     def _delete_from_db(self, name, uid, dorevision=True):
         """
         Deletes the specified entry from all dbs.
@@ -848,7 +823,6 @@ class CalendarIndex (AbstractCalendarIndex):
             )
 
 
-
 def wrapInDeferred(f):
     def _(*args, **kwargs):
         return maybeDeferred(f, *args, **kwargs)
@@ -856,7 +830,6 @@ def wrapInDeferred(f):
     return _
 
 
-
 class MemcachedUIDReserver(CachePoolUserMixIn):
     log = Logger()
 
@@ -864,18 +837,16 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
         self.index = index
         self._cachePool = cachePool
 
-
     def _key(self, uid):
         return 'reservation:%s' % (
             hashlib.md5('%s:%s' % (uid,
                                    self.index.resource.fp.path)).hexdigest())
 
-
     def reserveUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log.debug("Reserving UID %r @ %r" % (
-            uid,
-            self.index.resource.fp.path)
+        self.log.debug(
+            "Reserving UID {uid} @ {path}",
+            uid=uid, path=self.index.resource.fp.path,
         )
 
         def _handleFalse(result):
@@ -891,12 +862,11 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
         d.addCallback(_handleFalse)
         return d
 
-
     def unreserveUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log.debug("Unreserving UID %r @ %r" % (
-            uid,
-            self.index.resource.fp.path)
+        self.log.debug(
+            "Unreserving UID {uid} @ {path}",
+            uid=uid, path=self.index.resource.fp.path,
         )
 
         def _handleFalse(result):
@@ -910,12 +880,11 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
         d.addCallback(_handleFalse)
         return d
 
-
     def isReservedUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log.debug("Is reserved UID %r @ %r" % (
-            uid,
-            self.index.resource.fp.path)
+        self.log.debug(
+            "Is reserved UID {uid} @ {path}",
+            uid=uid, path=self.index.resource.fp.path,
         )
 
         def _checkValue((flags, value)):
@@ -929,12 +898,11 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
         return d
 
 
-
 class SQLUIDReserver(object):
+
     def __init__(self, index):
         self.index = index
 
-
     @wrapInDeferred
     def reserveUID(self, uid):
         """
@@ -953,11 +921,10 @@ class SQLUIDReserver(object):
                 % (uid, self.index.resource)
             )
         except sqlite.Error, e:
-            log.error("Unable to reserve UID: %s", (e,))
+            log.error("Unable to reserve UID: {ex}", ex=e)
             self.index._db_rollback()
             raise
 
-
     def unreserveUID(self, uid):
         """
         Unreserve a UID for this index's resource.
@@ -977,7 +944,7 @@ class SQLUIDReserver(object):
                         "delete from RESERVED where UID = :1", uid)
                     self.index._db_commit()
                 except sqlite.Error, e:
-                    log.error("Unable to unreserve UID: %s", (e,))
+                    log.error("Unable to unreserve UID: {ex}", ex=e)
                     self.index._db_rollback()
                     raise
 
@@ -985,7 +952,6 @@ class SQLUIDReserver(object):
         d.addCallback(_cb)
         return d
 
-
     @wrapInDeferred
     def isReservedUID(self, uid):
         """
@@ -1005,7 +971,7 @@ class SQLUIDReserver(object):
                     self.index._db_execute("delete from RESERVED where UID = :1", uid)
                     self.index._db_commit()
                 except sqlite.Error, e:
-                    log.error("Unable to unreserve UID: %s", (e,))
+                    log.error("Unable to unreserve UID: {ex}", ex=e)
                     self.index._db_rollback()
                     raise
                 return False
@@ -1015,7 +981,6 @@ class SQLUIDReserver(object):
         return False
 
 
-
 class Index (CalendarIndex):
     """
     Calendar collection index - regular collection that enforces CalDAV UID uniqueness requirement.
@@ -1038,7 +1003,6 @@ class Index (CalendarIndex):
         else:
             self.reserver = SQLUIDReserver(self)
 
-
     #
     # A dict of sets. The dict keys are calendar collection paths,
     # and the sets contains reserved UIDs for each path.
@@ -1047,15 +1011,12 @@ class Index (CalendarIndex):
     def reserveUID(self, uid):
         return self.reserver.reserveUID(uid)
 
-
     def unreserveUID(self, uid):
         return self.reserver.unreserveUID(uid)
 
-
     def isReservedUID(self, uid):
         return self.reserver.isReservedUID(uid)
 
-
     def isAllowedUID(self, uid, *names):
         """
         Checks to see whether to allow an operation which would add the
@@ -1070,14 +1031,12 @@ class Index (CalendarIndex):
         rname = self.resourceNameForUID(uid)
         return (rname is None or rname in names)
 
-
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         return collection_types["Calendar"]
 
-
     def _db_init_data_tables(self, q):
         """
         Initialise the underlying database tables.
@@ -1087,7 +1046,6 @@ class Index (CalendarIndex):
         # Create database where the RESOURCE table has unique UID column.
         self._db_init_data_tables_base(q, True)
 
-
     def _db_recreate(self, do_commit=True):
         """
         Re-create the database tables from existing calendar data.
@@ -1103,10 +1061,13 @@ class Index (CalendarIndex):
             if name.startswith("."):
                 continue
 
+            child = fp.child(name)
+            if not child.isfile():
+                continue
             try:
-                stream = fp.child(name).open()
+                stream = child.open()
             except (IOError, OSError), e:
-                log.error("Unable to open resource %s: %s" % (name, e))
+                log.error("Unable to open resource {name}: {ex}", name=name, ex=e)
                 continue
 
             # FIXME: This is blocking I/O
@@ -1115,9 +1076,9 @@ class Index (CalendarIndex):
                 calendar.validCalendarData()
                 calendar.validCalendarForCalDAV(methodAllowed=False)
             except ValueError:
-                log.error("Non-calendar resource: %s" % (name,))
+                log.error("Non-calendar resource: {name}", name=name)
             else:
-                # og.info("Indexing resource: %s" % (name,))
+                # og.info("Indexing resource: {name}", name=name)
                 self.addResource(name, calendar, True, reCreate=True)
             finally:
                 stream.close()
@@ -1127,13 +1088,12 @@ class Index (CalendarIndex):
             self._db_commit()
 
 
-
 class IndexSchedule (CalendarIndex):
     """
     Schedule collection index - does not require UID uniqueness.
     """
 
-    def reserveUID(self, uid): #@UnusedVariable
+    def reserveUID(self, uid):  # @UnusedVariable
         """
         Reserve a UID for this index's resource.
         @param uid: the UID to reserve
@@ -1143,8 +1103,7 @@ class IndexSchedule (CalendarIndex):
         # iTIP does not require unique UIDs
         return succeed(None)
 
-
-    def unreserveUID(self, uid): #@UnusedVariable
+    def unreserveUID(self, uid):  # @UnusedVariable
         """
         Unreserve a UID for this index's resource.
         @param uid: the UID to reserve
@@ -1154,8 +1113,7 @@ class IndexSchedule (CalendarIndex):
         # iTIP does not require unique UIDs
         return succeed(None)
 
-
-    def isReservedUID(self, uid): #@UnusedVariable
+    def isReservedUID(self, uid):  # @UnusedVariable
         """
         Check to see whether a UID is reserved.
         @param uid: the UID to check
@@ -1165,8 +1123,7 @@ class IndexSchedule (CalendarIndex):
         # iTIP does not require unique UIDs
         return succeed(False)
 
-
-    def isAllowedUID(self, uid, *names): #@UnusedVariable
+    def isAllowedUID(self, uid, *names):  # @UnusedVariable
         """
         Checks to see whether to allow an operation with adds the the specified
         UID is allowed to the index.  Specifically, the operation may not
@@ -1182,14 +1139,12 @@ class IndexSchedule (CalendarIndex):
         # iTIP does not require unique UIDs
         return True
 
-
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         return collection_types["iTIP"]
 
-
     def _db_init_data_tables(self, q):
         """
         Initialise the underlying database tables.
@@ -1199,7 +1154,6 @@ class IndexSchedule (CalendarIndex):
         # Create database where the RESOURCE table has a UID column that is not unique.
         self._db_init_data_tables_base(q, False)
 
-
     def _db_recreate(self, do_commit=True):
         """
         Re-create the database tables from existing calendar data.
@@ -1215,10 +1169,13 @@ class IndexSchedule (CalendarIndex):
             if name.startswith("."):
                 continue
 
+            child = fp.child(name)
+            if not child.isfile():
+                continue
             try:
-                stream = fp.child(name).open()
+                stream = child.open()
             except (IOError, OSError), e:
-                log.error("Unable to open resource %s: %s" % (name, e))
+                log.error("Unable to open resource {name}: {ex}", name=name, ex=e)
                 continue
 
             # FIXME: This is blocking I/O
@@ -1227,9 +1184,9 @@ class IndexSchedule (CalendarIndex):
                 calendar.validCalendarData()
                 calendar.validCalendarForCalDAV(methodAllowed=True)
             except ValueError:
-                log.error("Non-calendar resource: %s" % (name,))
+                log.error("Non-calendar resource: {name}", name=name)
             else:
-                # log.info("Indexing resource: %s" % (name,))
+                # log.info("Indexing resource: {name}", name=name)
                 self.addResource(name, calendar, True, reCreate=True)
             finally:
                 stream.close()
