@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -46,11 +46,9 @@ class FilterBase(object):
     def serialize_register(cls, register):
         cls.deserialize_names[register.serialized_name] = register
 
-
     def __init__(self, xml_element):
         pass
 
-
     @classmethod
     def deserialize(cls, data):
         """
@@ -60,14 +58,12 @@ class FilterBase(object):
         obj._deserialize(data)
         return obj
 
-
     def _deserialize(self, data):
         """
         Convert a JSON compatible serialization of this object into the actual object.
         """
         pass
 
-
     def serialize(self):
         """
         Create a JSON compatible serialization of this object - will be used in a cross-pod request.
@@ -76,16 +72,13 @@ class FilterBase(object):
             "type": self.serialized_name,
         }
 
-
     def match(self, item, access=None):
         raise NotImplementedError
 
-
     def valid(self, level=0):
         raise NotImplementedError
 
 
-
 class Filter(FilterBase):
     """
     Determines which matching components are returned.
@@ -105,14 +98,12 @@ class Filter(FilterBase):
 
         self.child = ComponentFilter(xml_element.children[0])
 
-
     def _deserialize(self, data):
         """
         Convert a JSON compatible serialization of this object into the actual object.
         """
         self.child = FilterBase.deserialize(data["child"])
 
-
     def serialize(self):
         """
         Create a JSON compatible serialization of this object - will be used in a cross-pod request.
@@ -123,7 +114,6 @@ class Filter(FilterBase):
         })
         return result
 
-
     def match(self, component, access=None):
         """
         Returns True if the given calendar component matches this filter, False
@@ -155,7 +145,6 @@ class Filter(FilterBase):
         # <filter> contains exactly one <comp-filter>
         return self.child.match(component, access)
 
-
     def valid(self):
         """
         Indicate whether this filter element's structure is valid wrt iCalendar
@@ -167,7 +156,6 @@ class Filter(FilterBase):
         # Must have one child element for VCALENDAR
         return self.child.valid(0)
 
-
     def settimezone(self, tzelement):
         """
         Set the default timezone to use with this query.
@@ -187,7 +175,6 @@ class Filter(FilterBase):
         self.child.settzinfo(tz)
         return tz
 
-
     def getmaxtimerange(self):
         """
         Get the date farthest into the future in any time-range elements
@@ -195,7 +182,6 @@ class Filter(FilterBase):
 
         return self.child.getmaxtimerange(None, False)
 
-
     def getmintimerange(self):
         """
         Get the date farthest into the past in any time-range elements. That is either
@@ -207,7 +193,6 @@ class Filter(FilterBase):
 FilterBase.serialize_register(Filter)
 
 
-
 class FilterChildBase(FilterBase):
     """
     CalDAV filter element.
@@ -264,7 +249,6 @@ class FilterChildBase(FilterBase):
             raise ValueError("Test must be only one of anyof, allof")
         self.filter_test = filter_test
 
-
     def _deserialize(self, data):
         """
         Convert a JSON compatible serialization of this object into the actual object.
@@ -275,7 +259,6 @@ class FilterChildBase(FilterBase):
         self.defined = data["defined"]
         self.filter_test = data["filter_test"]
 
-
     def serialize(self):
         """
         Create a JSON compatible serialization of this object - will be used in a cross-pod request.
@@ -290,7 +273,6 @@ class FilterChildBase(FilterBase):
         })
         return result
 
-
     def match(self, item, access=None):
         """
         Returns True if the given calendar item (either a component, property or parameter value)
@@ -315,7 +297,6 @@ class FilterChildBase(FilterBase):
             return True
 
 
-
 class ComponentFilter (FilterChildBase):
     """
     Limits a search to only the chosen component types.
@@ -348,7 +329,6 @@ class ComponentFilter (FilterChildBase):
         else:
             return True
 
-
     def _match(self, component, access):
         # At least one subcomponent must match (or is-not-defined is set)
         for subcomponent in component.subcomponents():
@@ -370,7 +350,6 @@ class ComponentFilter (FilterChildBase):
             return not self.defined
         return self.defined
 
-
     def setInstances(self, instances):
         """
         Give the list of instances to each comp-filter element.
@@ -380,7 +359,6 @@ class ComponentFilter (FilterChildBase):
         for compfilter in [x for x in self.filters if isinstance(x, ComponentFilter)]:
             compfilter.setInstances(instances)
 
-
     def valid(self, level):
         """
         Indicate whether this filter element's structure is valid wrt iCalendar
@@ -396,32 +374,32 @@ class ComponentFilter (FilterChildBase):
         if level == 0:
             # Must have VCALENDAR at the top
             if (self.filter_name != "VCALENDAR") or timerange:
-                log.info("Top-level comp-filter must be VCALENDAR, instead: %s" % (self.filter_name,))
+                log.info("Top-level comp-filter must be VCALENDAR, instead: {name}", name=self.filter_name)
                 return False
         elif level == 1:
             # Disallow VCALENDAR, VALARM, STANDARD, DAYLIGHT, AVAILABLE at the top, everything else is OK
             if self.filter_name in ("VCALENDAR", "VALARM", "STANDARD", "DAYLIGHT", "AVAILABLE"):
-                log.info("comp-filter wrong component type: %s" % (self.filter_name,))
+                log.info("comp-filter wrong component type: {name}", name=self.filter_name)
                 return False
 
             # time-range only on VEVENT, VTODO, VJOURNAL, VFREEBUSY, VAVAILABILITY
             if timerange and self.filter_name not in ("VEVENT", "VTODO", "VJOURNAL", "VFREEBUSY", "VAVAILABILITY"):
-                log.info("time-range cannot be used with component %s" % (self.filter_name,))
+                log.info("time-range cannot be used with component {name}", name=self.filter_name)
                 return False
         elif level == 2:
             # Disallow VCALENDAR, VTIMEZONE, VEVENT, VTODO, VJOURNAL, VFREEBUSY, VAVAILABILITY at the top, everything else is OK
             if (self.filter_name in ("VCALENDAR", "VTIMEZONE", "VEVENT", "VTODO", "VJOURNAL", "VFREEBUSY", "VAVAILABILITY")):
-                log.info("comp-filter wrong sub-component type: %s" % (self.filter_name,))
+                log.info("comp-filter wrong sub-component type: {name}", name=self.filter_name)
                 return False
 
             # time-range only on VALARM, AVAILABLE
             if timerange and self.filter_name not in ("VALARM", "AVAILABLE",):
-                log.info("time-range cannot be used with sub-component %s" % (self.filter_name,))
+                log.info("time-range cannot be used with sub-component {name}", name=self.filter_name)
                 return False
         else:
             # Disallow all standard iCal components anywhere else
             if (self.filter_name in ("VCALENDAR", "VTIMEZONE", "VEVENT", "VTODO", "VJOURNAL", "VFREEBUSY", "VALARM", "STANDARD", "DAYLIGHT", "AVAILABLE")) or timerange:
-                log.info("comp-filter wrong standard component type: %s" % (self.filter_name,))
+                log.info("comp-filter wrong standard component type: {name}", name=self.filter_name)
                 return False
 
         # Test each property
@@ -441,7 +419,6 @@ class ComponentFilter (FilterChildBase):
 
         return True
 
-
     def settzinfo(self, tzinfo):
         """
         Set the default timezone to use with this query.
@@ -456,7 +433,6 @@ class ComponentFilter (FilterChildBase):
         for x in self.filters:
             x.settzinfo(tzinfo)
 
-
     def getmaxtimerange(self, currentMaximum, currentIsStartTime):
         """
         Get the date farthest into the future in any time-range elements
@@ -480,7 +456,6 @@ class ComponentFilter (FilterChildBase):
 
         return currentMaximum, currentIsStartTime
 
-
     def getmintimerange(self, currentMinimum, currentIsEndTime):
         """
         Get the date farthest into the past in any time-range elements. That is either
@@ -505,7 +480,6 @@ class ComponentFilter (FilterChildBase):
 FilterBase.serialize_register(ComponentFilter)
 
 
-
 class PropertyFilter (FilterChildBase):
     """
     Limits a search to specific properties.
@@ -534,7 +508,6 @@ class PropertyFilter (FilterChildBase):
             return not self.defined
         return self.defined
 
-
     def valid(self):
         """
         Indicate whether this filter element's structure is valid wrt iCalendar
@@ -548,7 +521,7 @@ class PropertyFilter (FilterChildBase):
 
         # time-range only on COMPLETED, CREATED, DTSTAMP, LAST-MODIFIED
         if timerange and self.filter_name.upper() not in ("COMPLETED", "CREATED", "DTSTAMP", "LAST-MODIFIED"):
-            log.info("time-range cannot be used with property %s" % (self.filter_name,))
+            log.info("time-range cannot be used with property {name}", name=self.filter_name)
             return False
 
         # Test the time-range
@@ -559,7 +532,6 @@ class PropertyFilter (FilterChildBase):
         # No other tests
         return True
 
-
     def settzinfo(self, tzinfo):
         """
         Set the default timezone to use with this query.
@@ -570,7 +542,6 @@ class PropertyFilter (FilterChildBase):
         if isinstance(self.qualifier, TimeRange):
             self.qualifier.settzinfo(tzinfo)
 
-
     def getmaxtimerange(self, currentMaximum, currentIsStartTime):
         """
         Get the date farthest into the future in any time-range elements
@@ -590,7 +561,6 @@ class PropertyFilter (FilterChildBase):
 
         return currentMaximum, currentIsStartTime
 
-
     def getmintimerange(self, currentMinimum, currentIsEndTime):
         """
         Get the date farthest into the past in any time-range elements. That is either
@@ -611,7 +581,6 @@ class PropertyFilter (FilterChildBase):
 FilterBase.serialize_register(PropertyFilter)
 
 
-
 class ParameterFilter (FilterChildBase):
     """
     Limits a search to specific parameters.
@@ -633,7 +602,6 @@ class ParameterFilter (FilterChildBase):
 FilterBase.serialize_register(ParameterFilter)
 
 
-
 class IsNotDefined (FilterBase):
     """
     Specifies that the named iCalendar item does not exist.
@@ -695,7 +663,6 @@ class TextMatch (FilterBase):
         else:
             self.match_type = "contains"
 
-
     def _deserialize(self, data):
         """
         Convert a JSON compatible serialization of this object into the actual object.
@@ -705,7 +672,6 @@ class TextMatch (FilterBase):
         self.negate = data["negate"]
         self.match_type = data["match_type"]
 
-
     def serialize(self):
         """
         Create a JSON compatible serialization of this object - will be used in a cross-pod request.
@@ -719,7 +685,6 @@ class TextMatch (FilterBase):
         })
         return result
 
-
     def match(self, item, access):
         """
         Match the text for the item.
@@ -769,7 +734,6 @@ class TextMatch (FilterBase):
 FilterBase.serialize_register(TextMatch)
 
 
-
 class TimeRange (FilterBase):
     """
     Specifies a time for testing components against.
@@ -791,7 +755,6 @@ class TimeRange (FilterBase):
         self.end = DateTime.parseText(xml_element.attributes["end"]) if "end" in xml_element.attributes else None
         self.tzinfo = None
 
-
     def _deserialize(self, data):
         """
         Convert a JSON compatible serialization of this object into the actual object.
@@ -800,7 +763,6 @@ class TimeRange (FilterBase):
         self.end = DateTime.parseText(data["end"]) if data["end"] else None
         self.tzinfo = Timezone(tzid=data["tzinfo"]) if data["tzinfo"] else None
 
-
     def serialize(self):
         """
         Create a JSON compatible serialization of this object - will be used in a cross-pod request.
@@ -813,7 +775,6 @@ class TimeRange (FilterBase):
         })
         return result
 
-
     def settzinfo(self, tzinfo):
         """
         Set the default timezone to use with this query.
@@ -823,7 +784,6 @@ class TimeRange (FilterBase):
         # Give tzinfo to any TimeRange we have
         self.tzinfo = tzinfo
 
-
     def valid(self, level=0):
         """
         Indicate whether the time-range is valid (must be date-time in UTC).
@@ -832,22 +792,21 @@ class TimeRange (FilterBase):
         """
 
         if self.start is not None and self.start.isDateOnly():
-            log.info("start attribute in <time-range> is not a date-time: %s" % (self.start,))
+            log.info("start attribute in <time-range> is not a date-time: {start}", start=self.start)
             return False
         if self.end is not None and self.end.isDateOnly():
-            log.info("end attribute in <time-range> is not a date-time: %s" % (self.end,))
+            log.info("end attribute in <time-range> is not a date-time: {end}", start=self.end)
             return False
         if self.start is not None and not self.start.utc():
-            log.info("start attribute in <time-range> is not UTC: %s" % (self.start,))
+            log.info("start attribute in <time-range> is not UTC: {start}", start=self.start)
             return False
         if self.end is not None and not self.end.utc():
-            log.info("end attribute in <time-range> is not UTC: %s" % (self.end,))
+            log.info("end attribute in <time-range> is not UTC: {end}", start=self.end)
             return False
 
         # No other tests
         return True
 
-
     def match(self, property, access=None):
         """
         NB This is only called when doing a time-range match on a property.
@@ -857,7 +816,6 @@ class TimeRange (FilterBase):
         else:
             return property.containsTimeRange(self.start, self.end, self.tzinfo)
 
-
     def matchinstance(self, component, instances):
         """
         Test whether this time-range element causes a match to the specified component
