@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,10 +14,11 @@
 # limitations under the License.
 ##
 
-from twext.python.log import Logger, LogLevel
+from twext.python.log import Logger
 from txweb2.dav.http import ErrorResponse
 
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed
+from twisted.logger import LogLevel
 from twisted.python.failure import Failure
 from txweb2 import responsecode
 from txweb2.http import HTTPError
@@ -49,6 +50,7 @@ __all__ = [
 
 log = Logger()
 
+
 class ScheduleViaCalDAV(DeliveryService):
 
     def __init__(self, scheduler, recipients, responses, freebusy):
@@ -58,12 +60,10 @@ class ScheduleViaCalDAV(DeliveryService):
         self.responses = responses
         self.freebusy = freebusy
 
-
     @classmethod
     def serviceType(cls):
         return DeliveryService.serviceType_caldav
 
-
     @classmethod
     def matchCalendarUserAddress(cls, cuaddr):
 
@@ -87,7 +87,6 @@ class ScheduleViaCalDAV(DeliveryService):
         # Do default match
         return super(ScheduleViaCalDAV, cls).matchCalendarUserAddress(cuaddr)
 
-
     @inlineCallbacks
     def generateSchedulingResponses(self):
 
@@ -98,9 +97,11 @@ class ScheduleViaCalDAV(DeliveryService):
         # Freebusy needs to be optimized by doing multiple attendee in parallel if possible
         if self.freebusy:
             # Look for special delegate extended free-busy request
-            event_details = [] if self.scheduler.calendar.getExtendedFreeBusy() else None
+            use_extended_free_busy = self.scheduler.calendar.getExtendedFreeBusy() is not None
 
             for recipient in self.recipients:
+                event_details = [] if use_extended_free_busy else None
+
                 # Check access controls - we do not do this right now. But if we ever implement access controls to
                 # determine which users can schedule with other users, here is where we would do that test.
                 yield self.generateFreeBusyResponse(recipient, self.responses, organizerProp, uid, event_details)
@@ -110,7 +111,6 @@ class ScheduleViaCalDAV(DeliveryService):
                 # determine which users can schedule with other users, here is where we would do that test.
                 yield self.generateResponse(recipient, self.responses)
 
-
     @inlineCallbacks
     def generateResponse(self, recipient, responses):
         # Hash the iCalendar data for use as the last path element of the URI path
@@ -190,7 +190,6 @@ class ScheduleViaCalDAV(DeliveryService):
                 self.scheduler.logItems["itip.auto"] = self.scheduler.logItems.get("itip.auto", 0) + 1
         returnValue(True)
 
-
     @inlineCallbacks
     def generateFreeBusyResponse(self, recipient, responses, organizerProp, uid, event_details):
 
