@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -57,7 +57,6 @@ class FBCacheEntry(object):
         self.timerange = timerange.getText()
         self.fbresults = fbresults
 
-
     @classmethod
     @inlineCallbacks
     def getCacheEntry(cls, calresource, useruid, timerange):
@@ -82,7 +81,6 @@ class FBCacheEntry(object):
 
         returnValue(None)
 
-
     @classmethod
     @inlineCallbacks
     def makeCacheEntry(cls, calresource, useruid, timerange, fbresults):
@@ -93,7 +91,6 @@ class FBCacheEntry(object):
         yield cls.fbcacher.set(key, entry)
 
 
-
 class FreebusyQuery(object):
     """
     Class that manages the process of getting free busy information of a particular attendee.
@@ -160,7 +157,6 @@ class FreebusyQuery(object):
         self.attendee_record = self.recipient.record if self.recipient and self.recipient.hosted() else None
         self.attendee_uid = self.attendee_record.uid if self.attendee_record else None
 
-
     @inlineCallbacks
     def checkRichOptions(self, txn):
         if not hasattr(self, "rich_options"):
@@ -194,7 +190,6 @@ class FreebusyQuery(object):
                 if config.Scheduling.Options.RoomResourceRichFreeBusy and self.attendee_record.getCUType() in ("RESOURCE", "ROOM",):
                     self.rich_options["resource"] = True
 
-
     @inlineCallbacks
     def generateAttendeeFreeBusyResponse(self, fbset=None, method="REPLY"):
 
@@ -221,7 +216,6 @@ class FreebusyQuery(object):
 
         returnValue(fbresult)
 
-
     def processAvailabilityFreeBusy(self, calendar, fbinfo):
         """
         Extract free-busy data from a VAVAILABILITY component.
@@ -264,7 +258,6 @@ class FreebusyQuery(object):
 
             getattr(fbinfo, self.FBInfo_mapper.get(fbtype, "unavailable")).extend(busyperiods)
 
-
     def processAvailablePeriods(self, calendar):
         """
         Extract instance period data from an AVAILABLE component.
@@ -311,7 +304,6 @@ class FreebusyQuery(object):
         normalizePeriodList(periods)
         return periods
 
-
     @inlineCallbacks
     def generateFreeBusyInfo(self, fbset, fbinfo, matchtotal=0):
         """
@@ -341,7 +333,6 @@ class FreebusyQuery(object):
 
         returnValue(matchtotal)
 
-
     @inlineCallbacks
     def _externalGenerateFreeBusyInfo(self, fbset, fbinfo, matchtotal):
         """
@@ -364,7 +355,6 @@ class FreebusyQuery(object):
                 fbinfo[i].extend([Period.parseText(p) for p in fbresults[i]])
         returnValue(matchtotal)
 
-
     @inlineCallbacks
     def _internalGenerateFreeBusyInfo(
         self,
@@ -462,7 +452,7 @@ class FreebusyQuery(object):
                     # between our initial index query and getting here. For now we will ignore this error, but in
                     # the longer term we need to implement some form of locking, perhaps.
                     if calendar is None:
-                        log.error("Calendar %s is missing from calendar collection %r" % (name, calresource))
+                        log.error("Calendar {name} is missing from calendar collection {coll!r}", name=name, coll=calresource)
                         continue
 
                     if self.accountingItems is not None:
@@ -499,7 +489,6 @@ class FreebusyQuery(object):
 
         returnValue(matchtotal)
 
-
     @inlineCallbacks
     def _matchResources(self, fbset):
         """
@@ -517,7 +506,6 @@ class FreebusyQuery(object):
 
         returnValue(results)
 
-
     @inlineCallbacks
     def _matchCalendarResources(self, calresource):
 
@@ -605,7 +593,6 @@ class FreebusyQuery(object):
 
         returnValue((aggregated_resources, tzinfo, filter,))
 
-
     @inlineCallbacks
     def _testIgnoreExcludeUID(self, uid, test_organizer, recordUIDCache, dirservice):
         """
@@ -642,7 +629,6 @@ class FreebusyQuery(object):
 
         returnValue(False)
 
-
     def _addEventDetails(self, calendar, rich_options, tzinfo):
         """
         Expand events within the specified time range and limit the set of properties to those allowed for
@@ -685,7 +671,6 @@ class FreebusyQuery(object):
 
         self.event_details.extend([subcomponent for subcomponent in expanded.subcomponents() if subcomponent.name() == "VEVENT"])
 
-
     def processEventFreeBusy(self, calendar, fbinfo, tzinfo):
         """
         Extract free busy data from a VEVENT component.
@@ -748,7 +733,6 @@ class FreebusyQuery(object):
                 else:
                     fbinfo.busy.append(clipped)
 
-
     def processFreeBusyFreeBusy(self, calendar, fbinfo):
         """
         Extract FREEBUSY data from a VFREEBUSY component.
@@ -779,7 +763,6 @@ class FreebusyQuery(object):
                     if clipped:
                         getattr(fbinfo, self.FBInfo_mapper.get(fbtype, "busy")).append(clipped)
 
-
     def buildFreeBusyResult(self, fbinfo, method=None):
         """
         Generate a VCALENDAR object containing a single VFREEBUSY that is the
