@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@ from twext.python.log import Logger
 
 from twistedcaldav import accounting
 from twistedcaldav.config import config
-from twistedcaldav.ical import Component, Property
+from twistedcaldav.ical import Component, Property, PRIVATE_COMMENT, DTSTAMP_PARAM
 from txdav.caldav.datastore.scheduling.utils import normalizeCUAddr
 from txdav.caldav.datastore.scheduling.itip import iTipGenerator
 
@@ -33,6 +33,7 @@ __all__ = [
 
 log = Logger()
 
+
 class iCalDiff(object):
     """
     This object is used for doing comparisons between two calendar objects to
@@ -61,7 +62,6 @@ class iCalDiff(object):
         self.smart_merge = smart_merge
         self.forceTRANSP = forceTRANSP
 
-
     def organizerDiff(self):
         """
         Diff the two calendars looking for changes that should trigger implicit scheduling if
@@ -82,7 +82,10 @@ class iCalDiff(object):
                 "DTSTAMP",
                 "LAST-MODIFIED",
             ))
-            calendar.removeXProperties(keep_properties=config.Scheduling.CalDAV.OrganizerPublicProperties)
+            calendar.removeXProperties(keep_properties=(
+                config.Scheduling.CalDAV.OrganizerPublicProperties +
+                config.Scheduling.CalDAV.AttendeePublicProperties
+            ))
             calendar.removePropertyParameters("ATTENDEE", ("RSVP", "SCHEDULE-STATUS", "SCHEDULE-FORCE-SEND",))
             calendar.normalizeAll()
             return calendar
@@ -94,7 +97,6 @@ class iCalDiff(object):
         result = oldcalendar_norm == newcalendar_norm
         return result
 
-
     def _organizerMerge(self):
         """
         Merge changes to ATTENDEE properties in oldcalendar into newcalendar.
@@ -102,7 +104,6 @@ class iCalDiff(object):
         organizer = normalizeCUAddr(self.newcalendar.masterComponent().propertyValue("ORGANIZER"))
         self._doSmartMerge(organizer, True)
 
-
     def _doSmartMerge(self, ignore_attendee, is_organizer):
         """
         Merge changes to ATTENDEE properties in oldcalendar into newcalendar.
@@ -169,12 +170,10 @@ class iCalDiff(object):
                     # Ignore as we have no state for the new instance
                     pass
 
-
     def _tryComponentMerge(self, old_comp, new_comp, ignore_attendee_value, is_organizer):
         if not is_organizer or not self._organizerChangePreventsMerge(old_comp, new_comp):
             self._transferAttendees(old_comp, new_comp, ignore_attendee_value)
 
-
     def _organizerChangePreventsMerge(self, old_comp, new_comp):
         """
         Check whether a change from an Organizer needs a re-schedule which means that any
@@ -199,7 +198,6 @@ class iCalDiff(object):
 
         return False
 
-
     def _transferAttendees(self, old_comp, new_comp, ignore_attendee_value):
         """
         Transfer Attendee PARTSTAT from old component to new component.
@@ -235,7 +233,6 @@ class iCalDiff(object):
                 self._transferParameter(old_attendee, new_attendee, "RSVP")
                 self._transferParameter(old_attendee, new_attendee, "SCHEDULE-STATUS")
 
-
     def _transferParameter(self, old_property, new_property, parameter):
         paramvalue = old_property.parameterValue(parameter)
         if paramvalue is None:
@@ -246,7 +243,6 @@ class iCalDiff(object):
         else:
             new_property.setParameter(parameter, paramvalue)
 
-
     def attendeeMerge(self, attendee):
         """
         Merge the ATTENDEE specific changes with the organizer's view of the attendee's event.
@@ -335,6 +331,13 @@ class iCalDiff(object):
         # All the components in oldcalendar must be in newcalendar unless they are CANCELLED
         for key in setold - setnew:
             _ignore_name, _ignore_uid, rid = key
+
+            # Some clients seem to remove the master component - this is not good. Rather than
+            # failing we will restore the original master - basically just ignore the master if
+            # it appears in this loop.
+            if rid is None:
+                continue
+
             component = mapold[key]
             if component.propertyValue("STATUS") != "CANCELLED":
                 # Attendee may decline by EXDATE'ing an instance - we need to handle that
@@ -461,7 +464,6 @@ class iCalDiff(object):
 
         return True, changeCausesReply, changedRids, returnCalendar
 
-
     def _checkVCALENDARProperties(self, serverData, clientData):
 
         self._transferProperty("X-CALENDARSERVER-ACCESS", serverData, clientData)
@@ -475,10 +477,9 @@ class iCalDiff(object):
 
         result = len(propdiff) == 0
         if not result:
-            log.debug("VCALENDAR properties differ: %s" % (propdiff,))
+            log.debug("VCALENDAR properties differ: {diff}", diff=propdiff)
         return result
 
-
     def _transferAttendeeData(self, serverComponent, clientComponent, declines):
 
         # We are skipping this check now - instead we let the server data override the broken client data
@@ -497,7 +498,7 @@ class iCalDiff(object):
 
         # Possible case where one ATTENDEE prop is missing - this happens with a "fake" master sometimes
         if serverAttendee is None or clientAttendee is None:
-            log.error("ATTENDEE for user making an attendee change is missing: %s" % (self.attendee,))
+            log.error("ATTENDEE for user making an attendee change is missing: {attendee}", attendee=self.attendee)
             return False, False
 
         if serverAttendee.parameterValue("PARTSTAT", "NEEDS-ACTION") != clientAttendee.parameterValue("PARTSTAT", "NEEDS-ACTION"):
@@ -505,7 +506,7 @@ class iCalDiff(object):
 
             # If PARTSTAT was changed by the attendee, add a timestamp if needed
             if config.Scheduling.Options.TimestampAttendeePartStatChanges:
-                serverAttendee.setParameter("X-CALENDARSERVER-DTSTAMP", DateTime.getNowUTC().getText())
+                serverAttendee.setParameter(DTSTAMP_PARAM, DateTime.getNowUTC().getText())
             serverAttendee.removeParameter("X-CALENDARSERVER-AUTO")
 
             replyNeeded = True
@@ -526,14 +527,26 @@ class iCalDiff(object):
             else:
                 serverAttendee.setParameter("RSVP", "TRUE")
 
+        for pname in config.Scheduling.CalDAV.AttendeePublicParameters:
+            serverValue = serverAttendee.parameterValue(pname)
+            clientValue = clientAttendee.parameterValue(pname)
+            if serverValue != clientValue:
+                if clientValue is None:
+                    serverAttendee.removeParameter(pname)
+                else:
+                    serverAttendee.setParameter(pname, clientValue)
+                replyNeeded = True
+
         # Transfer these properties from the client data
-        replyNeeded |= self._transferProperty("X-CALENDARSERVER-PRIVATE-COMMENT", serverComponent, clientComponent)
         self._transferProperty("TRANSP", serverComponent, clientComponent)
         self._transferProperty("DTSTAMP", serverComponent, clientComponent)
         self._transferProperty("LAST-MODIFIED", serverComponent, clientComponent)
         self._transferProperty("COMPLETED", serverComponent, clientComponent)
         for pname in config.Scheduling.CalDAV.PerAttendeeProperties:
             self._transferProperty(pname, serverComponent, clientComponent)
+        replyNeeded |= self._transferProperty(PRIVATE_COMMENT, serverComponent, clientComponent)
+        for pname in config.Scheduling.CalDAV.AttendeePublicProperties:
+            replyNeeded |= self._transferProperty(pname, serverComponent, clientComponent)
 
         # Dropbox - this now never returns false
         if config.EnableDropBox:
@@ -551,7 +564,6 @@ class iCalDiff(object):
 
         return True, replyNeeded
 
-
     def _transferDropBoxData(self, serverComponent, clientComponent):
 
         serverDropbox = serverComponent.propertyValue("X-APPLE-DROPBOX")
@@ -588,7 +600,6 @@ class iCalDiff(object):
 
             return True
 
-
     def _transferVPOLLData(self, serverComponent, clientComponent):
 
         changed = False
@@ -629,7 +640,6 @@ class iCalDiff(object):
 
         return changed
 
-
     def _checkInvalidChanges(self, serverComponent, clientComponent, declines):
 
         # Properties we care about: DTSTART, DTEND, DURATION, RRULE, RDATE, EXDATE
@@ -642,20 +652,19 @@ class iCalDiff(object):
             invalidChanges = []
             propNames = ("DTSTART", "DTEND", "DUE", "RRULE", "RDATE", "EXDATE")
             invalidChanges = [propName for ctr, propName in enumerate(propNames) if serverProps[ctr] != clientProps[ctr]]
-            log.debug("Critical properties do not match: %s" % (", ".join(invalidChanges),))
+            log.debug("Critical properties do not match: {props}", props=", ".join(invalidChanges))
             return False
         elif serverProps[-1] != clientProps[-1]:
             # Bad if EXDATEs have been removed
             missing = serverProps[-1] - clientProps[-1]
             if missing:
-                log.debug("EXDATEs missing: %s" % (", ".join([exdate.getText() for exdate in missing]),))
+                log.debug("EXDATEs missing: {exdates}", exdates=", ".join([exdate.getText() for exdate in missing]))
                 return False
             declines.extend(clientProps[-1] - serverProps[-1])
             return True
         else:
             return True
 
-
     def _getNormalizedDateTimeProperties(self, component):
 
         # Basic time properties
@@ -716,7 +725,6 @@ class iCalDiff(object):
 
         return timeRange.getStart(), timeRange.getEnd(), newdue, newrrules, newrdates, newexdates
 
-
     def _transferProperty(self, propName, serverComponent, clientComponent):
 
         changed = False
@@ -730,7 +738,6 @@ class iCalDiff(object):
             changed = True
         return changed
 
-
     def _attendeeDecline(self, component):
         """
         Mark attendee as DECLINED in the component.
@@ -744,7 +751,7 @@ class iCalDiff(object):
 
         # Possible case where ATTENDEE prop is missing - this happens with a "fake" master sometimes
         if attendee is None:
-            log.error("ATTENDEE for user making an attendee change is missing: %s" % (self.attendee,))
+            log.error("ATTENDEE for user making an attendee change is missing: {attendee}", attendee=self.attendee)
             return False
 
         partstatChanged = attendee.parameterValue("PARTSTAT", "NEEDS-ACTION") != "DECLINED"
@@ -755,8 +762,7 @@ class iCalDiff(object):
         component.replaceProperty(Property("TRANSP", "TRANSPARENT"))
         return partstatChanged
 
-
-    def whatIsDifferent(self, isiTip=True):
+    def whatIsDifferent(self, timeRangeCheck=False):
         """
         Compare the two calendar objects in their entirety and return a list of properties
         and PARTSTAT parameters that are different.
@@ -786,7 +792,7 @@ class iCalDiff(object):
         for key in (oldset & newset):
             component1 = oldmap[key]
             component2 = newmap[key]
-            self._diffComponents(component1, component2, rids, needs_action_changes, isiTip)
+            self._diffComponents(component1, component2, rids, needs_action_changes, timeRangeCheck)
 
         # Now verify that each additional component in oldset matches a derived component in newset
         for key in oldset - newset:
@@ -794,31 +800,24 @@ class iCalDiff(object):
             oldcomponent = oldmap[key]
             newcomponent = self.newcalendar.deriveInstance(rid)
             if newcomponent is None:
-                # For the non iTIP case we must report missing components on either side. Marking
-                # the DTSTART as changed is enough to trigger logic in the caller to treat this
-                # as a significant change.
-                if not isiTip:
-                    rids[rid.getText() if rid is not None else ""] = {"DTSTART": set()}
+                # Set the entry to None to indicate removal
+                rids[rid] = None
                 continue
-            self._diffComponents(oldcomponent, newcomponent, rids, needs_action_changes, isiTip)
+            self._diffComponents(oldcomponent, newcomponent, rids, needs_action_changes, timeRangeCheck)
 
         # Now verify that each additional component in oldset matches a derived component in newset
         for key in newset - oldset:
             rid = key[2]
             oldcomponent = self.oldcalendar.deriveInstance(rid)
             if oldcomponent is None:
-                # For the non iTIP case we must report missing components on either side. Marking
-                # the DTSTART as changed is enough to trigger logic in the caller to treat this
-                # as a significant change.
-                if not isiTip:
-                    rids[rid.getText() if rid is not None else ""] = {"DTSTART": set()}
+                # Set the entry to an empty dict to indicate addition
+                rids[rid] = {}
                 continue
             newcomponent = newmap[key]
-            self._diffComponents(oldcomponent, newcomponent, rids, needs_action_changes, isiTip)
+            self._diffComponents(oldcomponent, newcomponent, rids, needs_action_changes, timeRangeCheck)
 
         return (rids, needs_action_changes,)
 
-
     TRPROPS = frozenset((
         "DTSTART",
         "DTEND",
@@ -843,15 +842,17 @@ class iCalDiff(object):
         @rtype: L{bool}
         """
 
-        rids, _ignore_changes = self.whatIsDifferent(isiTip=False)
+        rids, _ignore_changes = self.whatIsDifferent(timeRangeCheck=True)
         for props in rids.values():
+            if not props:
+                # Component was added (props == {}) or removed (props is None)
+                return True
             props = frozenset(props.keys())
             if props & self.TRPROPS:
                 return True
         else:
             return False
 
-
     def attendeeNeedsAction(self, diffs):
         """
         Given a set of results from L{whatIsDifferent}, determine which recurrence-id's
@@ -868,6 +869,8 @@ class iCalDiff(object):
         recurrence_reschedule = False
 
         for rid, props in diffs.iteritems():
+            if props is None:
+                continue
             if any([testprop in props for testprop in (
                 "DTSTART",
                 "DTEND",
@@ -920,32 +923,30 @@ class iCalDiff(object):
 
         return (date_changed_rids, recurrence_reschedule,)
 
-
-    def _componentDuplicateAndNormalize(self, comp, isiTip=True):
+    def _componentDuplicateAndNormalize(self, comp, timeRangeCheck=False):
         comp = comp.duplicate()
         comp.normalizePropertyValueLists("EXDATE")
         comp.removeAlarms()
         comp.normalizeAll()
         comp.normalizeAttachments()
-        if isiTip:
+        if not timeRangeCheck:
             comp.removePropertyParameters("ORGANIZER", ("SCHEDULE-STATUS",))
             comp.removePropertyParameters("ATTENDEE", ("SCHEDULE-STATUS", "SCHEDULE-FORCE-SEND",))
             comp.removePropertyParameters("VOTER", ("SCHEDULE-STATUS", "SCHEDULE-FORCE-SEND",))
             iTipGenerator.prepareSchedulingMessage(comp, reply=True)
         return comp
 
-
-    def _diffComponents(self, comp1, comp2, rids, needs_action_rids, isiTip=True):
+    def _diffComponents(self, comp1, comp2, rids, needs_action_rids, timeRangeCheck=False):
 
         assert isinstance(comp1, Component) and isinstance(comp2, Component)
 
         if comp1.name() != comp2.name():
-            log.debug("Component names are different: '%s' and '%s'" % (comp1.name(), comp2.name()))
+            log.debug("Component names are different: '{comp1}' and '{comp2}'", comp1=comp1.name(), comp2=comp2.name())
             return
 
         # Duplicate then normalize for comparison
-        comp1 = self._componentDuplicateAndNormalize(comp1, isiTip)
-        comp2 = self._componentDuplicateAndNormalize(comp2, isiTip)
+        comp1 = self._componentDuplicateAndNormalize(comp1, timeRangeCheck)
+        comp2 = self._componentDuplicateAndNormalize(comp2, timeRangeCheck)
 
         # Diff all the properties
         propdiff = set(comp1.properties()) ^ set(comp2.properties())
@@ -958,7 +959,7 @@ class iCalDiff(object):
                 "DTSTAMP",
                 "CREATED",
                 "LAST-MODIFIED",
-                "X-CALENDARSERVER-PRIVATE-COMMENT",
+                PRIVATE_COMMENT,
             ):
                 continue
             if prop.name() == "ATTENDEE":
@@ -983,7 +984,6 @@ class iCalDiff(object):
         if attendeeChanges:
             self._diffNeedsAction(comp1, comp2, needs_action_rids)
 
-
     def _diffNeedsAction(self, comp1, comp2, needs_action_rids):
 
         rid = comp1.getRecurrenceIDUTC()
@@ -1003,7 +1003,6 @@ class iCalDiff(object):
             ):
                 needs_action_rids.setdefault(rid, set()).add(attendee)
 
-
     def _logDiffError(self, title):
 
         strcal1 = str(self.oldcalendar)
@@ -1032,4 +1031,4 @@ class iCalDiff(object):
             loggedUID = "Unknown"
         loggedName = accounting.emitAccounting("Implicit Errors", loggedUID, logstr)
         if loggedName:
-            log.error("Generating Implicit Error accounting at path: %s" % (loggedName,))
+            log.error("Generating Implicit Error accounting at path: {name}", name=loggedName)
