@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@ from __future__ import with_statement
 
 from cStringIO import StringIO
 import os
+import re
 
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
@@ -44,10 +45,12 @@ from txdav.caldav.datastore.scheduling.imip.smtpsender import SMTPSender
 from txdav.common.datastore.sql_tables import schema
 
 
-
 log = Logger()
 
 
+EMBEDDED_HEADER_CHECK = re.compile(r'\n[^ \t]+:')
+
+
 """ SCHEMA:
 create sequence WORKITEM_SEQ;
 
@@ -85,7 +88,6 @@ class IMIPInvitationWork(WorkItem, fromTable(schema.IMIP_INVITATION_WORK)):
                     settings.SuppressionDays, smtpSender, getLanguage(config))
         return cls.mailSender
 
-
     @inlineCallbacks
     def doWork(self):
         """
@@ -165,6 +167,7 @@ htmlInviteTemplate = u"""<html>
     </p>
     """.encode("utf-8")
 
+
 def _visit(document, node):
     if isinstance(node, DOMText):
         idx = node.parentNode.childNodes.index(node)
@@ -195,14 +198,12 @@ def _visit(document, node):
                 node.appendChild(elem)
 
 
-
 def _walk(document, n):
     _visit(document, n)
     for subn in n.childNodes:
         _walk(document, subn)
 
 
-
 def _fixup(data, rendererName):
     document = parseString(data, beExtremelyLenient=True)
     document.documentElement.setAttribute(
@@ -220,11 +221,11 @@ def _fixup(data, rendererName):
     return result
 
 
-
 class StringFormatTemplateLoader(object):
     """
     Loader for twisted.web.template that converts a template with %()s slots.
     """
+
     def __init__(self, fileFactory, rendererName):
         """
         @param fileFactory: a 1-argument callable which returns a file-like
@@ -237,13 +238,11 @@ class StringFormatTemplateLoader(object):
         self.fileFactory = fileFactory
         self.rendererName = rendererName
 
-
     def load(self):
         html = _fixup(self.fileFactory().read(), self.rendererName)
         return XMLString(html).load()
 
 
-
 def localizedLabels(language, canceled, inviteState):
     """
     Generate localized labels for an email in the given language.
@@ -299,7 +298,6 @@ def localizedLabels(language, canceled, inviteState):
     return subjectFormatString.decode("utf-8"), labels
 
 
-
 class MailSender(object):
     """
     Generates outbound IMIP messages and sends them.
@@ -312,7 +310,6 @@ class MailSender(object):
         self.smtpSender = smtpSender
         self.language = language
 
-
     @inlineCallbacks
     def outbound(self, txn, originator, recipient, calendar, onlyAfter=None):
         """
@@ -376,15 +373,19 @@ class MailSender(object):
 
             if record is None:
                 record = (yield txn.imipCreateToken(originator, toAddr.lower(), icaluid))
-                self.log.debug("Mail gateway created token %s for %s "
-                               "(originator), %s (recipient) and %s (icaluid)"
-                               % (record.token, originator, toAddr, icaluid))
+                self.log.debug(
+                    "Mail gateway created token {token}for {orig} "
+                    "(originator), {recip} (recipient) and {uid} (icaluid)",
+                    token=record.token, orig=originator, recip=toAddr, uid=icaluid,
+                )
                 inviteState = "new"
 
             else:
-                self.log.debug("Mail gateway reusing token %s for %s "
-                               "(originator), %s (recipient) and %s (icaluid)"
-                               % (record.token, originator, toAddr, icaluid))
+                self.log.debug(
+                    "Mail gateway reusing token {token} for {orig} "
+                    "(originator), {recip} (recipient) and {uid} (icaluid)",
+                    token=record.token, orig=originator, recip=toAddr, uid=icaluid,
+                )
                 inviteState = "update"
             token = record.token
 
@@ -429,7 +430,7 @@ class MailSender(object):
 
             # Reply-to address will be the server+token address
 
-        else: # REPLY
+        else:  # REPLY
             inviteState = "reply"
 
             # Look up the attendee property corresponding to the originator
@@ -485,9 +486,17 @@ class MailSender(object):
                 fromAddr, toAddr, msgId, message))
             returnValue(success)
         except Exception, e:
-            self.log.error("Failed to send IMIP message (%s)" % (str(e),))
+            self.log.error("Failed to send IMIP message ({ex})", ex=str(e))
             returnValue(False)
 
+    def _scrubHeader(self, value):
+        """
+        Check for what would be considered "embedded headers" and, if present,
+        remove the newlines.
+        """
+        if EMBEDDED_HEADER_CHECK.search(value) is not None:
+            value = value.replace("\n", " ")
+        return value
 
     def generateEmail(self, inviteState, calendar, orgEmail, orgCN,
                       attendees, fromAddress, replyToAddress, toAddress,
@@ -545,7 +554,7 @@ class MailSender(object):
         subjectFormat, labels = localizedLabels(language, canceled, inviteState)
         details.update(labels)
 
-        details['subject'] = subjectFormat % {'summary' : details['summary']}
+        details['subject'] = subjectFormat % {'summary': details['summary']}
 
         plainText = self.renderPlainText(details, (orgCN, orgEmail),
                                          attendees, canceled)
@@ -556,9 +565,9 @@ class MailSender(object):
 
         msg = MIMEMultipart()
         msg["From"] = fromAddress
-        msg["Subject"] = details['subject']
-        msg["Reply-To"] = replyToAddress
-        msg["To"] = toAddress
+        msg["Subject"] = self._scrubHeader(details['subject'])
+        msg["Reply-To"] = self._scrubHeader(replyToAddress)
+        msg["To"] = self._scrubHeader(toAddress)
         msg["Date"] = rfc822date()
         msgId = SMTPSender.betterMessageID()
         msg["Message-ID"] = msgId
@@ -583,8 +592,7 @@ class MailSender(object):
         # always be present (i.e., timezones-by-reference is not allowed in iMIP).
         calendarText = calendar.getTextWithTimezones(includeTimezones=True)
 
-        self.log.debug("Mail gateway sending calendar body: %s"
-                       % (calendarText,))
+        self.log.debug("Mail gateway sending calendar body: {body}", body=calendarText)
         msgIcal = MIMEText(calendarText, "calendar", "UTF-8")
         method = calendar.propertyValue("METHOD").lower()
         msgIcal.set_param("method", method)
@@ -596,7 +604,6 @@ class MailSender(object):
 
         return msgId, msg.as_string()
 
-
     def renderPlainText(self, details, (orgCN, orgEmail), attendees, canceled):
         """
         Render text/plain message part based on invitation details and a flag
@@ -629,7 +636,6 @@ class MailSender(object):
 
         return (plainTemplate % details).encode("UTF-8")
 
-
     def renderHTML(self, details, organizer, attendees, canceled):
         """
         Render HTML message part based on invitation details and a flag
@@ -681,13 +687,13 @@ class MailSender(object):
                 htmlTemplate = htmlCancelTemplate
             else:
                 htmlTemplate = htmlInviteTemplate
-        else: # HTML template file exists
+        else:  # HTML template file exists
 
             with open(templatePath) as templateFile:
                 htmlTemplate = templateFile.read()
 
         class EmailElement(Element):
-            loader = StringFormatTemplateLoader(lambda : StringIO(htmlTemplate),
+            loader = StringFormatTemplateLoader(lambda: StringIO(htmlTemplate),
                                                 "email")
 
             @renderer
@@ -699,7 +705,6 @@ class MailSender(object):
         htmlText = textCollector[0]
         return htmlText
 
-
     def getEventDetails(self, calendar, language='en'):
         """
         Create a dictionary mapping slot names - specifically: summary,
