@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,6 +35,8 @@ from txdav.common.datastore.test.util import CommonCommonTests
 from twext.enterprise.jobs.jobitem import JobItem
 
 import email
+from twisted.mail.imap4 import MessageSet
+
 
 class InboundTests(CommonCommonTests, unittest.TestCase):
 
@@ -47,11 +49,11 @@ class InboundTests(CommonCommonTests, unittest.TestCase):
         self.retriever = MailRetriever(
             self.store, self.directory,
             ConfigDict({
-                "Type" : "pop",
-                "UseSSL" : False,
-                "Server" : "example.com",
-                "Port" : 123,
-                "Username" : "xyzzy",
+                "Type": "pop",
+                "UseSSL": False,
+                "Server": "example.com",
+                "Port": 123,
+                "Username": "xyzzy",
             })
         )
 
@@ -62,7 +64,6 @@ class InboundTests(CommonCommonTests, unittest.TestCase):
         module = getModule(__name__)
         self.dataPath = module.filePath.sibling("data")
 
-
     def dataFile(self, name):
         """
         Get the contents of a given data file from the 'data/mail' test
@@ -70,13 +71,12 @@ class InboundTests(CommonCommonTests, unittest.TestCase):
         """
         return self.dataPath.child(name).getContent()
 
-
     def test_checkDSNFailure(self):
 
         data = {
-            'good_reply' : (False, None, None),
-            'dsn_failure_no_original' : (True, 'failed', None),
-            'dsn_failure_no_ics' : (True, 'failed', None),
+            'good_reply': (False, None, None),
+            'dsn_failure_no_original': (True, 'failed', None),
+            'dsn_failure_no_ics': (True, 'failed', None),
             'dsn_failure_with_ics' : (True, 'failed', '''BEGIN:VCALENDAR
 VERSION:2.0
 CALSCALE:GREGORIAN
@@ -123,7 +123,6 @@ END:VCALENDAR
             msg = email.message_from_string(self.dataFile(filename))
             self.assertEquals(self.receiver.checkDSN(msg), expected)
 
-
     @inlineCallbacks
     def test_processDSN(self):
 
@@ -188,7 +187,6 @@ END:VCALENDAR
 
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
     @inlineCallbacks
     def test_processReply(self):
         # Make sure an unknown token in an older email is deleted
@@ -216,7 +214,6 @@ END:VCALENDAR
 
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
     @inlineCallbacks
     def test_processReplyMissingOrganizer(self):
         msg = email.message_from_string(self.dataFile('reply_missing_organizer'))
@@ -236,7 +233,6 @@ END:VCALENDAR
 
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
     @inlineCallbacks
     def test_processReplyMissingAttendee(self):
         msg = email.message_from_string(self.dataFile('reply_missing_attendee'))
@@ -255,7 +251,6 @@ END:VCALENDAR
 
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
     @inlineCallbacks
     def test_processReplyMissingAttachment(self):
 
@@ -278,7 +273,6 @@ END:VCALENDAR
 
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
     @inlineCallbacks
     def test_injectMessage(self):
 
@@ -310,7 +304,6 @@ END:VCALENDAR
             result.responses[0].reqstatus.toString()
         )
 
-
     @inlineCallbacks
     def test_injectMessageWithError(self):
 
@@ -342,7 +335,6 @@ END:VCALENDAR
             result.responses[0].reqstatus.toString()
         )
 
-
     @inlineCallbacks
     def test_work(self):
 
@@ -370,7 +362,6 @@ END:VCALENDAR
         yield txn.commit()
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
     def test_shouldDeleteAllMail(self):
 
         # Delete if the mail server is on the same host and using our
@@ -403,7 +394,6 @@ END:VCALENDAR
             "com.apple.calendarserver"
         ))
 
-
     @inlineCallbacks
     def test_deletion(self):
         """
@@ -426,9 +416,9 @@ END:VCALENDAR
         self.patch(proto, "fetchNextMessage", stubFetchNextMessage)
         self.patch(proto, "cbFlagDeleted", stubCbFlagDeleted)
         results = {
-            "ignored" : (
+            "ignored": (
                 {
-                    "RFC822" : "a message"
+                    "RFC822": "a message"
                 }
             )
         }
@@ -457,7 +447,6 @@ END:VCALENDAR
         yield proto.cbGotMessage(results, "xyzzy")
         self.assertEquals(self.flagDeletedResult, "xyzzy")
 
-
     @inlineCallbacks
     def test_missingIMAPMessages(self):
         """
@@ -465,6 +454,7 @@ END:VCALENDAR
         """
 
         class DummyResult(object):
+
             def __init__(self):
                 self._values = []
 
@@ -477,13 +467,67 @@ END:VCALENDAR
 
         imap4 = IMAP4DownloadProtocol()
         imap4.messageUIDs = []
-        imap4.fetchNextMessage = lambda : None
+        imap4.fetchNextMessage = lambda: None
 
         result = yield imap4.cbGotMessage(noResult, [])
         self.assertTrue(result is None)
         result = yield imap4.cbGotMessage(missingKey, [])
         self.assertTrue(result is None)
 
+    @inlineCallbacks
+    def test_Twisted_16_6_bug(self):
+        """
+        Make sure L{IMAP4DownloadProtocol.cbGotSearch} works.
+
+        There is a bug in Twisted 16.6 where the IMAP4Client._fetch method does not
+        accept a MessageSet argument - though it previously did.
+
+        We also need to double check that IMAP4Client._store works with a MessageSet.
+        """
+
+        imap4 = IMAP4DownloadProtocol()
+        imap4.sendCommand = lambda cmd: succeed(cmd)
+        imap4._cbFetch = lambda result, requestedParts, structured: {123: {"UID": "456"}}
+        imap4.cbGotMessage = lambda results, messageList: succeed(True)
+        result = yield imap4.cbGotSearch((123,))
+        self.assertTrue(result is None)
+
+        imap4 = IMAP4DownloadProtocol()
+        imap4.sendCommand = lambda cmd: succeed(cmd)
+        imap4._cbFetch = lambda result, requestedParts, structured: {"123": {"UID": "456"}}
+        imap4.cbMessageUnseen = lambda results, messageList: succeed(True)
+        ms = MessageSet()
+        ms.add(123)
+        result = yield imap4.cbFlagUnseen(ms)
+        self.assertTrue(result is None)
+
+        imap4 = IMAP4DownloadProtocol()
+        imap4.sendCommand = lambda cmd: succeed(cmd)
+        imap4._cbFetch = lambda result, requestedParts, structured: {"123": {"UID": "456"}}
+        imap4.cbMessageDeleted = lambda results, messageList: succeed(True)
+        ms = MessageSet()
+        ms.add(123)
+        result = yield imap4.cbFlagDeleted(ms)
+        self.assertTrue(result is None)
+
+    def test_IMAP4Logger_err(self):
+        """
+        Make sure L{IMAP4Logger.err} works when twisted.mail.imap4 calls log.err().
+        """
+
+        class DummyTransport(object):
+
+            def __init__(self):
+                self.called = False
+
+            def loseConnection(self):
+                self.called = True
+
+        imap4 = IMAP4DownloadProtocol()
+        imap4.transport = DummyTransport()
+        imap4.state = "FOOBAR"
+        imap4.dispatchCommand("A001", "OK")
+        self.assertTrue(imap4.transport.called)
 
     def test_sanitizeCalendar(self):
         """
@@ -492,7 +536,7 @@ END:VCALENDAR
 
         data = """BEGIN:VCALENDAR
 VERSION:2.0
-METHOD:REPLY
+METHOD:Reply
 BEGIN:VEVENT
 UID:12345-67890
 DTSTAMP:20130208T120000Z
@@ -508,18 +552,20 @@ END:VCALENDAR
         calendar = Component.fromString(data)
         self.assertFalse(calendar.hasProperty("PRODID"))
         self.assertTrue(calendar.masterComponent().hasProperty("STATUS"))
+        prop = calendar.getProperty("METHOD")
+        self.assertEquals(prop.value(), "Reply")
         sanitizeCalendar(calendar)
+        prop = calendar.getProperty("METHOD")
+        self.assertEquals(prop.value(), "REPLY")
         self.assertTrue(calendar.hasProperty("PRODID"))
         self.assertFalse(calendar.masterComponent().hasProperty("STATUS"))
 
 
-
 class StubFactory(object):
 
     def __init__(self, actionTaken, deleteAllMail):
         self.actionTaken = actionTaken
         self.deleteAllMail = deleteAllMail
 
-
     def handleMessage(self, messageData):
         return succeed(self.actionTaken)
