@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,7 +56,7 @@ ATTENDEE;EMAIL=nocn@example.com;PARTSTAT=ACCEPTED:urn:uuid:A592CF8B-4FC8-4E4
  F-B543-B2F29A7EEB0B
 ORGANIZER;CN=Th\xe9 Organizer;EMAIL=organizer@example.com:urn:uuid:C3B38B00-
  4166-11DD-B22C-A07C87E02F6A
-SUMMARY:t\xe9sting outbound( )
+SUMMARY:testing outbound( )\\nEmbedded: Header
 DESCRIPTION:awesome description with "<" and "&"
 END:VEVENT
 END:VCALENDAR
@@ -255,13 +255,13 @@ ORGANIZER = "urn:uuid:C3B38B00-4166-11DD-B22C-A07C87E02F6A"
 ATTENDEE = "mailto:attendee@example.com"
 ICALUID = "CFDD5E46-4F74-478A-9311-B3FF905449C3"
 
+
 class DummySMTPSender(object):
 
     def __init__(self):
         self.reset()
         self.shouldSucceed = True
 
-
     def reset(self):
         self.sendMessageCalled = False
         self.fromAddr = None
@@ -269,7 +269,6 @@ class DummySMTPSender(object):
         self.msgId = None
         self.message = None
 
-
     def sendMessage(self, fromAddr, toAddr, msgId, message):
         self.sendMessageCalled = True
         self.fromAddr = fromAddr
@@ -279,7 +278,6 @@ class DummySMTPSender(object):
         return succeed(self.shouldSucceed)
 
 
-
 class OutboundTests(unittest.TestCase):
 
     @inlineCallbacks
@@ -294,7 +292,6 @@ class OutboundTests(unittest.TestCase):
             return self.sender
         self.patch(IMIPInvitationWork, "getMailSender", _getSender)
 
-
     @inlineCallbacks
     def test_work(self):
         txn = self.store.newTransaction()
@@ -320,7 +317,6 @@ class OutboundTests(unittest.TestCase):
         self.assertEquals(record.attendee, ATTENDEE)
         self.assertEquals(record.icaluid, ICALUID)
 
-
     @inlineCallbacks
     def test_workFailure(self):
         self.sender.smtpSender.shouldSucceed = False
@@ -335,7 +331,6 @@ class OutboundTests(unittest.TestCase):
         yield txn.commit()
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
     def _interceptEmail(
         self, inviteState, calendar, orgEmail, orgCn,
         attendees, fromAddress, replyToAddress, toAddress, language="en"
@@ -355,7 +350,6 @@ class OutboundTests(unittest.TestCase):
             language=language)
         return self.results
 
-
     @inlineCallbacks
     def test_outbound(self):
         """
@@ -386,6 +380,7 @@ class OutboundTests(unittest.TestCase):
                 u"Th\xe9 Organizer <organizer@example.com>",
                 "=?utf-8?q?Th=C3=A9_Organizer_=3Corganizer=40example=2Ecom=3E?=",
                 "attendee@example.com",
+                "Event invitation: testing outbound( ) Embedded: Header",
             ),
 
             # Update
@@ -420,6 +415,7 @@ END:VCALENDAR
                 u"Th\xe9 Organizer <organizer@example.com>",
                 "=?utf-8?q?Th=C3=A9_Organizer_=3Corganizer=40example=2Ecom=3E?=",
                 "attendee@example.com",
+                "=?utf-8?q?Event_update=3A_t=C3=A9sting_outbound=28_=29_*update*?=",
             ),
 
             # Reply
@@ -451,13 +447,14 @@ END:VCALENDAR
                 "attendee@example.com",
                 "attendee@example.com",
                 "organizer@example.com",
+                "=?utf-8?q?Event_reply=3A_t=C3=A9sting_outbound=28_=29_*reply*?=",
             ),
 
         )
         for (
             inputCalendar, UID, inputOriginator, inputRecipient, inviteState,
             outputOrganizerEmail, outputOrganizerName, outputAttendeeList,
-            outputFrom, encodedFrom, outputRecipient
+            outputFrom, encodedFrom, outputRecipient, outputSubject
         ) in data:
 
             txn = self.store.newTransaction()
@@ -478,8 +475,9 @@ END:VCALENDAR
             self.assertEquals(self.attendees, outputAttendeeList)
             self.assertEquals(self.fromAddress, outputFrom)
             self.assertEquals(self.toAddress, outputRecipient)
+            self.assertEquals(msg["Subject"], outputSubject)
 
-            if UID: # The organizer is local, and server is sending to remote
+            if UID:  # The organizer is local, and server is sending to remote
                     # attendee
                 txn = self.store.newTransaction()
                 record = (yield txn.imipGetToken(inputOriginator, inputRecipient, UID))
@@ -497,7 +495,7 @@ END:VCALENDAR
                     self.calendar.getAttendeeProperty([orgValue]).value()
                 )
 
-            else: # Reply only -- the attendee is local, and server is sending reply to remote organizer
+            else:  # Reply only -- the attendee is local, and server is sending reply to remote organizer
 
                 self.assertEquals(msg["Reply-To"], self.fromAddress)
 
@@ -515,7 +513,6 @@ END:VCALENDAR
             yield txn.commit()
             self.assertFalse(self.sender.smtpSender.sendMessageCalled)
 
-
     @inlineCallbacks
     def test_tokens(self):
         txn = self.store.newTransaction()
@@ -546,7 +543,6 @@ END:VCALENDAR
         self.assertEquals((yield txn.imipLookupByToken(record1.token)), [])
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_mailtoTokens(self):
         """
@@ -589,7 +585,6 @@ END:VCALENDAR
         yield txn.commit()
         self.assertEquals(record.token, origRecord.token)
 
-
     def generateSampleEmail(self, caltext=initialInviteText):
         """
         Invoke L{MailHandler.generateEmail} and parse the result.
@@ -609,7 +604,6 @@ END:VCALENDAR
         message = email.message_from_string(msgTxt)
         return msgID, message
 
-
     def test_generateEmail(self):
         """
         L{MailHandler.generateEmail} generates a MIME-formatted email with a
@@ -624,7 +618,6 @@ END:VCALENDAR
         ])
         self.assertEquals(actualTypes, expectedTypes)
 
-
     def test_generateEmail_noOrganizerCN(self):
         """
         L{MailHandler.generateEmail} generates a MIME-formatted email when
@@ -645,7 +638,6 @@ END:VCALENDAR
         message = email.message_from_string(msgTxt)
         self.assertTrue(message is not None)
 
-
     def test_generateEmail_noAttendeeCN(self):
         """
         L{MailHandler.generateEmail} generates a MIME-formatted email when
@@ -666,7 +658,6 @@ END:VCALENDAR
         message = email.message_from_string(msgTxt)
         self.assertTrue(message is not None)
 
-
     def test_messageID(self):
         """
         L{SMTPSender.betterMessageID} generates a Message-ID domain matching
@@ -677,7 +668,6 @@ END:VCALENDAR
         self.assertEquals(message['Message-ID'], msgID)
         self.assertEqual(msgID[:-1].split("@")[1], config.ServerHostName)
 
-
     def test_alwaysIncludeTimezones(self):
         """
         L{MailHandler.generateEmail} generates a MIME-formatted email with a
@@ -697,7 +687,6 @@ END:VCALENDAR
         self.assertTrue("BEGIN:VTIMEZONE" in caldata)
         self.assertTrue("TZID:America/New_York" in caldata)
 
-
     def test_emailEncoding(self):
         """
         L{MailHandler.generateEmail} will preserve any non-ASCII characters
@@ -723,7 +712,6 @@ END:VCALENDAR
             u'<a href="mailto:user01@localhost">User Z\u00e9ro One</a>',
             htmlText)
 
-
     def test_emailQuoting(self):
         """
         L{MailHandler.generateEmail} will HTML-quote all relevant fields in the
@@ -738,7 +726,6 @@ END:VCALENDAR
         self.assertIn(expectedPlain, plainPart)
         self.assertIn(expectedHTML, htmlPart)
 
-
     def test_stringFormatTemplateLoader(self):
         """
         L{StringFormatTemplateLoader.load} will convert a template with
@@ -765,7 +752,6 @@ END:VCALENDAR
             ["<test><alpha>hello</alpha>world</test>"]
         )
 
-
     def test_templateLoaderWithAttributes(self):
         """
         L{StringFormatTemplateLoader.load} will convert a template with
@@ -795,7 +781,6 @@ END:VCALENDAR
             ]
         )
 
-
     def test_templateLoaderTagSoup(self):
         """
         L{StringFormatTemplateLoader.load} will convert a template with
@@ -822,6 +807,10 @@ END:VCALENDAR
                           ['<test><alpha beta="before hello after">'
                            'inner</alpha>world</test>'])
 
+    def test_scrubHeader(self):
+
+        self.assertEquals(self.sender._scrubHeader("ABC"), "ABC")
+        self.assertEquals(self.sender._scrubHeader("ABC: 123\nXYZ: 456"), "ABC: 123 XYZ: 456")
 
 
 def partByType(message, contentType):
