@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -52,7 +52,6 @@ class ImplicitSchedulingWorkError(Exception):
     pass
 
 
-
 # TODO:
 #
 # Handle the case where a PUT removes the ORGANIZER property. That should be equivalent to cancelling the entire meeting.
@@ -91,7 +90,6 @@ class ImplicitScheduler(object):
         self.not_allowed = ImplicitScheduler.NotAllowedExceptionDetails(ex, ex_args, ex_kwargs)
         self.allowed_to_schedule = False
 
-
     def testSchedulingAllowed(self):
         """
         Called to raise an exception if scheduling is not allowed. This method should be called
@@ -101,7 +99,6 @@ class ImplicitScheduler(object):
         if not self.allowed_to_schedule:
             raise self.not_allowed.type(*self.not_allowed.args, **self.not_allowed.kwargs)
 
-
     @inlineCallbacks
     def testImplicitSchedulingPUT(self, parent, resource, calendar, internal_request=False):
         """
@@ -198,7 +195,6 @@ class ImplicitScheduler(object):
 
         returnValue((self.action != "none", new_type == "schedule",))
 
-
     @inlineCallbacks
     def testImplicitSchedulingDELETE(self, parent, resource, calendar, internal_request=False):
         """
@@ -230,7 +226,6 @@ class ImplicitScheduler(object):
 
         returnValue((self.action != "none", False,))
 
-
     @inlineCallbacks
     def testAttendeeEvent(self, parent, resource, calendar):
         """
@@ -261,7 +256,6 @@ class ImplicitScheduler(object):
 
         returnValue(self.state in ("attendee", "attendee-missing",))
 
-
     def checkValidOrganizer(self):
         """
         Make sure the ORGANIZER is allowed to do certain scheduling operations.
@@ -278,7 +272,6 @@ class ImplicitScheduler(object):
                     "Organizer cannot schedule",
                 ))
 
-
     @inlineCallbacks
     def checkSchedulingObjectResource(self, resource):
 
@@ -300,7 +293,6 @@ class ImplicitScheduler(object):
 
         returnValue(False)
 
-
     @inlineCallbacks
     def checkImplicitState(self):
         # Get some useful information from the calendar
@@ -321,7 +313,6 @@ class ImplicitScheduler(object):
 
         returnValue(self.state is not None)
 
-
     @inlineCallbacks
     def doImplicitScheduling(self, do_smart_merge=False, split_details=None):
         """
@@ -354,7 +345,6 @@ class ImplicitScheduler(object):
         else:
             returnValue(self.return_calendar if hasattr(self, "return_calendar") else self.calendar)
 
-
     @inlineCallbacks
     def refreshAllAttendeesExceptSome(self, txn, resource, except_attendees=(), only_attendees=None):
         """
@@ -378,6 +368,11 @@ class ImplicitScheduler(object):
 
         # Get some useful information from the calendar
         yield self.extractCalendarData()
+
+        # Double-check we still have an organizer as a delayed refresh might happen after the original event
+        # was changed and the organizer removed
+        if self.organizer is None:
+            returnValue(None)
         self.organizerAddress = (yield calendarUserFromCalendarUserAddress(self.organizer, self.txn))
 
         # Originator is the organizer in this case
@@ -404,7 +399,6 @@ class ImplicitScheduler(object):
         if refreshCount and self.logItems is not None:
             self.logItems["itip.refreshes"] = refreshCount
 
-
     @inlineCallbacks
     def queuedOrganizerProcessing(self, txn, action, home, resource, uid, calendar_old, calendar_new, smart_merge):
         """
@@ -469,7 +463,6 @@ class ImplicitScheduler(object):
 
         yield self.doImplicitOrganizer(queued=True)
 
-
     @inlineCallbacks
     def queuedOrganizerSending(self, txn, action, home, resource, uid, organizer, attendee, itipmsg, no_refresh):
         """
@@ -501,7 +494,6 @@ class ImplicitScheduler(object):
 
         yield self.processSend(attendee, itipmsg, jobqueue=False)
 
-
     @inlineCallbacks
     def sendAttendeeReply(self, txn, resource):
         self.txn = txn
@@ -523,7 +515,6 @@ class ImplicitScheduler(object):
         self.attendeeAddress = (yield calendarUserFromCalendarUserUID(self.calendar_home.uid(), self.txn))
         self.originator = self.attendee = self.attendeeAddress.record.canonicalCalendarUserAddress()
 
-
         # Check SCHEDULE-AGENT
         if not self.checkOrganizerScheduleAgent():
             returnValue(False)
@@ -533,7 +524,6 @@ class ImplicitScheduler(object):
 
         returnValue(result != False)
 
-
     @inlineCallbacks
     def extractCalendarData(self):
 
@@ -562,7 +552,6 @@ class ImplicitScheduler(object):
         self.uid = self.calendar.resourceUID()
         self.instances = set(self.calendar.getComponentInstances())
 
-
     @inlineCallbacks
     def extractAttendees(self):
         """
@@ -580,7 +569,6 @@ class ImplicitScheduler(object):
         for attendee, _ignore in self.attendeesByInstance:
             self.attendees.add(attendee)
 
-
     @inlineCallbacks
     def hasCalendarResourceUIDSomewhereElse(self, check_resource, mode):
         """
@@ -602,7 +590,6 @@ class ImplicitScheduler(object):
                 "Cannot duplicate scheduling object resource",
             ))
 
-
     @inlineCallbacks
     def isOrganizerScheduling(self):
         """
@@ -624,7 +611,6 @@ class ImplicitScheduler(object):
 
         returnValue(True)
 
-
     @inlineCallbacks
     def isAttendeeScheduling(self):
 
@@ -658,14 +644,12 @@ class ImplicitScheduler(object):
 
         returnValue(False)
 
-
     def makeScheduler(self):
         """
         Convenience method which we can override in unit tests to make testing easier.
         """
         return CalDAVScheduler(self.txn, self.calendar_home.uid(), logItems=self.logItems)
 
-
     @inlineCallbacks
     def doImplicitOrganizer(self, queued=False):
 
@@ -823,7 +807,6 @@ class ImplicitScheduler(object):
             except KeyError:
                 pass
 
-
     def isOrganizerChangeInsignificant(self):
         """
         Detect exactly how an organizer update has changed an event and report back the key items
@@ -858,6 +841,10 @@ class ImplicitScheduler(object):
             checkOrganizerValue = False
             for rid, props in diffs.iteritems():
 
+                # Ignore this case - it should only happen when there is no master component
+                if props is None:
+                    continue
+
                 # Ignore sequence only changes
                 if "SEQUENCE" in props and len(props) == 1:
                     continue
@@ -872,7 +859,7 @@ class ImplicitScheduler(object):
                     "DURATION",
                     "DUE",
                     "RECURRENCE-ID",
-                )]):
+                )]) or len(props) == 0:
                     date_changed_rids.add(rid)
 
                 # Check to see whether a change to R-ID's happened
@@ -959,7 +946,6 @@ class ImplicitScheduler(object):
             recurrence_reschedule, status_cancelled, only_status
         )
 
-
     def findRemovedAttendees(self):
         """
         Look for attendees that have been removed from any instances. Save those off
@@ -1035,7 +1021,6 @@ class ImplicitScheduler(object):
                 if (attendee, rid) not in mappedNew and rid not in oldexdates:
                     self.cancelledAttendees.add((attendee, rid,))
 
-
     def findRemovedAttendeesOnRecurrenceChange(self):
         """
         Look for attendees that have been removed during a change to the overall recurrence.
@@ -1059,7 +1044,6 @@ class ImplicitScheduler(object):
             if attendee not in new_master_attendees:
                 self.cancelledAttendees.add((attendee, rid,))
 
-
     def checkStatusCancelled(self, cancelled, only_status):
         """
         Check to see whether STATUS:CANCELLED has been added to any/all instances, and if so
@@ -1075,7 +1059,6 @@ class ImplicitScheduler(object):
             if only_status:
                 self.action = "modify-cancelled"
 
-
     def coerceAttendeesPartstatOnCreate(self):
         """
         Make sure any attendees handled by the server start off with PARTSTAT=NEEDS-ACTION as
@@ -1088,7 +1071,6 @@ class ImplicitScheduler(object):
             if attendee.parameterValue("SCHEDULE-AGENT", "SERVER").upper() == "SERVER" and attendee.hasParameter("PARTSTAT"):
                 attendee.setParameter("PARTSTAT", "NEEDS-ACTION")
 
-
     def coerceAttendeesPartstatOnModify(self):
         """
         Make sure that the organizer does not change attendees' PARTSTAT to anything
@@ -1125,7 +1107,6 @@ class ImplicitScheduler(object):
             if changed:
                 self.calendar.addComponent(newcomp)
 
-
     def compareAttendeePartstats(self, old_component, new_component):
         """
         Compare two components, old and new, and make sure the Organizer has not changed the PARTSTATs
@@ -1150,7 +1131,6 @@ class ImplicitScheduler(object):
 
         return changed
 
-
     def coerceOrganizerScheduleAgent(self):
         """
         Do not allow SCHEDULE-AGENT=CLIENT/NONE for organizers hosted by this server when they schedule. Coerce to
@@ -1159,7 +1139,6 @@ class ImplicitScheduler(object):
 
         self.calendar.removePropertyParameters("ORGANIZER", ("SCHEDULE-AGENT",))
 
-
     @inlineCallbacks
     def coerceAttendeeScheduleAgent(self):
         """
@@ -1177,7 +1156,6 @@ class ImplicitScheduler(object):
                 if coerced[cuaddr]:
                     attendee.removeParameter("SCHEDULE-AGENT")
 
-
     @inlineCallbacks
     def queuedScheduleWithAttendees(self):
 
@@ -1206,7 +1184,6 @@ class ImplicitScheduler(object):
 
         self.logItems["itip.requests"] = total
 
-
     @inlineCallbacks
     def scheduleWithAttendees(self):
 
@@ -1223,7 +1200,6 @@ class ImplicitScheduler(object):
         if self.logItems is not None:
             self.logItems["itip.requests"] = total
 
-
     @inlineCallbacks
     def processCancels(self, queued=False):
         """
@@ -1294,7 +1270,6 @@ class ImplicitScheduler(object):
 
         returnValue(count)
 
-
     @inlineCallbacks
     def processRequests(self, cancel_count=0, queued=False):
         """
@@ -1371,7 +1346,6 @@ class ImplicitScheduler(object):
 
         returnValue(count)
 
-
     @inlineCallbacks
     def processSend(self, attendee, itipmsg, jobqueue=True, count=0):
         """
@@ -1415,7 +1389,6 @@ class ImplicitScheduler(object):
             response = (yield scheduler.doSchedulingViaPUT(self.originator, (attendee,), itipmsg, internal_request=True, suppress_refresh=self.suppress_refresh))
             self.handleSchedulingResponse(response, True)
 
-
     def handleSchedulingResponse(self, response, is_organizer):
 
         # For a queued operation we stash the response away for the work item to deal with
@@ -1437,7 +1410,6 @@ class ImplicitScheduler(object):
                 for p in recipients[recipient]:
                     p.setParameter("SCHEDULE-STATUS", status.split(";")[0])
 
-
     @inlineCallbacks
     def doImplicitAttendee(self):
 
@@ -1592,7 +1564,6 @@ class ImplicitScheduler(object):
                 log.debug("Implicit - attendee '{attendee}' is updating UID without server scheduling: '{uid}'", attendee=self.attendee, uid=self.uid)
                 # Nothing else to do
 
-
     @inlineCallbacks
     def doImplicitMissingAttendee(self):
 
@@ -1627,7 +1598,7 @@ class ImplicitScheduler(object):
                             break
 
                     if found_old:
-                        log.error("valid-attendee-change: Cannot remove ATTENDEE: UID:%s" % (self.uid,))
+                        log.error("valid-attendee-change: Cannot remove ATTENDEE: UID: {uid}", uid=self.uid)
                         raise HTTPError(ErrorResponse(
                             responsecode.FORBIDDEN,
                             (caldav_namespace, "valid-attendee-change"),
@@ -1649,7 +1620,6 @@ class ImplicitScheduler(object):
                     None,
                 )
 
-
     def checkOrganizerScheduleAgent(self):
 
         is_server = self.calendar.getOrganizerScheduleAgent()
@@ -1673,7 +1643,6 @@ class ImplicitScheduler(object):
 
         return is_server
 
-
     @inlineCallbacks
     def getOrganizersCopy(self):
         """
@@ -1696,7 +1665,6 @@ class ImplicitScheduler(object):
             # of the event is up to date and "authoritative". So we pretend that is the organizer copy
             self.organizer_calendar = self.oldcalendar
 
-
     def isAttendeeChangeInsignificant(self):
         """
         Check whether the change is significant (PARTSTAT) or allowed
@@ -1722,7 +1690,6 @@ class ImplicitScheduler(object):
         differ = iCalDiff(oldcalendar, self.calendar, self.do_smart_merge, forceTRANSP=forceTRANSP)
         return differ.attendeeMerge(self.attendee)
 
-
     def scheduleWithOrganizer(self, changedRids=None):
 
         # First make sure we are allowed to schedule
@@ -1751,7 +1718,6 @@ class ImplicitScheduler(object):
             # Send scheduling message
             return self.sendToOrganizer("REPLY", itipmsg)
 
-
     def scheduleCancelWithOrganizer(self):
 
         # First make sure we are allowed to schedule
@@ -1772,7 +1738,6 @@ class ImplicitScheduler(object):
             # Send scheduling message
             return self.sendToOrganizer("CANCEL", itipmsg)
 
-
     @inlineCallbacks
     def sendToOrganizer(self, action, itipmsg):
 
