@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -68,7 +68,6 @@ __all__ = [
 log = Logger()
 
 
-
 class ScheduleViaISchedule(DeliveryService):
 
     domainServerMap = {}
@@ -78,7 +77,6 @@ class ScheduleViaISchedule(DeliveryService):
     def serviceType(cls):
         return DeliveryService.serviceType_ischedule
 
-
     @classmethod
     @inlineCallbacks
     def matchCalendarUserAddress(cls, cuaddr):
@@ -93,7 +91,6 @@ class ScheduleViaISchedule(DeliveryService):
         result = (yield super(ScheduleViaISchedule, cls).matchCalendarUserAddress(cuaddr))
         returnValue(result)
 
-
     @classmethod
     @inlineCallbacks
     def serverForDomain(cls, domain):
@@ -126,7 +123,6 @@ class ScheduleViaISchedule(DeliveryService):
 
         returnValue(cls.domainServerMap[domain])
 
-
     @inlineCallbacks
     def generateSchedulingResponses(self, refreshOnly=False):
         """
@@ -183,7 +179,6 @@ class ScheduleViaISchedule(DeliveryService):
 
         yield DeferredList(deferreds)
 
-
     def _getServerForOtherServerUser(self, recipient):
 
         if not hasattr(self, "otherServers"):
@@ -193,7 +188,7 @@ class ScheduleViaISchedule(DeliveryService):
         if serverURI not in self.otherServers:
             self.otherServers[serverURI] = IScheduleServerRecord(
                 uri=joinURL(serverURI, config.Servers.InboxName),
-                unNormalizeAddresses=not recipient.record.server().isImplicit,
+                rewriteCUAddresses=recipient.record.server().v5,
                 moreHeaders=[recipient.record.server().secretHeader(), ],
                 podding=True,
             )
@@ -201,7 +196,6 @@ class ScheduleViaISchedule(DeliveryService):
         return self.otherServers[serverURI]
 
 
-
 class IScheduleRequest(object):
 
     def __init__(self, scheduler, server, recipients, responses, refreshOnly=False):
@@ -215,7 +209,6 @@ class IScheduleRequest(object):
         self.data = None
         self.original_organizer = None
 
-
     @inlineCallbacks
     def doRequest(self):
 
@@ -287,7 +280,6 @@ class IScheduleRequest(object):
                 ))
                 self.responses.add(recipient.cuaddr, Failure(exc_value=err), reqstatus=failed_status)
 
-
     @inlineCallbacks
     def logRequest(self, request):
         """
@@ -321,7 +313,6 @@ class IScheduleRequest(object):
         iostr.write("\n\n>>>> Request end\n")
         returnValue(iostr.getvalue())
 
-
     @inlineCallbacks
     def logResponse(self, response):
         """
@@ -354,7 +345,6 @@ class IScheduleRequest(object):
         iostr.write("\n\n>>>> Response end\n")
         returnValue(iostr.getvalue())
 
-
     @inlineCallbacks
     def _prepareRequest(self, host, port):
         """
@@ -365,7 +355,6 @@ class IScheduleRequest(object):
         component, method = (yield self._prepareData())
         yield self._prepareHeaders(host, port, component, method)
 
-
     @inlineCallbacks
     def _prepareHeaders(self, host, port, component, method):
         """
@@ -379,13 +368,28 @@ class IScheduleRequest(object):
 
         # The Originator must be the ORGANIZER (for a request) or ATTENDEE (for a reply)
         originator = self.scheduler.organizer.cuaddr if self.scheduler.isiTIPRequest else self.scheduler.attendee
-        if self.server.unNormalizeAddresses:
-            originator = yield normalizeCUAddress(originator, normalizationLookup, self.scheduler.txn.directoryService().recordWithCalendarUserAddress, toCanonical=False)
+        if self.server.rewriteCUAddresses:
+            originator = yield normalizeCUAddress(
+                originator,
+                normalizationLookup,
+                self.scheduler.txn.directoryService().recordWithCalendarUserAddress,
+                toCanonical=False,
+                toURN_UUID=self.server.podding(),
+            )
         self.headers.addRawHeader("Originator", utf8String(originator))
         self.sign_headers.append("Originator")
 
         for recipient in self.recipients:
-            self.headers.addRawHeader("Recipient", utf8String(recipient.cuaddr))
+            cuaddr = recipient.cuaddr
+            if self.server.rewriteCUAddresses and self.server.podding():
+                cuaddr = yield normalizeCUAddress(
+                    cuaddr,
+                    normalizationLookup,
+                    self.scheduler.txn.directoryService().recordWithCalendarUserAddress,
+                    toCanonical=False,
+                    toURN_UUID=True,
+                )
+            self.headers.addRawHeader("Recipient", utf8String(cuaddr))
 
         # Only one Recipient header as they get concatenated in ischedule-relaxed canonicalization
         self.sign_headers.append("Recipient")
@@ -412,7 +416,6 @@ class IScheduleRequest(object):
         if self.refreshOnly:
             self.headers.addRawHeader("X-CALENDARSERVER-ITIP-REFRESHONLY", "T")
 
-
     def _doAuthentication(self):
         if self.server.authentication and self.server.authentication[0] == "basic":
             self.headers.setHeader(
@@ -421,7 +424,6 @@ class IScheduleRequest(object):
             )
             self.sign_headers.append("Authorization")
 
-
     @inlineCallbacks
     def _prepareData(self):
         """
@@ -434,11 +436,13 @@ class IScheduleRequest(object):
             # Need to remap cuaddrs from urn:uuid
             normalizedCalendar = self.scheduler.calendar.duplicate()
             self.original_organizer = normalizedCalendar.getOrganizer()
-            if self.server.unNormalizeAddresses:
+            if self.server.rewriteCUAddresses:
                 yield normalizedCalendar.normalizeCalendarUserAddresses(
                     normalizationLookup,
                     self.scheduler.txn.directoryService().recordWithCalendarUserAddress,
-                    toCanonical=False)
+                    toCanonical=False,
+                    toURN_UUID=self.server.podding(),
+                )
 
             # For VFREEBUSY we need to strip out ATTENDEEs that do not match the recipient list
             if self.scheduler.isfreebusy:
@@ -454,7 +458,6 @@ class IScheduleRequest(object):
             method = cal.propertyValue("METHOD")
             returnValue((component, method,))
 
-
     @inlineCallbacks
     def _processRequest(self, ssl, host, port, path):
         if not self.server.podding() and config.Scheduling.iSchedule.DKIM.Enabled:
@@ -484,14 +487,13 @@ class IScheduleRequest(object):
         response = yield self._submitRequest(ssl, host, port, request)
         returnValue(response)
 
-
     @inlineCallbacks
     def _submitRequest(self, ssl, host, port, request):
         from twisted.internet import reactor
         f = Factory()
         f.protocol = HTTPClientProtocol
         if ssl:
-            ep = GAIEndpoint(reactor, host, port, _configuredClientContextFactory())
+            ep = GAIEndpoint(reactor, host, port, _configuredClientContextFactory(host))
         else:
             ep = GAIEndpoint(reactor, host, port)
         proto = (yield ep.connect(f))
@@ -500,7 +502,6 @@ class IScheduleRequest(object):
 
         returnValue(response)
 
-
     def _parseResponse(self, xml):
 
         # Check for correct root element
@@ -524,7 +525,7 @@ class IScheduleRequest(object):
             calendar_data = response.childOfType(CalendarData)
             if calendar_data:
                 calendar_data = str(calendar_data)
-                if self.server.unNormalizeAddresses and self.original_organizer is not None:
+                if self.server.rewriteCUAddresses and self.original_organizer is not None:
                     # Need to restore original ORGANIZER value if it got unnormalized
                     calendar = Component.fromString(calendar_data)
                     organizers = calendar.getAllPropertiesInAnyComponent("ORGANIZER", depth=1)
