@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -132,7 +132,6 @@ class DKIMUtils(object):
         else:
             log.info("DKIM: Disabled")
 
-
     @staticmethod
     def getConfiguration(config):
         """
@@ -153,40 +152,36 @@ class DKIMUtils(object):
 
         return domain, selector, key_file, algorithm, useDNSKey, useHTTPKey, usePrivateExchangeKey, expire
 
-
     @staticmethod
     def hashlib_method(algorithm):
         """
         Return hashlib function for DKIM algorithm.
         """
         return {
-            RSA1  : hashlib.sha1,
+            RSA1: hashlib.sha1,
             RSA256: hashlib.sha256,
         }[algorithm]
 
-
     @staticmethod
     def hash_name(algorithm):
         """
         Return RSA hash name for DKIM algorithm.
         """
         return {
-            RSA1  : "SHA-1",
+            RSA1: "SHA-1",
             RSA256: "SHA-256",
         }[algorithm]
 
-
     @staticmethod
     def hash_func(algorithm):
         """
         Return RSA hash name for DKIM algorithm.
         """
         return {
-            RSA1  : SHA,
+            RSA1: SHA,
             RSA256: SHA256,
         }[algorithm]
 
-
     @staticmethod
     def extractTags(data):
         """
@@ -203,7 +198,6 @@ class DKIMUtils(object):
                 pass
         return dkim_tags
 
-
     @staticmethod
     def canonicalizeHeader(name, value, dkim_tags=None, method="ischedule-relaxed"):
 
@@ -214,7 +208,6 @@ class DKIMUtils(object):
         else:
             assert "Invalid header canonicalization method: %s" % (method,)
 
-
     @staticmethod
     def relaxedHeader(name, value, dkim_tags=None):
         """
@@ -244,7 +237,6 @@ class DKIMUtils(object):
         crlf = "" if name == DKIM_SIGNATURE.lower() else "\r\n"
         return "%s:%s%s" % (name, value, crlf)
 
-
     @staticmethod
     def ischeduleHeader(name, value, dkim_tags=None):
         """
@@ -276,7 +268,6 @@ class DKIMUtils(object):
         crlf = "" if name == DKIM_SIGNATURE.lower() else "\r\n"
         return "%s:%s%s" % (name, value, crlf)
 
-
     @staticmethod
     def canonicalizeDKIMHeaderFields(value, dkim_tags):
         """
@@ -291,7 +282,6 @@ class DKIMUtils(object):
         value = " ".join(value.split())
         return value
 
-
     @staticmethod
     def canonicalizeBody(data):
         """
@@ -306,14 +296,12 @@ class DKIMUtils(object):
         data += "\r\n"
         return data
 
-
     @staticmethod
     def sign(data, privkey, hashfunc):
         h = hashfunc.new(data)
         signer = PKCS1_v1_5.new(privkey)
         return base64.b64encode(signer.sign(h))
 
-
     @staticmethod
     def verify(data, signature, pubkey, hashfunc):
         h = hashfunc.new(data)
@@ -322,7 +310,6 @@ class DKIMUtils(object):
             raise ValueError()
 
 
-
 class DKIMRequest(ClientRequest):
     """
     A ClientRequest that optionally creates a DKIM signature.
@@ -404,7 +391,6 @@ class DKIMRequest(ClientRequest):
 
         self.message_id = str(uuid.uuid4())
 
-
     @inlineCallbacks
     def sign(self):
         """
@@ -423,12 +409,11 @@ class DKIMRequest(ClientRequest):
         dkim_header = "; ".join(["%s=%s" % item for item in dkim_tags])
         self.headers.addRawHeader(DKIM_SIGNATURE, dkim_header)
 
-        log.debug("DKIM: Generated header: DKIM-Signature:%s" % (dkim_header,))
-        log.debug("DKIM: Signed headers:\n%s" % (headers,))
+        log.debug("DKIM: Generated header: DKIM-Signature:{hdr}", hdr=dkim_header)
+        log.debug("DKIM: Signed headers:\n{hdrs}", hdrs=headers)
 
         returnValue(signature)
 
-
     @inlineCallbacks
     def bodyHash(self):
         """
@@ -444,7 +429,6 @@ class DKIMRequest(ClientRequest):
 
         returnValue(base64.b64encode(self.hash_method(DKIMUtils.canonicalizeBody(data)).digest()))
 
-
     @inlineCallbacks
     def signatureHeaders(self):
         """
@@ -490,15 +474,15 @@ class DKIMRequest(ClientRequest):
 
         returnValue((headers, dkim_tags,))
 
-
     def generateSignature(self, headers):
         # Sign the hash
         if self.key_file not in self.keys:
-            self.keys[self.key_file] = RSA.importKey(open(self.key_file).read())
+            with open(self.key_file) as f:
+                key = f.read()
+            self.keys[self.key_file] = RSA.importKey(key)
         return DKIMUtils.sign(headers, self.keys[self.key_file], self.hash_func)
 
 
-
 class DKIMMissingError(Exception):
     """
     Used to indicate that the DKIM-Signature header is not present when
@@ -507,7 +491,6 @@ class DKIMMissingError(Exception):
     pass
 
 
-
 class DKIMVerificationError(Exception):
     """
     Used to indicate a DKIM verification error.
@@ -515,7 +498,6 @@ class DKIMVerificationError(Exception):
     pass
 
 
-
 class DKIMVerifier(object):
     """
     Class used to verify an DKIM-signed HTTP request.
@@ -542,7 +524,6 @@ class DKIMVerifier(object):
 
         self.time = int(time.time())
 
-
     @inlineCallbacks
     def verify(self):
         """
@@ -554,7 +535,7 @@ class DKIMVerifier(object):
 
         # Extract the set of canonicalized headers being signed
         headers = self.extractSignedHeaders()
-        log.debug("DKIM: Signed headers:\n%s" % (headers,))
+        log.debug("DKIM: Signed headers:\n{hdrs}", hdrs=headers)
 
         # Locate the public key
         pubkey = (yield self.locatePublicKey())
@@ -575,7 +556,7 @@ Headers to evaluate:
 Public key used:
 %s
 """ % (self.headers.getRawHeaders(DKIM_SIGNATURE)[0], headers, pubkey._original_data,)
-            log.debug("DKIM: %s:%s" % (msg, _debug_msg,))
+            log.debug("DKIM: {msg}:{debug}", msg=msg, debug=_debug_msg)
             if self._debug:
                 msg = "%s:%s" % (msg, _debug_msg,)
             raise DKIMVerificationError(msg)
@@ -593,12 +574,11 @@ Hash Method: %s
 Base64 encoded body:
 %s
 """ % (self.headers.getRawHeaders(DKIM_SIGNATURE), self.hash_method.__name__, base64.b64encode(body),)
-            log.debug("DKIM: %s:%s" % (msg, _debug_msg,))
+            log.debug("DKIM: {msg}:{debug}", msg=msg, debug=_debug_msg)
             if self._debug:
                 msg = "%s:%s" % (msg, _debug_msg,)
             raise DKIMVerificationError(msg)
 
-
     def processDKIMHeader(self):
         """
         Extract the DKIM-Signature header and process the tags.
@@ -610,16 +590,16 @@ Base64 encoded body:
         dkim = self.headers.getRawHeaders(DKIM_SIGNATURE)
         if dkim is None:
             msg = "No DKIM-Signature header present in the request"
-            log.debug("DKIM: " + msg)
+            log.debug("DKIM: {msg}", msg=msg)
             raise DKIMMissingError(msg)
         if len(dkim) != 1:
             # TODO: This might need to be changed if we ever support forwarding of iSchedule messages - the forwarder
             # might also sign the message and add its own header
             msg = "Only one DKIM-Signature allowed in the request"
-            log.debug("DKIM: " + msg)
+            log.debug("DKIM: {msg}", msg=msg)
             raise DKIMVerificationError(msg)
         dkim = dkim[0]
-        log.debug("DKIM: Found header: DKIM-Signature:%s" % (dkim,))
+        log.debug("DKIM: Found header: DKIM-Signature:{hdr}", hdr=dkim)
 
         # Extract tags from the header
         self.dkim_tags = DKIMUtils.extractTags(dkim)
@@ -629,7 +609,7 @@ Base64 encoded body:
         for tag in required_tags:
             if tag not in self.dkim_tags:
                 msg = "Missing DKIM-Signature tag: %s" % (tag,)
-                log.debug("DKIM: " + msg)
+                log.debug("DKIM: {msg}", msg=msg)
                 raise DKIMVerificationError(msg)
 
         check_values = {
@@ -650,7 +630,7 @@ Base64 encoded body:
             for item in test:
                 if item not in values:
                     msg = "Tag: %s has incorrect value: %s" % (tag, self.dkim_tags[tag],)
-                    log.debug("DKIM: " + msg)
+                    log.debug("DKIM: {msg}", msg=msg)
                     raise DKIMVerificationError(msg)
 
         # Check time stamp
@@ -658,7 +638,7 @@ Base64 encoded body:
             diff_time = self.time - int(self.dkim_tags["t"])
             if diff_time < -360:
                 msg = "Signature time too far in the future: %d seconds" % (diff_time,)
-                log.debug("DKIM: " + msg)
+                log.debug("DKIM: {msg}", msg=msg)
                 raise DKIMVerificationError(msg)
 
         # Check expiration
@@ -666,7 +646,7 @@ Base64 encoded body:
             diff_time = self.time - int(self.dkim_tags["x"])
             if diff_time > 0:
                 msg = "Signature expired: %d seconds" % (diff_time,)
-                log.debug("DKIM: " + msg)
+                log.debug("DKIM: {msg}", msg=msg)
                 raise DKIMVerificationError(msg)
 
         # Base64 encoded tags might include WSP which we need to ignore
@@ -678,7 +658,6 @@ Base64 encoded body:
         self.hash_func = DKIMUtils.hash_func(self.dkim_tags["a"])
         self.key_methods = self.dkim_tags["q"].split(":")
 
-
     def extractSignedHeaders(self):
         """
         Extract the set of headers from the request that are supposed to be signed. Canonicalize them
@@ -701,7 +680,6 @@ Base64 encoded body:
         # Now canonicalize the values
         return "".join([DKIMUtils.canonicalizeHeader(name, value, dkim_tags=self.dkim_tags) for name, value in headers])
 
-
     @inlineCallbacks
     def locatePublicKey(self):
         """
@@ -717,7 +695,6 @@ Base64 encoded body:
             returnValue(None)
 
 
-
 class PublicKeyLookup(object):
     """
     Abstract base class for public key lookup methods.
@@ -732,7 +709,6 @@ class PublicKeyLookup(object):
     def __init__(self, dkim_tags):
         self.dkim_tags = dkim_tags
 
-
     @inlineCallbacks
     def getPublicKey(self, useCache=True):
         """
@@ -748,7 +724,6 @@ class PublicKeyLookup(object):
 
         returnValue(self._selectKey())
 
-
     def _getSelectorKey(self):
         """
         Get a token used to uniquely identify the key being looked up. Token format will
@@ -756,7 +731,6 @@ class PublicKeyLookup(object):
         """
         raise NotImplementedError
 
-
     def _lookupKeys(self):
         """
         Do the key lookup using the actual lookup method. Return a C{list} of C{dict}
@@ -764,7 +738,6 @@ class PublicKeyLookup(object):
         """
         raise NotImplementedError
 
-
     def _selectKey(self):
         """
         Select a specific key from the list that best matches the DKIM-Signature tags
@@ -795,10 +768,9 @@ class PublicKeyLookup(object):
 
             return self._makeKey(pkey)
 
-        log.debug("DKIM: No valid public key: %s %s" % (self._getSelectorKey(), pubkeys,))
+        log.debug("DKIM: No valid public key: {sel} {keys}", sel=self._getSelectorKey(), keys=pubkeys)
         return None
 
-
     def _makeKey(self, pkey):
         """
         Turn the key tag list into an actual RSA public key object
@@ -816,16 +788,14 @@ class PublicKeyLookup(object):
             key._original_data = key_data
             return key
         except:
-            log.debug("DKIM: Unable to make public key:\n%s" % (key_data,))
+            log.debug("DKIM: Unable to make public key:\n{key}", key=key_data)
             return None
 
-
     @staticmethod
     def flushCache():
         PublicKeyLookup.keyCache = {}
 
 
-
 class PublicKeyLookup_DNSTXT(PublicKeyLookup):
 
     method = Q_DNS
@@ -837,19 +807,17 @@ class PublicKeyLookup_DNSTXT(PublicKeyLookup):
         """
         return "%s._domainkey.%s" % (self.dkim_tags["s"], self.dkim_tags["d"],)
 
-
     @inlineCallbacks
     def _lookupKeys(self):
         """
         Do the key lookup using the actual lookup method.
         """
-        log.debug("DKIM: TXT lookup: %s" % (self._getSelectorKey(),))
+        log.debug("DKIM: TXT lookup: {key}", key=self._getSelectorKey())
         data = (yield lookupDataViaTXT(self._getSelectorKey()))
-        log.debug("DKIM: TXT lookup results: %s\n%s" % (self._getSelectorKey(), "\n".join(data),))
+        log.debug("DKIM: TXT lookup results: {key}\n{data}", key=self._getSelectorKey(), data="\n".join(data))
         returnValue(tuple([DKIMUtils.extractTags(line) for line in data]))
 
 
-
 class PublicKeyLookup_HTTP_WellKnown(PublicKeyLookup):
 
     method = Q_HTTP
@@ -863,7 +831,6 @@ class PublicKeyLookup_HTTP_WellKnown(PublicKeyLookup):
         host = ".".join(self.dkim_tags["d"].split(".")[-2:])
         return "https://%s/.well-known/domainkey/%s/%s" % (host, self.dkim_tags["d"], self.dkim_tags["s"],)
 
-
     @inlineCallbacks
     def _getURI(self):
         """
@@ -873,7 +840,7 @@ class PublicKeyLookup_HTTP_WellKnown(PublicKeyLookup):
         # First we do an SRV lookup for _domainkey to get the public key server host/port
         result = (yield lookupServerViaSRV(self.dkim_tags["d"], service="_domainkey_lookup"))
         if result is None:
-            log.debug("DKIM: SRV _domainkey failed on: %s trying domain directly" % (self.dkim_tags["d"],))
+            log.debug("DKIM: SRV _domainkey failed on: {tag} trying domain directly", tag=self.dkim_tags["d"])
             host = self.dkim_tags["d"]
             port = ""
             scheme = "https"
@@ -887,7 +854,6 @@ class PublicKeyLookup_HTTP_WellKnown(PublicKeyLookup):
 
         returnValue("%s://%s%s/.well-known/domainkey/%s/%s" % (scheme, host, port, self.dkim_tags["d"], self.dkim_tags["s"],))
 
-
     @inlineCallbacks
     def _lookupKeys(self):
         """
@@ -897,24 +863,23 @@ class PublicKeyLookup_HTTP_WellKnown(PublicKeyLookup):
         # First we do an SRV lookup for _domainkey to get the public key server URI
         uri = (yield self._getURI())
 
-        log.debug("DKIM: HTTP/.well-known lookup: %s" % (uri,))
+        log.debug("DKIM: HTTP/.well-known lookup: {uri}", uri=uri)
         response = (yield getURL(uri))
         if response is None or response.code / 100 != 2:
-            log.debug("DKIM: Failed http/well-known lookup: %s %s" % (uri, response,))
+            log.debug("DKIM: Failed http/well-known lookup: {uri} {resp}", uri=uri, resp=response)
             returnValue(())
 
         ct = response.headers.getRawHeaders("content-type", ("bogus/type",))[0]
         ct = ct.split(";", 1)
         ct = ct[0].strip()
         if ct not in ("text/plain",):
-            log.debug("DKIM: Failed http/well-known lookup: wrong content-type returned %s %s" % (uri, ct,))
+            log.debug("DKIM: Failed http/well-known lookup: wrong content-type returned {uri} {ct}", uri=uri, ct=ct)
             returnValue(())
 
-        log.debug("DKIM: HTTP/.well-known lookup results: %s\n%s" % (uri, response.data,))
+        log.debug("DKIM: HTTP/.well-known lookup results: {uri}\n{resp}", uri=uri, resp=response.data)
         returnValue(tuple([DKIMUtils.extractTags(line) for line in response.data.splitlines()]))
 
 
-
 class PublicKeyLookup_PrivateExchange(PublicKeyLookup):
 
     method = Q_PRIVATE
@@ -927,7 +892,6 @@ class PublicKeyLookup_PrivateExchange(PublicKeyLookup):
         """
         return "%s#%s" % (self.dkim_tags["d"], self.dkim_tags["s"],)
 
-
     def _lookupKeys(self):
         """
         Key information is stored in a file, one record per line.
@@ -939,23 +903,22 @@ class PublicKeyLookup_PrivateExchange(PublicKeyLookup):
             return succeed(())
         keyfile = os.path.join(PublicKeyLookup_PrivateExchange.directory, self._getSelectorKey())
         if not os.path.exists(keyfile):
-            log.debug("DKIM: Failed private-exchange lookup: no path %s" % (keyfile,))
+            log.debug("DKIM: Failed private-exchange lookup: no path {path}", path=keyfile)
             return succeed(())
 
         # Now read the data
-        log.debug("DKIM: Private exchange lookup: %s" % (keyfile,))
+        log.debug("DKIM: Private exchange lookup: {path}", path=keyfile)
         try:
             with open(keyfile) as f:
                 keys = f.read()
         except IOError, e:
-            log.debug("DKIM: Failed private-exchange lookup: could not read %s %s" % (keyfile, e,))
+            log.debug("DKIM: Failed private-exchange lookup: could not read {path} {ex}", path=keyfile, ex=e)
             return succeed(())
 
-        log.debug("DKIM: Private exchange lookup results: %s\n%s" % (keyfile, keys))
+        log.debug("DKIM: Private exchange lookup results: {path}\n{keys}", path=keyfile, keys=keys)
         return succeed(tuple([DKIMUtils.extractTags(line) for line in keys.splitlines()]))
 
 
-
 class DomainKeyResource (SimpleResource):
     """
     Domainkey well-known resource.
@@ -972,7 +935,6 @@ class DomainKeyResource (SimpleResource):
         self.domain = domain
         self.selector = selector
 
-
     def makeKeyData(self, domain, selector, pubkeyfile):
         """
         Check that a valid key exists, create the TXT record format data and make the needed child resources.
@@ -983,14 +945,14 @@ class DomainKeyResource (SimpleResource):
             with open(pubkeyfile) as f:
                 key_data = f.read()
         except IOError, e:
-            log.error("DKIM: Unable to open the public key file: %s because of %s" % (pubkeyfile, e,))
+            log.error("DKIM: Unable to open the public key file: {path} because of {ex}", path=pubkeyfile, ex=e)
             raise
 
         # Make sure we can parse a valid public key
         try:
             RSA.importKey(key_data)
         except:
-            log.error("DKIM: Invalid public key file: %s" % (pubkeyfile,))
+            log.error("DKIM: Invalid public key file: {path}", path=pubkeyfile)
             raise
 
         # Make the TXT record
@@ -1004,11 +966,9 @@ class DomainKeyResource (SimpleResource):
         selectorResource = SimpleDataResource(principalCollections=None, content_type=MimeType.fromString("text/plain"), data=txt_data, defaultACL=SimpleResource.allReadACL)
         domainResource.putChild(selector, selectorResource)
 
-
     def contentType(self):
         return MimeType.fromString("text/html; charset=utf-8")
 
-
     def render(self, request):
         output = """<html>
 <head>
