@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -37,6 +37,7 @@ import hashlib
 import os
 import time
 
+
 class TestDKIMBase (unittest.TestCase):
     """
     DKIM support tests
@@ -52,13 +53,12 @@ class TestDKIMBase (unittest.TestCase):
             """
             return succeed(self.keys)
 
-
     def setUp(self):
         super(TestDKIMBase, self).setUp()
 
         self.private_keyfile = self.mktemp()
-        f = open(self.private_keyfile, "w")
-        f.write("""-----BEGIN RSA PRIVATE KEY-----
+        with open(self.private_keyfile, "w") as f:
+            f.write("""-----BEGIN RSA PRIVATE KEY-----
 MIIEogIBAAKCAQEAw7bJxD1k5VSA5AqdfmJ7vj99oKQ4qYtSeJ5HiK6W40dzC++k
 LweUWLzeUErgXwcJlyOC6rqVVPBfSJq4l7yPdVqpWUo6s2jnUsSWOfhpre22yc4B
 K0QY2Euc3R+gT59eM0mtJPtWaQw5BmQ2GrV6f0OUiKi17jEPasKcxf1qZrWU0+Ik
@@ -86,7 +86,6 @@ iblllHCLZ2Q/rHSH3cQor94kxePm+b3KH9ZwAgInMModuSPcScrR5/vsORZCtJEO
 CAXnxZHhrExMGIIa7KV33W5v7Hstl7SnPWKFgCvlBH2QoMTjoUE=
 -----END RSA PRIVATE KEY-----
 """)
-        f.close()
 
         pkey_data = """MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAw7bJxD1k5VSA5AqdfmJ7
 vj99oKQ4qYtSeJ5HiK6W40dzC++kLweUWLzeUErgXwcJlyOC6rqVVPBfSJq4l7yP
@@ -97,15 +96,13 @@ hE/Eso0gS79FSti1fkDeoPZ296Gu5uYWdpaLl03Nr0w65Gbw+2v79AcwOyvbZD6y
 jQIDAQAB
 """
         self.public_keyfile = self.mktemp()
-        f = open(self.public_keyfile, "w")
-        f.write("""-----BEGIN PUBLIC KEY-----
+        with open(self.public_keyfile, "w") as f:
+            f.write("""-----BEGIN PUBLIC KEY-----
 %s-----END PUBLIC KEY-----
 """ % (pkey_data,))
-        f.close()
         self.public_key_data = pkey_data.replace("\n", "")
 
 
-
 class TestDKIMRequest (TestDKIMBase):
     """
     L{DKIMRequest} support tests.
@@ -129,7 +126,6 @@ class TestDKIMRequest (TestDKIMBase):
             result = (yield request.bodyHash())
             self.assertEqual(result, hash)
 
-
     def test_generateSignature(self):
 
         data = "Hello World!"
@@ -155,12 +151,13 @@ dkim-signature:v=1; d=example.com; s=dkim; t=%s; x=%s; a=%s; q=dns/txt:http/well
 
             result = request.generateSignature(sign_this)
 
-            key = RSA.importKey(open(self.private_keyfile).read())
+            with open(self.private_keyfile) as f:
+                key = f.read()
+            key = RSA.importKey(key)
             signature = DKIMUtils.sign(sign_this, key, DKIMUtils.hash_func(algorithm))
 
             self.assertEqual(result, signature)
 
-
     @inlineCallbacks
     def test_signatureHeaders(self):
 
@@ -189,7 +186,6 @@ dkim-signature:v=1; d=example.com; s=dkim; t=%s; x=%s; a=%s; q=private-exchange:
 
             self.assertEqual(result, sign_this)
 
-
     @inlineCallbacks
     def test_sign(self):
 
@@ -214,7 +210,9 @@ content-type:%s
 ischedule-version:1.0
 ischedule-message-id:%s
 dkim-signature:v=1; d=example.com; s=dkim; t=%s; x=%s; a=%s; q=private-exchange:http/well-known:dns/txt; c=ischedule-relaxed/simple; h=Originator:Recipient:Content-Type:iSchedule-Version:iSchedule-Message-ID; bh=%s; b=""".replace("\n", "\r\n") % (headers.getRawHeaders("Content-Type")[0], request.message_id, request.time, request.expire, algorithm, bodyhash)
-            key = RSA.importKey(open(self.private_keyfile).read())
+            with open(self.private_keyfile) as f:
+                key = f.read()
+            key = RSA.importKey(key)
             signature = DKIMUtils.sign(sign_this, key, DKIMUtils.hash_func(algorithm))
 
             self.assertEqual(result, signature)
@@ -224,11 +222,12 @@ dkim-signature:v=1; d=example.com; s=dkim; t=%s; x=%s; a=%s; q=private-exchange:
             self.assertEqual(request.headers.getRawHeaders("DKIM-Signature")[0], updated_header)
 
             # Try to verify result using public key
-            pubkey = RSA.importKey(open(self.public_keyfile).read())
+            with open(self.public_keyfile) as f:
+                pubkey = f.read()
+            pubkey = RSA.importKey(pubkey)
             self.assertEqual(DKIMUtils.verify(sign_this, result, pubkey, DKIMUtils.hash_func(algorithm)), None)
 
 
-
 class TestDKIMVerifier (TestDKIMBase):
     """
     L{DKIMVerifier} support tests.
@@ -244,14 +243,12 @@ class TestDKIMVerifier (TestDKIMBase):
                 self.headers.addRawHeader(name, value)
             self.stream = MemoryStream(body)
 
-
     def _makeHeaders(self, headers_pairs):
         headers = Headers()
         for name, value in headers_pairs:
             headers.addRawHeader(name, value)
         return headers
 
-
     def test_valid_dkim_headers(self):
         """
         L{DKIMVerifier.processDKIMHeader} correctly validates DKIM-Signature headers.
@@ -290,7 +287,6 @@ class TestDKIMVerifier (TestDKIMBase):
             else:
                 self.assertRaises(DKIMVerificationError, verifier.processDKIMHeader)
 
-
     def test_canonicalize_header(self):
         """
         L{DKIMVerifier.canonicalizeHeader} correctly canonicalizes headers.
@@ -320,7 +316,6 @@ class TestDKIMVerifier (TestDKIMBase):
             canonicalized = DKIMUtils.canonicalizeHeader(name, value, verifier.dkim_tags if name == "DKIM-Signature" else None)
             self.assertEqual(canonicalized, result)
 
-
     def test_extract_headers(self):
         """
         L{DKIMVerifier.extractSignedHeaders} correctly extracts canonicalizes headers.
@@ -388,7 +383,6 @@ dkim-signature:v=1; d=example.com; s = dkim; t = 1234; a=rsa-sha1; q=dns/txt:htt
             extracted = verifier.extractSignedHeaders()
             self.assertEqual(extracted, result.replace("\n", "\r\n"))
 
-
     def test_canonicalize_body(self):
         """
         L{DKIMUtils.canonicalizeBody} correctly canonicalizes bodies.
@@ -415,7 +409,6 @@ dkim-signature:v=1; d=example.com; s = dkim; t = 1234; a=rsa-sha1; q=dns/txt:htt
                 result.replace("\n", "\r\n"),
             )
 
-
     @inlineCallbacks
     def test_locate_public_key(self):
         """
@@ -476,7 +469,6 @@ Connection:close
             else:
                 self.assertTrue(pkey is None)
 
-
     @inlineCallbacks
     def test_verify(self):
         """
@@ -648,7 +640,6 @@ END:DATA
         )
 
 
-
 class TestPublicKeyLookup (TestDKIMBase):
     """
     L{PublicKeyLookup} support tests.
@@ -662,7 +653,6 @@ class TestPublicKeyLookup (TestDKIMBase):
         client.theResolver = None
         utils.DebugResolver = None
 
-
     def test_selector_key(self):
 
         for lookup, d, result in (
@@ -677,7 +667,6 @@ class TestPublicKeyLookup (TestDKIMBase):
             tester = lookup(DKIMUtils.extractTags(dkim))
             self.assertEqual(tester._getSelectorKey(), result)
 
-
     @inlineCallbacks
     def test_get_key(self):
 
@@ -765,7 +754,6 @@ class TestPublicKeyLookup (TestDKIMBase):
         pubkey = (yield lookup.getPublicKey())
         self.assertTrue(pubkey is None)
 
-
     @inlineCallbacks
     def test_cached_key(self):
 
@@ -792,7 +780,6 @@ class TestPublicKeyLookup (TestDKIMBase):
         pubkey = (yield lookup.getPublicKey())
         self.assertTrue(pubkey is None)
 
-
     @inlineCallbacks
     def test_TXT_key(self):
 
@@ -816,7 +803,6 @@ class TestPublicKeyLookup (TestDKIMBase):
             pkey = yield tester.getPublicKey(False)
             self.assertEqual(pkey is not None, result)
 
-
     @inlineCallbacks
     def test_HTTP_URI_key(self):
 
@@ -838,7 +824,6 @@ class TestPublicKeyLookup (TestDKIMBase):
             uri = (yield tester._getURI())
             self.assertEqual(uri, result)
 
-
     @inlineCallbacks
     def test_private_exchange(self):
 
