@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ from twisted.trial import unittest
 
 from txdav.caldav.datastore.scheduling.ischedule.remoteservers import IScheduleServersParser
 
+
 class Test_IScheduleServersParser(unittest.TestCase):
     """
     Test L{IScheduleServersParser} implementation.
@@ -27,7 +28,8 @@ class Test_IScheduleServersParser(unittest.TestCase):
     def test_readXML(self):
 
         fp = FilePath(self.mktemp())
-        fp.open("w").write("""<?xml version="1.0" encoding="utf-8"?>
+        with fp.open("w") as f:
+            f.write("""<?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE servers SYSTEM "servertoserver.dtd">
 <servers>
   <server>
