@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -52,7 +52,6 @@ def getIPsFromHost(host):
     return ips
 
 
-
 @inlineCallbacks
 def lookupServerViaSRV(domain, service="_ischedules"):
 
@@ -67,8 +66,8 @@ def lookupServerViaSRV(domain, service="_ischedules"):
         returnValue(None)
 
     if (
-        len(answers) == 1 and answers[0].type == dns.SRV and answers[0].payload
-        and answers[0].payload.target == dns.Name('.')
+        len(answers) == 1 and answers[0].type == dns.SRV and answers[0].payload and
+        answers[0].payload.target == dns.Name('.')
     ):
         # decidedly not available
         log.debug("DNS SRV: disabled: {l}", l=lookup)
@@ -84,7 +83,6 @@ def lookupServerViaSRV(domain, service="_ischedules"):
 
     log.debug("DNS SRV: lookup results: {l}\n{s}", l=lookup, s=servers)
 
-
     def _serverCmp(a, b):
         if a[0] != b[0]:
             return cmp(a[0], b[0])
@@ -116,7 +114,6 @@ def lookupServerViaSRV(domain, service="_ischedules"):
     returnValue((host, port,))
 
 
-
 @inlineCallbacks
 def lookupDataViaTXT(domain, prefix=""):
 
@@ -142,7 +139,6 @@ def lookupDataViaTXT(domain, prefix=""):
     returnValue(results)
 
 
-
 class FakeBindAuthority(BindAuthority):
 
     @inlineCallbacks
@@ -163,7 +159,6 @@ class FakeBindAuthority(BindAuthority):
         )
         returnValue(result)
 
-
     def stripComments(self, lines):
         """
         Work around a bug in the base implementation that causes parsing of TXT RRs with
@@ -176,7 +171,6 @@ class FakeBindAuthority(BindAuthority):
             ]
         ]
 
-
     def parseLines(self, lines):
         """
         Work around a bug in the base implementation that causes parsing of TXT RRs with
@@ -191,7 +185,6 @@ class FakeBindAuthority(BindAuthority):
         BindAuthority.parseLines(self, lines)
 
 
-
 def _initResolver():
     global DebugResolver
     if DebugResolver is None:
