@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ from uuid import UUID
 
 log = Logger()
 
+
 @inlineCallbacks
 def getCalendarObjectForRecord(txn, record, uid):
     """
@@ -41,7 +42,7 @@ def getCalendarObjectForRecord(txn, record, uid):
 
         if len(objectResources) > 1:
             # Delete all but the first one
-            log.debug("Should only have zero or one scheduling object resource with UID '%s' in calendar home: %s" % (uid, calendar_home,))
+            log.debug("Should only have zero or one scheduling object resource with UID '{uid}' in calendar home: {home!r}", uid=uid, home=calendar_home)
             for resource in objectResources[1:]:
                 yield resource._removeInternal(internal_state=ComponentRemoveState.INTERNAL, useTrash=False)
             objectResources = objectResources[:1]
@@ -51,7 +52,6 @@ def getCalendarObjectForRecord(txn, record, uid):
         returnValue(None)
 
 
-
 def normalizeCUAddr(addr):
     """
     Normalize a cuaddr string by lower()ing it if it's a mailto:, or
@@ -72,7 +72,6 @@ def normalizeCUAddr(addr):
         return addr
 
 
-
 def uidFromCalendarUserAddress(address):
     """
     Try to extract a record UID from a calendar user address of the appropriate format.
@@ -107,7 +106,6 @@ def uidFromCalendarUserAddress(address):
     return None
 
 
-
 def extractEmailDomain(mailtoURI):
     try:
         addr = mailtoURI[7:].split("?")[0]
