@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,6 +34,7 @@ from pycalendar.datetime import DateTime
 
 import os
 
+
 class MinimalCalendarObjectReplacement(object):
     """
     Provide the minimal set of attributes and methods from CalDAVFile required
@@ -43,9 +44,9 @@ class MinimalCalendarObjectReplacement(object):
     def __init__(self, filePath):
         self.fp = filePath
 
-
     def iCalendar(self):
-        text = self.fp.open().read()
+        with self.fp.open() as f:
+            text = f.read()
         try:
             component = Component.fromString(text)
             # Fix any bogus data we can
@@ -59,31 +60,35 @@ class MinimalCalendarObjectReplacement(object):
         return component
 
 
-
 class MinimalResourceReplacement(object):
     """
     Provide the minimal set of attributes and methods from CalDAVFile required
     by L{Index}.
     """
 
+    class MinimalTxn(object):
+
+        def postCommit(self, _ignore):
+            pass
+
+        def postAbort(self, _ignore):
+            pass
+
     def __init__(self, filePath):
         self.fp = filePath
-
+        self._txn = MinimalResourceReplacement.MinimalTxn()
 
     def isCalendarCollection(self):
         return True
 
-
     def getChild(self, name):
         # FIXME: this should really return something with a child method
         return MinimalCalendarObjectReplacement(self.fp.child(name))
 
-
     def initSyncToken(self):
         pass
 
 
-
 class SQLIndexTests (twistedcaldav.test.util.TestCase):
     """
     Test abstract SQL DB class
@@ -97,6 +102,8 @@ class SQLIndexTests (twistedcaldav.test.util.TestCase):
         # have all the associated backend machinery to actually get children.
         self.db = Index(MinimalResourceReplacement(self.indexDirPath))
 
+    def tearDown(self):
+        self.db._db_close()
 
     def test_reserve_uid_ok(self):
         uid = "test-test-test"
@@ -111,7 +118,6 @@ class SQLIndexTests (twistedcaldav.test.util.TestCase):
 
         return d
 
-
     def test_reserve_uid_twice(self):
         uid = "test-test-test"
         d = self.db.reserveUID(uid)
@@ -122,13 +128,11 @@ class SQLIndexTests (twistedcaldav.test.util.TestCase):
                                          ReservationError))
         return d
 
-
     def test_unreserve_unreserved(self):
         uid = "test-test-test"
         return self.assertFailure(self.db.unreserveUID(uid),
                                   ReservationError)
 
-
     def test_reserve_uid_timeout(self):
         # WARNING: This test is fundamentally flawed and will fail
         # intermittently because it uses the real clock.
@@ -152,7 +156,6 @@ class SQLIndexTests (twistedcaldav.test.util.TestCase):
 
         return d
 
-
     def test_index(self):
         data = (
             (
@@ -294,9 +297,8 @@ END:VCALENDAR
         for description, name, calendar_txt, reCreate, ok in data:
             calendar = Component.fromString(calendar_txt)
             if ok:
-                f = open(os.path.join(self.indexDirPath.path, name), "w")
-                f.write(calendar_txt)
-                del f
+                with open(os.path.join(self.indexDirPath.path, name), "w") as f:
+                    f.write(calendar_txt)
 
                 self.db.addResource(name, calendar, reCreate=reCreate)
                 self.assertTrue(self.db.resourceExists(name), msg=description)
@@ -318,7 +320,6 @@ END:VCALENDAR
             else:
                 self.assertFalse(self.db.resourceExists(name), msg=description)
 
-
     @inlineCallbacks
     def test_index_timerange(self):
         """
@@ -460,9 +461,8 @@ END:VCALENDAR
         for description, name, calendar_txt, trstart, trend in data:
             calendar = Component.fromString(calendar_txt)
 
-            f = open(os.path.join(self.indexDirPath.path, name), "w")
-            f.write(calendar_txt)
-            del f
+            with open(os.path.join(self.indexDirPath.path, name), "w") as f:
+                f.write(calendar_txt)
 
             self.db.addResource(name, calendar)
             self.assertTrue(self.db.resourceExists(name), msg=description)
@@ -489,7 +489,6 @@ END:VCALENDAR
 
             self.assertEqual(set((name,)), index_results, msg=description)
 
-
     @inlineCallbacks
     def test_index_timespan(self):
         data = (
@@ -646,9 +645,8 @@ END:VCALENDAR
         for description, name, calendar_txt, trstart, trend, organizer, instances in data:
             calendar = Component.fromString(calendar_txt)
 
-            f = open(os.path.join(self.indexDirPath.path, name), "w")
-            f.write(calendar_txt)
-            del f
+            with open(os.path.join(self.indexDirPath.path, name), "w") as f:
+                f.write(calendar_txt)
 
             self.db.addResource(name, calendar)
             self.assertTrue(self.db.resourceExists(name), msg=description)
@@ -676,7 +674,6 @@ END:VCALENDAR
 
             self.assertEqual(set(instances), index_results, msg=description)
 
-
     @inlineCallbacks
     def test_index_timespan_per_user(self):
         data = (
@@ -1053,9 +1050,8 @@ END:VCALENDAR
         for description, name, calendar_txt, trstart, trend, organizer, peruserinstances in data:
             calendar = Component.fromString(calendar_txt)
 
-            f = open(os.path.join(self.indexDirPath.path, name), "w")
-            f.write(calendar_txt)
-            del f
+            with open(os.path.join(self.indexDirPath.path, name), "w") as f:
+                f.write(calendar_txt)
 
             self.db.addResource(name, calendar)
             self.assertTrue(self.db.resourceExists(name), msg=description)
@@ -1086,7 +1082,6 @@ END:VCALENDAR
 
             self.db.deleteResource(name)
 
-
     def test_index_revisions(self):
         data1 = """BEGIN:VCALENDAR
 VERSION:2.0
@@ -1154,15 +1149,15 @@ END:VCALENDAR
             self.assertEquals(self.db.whatchanged(revision), results, "Mismatched results for whatchanged with revision %d" % (revision,))
 
 
-
 class MemcacheTests(SQLIndexTests):
+
     def setUp(self):
         super(MemcacheTests, self).setUp()
         self.memcache = InMemoryMemcacheProtocol()
         self.db.reserver = MemcachedUIDReserver(self.db, self.memcache)
 
-
     def tearDown(self):
+        super(MemcacheTests, self).tearDown()
         for _ignore_k, v in self.memcache._timeouts.iteritems():
             if v.active():
                 v.cancel()
