@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2013-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2013-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,13 +15,24 @@
 ##
 
 
-from twisted.internet.defer import inlineCallbacks
+from calendarserver.push.ipush import PushPriority
+
+from operator import methodcaller
 
 from twext.python.clsprop import classproperty
-from txdav.common.datastore.test.util import populateCalendarsFrom
-from txdav.common.datastore.sql_tables import _BIND_MODE_READ, \
-    _BIND_STATUS_INVITED, _BIND_MODE_DIRECT, _BIND_STATUS_ACCEPTED
+
+from twisted.internet.defer import inlineCallbacks
+
+from twistedcaldav.ical import Component
+
+from txdav.common.datastore.podding.base import FailedCrossPodRequestError
 from txdav.common.datastore.podding.test.util import MultiStoreConduitTest
+from txdav.common.datastore.sql_tables import _BIND_MODE_READ, \
+    _BIND_STATUS_INVITED, _BIND_MODE_DIRECT, _BIND_STATUS_ACCEPTED, \
+    _HOME_STATUS_EXTERNAL, _BIND_MODE_WRITE
+from txdav.common.datastore.test.util import populateCalendarsFrom
+from txdav.common.icommondatastore import ExternalShareFailed
+from txdav.idav import ChangeCategory
 
 
 class BaseSharingTests(MultiStoreConduitTest):
@@ -35,7 +46,6 @@ class BaseSharingTests(MultiStoreConduitTest):
         yield super(BaseSharingTests, self).setUp()
         yield self.populate()
 
-
     @inlineCallbacks
     def populate(self):
         yield populateCalendarsFrom(self.requirements, self.theStoreUnderTest(0))
@@ -54,10 +64,25 @@ DTSTAMP:20051222T210507Z
 SUMMARY:event 1
 END:VEVENT
 END:VCALENDAR
+"""
+
+    cal2 = """BEGIN:VCALENDAR
+VERSION:2.0
+CALSCALE:GREGORIAN
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:uid2
+DTSTART:20131123T140000
+DURATION:PT1H
+CREATED:20060102T190000Z
+DTSTAMP:20051222T210507Z
+SUMMARY:event 2
+END:VEVENT
+END:VCALENDAR
 """
 
     @classproperty(cache=False)
-    def requirements(cls): #@NoSelf
+    def requirements(cls):  # @NoSelf
         return {
             "user01": {
                 "calendar": {
@@ -81,9 +106,15 @@ END:VCALENDAR
         }
 
 
-
 class CalendarSharing(BaseSharingTests):
 
+    @inlineCallbacks
+    def setUp(self):
+        yield super(CalendarSharing, self).setUp()
+        for store in self.theStores:
+            store._poddingFailure = None
+            store._poddingError = None
+
     @inlineCallbacks
     def test_no_shares(self):
         """
@@ -93,8 +124,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
-
+        self.assertFalse(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_invite_sharee(self):
@@ -106,7 +136,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("puser02", _BIND_MODE_READ, "summary")
         invites = yield calendar.sharingInvites()
@@ -121,7 +151,7 @@ class CalendarSharing(BaseSharingTests):
         inviteUID = shareeView.shareUID()
         sharedName = shareeView.name()
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commitTransaction(0)
 
@@ -130,7 +160,7 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(1).notificationsWithUID("puser02")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID, ])
+        self.assertEqual(notifications, [inviteUID + ".xml", ])
         yield self.commitTransaction(1)
 
         # Uninvite
@@ -142,7 +172,7 @@ class CalendarSharing(BaseSharingTests):
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commitTransaction(0)
 
@@ -152,10 +182,9 @@ class CalendarSharing(BaseSharingTests):
         yield self.commitTransaction(1)
 
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
         yield calendar.setShared(False)
-        self.assertFalse(calendar.isShared())
-
+        self.assertFalse(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_share(self):
@@ -167,7 +196,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("puser02", _BIND_MODE_READ, "summary")
         invites = yield calendar.sharingInvites()
@@ -176,7 +205,7 @@ class CalendarSharing(BaseSharingTests):
         inviteUID = shareeView.shareUID()
         sharedName = shareeView.name()
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commitTransaction(0)
 
@@ -199,10 +228,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(0).notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commitTransaction(0)
 
@@ -217,11 +246,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(0).notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_decline_share(self):
@@ -233,7 +261,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("puser02", _BIND_MODE_READ, "summary")
         invites = yield calendar.sharingInvites()
@@ -242,7 +270,7 @@ class CalendarSharing(BaseSharingTests):
         inviteUID = shareeView.shareUID()
         sharedName = shareeView.name()
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commitTransaction(0)
 
@@ -266,10 +294,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(0).notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commitTransaction(0)
 
@@ -284,11 +312,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(0).notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_decline_share(self):
@@ -301,7 +328,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("puser02", _BIND_MODE_READ, "summary")
         invites = yield calendar.sharingInvites()
@@ -310,7 +337,7 @@ class CalendarSharing(BaseSharingTests):
 
         sharedName = shareeView.name()
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commitTransaction(0)
 
@@ -334,10 +361,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(0).notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commitTransaction(0)
 
@@ -352,11 +379,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(0).notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_remove_share(self):
@@ -399,7 +425,7 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(0).notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         yield self.commitTransaction(0)
 
@@ -416,8 +442,7 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.theTransactionUnderTest(0).notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
-
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
     @inlineCallbacks
     def test_accept_remove_accept(self):
@@ -430,7 +455,6 @@ class CalendarSharing(BaseSharingTests):
         self.assertTrue(otherCal is not None)
         yield self.commitTransaction(1)
 
-
     @inlineCallbacks
     def test_accept_remove_accept_newcalendar(self):
         """
@@ -469,7 +493,6 @@ class CalendarSharing(BaseSharingTests):
         self.assertTrue(otherCal is not None)
         yield self.commitTransaction(1)
 
-
     @inlineCallbacks
     def test_inviteProperties(self):
 
@@ -483,7 +506,6 @@ class CalendarSharing(BaseSharingTests):
         shared = yield self.calendarUnderTest(txn=txn2, home="puser02", name=shared_name)
         self.assertFalse(shared.isUsedForFreeBusy())
 
-
     @inlineCallbacks
     def test_direct_sharee(self):
         """
@@ -494,7 +516,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.directShareWithUser("puser02")
         invites = yield calendar.sharingInvites()
@@ -550,7 +572,6 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield home.calendarWithName(shared_name)
         self.assertEquals(calendar.notifierID(), ("CalDAV", "user01/calendar",))
 
-
     @inlineCallbacks
     def test_sharedWithTwo(self):
         shared_name1 = yield self.createShare(shareeGUID="puser02")
@@ -566,6 +587,209 @@ class CalendarSharing(BaseSharingTests):
         self.assertTrue(otherCal is not None)
         yield self.commitTransaction(1)
 
+    @inlineCallbacks
+    def test_invite_sharee_failure(self):
+        """
+        Test invite fails gracefully when the other pod is down.
+        """
+
+        # Force store to generate 500 error
+        self.patch(self.theStores[1], "_poddingFailure", ValueError)
+
+        # Invite
+        calendar = yield self.calendarUnderTest(txn=self.theTransactionUnderTest(0), home="user01", name="calendar")
+        invites = yield calendar.sharingInvites()
+        self.assertEqual(len(invites), 0)
+        self.assertFalse(calendar.isSharedByOwner())
+
+        yield self.assertFailure(calendar.inviteUIDToShare("puser02", _BIND_MODE_READ, "summary"), FailedCrossPodRequestError)
+
+    @inlineCallbacks
+    def test_uninvite_sharee_failure(self):
+        """
+        Test uninvite fails gracefully when the other pod is down.
+        Also test that the sharee bind entry is removed when an invalid share is detected.
+        """
+
+        # Invite
+        sharedName = yield self.createShare("user01", "puser02", "calendar")
+
+        # Has external sharee bind entry
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(0), name="puser02", status=_HOME_STATUS_EXTERNAL
+        )
+        calendar = yield home.anyObjectWithShareUID(sharedName)
+        self.assertTrue(calendar is not None)
+        yield self.commitTransaction(0)
+
+        # Force store to generate 500 error
+        self.patch(self.theStores[1], "_poddingFailure", ValueError)
+
+        yield self.removeShare("user01", "puser02", "calendar")
+
+        # Store working again
+        self.patch(self.theStores[1], "_poddingFailure", None)
+
+        # No external sharee bind entry
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(0), name="puser02", status=_HOME_STATUS_EXTERNAL
+        )
+        calendar = yield home.anyObjectWithShareUID(sharedName)
+        self.assertTrue(calendar is None)
+        yield self.commitTransaction(0)
+
+        # Has external sharer bind entry
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="user01", status=_HOME_STATUS_EXTERNAL
+        )
+        calendar = yield home.anyObjectWithShareUID("calendar")
+        self.assertTrue(calendar is not None)
+        yield self.commitTransaction(1)
+
+        # Has sharee bind entry
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="puser02"
+        )
+        calendar = yield home.anyObjectWithShareUID(sharedName)
+        self.assertTrue(calendar is not None)
+        yield self.commitTransaction(1)
+
+        # Force clean-up of sharee calendar
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="puser02"
+        )
+        calendar = yield home.anyObjectWithShareUID(sharedName)
+        yield self.assertFailure(calendar.syncTokenRevision(), ExternalShareFailed)
+        yield self.commitTransaction(1)
+
+        # External sharer bind entry gone
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="user01", status=_HOME_STATUS_EXTERNAL
+        )
+        calendar = yield home.anyObjectWithShareUID("calendar")
+        self.assertTrue(calendar is None)
+        yield self.commitTransaction(1)
+
+        # Sharee bind entry gone
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="puser02"
+        )
+        calendar = yield home.anyObjectWithShareUID(sharedName)
+        self.assertTrue(calendar is None)
+        yield self.commitTransaction(1)
+
+    @inlineCallbacks
+    def test_reply_sharee_failure(self):
+        """
+        Test sharee reply fails and cleans up when the share is invalid.
+        """
+
+        # Invite
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(0), name="user01", create=True
+        )
+        calendar = yield home.calendarWithName("calendar")
+        yield calendar.inviteUIDToShare(
+            "puser02", _BIND_MODE_WRITE, "shared", shareName="shared-calendar"
+        )
+        yield self.commitTransaction(0)
+
+        # Has external sharee bind entry
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(0), name="puser02", status=_HOME_STATUS_EXTERNAL
+        )
+        calendar = yield home.anyObjectWithShareUID("shared-calendar")
+        self.assertTrue(calendar is not None)
+        yield self.commitTransaction(0)
+
+        # Has external sharer bind entry
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="user01", status=_HOME_STATUS_EXTERNAL
+        )
+        calendar = yield home.anyObjectWithShareUID("calendar")
+        self.assertTrue(calendar is not None)
+        yield self.commitTransaction(1)
+
+        # Has sharee bind entry
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="puser02"
+        )
+        calendar = yield home.anyObjectWithShareUID("shared-calendar")
+        self.assertTrue(calendar is not None)
+        yield self.commitTransaction(1)
+
+        # Force store to generate an error
+        self.patch(self.theStores[0], "_poddingError", ExternalShareFailed)
+
+        # ACK: home2 is None
+        home2 = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="puser02"
+        )
+        yield self.assertFailure(home2.acceptShare("shared-calendar"), ExternalShareFailed)
+        yield self.commitTransaction(1)
+
+        # External sharer bind entry gone
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="user01", status=_HOME_STATUS_EXTERNAL
+        )
+        calendar = yield home.anyObjectWithShareUID("calendar")
+        self.assertTrue(calendar is None)
+        yield self.commitTransaction(1)
+
+        # Sharee bind entry gone
+        home = yield self.homeUnderTest(
+            txn=self.theTransactionUnderTest(1), name="puser02"
+        )
+        calendar = yield home.anyObjectWithShareUID("shared-calendar")
+        self.assertTrue(calendar is None)
+        yield self.commitTransaction(1)
+
+    @inlineCallbacks
+    def test_shared_notifications(self):
+        shared_name_puser02 = yield self.createShare()
+        shared_name_user02 = yield self.createShare(shareeGUID="user02", pod=0)
+
+        map(methodcaller("reset"), self.theNotifiers)
+
+        def _checkNotifications(priority=PushPriority.high):
+            self.assertEqual(set(self.theNotifiers[0].history), set([("/CalDAV/example.com/user01/", priority), ("/CalDAV/example.com/user01/calendar/", priority)]))
+            self.assertEqual(set(self.theNotifiers[1].history), set([("/CalDAV/example.com/user01/", priority), ("/CalDAV/example.com/user01/calendar/", priority)]))
+            map(methodcaller("reset"), self.theNotifiers)
+
+        # Change by owner
+        home = yield self.homeUnderTest(txn=self.theTransactionUnderTest(0), name="user01")
+        self.assertEquals(home.notifierID(), ("CalDAV", "user01",))
+        calendar = yield home.calendarWithName("calendar")
+        yield calendar.createObjectResourceWithName("2.ics", Component.fromString(self.cal2))
+        yield self.commitTransaction(0)
+        _checkNotifications()
+
+        # Change by sharee on other pod
+        txn2 = self.theTransactionUnderTest(1)
+        home = yield self.homeUnderTest(txn=txn2, name="puser02")
+        self.assertEquals(home.notifierID(), ("CalDAV", "puser02",))
+        calendar = yield home.calendarWithName(shared_name_puser02)
+        cobj = yield calendar.calendarObjectWithName("2.ics")
+        yield cobj.remove()
+        yield self.commitTransaction(1)
+        _checkNotifications()
+
+        # Change by sharee on same pod
+        txn2 = self.theTransactionUnderTest(0)
+        home = yield self.homeUnderTest(txn=txn2, name="user02")
+        self.assertEquals(home.notifierID(), ("CalDAV", "user02",))
+        calendar = yield home.calendarWithName(shared_name_user02)
+        yield calendar.createObjectResourceWithName("2_1.ics", Component.fromString(self.cal2))
+        yield self.commitTransaction(0)
+        _checkNotifications()
+
+        # Different priority for owner change
+        home = yield self.homeUnderTest(txn=self.theTransactionUnderTest(0), name="user01")
+        self.assertEquals(home.notifierID(), ("CalDAV", "user01",))
+        calendar = yield home.calendarWithName("calendar")
+        yield calendar.notifyChanged(category=ChangeCategory.attendeeITIPUpdate)
+        yield self.commitTransaction(0)
+        _checkNotifications(priority=PushPriority.medium)
 
 
 class SharingRevisions(BaseSharingTests):
@@ -587,7 +811,6 @@ class SharingRevisions(BaseSharingTests):
         otherCal = yield self.calendarUnderTest(txn=txn2, home="puser02", name=sharedName)
         self.assertNotEqual(otherCal._bindRevision, 0)
 
-
     @inlineCallbacks
     def test_updateShareRevision(self):
         """
@@ -625,7 +848,6 @@ class SharingRevisions(BaseSharingTests):
         otherCal = yield self.calendarUnderTest(txn=txn2, home="puser02", name=sharedName)
         self.assertNotEqual(otherCal._bindRevision, 0)
 
-
     @inlineCallbacks
     def test_sharedRevisions(self):
         """
