@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,6 +36,7 @@ from txdav.xml.base import WebDAVTextElement
 from txdav.xml.element import registerElement, registerElementClass, DisplayName
 import os
 
+
 class BaseSharingTests(CommonCommonTests, TestCase):
     """
     Test store-based calendar sharing.
@@ -47,7 +48,6 @@ class BaseSharingTests(CommonCommonTests, TestCase):
         yield self.buildStoreAndDirectory()
         yield self.populate()
 
-
     @inlineCallbacks
     def populate(self):
         yield populateCalendarsFrom(self.requirements, self.storeUnderTest())
@@ -69,7 +69,7 @@ END:VCALENDAR
 """
 
     @classproperty(cache=False)
-    def requirements(cls): #@NoSelf
+    def requirements(cls):  # @NoSelf
         return {
             "user01": {
                 "calendar": {
@@ -92,14 +92,12 @@ END:VCALENDAR
             },
         }
 
-
     def storeUnderTest(self):
         """
         Create and return a L{CalendarStore} for testing.
         """
         return self._sqlCalendarStore
 
-
     @inlineCallbacks
     def _createShare(self):
         # Invite
@@ -120,7 +118,6 @@ END:VCALENDAR
         returnValue(sharedName)
 
 
-
 class CalendarSharing(BaseSharingTests):
 
     @inlineCallbacks
@@ -132,8 +129,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
-
+        self.assertFalse(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_invite_sharee(self):
@@ -145,7 +141,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("user02", _BIND_MODE_READ, "summary")
         invites = yield calendar.sharingInvites()
@@ -164,9 +160,9 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user02")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID, ])
+        self.assertEqual(notifications, [inviteUID + ".xml", ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -183,15 +179,14 @@ class CalendarSharing(BaseSharingTests):
         notifications = yield notifyHome.listNotificationObjects()
         self.assertEqual(notifications, [])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
         yield calendar.setShared(False)
-        self.assertFalse(calendar.isShared())
-
+        self.assertFalse(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_share(self):
@@ -203,7 +198,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("user02", _BIND_MODE_READ, "summary")
         invites = yield calendar.sharingInvites()
@@ -216,9 +211,9 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user02")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(len(notifications), 1)
+        self.assertEqual(notifications, [inviteUID + ".xml", ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -231,10 +226,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -247,11 +242,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_decline_share(self):
@@ -263,7 +257,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("user02", _BIND_MODE_READ, "summary")
         invites = yield calendar.sharingInvites()
@@ -276,9 +270,9 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user02")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(len(notifications), 1)
+        self.assertEqual(notifications, [inviteUID + ".xml", ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -291,10 +285,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -307,11 +301,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_decline_share(self):
@@ -324,7 +317,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("user02", _BIND_MODE_READ, "summary")
         invites = yield calendar.sharingInvites()
@@ -337,9 +330,9 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user02")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(len(notifications), 1)
+        self.assertEqual(notifications, [inviteUID + ".xml", ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -352,10 +345,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -368,11 +361,10 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_remove_share(self):
@@ -397,7 +389,7 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user02")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(len(notifications), 1)
+        self.assertEqual(notifications, [inviteUID + ".xml", ])
 
         yield self.commit()
 
@@ -410,7 +402,7 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
         yield self.commit()
 
@@ -423,8 +415,7 @@ class CalendarSharing(BaseSharingTests):
 
         notifyHome = yield self.transactionUnderTest().notificationsWithUID("user01")
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(notifications, [inviteUID + "-reply", ])
-
+        self.assertEqual(notifications, [inviteUID + "-reply.xml", ])
 
     @inlineCallbacks
     def test_inviteProperties(self):
@@ -438,7 +429,6 @@ class CalendarSharing(BaseSharingTests):
         shared = yield self.calendarUnderTest(home="user02", name=shared_name)
         self.assertFalse(shared.isUsedForFreeBusy())
 
-
     @inlineCallbacks
     def test_direct_sharee_without_displayname(self):
         """
@@ -450,7 +440,7 @@ class CalendarSharing(BaseSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.directShareWithUser("user02")
         invites = yield calendar.sharingInvites()
@@ -484,7 +474,6 @@ class CalendarSharing(BaseSharingTests):
         notifications = yield notifyHome.listNotificationObjects()
         self.assertEqual(len(notifications), 0)
 
-
     @inlineCallbacks
     def test_direct_sharee_with_displayname(self):
         """
@@ -498,7 +487,7 @@ class CalendarSharing(BaseSharingTests):
         )
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.directShareWithUser("user02")
         invites = yield calendar.sharingInvites()
@@ -532,7 +521,6 @@ class CalendarSharing(BaseSharingTests):
         notifications = yield notifyHome.listNotificationObjects()
         self.assertEqual(len(notifications), 0)
 
-
     @inlineCallbacks
     def test_sharedNotifierID(self):
         shared_name = yield self._createShare()
@@ -549,7 +537,6 @@ class CalendarSharing(BaseSharingTests):
         self.assertEquals(calendar.notifierID(), ("CalDAV", "user01/calendar",))
         yield self.commit()
 
-
     @inlineCallbacks
     def test_perUserSharedProxyCollectionProperties(self):
         """
@@ -636,7 +623,6 @@ class CalendarSharing(BaseSharingTests):
         self.assertEqual(str(calendar.properties()[PropertyName.fromElement(customxml.CalendarColor)]), "#000004")
         yield self.commit()
 
-
     @inlineCallbacks
     def test_sharingBindRecords(self):
 
@@ -651,7 +637,6 @@ class CalendarSharing(BaseSharingTests):
         self.assertEqual(results.keys(), ["user02"])
         self.assertEqual(results["user02"].calendarResourceName, shared_name)
 
-
     @inlineCallbacks
     def test_sharedToBindRecords(self):
 
@@ -672,7 +657,6 @@ class CalendarSharing(BaseSharingTests):
         self.assertEqual(metadataRecord.supportedComponents, None)
 
 
-
 class GroupSharingTests(BaseSharingTests):
     """
     Test store-based group sharing.
@@ -693,7 +677,6 @@ class GroupSharingTests(BaseSharingTests):
 
         self.paths = {}
 
-
     def configure(self):
         super(GroupSharingTests, self).configure()
         config.Sharing.Enabled = True
@@ -701,13 +684,11 @@ class GroupSharingTests(BaseSharingTests):
         config.Sharing.Calendars.Groups.Enabled = True
         config.Sharing.Calendars.Groups.ReconciliationDelaySeconds = 0
 
-
     @inlineCallbacks
     def _check_notifications(self, uid, items):
         notifyHome = yield self.transactionUnderTest().notificationsWithUID(uid, create=True)
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(set(notifications), set(items))
-
+        self.assertEqual(set(notifications), set([item + ".xml" for item in items]))
 
 
 class GroupSharing(GroupSharingTests):
@@ -722,7 +703,6 @@ class GroupSharing(GroupSharingTests):
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
 
-
     @inlineCallbacks
     def test_invite_owner_group(self):
         """
@@ -735,7 +715,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         yield self._check_notifications("user01", [])
         shareeViews = yield calendar.inviteUIDToShare("group01", _BIND_MODE_READ, "summary")
@@ -743,7 +723,7 @@ class GroupSharing(GroupSharingTests):
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -758,10 +738,9 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
         yield calendar.setShared(False)
-        self.assertFalse(calendar.isShared())
-
+        self.assertFalse(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_invite_group(self):
@@ -773,7 +752,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ, "summary")
         self.assertEqual(len(shareeViews), 3)
@@ -790,7 +769,7 @@ class GroupSharing(GroupSharingTests):
             self.assertEqual(invite.summary, "summary")
             yield self._check_notifications(invite.shareeUID, [invite.uid, ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -798,12 +777,12 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 3)
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield calendar.uninviteUIDFromShare("group02")
         uninvites = yield calendar.sharingInvites()
         self.assertEqual(len(uninvites), 0)
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         for invite in invites:
             yield self._check_notifications(invite.shareeUID, [])
@@ -811,10 +790,9 @@ class GroupSharing(GroupSharingTests):
         yield self.commit()
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
         yield calendar.setShared(False)
-        self.assertFalse(calendar.isShared())
-
+        self.assertFalse(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_group(self):
@@ -826,7 +804,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ, "summary")
         self.assertEqual(len(shareeViews), 3)
@@ -843,7 +821,7 @@ class GroupSharing(GroupSharingTests):
             self.assertEqual(invite.summary, "summary")
             yield self._check_notifications(invite.shareeUID, [invite.uid, ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -855,7 +833,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -867,8 +845,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_decline_group(self):
@@ -880,7 +857,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ, "summary")
         self.assertEqual(len(shareeViews), 3)
@@ -897,7 +874,7 @@ class GroupSharing(GroupSharingTests):
             self.assertEqual(invite.summary, "summary")
             yield self._check_notifications(invite.shareeUID, [invite.uid, ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -909,7 +886,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -921,8 +898,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_decline_share(self):
@@ -934,7 +910,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ, "summary")
         self.assertEqual(len(shareeViews), 3)
@@ -951,7 +927,7 @@ class GroupSharing(GroupSharingTests):
             self.assertEqual(invite.summary, "summary")
             yield self._check_notifications(invite.shareeUID, [invite.uid, ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -963,7 +939,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -975,8 +951,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_remove_group(self):
@@ -988,7 +963,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ, "summary")
         self.assertEqual(len(shareeViews), 3)
@@ -1005,7 +980,7 @@ class GroupSharing(GroupSharingTests):
             self.assertEqual(invite.summary, "summary")
             yield self._check_notifications(invite.shareeUID, [invite.uid, ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1017,7 +992,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1029,8 +1004,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_uninvite_group(self):
@@ -1042,7 +1016,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ, "summary")
         self.assertEqual(len(shareeViews), 3)
@@ -1059,7 +1033,7 @@ class GroupSharing(GroupSharingTests):
             self.assertEqual(invite.summary, "summary")
             yield self._check_notifications(invite.shareeUID, [invite.uid, ])
 
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1071,7 +1045,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1090,8 +1064,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
-
+        self.assertTrue(calendar.isSharedByOwner())
 
     @inlineCallbacks
     def test_accept_two_groups(self):
@@ -1103,7 +1076,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewsGroup02 = yield calendar.inviteUIDToShare("group02", _BIND_MODE_WRITE, "summary")
         self.assertEqual(len(shareeViewsGroup02), 3)
@@ -1133,11 +1106,10 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
-
     @inlineCallbacks
     def test_accept_uninvite_two_groups(self):
         """
@@ -1148,7 +1120,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewsGroup02 = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ, "summary")
         self.assertEqual(len(shareeViewsGroup02), 3)
@@ -1178,7 +1150,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1209,7 +1181,6 @@ class GroupSharing(GroupSharingTests):
         for invite in invites:
             self.assertEqual((yield self.calendarUnderTest(home=invite.shareeUID, name=invite.uid)), None)
 
-
     @inlineCallbacks
     def test_accept_uninvite_two_groups_different_access(self):
         """
@@ -1221,7 +1192,7 @@ class GroupSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewsGroup02 = yield calendar.inviteUIDToShare("group02", _BIND_MODE_WRITE, "summary")
         self.assertEqual(len(shareeViewsGroup02), 3)
@@ -1251,7 +1222,7 @@ class GroupSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1283,7 +1254,6 @@ class GroupSharing(GroupSharingTests):
             self.assertEqual((yield self.calendarUnderTest(home=invite.shareeUID, name=invite.uid)), None)
 
 
-
 class MixedSharing(GroupSharingTests):
     """
     Test store-based combined individual and group sharing.
@@ -1300,7 +1270,7 @@ class MixedSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewUser07 = yield calendar.inviteUIDToShare("user07", _BIND_MODE_READ, "summary")
         self.assertNotEqual(shareeViewUser07, None)
@@ -1331,7 +1301,7 @@ class MixedSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1360,7 +1330,6 @@ class MixedSharing(GroupSharingTests):
         for invite in invites:
             self.assertEqual((yield self.calendarUnderTest(home=invite.shareeUID, name=invite.uid)), None)
 
-
     @inlineCallbacks
     def test_accept_uninvite_individual_direct_and_group(self):
         """
@@ -1372,7 +1341,7 @@ class MixedSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewUser07 = yield calendar.directShareWithUser("user07")
         self.assertNotEqual(shareeViewUser07, None)
@@ -1412,7 +1381,7 @@ class MixedSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites if invite.shareeUID != "user07"])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1441,7 +1410,6 @@ class MixedSharing(GroupSharingTests):
         for invite in invites:
             self.assertEqual((yield self.calendarUnderTest(home=invite.shareeUID, name=invite.uid)), None)
 
-
     @inlineCallbacks
     def test_accept_uninvite_group_and_individual(self):
         """
@@ -1452,7 +1420,7 @@ class MixedSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewsGroup02 = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ, "summary")
         self.assertEqual(len(shareeViewsGroup02), 3)
@@ -1484,7 +1452,7 @@ class MixedSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1516,7 +1484,6 @@ class MixedSharing(GroupSharingTests):
         for invite in invites:
             self.assertEqual((yield self.calendarUnderTest(home=invite.shareeUID, name=invite.uid)), None)
 
-
     @inlineCallbacks
     def test_accept_uninvite_group_and_individual_direct(self):
         """
@@ -1527,7 +1494,7 @@ class MixedSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewsGroup02 = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ)
         self.assertEqual(len(shareeViewsGroup02), 3)
@@ -1559,7 +1526,7 @@ class MixedSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites if invite.shareeUID != "user07"])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1592,7 +1559,6 @@ class MixedSharing(GroupSharingTests):
         for invite in invites:
             self.assertEqual((yield self.calendarUnderTest(home=invite.shareeUID, name=invite.uid)), None)
 
-
     @inlineCallbacks
     def test_accept_uninvite_individual_and_groups(self):
         """
@@ -1603,7 +1569,7 @@ class MixedSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewUser07 = yield calendar.inviteUIDToShare("user07", _BIND_MODE_READ, "summary")
         self.assertNotEqual(shareeViewUser07, None)
@@ -1637,7 +1603,7 @@ class MixedSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1683,7 +1649,6 @@ class MixedSharing(GroupSharingTests):
         for invite in invites:
             self.assertEqual((yield self.calendarUnderTest(home=invite.shareeUID, name=invite.uid)), None)
 
-
     @inlineCallbacks
     def test_accept_uninvite_individual_and_groups_different_access(self):
         """
@@ -1696,7 +1661,7 @@ class MixedSharing(GroupSharingTests):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViewUser07 = yield calendar.inviteUIDToShare("user07", _BIND_MODE_WRITE)
         self.assertNotEqual(shareeViewUser07, None)
@@ -1730,7 +1695,7 @@ class MixedSharing(GroupSharingTests):
         yield self._check_notifications("user01", [invite.uid + "-reply" for invite in invites])
 
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
-        self.assertTrue(calendar.isShared())
+        self.assertTrue(calendar.isSharedByOwner())
 
         yield self.commit()
 
@@ -1815,7 +1780,6 @@ class MixedSharing(GroupSharingTests):
             self.assertEqual((yield self.calendarUnderTest(home=invite.shareeUID, name=invite.uid)), None)
 
 
-
 class SharingRevisions(BaseSharingTests):
     """
     Test store-based sharing and interaction with revision table.
@@ -1833,7 +1797,6 @@ class SharingRevisions(BaseSharingTests):
         otherCal = yield self.calendarUnderTest(home="user02", name=sharedName)
         self.assertNotEqual(otherCal._bindRevision, 0)
 
-
     @inlineCallbacks
     def test_updateShareRevision(self):
         """
@@ -1865,7 +1828,6 @@ class SharingRevisions(BaseSharingTests):
         otherCal = yield self.calendarUnderTest(home="user02", name=sharedName)
         self.assertNotEqual(otherCal._bindRevision, 0)
 
-
     @inlineCallbacks
     def test_sharedRevisions(self):
         """
