@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.caldav.datastore.test.test_util -*-
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -57,6 +57,7 @@ log = Logger()
 
 validationBypass = False
 
+
 def validateCalendarComponent(calendarObject, calendar, component, inserting, migrating):
     """
     Validate a calendar component for a particular calendar.
@@ -99,7 +100,6 @@ def validateCalendarComponent(calendarObject, calendar, component, inserting, mi
         raise InvalidObjectResourceError(e)
 
 
-
 @inlineCallbacks
 def normalizationLookup(cuaddr, recordFunction, config):
     """
@@ -111,7 +111,7 @@ def normalizationLookup(cuaddr, recordFunction, config):
     try:
         record = yield recordFunction(cuaddr)
     except Exception, e:
-        log.debug("Lookup of %s failed: %s" % (cuaddr, e))
+        log.debug("Lookup of {cuaddr} failed: {ex}", cuaddr=cuaddr, ex=e)
         record = None
 
     if record is None:
@@ -130,7 +130,6 @@ def normalizationLookup(cuaddr, recordFunction, config):
         returnValue((fullName, record.uid, record.getCUType(), cuas))
 
 
-
 @inlineCallbacks
 def dropboxIDFromCalendarObject(calendarObject):
     """
@@ -177,7 +176,6 @@ def dropboxIDFromCalendarObject(calendarObject):
     returnValue(uid + ".dropbox")
 
 
-
 @inlineCallbacks
 def _migrateCalendar(inCalendar, outCalendar, getComponent, merge=False):
     """
@@ -203,19 +201,22 @@ def _migrateCalendar(inCalendar, outCalendar, getComponent, merge=False):
     for calendarObject in (yield inCalendar.calendarObjects()):
         try:
             ctype = yield calendarObject.componentType()
-        except Exception, e: # Don't stop for any error
-            log.error("  Failed to migrate calendar object: %s/%s/%s (%s)" % (
-                inCalendar.ownerHome().name(),
-                inCalendar.name(),
-                calendarObject.name(),
-                str(e)
-            ))
+        except Exception, e:  # Don't stop for any error
+            log.error(
+                "  Failed to migrate calendar object: {home}/{cal}/{rsrc} (ex)",
+                home=inCalendar.ownerHome().name(),
+                cal=inCalendar.name(),
+                rsrc=calendarObject.name(),
+                ex=str(e),
+            )
             bad_count += 1
             continue
 
         if ctype not in ("VEVENT", "VTODO"):
-            log.error("Migration skipping unsupported (%s) calendar object %r"
-                      % (ctype, calendarObject))
+            log.error(
+                "Migration skipping unsupported ({ctype}) calendar object {obj!r}",
+                ctype=ctype, obj=calendarObject,
+            )
             continue
         if merge:
             mightConflict = yield outHome.hasCalendarResourceUIDSomewhereElse(
@@ -223,8 +224,8 @@ def _migrateCalendar(inCalendar, outCalendar, getComponent, merge=False):
             )
             if mightConflict is not None:
                 log.warn(
-                    "Not migrating object %s/%s/%s due to potential conflict" %
-                    (outHome.uid(), outCalendar.name(), calendarObject.name())
+                    "Not migrating object {home}/{cal}/{rsrc} due to potential conflict",
+                    home=outHome.uid(), cal=outCalendar.name(), rsrc=calendarObject.name(),
                 )
                 continue
         try:
@@ -264,26 +265,27 @@ def _migrateCalendar(inCalendar, outCalendar, getComponent, merge=False):
             ok_count += 1
 
         except InternalDataStoreError:
-            log.error("  InternalDataStoreError: Failed to migrate calendar object: %s/%s/%s" % (
-                inCalendar.ownerHome().name(),
-                inCalendar.name(),
-                calendarObject.name(),
-            ))
+            log.error(
+                "  InternalDataStoreError: Failed to migrate calendar object: {home}/{cal}/{rsrc}",
+                home=inCalendar.ownerHome().name(),
+                cal=inCalendar.name(),
+                rsrc=calendarObject.name(),
+            )
             bad_count += 1
 
         except Exception, e:
-            log.error("  %s: Failed to migrate calendar object: %s/%s/%s" % (
-                str(e),
-                inCalendar.ownerHome().name(),
-                inCalendar.name(),
-                calendarObject.name(),
-            ))
+            log.error(
+                "  {ex}: Failed to migrate calendar object: {home}/{cal}/{rsrc}",
+                ex=str(e),
+                home=inCalendar.ownerHome().name(),
+                cal=inCalendar.name(),
+                rsrc=calendarObject.name(),
+            )
             bad_count += 1
 
     returnValue((ok_count, bad_count,))
 
 
-
 # MIME helpers - mostly copied from txweb2.static
 
 def loadMimeTypes(mimetype_locations=['/etc/mime.types']):
@@ -295,7 +297,7 @@ def loadMimeTypes(mimetype_locations=['/etc/mime.types']):
     """
     import mimetypes
     # Grab Python's built-in mimetypes dictionary.
-    contentTypes = mimetypes.types_map #@UndefinedVariable
+    contentTypes = mimetypes.types_map  # @UndefinedVariable
     # Update Python's semi-erroneous dictionary with a few of the
     # usual suspects.
     contentTypes.update(
@@ -324,28 +326,24 @@ def loadMimeTypes(mimetype_locations=['/etc/mime.types']):
     return contentTypes
 
 
-
 def getType(filename, types, defaultType="application/octet-stream"):
     _ignore_p, ext = os.path.splitext(filename)
     ext = ext.lower()
     return types.get(ext, defaultType)
 
 
-
 class _AttachmentMigrationProto(Protocol, object):
+
     def __init__(self, storeTransport):
         self.storeTransport = storeTransport
         self.done = Deferred()
 
-
     def connectionMade(self):
         self.storeTransport.registerProducer(self.transport, False)
 
-
     def dataReceived(self, data):
         self.storeTransport.write(data)
 
-
     @inlineCallbacks
     def connectionLost(self, reason):
         try:
@@ -356,7 +354,6 @@ class _AttachmentMigrationProto(Protocol, object):
             self.done.callback(None)
 
 
-
 @inlineCallbacks
 def migrateHome(inHome, outHome, getComponent=lambda x: x.component(), merge=False):
     """
@@ -404,7 +401,7 @@ def migrateHome(inHome, outHome, getComponent=lambda x: x.component(), merge=Fal
             yield _migrateCalendar(calendar, outCalendar, getComponent, merge=merge)
         except InternalDataStoreError:
             log.error(
-                "  Failed to migrate calendar: %s/%s" % (inHome.name(), name,)
+                "  Failed to migrate calendar: {home}/{cal}", home=inHome.name(), cal=name,
             )
 
     # No migration for notifications, since they weren't present in earlier
@@ -420,7 +417,6 @@ def migrateHome(inHome, outHome, getComponent=lambda x: x.component(), merge=Fal
         yield outHome.splitCalendars()
 
 
-
 class CalendarObjectBase(object):
     """
     Base logic shared between file- and sql-based L{ICalendarObject}
@@ -464,7 +460,6 @@ class CalendarObjectBase(object):
         returnValue(component)
 
 
-
 class StorageTransportAddress(object):
     """
     Peer / host address for L{IAttachmentStorageTransport} implementations.
@@ -485,7 +480,6 @@ class StorageTransportAddress(object):
         self.attachment = attachment
         self.isHost = isHost
 
-
     def __repr__(self):
         if self.isHost:
             host = " (host)"
@@ -494,7 +488,6 @@ class StorageTransportAddress(object):
         return '<Storing Attachment: %r%s>' % (self.attachment.name(), host)
 
 
-
 class StorageTransportBase(object):
     """
     Base logic shared between file- and sql-based L{IAttachmentStorageTransport}
@@ -521,12 +514,10 @@ class StorageTransportBase(object):
         if self._contentType is None:
             self._contentType = http_headers.MimeType.fromString(getType(self._attachment.name(), self.contentTypes))
 
-
     def resetDetails(self, contentType, dispositionName):
         self._contentType = contentType
         self._dispositionName = dispositionName
 
-
     def write(self, data):
         """
         Children must override this to actually write the data, but should
@@ -538,29 +529,23 @@ class StorageTransportBase(object):
             # (etc) forever.
             self._clock.callLater(0, self._producer.resumeProducing)
 
-
     def registerProducer(self, producer, streaming):
         self._producer = producer
         self._streamingProducer = streaming
 
-
     def getPeer(self):
         return StorageTransportAddress(self._attachment, False)
 
-
     def getHost(self):
         return StorageTransportAddress(self._attachment, True)
 
-
     def writeSequence(self, seq):
         return self.write(''.join(seq))
 
-
     def stopProducing(self):
         return self.loseConnection()
 
 
-
 class AttachmentRetrievalTransport(FileSender, object):
     """
     The transport for a protocol that does L{IAttachment.retrieve}.
@@ -572,43 +557,40 @@ class AttachmentRetrievalTransport(FileSender, object):
         self.filePath = filePath
         self.clock = reactor
 
-
     def start(self, protocol):
         this = self
+
         class Consumer(object):
             implements(IConsumer)
+
             def registerProducer(self, producer, streaming):
                 protocol.makeConnection(producer)
                 this._maybeLoopDelivery()
+
             def write(self, data):
                 protocol.dataReceived(data)
+
             def unregisterProducer(self):
                 this._done(protocol)
         self.beginFileTransfer(self.filePath.open(), Consumer())
 
-
     def _done(self, protocol):
         if self._deliveryLoop:
             self._deliveryLoop.stop()
         protocol.connectionLost(Failure(ConnectionLost()))
 
-
     def write(self, data):
         raise NotImplemented("This is a read-only transport.")
 
-
     def writeSequence(self, datas):
         self.write("".join(datas))
 
-
     def loseConnection(self):
         pass
 
-
     def getPeer(self):
         return self
 
-
     def getHost(self):
         return self
 
@@ -633,7 +615,6 @@ class AttachmentRetrievalTransport(FileSender, object):
             self._deliveryLoop.start(0.01, True)
 
 
-
 def fixOneCalendarObject(component):
     """
     Correct the properties which may contain a user's directory UUID within a
@@ -661,7 +642,6 @@ def fixOneCalendarObject(component):
     return fixes, component
 
 
-
 @inlineCallbacks
 def fixOneCalendarHome(home):
     """
