@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,6 +56,7 @@ log = Logger()
 db_basename = db_prefix + "sqlite"
 schema_version = "2"
 
+
 def wrapInDeferred(f):
     def _(*args, **kwargs):
         return maybeDeferred(f, *args, **kwargs)
@@ -63,7 +64,6 @@ def wrapInDeferred(f):
     return _
 
 
-
 class MemcachedUIDReserver(CachePoolUserMixIn):
     log = Logger()
 
@@ -71,19 +71,17 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
         self.index = index
         self._cachePool = cachePool
 
-
     def _key(self, uid):
         return 'reservation:%s' % (
             hashlib.md5('%s:%s' % (uid,
                                    self.index.resource.fp.path)).hexdigest())
 
-
     def reserveUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log.debug("Reserving UID %r @ %r" % (
-            uid,
-            self.index.resource.fp.path
-        ))
+        self.log.debug(
+            "Reserving UID {uid} @ {path}",
+            uid=uid, path=self.index.resource.fp.path,
+        )
 
         def _handleFalse(result):
             if result is False:
@@ -98,13 +96,12 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
         d.addCallback(_handleFalse)
         return d
 
-
     def unreserveUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log.debug("Unreserving UID %r @ %r" % (
-            uid,
-            self.index.resource.fp.path
-        ))
+        self.log.debug(
+            "Unreserving UID {uid} @ {path}",
+            uid=uid, path=self.index.resource.fp.path,
+        )
 
         def _handleFalse(result):
             if result is False:
@@ -117,13 +114,12 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
         d.addCallback(_handleFalse)
         return d
 
-
     def isReservedUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log.debug("Is reserved UID %r @ %r" % (
-            uid,
-            self.index.resource.fp.path
-        ))
+        self.log.debug(
+            "Is reserved UID {uid} @ {path}",
+            uid=uid, path=self.index.resource.fp.path,
+        )
 
         def _checkValue((flags, value)):
             if value is None:
@@ -136,12 +132,11 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
         return d
 
 
-
 class SQLUIDReserver(object):
+
     def __init__(self, index):
         self.index = index
 
-
     @wrapInDeferred
     def reserveUID(self, uid):
         """
@@ -160,11 +155,10 @@ class SQLUIDReserver(object):
                 % (uid, self.index.resource)
             )
         except sqlite.Error, e:
-            log.error("Unable to reserve UID: %s", (e,))
+            log.error("Unable to reserve UID: {ex}", ex=e)
             self.index._db_rollback()
             raise
 
-
     def unreserveUID(self, uid):
         """
         Unreserve a UID for this index's resource.
@@ -184,7 +178,7 @@ class SQLUIDReserver(object):
                         "delete from RESERVED where UID = :1", uid)
                     self.index._db_commit()
                 except sqlite.Error, e:
-                    log.error("Unable to unreserve UID: %s", (e,))
+                    log.error("Unable to unreserve UID: {ex}", ex=e)
                     self.index._db_rollback()
                     raise
 
@@ -192,7 +186,6 @@ class SQLUIDReserver(object):
         d.addCallback(_cb)
         return d
 
-
     @wrapInDeferred
     def isReservedUID(self, uid):
         """
@@ -212,7 +205,7 @@ class SQLUIDReserver(object):
                     self.index._db_execute("delete from RESERVED where UID = :1", uid)
                     self.index._db_commit()
                 except sqlite.Error, e:
-                    log.error("Unable to unreserve UID: %s", (e,))
+                    log.error("Unable to unreserve UID: {ex}", ex=e)
                     self.index._db_rollback()
                     raise
                 return False
@@ -222,7 +215,6 @@ class SQLUIDReserver(object):
         return False
 
 
-
 def sqladdressbookquery(filter, addressbookid=None):
     """
     Convert the supplied addressbook-query into a partial SQL statement.
@@ -240,7 +232,6 @@ def sqladdressbookquery(filter, addressbookid=None):
         return None
 
 
-
 class AddressBookIndex(AbstractSQLDatabase):
     """
     AddressBook collection index abstract base class that defines the apis for the index.
@@ -265,6 +256,8 @@ class AddressBookIndex(AbstractSQLDatabase):
         else:
             self.reserver = SQLUIDReserver(self)
 
+        self.resource._txn.postCommit(self._db_close)
+        self.resource._txn.postAbort(self._db_close)
 
     def create(self):
         """
@@ -272,7 +265,6 @@ class AddressBookIndex(AbstractSQLDatabase):
         """
         self._db()
 
-
     def recreate(self):
         """
         Delete the database and re-create it
@@ -283,7 +275,6 @@ class AddressBookIndex(AbstractSQLDatabase):
             pass
         self.create()
 
-
     #
     # A dict of sets. The dict keys are address book collection paths,
     # and the sets contains reserved UIDs for each path.
@@ -292,15 +283,12 @@ class AddressBookIndex(AbstractSQLDatabase):
     def reserveUID(self, uid):
         return self.reserver.reserveUID(uid)
 
-
     def unreserveUID(self, uid):
         return self.reserver.unreserveUID(uid)
 
-
     def isReservedUID(self, uid):
         return self.reserver.isReservedUID(uid)
 
-
     def isAllowedUID(self, uid, *names):
         """
         Checks to see whether to allow an operation which would add the
@@ -315,7 +303,6 @@ class AddressBookIndex(AbstractSQLDatabase):
         rname = self.resourceNameForUID(uid)
         return (rname is None or rname in names)
 
-
     def resourceNamesForUID(self, uid):
         """
         Looks up the names of the resources with the given UID.
@@ -333,7 +320,7 @@ class AddressBookIndex(AbstractSQLDatabase):
             name_utf8 = name.encode("utf-8")
             if name is not None and self.resource.getChild(name_utf8) is None:
                 # Clean up
-                log.error("Stale resource record found for child %s with UID %s in %s" % (name, uid, self.resource))
+                log.error("Stale resource record found for child {name} with UID {uid} in {rsrc!r}", name=name, uid=uid, rsrc=self.resource)
                 self._delete_from_db(name, uid, False)
                 self._db_commit()
             else:
@@ -341,7 +328,6 @@ class AddressBookIndex(AbstractSQLDatabase):
 
         return resources
 
-
     def resourceNameForUID(self, uid):
         """
         Looks up the name of the resource with the given UID.
@@ -356,7 +342,6 @@ class AddressBookIndex(AbstractSQLDatabase):
 
         return result
 
-
     def resourceUIDForName(self, name):
         """
         Looks up the UID of the resource with the given name.
@@ -368,7 +353,6 @@ class AddressBookIndex(AbstractSQLDatabase):
 
         return uid
 
-
     def addResource(self, name, vcard, fast=False):
         """
         Adding or updating an existing resource.
@@ -387,7 +371,6 @@ class AddressBookIndex(AbstractSQLDatabase):
         if not fast:
             self._db_commit()
 
-
     def deleteResource(self, name):
         """
         Remove this resource from the index.
@@ -399,7 +382,6 @@ class AddressBookIndex(AbstractSQLDatabase):
             self._delete_from_db(name, uid)
             self._db_commit()
 
-
     def resourceExists(self, name):
         """
         Determines whether the specified resource name exists in the index.
@@ -409,7 +391,6 @@ class AddressBookIndex(AbstractSQLDatabase):
         uid = self._db_value_for_sql("select UID from RESOURCE where NAME = :1", name)
         return uid is not None
 
-
     def resourcesExist(self, names):
         """
         Determines whether the specified resource name exists in the index.
@@ -425,7 +406,6 @@ class AddressBookIndex(AbstractSQLDatabase):
         results = self._db_values_for_sql(statement, *names)
         return results
 
-
     def whatchanged(self, revision):
 
         results = [(name.encode("utf-8"), deleted) for name, deleted in self._db_execute("select NAME, DELETED from REVISIONS where REVISION > :1", revision)]
@@ -446,13 +426,11 @@ class AddressBookIndex(AbstractSQLDatabase):
 
         return (changed, deleted, invalid)
 
-
     def lastRevision(self):
         return self._db_value_for_sql(
             "select REVISION from REVISION_SEQUENCE"
         )
 
-
     def bumpRevision(self, fast=False):
         self._db_execute(
             """
@@ -467,7 +445,6 @@ class AddressBookIndex(AbstractSQLDatabase):
             """,
         )
 
-
     def searchValid(self, filter):
         if isinstance(filter, Filter):
             qualifiers = sqladdressbookquery(filter)
@@ -476,7 +453,6 @@ class AddressBookIndex(AbstractSQLDatabase):
 
         return qualifiers is not None
 
-
     def search(self, filter):
         """
         Finds resources matching the given qualifiers.
@@ -504,11 +480,12 @@ class AddressBookIndex(AbstractSQLDatabase):
             if self.resource.getChild(name.encode("utf-8")):
                 yield row
             else:
-                log.error("vCard resource %s is missing from %s. Removing from index."
-                          % (name, self.resource))
+                log.error(
+                    "vCard resource {name} is missing from {rsrc}. Removing from index.",
+                    name=name, rsrc=self.resource,
+                )
                 self.deleteResource(name, None)
 
-
     def bruteForceSearch(self):
         """
         List the whole index and tests for existence, updating the index
@@ -524,25 +501,24 @@ class AddressBookIndex(AbstractSQLDatabase):
             if self.resource.getChild(name.encode("utf-8")):
                 yield row
             else:
-                log.error("AddressBook resource %s is missing from %s. Removing from index."
-                          % (name, self.resource))
+                log.error(
+                    "AddressBook resource {name} is missing from {rsrc}. Removing from index.",
+                    name=name, rsrc=self.resource,
+                )
                 self.deleteResource(name)
 
-
     def _db_version(self):
         """
         @return: the schema version assigned to this index.
         """
         return schema_version
 
-
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         return "AddressBook"
 
-
     def _db_init_data_tables(self, q):
         """
         Initialise the underlying database tables.
@@ -552,7 +528,6 @@ class AddressBookIndex(AbstractSQLDatabase):
         # Create database where the RESOURCE table has unique UID column.
         self._db_init_data_tables_base(q, True)
 
-
     def _db_init_data_tables_base(self, q, uidunique):
         """
         Initialise the underlying database tables.
@@ -619,7 +594,6 @@ class AddressBookIndex(AbstractSQLDatabase):
             """
         )
 
-
     def _db_recreate(self, do_commit=True):
         """
         Re-create the database tables from existing address book data.
@@ -635,10 +609,13 @@ class AddressBookIndex(AbstractSQLDatabase):
             if name.startswith("."):
                 continue
 
+            child = fp.child(name)
+            if not child.isfile():
+                continue
             try:
-                stream = fp.child(name).open()
+                stream = child.open()
             except (IOError, OSError), e:
-                log.error("Unable to open resource %s: %s" % (name, e))
+                log.error("Unable to open resource {name}: {ex}", name=name, ex=e)
                 continue
 
             try:
@@ -648,9 +625,9 @@ class AddressBookIndex(AbstractSQLDatabase):
                     vcard.validVCardData()
                     vcard.validForCardDAV()
                 except ValueError:
-                    log.error("Non-addressbook resource: %s" % (name,))
+                    log.error("Non-addressbook resource: {name}", name=name)
                 else:
-                    # log.info("Indexing resource: %s" % (name,))
+                    # log.info("Indexing resource: {name}", name=name)
                     self.addResource(name, vcard, True)
             finally:
                 stream.close()
@@ -659,7 +636,6 @@ class AddressBookIndex(AbstractSQLDatabase):
         if do_commit:
             self._db_commit()
 
-
     def _db_can_upgrade(self, old_version):
         """
         Can we do an in-place upgrade
@@ -668,7 +644,6 @@ class AddressBookIndex(AbstractSQLDatabase):
         # v2 is a minor change
         return True
 
-
     def _db_upgrade_data_tables(self, q, old_version):
         """
         Upgrade the data from an older version of the DB.
@@ -711,7 +686,6 @@ class AddressBookIndex(AbstractSQLDatabase):
                 """
             )
 
-
     def _add_to_db(self, name, vcard, cursor=None):
         """
         Records the given address book resource in the index with the given name.
@@ -739,7 +713,6 @@ class AddressBookIndex(AbstractSQLDatabase):
             """, name, self.bumpRevision(fast=True), 'N',
         )
 
-
     def _delete_from_db(self, name, uid, dorevision=True):
         """
         Deletes the specified entry from all dbs.
