@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2015 Apple Inc. All rights reserved.
+# Copyright (c) 2015-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -41,6 +41,7 @@ log = Logger()
 ACCOUNTING_TYPE = "migration"
 ACCOUNTING_LOG = "migration.log"
 
+
 def inTransactionWrapper(operation):
     """
     This wrapper converts an instance method that takes a transaction as its
@@ -77,7 +78,7 @@ def inTransactionWrapper(operation):
             except Exception as ex:
                 f = Failure()
                 yield txn.abort()
-                log.error("{label} failed: {e}".format(label=label, e=str(ex)))
+                log.error("{label} failed: {e}", label=label, e=str(ex))
                 returnValue(f)
             else:
                 yield txn.commit()
@@ -86,7 +87,6 @@ def inTransactionWrapper(operation):
     return _inTxn
 
 
-
 # Cross-pod synchronization of an entire calendar home
 class CrossPodHomeSync(object):
 
@@ -112,17 +112,14 @@ class CrossPodHomeSync(object):
         self.record = None
         self.homeId = None
 
-
     def label(self, detail):
         return "Cross-pod Migration Sync for {}: {}".format(self.diruid, detail)
 
-
     def accounting(self, logstr):
-        emitAccounting(ACCOUNTING_TYPE, self.record, "{} {}\n".format(datetime.datetime.now().isoformat(), logstr), filename=ACCOUNTING_LOG)
+        emitAccounting(ACCOUNTING_TYPE, self.record if self.record is None else self.diruid, "{} {}\n".format(datetime.datetime.now().isoformat(), logstr), filename=ACCOUNTING_LOG)
         if self.uselog is not None:
             self.uselog.write("CrossPodHomeSync: {}\n".format(logstr))
 
-
     @inlineCallbacks
     def migrateHere(self):
         """
@@ -160,7 +157,6 @@ class CrossPodHomeSync(object):
         # Step 8 - say phew! TODO: Actually alert everyone else
         pass
 
-
     @inlineCallbacks
     def sync(self):
         """
@@ -184,7 +180,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("Completed: sync.\n")
 
-
     @inlineCallbacks
     def finalSync(self):
         """
@@ -223,7 +218,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("Completed: finalSync.\n")
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def disableRemoteHome(self, txn):
@@ -251,7 +245,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("Completed: disableRemoteHome.\n")
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def enableLocalHome(self, txn):
@@ -298,7 +291,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("Completed: enableLocalHome.\n")
 
-
     @inlineCallbacks
     def removeRemoteHome(self):
         """
@@ -315,7 +307,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("Completed: removeRemoteHome.\n")
 
-
     @inTransactionWrapper
     def _migratedHome(self, txn):
         """
@@ -323,7 +314,6 @@ class CrossPodHomeSync(object):
         """
         return txn.store().conduit.send_migrated_home(txn, self.diruid)
 
-
     @inlineCallbacks
     def loadRecord(self):
         """
@@ -337,7 +327,6 @@ class CrossPodHomeSync(object):
             if self.record.thisServer():
                 raise ValueError("Cross-pod Migration Sync cannot sync with user already on this server: {}".format(self.diruid))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def prepareCalendarHome(self, txn):
@@ -355,7 +344,6 @@ class CrossPodHomeSync(object):
                     self.accounting("  Created new home collection to migrate into.")
             self.homeId = home.id() if home is not None else None
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def syncCalendarHomeMetaData(self, txn):
@@ -375,7 +363,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("Completed: syncCalendarHomeMetaData.")
 
-
     @inlineCallbacks
     def _remoteHome(self, txn):
         """
@@ -389,7 +376,6 @@ class CrossPodHomeSync(object):
             home._migratingHome = True
         returnValue(home)
 
-
     @inlineCallbacks
     def _remoteNotificationsHome(self, txn):
         """
@@ -401,7 +387,6 @@ class CrossPodHomeSync(object):
             notifications._migratingHome = True
         returnValue(notifications)
 
-
     def _localHome(self, txn):
         """
         Get the home on this pod that will have data migrated to it.
@@ -409,7 +394,6 @@ class CrossPodHomeSync(object):
 
         return txn.calendarHomeWithUID(self.diruid, status=_HOME_STATUS_MIGRATING)
 
-
     @inlineCallbacks
     def syncCalendarList(self):
         """
@@ -435,7 +419,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("Completed: syncCalendarList.")
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def getCalendarSyncList(self, txn):
@@ -461,7 +444,6 @@ class CrossPodHomeSync(object):
 
         returnValue(results)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def getSyncState(self, txn):
@@ -473,7 +455,6 @@ class CrossPodHomeSync(object):
         )
         returnValue(dict([(record.remoteResourceID, record) for record in records]))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def updateSyncState(self, txn, stateRecord, newSyncToken):
@@ -490,7 +471,6 @@ class CrossPodHomeSync(object):
             stateRecord.transaction = txn
             yield stateRecord.update(lastSyncToken=newSyncToken)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def purgeLocal(self, txn, local_sync_state, remote_sync_state):
@@ -512,7 +492,6 @@ class CrossPodHomeSync(object):
             del local_sync_state[localID]
             self.accounting("  Purged calendar local-id={} that no longer exists on the remote pod.".format(localID))
 
-
     @inlineCallbacks
     def syncCalendar(self, remoteID, local_sync_state, remote_sync_state):
         """
@@ -558,7 +537,6 @@ class CrossPodHomeSync(object):
         yield self.updateSyncState(local_record, remote_token)
         self.accounting("Completed: syncCalendar.")
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def newCalendar(self, txn):
@@ -571,7 +549,6 @@ class CrossPodHomeSync(object):
         calendar = yield home.createChildWithName(str(uuid4()))
         returnValue(calendar.id())
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def syncCalendarMetaData(self, txn, migrationRecord):
@@ -594,7 +571,6 @@ class CrossPodHomeSync(object):
         yield local_calendar.copyMetadata(remote_calendar)
         self.accounting("  Copied calendar meta-data for calendar local-id={0.localResourceID}, remote-id={0.remoteResourceID}.".format(migrationRecord))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def findObjectsToSync(self, txn, migrationRecord):
@@ -634,7 +610,6 @@ class CrossPodHomeSync(object):
 
         returnValue((actual_changes, deleted,))
 
-
     @inlineCallbacks
     def purgeDeletedObjectsInBatches(self, migrationRecord, deleted):
         """
@@ -653,7 +628,6 @@ class CrossPodHomeSync(object):
             yield self.purgeBatch(migrationRecord.localResourceID, remaining[:self.BATCH_SIZE])
             del remaining[:self.BATCH_SIZE]
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def purgeBatch(self, txn, localID, purge_names):
@@ -677,7 +651,6 @@ class CrossPodHomeSync(object):
             yield local_object.purge(implicitly=False)
             self.accounting("  Purged calendar object local-id={}.".format(local_object.id()))
 
-
     @inlineCallbacks
     def updateChangedObjectsInBatches(self, migrationRecord, changed):
         """
@@ -700,7 +673,6 @@ class CrossPodHomeSync(object):
             )
             del remaining[:self.BATCH_SIZE]
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def updateBatch(self, txn, localID, remoteID, remaining):
@@ -767,7 +739,6 @@ class CrossPodHomeSync(object):
             yield local_object.purge(implicitly=False)
             self.accounting("  Purged calendar object local-id={}.".format(local_object.id()))
 
-
     @inlineCallbacks
     def syncAttachments(self):
         """
@@ -787,7 +758,6 @@ class CrossPodHomeSync(object):
 
         returnValue((changed_ids, removed_ids,))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def syncAttachmentTable(self, txn):
@@ -845,7 +815,6 @@ class CrossPodHomeSync(object):
 
         returnValue((data_ids, removed,))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def syncAttachmentData(self, txn, local_id):
@@ -867,7 +836,6 @@ class CrossPodHomeSync(object):
             yield remote_home.readAttachmentData(records[0].remoteResourceID, attachment)
             self.accounting("  Read attachment local-id={0.localResourceID}, remote-id={0.remoteResourceID}".format(records[0]))
 
-
     @inlineCallbacks
     def linkAttachments(self):
         """
@@ -893,7 +861,6 @@ class CrossPodHomeSync(object):
 
         returnValue(len_links)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def getAttachmentLinks(self, txn):
@@ -906,7 +873,6 @@ class CrossPodHomeSync(object):
         links = yield remote_home.getAttachmentLinks()
         returnValue(links)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def getAttachmentMappings(self, txn):
@@ -927,7 +893,6 @@ class CrossPodHomeSync(object):
 
         returnValue((attachmentIDMap, objectIDMap,))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def makeAttachmentLinks(self, txn, links, attachmentIDMap, objectIDMap):
@@ -945,7 +910,6 @@ class CrossPodHomeSync(object):
 
             yield link.insert()
 
-
     @inlineCallbacks
     def delegateReconcile(self):
         """
@@ -961,7 +925,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("Completed: delegateReconcile.")
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def individualDelegateReconcile(self, txn):
@@ -975,7 +938,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("  Found {} individual delegates".format(len(remote_records)))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def groupDelegateReconcile(self, txn):
@@ -992,7 +954,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("  Found {} group delegates".format(len(remote_records)))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def externalDelegateReconcile(self, txn):
@@ -1006,7 +967,6 @@ class CrossPodHomeSync(object):
 
         self.accounting("  Found {} external delegates".format(len(remote_records)))
 
-
     @inlineCallbacks
     def groupAttendeeReconcile(self):
         """
@@ -1029,7 +989,6 @@ class CrossPodHomeSync(object):
 
         returnValue(number_of_links)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def groupAttendeeData(self, txn):
@@ -1047,7 +1006,6 @@ class CrossPodHomeSync(object):
 
         returnValue((remote_group_attendees, objectIDMap,))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def groupAttendeeProcess(self, txn, results, objectIDMap):
@@ -1064,7 +1022,6 @@ class CrossPodHomeSync(object):
                 continue
             yield groupAttendee.insert(txn)
 
-
     @inlineCallbacks
     def notificationsReconcile(self):
         """
@@ -1085,7 +1042,6 @@ class CrossPodHomeSync(object):
 
         returnValue(len_records)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def notificationRecords(self, txn):
@@ -1104,7 +1060,6 @@ class CrossPodHomeSync(object):
 
         returnValue(records)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def makeNotifications(self, txn, records):
@@ -1119,7 +1074,6 @@ class CrossPodHomeSync(object):
             notification = yield notifications.writeNotificationObject(record.notificationUID, record.notificationType, record.notificationData)
             self.accounting("  Added notification local-id={}.".format(notification.id()))
 
-
     @inlineCallbacks
     def sharedByCollectionsReconcile(self):
         """
@@ -1168,7 +1122,6 @@ class CrossPodHomeSync(object):
 
         returnValue(len_records)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def sharedByCollectionRecords(self, txn, remote_id, local_id):
@@ -1192,7 +1145,6 @@ class CrossPodHomeSync(object):
 
         returnValue((records, local_records[0].bindUID,))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def makeSharedByCollections(self, txn, records, calendar_id):
@@ -1228,7 +1180,6 @@ class CrossPodHomeSync(object):
                 yield record.insert(txn)
                 self.accounting("    Adding new sharee {}".format(shareeHome.uid()))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def syncGroupSharees(self, txn, remote_id, local_id):
@@ -1246,7 +1197,6 @@ class CrossPodHomeSync(object):
             yield share.insert(txn)
             self.accounting("    Adding group sharee {}".format(local_group.groupUID))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def updatedRemoteSharedByCollections(self, txn, remote_id, bindUID):
@@ -1260,7 +1210,6 @@ class CrossPodHomeSync(object):
         self.accounting("    Updating remote records")
         returnValue(records)
 
-
     @inlineCallbacks
     def sharedToCollectionsReconcile(self):
         """
@@ -1293,7 +1242,6 @@ class CrossPodHomeSync(object):
 
         returnValue(len_records)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def sharedToCollectionRecords(self, txn):
@@ -1308,7 +1256,6 @@ class CrossPodHomeSync(object):
         results = yield home.sharedToBindRecords()
         returnValue(results)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def makeSharedToCollections(self, txn, records):
@@ -1372,7 +1319,6 @@ class CrossPodHomeSync(object):
 
                 yield self.updatedRemoteSharedToCollection(remote_id, txn=txn)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def updatedRemoteSharedToCollection(self, txn, remote_id):
@@ -1386,7 +1332,6 @@ class CrossPodHomeSync(object):
         self.accounting("    Updating remote records")
         returnValue(records)
 
-
     @inlineCallbacks
     def iMIPTokensReconcile(self):
         """
@@ -1407,7 +1352,6 @@ class CrossPodHomeSync(object):
 
         returnValue(len_records)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def iMIPTokenRecords(self, txn):
@@ -1419,7 +1363,6 @@ class CrossPodHomeSync(object):
         records = yield remote_home.iMIPTokens()
         returnValue(records)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def makeiMIPTokens(self, txn, records):
@@ -1430,7 +1373,6 @@ class CrossPodHomeSync(object):
         for record in records:
             yield record.insert(txn)
 
-
     @inlineCallbacks
     def workItemsReconcile(self):
         """
@@ -1451,7 +1393,6 @@ class CrossPodHomeSync(object):
 
         returnValue(len_records)
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def workItemRecords(self, txn):
@@ -1476,7 +1417,6 @@ class CrossPodHomeSync(object):
 
         returnValue((records, mapping,))
 
-
     @inTransactionWrapper
     @inlineCallbacks
     def makeWorkItems(self, txn, records, mapping):
