@@ -1,6 +1,6 @@
 # -*- test-case-name: twext.enterprise.dal.test.test_record -*-
 ##
-# Copyright (c) 2015 Apple Inc. All rights reserved.
+# Copyright (c) 2015-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,6 +32,7 @@ Classes and methods that relate to directory objects in the SQL store. e.g.,
 delegates, groups etc
 """
 
+
 class GroupsRecord(SerializableRecord, fromTable(schema.GROUPS)):
     """
     @DynamicAttrs
@@ -51,6 +52,13 @@ class GroupsRecord(SerializableRecord, fromTable(schema.GROUPS)):
             ),
         )
 
+    @classmethod
+    def groupsMissingSince(cls, txn, since):
+
+        return GroupsRecord.query(
+            txn,
+            (cls.extant == 0).And(cls.modified < since)
+        )
 
 
 class GroupMembershipRecord(SerializableRecord, fromTable(schema.GROUP_MEMBERSHIP)):
@@ -61,7 +69,6 @@ class GroupMembershipRecord(SerializableRecord, fromTable(schema.GROUP_MEMBERSHI
     pass
 
 
-
 class DelegateRecord(SerializableRecord, fromTable(schema.DELEGATES)):
     """
     @DynamicAttrs
@@ -70,7 +77,6 @@ class DelegateRecord(SerializableRecord, fromTable(schema.DELEGATES)):
     pass
 
 
-
 class DelegateGroupsRecord(SerializableRecord, fromTable(schema.DELEGATE_GROUPS)):
     """
     @DynamicAttrs
@@ -93,7 +99,6 @@ class DelegateGroupsRecord(SerializableRecord, fromTable(schema.DELEGATE_GROUPS)
             ),
         )
 
-
     @classmethod
     def delegateGroups(cls, txn, delegator, readWrite):
         """
@@ -112,7 +117,6 @@ class DelegateGroupsRecord(SerializableRecord, fromTable(schema.DELEGATE_GROUPS)
             ),
         )
 
-
     @classmethod
     def indirectDelegators(cls, txn, delegate, readWrite):
         """
@@ -129,7 +133,6 @@ class DelegateGroupsRecord(SerializableRecord, fromTable(schema.DELEGATE_GROUPS)
             ).And(cls.readWrite == (1 if readWrite else 0)),
         )
 
-
     @classmethod
     def indirectDelegates(cls, txn, delegator, readWrite):
         """
@@ -148,7 +151,6 @@ class DelegateGroupsRecord(SerializableRecord, fromTable(schema.DELEGATE_GROUPS)
             ),
         )
 
-
     @classmethod
     @inlineCallbacks
     def delegatorGroups(cls, txn, delegator):
@@ -181,7 +183,6 @@ class DelegateGroupsRecord(SerializableRecord, fromTable(schema.DELEGATE_GROUPS)
         returnValue(results)
 
 
-
 class ExternalDelegateGroupsRecord(SerializableRecord, fromTable(schema.EXTERNAL_DELEGATE_GROUPS)):
     """
     @DynamicAttrs
@@ -190,7 +191,6 @@ class ExternalDelegateGroupsRecord(SerializableRecord, fromTable(schema.EXTERNAL
     pass
 
 
-
 class GroupsAPIMixin(object):
     """
     A mixin for L{CommonStoreTransaction} that covers the groups API.
@@ -217,7 +217,6 @@ class GroupsAPIMixin(object):
         yield self.refreshGroup(group, record)
         returnValue(group)
 
-
     def updateGroup(self, groupUID, name, membershipHash, extant=True):
         """
         @type groupUID: C{unicode}
@@ -235,7 +234,6 @@ class GroupsAPIMixin(object):
                 modified=timestamp,
             )
 
-
     @inlineCallbacks
     def groupByUID(self, groupUID, create=True):
         """
@@ -276,7 +274,6 @@ class GroupsAPIMixin(object):
         else:
             returnValue(None)
 
-
     @inlineCallbacks
     def groupByID(self, groupID):
         """
@@ -296,7 +293,6 @@ class GroupsAPIMixin(object):
             raise NotFoundError
 
 
-
 class GroupCacherAPIMixin(object):
     """
     A mixin for L{CommonStoreTransaction} that covers the group cacher API.
@@ -305,13 +301,11 @@ class GroupCacherAPIMixin(object):
     def addMemberToGroup(self, memberUID, groupID):
         return GroupMembershipRecord.create(self, groupID=groupID, memberUID=memberUID.encode("utf-8"))
 
-
     def removeMemberFromGroup(self, memberUID, groupID):
         return GroupMembershipRecord.deletesimple(
             self, groupID=groupID, memberUID=memberUID.encode("utf-8")
         )
 
-
     @inlineCallbacks
     def groupMemberUIDs(self, groupID):
         """
@@ -329,7 +323,6 @@ class GroupCacherAPIMixin(object):
         members = yield GroupMembershipRecord.query(self, GroupMembershipRecord.groupID == groupID)
         returnValue(set([record.memberUID.decode("utf-8") for record in members]))
 
-
     @inlineCallbacks
     def refreshGroup(self, group, record):
         """
@@ -379,7 +372,6 @@ class GroupCacherAPIMixin(object):
 
         returnValue((membershipChanged, addedUIDs, removedUIDs,))
 
-
     @inlineCallbacks
     def synchronizeMembers(self, groupID, newMemberUIDs):
         """
@@ -407,7 +399,6 @@ class GroupCacherAPIMixin(object):
 
         returnValue((added, removed,))
 
-
     @inlineCallbacks
     def groupChanged(self, groupID, addedUIDs, removedUIDs):
         """
@@ -422,7 +413,6 @@ class GroupCacherAPIMixin(object):
         """
         yield Delegates.groupChanged(self, groupID, addedUIDs, removedUIDs)
 
-
     @inlineCallbacks
     def groupMembers(self, groupID):
         """
@@ -436,7 +426,6 @@ class GroupCacherAPIMixin(object):
                 members.add(record)
         returnValue(members)
 
-
     @inlineCallbacks
     def groupUIDsFor(self, uid):
         """
@@ -451,6 +440,37 @@ class GroupCacherAPIMixin(object):
         groups = yield GroupsRecord.groupsForMember(self, uid)
         returnValue(set([group.groupUID.decode("utf-8") for group in groups]))
 
+    @inlineCallbacks
+    def allGroups(self):
+        """
+        Return the UIDs of all groups.
+
+        @returns: the UIDs of all groups
+        @rtype: a Deferred resulting in a set
+        """
+
+        results = yield GroupsRecord.all(self)
+        groups = set([record.groupUID.decode("utf-8") for record in results])
+
+        returnValue(groups)
+
+    @inlineCallbacks
+    def groupsMissingSince(self, since):
+        """
+        @param since: datetime to compare against
+        @type since: C{datetime.datetime}
+
+        Return the UIDs of all groups marked as non-extant who have not been
+            modified since the given datetime.
+
+        @returns: UIDs
+        @rtype: a Deferred resulting in a set
+        """
+
+        results = yield GroupsRecord.groupsMissingSince(self, since)
+        groups = set([record.groupUID.decode("utf-8") for record in results])
+
+        returnValue(groups)
 
 
 class DelegatesAPIMixin(object):
@@ -486,7 +506,6 @@ class DelegatesAPIMixin(object):
         except AllRetriesFailed:
             pass
 
-
     @inlineCallbacks
     def addDelegateGroup(self, delegator, delegateGroupID, readWrite,
                          isExternal=False):
@@ -517,7 +536,6 @@ class DelegatesAPIMixin(object):
         except AllRetriesFailed:
             pass
 
-
     def removeDelegate(self, delegator, delegate, readWrite):
         """
         Removes a row from the DELEGATES table.  The delegate should not be a
@@ -538,7 +556,6 @@ class DelegatesAPIMixin(object):
             readWrite=(1 if readWrite else 0),
         )
 
-
     def removeDelegates(self, delegator, readWrite):
         """
         Removes all rows for this delegator/readWrite combination from the
@@ -556,7 +573,6 @@ class DelegatesAPIMixin(object):
             readWrite=(1 if readWrite else 0)
         )
 
-
     def removeDelegateGroup(self, delegator, delegateGroupID, readWrite):
         """
         Removes a row from the DELEGATE_GROUPS table.  The delegate should be a
@@ -577,7 +593,6 @@ class DelegatesAPIMixin(object):
             readWrite=(1 if readWrite else 0),
         )
 
-
     def removeDelegateGroups(self, delegator, readWrite):
         """
         Removes all rows for this delegator/readWrite combination from the
@@ -595,7 +610,6 @@ class DelegatesAPIMixin(object):
             readWrite=(1 if readWrite else 0),
         )
 
-
     @inlineCallbacks
     def delegates(self, delegator, readWrite, expanded=False):
         """
@@ -642,7 +656,6 @@ class DelegatesAPIMixin(object):
 
         returnValue(delegates)
 
-
     @inlineCallbacks
     def delegators(self, delegate, readWrite):
         """
@@ -680,7 +693,6 @@ class DelegatesAPIMixin(object):
 
         returnValue(delegators)
 
-
     @inlineCallbacks
     def delegatorsToGroup(self, delegateGroupID, readWrite):
         """
@@ -706,7 +718,6 @@ class DelegatesAPIMixin(object):
         delegators = set([record.delegator.decode("utf-8") for record in results])
         returnValue(delegators)
 
-
     @inlineCallbacks
     def allGroupDelegates(self):
         """
@@ -723,7 +734,6 @@ class DelegatesAPIMixin(object):
 
         returnValue(delegates)
 
-
     @inlineCallbacks
     def externalDelegates(self):
         """
@@ -747,7 +757,6 @@ class DelegatesAPIMixin(object):
 
         returnValue(delegates)
 
-
     @inlineCallbacks
     def assignExternalDelegates(
         self, delegator, readDelegateGroupID, writeDelegateGroupID,
@@ -765,7 +774,6 @@ class DelegatesAPIMixin(object):
         yield DelegateGroupsRecord.deletesimple(
             self,
             delegator=str(delegator),
-            isExternal=1,
         )
 
         # Remove from the external comparison table
@@ -799,14 +807,12 @@ class DelegatesAPIMixin(object):
                 delegator, writeDelegateGroupID, True, isExternal=True
             )
 
-
     def dumpIndividualDelegatesLocal(self, delegator):
         """
         Get the L{DelegateRecord} for all delegates associated with this delegator.
         """
         return DelegateRecord.querysimple(self, delegator=delegator.encode("utf-8"))
 
-
     @inlineCallbacks
     def dumpIndividualDelegatesExternal(self, delegator):
         """
@@ -815,14 +821,12 @@ class DelegatesAPIMixin(object):
         raw_results = yield self.store().conduit.send_dump_individual_delegates(self, delegator)
         returnValue([DelegateRecord.deserialize(row) for row in raw_results])
 
-
     def dumpGroupDelegatesLocal(self, delegator):
         """
         Get the L{DelegateGroupsRecord},L{GroupsRecord} for all group delegates associated with this delegator.
         """
         return DelegateGroupsRecord.delegatorGroups(self, delegator)
 
-
     @inlineCallbacks
     def dumpGroupDelegatesExternal(self, delegator):
         """
@@ -831,14 +835,12 @@ class DelegatesAPIMixin(object):
         raw_results = yield self.store().conduit.send_dump_group_delegates(self, delegator)
         returnValue([(DelegateGroupsRecord.deserialize(row[0]), GroupsRecord.deserialize(row[1]),) for row in raw_results])
 
-
     def dumpExternalDelegatesLocal(self, delegator):
         """
         Get the L{ExternalDelegateGroupsRecord} for all delegates associated with this delegator.
         """
         return ExternalDelegateGroupsRecord.querysimple(self, delegator=delegator.encode("utf-8"))
 
-
     @inlineCallbacks
     def dumpExternalDelegatesExternal(self, delegator):
         """
