@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.common.datastore.test.test_sql_tables -*-
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -49,18 +49,18 @@ DEFAULTVALUE_MAP_ORACLE = {
     "null": None,
 }
 
+
 def dumpSchema(txn, title, schemaname="public"):
     """
     Generate the L{Schema}.
     """
 
-    if txn.dialect == POSTGRES_DIALECT:
+    if txn.dbtype.dialect == POSTGRES_DIALECT:
         return dumpSchema_postgres(txn, title, schemaname)
-    elif txn.dialect == ORACLE_DIALECT:
+    elif txn.dbtype.dialect == ORACLE_DIALECT:
         return dumpSchema_oracle(txn, title, schemaname)
 
 
-
 @inlineCallbacks
 def dumpSchema_postgres(txn, title, schemaname="public"):
     """
@@ -142,7 +142,6 @@ def dumpSchema_postgres(txn, title, schemaname="public"):
     returnValue(schema)
 
 
-
 @inlineCallbacks
 def dumpSchema_oracle(txn, title, schemaname="public"):
     """
@@ -228,4 +227,10 @@ def dumpSchema_oracle(txn, title, schemaname="public"):
             for column in columns:
                 index.addColumn(column)
 
+    # Functions
+    rows = yield txn.execSQL("select object_name from all_procedures where owner = '%s'" % (schemaname.upper(),))
+    for row in rows:
+        name = row[0]
+        Function(schema, name)
+
     returnValue(schema)
