@@ -1,6 +1,6 @@
 # -*- test-case-name: twext.enterprise.dal.test.test_record -*-
 ##
-# Copyright (c) 2015 Apple Inc. All rights reserved.
+# Copyright (c) 2015-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,7 +29,7 @@ from txdav.base.propertystore.sql import PropertyStore
 from txdav.common.datastore.sql_tables import schema, _HOME_STATUS_NORMAL, \
     _HOME_STATUS_EXTERNAL, _HOME_STATUS_DISABLED, _HOME_STATUS_MIGRATING
 from txdav.common.datastore.sql_util import _SharedSyncLogic
-from txdav.common.icommondatastore import RecordNotAllowedError
+from txdav.common.icommondatastore import RecordNotAllowedError, ConcurrentModification
 from txdav.common.idirectoryservice import DirectoryRecordNotFoundError
 from txdav.common.inotifications import INotificationCollection, \
     INotificationObject
@@ -43,6 +43,8 @@ import json
 """
 Classes and methods that relate to the Notification collection in the SQL store.
 """
+
+
 class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
     log = Logger()
 
@@ -58,7 +60,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
 
     _externalClass = None
 
-
     @classmethod
     def makeClass(cls, transaction, homeData):
         """
@@ -78,7 +79,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
             home = cls(transaction, homeData)
         return home.initFromStore()
 
-
     @classmethod
     def homeColumns(cls):
         """
@@ -93,7 +93,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
             cls._homeSchema.STATUS,
         )
 
-
     @classmethod
     def homeAttributes(cls):
         """
@@ -108,7 +107,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
             "_status",
         )
 
-
     def __init__(self, txn, homeData):
 
         self._txn = txn
@@ -126,7 +124,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         # as well as the home it is in
         self._notifiers = dict([(factory_name, factory.newNotifier(self),) for factory_name, factory in txn._notifierFactories.items()])
 
-
     @inlineCallbacks
     def initFromStore(self):
         """
@@ -136,7 +133,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         yield self._loadPropertyStore()
         returnValue(self)
 
-
     @property
     def _home(self):
         """
@@ -145,17 +141,14 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         """
         return self
 
-
     @classmethod
     def notificationsWithUID(cls, txn, uid, status=None, create=False):
         return cls.notificationsWith(txn, None, uid, status=status, create=create)
 
-
     @classmethod
     def notificationsWithResourceID(cls, txn, rid):
         return cls.notificationsWith(txn, rid, None)
 
-
     @classmethod
     @inlineCallbacks
     def notificationsWith(cls, txn, rid, uid, status=None, create=False):
@@ -266,7 +259,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
                     yield homeObject.notifyChanged()
                 returnValue(homeObject)
 
-
     @inlineCallbacks
     def _loadPropertyStore(self):
         self._propertyStore = yield PropertyStore.load(
@@ -278,11 +270,9 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
             notifyCallback=self.notifyChanged
         )
 
-
     def __repr__(self):
         return "<%s: %s>" % (self.__class__.__name__, self._resourceID)
 
-
     def id(self):
         """
         Retrieve the store identifier for this collection.
@@ -292,7 +282,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         """
         return self._resourceID
 
-
     @classproperty
     def _dataVersionQuery(cls):
         nh = cls._homeSchema
@@ -301,7 +290,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
             Where=nh.RESOURCE_ID == Parameter("resourceID")
         )
 
-
     @inlineCallbacks
     def dataVersion(self):
         if self._dataVersion is None:
@@ -309,19 +297,15 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
                 self._txn, resourceID=self._resourceID))[0][0]
         returnValue(self._dataVersion)
 
-
     def name(self):
         return "notification"
 
-
     def uid(self):
         return self._ownerUID
 
-
     def status(self):
         return self._status
 
-
     @inlineCallbacks
     def setStatus(self, newStatus):
         """
@@ -335,7 +319,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
             ).on(self._txn)
             self._status = newStatus
 
-
     def normal(self):
         """
         Is this an normal (internal) home.
@@ -344,7 +327,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         """
         return self._status == _HOME_STATUS_NORMAL
 
-
     def external(self):
         """
         Is this an external home.
@@ -353,23 +335,18 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         """
         return self._status == _HOME_STATUS_EXTERNAL
 
-
     def owned(self):
         return True
 
-
     def ownerHome(self):
         return self._home
 
-
     def viewerHome(self):
         return self._home
 
-
     def notificationObjectRecords(self):
         return NotificationObjectRecord.querysimple(self._txn, notificationHomeResourceID=self.id())
 
-
     @inlineCallbacks
     def notificationObjects(self):
         results = (yield NotificationObject.loadAllObjects(self))
@@ -383,21 +360,22 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         Where=schema.NOTIFICATION.NOTIFICATION_HOME_RESOURCE_ID ==
         Parameter("resourceID"))
 
-
     @inlineCallbacks
     def listNotificationObjects(self):
+        """
+        List the names of all notification objects in this collection. Note that the name
+        is actually the UID value with ".xml" appended, as per L{NotificationObject.name}.
+        """
         if self._notificationNames is None:
             rows = yield self._notificationUIDsForHomeQuery.on(
                 self._txn, resourceID=self._resourceID)
-            self._notificationNames = sorted([row[0] for row in rows])
+            self._notificationNames = sorted([row[0] + ".xml" for row in rows])
         returnValue(self._notificationNames)
 
-
     # used by _SharedSyncLogic.resourceNamesSinceRevision()
     def listObjectResources(self):
         return self.listNotificationObjects()
 
-
     def _nameToUID(self, name):
         """
         Based on the file-backed implementation, the 'name' is just uid +
@@ -405,11 +383,9 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         """
         return name.rsplit(".", 1)[0]
 
-
     def notificationObjectWithName(self, name):
         return self.notificationObjectWithUID(self._nameToUID(name))
 
-
     @memoizedKey("uid", "_notifications")
     @inlineCallbacks
     def notificationObjectWithUID(self, uid):
@@ -421,7 +397,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         no = (yield no.initFromStore())
         returnValue(no)
 
-
     @inlineCallbacks
     def writeNotificationObject(self, uid, notificationtype, notificationdata):
 
@@ -432,26 +407,24 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
             inserting = True
         yield notificationObject.setData(uid, notificationtype, notificationdata, inserting=inserting)
         if inserting:
-            yield self._insertRevision("%s.xml" % (uid,))
+            yield self._insertRevision(notificationObject.name())
             if self._notificationNames is not None:
-                self._notificationNames.append(notificationObject.uid())
+                self._notificationNames.append(notificationObject.name())
         else:
-            yield self._updateRevision("%s.xml" % (uid,))
+            yield self._updateRevision(notificationObject.name())
         yield self.notifyChanged()
         returnValue(notificationObject)
 
-
     def removeNotificationObjectWithName(self, name):
         if self._notificationNames is not None:
-            self._notificationNames.remove(self._nameToUID(name))
+            self._notificationNames.remove(name)
         return self.removeNotificationObjectWithUID(self._nameToUID(name))
 
     _removeByUIDQuery = Delete(
         From=schema.NOTIFICATION,
         Where=(schema.NOTIFICATION.NOTIFICATION_UID == Parameter("uid")).And(
-            schema.NOTIFICATION.NOTIFICATION_HOME_RESOURCE_ID
-            == Parameter("resourceID")))
-
+            schema.NOTIFICATION.NOTIFICATION_HOME_RESOURCE_ID == Parameter("resourceID"))
+    )
 
     @inlineCallbacks
     def removeNotificationObjectWithUID(self, uid):
@@ -463,14 +436,13 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
 
     _initSyncTokenQuery = Insert(
         {
-            _revisionsSchema.HOME_RESOURCE_ID : Parameter("resourceID"),
-            _revisionsSchema.RESOURCE_NAME    : None,
-            _revisionsSchema.REVISION         : schema.REVISION_SEQ,
-            _revisionsSchema.DELETED          : False
+            _revisionsSchema.HOME_RESOURCE_ID: Parameter("resourceID"),
+            _revisionsSchema.RESOURCE_NAME: None,
+            _revisionsSchema.REVISION: schema.REVISION_SEQ,
+            _revisionsSchema.DELETED: False
         }, Return=_revisionsSchema.REVISION
     )
 
-
     @inlineCallbacks
     def _initSyncToken(self):
         self._syncTokenRevision = (yield self._initSyncTokenQuery.on(
@@ -481,14 +453,12 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
         Where=_revisionsSchema.HOME_RESOURCE_ID == Parameter("resourceID")
     )
 
-
     @inlineCallbacks
     def syncToken(self):
         if self._syncTokenRevision is None:
             self._syncTokenRevision = yield self.syncTokenRevision()
         returnValue("%s_%s" % (self._resourceID, self._syncTokenRevision))
 
-
     @inlineCallbacks
     def syncTokenRevision(self):
         revision = (yield self._syncTokenQuery.on(self._txn, resourceID=self._resourceID))[0][0]
@@ -496,29 +466,21 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
             revision = int((yield self._txn.calendarserverValue("MIN-VALID-REVISION")))
         returnValue(revision)
 
-
     def properties(self):
         return self._propertyStore
 
-
     def addNotifier(self, factory_name, notifier):
-        if self._notifiers is None:
-            self._notifiers = {}
         self._notifiers[factory_name] = notifier
 
-
     def getNotifier(self, factory_name):
         return self._notifiers.get(factory_name)
 
-
     def notifierID(self):
         return (self._txn._homeClass[self._txn._primaryHomeType]._notifierPrefix, "%s/notification" % (self.ownerHome().uid(),),)
 
-
     def parentNotifierID(self):
         return (self._txn._homeClass[self._txn._primaryHomeType]._notifierPrefix, "%s" % (self.ownerHome().uid(),),)
 
-
     @inlineCallbacks
     def notifyChanged(self, category=ChangeCategory.default):
         """
@@ -543,7 +505,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
 
         returnValue(None)
 
-
     @classproperty
     def _completelyNewRevisionQuery(cls):
         rev = cls._revisionsSchema
@@ -554,14 +515,12 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
                        rev.DELETED: False},
                       Return=rev.REVISION)
 
-
     def _maybeNotify(self):
         """
         Emit a push notification after C{_changeRevision}.
         """
         return self.notifyChanged()
 
-
     @inlineCallbacks
     def remove(self):
         """
@@ -589,7 +548,6 @@ class NotificationCollection(FancyEqMixin, _SharedSyncLogic):
     purge = remove
 
 
-
 class NotificationObjectRecord(SerializableRecord, fromTable(schema.NOTIFICATION)):
     """
     @DynamicAttrs
@@ -598,7 +556,6 @@ class NotificationObjectRecord(SerializableRecord, fromTable(schema.NOTIFICATION
     pass
 
 
-
 class NotificationObject(FancyEqMixin, object):
     """
     This used to store XML data and an XML element for the type. But we are now switching it
@@ -627,11 +584,9 @@ class NotificationObject(FancyEqMixin, object):
         self._notificationType = None
         self._notificationData = None
 
-
     def __repr__(self):
         return "<%s: %s>" % (self.__class__.__name__, self._resourceID)
 
-
     @classproperty
     def _allColumnsByHomeIDQuery(cls):
         """
@@ -645,7 +600,6 @@ class NotificationObject(FancyEqMixin, object):
             Where=(obj.NOTIFICATION_HOME_RESOURCE_ID == Parameter("homeID"))
         )
 
-
     @classmethod
     @inlineCallbacks
     def loadAllObjects(cls, parent):
@@ -700,7 +654,6 @@ class NotificationObject(FancyEqMixin, object):
 
         returnValue(results)
 
-
     @classproperty
     def _oneNotificationQuery(cls):
         no = cls._objectSchema
@@ -718,7 +671,6 @@ class NotificationObject(FancyEqMixin, object):
                    Parameter("uid")).And(no.NOTIFICATION_HOME_RESOURCE_ID ==
                                          Parameter("homeID")))
 
-
     @inlineCallbacks
     def initFromStore(self):
         """
@@ -750,17 +702,14 @@ class NotificationObject(FancyEqMixin, object):
         else:
             returnValue(None)
 
-
     def _loadPropertyStore(self, props=None, created=False):
         if props is None:
             props = NonePropertyStore(self._home.uid())
         self._propertyStore = props
 
-
     def properties(self):
         return self._propertyStore
 
-
     def id(self):
         """
         Retrieve the store identifier for this object.
@@ -770,24 +719,19 @@ class NotificationObject(FancyEqMixin, object):
         """
         return self._resourceID
 
-
     @property
     def _txn(self):
         return self._home._txn
 
-
     def notificationCollection(self):
         return self._home
 
-
     def uid(self):
         return self._uid
 
-
     def name(self):
         return self.uid() + ".xml"
 
-
     @classproperty
     def _newNotificationQuery(cls):
         no = cls._objectSchema
@@ -802,7 +746,6 @@ class NotificationObject(FancyEqMixin, object):
             Return=[no.RESOURCE_ID, no.CREATED, no.MODIFIED]
         )
 
-
     @classproperty
     def _updateNotificationQuery(cls):
         no = cls._objectSchema
@@ -817,7 +760,6 @@ class NotificationObject(FancyEqMixin, object):
             Return=no.MODIFIED
         )
 
-
     @inlineCallbacks
     def setData(self, uid, notificationtype, notificationdata, inserting=False):
         """
@@ -853,11 +795,14 @@ class NotificationObject(FancyEqMixin, object):
         [_objectSchema.NOTIFICATION_DATA], From=_objectSchema,
         Where=_objectSchema.RESOURCE_ID == Parameter("resourceID"))
 
-
     @inlineCallbacks
     def notificationData(self):
         if self._notificationData is None:
-            self._notificationData = (yield self._notificationDataFromID.on(self._txn, resourceID=self._resourceID))[0][0]
+            self._notificationData = (yield self._notificationDataFromID.on(self._txn, resourceID=self._resourceID))
+            if self._notificationData:
+                self._notificationData = self._notificationData[0][0]
+            else:
+                raise ConcurrentModification()
             try:
                 self._notificationData = json.loads(self._notificationData)
             except ValueError:
@@ -866,29 +811,23 @@ class NotificationObject(FancyEqMixin, object):
                 self._notificationData = self._notificationData.encode("utf-8")
         returnValue(self._notificationData)
 
-
     def contentType(self):
         """
         The content type of NotificationObjects is text/xml.
         """
         return MimeType.fromString("text/xml")
 
-
     def md5(self):
         return self._md5
 
-
     def size(self):
         return self._size
 
-
     def notificationType(self):
         return self._notificationType
 
-
     def created(self):
         return datetimeMktime(self._created)
 
-
     def modified(self):
         return datetimeMktime(self._modified)
