@@ -1,6 +1,6 @@
 # -*- test-case-name: twext.enterprise.dal.test.test_record -*-
 ##
-# Copyright (c) 2015 Apple Inc. All rights reserved.
+# Copyright (c) 2015-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,6 +32,7 @@ from txdav.common.datastore.sql_tables import _BIND_MODE_OWN, _BIND_MODE_DIRECT,
     _HOME_STATUS_EXTERNAL, _HOME_STATUS_DISABLED
 from txdav.common.icommondatastore import ExternalShareFailed, \
     HomeChildNameAlreadyExistsError, AllRetriesFailed
+from txdav.idav import ChangeCategory
 from txdav.xml import element
 
 from uuid import uuid4
@@ -43,6 +44,7 @@ log = Logger()
 Classes and methods that relate to sharing in the SQL store.
 """
 
+
 class SharingHomeMixIn(object):
     """
     Common class for CommonHome to implement sharing operations
@@ -60,7 +62,6 @@ class SharingHomeMixIn(object):
 
         returnValue(shareeView)
 
-
     @inlineCallbacks
     def declineShare(self, shareUID):
         """
@@ -73,7 +74,6 @@ class SharingHomeMixIn(object):
 
         returnValue(shareeView is not None)
 
-
     #
     # External (cross-pod) sharing - entry point is the sharee's home collection.
     #
@@ -110,7 +110,6 @@ class SharingHomeMixIn(object):
 
         shareeView.setInviteCopyProperties(copy_invite_properties)
 
-
     @inlineCallbacks
     def processExternalUninvite(self, ownerUID, bindUID, shareUID):
         """
@@ -132,10 +131,7 @@ class SharingHomeMixIn(object):
 
         # See if there are any references to the external share. If not,
         # remove it
-        invites = yield ownerView.sharingInvites()
-        if len(invites) == 0:
-            yield ownerHome.removeExternalChild(ownerView)
-
+        yield ownerView.cleanExternalShare()
 
     @inlineCallbacks
     def processExternalReply(
@@ -168,7 +164,6 @@ class SharingHomeMixIn(object):
             else:
                 yield shareeHome.declineShare(shareUID)
 
-
     @inlineCallbacks
     def createCollectionForExternalShare(self, name, bindUID, supported_components):
         """
@@ -218,7 +213,6 @@ class SharingHomeMixIn(object):
 
         returnValue(ownerView)
 
-
     @inlineCallbacks
     def sharedToBindRecords(self):
         """
@@ -273,14 +267,12 @@ class SharingHomeMixIn(object):
         returnValue(dict([(homeMap[calendarID], (records[calendarID], ownerRecords[calendarID], metadataRecords[calendarID],),) for calendarID in records]))
 
 
-
 SharingInvitation = namedtuple(
     "SharingInvitation",
-    ["uid", "ownerUID", "ownerHomeID", "shareeUID", "shareeHomeID", "mode", "status", "summary"]
+    ["uid", "ownerUID", "ownerHomeID", "shareeUID", "shareeHomeID", "shareeHomeStatus", "mode", "status", "summary"]
 )
 
 
-
 class SharingMixIn(object):
     """
     Common class for CommonHomeChild and AddressBookObject
@@ -303,7 +295,6 @@ class SharingMixIn(object):
             bind.MESSAGE: Parameter("message"),
         })
 
-
     @classmethod
     def _updateBindColumnsQuery(cls, columnMap):
         bind = cls._bindSchema
@@ -313,7 +304,6 @@ class SharingMixIn(object):
                 bind.HOME_RESOURCE_ID == Parameter("homeID")),
         )
 
-
     @classproperty
     def _deleteBindForResourceIDAndHomeID(cls):
         bind = cls._bindSchema
@@ -323,7 +313,6 @@ class SharingMixIn(object):
                 bind.HOME_RESOURCE_ID == Parameter("homeID")),
         )
 
-
     @classmethod
     def _bindFor(cls, condition):
         bind = cls._bindSchema
@@ -334,7 +323,6 @@ class SharingMixIn(object):
             Where=condition
         )
 
-
     @classmethod
     def _bindInviteFor(cls, condition):
         home = cls._homeSchema
@@ -343,6 +331,7 @@ class SharingMixIn(object):
             [
                 home.OWNER_UID,
                 bind.HOME_RESOURCE_ID,
+                home.STATUS,
                 bind.RESOURCE_ID,
                 bind.RESOURCE_NAME,
                 bind.BIND_MODE,
@@ -353,7 +342,6 @@ class SharingMixIn(object):
             Where=condition
         )
 
-
     @classproperty
     def _sharedInvitationBindForResourceID(cls):
         bind = cls._bindSchema
@@ -362,14 +350,12 @@ class SharingMixIn(object):
             (bind.BIND_MODE != _BIND_MODE_OWN)
         )
 
-
     @classproperty
     def _acceptedBindForHomeID(cls):
         bind = cls._bindSchema
         return cls._bindFor((bind.HOME_RESOURCE_ID == Parameter("homeID"))
                             .And(bind.BIND_STATUS == _BIND_STATUS_ACCEPTED))
 
-
     @classproperty
     def _bindForResourceIDAndHomeID(cls):
         """
@@ -380,7 +366,6 @@ class SharingMixIn(object):
         return cls._bindFor((bind.RESOURCE_ID == Parameter("resourceID"))
                             .And(bind.HOME_RESOURCE_ID == Parameter("homeID")))
 
-
     @classproperty
     def _bindForBindUIDAndHomeID(cls):
         """
@@ -391,7 +376,6 @@ class SharingMixIn(object):
         return cls._bindFor((bind.BIND_UID == Parameter("bindUID"))
                             .And(bind.HOME_RESOURCE_ID == Parameter("homeID")))
 
-
     @classproperty
     def _bindForNameAndHomeID(cls):
         """
@@ -402,7 +386,6 @@ class SharingMixIn(object):
         return cls._bindFor((bind.RESOURCE_NAME == Parameter("name"))
                             .And(bind.HOME_RESOURCE_ID == Parameter("homeID")))
 
-
     #
     # Higher level API
     #
@@ -436,7 +419,6 @@ class SharingMixIn(object):
             yield self._sendInviteNotification(shareeView)
         returnValue(shareeView)
 
-
     @inlineCallbacks
     def directShareWithUser(self, shareeUID, shareName=None, displayName=None):
         """
@@ -462,7 +444,6 @@ class SharingMixIn(object):
 
         returnValue(shareeView)
 
-
     @inlineCallbacks
     def uninviteUIDFromShare(self, shareeUID):
         """
@@ -476,7 +457,14 @@ class SharingMixIn(object):
         shareeView = yield self.shareeView(shareeUID)
         if shareeView is not None:
             if shareeView.viewerHome().external():
-                yield self._sendExternalUninvite(shareeView)
+                try:
+                    yield self._sendExternalUninvite(shareeView)
+                except Exception as e:
+                    # If the cross-pod request fails for some reason, ignore the exception and go ahead
+                    # and remove the share on this pod. It is up to the other pod to "heal" itself
+                    # by  detecting an invalid share when it is running properly again.
+                    log.error("Could not send sharing uninvite '{userid}': {ex}", userid=shareeUID, ex=e)
+
             else:
                 # If current user state is accepted then we send an invite with the new state, otherwise
                 # we cancel any existing invites for the user. Also, if the ownerHome is disabled, we assume
@@ -490,7 +478,6 @@ class SharingMixIn(object):
             # Remove the bind
             yield self.removeShare(shareeView)
 
-
     @inlineCallbacks
     def acceptShare(self, summary=None):
         """
@@ -506,7 +493,6 @@ class SharingMixIn(object):
             if not ownerView.external():
                 yield self._sendReplyNotification(ownerView, summary)
 
-
     @inlineCallbacks
     def declineShare(self):
         """
@@ -521,7 +507,6 @@ class SharingMixIn(object):
             if not ownerView.external():
                 yield self._sendReplyNotification(ownerView)
 
-
     @inlineCallbacks
     def deleteShare(self):
         """
@@ -536,7 +521,6 @@ class SharingMixIn(object):
         else:
             yield self.declineShare()
 
-
     @inlineCallbacks
     def ownerDeleteShare(self):
         """
@@ -550,6 +534,18 @@ class SharingMixIn(object):
         for invitation in (yield self.sharingInvites()):
             yield self.uninviteUIDFromShare(invitation.shareeUID)
 
+    @inlineCallbacks
+    def cleanExternalShare(self):
+        """
+        Called when an external share is removed. This method will remove the reference to
+        the external shared calendar when there are no more sharees on this pod.
+        """
+
+        # See if there are any references to the external share. If not,
+        # remove it
+        invites = yield self.sharingInvites()
+        if len(invites) == 0:
+            yield self._home.removeExternalChild(self)
 
     def newShare(self, displayname=None):
         """
@@ -558,6 +554,37 @@ class SharingMixIn(object):
         """
         return succeed(None)
 
+    @inlineCallbacks
+    def notifyExternalShare(self, category=ChangeCategory.default):
+        """
+        Send a cross-pod message to all external shares to ensure change notifications are sent
+        to external sharees' clients subscribed to notifications on the other pods.
+        """
+
+        # We need to resolve the ownerView which may be this object, or if this is
+        # a shared object, may be something else. The ownerView is the one that
+        # knows about all the sharing invites and the bindUID (which is the identifier
+        # used to identify it on another pod).
+        if not self.owned():
+            ownerView = yield self.ownerView()
+        else:
+            ownerView = self
+
+        # Get all invites and figure out if any are external
+        externalUIDs = set()
+        for invitation in (yield ownerView.sharingInvites()):
+            if invitation.shareeHomeStatus == _HOME_STATUS_EXTERNAL:
+                externalUIDs.add(invitation.shareeUID)
+
+        if externalUIDs and ownerView.bindUID() is not None:
+            yield self._txn.store().conduit.send_sharenotification(
+                self._txn,
+                self.ownerHome()._homeType,
+                self.ownerHome().uid(),
+                ownerView.bindUID(),
+                externalUIDs,
+                category,
+            )
 
     @inlineCallbacks
     def allInvitations(self):
@@ -571,7 +598,6 @@ class SharingMixIn(object):
         invitations.sort(key=lambda invitation: invitation.shareeUID)
         returnValue(invitations)
 
-
     @inlineCallbacks
     def _sendInviteNotification(self, shareeView, notificationState=None):
         """
@@ -605,7 +631,6 @@ class SharingMixIn(object):
         notifications = yield self._txn.notificationsWithUID(shareeView.viewerHome().uid(), create=True)
         yield notifications.writeNotificationObject(shareeView.shareUID(), notificationtype, notificationdata)
 
-
     @inlineCallbacks
     def _sendReplyNotification(self, ownerView, summary=None):
         """
@@ -636,7 +661,6 @@ class SharingMixIn(object):
         notifications = yield self._txn.notificationsWithUID(self.ownerHome().uid(), create=True)
         yield notifications.writeNotificationObject(notificationUID, notificationtype, notificationdata)
 
-
     @inlineCallbacks
     def _removeInviteNotification(self, shareeView):
         """
@@ -647,7 +671,6 @@ class SharingMixIn(object):
         notifications = yield self._txn.notificationsWithUID(shareeView.viewerHome().uid())
         yield notifications.removeNotificationObjectWithUID(shareeView.shareUID())
 
-
     #
     # External/cross-pod API
     #
@@ -677,7 +700,6 @@ class SharingMixIn(object):
             supported_components=self.getSupportedComponents() if hasattr(self, "getSupportedComponents") else None,
         )
 
-
     @inlineCallbacks
     def _sendExternalUninvite(self, shareeView):
 
@@ -690,20 +712,24 @@ class SharingMixIn(object):
             shareeView.shareUID(),
         )
 
-
     @inlineCallbacks
     def _replyExternalInvite(self, status, summary=None):
 
-        yield self._txn.store().conduit.send_sharereply(
-            self._txn,
-            self.viewerHome()._homeType,
-            self.ownerHome().uid(),
-            self.viewerHome().uid(),
-            self.shareUID(),
-            status,
-            summary,
-        )
-
+        # If a reply to an external share fails, then assume the external share
+        # is broken and remove it from the local pod
+        try:
+            yield self._txn.store().conduit.send_sharereply(
+                self._txn,
+                self.viewerHome()._homeType,
+                self.ownerHome().uid(),
+                self.viewerHome().uid(),
+                self.shareUID(),
+                status,
+                summary,
+            )
+        except ExternalShareFailed:
+            yield self.fixNonExistentExternalShare()
+            raise ExternalShareFailed("External share does not exist")
 
     #
     # Lower level API
@@ -720,7 +746,6 @@ class SharingMixIn(object):
         ownerView = yield self.ownerHome().childWithID(self.id())
         returnValue(ownerView)
 
-
     @inlineCallbacks
     def shareeView(self, shareeUID):
         """
@@ -739,7 +764,6 @@ class SharingMixIn(object):
         shareeView = (yield shareeHome.allChildWithID(self.id())) if shareeHome is not None else None
         returnValue(shareeView)
 
-
     @inlineCallbacks
     def shareWithUID(self, shareeUID, mode, status=None, summary=None, shareName=None):
         """
@@ -768,7 +792,6 @@ class SharingMixIn(object):
             (yield self.shareWith(shareeHome, mode, status, summary, shareName))
         )
 
-
     @inlineCallbacks
     def shareWith(self, shareeHome, mode, status=None, summary=None, shareName=None):
         """
@@ -838,7 +861,6 @@ class SharingMixIn(object):
 
         returnValue(bindName)
 
-
     @inlineCallbacks
     def createShare(self, shareeUID, mode, summary=None, shareName=None):
         """
@@ -857,7 +879,6 @@ class SharingMixIn(object):
         shareeView = yield self.shareeView(shareeUID)
         returnValue(shareeView)
 
-
     @inlineCallbacks
     def updateShare(self, shareeView, mode=None, status=None, summary=None):
         """
@@ -921,11 +942,9 @@ class SharingMixIn(object):
             yield self.notifyPropertyChanged()
             yield shareeView.viewerHome().notifyChanged()
 
-
     def _previousAcceptCount(self):
         return succeed(1)
 
-
     @inlineCallbacks
     def _changedStatus(self, previouslyAcceptedCount):
         key = self._home._childrenKey(self.isInTrash())
@@ -939,7 +958,6 @@ class SharingMixIn(object):
             self._home._children[key].pop(self.name(), None)
             self._home._children[key].pop(self.id(), None)
 
-
     @inlineCallbacks
     def removeShare(self, shareeView):
         """
@@ -973,7 +991,6 @@ class SharingMixIn(object):
 
         yield shareeView.invalidateQueryCache()
 
-
     @inlineCallbacks
     def unshare(self):
         """
@@ -990,7 +1007,6 @@ class SharingMixIn(object):
             ownerView = yield self.ownerView()
             yield ownerView.removeShare(self)
 
-
     @inlineCallbacks
     def sharingInvites(self):
         """
@@ -1008,13 +1024,14 @@ class SharingMixIn(object):
         )
 
         result = []
-        for homeUID, homeRID, _ignore_resourceID, resourceName, bindMode, bindStatus, bindMessage in invitedRows:
+        for homeUID, homeRID, homeStatus, _ignore_resourceID, resourceName, bindMode, bindStatus, bindMessage in invitedRows:
             invite = SharingInvitation(
                 resourceName,
                 self.ownerHome().name(),
                 self.ownerHome().id(),
                 homeUID,
                 homeRID,
+                homeStatus,
                 bindMode,
                 bindStatus,
                 bindMessage,
@@ -1022,7 +1039,6 @@ class SharingMixIn(object):
             result.append(invite)
         returnValue(result)
 
-
     @inlineCallbacks
     def sharingBindRecords(self):
         """
@@ -1044,7 +1060,6 @@ class SharingMixIn(object):
 
         returnValue(dict([(homeMap[getattr(record, self._bindHomeIDAttributeName)], record,) for record in records if record.bindMode != _BIND_MODE_OWN]))
 
-
     def migrateBindRecords(self, bindUID):
         """
         The user that owns this collection is being migrated to another pod. We need to switch over
@@ -1055,7 +1070,6 @@ class SharingMixIn(object):
         else:
             return self.migrateSharedToRecords()
 
-
     @inlineCallbacks
     def migrateSharedByRecords(self, bindUID):
         """
@@ -1097,7 +1111,6 @@ class SharingMixIn(object):
         if not remaining:
             yield calendar.remove()
 
-
     @inlineCallbacks
     def migrateSharedToRecords(self):
         """
@@ -1139,7 +1152,6 @@ class SharingMixIn(object):
         else:
             raise AssertionError("We must have a bind record for this calendar.")
 
-
     def externalHome(self):
         """
         Create and return an L{CommonHome} for the user being migrated. Note that when called, the user
@@ -1149,15 +1161,14 @@ class SharingMixIn(object):
         currentHome = self.viewerHome()
         return self._txn.homeWithUID(currentHome._homeType, currentHome.uid(), status=_HOME_STATUS_EXTERNAL, create=True)
 
-
     @inlineCallbacks
     def _initBindRevision(self):
-        yield self.syncToken() # init self._syncTokenRevision if None
+        yield self.syncToken()  # init self._syncTokenRevision if None
         self._bindRevision = self._syncTokenRevision
 
         bind = self._bindSchema
         yield self._updateBindColumnsQuery(
-            {bind.BIND_REVISION : Parameter("revision"), }
+            {bind.BIND_REVISION: Parameter("revision"), }
         ).on(
             self._txn,
             revision=self._bindRevision,
@@ -1166,7 +1177,6 @@ class SharingMixIn(object):
         )
         yield self.invalidateQueryCache()
 
-
     def sharedResourceType(self):
         """
         The sharing resource type. Needs to be overridden by each type of resource that can be shared.
@@ -1176,22 +1186,19 @@ class SharingMixIn(object):
         """
         return ""
 
-
     def newShareName(self):
         """
         Name used when creating a new share. By default this is a UUID.
         """
         return str(uuid4())
 
-
     def owned(self):
         """
         @see: L{ICalendar.owned}
         """
         return self._bindMode == _BIND_MODE_OWN
 
-
-    def isShared(self):
+    def isSharedByOwner(self):
         """
         For an owned collection indicate whether it is shared.
 
@@ -1200,7 +1207,6 @@ class SharingMixIn(object):
         """
         return self.owned() and self._bindMessage == "shared"
 
-
     @inlineCallbacks
     def setShared(self, shared):
         """
@@ -1231,7 +1237,6 @@ class SharingMixIn(object):
         yield self.invalidateQueryCache()
         yield self.notifyPropertyChanged()
 
-
     def direct(self):
         """
         Is this a "direct" share?
@@ -1240,7 +1245,6 @@ class SharingMixIn(object):
         """
         return self._bindMode == _BIND_MODE_DIRECT
 
-
     def indirect(self):
         """
         Is this an "indirect" share?
@@ -1249,35 +1253,30 @@ class SharingMixIn(object):
         """
         return self._bindMode == _BIND_MODE_INDIRECT
 
-
     def shareUID(self):
         """
         @see: L{ICalendar.shareUID}
         """
         return self.name()
 
-
     def shareMode(self):
         """
         @see: L{ICalendar.shareMode}
         """
         return self._bindMode
 
-
     def _effectiveShareMode(self, bindMode, viewerUID, txn):
         """
         Get the effective share mode without a calendar object
         """
         return bindMode
 
-
     def effectiveShareMode(self):
         """
         @see: L{ICalendar.shareMode}
         """
         return self._bindMode
 
-
     def shareName(self):
         """
         This is a path like name for the resource within the home being shared. For object resource
@@ -1290,35 +1289,30 @@ class SharingMixIn(object):
             name = self.parentCollection().name() + "/" + name
         return name
 
-
     def shareStatus(self):
         """
         @see: L{ICalendar.shareStatus}
         """
         return self._bindStatus
 
-
     def bindUID(self):
         """
         @see: L{ICalendar.bindUID}
         """
         return self._bindUID
 
-
     def accepted(self):
         """
         @see: L{ICalendar.shareStatus}
         """
         return self._bindStatus == _BIND_STATUS_ACCEPTED
 
-
     def shareMessage(self):
         """
         @see: L{ICalendar.shareMessage}
         """
         return self._bindMessage
 
-
     def getInviteCopyProperties(self):
         """
         Get a dictionary of property name/values (as strings) for properties that are shadowable and
@@ -1327,7 +1321,6 @@ class SharingMixIn(object):
         """
         return {}
 
-
     def setInviteCopyProperties(self, props):
         """
         Copy a set of shadowable properties (as name/value strings) onto this shared resource when
@@ -1336,7 +1329,6 @@ class SharingMixIn(object):
         """
         pass
 
-
     @classmethod
     def metadataColumns(cls):
         """
@@ -1352,7 +1344,6 @@ class SharingMixIn(object):
             cls._homeChildMetaDataSchema.MODIFIED,
         )
 
-
     @classmethod
     def metadataAttributes(cls):
         """
@@ -1368,7 +1359,6 @@ class SharingMixIn(object):
             "_modified",
         )
 
-
     @classmethod
     def bindColumns(cls):
         """
@@ -1388,7 +1378,6 @@ class SharingMixIn(object):
             cls._bindSchema.MESSAGE
         )
 
-
     @classmethod
     def bindAttributes(cls):
         """
@@ -1420,7 +1409,6 @@ class SharingMixIn(object):
 
         return ()
 
-
     @classmethod
     def additionalBindAttributes(cls):
         """
@@ -1431,7 +1419,6 @@ class SharingMixIn(object):
 
         return ()
 
-
     @classproperty
     def _childrenAndMetadataForHomeID(cls):
         bind = cls._bindSchema
@@ -1450,7 +1437,6 @@ class SharingMixIn(object):
                 bind.BIND_STATUS == _BIND_STATUS_ACCEPTED)
         )
 
-
     @inlineCallbacks
     def invalidateQueryCache(self):
         queryCacher = self._txn._queryCacher
