@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.common.datastore.test.test_sql_tables -*-
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,12 +25,14 @@ from twext.enterprise.dal.model import NO_DEFAULT
 from twext.enterprise.dal.model import Sequence, ProcedureCall
 from twext.enterprise.dal.parseschema import schemaFromPath
 from twext.enterprise.dal.syntax import FixedPlaceholder
-from twext.enterprise.ienterprise import ORACLE_DIALECT, POSTGRES_DIALECT
+from twext.enterprise.ienterprise import ORACLE_DIALECT, POSTGRES_DIALECT, \
+    DatabaseType
 from twext.enterprise.dal.syntax import Insert
 from twext.enterprise.ienterprise import ORACLE_TABLE_NAME_MAX
 import hashlib
 import itertools
 
+
 def _schemaFiles(version=None):
     """
     Find the set of files to process, either the current set if C{version} is L{None}, otherwise look
@@ -51,7 +53,6 @@ def _schemaFiles(version=None):
     return currentObj, extrasObj, outObj
 
 
-
 def _populateSchema(pathObj=None):
     """
     Generate the global L{SchemaSyntax}.
@@ -62,7 +63,6 @@ def _populateSchema(pathObj=None):
     return SchemaSyntax(schemaFromPath(pathObj))
 
 
-
 def _schemaExtras(out, extras):
     """
     If an extras file exists, add its entire content to the output stream..
@@ -72,7 +72,6 @@ def _schemaExtras(out, extras):
         out.write("\n".join(itertools.dropwhile(lambda x: not x.startswith("-- Extra schema to add"), extras.getContent().splitlines())) + "\n")
 
 
-
 schema = _populateSchema()
 
 
@@ -111,7 +110,6 @@ schema.ADDRESSBOOK_OBJECT.UID = schema.ADDRESSBOOK_OBJECT.VCARD_UID
 schema.ADDRESSBOOK_OBJECT.PARENT_RESOURCE_ID = schema.ADDRESSBOOK_OBJECT.ADDRESSBOOK_HOME_RESOURCE_ID
 
 
-
 def _combine(**kw):
     """
     Combine two table dictionaries used in a join to produce a single dictionary
@@ -124,7 +122,6 @@ def _combine(**kw):
     return result
 
 
-
 def _S(tableSyntax):
     """
     Construct a dictionary of strings from a L{TableSyntax} for those queries
@@ -145,7 +142,6 @@ def _S(tableSyntax):
     return result
 
 
-
 def _schemaConstants(nameColumn, valueColumn):
     """
     Get a constant value from the rows defined in the schema.
@@ -157,7 +153,6 @@ def _schemaConstants(nameColumn, valueColumn):
     return get
 
 
-
 def _schemaConstantsMaps(nameColumn, valueColumn):
     """
     Generate two dicts that map back and forth between SQL enum values and their
@@ -173,7 +168,6 @@ def _schemaConstantsMaps(nameColumn, valueColumn):
     return (toSQL, fromSQL)
 
 
-
 # Various constants
 
 _homeStatus = _schemaConstants(
@@ -276,7 +270,6 @@ _translatedTypes = {
 }
 
 
-
 def _quoted(x):
     """
     Quote an object for inclusion into an SQL string.
@@ -294,7 +287,6 @@ def _quoted(x):
         return str(x)
 
 
-
 def _staticSQL(sql, doquote=False):
     """
     Statically generate some SQL from some DAL syntax objects, interpolating
@@ -314,7 +306,7 @@ def _staticSQL(sql, doquote=False):
     @return: the generated SQL string.
     @rtype: C{str}
     """
-    qgen = QueryGenerator(ORACLE_DIALECT, FixedPlaceholder('%s'))
+    qgen = QueryGenerator(DatabaseType(ORACLE_DIALECT, "pyformat"), FixedPlaceholder('%s'))
     if doquote:
         qgen.shouldQuote = lambda name: True
     if hasattr(sql, 'subSQL'):
@@ -326,7 +318,6 @@ def _staticSQL(sql, doquote=False):
     return result
 
 
-
 def _translateSchema(out, schema=schema):
     """
     When run as a script, translate the schema to another dialect.  Currently
@@ -335,6 +326,8 @@ def _translateSchema(out, schema=schema):
     """
     for sequence in schema.model.sequences:
         out.write('create sequence %s;\n' % (sequence.name,))
+    out.write("\n\n")
+
     for table in schema:
         # The only table name which actually exceeds the length limit right now
         # is CALENDAR_OBJECT_ATTACHMENTS_MODE, which isn't actually _used_
@@ -377,11 +370,11 @@ def _translateSchema(out, schema=schema):
                             default = 0
                         out.write(" " + repr(default))
             if (
-                (not column.model.canBeNull())
+                (not column.model.canBeNull()) and
                 # Oracle treats empty strings as NULLs, so we have to accept
                 # NULL values in columns of a string type.  Other types should
                 # be okay though.
-                and typeName not in ('varchar', 'nclob', 'char', 'nchar', 'nvarchar', 'nvarchar2')
+                typeName not in ('varchar', 'nclob', 'char', 'nchar', 'nvarchar', 'nvarchar2')
             ):
                 out.write(' not null')
             if [column.model] in list(table.model.uniques()):
@@ -392,7 +385,7 @@ def _translateSchema(out, schema=schema):
                 out.write(" on delete %s" % (column.model.deleteAction,))
 
         def writeConstraint(name, cols):
-            out.write(", \n") # the table has to have some preceding columns
+            out.write(", \n")  # the table has to have some preceding columns
             out.write("    %s(%s)" % (
                 name, ", ".join('"' + col.name + '"' for col in cols)
             ))
@@ -403,7 +396,7 @@ def _translateSchema(out, schema=schema):
 
         for uniqueColumns in table.model.uniques():
             if len(uniqueColumns) == 1:
-                continue # already done inline, skip
+                continue  # already done inline, skip
             writeConstraint("unique ", uniqueColumns)
 
         for checkConstraint in table.model.constraints:
@@ -418,9 +411,10 @@ def _translateSchema(out, schema=schema):
 
         for row in table.model.schemaRows:
             cmap = dict([(getattr(table, cmodel.name), val)
-                        for (cmodel, val) in row.items()])
+                         for (cmodel, val) in row.items()])
             out.write(_staticSQL(Insert(cmap)))
             out.write(";\n")
+        out.write("\n\n" if len(table.model.schemaRows) else "\n")
 
     for index in schema.model.indexes:
         # Index names combine and repeat multiple table names and column names,
