@@ -1,6 +1,6 @@
 # -*- test-case-name: twext.enterprise.dal.test.test_record -*-
 ##
-# Copyright (c) 2015 Apple Inc. All rights reserved.
+# Copyright (c) 2015-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,21 +34,19 @@ log = Logger()
 Classes and methods for the SQL store.
 """
 
+
 class _EmptyCacher(object):
 
     def set(self, key, value):
         return succeed(True)
 
-
     def get(self, key, withIdentifier=False):
         return succeed(None)
 
-
     def delete(self, key):
         return succeed(True)
 
 
-
 class _SharedSyncLogic(object):
     """
     Logic for maintaining sync-token shared between notification collections and
@@ -65,7 +63,6 @@ class _SharedSyncLogic(object):
         return Select([Max(rev.REVISION)], From=rev,
                       Where=rev.RESOURCE_ID == Parameter("resourceID"))
 
-
     @classmethod
     def _revisionsForResourceIDs(cls, resourceIDs):
         rev = cls._revisionsSchema
@@ -77,7 +74,6 @@ class _SharedSyncLogic(object):
             GroupBy=rev.RESOURCE_ID
         )
 
-
     def revisionFromToken(self, token):
         if token is None:
             return 0
@@ -87,14 +83,12 @@ class _SharedSyncLogic(object):
         else:
             return token
 
-
     @inlineCallbacks
     def syncToken(self):
         if self._syncTokenRevision is None:
             self._syncTokenRevision = yield self.syncTokenRevision()
         returnValue(("%s_%s" % (self._resourceID, self._syncTokenRevision,)))
 
-
     @inlineCallbacks
     def syncTokenRevision(self):
         revision = (yield self._childSyncTokenQuery.on(self._txn, resourceID=self._resourceID))[0][0]
@@ -102,7 +96,6 @@ class _SharedSyncLogic(object):
             revision = int((yield self._txn.calendarserverValue("MIN-VALID-REVISION")))
         returnValue(revision)
 
-
     @classmethod
     @inlineCallbacks
     def childSyncTokenRevisions(cls, home, childResourceIDs):
@@ -117,11 +110,9 @@ class _SharedSyncLogic(object):
                 revisions[resourceID] = min_revision
         returnValue(revisions)
 
-
     def objectResourcesSinceToken(self, token):
         raise NotImplementedError()
 
-
     @classmethod
     def _objectNamesSinceRevisionQuery(cls, deleted=True):
         """
@@ -137,7 +128,6 @@ class _SharedSyncLogic(object):
             Where=where,
         )
 
-
     def resourceNamesSinceToken(self, token):
         """
         Return the changed and deleted resources since a particular sync-token. This simply extracts
@@ -149,7 +139,6 @@ class _SharedSyncLogic(object):
 
         return self.resourceNamesSinceRevision(self.revisionFromToken(token))
 
-
     @inlineCallbacks
     def resourceNamesSinceRevision(self, revision):
         """
@@ -185,7 +174,6 @@ class _SharedSyncLogic(object):
 
         returnValue((changed, deleted, invalid))
 
-
     @classproperty
     def _removeDeletedRevision(cls):
         rev = cls._revisionsSchema
@@ -193,7 +181,6 @@ class _SharedSyncLogic(object):
                       Where=(rev.HOME_RESOURCE_ID == Parameter("homeID")).And(
                           rev.COLLECTION_NAME == Parameter("collectionName")))
 
-
     @classproperty
     def _addNewRevision(cls):
         rev = cls._revisionsSchema
@@ -210,7 +197,6 @@ class _SharedSyncLogic(object):
             Return=[rev.REVISION]
         )
 
-
     @inlineCallbacks
     def _initSyncToken(self):
         yield self._removeDeletedRevision.on(
@@ -222,7 +208,6 @@ class _SharedSyncLogic(object):
                                     collectionName=self._name)))[0][0]
         self._txn.bumpRevisionForObject(self)
 
-
     @classproperty
     def _renameSyncTokenQuery(cls):
         """
@@ -241,7 +226,6 @@ class _SharedSyncLogic(object):
             Return=rev.REVISION
         )
 
-
     @inlineCallbacks
     def _renameSyncToken(self):
         rows = yield self._renameSyncTokenQuery.on(
@@ -252,7 +236,6 @@ class _SharedSyncLogic(object):
         else:
             yield self._initSyncToken()
 
-
     @classproperty
     def _bumpSyncTokenQuery(cls):
         """
@@ -269,7 +252,6 @@ class _SharedSyncLogic(object):
                   (rev.RESOURCE_NAME == None)
         )
 
-
     @inlineCallbacks
     def _bumpSyncToken(self):
 
@@ -281,7 +263,6 @@ class _SharedSyncLogic(object):
             )
             self._syncTokenRevision = None
 
-
     @classproperty
     def _deleteSyncTokenQuery(cls):
         """
@@ -296,7 +277,6 @@ class _SharedSyncLogic(object):
                   (rev.COLLECTION_NAME == None)
         )
 
-
     @classproperty
     def _sharedRemovalQuery(cls):
         """
@@ -315,7 +295,6 @@ class _SharedSyncLogic(object):
                 rev.RESOURCE_NAME == None)
         )
 
-
     @classproperty
     def _unsharedRemovalQuery(cls):
         """
@@ -333,7 +312,6 @@ class _SharedSyncLogic(object):
                 rev.RESOURCE_NAME == None),
         )
 
-
     @inlineCallbacks
     def _deletedSyncToken(self, sharedRemoval=False):
         """
@@ -362,19 +340,15 @@ class _SharedSyncLogic(object):
                                                 resourceID=self._resourceID)
         self._syncTokenRevision = None
 
-
     def _insertRevision(self, name):
         return self._changeRevision("insert", name)
 
-
     def _updateRevision(self, name):
         return self._changeRevision("update", name)
 
-
     def _deleteRevision(self, name):
         return self._changeRevision("delete", name)
 
-
     @classproperty
     def _deleteBumpTokenQuery(cls):
         rev = cls._revisionsSchema
@@ -389,7 +363,6 @@ class _SharedSyncLogic(object):
             Return=rev.REVISION
         )
 
-
     @classproperty
     def _updateBumpTokenQuery(cls):
         rev = cls._revisionsSchema
@@ -403,7 +376,6 @@ class _SharedSyncLogic(object):
             Return=rev.REVISION
         )
 
-
     @classproperty
     def _insertFindPreviouslyNamedQuery(cls):
         rev = cls._revisionsSchema
@@ -414,7 +386,6 @@ class _SharedSyncLogic(object):
                 rev.RESOURCE_NAME == Parameter("name"))
         )
 
-
     @classproperty
     def _updatePreviouslyNamedQuery(cls):
         rev = cls._revisionsSchema
@@ -429,7 +400,6 @@ class _SharedSyncLogic(object):
             Return=rev.REVISION
         )
 
-
     @classproperty
     def _completelyNewRevisionQuery(cls):
         rev = cls._revisionsSchema
@@ -444,7 +414,6 @@ class _SharedSyncLogic(object):
             Return=rev.REVISION
         )
 
-
     @classproperty
     def _completelyNewDeletedRevisionQuery(cls):
         rev = cls._revisionsSchema
@@ -459,7 +428,6 @@ class _SharedSyncLogic(object):
             Return=rev.REVISION
         )
 
-
     @inlineCallbacks
     def _changeRevision(self, action, name):
 
@@ -515,7 +483,6 @@ class _SharedSyncLogic(object):
         yield self._maybeNotify()
         returnValue(self._syncTokenRevision)
 
-
     def _maybeNotify(self):
         """
         Maybe notify changed.  (Overridden in NotificationCollection.)
@@ -523,7 +490,6 @@ class _SharedSyncLogic(object):
         return succeed(None)
 
 
-
 def determineNewest(uid, homeType):
     """
     Construct a query to determine the modification time of the newest object
@@ -564,7 +530,6 @@ def determineNewest(uid, homeType):
     )
 
 
-
 @inlineCallbacks
 def mergeHomes(sqlTxn, one, other, homeType):
     """
@@ -623,7 +588,6 @@ def mergeHomes(sqlTxn, one, other, homeType):
     yield returnValue(newer)
 
 
-
 def _renameHome(txn, table, oldUID, newUID):
     """
     Rename a calendar, addressbook, or notification home.  Note that this
@@ -650,7 +614,6 @@ def _renameHome(txn, table, oldUID, newUID):
                   Where=table.OWNER_UID == oldUID).on(txn)
 
 
-
 def _dontBotherWithNotifications(older, newer, merge):
     """
     Notifications are more transient and can be easily worked around; don't
@@ -659,7 +622,6 @@ def _dontBotherWithNotifications(older, newer, merge):
     pass
 
 
-
 @inlineCallbacks
 def _normalizeHomeUUIDsIn(t, homeType):
     """
@@ -746,7 +708,6 @@ def _normalizeHomeUUIDsIn(t, homeType):
     returnValue(None)
 
 
-
 def _getHome(txn, homeType, uid):
     """
     Like L{CommonHome.homeWithUID} but also honoring ENOTIFICATIONTYPE which
@@ -770,7 +731,6 @@ def _getHome(txn, homeType, uid):
         return txn.homeWithUID(homeType, uid)
 
 
-
 @inlineCallbacks
 def _normalizeColumnUUIDs(txn, column):
     """
@@ -804,7 +764,6 @@ def _normalizeColumnUUIDs(txn, column):
             yield Update({column: after}, Where=where).on(txn)
 
 
-
 class _AndNothing(object):
     """
     Simple placeholder for iteratively generating a 'Where' clause; the 'And'
@@ -818,7 +777,6 @@ class _AndNothing(object):
         return self
 
 
-
 @inlineCallbacks
 def _needsNormalizationUpgrade(txn):
     """
@@ -842,14 +800,13 @@ def _needsNormalizationUpgrade(txn):
     returnValue(False)
 
 
-
 @inlineCallbacks
 def fixUUIDNormalization(store):
     """
     Fix all UUIDs in the given SQL store to be in a canonical form;
     00000000-0000-0000-0000-000000000000 format and upper-case.
     """
-    t = store.newTransaction(disableCache=True)
+    t = store.newTransaction(label="fixUUIDNormalization", disableCache=True)
 
     # First, let's see if there are any calendar, addressbook, or notification
     # homes that have a de-normalized OWNER_UID.  If there are none, then we can
