@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -39,7 +39,6 @@ from txdav.common.icommondatastore import AllRetriesFailed
 from txdav.xml import element as davxml
 
 
-
 exampleUID = UUID("a" * 32)
 denormalizedUID = unicode(exampleUID)
 normalizedUID = denormalizedUID.upper()
@@ -60,7 +59,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
             extraUids=(denormalizedUID, normalizedUID, u"uid")
         )
 
-
     @inlineCallbacks
     def test_logging(self):
         """
@@ -83,7 +81,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         self.assertEqual(len(version), 1)
         self.assertEqual(len(version[0]), 1)
 
-
     def test_logWaits(self):
         """
         CommonStoreTransactionMonitor logs waiting transactions.
@@ -108,7 +105,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         self.assertNotEqual(ctr[0], 0)
         txn.abort()
 
-
     def test_txnTimeout(self):
         """
         CommonStoreTransactionMonitor terminates long transactions.
@@ -135,7 +131,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         self.assertTrue(txn._sqlTxn._completed)
         self.assertTrue(txn.timedout)
 
-
     def test_logWaitsAndTxnTimeout(self):
         """
         CommonStoreTransactionMonitor logs waiting transactions and terminates
@@ -166,7 +161,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         self.assertNotEqual(ctr[1], 0)
         self.assertTrue(txn._sqlTxn._completed)
 
-
     @inlineCallbacks
     def test_subtransactionOK(self):
         """
@@ -188,7 +182,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         (yield txn.subtransaction(_test, retries=0))[0][0]
         self.assertEqual(ctr[0], 1)
 
-
     @inlineCallbacks
     def test_subtransactionOKAfterRetry(self):
         """
@@ -212,7 +205,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         (yield txn.subtransaction(_test, retries=1))[0][0]
         self.assertEqual(ctr[0], 2)
 
-
     @inlineCallbacks
     def test_subtransactionFailNoRetry(self):
         """
@@ -235,12 +227,11 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         try:
             (yield txn.subtransaction(_test, retries=0))[0][0]
         except AllRetriesFailed:
-            pass
+            self.flushLoggedErrors()
         else:
             self.fail("AllRetriesFailed not raised")
         self.assertEqual(ctr[0], 1)
 
-
     @inlineCallbacks
     def test_subtransactionFailSomeRetries(self):
         """
@@ -264,12 +255,11 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         try:
             (yield txn.subtransaction(_test, retries=2))[0][0]
         except AllRetriesFailed:
-            pass
+            self.flushLoggedErrors()
         else:
             self.fail("AllRetriesFailed not raised")
         self.assertEqual(ctr[0], 3)
 
-
     @inlineCallbacks
     def test_subtransactionAbortOuterTransaction(self):
         """
@@ -294,12 +284,11 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         try:
             result = yield started
         except AllRetriesFailed:
-            pass
+            self.flushLoggedErrors()
         else:
             self.fail("AllRetriesFailed not raised, %r returned instead" %
                       (result,))
 
-
     @inlineCallbacks
     def test_changeRevision(self):
         """
@@ -310,6 +299,7 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
             pass
 
         class TestCommonHomeChild(CommonHomeChild):
+            _homeSchema = schema.CALENDAR_HOME
             _homeChildSchema = schema.CALENDAR
             _homeChildMetaDataSchema = schema.CALENDAR_METADATA
             _bindSchema = schema.CALENDAR_BIND
@@ -354,7 +344,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
 
         yield txn.abort()
 
-
     @inlineCallbacks
     def test_normalizeColumnUUIDs(self):
         """
@@ -396,7 +385,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
             ]
         )
 
-
     @inlineCallbacks
     def allHomeUIDs(self, table=schema.CALENDAR_HOME):
         """
@@ -407,7 +395,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         yield self.commit()
         returnValue(results)
 
-
     @inlineCallbacks
     def test_fixUUIDNormalization_lowerToUpper(self):
         """
@@ -424,7 +411,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
             [[normalizedUID]]
         )
 
-
     @inlineCallbacks
     def test_fixUUIDNormalization_lowerToUpper_notification(self):
         """
@@ -441,7 +427,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
             [[normalizedUID]]
         )
 
-
     @inlineCallbacks
     def test_fixUUIDNormalization_lowerToUpper_addressbook(self):
         """
@@ -458,7 +443,6 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
             [[normalizedUID]]
         )
 
-
     @inlineCallbacks
     def test_inTransaction(self):
         """
@@ -491,19 +475,16 @@ class CommonSQLStoreTests(CommonCommonTests, TestCase):
         self.assertEquals(self.txn.label, "bad")
 
 
-
 class StubTransaction(object):
 
     def __init__(self, label):
         self.label = label
         self.action = None
 
-
     def commit(self):
         self.action = "committed"
         return succeed(None)
 
-
     def abort(self):
         self.action = "aborted"
         return succeed(None)
