@@ -1,5 +1,5 @@
 # #
-# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import re
 Tests for L{txdav.common.datastore.sql}.
 """
 
+
 class SQLSchemaFiles(TestCase):
     """
     Tests for txdav.common.datastore.sql_schema having complete information. Note that upgrade files are checked elsewhere.
@@ -39,7 +40,6 @@ class SQLSchemaFiles(TestCase):
 
         return int(found.group(1))
 
-
     def test_old_files(self):
         """
         Make sure txdav.common.datastore.sql_schema.old contains all the appropriate old versions
@@ -65,7 +65,6 @@ class SQLSchemaFiles(TestCase):
                 old_set.add(int(found.group(1)))
             self.assertEqual(current_set, old_set, msg="Missing old schema file for dialect: %s" % (child.basename(),))
 
-
     def test_old_files_consistent(self):
         """
         Make sure txdav.common.datastore.sql_schema.old contains all the appropriate old versions
@@ -87,7 +86,6 @@ class SQLSchemaFiles(TestCase):
                 old_version = self.versionFromSchema(oldVersion)
                 self.assertEqual(old_name_version, old_version, "Name of schema file does not match actual schema version: %s" % (oldVersion.path,))
 
-
     def test_current_oracle(self):
         """
         Make sure current-oracle-dialect.sql matches current.sql
@@ -114,10 +112,12 @@ class SQLSchemaFiles(TestCase):
                     if constraint.affectsColumns[0].type.name in ("text", "char", "varchar"):
                         table.constraints.remove(constraint)
 
+        # Remove stored procedures which we only use on Oracle
+        schema_oracle.functions = []
+
         mismatched = schema_current.compare(schema_oracle)
         self.assertEqual(len(mismatched), 0, msg=", ".join(mismatched))
 
-
     def test_schema_compare(self):
 
         sqlSchema = getModule(__name__).filePath.parent().sibling("sql_schema")
@@ -132,8 +132,7 @@ class SQLSchemaFiles(TestCase):
         v6Schema = schemaFromPath(sqlSchema.child("old").child("postgres-dialect").child("v6.sql"))
         v5Schema = schemaFromPath(sqlSchema.child("old").child("postgres-dialect").child("v5.sql"))
         mismatched = v6Schema.compare(v5Schema)
-        self.assertEqual(len(mismatched), 4, msg="\n".join(mismatched))
-
+        self.assertEqual(len(mismatched), 5, msg="\n".join(mismatched))
 
     def test_references_index(self):
         """
@@ -159,7 +158,6 @@ class SQLSchemaFiles(TestCase):
 
         self.assertEqual(len(failures), 0, msg="Missing index for references columns: %s" % (", ".join(sorted(failures))))
 
-
     def test_primary_keys(self):
         """
         Make sure current-oracle-dialect.sql matches current.sql
