@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.caldav.datastore.test.test_sql -*-
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,6 +38,7 @@ from twext.enterprise.dal.test.test_parseschema import SchemaTestHelper
 
 from textwrap import dedent
 
+
 class SampleSomeColumns(TestCase, SchemaTestHelper):
     """
     Sample some columns from the tables defined by L{schema} and verify that
@@ -53,7 +54,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
         _translateSchema(io, *schema)
         return io.getvalue()
 
-
     def assertSortaEquals(self, a, b):
         """
         Assert that two strings are equals, modulo whitespace differences.
@@ -62,13 +62,11 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
         sortaB = " ".join(b.split())
         self.assertEquals(sortaA, sortaB)
 
-
     def test_addressbookObjectResourceID(self):
         ao = schema.ADDRESSBOOK_OBJECT
         self.assertEquals(ao.RESOURCE_ID.model.name,
                           "RESOURCE_ID")
 
-
     def test_schemaTranslation(self):
         """
         Basic integration test to make sure that the current, production schema
@@ -76,7 +74,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
         """
         self.translated()
 
-
     def test_schemaTranslationIncludesVersion(self):
         """
         _translateSchema includes 'insert' rows too.
@@ -93,7 +90,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
                       "values ('VERSION', '%s');" % version,
                       self.translated())
 
-
     def test_translateSingleUnique(self):
         """
         L{_translateSchema} translates single-column 'unique' statements inline.
@@ -109,7 +105,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             'create table alpha ( "beta" integer unique );'
         )
 
-
     def test_translateSingleTableUnique(self):
         """
         L{_translateSchema} translates single-column 'unique' statements inline,
@@ -125,7 +120,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             'create table alpha ( "beta" integer unique );'
         )
 
-
     def test_multiTableUnique(self):
         """
         L{_translateSchema} translates multi-column 'unique' statements.
@@ -143,7 +137,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             'unique ("beta", "gamma") );'
         )
 
-
     def test_translateSinglePrimaryKey(self):
         """
         L{_translateSchema} translates single-column 'primary key' statements
@@ -160,7 +153,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             'create table alpha ( "beta" integer primary key );'
         )
 
-
     def test_translateSingleTablePrimaryKey(self):
         """
         L{_translateSchema} translates single-column 'primary key' statements
@@ -176,7 +168,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             'create table alpha ( "beta" integer primary key );'
         )
 
-
     def test_multiTablePrimaryKey(self):
         """
         L{_translateSchema} translates multi-column 'primary key' statements.
@@ -194,7 +185,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             'primary key ("beta", "gamma") );'
         )
 
-
     def test_primaryKeyUniqueOrdering(self):
         """
         If a table specifies both a PRIMARY KEY and a UNIQUE constraint, the
@@ -214,7 +204,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             'primary key ("beta", "gamma"), unique ("beta", "delta") );'
         )
 
-
     def test_anonymousCheckConstraint(self):
         """
         Named 'check' constraints are propagated through translation without
@@ -231,7 +220,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             " );"
         )
 
-
     def test_namedCheckConstraint(self):
         """
         Named 'check' constraints are propagated through translation without
@@ -248,7 +236,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
             " );"
         )
 
-
     def test_youBrokeTheSchema(self):
         """
         Oracle table names have a 30-character limit.  Our schema translator
@@ -270,7 +257,6 @@ class SampleSomeColumns(TestCase, SchemaTestHelper):
         )
 
 
-
 class SQLSplitterTests(TestCase):
     """
     Test that strings which mix zero or more sql statements with zero or more
@@ -286,7 +272,6 @@ class SQLSplitterTests(TestCase):
         self.assertEquals(r1, "select * from foo")
         self.assertRaises(StopIteration, result.next)
 
-
     def test_returnTwoSimpleStatements(self):
         """
         Two simple sql statements yield two separate strings
@@ -298,7 +283,6 @@ class SQLSplitterTests(TestCase):
         self.assertEquals(r2, "select bang from boop")
         self.assertRaises(StopIteration, result.next)
 
-
     def test_returnOneComplexStatement(self):
         """
         One complex sql statement yields a single string
@@ -329,7 +313,6 @@ class SQLSplitterTests(TestCase):
         self.assertEquals(r1, bigSQL.rstrip(";"))
         self.assertRaises(StopIteration, result.next)
 
-
     def test_returnOnePlSQL(self):
         """
         One pl/sql block yields a single string
@@ -348,7 +331,6 @@ class SQLSplitterTests(TestCase):
         self.assertEquals(r1, s1)
         self.assertRaises(StopIteration, result.next)
 
-
     def test_returnOnePlSQLAndOneSQL(self):
         """
         One sql statement and one pl/sql statement yields two separate strings
@@ -379,7 +361,6 @@ class SQLSplitterTests(TestCase):
         self.assertEquals(r2, s2)
         self.assertRaises(StopIteration, result.next)
 
-
     def test_returnOnePlSQLAndOneSQLAndOneFunction(self):
         """
         One sql statement and one pl/sql statement yields two separate strings
@@ -421,6 +402,65 @@ class SQLSplitterTests(TestCase):
         self.assertEquals(r3, s3)
         self.assertRaises(StopIteration, result.next)
 
+    def test_returnTwoPlSQLCreateOrReplace(self):
+        """
+        One pl/sql block yields a single string
+        """
+        plsql = """create or replace function next_job_all(now timestamp)
+  return integer is
+  cursor c1 is
+   select JOB_ID from JOB
+   where ASSIGNED is NULL and PAUSE = 0 and NOT_BEFORE <= now
+   order by PRIORITY desc
+   for update skip locked;
+  result integer;
+begin
+  open c1;
+  fetch c1 into result;
+  close c1;
+  return result;
+end;
+/
+
+create or replace function next_job_medium_high(now timestamp)
+  return integer is
+  cursor c1 is
+    select JOB_ID from JOB
+    where PRIORITY != 0 and ASSIGNED is NULL and PAUSE = 0 and NOT_BEFORE <= now
+    order by PRIORITY desc
+    for update skip locked;
+  result integer;
+begin
+  open c1;
+  fetch c1 into result;
+  close c1;
+  return result;
+end;
+/
+"""
+        s1 = """create or replace function next_job_all(now timestamp)
+return integer is
+cursor c1 is
+select JOB_ID from JOB
+where ASSIGNED is NULL and PAUSE = 0 and NOT_BEFORE <= now
+order by PRIORITY desc
+for update skip locked;result integer;begin
+open c1;fetch c1 into result;close c1;return result;end;"""
+        s2 = """create or replace function next_job_medium_high(now timestamp)
+return integer is
+cursor c1 is
+select JOB_ID from JOB
+where PRIORITY != 0 and ASSIGNED is NULL and PAUSE = 0 and NOT_BEFORE <= now
+order by PRIORITY desc
+for update skip locked;result integer;begin
+open c1;fetch c1 into result;close c1;return result;end;"""
+
+        result = splitSQLString(plsql)
+        r1 = result.next()
+        self.assertEquals(r1, s1)
+        r2 = result.next()
+        self.assertEquals(r2, s2)
+        self.assertRaises(StopIteration, result.next)
 
     def test_actualSchemaUpgrade(self):
         """
@@ -430,7 +470,7 @@ class SQLSplitterTests(TestCase):
         realsql = dedent(
             '''
             ----
-            -- Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+            -- Copyright (c) 2011-2017 Apple Inc. All rights reserved.
             --
             -- Licensed under the Apache License, Version 2.0 (the "License");
             -- you may not use this file except in compliance with the License.
