@@ -1,5 +1,6 @@
+# coding=utf-8
 ##
-# Copyright (c) 2015 Apple Inc. All rights reserved.
+# Copyright (c) 2015-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,28 +14,30 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
+from txdav.xml import element
+from txdav.base.propertystore.base import PropertyName
 
 """
 Trash-specific tests for L{txdav.common.datastore.sql}.
 """
 
-from calendarserver.tools.trash import emptyTrashForPrincipal
+from calendarserver.tools.trash import emptyTrashForPrincipal,\
+    restoreTrashedEvent, restoreTrashedCollection
 from pycalendar.datetime import DateTime
 from twext.enterprise.jobs.jobitem import JobItem
 from twisted.internet import reactor
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twistedcaldav.ical import Component
 from twistedcaldav.test.util import StoreTestCase
+from txdav.caldav.datastore.sql import CalendarObject
 from txdav.common.datastore.sql_tables import _BIND_MODE_WRITE
 
 
-class TrashTests(StoreTestCase):
-
+class TrashTestsBase(StoreTestCase):
 
     def _homeForUser(self, txn, userName):
         return txn.calendarHomeWithUID(userName, create=True)
 
-
     @inlineCallbacks
     def _collectionForUser(self, txn, userName, collectionName, create=False, onlyInTrash=False):
         home = yield txn.calendarHomeWithUID(userName, create=True)
@@ -44,7 +47,6 @@ class TrashTests(StoreTestCase):
                 collection = yield home.createCalendarWithName(collectionName)
         returnValue(collection)
 
-
     @inlineCallbacks
     def _createResource(self, txn, userName, collectionName, resourceName, data):
         collection = yield self._collectionForUser(txn, userName, collectionName)
@@ -53,7 +55,6 @@ class TrashTests(StoreTestCase):
         )
         returnValue(resource)
 
-
     @inlineCallbacks
     def _getResource(self, txn, userName, collectionName, resourceName):
         collection = yield self._collectionForUser(txn, userName, collectionName)
@@ -66,14 +67,12 @@ class TrashTests(StoreTestCase):
         resource = yield collection.calendarObjectWithName(resourceName)
         returnValue(resource)
 
-
     @inlineCallbacks
     def _getResourceNames(self, txn, userName, collectionName):
         collection = yield self._collectionForUser(txn, userName, collectionName)
         resourceNames = yield collection.listObjectResources()
         returnValue(resourceNames)
 
-
     @inlineCallbacks
     def _getTrashNames(self, txn, userName):
         home = yield txn.calendarHomeWithUID(userName)
@@ -81,14 +80,12 @@ class TrashTests(StoreTestCase):
         resourceNames = yield trash.listObjectResources()
         returnValue(resourceNames)
 
-
     @inlineCallbacks
     def _updateResource(self, txn, userName, collectionName, resourceName, data):
         resource = yield self._getResource(txn, userName, collectionName, resourceName)
         yield resource.setComponent(Component.fromString(data))
         returnValue(resource)
 
-
     @inlineCallbacks
     def _getResourceData(self, txn, userName, collectionName, resourceName):
         resource = yield self._getResource(txn, userName, collectionName, resourceName)
@@ -98,6 +95,8 @@ class TrashTests(StoreTestCase):
         returnValue(str(component).replace("\r\n ", ""))
 
 
+class TrashTests(TrashTestsBase):
+
     @inlineCallbacks
     def test_trashUnscheduled(self):
         """
@@ -107,7 +106,6 @@ class TrashTests(StoreTestCase):
         from twistedcaldav.stdconfig import config
         self.patch(config, "EnableTrashCollection", True)
 
-
         data1 = """BEGIN:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
@@ -203,7 +201,6 @@ END:VCALENDAR
         resource = yield self._getResource(txn, "user01", trash.name(), "")
         self.assertTrue(resource is None)
 
-
         # One object in collection
         resourceNames = yield self._getResourceNames(txn, "user01", "calendar")
         self.assertEqual(len(resourceNames), 1)
@@ -218,7 +215,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashScheduledFullyInFuture(self):
 
@@ -369,7 +365,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashScheduledFullyInFutureAttendeeTrashedThenOrganizerChanged(self):
 
@@ -420,7 +415,6 @@ END:VEVENT
 END:VCALENDAR
 """ % subs
 
-
         start.offsetHours(1)
         end.offsetHours(1)
 
@@ -456,7 +450,6 @@ END:VEVENT
 END:VCALENDAR
 """ % subs
 
-
         # user01 invites user02
         txn = self.store.newTransaction()
         yield self._createResource(
@@ -521,7 +514,6 @@ END:VCALENDAR
         resource = yield self._getResource(txn, "user01", "inbox", "")
         yield resource.remove()
 
-
         # user02's copy is in the trash only, and still has ACCEPTED
         resourceNames = yield self._getResourceNames(txn, "user02", trash2.name())
         self.assertEqual(len(resourceNames), 1)
@@ -543,7 +535,6 @@ END:VCALENDAR
         # for row in result:
         #     print("calendar ROW", row)
 
-
         yield txn.commit()
 
         # user01 makes a change to event while user02's copy is in the trash
@@ -607,7 +598,6 @@ END:VCALENDAR
 
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
         txn = self.store.newTransaction()
 
         resourceNames = yield self._getResourceNames(txn, "user02", trash2.name())
@@ -621,7 +611,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashScheduledFullyInFutureAttendeeRemovedThenOrganizerChanged(self):
 
@@ -672,7 +661,6 @@ END:VEVENT
 END:VCALENDAR
 """ % subs
 
-
         data3 = """BEGIN:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
@@ -689,7 +677,6 @@ END:VEVENT
 END:VCALENDAR
 """ % subs
 
-
         start.offsetHours(1)
         end.offsetHours(1)
 
@@ -709,7 +696,6 @@ END:VEVENT
 END:VCALENDAR
 """ % subs
 
-
         # user01 invites user02
         txn = self.store.newTransaction()
         yield self._createResource(
@@ -788,7 +774,6 @@ END:VCALENDAR
         self.assertTrue("PARTSTAT=ACCEPTED" in data)
         yield txn.commit()
 
-
         # user02 removes the event completely from the trash
 
         txn = self.store.newTransaction()
@@ -798,7 +783,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
         # user01 makes a SUMMARY change to event (user02 does not get notified because they are fully declined)
         txn = self.store.newTransaction()
 
@@ -844,7 +828,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashScheduledFullyInFutureAttendeeTrashedThenPutBack(self):
 
@@ -895,7 +878,6 @@ END:VEVENT
 END:VCALENDAR
 """ % subs
 
-
         # user01 invites user02
         txn = self.store.newTransaction()
         yield self._createResource(
@@ -1013,7 +995,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashScheduledFullyInPast(self):
 
@@ -1152,10 +1133,8 @@ END:VCALENDAR
         data = yield self._getResourceData(txn, "user02", "calendar", "")
         self.assertTrue("PARTSTAT=TENTATIVE" in data)
 
-
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashScheduledFullyInPastAttendeeTrashedThenPutBack(self):
 
@@ -1319,7 +1298,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashScheduledSpanningNow(self):
 
@@ -1485,7 +1463,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashScheduledSpanningNowAttendeeTrashedThenPutBack(self):
 
@@ -1648,7 +1625,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashCalendar(self):
 
@@ -1711,7 +1687,6 @@ END:VCALENDAR
         self.assertFalse("test" in names)
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashCalendarWithUnscheduled(self):
 
@@ -1813,6 +1788,129 @@ END:VCALENDAR
 
         yield txn.commit()
 
+    @inlineCallbacks
+    def test_trashCalendarRestoreWithFailures(self):
+
+        from twistedcaldav.stdconfig import config
+        self.patch(config, "EnableTrashCollection", True)
+
+        txn = self.store.newTransaction()
+
+        collection = yield self._collectionForUser(txn, "user01", "test", create=True)
+
+        data1 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:{uid}
+DTSTART;TZID=America/Los_Angeles:20141108T093000
+DTEND;TZID=America/Los_Angeles:20141108T103000
+CREATED:20141106T192546Z
+DTSTAMP:20141106T192546Z
+RRULE:FREQ=DAILY
+SEQUENCE:0
+SUMMARY:repeating event
+TRANSP:OPAQUE
+END:VEVENT
+BEGIN:VEVENT
+UID:{uid}
+RECURRENCE-ID;TZID=America/Los_Angeles:20141111T093000
+DTSTART;TZID=America/Los_Angeles:20141111T110000
+DTEND;TZID=America/Los_Angeles:20141111T120000
+CREATED:20141106T192546Z
+DTSTAMP:20141106T192546Z
+SEQUENCE:0
+SUMMARY:repeating event
+TRANSP:OPAQUE
+END:VEVENT
+END:VCALENDAR
+"""
+
+        # Create objects
+        yield collection.createObjectResourceWithName(
+            "test1.ics",
+            Component.allFromString(data1.format(uid="1CE3B280-DBC9-4E8E-B0B2-996754020E5F"))
+        )
+        yield collection.createObjectResourceWithName(
+            "test2.ics",
+            Component.allFromString(data1.format(uid="2CE3B280-DBC9-4E8E-B0B2-996754020E5F"))
+        )
+
+        # Two objects in collection
+        objects = yield collection.listObjectResources()
+        self.assertEquals(len(objects), 2)
+
+        # No objects in trash
+        home1 = yield self._homeForUser(txn, "user01")
+        trash1 = yield home1.getTrash(create=True)
+        trash = yield self._collectionForUser(txn, "user01", trash1.name())
+        objects = yield trash.listObjectResources()
+        self.assertEquals(len(objects), 0)
+
+        # # Verify it's not in the trash
+        # for resource in (resource1, resource2):
+        #     self.assertFalse(resource.isInTrash())
+        #     trashed = resource.whenTrashed()
+        #     self.assertTrue(trashed is None)
+
+        # collection = yield self._collectionForUser(txn, "user01", "test")
+        # resources = yield trash.trashForCollection(collection._resourceID)
+        # self.assertEquals(len(resources), 0)
+
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        collection = yield self._collectionForUser(txn, "user01", "test")
+        yield collection.remove()
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        # Two objects in trash
+        trash = yield self._collectionForUser(txn, "user01", trash1.name())
+        objects = yield trash.listObjectResources()
+        self.assertEquals(len(objects), 2)
+
+        resources = yield trash.trashForCollection(collection._resourceID)
+        self.assertEquals(len(resources), 2)
+
+        home = yield self._homeForUser(txn, "user01")
+        names = yield home.listChildren(onlyInTrash=True)
+        trashedName = names[0]
+        collection = yield self._collectionForUser(txn, "user01", trashedName, onlyInTrash=True)
+
+        # Patch CalendarObject.fromTrash to raise an exception for one specific resource
+        originalFromTrash = CalendarObject.fromTrash
+
+        def fakeFromTrash(self):
+            if self._uid == "2CE3B280-DBC9-4E8E-B0B2-996754020E5F":
+                raise 1 / 0
+            else:
+                return originalFromTrash(self)
+        self.patch(CalendarObject, "fromTrash", fakeFromTrash)
+
+        yield collection.fromTrash()
+
+        yield txn.commit()
+
+        yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
+
+        # Now when we restore from trash, since one event failed to restore, we
+        # should still have gotten the other back
+        txn = self.store.newTransaction()
+        home = yield self._homeForUser(txn, "user01")
+        names = yield home.listChildren()
+        self.assertTrue(trashedName in names)
+        names = yield home.listChildren(onlyInTrash=True)
+        self.assertFalse(trashedName in names)
+        resourceNames = yield self._getResourceNames(txn, "user01", trashedName)
+        self.assertEqual(len(resourceNames), 1)
+
+        # Failed event still in the trash
+        trash = yield self._collectionForUser(txn, "user01", trash1.name())
+        objects = yield trash.listObjectResources()
+        self.assertEquals(len(objects), 1)
+
+        yield txn.commit()
 
     @inlineCallbacks
     def test_shareeDelete(self):
@@ -1900,7 +1998,6 @@ END:VCALENDAR
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashDuplicateUID(self):
         """
@@ -1969,7 +2066,6 @@ END:VCALENDAR
         self.assertEqual(len(resourceNames), 0)
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashDuplicateUIDDifferentOrganizer(self):
         """
@@ -2053,7 +2149,6 @@ END:VCALENDAR
         yield txn.commit()
         yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
-
     @inlineCallbacks
     def test_tool_emptyTrashForPrincipal(self):
 
@@ -2142,7 +2237,6 @@ END:VCALENDAR
         self.assertEquals(len(result), 0)
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_trashedCalendars(self):
         """
@@ -2154,16 +2248,16 @@ END:VCALENDAR
 
         txn = self.store.newTransaction()
         home = yield self._homeForUser(txn, "user01")
-        yield home.getTrash(create=True) # force loading trash
+        yield home.getTrash(create=True)  # force loading trash
         calendars = yield home.calendars(onlyInTrash=False)
         self.assertEquals(
             set([c.name() for c in calendars]),
-            set(["tasks", "inbox", "calendar"]) # trash not there
+            set(["tasks", "inbox", "calendar"])  # trash not there
         )
         calendars = yield home.calendars(onlyInTrash=True)
         self.assertEquals(
             set([c.name() for c in calendars]),
-            set() # trash not there either
+            set()  # trash not there either
         )
         yield txn.commit()
 
@@ -2175,7 +2269,7 @@ END:VCALENDAR
 
         txn = self.store.newTransaction()
         home = yield self._homeForUser(txn, "user01")
-        yield home.getTrash(create=True) # force loading trash
+        yield home.getTrash(create=True)  # force loading trash
         calendars = yield home.calendars(onlyInTrash=False)
         self.assertEquals(
             set([c.name() for c in calendars]),
@@ -2185,3 +2279,163 @@ END:VCALENDAR
         self.assertEquals(len(calendars), 1)
         self.assertEquals(calendars[0]._resourceID, resourceID)
         yield txn.commit()
+
+    @inlineCallbacks
+    def test_emptyTrashNonAscii(self):
+
+        from twistedcaldav.stdconfig import config
+        self.patch(config, "EnableTrashCollection", True)
+
+        data1 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:5CE3B280-DBC9-4E8E-B0B2-996754020E5F
+DTSTART;TZID=America/Los_Angeles:20141108T093000
+DTEND;TZID=America/Los_Angeles:20141108T103000
+CREATED:20141106T192546Z
+DTSTAMP:20141106T192546Z
+RRULE:FREQ=DAILY
+SEQUENCE:0
+SUMMARY:repeating évent
+TRANSP:OPAQUE
+END:VEVENT
+END:VCALENDAR
+"""
+
+        txn = self.store.newTransaction()
+        calendar = yield self._collectionForUser(txn, "user01", "tést-calendar", create=True)
+        calendar.properties()[PropertyName.fromElement(element.DisplayName)] = element.DisplayName.fromString("tést-calendar-name")
+
+        yield calendar.createObjectResourceWithName(
+            "tést-resource.ics",
+            Component.allFromString(data1)
+        )
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        resource = yield self._getResource(txn, "user01", "tést-calendar", "tést-resource.ics")
+        yield resource.remove()
+        names = yield self._getTrashNames(txn, "user01")
+        self.assertEquals(len(names), 1)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        yield emptyTrashForPrincipal(None, self.store, "user01", 0, txn=txn, verbose=True)
+        names = yield self._getTrashNames(txn, "user01")
+        self.assertEquals(len(names), 0)
+        result = yield txn.execSQL("select * from calendar_object", [])
+        self.assertEquals(len(result), 0)
+        yield txn.commit()
+
+        # Add event again, and this time remove the containing calendar
+        txn = self.store.newTransaction()
+        calendar = yield self._collectionForUser(txn, "user01", "tést-calendar")
+        yield calendar.createObjectResourceWithName(
+            "tést-resource.ics",
+            Component.allFromString(data1)
+        )
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        calendar = yield self._collectionForUser(txn, "user01", "tést-calendar")
+        result = yield txn.execSQL("select * from calendar_object", [])
+        yield calendar.remove()
+        home = yield self._homeForUser(txn, "user01")
+        trashedCollections = yield home.children(onlyInTrash=True)
+        self.assertEquals(len(trashedCollections), 1)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        yield emptyTrashForPrincipal(None, self.store, "user01", 0, txn=txn, verbose=True)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        home = yield self._homeForUser(txn, "user01")
+        trashedCollections = yield home.children(onlyInTrash=True)
+        self.assertEquals(len(trashedCollections), 0)
+        result = yield txn.execSQL("select * from calendar_object", [])
+        self.assertEquals(len(result), 0)
+        yield txn.commit()
+
+    @inlineCallbacks
+    def test_recoverTrashNonAscii(self):
+
+        from twistedcaldav.stdconfig import config
+        self.patch(config, "EnableTrashCollection", True)
+
+        data1 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:5CE3B280-DBC9-4E8E-B0B2-996754020E5F
+DTSTART;TZID=America/Los_Angeles:20141108T093000
+DTEND;TZID=America/Los_Angeles:20141108T103000
+CREATED:20141106T192546Z
+DTSTAMP:20141106T192546Z
+RRULE:FREQ=DAILY
+SEQUENCE:0
+SUMMARY:repeating évent
+TRANSP:OPAQUE
+END:VEVENT
+END:VCALENDAR
+"""
+
+        txn = self.store.newTransaction()
+        calendar = yield self._collectionForUser(txn, "user01", "tést-calendar", create=True)
+        calendar.properties()[PropertyName.fromElement(element.DisplayName)] = element.DisplayName.fromString("tést-calendar-name")
+
+        yield calendar.createObjectResourceWithName(
+            "tést-resource.ics",
+            Component.allFromString(data1)
+        )
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        resource = yield self._getResource(txn, "user01", "tést-calendar", "tést-resource.ics")
+        trashed_id = resource.id()
+        yield resource.remove()
+        names = yield self._getResourceNames(txn, "user01", "tést-calendar")
+        self.assertEquals(len(names), 0)
+        names = yield self._getTrashNames(txn, "user01")
+        self.assertEquals(len(names), 1)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        yield restoreTrashedEvent(None, self.store, "user01", trashed_id)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        names = yield self._getResourceNames(txn, "user01", "tést-calendar")
+        self.assertEquals(len(names), 1)
+        names = yield self._getTrashNames(txn, "user01")
+        self.assertEquals(len(names), 0)
+        yield txn.commit()
+
+        # This time remove the containing calendar
+        txn = self.store.newTransaction()
+        calendar = yield self._collectionForUser(txn, "user01", "tést-calendar")
+        trashed_id = calendar.id()
+        yield calendar.remove()
+        home = yield self._homeForUser(txn, "user01")
+        trashedCollections = yield home.children(onlyInTrash=True)
+        self.assertEquals(len(trashedCollections), 1)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        home = yield self._homeForUser(txn, "user01")
+        contents = yield home.getTrashContents()
+        self.assertEqual(len(contents["trashedcollections"]), 1)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        yield restoreTrashedCollection(None, self.store, "user01", trashed_id)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        home = yield self._homeForUser(txn, "user01")
+        trashedCollections = yield home.children(onlyInTrash=True)
+        self.assertEquals(len(trashedCollections), 0)
+        collections = yield home.children(onlyInTrash=False)
+        self.assertTrue(trashed_id in map(lambda x: x.id(), collections))
+        yield txn.commit()
