@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.carddav.datastore.test -*-
 ##
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,7 +36,7 @@ from twext.python.log import Logger
 from twext.python.filepath import CachingFilePath as FilePath
 from twext.enterprise.adbapi2 import ConnectionPool
 from twext.enterprise.ienterprise import AlreadyFinishedError, POSTGRES_DIALECT, \
-    ORACLE_DIALECT
+    ORACLE_DIALECT, DatabaseType
 from twext.enterprise.jobs.jobitem import JobItem
 from twext.enterprise.jobs.queue import ControllerQueue
 from twext.who.directory import DirectoryRecord
@@ -81,7 +81,6 @@ log = Logger()
 md5key = PropertyName.fromElement(TwistedGETContentMD5)
 
 
-
 def allInstancesOf(cls):
     """
     Use L{gc.get_referrers} to retrieve all instances of a given class.
@@ -91,7 +90,6 @@ def allInstancesOf(cls):
             yield o
 
 
-
 def dumpConnectionStatus():
     """
     Dump all L{DiagnosticConnectionWrapper} objects to standard output.  This
@@ -105,11 +103,11 @@ def dumpConnectionStatus():
     print("--- CONNECTIONS END ---")
 
 
-
 class SQLStoreBuilder(object):
     """
     Test-fixture-builder which can construct a PostgresStore.
     """
+
     def __init__(self, count=0, **options):
         self.sharedService = None
         self.currentTestID = None
@@ -120,7 +118,6 @@ class SQLStoreBuilder(object):
 
         self.options = options
 
-
     def createService(self, serviceFactory):
         """
         Create a L{PostgresService} to use for building a store.
@@ -148,7 +145,6 @@ class SQLStoreBuilder(object):
                 dsnUser=self.options.get("dsnUser"),
             )
 
-
     def childStore(self):
         """
         Create a store suitable for use in a child process, that is hooked up
@@ -162,8 +158,7 @@ class SQLStoreBuilder(object):
         cp = ConnectionPool(
             stubsvc.produceConnection,
             maxConnections=1,
-            dialect=DB_TYPE[0],
-            paramstyle=DB_TYPE[1],
+            dbtype=DatabaseType(DB_TYPE[0], DB_TYPE[1]),
         )
         # Attach the service to the running reactor.
         cp.startService()
@@ -177,7 +172,6 @@ class SQLStoreBuilder(object):
         )
         return cds
 
-
     def buildStore(
         self, testCase, notifierFactory,
         directoryService=None, homes=None, enableJobProcessing=True,
@@ -226,7 +220,6 @@ class SQLStoreBuilder(object):
         testCase.addCleanup(cleanUp)
         return result
 
-
     @inlineCallbacks
     def makeAndCleanStore(
         self, testCase, notifierFactory, directoryService,
@@ -251,8 +244,7 @@ class SQLStoreBuilder(object):
         cp = ConnectionPool(
             self.sharedService.produceConnection,
             maxConnections=4,
-            dialect=DB_TYPE[0],
-            paramstyle=DB_TYPE[1],
+            dbtype=DatabaseType(DB_TYPE[0], DB_TYPE[1]),
         )
         quota = deriveQuota(testCase)
         store = CommonDataStore(
@@ -308,7 +300,6 @@ class SQLStoreBuilder(object):
 
         returnValue(store)
 
-
     @inlineCallbacks
     def cleanStore(self, testCase, storeToClean):
 
@@ -324,7 +315,7 @@ class SQLStoreBuilder(object):
         # later table.  Therefore it's OK to drop them in the (reverse) order
         # that they happen to be in.
         tables = [
-            t.name for t in schema.model.tables #@UndefinedVariable
+            t.name for t in schema.model.tables  # @UndefinedVariable
             # All tables with rows _in_ the schema are populated
             # exclusively _by_ the schema and shouldn't be manipulated
             # while the server is running, so we leave those populated.
@@ -338,17 +329,17 @@ class SQLStoreBuilder(object):
                 log.failure("delete table {table} failed", table=table)
 
         # Change the starting values of sequences to random values
-        for sequence in schema.model.sequences: #@UndefinedVariable
+        for sequence in schema.model.sequences:  # @UndefinedVariable
             try:
-                if cleanupTxn.dialect == POSTGRES_DIALECT:
+                if cleanupTxn.dbtype.dialect == POSTGRES_DIALECT:
                     curval = (yield cleanupTxn.execSQL("select nextval('{}')".format(sequence.name), []))[0][0]
                     yield cleanupTxn.execSQL("select setval('{}', {})".format(sequence.name, curval + randint(1, 10000)), [])
-                elif cleanupTxn.dialect == ORACLE_DIALECT:
+                elif cleanupTxn.dbtype.dialect == ORACLE_DIALECT:
                     yield cleanupTxn.execSQL("alter sequence {} increment by {}".format(sequence.name, randint(1, 10000)), [])
                     yield cleanupTxn.execSQL("select {}.nextval from dual".format(sequence.name), [])
                     yield cleanupTxn.execSQL("alter sequence {} increment by {}".format(sequence.name, 1), [])
             except:
-                log.failure("setval sequence '{}' failed", sequence=sequence.name)
+                log.failure("setval sequence '{sequence}' failed", sequence=sequence.name)
         yield cleanupTxn.execSQL("update CALENDARSERVER set VALUE = '1' where NAME = 'MIN-VALID-REVISION'", [])
 
         yield cleanupTxn.commit()
@@ -366,7 +357,6 @@ cleanStore = theStoreBuilder.cleanStore
 _notSet = object()
 
 
-
 def deriveValue(testCase, attribute, computeDefault):
     """
     Derive a value for a specific test method, defined by L{withSpecialValue}
@@ -398,7 +388,6 @@ def deriveValue(testCase, attribute, computeDefault):
         return value
 
 
-
 def withSpecialValue(attribute, value):
     """
     Decorator for a test method which has a special value.  Should be used by
@@ -410,7 +399,6 @@ def withSpecialValue(attribute, value):
     return thunk
 
 
-
 def _computeDefaultQuota(testCase):
     """
     Compute a default value for quota in tests.
@@ -423,12 +411,10 @@ def _computeDefaultQuota(testCase):
     return baseline + fuzz
 
 
-
 _SPECIAL_QUOTA = "__special_quota__"
 _SPECIAL_TXN_CLEAN = "__special_txn_clean__"
 
 
-
 def deriveQuota(testCase):
     """
     Derive a distinctive quota number for a specific test, based on its ID.
@@ -453,7 +439,6 @@ def deriveQuota(testCase):
     return deriveValue(testCase, _SPECIAL_QUOTA, _computeDefaultQuota)
 
 
-
 def withSpecialQuota(quotaValue):
     """
     Test method decorator that will cause L{deriveQuota} to return a different
@@ -464,7 +449,6 @@ def withSpecialQuota(quotaValue):
     return withSpecialValue(_SPECIAL_QUOTA, quotaValue)
 
 
-
 def transactionClean(f=None):
     """
     Test method decorator that will cause L{buildStore} to check that no
@@ -481,7 +465,6 @@ def transactionClean(f=None):
         return decorator
 
 
-
 @inlineCallbacks
 def populateCalendarsFrom(requirements, store, migrating=False):
     """
@@ -538,7 +521,6 @@ def populateCalendarsFrom(requirements, store, migrating=False):
     yield populateTxn.commit()
 
 
-
 def updateToCurrentYear(data):
     """
     Update the supplied iCalendar data so that all dates are updated to the
@@ -554,7 +536,6 @@ def updateToCurrentYear(data):
     return data % subs
 
 
-
 @inlineCallbacks
 def resetCalendarMD5s(md5s, store):
     """
@@ -587,7 +568,6 @@ def resetCalendarMD5s(md5s, store):
     yield populateTxn.commit()
 
 
-
 @inlineCallbacks
 def populateAddressBooksFrom(requirements, store):
     """
@@ -626,7 +606,6 @@ def populateAddressBooksFrom(requirements, store):
     yield populateTxn.commit()
 
 
-
 @inlineCallbacks
 def resetAddressBookMD5s(md5s, store):
     """
@@ -661,7 +640,6 @@ def resetAddressBookMD5s(md5s, store):
     yield populateTxn.commit()
 
 
-
 def assertProvides(testCase, interface, provider):
     """
     Verify that C{provider} properly provides C{interface}
@@ -678,7 +656,6 @@ def assertProvides(testCase, interface, provider):
                       (provider, interface.__module__, interface.getName()))
 
 
-
 def buildTestDirectory(
     store, dataRoot, accounts=None, resources=None, augments=None,
     proxies=None, serversDB=None, cacheSeconds=0,
@@ -753,7 +730,8 @@ def buildTestDirectory(
     )
     augmentServiceInfo = ConfigDict(
         {
-            "type": "twistedcaldav.directory.augment.AugmentXMLDB",
+            "Enabled": True,
+            "type": "xml",
             "params": {
                 "xmlFiles": ["augments.xml", ],
                 "statSeconds": 15,
@@ -776,7 +754,6 @@ def buildTestDirectory(
     return directory
 
 
-
 class CommonCommonTests(object):
     """
     Common utility functionality for file/store combination tests.
@@ -786,7 +763,6 @@ class CommonCommonTests(object):
     savedStore = None
     assertProvides = assertProvides
 
-
     @inlineCallbacks
     def buildStoreAndDirectory(
         self, accounts=None, resources=None, augments=None, proxies=None,
@@ -823,7 +799,6 @@ class CommonCommonTests(object):
                     }
                 )
 
-
     def configure(self):
         """
         Modify the configuration to suit unit tests, with a mktemp-created
@@ -854,16 +829,14 @@ class CommonCommonTests(object):
 
         self.config = config
 
-
-    def buildStore(self, storeBuilder=theStoreBuilder):
+    def buildStore(self, storeBuilder=theStoreBuilder, notifierFactory=None):
         """
         Builds and returns a store
         """
 
         # Build the store before the directory; the directory will be assigned
         # to the store via setDirectoryService()
-        return storeBuilder.buildStore(self, self.notifierFactory, None)
-
+        return storeBuilder.buildStore(self, notifierFactory if notifierFactory is not None else self.notifierFactory, None)
 
     def transactionUnderTest(self, txn=None):
         """
@@ -875,7 +848,6 @@ class CommonCommonTests(object):
             self.lastTransaction = self.concurrentTransaction(txn)
         return self.lastTransaction
 
-
     def concurrentTransaction(self, txn=None):
         """
         Create a transaction from C{storeUnderTest} and save it for later
@@ -904,7 +876,6 @@ class CommonCommonTests(object):
 
         return txn
 
-
     def commit(self):
         """
         Commit the last transaction created from C{transactionUnderTest}, and
@@ -914,7 +885,6 @@ class CommonCommonTests(object):
         self.lastTransaction = None
         return result
 
-
     def abort(self):
         """
         Abort the last transaction created from C{transactionUnderTest}, and
@@ -924,14 +894,12 @@ class CommonCommonTests(object):
         self.lastTransaction = None
         return result
 
-
     def storeUnderTest(self):
         """
         Create and return the L{CalendarStore} for testing.
         """
         return self.store
 
-
     def homeUnderTest(self, txn=None, name="home1", status=None, create=False):
         """
         Get the calendar home detailed by C{requirements['home1']}.
@@ -940,7 +908,6 @@ class CommonCommonTests(object):
             txn = self.transactionUnderTest()
         return txn.calendarHomeWithUID(name, status=status, create=create)
 
-
     @inlineCallbacks
     def calendarUnderTest(self, txn=None, name="calendar_1", home="home1", status=None):
         """
@@ -950,7 +917,6 @@ class CommonCommonTests(object):
         calendar = yield home.calendarWithName(name)
         returnValue(calendar)
 
-
     @inlineCallbacks
     def calendarObjectUnderTest(
         self, txn=None, name="1.ics", calendar_name="calendar_1", home="home1", status=None
@@ -965,7 +931,6 @@ class CommonCommonTests(object):
         object = yield calendar.calendarObjectWithName(name)
         returnValue(object)
 
-
     @inlineCallbacks
     def trashObjectsUnderTest(
         self, txn=None, home="home1", status=None
@@ -979,7 +944,6 @@ class CommonCommonTests(object):
         objects = yield trash.objectResources()
         returnValue(objects)
 
-
     def addressbookHomeUnderTest(self, txn=None, name="home1"):
         """
         Get the addressbook home detailed by C{requirements['home1']}.
@@ -988,7 +952,6 @@ class CommonCommonTests(object):
             txn = self.transactionUnderTest()
         return txn.addressbookHomeWithUID(name)
 
-
     @inlineCallbacks
     def addressbookUnderTest(self, txn=None, name="addressbook", home="home1"):
         """
@@ -999,7 +962,6 @@ class CommonCommonTests(object):
         addressbook = yield home.addressbookWithName(name)
         returnValue(addressbook)
 
-
     @inlineCallbacks
     def addressbookObjectUnderTest(
         self, txn=None, name="1.vcf",
@@ -1015,19 +977,16 @@ class CommonCommonTests(object):
         object = yield addressBook.addressbookObjectWithName(name)
         returnValue(object)
 
-
     def notificationCollectionUnderTest(self, txn=None, name="home1", status=None, create=False):
         if txn is None:
             txn = self.transactionUnderTest()
         return txn.notificationsWithUID(name, status=status, create=create)
 
-
     def userRecordWithShortName(self, shortname):
         return self.directory.recordWithShortName(
             self.directory.recordType.user, shortname
         )
 
-
     @inlineCallbacks
     def userUIDFromShortName(self, shortname):
         record = yield self.directory.recordWithShortName(
@@ -1035,16 +994,13 @@ class CommonCommonTests(object):
         )
         returnValue(record.uid if record is not None else None)
 
-
     def addRecordFromFields(self, fields):
         updatedRecord = DirectoryRecord(self.directory, fields)
         return self.directory.updateRecords((updatedRecord,), create=True)
 
-
     def removeRecord(self, uid):
         return self.directory.removeRecords([uid])
 
-
     def changeRecord(self, record, fieldname, value, directory=None):
         if directory is None:
             directory = self.directory
@@ -1054,7 +1010,6 @@ class CommonCommonTests(object):
         return directory.updateRecords((updatedRecord,))
 
 
-
 class StubNotifierFactory(object):
     """
     For testing push notifications without an XMPP server.
@@ -1064,24 +1019,19 @@ class StubNotifierFactory(object):
         self.reset()
         self.hostname = "example.com"
 
-
     def newNotifier(self, storeObject):
         return Notifier(self, storeObject)
 
-
     def pushKeyForId(self, prefix, id):
         return "/%s/%s/%s/" % (prefix, self.hostname, id)
 
-
     def send(self, prefix, id, txn, priority):
         self.history.append((self.pushKeyForId(prefix, id), priority))
 
-
     def reset(self):
         self.history = []
 
 
-
 def disableMemcacheForTest(aTest):
     """
     Disable all memcache logic for the duration of a test; we shouldn't be
