@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2014-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2014-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -44,7 +44,8 @@ from txdav.dps.commands import (
     WikiAccessForUIDCommand, ContinuationCommand,
     StatsCommand, ExternalDelegatesCommand, ExpandedMemberUIDsCommand,
     AddMembersCommand, RemoveMembersCommand,
-    UpdateRecordsCommand, ExpandedMembersCommand, FlushCommand
+    UpdateRecordsCommand, ExpandedMembersCommand, FlushCommand,
+    SetAutoScheduleModeCommand, ContainsUIDsCommand
 )
 from txdav.who.delegates import RecordType as DelegatesRecordType
 from txdav.who.directory import (
@@ -63,7 +64,6 @@ log = Logger()
 #
 
 
-
 # MOVE2WHO TODOs:
 # LDAP
 # Store based directory service (records in the store, i.e.
@@ -91,7 +91,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
          txdav.who.augment.FieldName)
     )
 
-
     def _dictToRecord(self, serializedFields):
         """
         Turn a dictionary of fields sent from the server into a directory
@@ -127,7 +126,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
         # print("AFTER:", fields)
         return DirectoryRecord(self, fields)
 
-
     def _processSingleRecord(self, result):
         """
         Takes a dictionary with a "fields" key whose value is a pickled
@@ -136,7 +134,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
         serializedFields = pickle.loads(result['fields'])
         return self._dictToRecord(serializedFields)
 
-
     def _processMultipleRecords(self, result):
         """
         Takes a dictionary with a "items" key whose value is an iterable
@@ -154,7 +151,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
                 results.append(record)
         return results
 
-
     @inlineCallbacks
     def _getConnection(self):
 
@@ -168,7 +164,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             self._connection = connection
         returnValue(self._connection)
 
-
     @inlineCallbacks
     def _sendCommand(self, command, **kwds):
         """
@@ -188,7 +183,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             raise
         returnValue(results)
 
-
     def _logResultTiming(self, command, startTime, results):
         duration = time.time() - startTime
         numResults = 0
@@ -201,7 +195,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             command=command, duration=1000.0 * duration, numResults=numResults
         )
 
-
     @inlineCallbacks
     def _call(self, command, postProcess, **kwds):
         """
@@ -243,7 +236,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
         self._logResultTiming(command, startTime, results)
         returnValue(postProcess(results))
 
-
     def recordWithShortName(self, recordType, shortName, timeoutSeconds=None):
         # MOVE2WHO
         # temporary hack until we can fix all callers not to pass strings:
@@ -268,7 +260,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             **kwds
         )
 
-
     def recordWithUID(self, uid, timeoutSeconds=None):
         # MOVE2WHO, REMOVE THIS:
         if not isinstance(uid, unicode):
@@ -287,7 +278,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             **kwds
         )
 
-
     def recordWithGUID(self, guid, timeoutSeconds=None):
         kwds = {
             "guid": str(guid),
@@ -301,7 +291,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             **kwds
         )
 
-
     def recordsWithRecordType(
         self, recordType, limitResults=None, timeoutSeconds=None
     ):
@@ -318,7 +307,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             **kwds
         )
 
-
     def recordsWithEmailAddress(
         self, emailAddress, limitResults=None, timeoutSeconds=None
     ):
@@ -336,7 +324,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             **kwds
         )
 
-
     def recordsMatchingTokens(
         self, tokens, context=None, limitResults=None, timeoutSeconds=None
     ):
@@ -355,7 +342,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             **kwds
         )
 
-
     def recordsMatchingFields(
         self, fields, operand=Operand.OR, recordType=None,
         limitResults=None, timeoutSeconds=None
@@ -387,28 +373,24 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
         if timeoutSeconds is not None:
             kwds["timeoutSeconds"] = timeoutSeconds
 
-
         return self._call(
             RecordsMatchingFieldsCommand,
             self._processMultipleRecords,
             **kwds
         )
 
-
     def recordsWithDirectoryBasedDelegates(self):
         return self._call(
             ExternalDelegatesCommand,
             self._processMultipleRecords
         )
 
-
     def recordsFromExpression(self, expression, recordTypes=None):
         raise NotImplementedError(
             "This won't work until expressions are serializable to send "
             "across AMP"
         )
 
-
     def updateRecords(self, records, create=False):
         # FIXME: cannot create right now
         if create:
@@ -421,6 +403,12 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             create=False,
         )
 
+    def setAutoScheduleMode(self, record, autoScheduleMode):
+        return self._sendCommand(
+            SetAutoScheduleModeCommand,
+            uid=record.uid.encode("utf-8"),
+            autoScheduleMode=autoScheduleMode.name,
+        )
 
     @inlineCallbacks
     def flush(self):
@@ -430,7 +418,6 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
         except ConnectError:
             returnValue(None)
 
-
     @inlineCallbacks
     def stats(self):
         try:
@@ -440,20 +427,19 @@ class DirectoryService(BaseDirectoryService, CalendarDirectoryServiceMixin):
             returnValue({})
 
 
-
 @implementer(ICalendarStoreDirectoryRecord)
 class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
 
-
     def verifyPlaintextPassword(self, password):
+        if isinstance(password, unicode):
+            password = password.encode("utf-8")
         return self.service._call(
             VerifyPlaintextPasswordCommand,
             lambda x: x['authenticated'],
             uid=self.uid.encode("utf-8"),
-            password=password.encode("utf-8")
+            password=password,
         )
 
-
     def verifyHTTPDigest(
         self, username, realm, uri, nonce, cnonce,
         algorithm, nc, qop, response, method,
@@ -474,7 +460,6 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
             method=method.encode("utf-8"),
         )
 
-
     def members(self):
         if self.recordType in (
             RecordType.group,
@@ -491,7 +476,6 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
         else:
             return succeed([])
 
-
     def expandedMembers(self):
         """
         Only allowed for delegates - not regular groups.
@@ -510,7 +494,6 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
         else:
             return succeed([])
 
-
     def groups(self):
         return self.service._call(
             GroupsCommand,
@@ -518,7 +501,6 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
             uid=self.uid.encode("utf-8")
         )
 
-
     def addMembers(self, members):
         log.debug("DPS Client addMembers")
         memberUIDs = [m.uid.encode("utf-8") for m in members]
@@ -529,7 +511,6 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
             memberUIDs=memberUIDs
         )
 
-
     def removeMembers(self, members):
         log.debug("DPS Client removeMembers")
         memberUIDs = [m.uid.encode("utf-8") for m in members]
@@ -540,7 +521,6 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
             memberUIDs=memberUIDs
         )
 
-
     def setMembers(self, members):
         log.debug("DPS Client setMembers")
         memberUIDs = [m.uid.encode("utf-8") for m in members]
@@ -551,14 +531,12 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
             memberUIDs=memberUIDs
         )
 
-
     def _convertUIDs(self, results):
         uids = []
         for uid in results["items"]:
             uids.append(uid.decode("utf-8"))
         return uids
 
-
     def expandedMemberUIDs(self):
         log.debug("DPS Client expandedMemberUIDs")
         if self.recordType is RecordType.group:
@@ -570,12 +548,28 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
         else:
             return succeed([])
 
+    def containsUID(self, uid):
+        """
+        Is the supplied UID an expanded member of this proxy group.
+
+        @param uid: UID to test
+        @type uid: L{str}
+
+        @return: result
+        @rtype: L{bool}
+        """
+        log.debug("DPS Client containsUID")
+        return self.service._call(
+            ContainsUIDsCommand,
+            lambda x: x['result'],
+            uid=self.uid.encode("utf-8"),
+            testUid=uid.encode("utf-8")
+        )
 
     def _convertAccess(self, results):
         access = results["access"].decode("utf-8")
         return txdav.who.wiki.WikiAccessLevel.lookupByName(access)
 
-
     def accessForRecord(self, record):
         log.debug("DPS Client accessForRecord")
         return self.service._call(
@@ -586,7 +580,6 @@ class DirectoryRecord(BaseDirectoryRecord, CalendarDirectoryRecordMixin):
         )
 
 
-
 # Test client:
 
 @inlineCallbacks
@@ -618,13 +611,11 @@ def makeEvenBetterRequest():
     # print("emailAddress: {r}".format(r=records))
 
 
-
 def succeeded(result):
     print("yay")
     reactor.stop()
 
 
-
 def failed(failure):
     print("boo: {f}".format(f=failure))
     reactor.stop()
