@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.who.test.test_augment -*-
 ##
-# Copyright (c) 2013-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2013-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,7 +33,7 @@ from twext.python.log import Logger
 from twext.who.directory import DirectoryRecord
 from twext.who.directory import DirectoryService as BaseDirectoryService
 from twext.who.idirectory import (
-    IDirectoryService, RecordType, FieldName as BaseFieldName
+    IDirectoryService, RecordType, FieldName as BaseFieldName, NotAllowedError
 )
 from twext.who.util import ConstantsContainer
 
@@ -48,7 +48,6 @@ from txdav.who.idirectory import (
 log = Logger()
 
 
-
 def timed(f):
     """
     A decorator which keeps track of the wrapped function's call count and
@@ -71,7 +70,6 @@ def timed(f):
         AugmentedDirectoryService._addTiming(key, time.time() - startTime)
         return result
 
-
     def timingWrapper(self, *args, **kwds):
         """
         Records the start time of the call and the method's name
@@ -84,7 +82,6 @@ def timed(f):
     return timingWrapper
 
 
-
 @implementer(IDirectoryService, IStoreDirectoryService)
 class AugmentedDirectoryService(
     BaseDirectoryService, CalendarDirectoryServiceMixin
@@ -103,7 +100,6 @@ class AugmentedDirectoryService(
 
     _timings = {}
 
-
     def __init__(self, directory, store, augmentDB):
         BaseDirectoryService.__init__(self, directory.realmName)
         self._directory = directory
@@ -114,7 +110,6 @@ class AugmentedDirectoryService(
         # This is assigned in buildDirectory()
         self._ldapDS = None
 
-
     @classmethod
     def _addTiming(cls, key, duration):
         if key not in cls._timings:
@@ -124,11 +119,9 @@ class AugmentedDirectoryService(
         timeSpent += duration
         cls._timings[key] = (count, timeSpent)
 
-
     def flush(self):
         return self._directory.flush()
 
-
     def stats(self):
         results = {}
         results.update(self._timings)
@@ -139,18 +132,15 @@ class AugmentedDirectoryService(
 
         return succeed(results)
 
-
     @property
     def recordType(self):
         # Defer to the directory service we're augmenting
         return self._directory.recordType
 
-
     def recordTypes(self):
         # Defer to the directory service we're augmenting
         return self._directory.recordTypes()
 
-
     @inlineCallbacks
     def recordsFromExpression(
         self, expression, recordTypes=None,
@@ -166,7 +156,6 @@ class AugmentedDirectoryService(
             augmented.append(record)
         returnValue(augmented)
 
-
     @inlineCallbacks
     def recordsWithFieldValue(
         self, fieldName, value, limitResults=None, timeoutSeconds=None
@@ -181,7 +170,6 @@ class AugmentedDirectoryService(
             augmented.append(record)
         returnValue(augmented)
 
-
     @timed
     @inlineCallbacks
     def recordWithUID(self, uid, timeoutSeconds=None):
@@ -196,7 +184,6 @@ class AugmentedDirectoryService(
         record = yield self._augment(record)
         returnValue(record)
 
-
     @timed
     @inlineCallbacks
     def recordWithGUID(self, guid, timeoutSeconds=None):
@@ -206,7 +193,6 @@ class AugmentedDirectoryService(
         record = yield self._augment(record)
         returnValue(record)
 
-
     @timed
     @inlineCallbacks
     def recordsWithRecordType(
@@ -221,7 +207,6 @@ class AugmentedDirectoryService(
             augmented.append(record)
         returnValue(augmented)
 
-
     @timed
     @inlineCallbacks
     def recordWithShortName(self, recordType, shortName, timeoutSeconds=None):
@@ -236,7 +221,6 @@ class AugmentedDirectoryService(
         record = yield self._augment(record)
         returnValue(record)
 
-
     @timed
     @inlineCallbacks
     def recordsWithEmailAddress(
@@ -257,28 +241,24 @@ class AugmentedDirectoryService(
             augmented.append(record)
         returnValue(augmented)
 
-
     @timed
     def recordWithCalendarUserAddress(self, *args, **kwds):
         return CalendarDirectoryServiceMixin.recordWithCalendarUserAddress(
             self, *args, **kwds
         )
 
-
     @timed
     def recordsMatchingTokens(self, *args, **kwds):
         return CalendarDirectoryServiceMixin.recordsMatchingTokens(
             self, *args, **kwds
         )
 
-
     @timed
     def recordsMatchingFields(self, *args, **kwds):
         return CalendarDirectoryServiceMixin.recordsMatchingFields(
             self, *args, **kwds
         )
 
-
     @timed
     @inlineCallbacks
     def updateRecords(self, records, create=False):
@@ -334,8 +314,10 @@ class AugmentedDirectoryService(
 
         # Apply the base records
         if baseRecords:
-            yield self._directory.updateRecords(baseRecords, create=create)
-
+            try:
+                yield self._directory.updateRecords(baseRecords, create=create)
+            except NotAllowedError:
+                pass
 
     def _splitFields(self, record):
         """
@@ -359,13 +341,11 @@ class AugmentedDirectoryService(
 
         return (baseFields, augmentFields)
 
-
     @inlineCallbacks
     def removeRecords(self, uids):
         yield self._augmentDB.removeAugmentRecords(uids)
         yield self._directory.removeRecords(uids)
 
-
     def _assignToField(self, fields, name, value):
         """
         Assign a value to a field only if not already present in fields.
@@ -374,7 +354,6 @@ class AugmentedDirectoryService(
         if field not in fields:
             fields[field] = value
 
-
     @inlineCallbacks
     def _augment(self, record):
         if record is None:
@@ -454,7 +433,6 @@ class AugmentedDirectoryService(
                 augmentRecord.serverID.decode("utf-8")
             )
 
-
         else:
             self._assignToField(fields, "hasCalendars", False)
             self._assignToField(fields, "hasContacts", False)
@@ -467,6 +445,24 @@ class AugmentedDirectoryService(
 
         returnValue(augmentedRecord)
 
+    @inlineCallbacks
+    def setAutoScheduleMode(self, record, autoScheduleMode):
+        augmentRecord = yield self._augmentDB.getAugmentRecord(
+            record.uid,
+            self.recordTypeToOldName(record.recordType)
+        )
+        if augmentRecord is not None:
+            autoScheduleMode = {
+                AutoScheduleMode.none: "none",
+                AutoScheduleMode.accept: "accept-always",
+                AutoScheduleMode.decline: "decline-always",
+                AutoScheduleMode.acceptIfFree: "accept-if-free",
+                AutoScheduleMode.declineIfBusy: "decline-if-busy",
+                AutoScheduleMode.acceptIfFreeDeclineIfBusy: "automatic",
+            }.get(autoScheduleMode)
+
+            augmentRecord.autoScheduleMode = autoScheduleMode
+            yield self._augmentDB.addAugmentRecords([augmentRecord])
 
 
 class AugmentedDirectoryRecord(DirectoryRecord, CalendarDirectoryRecordMixin):
@@ -476,9 +472,9 @@ class AugmentedDirectoryRecord(DirectoryRecord, CalendarDirectoryRecordMixin):
 
     def __init__(self, service, baseRecord, augmentedFields):
         DirectoryRecord.__init__(self, service, augmentedFields)
+        CalendarDirectoryRecordMixin.__init__(self)
         self._baseRecord = baseRecord
 
-
     @timed
     @inlineCallbacks
     def members(self):
@@ -490,19 +486,15 @@ class AugmentedDirectoryRecord(DirectoryRecord, CalendarDirectoryRecordMixin):
 
         returnValue(augmented)
 
-
     def addMembers(self, memberRecords):
         return self._baseRecord.addMembers(memberRecords)
 
-
     def removeMembers(self, memberRecords):
         return self._baseRecord.removeMembers(memberRecords)
 
-
     def setMembers(self, memberRecords):
         return self._baseRecord.setMembers(memberRecords)
 
-
     @timed
     @inlineCallbacks
     def groups(self):
@@ -525,17 +517,14 @@ class AugmentedDirectoryRecord(DirectoryRecord, CalendarDirectoryRecordMixin):
 
         returnValue(augmented)
 
-
     @timed
     def verifyPlaintextPassword(self, password):
         return self._baseRecord.verifyPlaintextPassword(password)
 
-
     @timed
     def verifyHTTPDigest(self, *args):
         return self._baseRecord.verifyHTTPDigest(*args)
 
-
     @timed
     def accessForRecord(self, record):
         return self._baseRecord.accessForRecord(record)
