@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.who.test.test_delegates -*-
 ##
-# Copyright (c) 2013-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2013-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -39,7 +39,6 @@ from twext.who.expression import MatchExpression, MatchType
 log = Logger()
 
 
-
 class RecordType(Names):
     """
     Constants for read-only delegates and read-write delegate groups
@@ -58,16 +57,15 @@ class RecordType(Names):
     writeDelegatorGroup.description = u"write-delegator-group"
 
 
-
 class DirectoryRecord(BaseDirectoryRecord):
 
     @inlineCallbacks
-    def members(self, expanded=False):
+    def _membersUIDs(self, expanded=False):
         """
-        If this is a readDelegateGroup or writeDelegateGroup, the members
-        will consist of the records who are delegates *of* this record.
+        If this is a readDelegateGroup or writeDelegateGroup, the result
+        will consist of the UIDs who are delegates *of* this record.
         If this is a readDelegatorGroup or writeDelegatorGroup,
-        the members will consist of the records who have delegated *to*
+        the results will consist of the UIDs who have delegated *to*
         this record.
         """
         parentUID, _ignore_proxyType = self.uid.split(u"#")
@@ -90,6 +88,20 @@ class DirectoryRecord(BaseDirectoryRecord):
             "DirectoryRecord.members", _members
         )
 
+        returnValue(delegateUIDs)
+
+    @inlineCallbacks
+    def members(self, expanded=False):
+        """
+        If this is a readDelegateGroup or writeDelegateGroup, the members
+        will consist of the records who are delegates *of* this record.
+        If this is a readDelegatorGroup or writeDelegatorGroup,
+        the members will consist of the records who have delegated *to*
+        this record.
+        """
+        parentUID, _ignore_proxyType = self.uid.split(u"#")
+        delegateUIDs = yield self._membersUIDs(expanded=expanded)
+
         records = []
         for uid in delegateUIDs:
             if uid != parentUID:
@@ -99,11 +111,9 @@ class DirectoryRecord(BaseDirectoryRecord):
 
         returnValue(records)
 
-
     def expandedMembers(self):
         return self.members(expanded=True)
 
-
     @inlineCallbacks
     def setMembers(self, memberRecords):
         """
@@ -137,6 +147,19 @@ class DirectoryRecord(BaseDirectoryRecord):
             "DirectoryRecord.setMembers", _setMembers
         )
 
+    @inlineCallbacks
+    def containsUID(self, uid):
+        """
+        Is the supplied UID an expanded member of this proxy group.
+
+        @param uid: UID to test
+        @type uid: L{str}
+
+        @return: result
+        @rtype: L{bool}
+        """
+        delegateUIDs = yield self._membersUIDs(expanded=True)
+        returnValue(uid in delegateUIDs)
 
 
 def recordTypeToProxyType(recordType):
@@ -148,7 +171,6 @@ def recordTypeToProxyType(recordType):
     }.get(recordType, None)
 
 
-
 def proxyTypeToRecordType(proxyType):
     return {
         "calendar-proxy-read": RecordType.readDelegateGroup,
@@ -158,7 +180,6 @@ def proxyTypeToRecordType(proxyType):
     }.get(proxyType, None)
 
 
-
 class DirectoryService(BaseDirectoryService):
     """
     Delegate directory service
@@ -166,17 +187,14 @@ class DirectoryService(BaseDirectoryService):
 
     recordType = RecordType
 
-
     def __init__(self, realmName, store):
         BaseDirectoryService.__init__(self, realmName)
         self._store = store
         self._masterDirectory = None
 
-
     def setMasterDirectory(self, masterDirectory):
         self._masterDirectory = masterDirectory
 
-
     def recordWithShortName(self, recordType, shortName, timeoutSeconds=None):
         uid = shortName + "#" + recordTypeToProxyType(recordType)
 
@@ -187,7 +205,6 @@ class DirectoryService(BaseDirectoryService):
         })
         return succeed(record)
 
-
     def recordWithUID(self, uid, timeoutSeconds=None):
         if "#" not in uid:  # Not a delegate group uid
             return succeed(None)
@@ -199,7 +216,6 @@ class DirectoryService(BaseDirectoryService):
             recordType, uid, timeoutSeconds=timeoutSeconds
         )
 
-
     @inlineCallbacks
     def recordsFromExpression(
         self, expression, recordTypes=None, records=None,
@@ -227,12 +243,13 @@ class DirectoryService(BaseDirectoryService):
         returnValue(())
 
 
-
 class CachingDelegates(object):
     """
     Manages access to the store's delegates API, including caching of results.
     """
 
+    cacheNotifier = None
+
     class DelegatesMemcacher(Memcacher):
 
         def __init__(self, namespace):
@@ -298,11 +315,9 @@ class CachingDelegates(object):
             """
             yield self.delete(self._membershipsKey(uid, readWrite))
 
-
     def __init__(self):
         self._memcacher = CachingDelegates.DelegatesMemcacher("DelegatesDB")
 
-
     @inlineCallbacks
     def setDelegates(self, txn, delegator, delegates, readWrite):
         """
@@ -330,7 +345,6 @@ class CachingDelegates(object):
         else:
             yield self._podSetDelegates(txn, delegator, delegates, readWrite)
 
-
     @inlineCallbacks
     def addDelegate(self, txn, delegator, delegate, readWrite):
         """
@@ -358,6 +372,11 @@ class CachingDelegates(object):
         else:
             yield txn.addDelegate(delegator.uid, delegate.uid, readWrite)
 
+        # Make sure notifications are sent
+        if self.cacheNotifier is not None:
+            yield self.cacheNotifier.changed("/principals/__uids__/{}/".format(delegator.uid))
+            yield self.cacheNotifier.changed("/principals/__uids__/{}/".format(delegate.uid))
+
         # Update cache (remove the member cache entry first as we need to recalculate it for
         # memberships removal)
         yield self._memcacher.deleteMember(delegator.uid, readWrite)
@@ -365,7 +384,6 @@ class CachingDelegates(object):
         for uid in set(newDelegateUIDs) - set(existingDelegateUIDs):
             yield self._memcacher.deleteMembership(uid, readWrite)
 
-
     @inlineCallbacks
     def removeDelegate(self, txn, delegator, delegate, readWrite):
         """
@@ -393,6 +411,11 @@ class CachingDelegates(object):
         else:
             yield txn.removeDelegate(delegator.uid, delegate.uid, readWrite)
 
+        # Make sure notifications are sent
+        if self.cacheNotifier is not None:
+            yield self.cacheNotifier.changed("/principals/__uids__/{}/".format(delegator.uid))
+            yield self.cacheNotifier.changed("/principals/__uids__/{}/".format(delegate.uid))
+
         # Update cache (remove the member cache entry first as we need to recalculate it for
         # memberships removal)
         yield self._memcacher.deleteMember(delegator.uid, readWrite)
@@ -400,7 +423,6 @@ class CachingDelegates(object):
         for uid in set(existingDelegateUIDs) - set(newDelegateUIDs):
             yield self._memcacher.deleteMembership(uid, readWrite)
 
-
     @inlineCallbacks
     def groupChanged(self, txn, groupID, addedUIDs, removedUIDs):
         """
@@ -429,7 +451,6 @@ class CachingDelegates(object):
             yield self._memcacher.deleteMembership(delegate, True)
             yield self._memcacher.deleteMembership(delegate, False)
 
-
     @inlineCallbacks
     def delegatesOf(self, txn, delegator, readWrite, expanded=False):
         """
@@ -454,7 +475,6 @@ class CachingDelegates(object):
                     records.append(record)
         returnValue(records)
 
-
     @inlineCallbacks
     def delegatedTo(self, txn, delegate, readWrite):
         """
@@ -479,7 +499,6 @@ class CachingDelegates(object):
                     records.append(record)
         returnValue(records)
 
-
     @inlineCallbacks
     def _delegatesOfUIDs(self, txn, delegator, readWrite, expanded=False):
         """
@@ -500,11 +519,11 @@ class CachingDelegates(object):
         # Try cache first
         delegateUIDs = yield self._memcacher.getMembers(delegator.uid, readWrite, expanded)
         if delegateUIDs is not None:
-            log.debug("_delegatesOfUIDs cached for: {} and read-write = {} and expanded = {}".format(delegator.uid, readWrite, expanded,))
+            log.debug("_delegatesOfUIDs cached for: {uid} and read-write = {rw} and expanded = {expanded}", uid=delegator.uid, rw=readWrite, expanded=expanded)
             returnValue(delegateUIDs)
 
         # Get from the store
-        log.debug("_delegatesOfUIDs for: {} and read-write = {} and expanded = {}".format(delegator.uid, readWrite, expanded,))
+        log.debug("_delegatesOfUIDs for: {uid} and read-write = {rw} and expanded = {expanded}", uid=delegator.uid, rw=readWrite, expanded=expanded)
         if delegator.thisServer():
             delegateUIDs = yield txn.delegates(delegator.uid, readWrite, expanded=expanded)
 
@@ -515,7 +534,6 @@ class CachingDelegates(object):
 
         returnValue(delegateUIDs)
 
-
     @inlineCallbacks
     def _delegatedToUIDs(self, txn, delegate, readWrite, onlyThisServer=False):
         """
@@ -539,11 +557,11 @@ class CachingDelegates(object):
         # Try cache first
         delegatorUIDs = yield self._memcacher.getMemberships(delegate.uid, readWrite)
         if delegatorUIDs is not None:
-            log.debug("_delegatedToUIDs cached for: {} and read-write = {}".format(delegate.uid, readWrite,))
+            log.debug("_delegatedToUIDs cached for: {uid} and read-write = {rw}", uid=delegate.uid, rw=readWrite)
             returnValue(delegatorUIDs)
 
         # Get from the store
-        log.debug("_delegatedToUIDs for: {} and read-write = {}".format(delegate.uid, readWrite,))
+        log.debug("_delegatedToUIDs for: {uid} and read-write = {rw}", uid=delegate.uid, rw=readWrite)
         delegatorUIDs = (yield txn.delegators(delegate.uid, readWrite))
         if not onlyThisServer and config.Servers.Enabled:
             delegatorUIDs.update((yield self._podDelegators(txn, delegate, readWrite)))
@@ -553,7 +571,6 @@ class CachingDelegates(object):
 
         returnValue(delegatorUIDs)
 
-
     def _podSetDelegates(self, txn, delegator, delegates, readWrite):
         """
         Sets the full set of delegates for a delegator.
@@ -568,8 +585,10 @@ class CachingDelegates(object):
         @param readWrite: if True, read and write access is granted; read-only
             access otherwise
         """
-        return txn.store().conduit.send_set_delegates(txn, delegator, delegates, readWrite)
-
+        if delegator.server().v5:
+            return succeed(None)
+        else:
+            return txn.store().conduit.send_set_delegates(txn, delegator, delegates, readWrite)
 
     def _podDelegates(self, txn, delegator, readWrite, expanded=False):
         """
@@ -583,9 +602,11 @@ class CachingDelegates(object):
         @rtype: a Deferred which fires a set of L{str}
         """
 
-        log.debug("_podDelegates for: {} and read-write = {} and expanded = {}".format(delegator.uid, readWrite, expanded,))
-        return txn.store().conduit.send_get_delegates(txn, delegator, readWrite, expanded)
-
+        log.debug("_podDelegates for: {uid} and read-write = {rw} and expanded = {expanded}", uid=delegator.uid, rw=readWrite, expanded=expanded)
+        if delegator.server().v5:
+            return succeed(set())
+        else:
+            return txn.store().conduit.send_get_delegates(txn, delegator, readWrite, expanded)
 
     @inlineCallbacks
     def _podDelegators(self, txn, delegate, readWrite):
@@ -601,11 +622,15 @@ class CachingDelegates(object):
         @rtype: a Deferred which fires a set of L{str}
         """
 
-        log.debug("_podDelegators for: {} and read-write = {}".format(delegate.uid, readWrite,))
-        results = yield DeferredList([
-            txn.store().conduit.send_get_delegators(txn, server, delegate, readWrite) for
-            server in txn.directoryService().serversDB().allServersExceptThis()
-        ], consumeErrors=True)
+        log.debug("_podDelegators for: {uid} and read-write = {rw}", uid=delegate.uid, rw=readWrite)
+        otherServers = txn.directoryService().serversDB().allServersExceptThis(filter_v5=True)
+        if len(otherServers) != 0:
+            results = yield DeferredList([
+                txn.store().conduit.send_get_delegators(txn, server, delegate, readWrite) for
+                server in txn.directoryService().serversDB().allServersExceptThis()
+            ], consumeErrors=True)
+        else:
+            results = []
         delegators = set()
         for result in results:
             if result and result[0]:
