@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ from twext.who.expression import (
     MatchType, Operand, MatchExpression, CompoundExpression, MatchFlags,
     ExistsExpression, BooleanExpression
 )
-from twext.who.idirectory import RecordType as BaseRecordType
+from twext.who.idirectory import RecordType as BaseRecordType, FieldName as BaseFieldName
 from twisted.cred.credentials import UsernamePassword
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twistedcaldav.config import config
@@ -48,7 +48,6 @@ __all__ = [
 ]
 
 
-
 class CalendarDirectoryServiceMixin(object):
 
     guid = "1332A615-4D3A-41FE-B636-FBE25BFB982E"
@@ -58,11 +57,9 @@ class CalendarDirectoryServiceMixin(object):
     def serversDB(self):
         return self._serversDB
 
-
     def setServersDB(self, serversDB):
         self._serversDB = serversDB
 
-
     # Must maintain the hack for a bit longer:
     def setPrincipalCollection(self, principalCollection):
         """
@@ -73,7 +70,6 @@ class CalendarDirectoryServiceMixin(object):
         """
         self.principalCollection = principalCollection
 
-
     @inlineCallbacks
     def recordWithCalendarUserAddress(
         self, address, timeoutSeconds=None
@@ -81,6 +77,17 @@ class CalendarDirectoryServiceMixin(object):
         address = normalizeCUAddr(address)
         record = None
 
+        if config.Scheduling.Options.FakeResourceLocationEmail:
+            if address.startswith("mailto:") and address.endswith("@do_not_reply"):
+                try:
+                    address = "urn:x-uid:{}".format(address[7:-13].decode("hex"))
+                except Exception:
+                    try:
+                        address = "urn:uuid:{}".format(address[7:-13])
+                    except Exception as e:
+                        log.error("Invalid @do_not_reply cu-address: '{address}' {exc}", address=address, exc=e)
+                        address = ""
+
         if address.startswith("urn:x-uid:"):
             uid = address[10:]
             record = yield self.recordWithUID(
@@ -132,7 +139,6 @@ class CalendarDirectoryServiceMixin(object):
     searchContext_group = "group"
     searchContext_attendee = "attendee"
 
-
     def recordTypesForSearchContext(self, context):
         """
         Map calendarserver-principal-search REPORT context value to applicable record types
@@ -160,7 +166,6 @@ class CalendarDirectoryServiceMixin(object):
             recordTypes = list(self.recordTypes())
         return recordTypes
 
-
     @inlineCallbacks
     def recordsMatchingTokens(self, tokens, context=None, limitResults=None,
                               timeoutSeconds=None):
@@ -221,7 +226,6 @@ class CalendarDirectoryServiceMixin(object):
 
         returnValue(results)
 
-
     def recordsMatchingFields(
         self, fields, operand=Operand.OR, recordType=None,
         limitResults=None, timeoutSeconds=None
@@ -265,7 +269,6 @@ class CalendarDirectoryServiceMixin(object):
             limitResults=limitResults, timeoutSeconds=timeoutSeconds
         )
 
-
     def setFilter(self, filter):
         """
         Assign a filter for post-processing recordsMatchingTokens and
@@ -277,7 +280,6 @@ class CalendarDirectoryServiceMixin(object):
         """
         self._resultFilter = filter
 
-
     _oldRecordTypeNames = {
         "address": "addresses",
         "group": "groups",
@@ -291,21 +293,18 @@ class CalendarDirectoryServiceMixin(object):
         "writeDelegatorGroup": "writeDelegatorGroups",
     }
 
-
     # Maps record types <--> url path segments, i.e. the segment after
     # /principals/ e.g. "users" or "groups"
 
     def recordTypeToOldName(self, recordType):
         return self._oldRecordTypeNames[recordType.name]
 
-
     def oldNameToRecordType(self, oldName):
         for name, value in self._oldRecordTypeNames.iteritems():
             if oldName == value:
                 return self.recordType.lookupByName(name)
         return None
 
-
     @inlineCallbacks
     def recordsWithDirectoryBasedDelegates(self):
         """
@@ -334,13 +333,17 @@ class CalendarDirectoryServiceMixin(object):
         returnValue(records)
 
 
-
 class CalendarDirectoryRecordMixin(object):
     """
     Calendar (and Contacts) specific logic for directory records lives in this
     class
     """
 
+    def __init__(self):
+        if config.Scheduling.Options.FakeResourceLocationEmail:
+            if self.recordType in (DAVRecordType.location, DAVRecordType.resource) and not getattr(self, "emailAddresses", None):
+                self.fields[BaseFieldName.emailAddresses] = (u"{}@do_not_reply".format(self.uid.encode("hex"),),)
+
     @inlineCallbacks
     def verifyCredentials(self, credentials):
 
@@ -373,7 +376,6 @@ class CalendarDirectoryRecordMixin(object):
                 ))
             )
 
-
     def _calendarAddresses(self):
         cuas = set()
 
@@ -402,7 +404,6 @@ class CalendarDirectoryRecordMixin(object):
 
         return frozenset(cuas)
 
-
     @property
     def calendarUserAddresses(self):
         try:
@@ -426,11 +427,9 @@ class CalendarDirectoryRecordMixin(object):
         DAVRecordType.location: 'ROOM',
     }
 
-
     def getCUType(self):
         return self._cuTypes.get(self.recordType, "UNKNOWN")
 
-
     @classmethod
     def fromCUType(cls, cuType):
         for key, val in cls._cuTypes.iteritems():
@@ -438,7 +437,6 @@ class CalendarDirectoryRecordMixin(object):
                 return key
         return None
 
-
     def applySACLs(self):
         """
         Disable calendaring and addressbooks as dictated by SACLs
@@ -448,12 +446,12 @@ class CalendarDirectoryRecordMixin(object):
         # if config.EnableSACLs and self.CheckSACL:
         #     username = self.shortNames[0]
         #     if self.CheckSACL(username, "calendar") != 0:
-        #         self.log.debug("%s is not enabled for calendaring due to SACL"
-        #                        % (username,))
+        #         self.log.debug("{user} is not enabled for calendaring due to SACL"
+        #                        user=username)
         #         self.enabledForCalendaring = False
         #     if self.CheckSACL(username, "addressbook") != 0:
-        #         self.log.debug("%s is not enabled for addressbooks due to SACL"
-        #                        % (username,))
+        #         self.log.debug("{user} is not enabled for addressbooks due to SACL"
+        #                        user=username)
         #         self.enabledForAddressBooks = False
 
     @property
@@ -463,7 +461,6 @@ class CalendarDirectoryRecordMixin(object):
         except AttributeError:
             return u""
 
-
     def cacheToken(self):
         """
         Generate a token that can be uniquely used to identify the state of this record for use
@@ -478,7 +475,6 @@ class CalendarDirectoryRecordMixin(object):
             self.hasCalendars,
         ))
 
-
     def canonicalCalendarUserAddress(self, checkCal=True):
         """
             Return a CUA for this record, preferring in this order:
@@ -509,7 +505,6 @@ class CalendarDirectoryRecordMixin(object):
         # fall back to using the first one
         return sortedCuas[0] if sortedCuas else None  # groups may not have cua
 
-
     def enabledAsOrganizer(self):
         if self.recordType == self.service.recordType.user:
             return True
@@ -522,7 +517,6 @@ class CalendarDirectoryRecordMixin(object):
         else:
             return False
 
-
     def serverURI(self):
         """
         URL of the server hosting this record. Return None if hosted on this server.
@@ -535,7 +529,6 @@ class CalendarDirectoryRecordMixin(object):
         else:
             return None
 
-
     def server(self):
         """
         Server hosting this record. Return None if hosted on this server.
@@ -548,20 +541,16 @@ class CalendarDirectoryRecordMixin(object):
         else:
             return None
 
-
     def thisServer(self):
         s = self.server()
         return s.thisServer if s is not None else True
 
-
     def isLoginEnabled(self):
         return self.loginAllowed
 
-
     def calendarsEnabled(self):
         return config.EnableCalDAV and self.hasCalendars
 
-
     @inlineCallbacks
     def canAutoSchedule(self, organizer=None):
         if config.Scheduling.Options.AutoSchedule.Enabled:
@@ -579,7 +568,6 @@ class CalendarDirectoryRecordMixin(object):
                     returnValue(True)
         returnValue(False)
 
-
     @inlineCallbacks
     def getAutoScheduleMode(self, organizer):
         autoScheduleMode = self.autoScheduleMode
@@ -596,6 +584,8 @@ class CalendarDirectoryRecordMixin(object):
 
         returnValue(autoScheduleMode)
 
+    def setAutoScheduleMode(self, autoScheduleMode):
+        return self.service.setAutoScheduleMode(self, autoScheduleMode)
 
     @inlineCallbacks
     def autoAcceptFromOrganizer(self, organizer):
@@ -617,7 +607,6 @@ class CalendarDirectoryRecordMixin(object):
                     returnValue(True)
         returnValue(False)
 
-
     @inlineCallbacks
     def expandedMembers(self, members=None, seen=None):
         """
@@ -644,7 +633,6 @@ class CalendarDirectoryRecordMixin(object):
 
         returnValue(members)
 
-
     @inlineCallbacks
     def expandedMemberUIDs(self):
         """
@@ -654,14 +642,12 @@ class CalendarDirectoryRecordMixin(object):
         members = yield self.expandedMembers()
         returnValue([member.uid for member in members])
 
-
     # For scheduling/freebusy
     @inlineCallbacks
     def isProxyFor(self, other):
         proxymode = yield self.proxyMode(other)
         returnValue(proxymode != "none")
 
-
     @inlineCallbacks
     def proxyMode(self, other):
         """
@@ -686,7 +672,6 @@ class CalendarDirectoryRecordMixin(object):
 
         returnValue("none")
 
-
     @inlineCallbacks
     def proxyFor(self, readWrite, ignoreDisabled=True):
         """
@@ -721,7 +706,6 @@ class CalendarDirectoryRecordMixin(object):
 
         returnValue(proxyFors)
 
-
     def attendeeProperty(self, params={}):
         """
         Returns a pycalendar ATTENDEE property for this record.
@@ -743,7 +727,7 @@ class CalendarDirectoryRecordMixin(object):
                 params["EMAIL"] = list(self.emailAddresses)[0].encode("utf-8")
         if "CUTYPE" not in params:
             cuType = self.getCUType()
-            if cuType is not "INDIVIDUAL":
+            if cuType != "INDIVIDUAL":
                 params["CUTYPE"] = cuType
 
         return Property("ATTENDEE", self.canonicalCalendarUserAddress().encode("utf-8"), params=params)
