@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.who.test.test_groups -*-
 ##
-# Copyright (c) 2013-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2013-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -54,7 +54,6 @@ class GroupCacherPollingWork(
         else:
             return succeed(None)
 
-
     def regenerateInterval(self):
         """
         Return the interval in seconds between regenerating instances.
@@ -62,7 +61,6 @@ class GroupCacherPollingWork(
         groupCacher = getattr(self.transaction, "_groupCacher", None)
         return groupCacher.updateSeconds if groupCacher else 10
 
-
     @inlineCallbacks
     def doWork(self):
 
@@ -84,7 +82,6 @@ class GroupCacherPollingWork(
             )
 
 
-
 class GroupRefreshWork(AggregatedWorkItem, fromTable(schema.GROUP_REFRESH_WORK)):
 
     group = property(lambda self: (self.table.GROUP_UID == self.groupUID))
@@ -113,7 +110,6 @@ class GroupRefreshWork(AggregatedWorkItem, fromTable(schema.GROUP_REFRESH_WORK))
             yield self.reschedule(self.transaction, 10, groupUID=self.groupUID)
 
 
-
 class GroupDelegateChangesWork(AggregatedWorkItem, fromTable(schema.GROUP_DELEGATE_CHANGES_WORK)):
 
     group = property(lambda self: (self.table.DELEGATOR_UID == self.delegatorUID))
@@ -137,7 +133,6 @@ class GroupDelegateChangesWork(AggregatedWorkItem, fromTable(schema.GROUP_DELEGA
                 )
 
 
-
 class GroupAttendeeReconciliationWork(
     AggregatedWorkItem, fromTable(schema.GROUP_ATTENDEE_RECONCILE_WORK)
 ):
@@ -146,7 +141,6 @@ class GroupAttendeeReconciliationWork(
         lambda self: (self.table.RESOURCE_ID == self.resourceID)
     )
 
-
     @inlineCallbacks
     def doWork(self):
 
@@ -159,7 +153,6 @@ class GroupAttendeeReconciliationWork(
         yield calendarObject.groupAttendeeChanged(self.groupID)
 
 
-
 class GroupShareeReconciliationWork(
     AggregatedWorkItem, fromTable(schema.GROUP_SHAREE_RECONCILE_WORK)
 ):
@@ -168,7 +161,6 @@ class GroupShareeReconciliationWork(
         lambda self: (self.table.CALENDAR_ID == self.calendarID)
     )
 
-
     @inlineCallbacks
     def doWork(self):
 
@@ -190,7 +182,6 @@ class GroupShareeReconciliationWork(
                 yield calendar.reconcileGroupSharee(group.groupUID)
 
 
-
 def diffAssignments(old, new):
     """
     Compare two proxy assignment lists and return their differences in the form
@@ -222,14 +213,15 @@ def diffAssignments(old, new):
     return changed, removed
 
 
-
 class GroupCacher(object):
     log = Logger()
 
-
     def __init__(
         self, directory,
         updateSeconds=600,
+        initialSchedulingDelaySeconds=10,
+        batchSize=100,
+        batchSchedulingIntervalSeconds=2,
         useDirectoryBasedDelegates=False,
         directoryBasedDelegatesSource=None,
         cacheNotifier=None,
@@ -241,7 +233,9 @@ class GroupCacher(object):
         self.directoryBasedDelegatesSource = directoryBasedDelegatesSource
         self.cacheNotifier = cacheNotifier
         self.updateSeconds = updateSeconds
-
+        self.initialSchedulingDelaySeconds = initialSchedulingDelaySeconds
+        self.batchSize = batchSize
+        self.batchSchedulingIntervalSeconds = batchSchedulingIntervalSeconds
 
     @inlineCallbacks
     def update(self, txn):
@@ -272,37 +266,54 @@ class GroupCacher(object):
         #     "Groups to refresh: {g}", g=groupUIDs
         # )
 
-        if config.AutomaticPurging.Enabled and groupUIDs:
-            # remove unused groups and groups that have not been seen in a while
+        # Get the set of all known groups in the DB
+        knownGroupUIDs = yield txn.allGroups()
+
+        # We'll want to remove groups no longer in use
+        groupsToRemove = knownGroupUIDs - groupUIDs
+
+        # Also look for groups which have been marked as missing for a while
+        if config.AutomaticPurging.Enabled:
             dateLimit = (
                 datetime.datetime.utcnow() -
                 datetime.timedelta(seconds=float(config.AutomaticPurging.GroupPurgeIntervalSeconds))
             )
+            missingGroups = yield txn.groupsMissingSince(dateLimit)
+            groupsToRemove |= missingGroups
+
+        # Delete the groups in batches
+        groupsToRemove = list(groupsToRemove)
+        batchSize = 100
+        deletedGroupUIDs = []
+        while groupsToRemove:
+            batch = groupsToRemove[:batchSize]
+            del groupsToRemove[:batchSize]
+
             rows = yield GroupsRecord.deletesome(
                 txn,
-                (
-                    (GroupsRecord.extant == 0).And(GroupsRecord.modified < dateLimit)
-                ).Or(
-                    GroupsRecord.groupUID.NotIn(groupUIDs)
-                ),
+                GroupsRecord.groupUID.In(batch),
                 returnCols=GroupsRecord.groupUID,
             )
-        else:
-            # remove unused groups
-            rows = yield GroupsRecord.deletesome(
-                txn,
-                GroupsRecord.groupUID.NotIn(groupUIDs) if groupUIDs else None,
-                returnCols=GroupsRecord.groupUID,
-            )
-        deletedGroupUIDs = [row[0] for row in rows]
+            deletedGroupUIDs.extend([row[0] for row in rows])
+
         if deletedGroupUIDs:
-            self.log.debug("Deleted old or unused groups {d}", d=deletedGroupUIDs)
+            self.log.debug(
+                "Deleted old or unused groups {d}", d=deletedGroupUIDs
+            )
 
         # For each of those groups, create a per-group refresh work item
+        futureSeconds = self.initialSchedulingDelaySeconds
+        i = 0
         for groupUID in set(groupUIDs) - set(deletedGroupUIDs):
-            self.log.debug("Enqueuing group refresh for {u}", u=groupUID)
-            yield GroupRefreshWork.reschedule(txn, 0, groupUID=groupUID)
-
+            self.log.debug(
+                "Enqueuing group refresh for {u} in {sec} seconds",
+                u=groupUID, sec=futureSeconds
+            )
+            yield GroupRefreshWork.reschedule(txn, futureSeconds, groupUID=groupUID)
+            i += 1
+            if i % self.batchSize == 0:
+                i = 0
+                futureSeconds += self.batchSchedulingIntervalSeconds
 
     @inlineCallbacks
     def scheduleExternalAssignments(
@@ -354,7 +365,6 @@ class GroupCacher(object):
                         readDelegateUID="", writeDelegateUID=""
                     )
 
-
     @inlineCallbacks
     def applyExternalAssignments(
         self, txn, delegatorUID, readDelegateUID, writeDelegateUID
@@ -386,14 +396,21 @@ class GroupCacher(object):
             readDelegateUID, writeDelegateUID
         )
 
+        # Send cache change notifications
+        if self.cacheNotifier is not None:
+            self.cacheNotifier.changed(delegatorUID)
+            if readDelegateUID:
+                self.cacheNotifier.changed(readDelegateUID)
+            if writeDelegateUID:
+                self.cacheNotifier.changed(writeDelegateUID)
 
     @inlineCallbacks
     def refreshGroup(self, txn, groupUID):
         """
-            Does the work of a per-group refresh work item
-            Faults in the flattened membership of a group, as UIDs
-            and updates the GROUP_MEMBERSHIP table
-            WorkProposal is returned for tests
+        Does the work of a per-group refresh work item
+        Faults in the flattened membership of a group, as UIDs
+        and updates the GROUP_MEMBERSHIP table
+        WorkProposal is returned for tests
         """
         self.log.debug("Refreshing group: {g}", g=groupUID)
 
@@ -438,25 +455,21 @@ class GroupCacher(object):
 
         returnValue(tuple())
 
-
     def synchronizeMembers(self, txn, groupID, newMemberUIDs):
         return txn.synchronizeMembers(groupID, newMemberUIDs)
 
-
     def cachedMembers(self, txn, groupID):
         """
         The members of the given group as recorded in the db
         """
         return txn.groupMembers(groupID)
 
-
     def cachedGroupsFor(self, txn, uid):
         """
         The UIDs of the groups the uid is a member of
         """
         return txn.groupUIDsFor(uid)
 
-
     @inlineCallbacks
     def scheduleGroupAttendeeReconciliations(self, txn, groupID):
         """
@@ -478,7 +491,6 @@ class GroupCacher(object):
             workItems.append(work)
         returnValue(tuple(workItems))
 
-
     @inlineCallbacks
     def scheduleGroupShareeReconciliations(self, txn, groupID):
         """
@@ -504,7 +516,6 @@ class GroupCacher(object):
             workItems.append(work)
         returnValue(tuple(workItems))
 
-
     @inlineCallbacks
     def groupsToRefresh(self, txn):
         delegatedUIDs = set((yield txn.allGroupDelegates()))
@@ -513,10 +524,13 @@ class GroupCacher(object):
         )
 
         # Also get group delegates from other pods
-        if txn.directoryService().serversDB() is not None and len(txn.directoryService().serversDB().allServersExceptThis()) != 0:
+        if (
+            txn.directoryService().serversDB() is not None and
+            len(txn.directoryService().serversDB().allServersExceptThis(filter_v5=True)) != 0
+        ):
             results = yield DeferredList([
                 txn.store().conduit.send_all_group_delegates(txn, server) for
-                server in txn.directoryService().serversDB().allServersExceptThis()
+                server in txn.directoryService().serversDB().allServersExceptThis(filter_v5=True)
             ], consumeErrors=True)
             for result in results:
                 if result and result[0]:
