@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2014-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2014-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -53,7 +53,6 @@ class GroupShareeTestBase(CommonCommonTests, unittest.TestCase):
 
         self.paths = {}
 
-
     def configure(self):
         super(GroupShareeTestBase, self).configure()
         config.Sharing.Enabled = True
@@ -61,20 +60,19 @@ class GroupShareeTestBase(CommonCommonTests, unittest.TestCase):
         config.Sharing.Calendars.Groups.Enabled = True
         config.Sharing.Calendars.Groups.ReconciliationDelaySeconds = 0
 
-
     @inlineCallbacks
     def populate(self):
         yield populateCalendarsFrom(self.requirements, self.storeUnderTest())
 
     requirements = {
-        "user01" : None,
-        "user02" : None,
-        "user03" : None,
-        "user06" : None,
-        "user07" : None,
-        "user08" : None,
-        "user09" : None,
-        "user10" : None,
+        "user01": None,
+        "user02": None,
+        "user03": None,
+        "user06": None,
+        "user07": None,
+        "user08": None,
+        "user09": None,
+        "user10": None,
     }
 
     @inlineCallbacks
@@ -83,13 +81,11 @@ class GroupShareeTestBase(CommonCommonTests, unittest.TestCase):
         count = yield cal6.countObjectResources()
         self.assertEqual(count, expected_count)
 
-
     @inlineCallbacks
     def _check_notifications(self, uid, items):
         notifyHome = yield self.transactionUnderTest().notificationsWithUID(uid, create=True)
         notifications = yield notifyHome.listNotificationObjects()
-        self.assertEqual(set(notifications), set(items))
-
+        self.assertEqual(set(notifications), set([item + ".xml" for item in items]))
 
     @inlineCallbacks
     def shareeViewUnderTest(self, txn=None, shareUID="calendar_1", home="user01"):
@@ -101,7 +97,6 @@ class GroupShareeTestBase(CommonCommonTests, unittest.TestCase):
         ))
 
 
-
 class GroupShareeReconciliationTests(GroupShareeTestBase):
 
     @inlineCallbacks
@@ -133,7 +128,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         yield self._check_notifications("user01", [])
         shareeViews = yield calendar.inviteUIDToShare("group04", _BIND_MODE_READ)
@@ -184,7 +179,6 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         noinvites = yield calendar.sharingInvites()
         self.assertEqual(len(noinvites), 0)
 
-
     @inlineCallbacks
     def test_group_change_invite_larger(self):
         """
@@ -217,7 +211,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         yield self._check_notifications("user01", [])
         shareeViews = yield calendar.inviteUIDToShare("group04", _BIND_MODE_READ)
@@ -265,7 +259,6 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         noinvites = yield calendar.sharingInvites()
         self.assertEqual(len(noinvites), 0)
 
-
     @inlineCallbacks
     def test_group_member_removal_refresh_slow(self):
         """
@@ -301,7 +294,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         yield self._check_notifications("user01", [])
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ)
@@ -351,7 +344,6 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         noinvites = yield calendar.sharingInvites()
         self.assertEqual(len(noinvites), 3)
 
-
     @inlineCallbacks
     def test_no_self_invite(self):
         """
@@ -384,7 +376,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         yield self._check_notifications("user01", [])
         shareeViews = yield calendar.inviteUIDToShare("group05", _BIND_MODE_READ)
@@ -431,7 +423,6 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         noinvites = yield calendar.sharingInvites()
         self.assertEqual(len(noinvites), 0)
 
-
     @inlineCallbacks
     def test_no_self_invite_on_add(self):
         """
@@ -465,7 +456,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user01", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         yield self._check_notifications("user01", [])
         shareeViews = yield calendar.inviteUIDToShare("group06", _BIND_MODE_READ)
@@ -512,7 +503,6 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         noinvites = yield calendar.sharingInvites()
         self.assertEqual(len(noinvites), 0)
 
-
     @inlineCallbacks
     def test_group_change_trashed_calendar(self):
         """
@@ -549,7 +539,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user01", name="shared")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         yield self._check_notifications("user01", [])
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ)
@@ -593,7 +583,6 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         yield self.commit()
         yield JobItem.waitEmpty(self._sqlCalendarStore.newTransaction, reactor, 60)
 
-
     @inlineCallbacks
     def test_group_change_removed_calendar(self):
         """
@@ -630,7 +619,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user01", name="shared")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         yield self._check_notifications("user01", [])
         shareeViews = yield calendar.inviteUIDToShare("group02", _BIND_MODE_READ)
@@ -673,7 +662,6 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         yield self.commit()
         yield JobItem.waitEmpty(self._sqlCalendarStore.newTransaction, reactor, 60)
 
-
     @inlineCallbacks
     def test_multiple_groups_remove_from_one_group(self):
         """
@@ -704,7 +692,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user03", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeViews = yield calendar.inviteUIDToShare("group05", _BIND_MODE_WRITE)
         self.assertEqual(len(shareeViews), 2)
@@ -756,7 +744,6 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         noinvites = yield calendar.sharingInvites()
         self.assertEqual(len(noinvites), 0)
 
-
     @inlineCallbacks
     def test_multiple_groups_with_individual_remove_from_one_group(self):
         """
@@ -787,7 +774,7 @@ class GroupShareeReconciliationTests(GroupShareeTestBase):
         calendar = yield self.calendarUnderTest(home="user03", name="calendar")
         invites = yield calendar.sharingInvites()
         self.assertEqual(len(invites), 0)
-        self.assertFalse(calendar.isShared())
+        self.assertFalse(calendar.isSharedByOwner())
 
         shareeView = yield calendar.inviteUIDToShare("user01", _BIND_MODE_READ)
         self.assertTrue(shareeView is not None)
