@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2013-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2013-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,7 +29,6 @@ from txdav.who.groups import GroupCacher, diffAssignments, GroupRefreshWork
 from txdav.who.test.support import TestRecord, CalendarInMemoryDirectoryService
 
 
-
 class GroupCacherTest(StoreTestCase):
 
     @inlineCallbacks
@@ -37,7 +36,6 @@ class GroupCacherTest(StoreTestCase):
         yield super(GroupCacherTest, self).setUp()
         self.groupCacher = GroupCacher(self.directory)
 
-
     @inlineCallbacks
     def test_multipleCalls(self):
         """
@@ -53,7 +51,6 @@ class GroupCacherTest(StoreTestCase):
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_refreshGroup(self):
         """
@@ -98,7 +95,6 @@ class GroupCacherTest(StoreTestCase):
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_synchronizeMembers(self):
         """
@@ -149,7 +145,6 @@ class GroupCacherTest(StoreTestCase):
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_groupByID(self):
 
@@ -171,7 +166,6 @@ class GroupCacherTest(StoreTestCase):
 
         yield txn.commit()
 
-
     @inlineCallbacks
     def test_externalAssignments(self):
 
@@ -375,7 +369,6 @@ class GroupCacherTest(StoreTestCase):
 
         yield txn.commit()
 
-
     def test_diffAssignments(self):
         """
         Ensure external proxy assignment diffing works
@@ -456,7 +449,6 @@ class GroupCacherTest(StoreTestCase):
             ))
         )
 
-
     @inlineCallbacks
     def test_recursiveGroup(self):
         """
@@ -476,9 +468,8 @@ class GroupCacherTest(StoreTestCase):
 
         yield txn.commit()
 
-
     @inlineCallbacks
-    def test_groupChangeCacheNotification(self):
+    def test_groupChangeCacheNotificationRefreshGroup(self):
         """
         Verify refreshGroup() triggers a cache notification for the group and all
         members that are added or removed
@@ -486,6 +477,7 @@ class GroupCacherTest(StoreTestCase):
 
         class TestNotifier(object):
             changedTokens = []
+
             def changed(self, token):
                 self.changedTokens.append(token)
 
@@ -602,11 +594,42 @@ class GroupCacherTest(StoreTestCase):
         ]))
         TestNotifier.changedTokens = []
 
+    @inlineCallbacks
+    def test_groupChangeCacheNotificationApplyExternalAssignments(self):
+        """
+        Verify applyExternalAssignments() triggers a cache notification for the
+        delegator and delegates
+        """
+
+        class TestNotifier(object):
+            changedTokens = []
+
+            def changed(self, token):
+                self.changedTokens.append(token)
+
+        self.groupCacher.cacheNotifier = TestNotifier()
+
+        yield self.groupCacher.applyExternalAssignments(self.transactionUnderTest(), "__dre1__", None, None)
+        yield self.commit()
+
+        self.assertEqual(
+            TestNotifier.changedTokens,
+            ["__dre1__"]
+        )
+        TestNotifier.changedTokens = []
+
+        yield self.groupCacher.applyExternalAssignments(self.transactionUnderTest(), "__dre1__", "__top_group_1__", "__sub_group_1__")
+        yield self.commit()
+
+        self.assertEqual(
+            TestNotifier.changedTokens,
+            ["__dre1__", "__top_group_1__", "__sub_group_1__"]
+        )
+        TestNotifier.changedTokens = []
 
 
 class DynamicGroupTest(StoreTestCase):
 
-
     @inlineCallbacks
     def setUp(self):
         yield super(DynamicGroupTest, self).setUp()
@@ -652,7 +675,6 @@ class DynamicGroupTest(StoreTestCase):
         members = yield self.directory.recordsWithRecordType(RecordType.user)
         yield group.setMembers(members)
 
-
         def doWork(self):
             self.transaction._groupCacher = groupCacher
             return unpatchedDoWork(self)
@@ -663,7 +685,6 @@ class DynamicGroupTest(StoreTestCase):
 
         config.AutomaticPurging.Enabled = True
 
-
     @inlineCallbacks
     def test_extant(self):
         """
@@ -732,7 +753,6 @@ class DynamicGroupTest(StoreTestCase):
             yield txn.commit()
             self.assertEquals(len(members), 100 if uid == u"testgroup" else 0)
 
-
     @inlineCallbacks
     def test_update_delete_unused(self):
         """
@@ -802,7 +822,6 @@ class DynamicGroupTest(StoreTestCase):
         self.assertEqual(testGroup, None)
         self.assertNotEqual(emptyGroup, None)
 
-
     @inlineCallbacks
     def test_update_delete_old_nonextant(self):
         """
