@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 # Copyright (c) 2007 Twisted Matrix Laboratories.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -80,7 +80,6 @@ def encodeXMLName(namespace, name):
     return sname.encode("utf-8")
 
 
-
 def decodeXMLName(name):
     """
     Decodes an XML (namespace, name) pair from an ASCII string as
@@ -92,7 +91,7 @@ def decodeXMLName(name):
     if not name:
         invalid()
 
-    if name[0] is "{":
+    if name[0] == "{":
         index = name.find("}")
         if (index is -1 or not len(name) > index):
             invalid()
@@ -116,12 +115,11 @@ def decodeXMLName(name):
     return (namespace, localname)
 
 
-
 class WebDAVElement (object):
     """
     WebDAV XML element. (RFC 2518, section 12)
     """
-    namespace = dav_namespace # Element namespace (class variable)
+    namespace = dav_namespace  # Element namespace (class variable)
     name = None          # Element name (class variable)
     allowed_children = None          # Types & count limits on child elements
     allowed_attributes = None          # Allowed attribute names
@@ -157,17 +155,14 @@ class WebDAVElement (object):
         self.children = tuple(my_children)
         self.attributes = attributes
 
-
     @classmethod
     def qname(cls):
         return (cls.namespace, cls.name)
 
-
     @classmethod
     def sname(cls):
         return encodeXMLName(cls.namespace, cls.name)
 
-
     def validate(self):
         children = self.children
         attributes = self.attributes
@@ -214,8 +209,8 @@ class WebDAVElement (object):
             else:
                 if not (isinstance(child, PCDATAElement) and child.isWhitespace()):
                     log.debug(
-                        "Child of type %s is unexpected and therefore ignored in %s element"
-                        % (child.sname(), self.sname())
+                        "Child of type {child} is unexpected and therefore ignored in {this} element",
+                        child=child.sname(), this=self.sname(),
                     )
 
         for qname, (min, max) in allowed_children.items():
@@ -233,7 +228,7 @@ class WebDAVElement (object):
         if self.allowed_attributes:
             for name in attributes:
                 if name not in self.allowed_attributes:
-                    log.debug("Attribute %s is unexpected in %s element" % (name, self.sname()))
+                    log.debug("Attribute {attr} is unexpected in {this} element", attr=name, this=self.sname())
                 my_attributes[name] = attributes[name]
 
             for name, required in self.allowed_attributes.items():
@@ -243,24 +238,20 @@ class WebDAVElement (object):
 
         else:
             if not isinstance(self, WebDAVUnknownElement) and attributes:
-                log.debug("Attributes %s are unexpected in %s element"
-                          % (attributes.keys(), self.sname()))
+                log.debug("Attributes {attrs} are unexpected in {this} element", attrs=attributes.keys(), this=self.sname())
             my_attributes.update(attributes)
 
         self.attributes = my_attributes
 
-
     def __str__(self):
         return self.sname()
 
-
     def __repr__(self):
         if hasattr(self, "attributes") and hasattr(self, "children"):
             return "<%s %r: %r>" % (self.sname(), self.attributes, self.children)
         else:
             return "<%s>" % (self.sname())
 
-
     def __eq__(self, other):
         if isinstance(other, WebDAVElement):
             return (
@@ -272,20 +263,16 @@ class WebDAVElement (object):
         else:
             return NotImplemented
 
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
     def __contains__(self, child):
         return child in self.children
 
-
     def writeXML(self, output, pretty=True):
         output.write("<?xml version='1.0' encoding='UTF-8'?>" + ("\n" if pretty else ""))
         self._writeToStream(output, "", 0, pretty)
 
-
     def _writeToStream(self, output, ns, level, pretty):
         """
         Fast XML output.
@@ -351,7 +338,6 @@ class WebDAVElement (object):
         if pretty and level:
             output.write("\r\n")
 
-
     def _writeAttributeToStream(self, output, name, value):
 
         # Quote any single quotes. We do not need to be any smarter than this.
@@ -359,13 +345,11 @@ class WebDAVElement (object):
 
         output.write(" %s='%s'" % (name, value,))
 
-
     def toxml(self, pretty=True):
         output = StringIO.StringIO()
         self.writeXML(output, pretty)
         return str(output.getvalue())
 
-
     def element(self, document):
         element = document.createElementNS(self.namespace, self.name)
         if hasattr(self, "attributes"):
@@ -376,7 +360,6 @@ class WebDAVElement (object):
                 element.setAttributeNodeNS(attribute)
         return element
 
-
     def addToDOM(self, document, parent):
         element = self.element(document)
 
@@ -390,10 +373,9 @@ class WebDAVElement (object):
                 try:
                     child.addToDOM(document, element)
                 except:
-                    log.error("Unable to add child %r of element %s to DOM" % (child, self))
+                    log.error("Unable to add child {child!r} of element {this!r} to DOM", child=child, this=self)
                     raise
 
-
     def childrenOfType(self, child_type):
         """
         Returns a list of children with the same qname as the given type.
@@ -405,7 +387,6 @@ class WebDAVElement (object):
 
         return [c for c in self.children if c.qname() == qname]
 
-
     def childOfType(self, child_type):
         """
         Returns a child of the given type, if any, or None.
@@ -418,7 +399,6 @@ class WebDAVElement (object):
             found = child
         return found
 
-
     def removeWhitespaceNodes(self):
         """ Removes all of the whitespace-only text decendants of a DOM node. """
         # prepare the list of text nodes to remove (and recurse when needed)
@@ -439,8 +419,8 @@ class WebDAVElement (object):
         self.children = tuple(newchildren)
 
 
-
 class PCDATAElement (object):
+
     def __init__(self, data):
         super(PCDATAElement, self).__init__()
 
@@ -453,36 +433,29 @@ class PCDATAElement (object):
 
         self.data = data
 
-
     @classmethod
     def qname(cls):
         return (None, "#PCDATA")
 
-
     @classmethod
     def sname(cls):
         return "#PCDATA"
 
-
     def validate(self):
         pass
 
-
     def __str__(self):
         return str(self.data)
 
-
     def __repr__(self):
         return "<%s: %r>" % (self.__class__.__name__, self.data)
 
-
     def __add__(self, other):
         if isinstance(other, PCDATAElement):
             return self.__class__(self.data + other.data)
         else:
             return self.__class__(self.data + other)
 
-
     def __eq__(self, other):
         if isinstance(other, PCDATAElement):
             return self.data == other.data
@@ -491,30 +464,25 @@ class PCDATAElement (object):
         else:
             return NotImplemented
 
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
     def isWhitespace(self):
         for char in str(self):
             if char not in string.whitespace:
                 return False
         return True
 
-
     def element(self, document):
         return document.createTextNode(self.data)
 
-
     def addToDOM(self, document, parent):
         try:
             parent.appendChild(self.element(document))
         except TypeError:
-            log.error("Invalid PCDATA: %r" % (self.data,))
+            log.error("Invalid PCDATA: {data!r}", data=self.data)
             raise
 
-
     def _writeToStream(self, output, ns, level, pretty):
         # Do escaping/CDATA behavior
         if "\r" in self.data or "\n" in self.data:
@@ -532,7 +500,6 @@ class PCDATAElement (object):
         output.write(cdata)
 
 
-
 class WebDAVOneShotElement (WebDAVElement):
     """
     Element with exactly one WebDAVEmptyElement child and no attributes.
@@ -565,7 +532,6 @@ class WebDAVOneShotElement (WebDAVElement):
         return WebDAVOneShotElement.__singletons[cls][child]
 
 
-
 class WebDAVUnknownElement (WebDAVElement):
     """
     Placeholder for unknown element tag names.
@@ -582,16 +548,13 @@ class WebDAVUnknownElement (WebDAVElement):
         child.name = name
         return child
 
-
     def qname(self):
         return (self.namespace, self.name)
 
-
     def sname(self):
         return encodeXMLName(self.namespace, self.name)
 
 
-
 class WebDAVEmptyElement (WebDAVElement):
     """
     WebDAV element with no contents.
@@ -611,7 +574,6 @@ class WebDAVEmptyElement (WebDAVElement):
     allowed_children = {}
     children = ()
 
-
     def __hash__(self):
         """
         Define a hash method, so that an empty element can serve as dictionary
@@ -621,7 +583,6 @@ class WebDAVEmptyElement (WebDAVElement):
         return hash((self.name, self.namespace))
 
 
-
 class WebDAVTextElement (WebDAVElement):
     """
     WebDAV element containing PCDATA.
@@ -643,14 +604,12 @@ class WebDAVTextElement (WebDAVElement):
         """
         return self.__str__().decode("utf-8")
 
-
     def __str__(self):
         """
         @return: a byte string containing the text in this element.
         """
         return b"".join([c.data for c in self.children])
 
-
     def __repr__(self):
         content = str(self)
         if content:
@@ -658,7 +617,6 @@ class WebDAVTextElement (WebDAVElement):
         else:
             return "<%s>" % (self.sname(),)
 
-
     def __eq__(self, other):
         if isinstance(other, self.__class__):
             return str(self) == str(other)
@@ -668,7 +626,6 @@ class WebDAVTextElement (WebDAVElement):
             return NotImplemented
 
 
-
 class WebDAVDateTimeElement (WebDAVTextElement):
     """
     WebDAV date-time element. (RFC 2518, section 23.2)
@@ -699,11 +656,9 @@ class WebDAVDateTimeElement (WebDAVTextElement):
 
         return clazz(PCDATAElement(date))
 
-
     def __init__(self, *children, **attributes):
         super(WebDAVDateTimeElement, self).__init__(*children, **attributes)
-        self.datetime() # Raise ValueError if the format is wrong
-
+        self.datetime()  # Raise ValueError if the format is wrong
 
     def __eq__(self, other):
         if isinstance(other, self.__class__):
@@ -711,7 +666,6 @@ class WebDAVDateTimeElement (WebDAVTextElement):
         else:
             return NotImplemented
 
-
     def datetime(self):
         s = str(self)
         if not s:
@@ -720,7 +674,6 @@ class WebDAVDateTimeElement (WebDAVTextElement):
             return parse_date(s)
 
 
-
 class DateTimeHeaderElement (WebDAVDateTimeElement):
     """
     WebDAV date-time element for elements that substitute for HTTP
@@ -756,7 +709,6 @@ class DateTimeHeaderElement (WebDAVDateTimeElement):
 
         return clazz(PCDATAElement(date))
 
-
     def datetime(self):
         s = str(self)
         if not s:
@@ -765,7 +717,6 @@ class DateTimeHeaderElement (WebDAVDateTimeElement):
             return parseDateTime(s)
 
 
-
 ##
 # Utilities
 ##
@@ -774,21 +725,19 @@ class FixedOffset (datetime.tzinfo):
     """
     Fixed offset in minutes east from UTC.
     """
+
     def __init__(self, offset, name=None):
         super(FixedOffset, self).__init__()
 
         self._offset = datetime.timedelta(minutes=offset)
         self._name = name
 
-
     def utcoffset(self, dt):
         return self._offset
 
-
     def tzname(self, dt):
         return self._name
 
-
     def dst(self, dt):
         return datetime.timedelta(0)
 
@@ -801,6 +750,7 @@ _regex_ISO8601Date = re.compile(
     "$"
 )
 
+
 def parse_date(date):
     """
     Parse an ISO 8601 date and return a corresponding datetime.datetime object.
