@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2017 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -44,17 +44,15 @@ class WebDAVContentHandler(xml.sax.handler.ContentHandler):
         xml.sax.handler.ContentHandler.__init__(self)
         self._characterBuffer = None
 
-
     def location(self):
         return "line %d, column %d" % (self.locator.getLineNumber(), self.locator.getColumnNumber())
 
-
     def startDocument(self):
         self.stack = [{
-            "name"       : None,
-            "class"      : None,
-            "attributes" : None,
-            "children"   : [],
+            "name": None,
+            "class": None,
+            "attributes": None,
+            "children": [],
         }]
 
         # Keep a cache of the subclasses we create for unknown XML
@@ -63,7 +61,6 @@ class WebDAVContentHandler(xml.sax.handler.ContentHandler):
         # multiple times in a document.
         self.unknownElementClasses = {}
 
-
     def endDocument(self):
         top = self.stack[-1]
 
@@ -75,7 +72,6 @@ class WebDAVContentHandler(xml.sax.handler.ContentHandler):
         self.dom = WebDAVDocument(top["children"][0])
         del(self.unknownElementClasses)
 
-
     def startElementNS(self, name, qname, attributes):
         if self._characterBuffer is not None:
             pcdata = PCDATAElement("".join(self._characterBuffer))
@@ -84,7 +80,7 @@ class WebDAVContentHandler(xml.sax.handler.ContentHandler):
 
         attributes_dict = {}
 
-        if attributes.getLength() is not 0:
+        if attributes.getLength() != 0:
             for attr_name in attributes.getQNames():
                 attributes_dict[attr_name.encode("utf-8")] = attributes.getValueByQName(attr_name)
 
@@ -103,13 +99,12 @@ class WebDAVContentHandler(xml.sax.handler.ContentHandler):
             self.unknownElementClasses[name] = element_class
 
         self.stack.append({
-            "name"       : name,
-            "class"      : element_class,
-            "attributes" : attributes_dict,
-            "children"   : [],
+            "name": name,
+            "class": element_class,
+            "attributes": attributes_dict,
+            "children": [],
         })
 
-
     def endElementNS(self, name, qname):
         if self._characterBuffer is not None:
             pcdata = PCDATAElement("".join(self._characterBuffer))
@@ -128,39 +123,32 @@ class WebDAVContentHandler(xml.sax.handler.ContentHandler):
             element = top["class"](*top["children"], **top["attributes"])
         except ValueError, e:
             e.args = ("%s at %s" % (e.args[0], self.location()),) + e.args[1:]
-            raise # Re-raises modified e, but preserves traceback
+            raise  # Re-raises modified e, but preserves traceback
 
         self.stack[-1]["children"].append(element)
 
-
     def characters(self, content):
         # Stash character data away in a list that we will "".join() when done
         if self._characterBuffer is None:
             self._characterBuffer = []
         self._characterBuffer.append(content)
 
-
     def ignorableWhitespace(self, whitespace):
         self.characters(self, whitespace)
 
-
     def startElement(self, name, attributes):
         raise AssertionError("startElement() should not be called by namespace-aware parser")
 
-
     def endElement(self, name):
         raise AssertionError("endElement() should not be called by namespace-aware parser")
 
-
     def processingInstruction(self, target, data):
         raise AssertionError("processing instructions are not allowed")
 
-
     def skippedEntity(self, name):
         raise AssertionError("skipped entities are not allowed")
 
 
-
 class WebDAVDocument(AbstractWebDAVDocument):
 
     @classmethod
@@ -180,7 +168,6 @@ class WebDAVDocument(AbstractWebDAVDocument):
 
         return handler.dom
 
-
     def writeXML(self, output):
         document = xml.dom.minidom.Document()
         self.root_element.addToDOM(document, None)
