@@ -25,7 +25,6 @@ else:
     from xattr import xattr
 
 
-
 class ExtendedAttributesPropertyStoreTests(TestCase):
     """
     Tests for L{xattrPropertyStore}.
@@ -33,7 +32,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
     if xattrPropertyStore is None:
         skip = "xattr package missing, cannot test xattr property store"
 
-
     def setUp(self):
         """
         Create a resource and a xattr property store for it.
@@ -44,7 +42,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self.resource = DAVFile(self.resourcePath.path)
         self.propertyStore = xattrPropertyStore(self.resource)
 
-
     def test_getAbsent(self):
         """
         L{xattrPropertyStore.get} raises L{HTTPError} with a I{NOT FOUND}
@@ -54,7 +51,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         error = self.assertRaises(HTTPError, self.propertyStore.get, ("foo", "bar"))
         self.assertEquals(error.response.code, NOT_FOUND)
 
-
     def _forbiddenTest(self, method):
         # Remove access to the directory containing the file so that getting
         # extended attributes from it fails with EPERM.
@@ -74,7 +70,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         # of the EPERM failure.
         self.assertEquals(error.response.code, FORBIDDEN)
 
-
     def _missingTest(self, method):
         # Remove access to the directory containing the file so that getting
         # extended attributes from it fails with EPERM.
@@ -94,7 +89,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         # of the EPERM failure.
         self.assertEquals(error.response.code, FORBIDDEN)
 
-
     def test_getErrors(self):
         """
         If there is a problem getting the specified property (aside from the
@@ -103,7 +97,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         """
         self._forbiddenTest('get')
 
-
     def test_getMissing(self):
         """
         Test missing file.
@@ -123,7 +116,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         # Make sure that the status is NOT FOUND.
         self.assertEquals(error.response.code, NOT_FOUND)
 
-
     def _makeValue(self, uid=None):
         """
         Create and return any old WebDAVDocument for use by the get tests.
@@ -132,7 +124,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         document = WebDAVDocument(element)
         return document
 
-
     def _setValue(self, originalDocument, value, uid=None):
         element = originalDocument.root_element
         attribute = (
@@ -141,7 +132,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
             element.sname())
         self.attrs[attribute] = value
 
-
     def _getValue(self, originalDocument, uid=None):
         element = originalDocument.root_element
         attribute = (
@@ -150,7 +140,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
             element.sname())
         return self.attrs[attribute]
 
-
     def _checkValue(self, originalDocument, uid=None):
         property = originalDocument.root_element.qname()
 
@@ -161,7 +150,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self.assertIsInstance(loadedDocument, Depth)
         self.assertEquals(str(loadedDocument), uid if uid else "0")
 
-
     def test_getXML(self):
         """
         If there is an XML document associated with the property name passed to
@@ -172,7 +160,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self._setValue(document, document.toxml())
         self._checkValue(document)
 
-
     def test_getCompressed(self):
         """
         If there is a compressed value associated with the property name passed
@@ -184,7 +171,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self._setValue(document, compress(document.toxml()))
         self._checkValue(document)
 
-
     def test_getPickled(self):
         """
         If there is a pickled document associated with the property name passed
@@ -195,7 +181,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self._setValue(document, dumps(document))
         self._checkValue(document)
 
-
     def test_getUpgradeXML(self):
         """
         If the value associated with the property name passed to
@@ -209,7 +194,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self.assertEquals(
             decompress(self._getValue(document)), document.root_element.toxml(pretty=False))
 
-
     def test_getUpgradeCompressedPickle(self):
         """
         If the value associated with the property name passed to
@@ -222,7 +206,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self.assertEquals(
             decompress(self._getValue(document)), document.root_element.toxml(pretty=False))
 
-
     def test_getInvalid(self):
         """
         If the value associated with the property name passed to
@@ -238,8 +221,7 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         error = self.assertRaises(HTTPError, self.propertyStore.get, property)
         self.assertEquals(error.response.code, INTERNAL_SERVER_ERROR)
         self.assertEquals(
-            len(self.flushLoggedErrors(UnpicklingError)), 1)
-
+            len(self.flushLoggedErrors(UnpicklingError, IndexError)), 1)
 
     def test_set(self):
         """
@@ -251,7 +233,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self.assertEquals(
             decompress(self._getValue(document)), document.root_element.toxml(pretty=False))
 
-
     def test_delete(self):
         """
         L{xattrPropertyStore.delete} deletes the named property.
@@ -261,7 +242,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self.propertyStore.delete(document.root_element.qname())
         self.assertRaises(KeyError, self._getValue, document)
 
-
     def test_deleteNonExistent(self):
         """
         L{xattrPropertyStore.delete} does nothing if passed a property which
@@ -271,7 +251,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self.propertyStore.delete(document.root_element.qname())
         self.assertRaises(KeyError, self._getValue, document)
 
-
     def test_deleteErrors(self):
         """
         If there is a problem deleting the specified property (aside from the
@@ -293,7 +272,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         # of the EEXIST failure.
         self.assertEquals(error.response.code, NOT_FOUND)
 
-
     def test_contains(self):
         """
         L{xattrPropertyStore.contains} returns C{True} if the given property
@@ -306,7 +284,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         self.assertTrue(
             self.propertyStore.contains(document.root_element.qname()))
 
-
     def test_containsError(self):
         """
         If there is a problem checking if the specified property exists (aside
@@ -316,7 +293,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         """
         self._forbiddenTest('contains')
 
-
     def test_containsMissing(self):
         """
         Test missing file.
@@ -330,7 +306,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         document = self._makeValue()
         self.assertFalse(propertyStore.contains(document.root_element.qname()))
 
-
     def test_list(self):
         """
         L{xattrPropertyStore.list} returns a C{list} of property names
@@ -343,7 +318,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
             set(self.propertyStore.list()),
             set([(u'foo', u'bar'), (u'bar', u'baz')]))
 
-
     def test_listError(self):
         """
         If there is a problem checking if the specified property exists (aside
@@ -366,7 +340,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         # of the EPERM failure.
         self.assertEquals(error.response.code, FORBIDDEN)
 
-
     def test_listMissing(self):
         """
         Test missing file.
@@ -379,7 +352,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
         # Try to get a property from it - and fail.
         self.assertEqual(propertyStore.list(), [])
 
-
     def test_get_uids(self):
         """
         L{xattrPropertyStore.get} accepts a L{WebDAVElement} and stores a
@@ -394,7 +366,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
             document = self._makeValue(uid)
             self._checkValue(document, uid=uid)
 
-
     def test_set_uids(self):
         """
         L{xattrPropertyStore.set} accepts a L{WebDAVElement} and stores a
@@ -407,7 +378,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
             self.assertEquals(
                 decompress(self._getValue(document, uid)), document.root_element.toxml(pretty=False))
 
-
     def test_delete_uids(self):
         """
         L{xattrPropertyStore.set} accepts a L{WebDAVElement} and stores a
@@ -427,7 +397,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
                 self.assertEquals(
                     decompress(self._getValue(document, uid)), document.root_element.toxml(pretty=False))
 
-
     def test_contains_uids(self):
         """
         L{xattrPropertyStore.contains} returns C{True} if the given property
@@ -441,7 +410,6 @@ class ExtendedAttributesPropertyStoreTests(TestCase):
             self.assertTrue(
                 self.propertyStore.contains(document.root_element.qname(), uid=uid))
 
-
     def test_list_uids(self):
         """
         L{xattrPropertyStore.list} returns a C{list} of property names
