@@ -1,7 +1,7 @@
 # -*- test-case-name: txweb2.test.test_http -*-
 ##
 # Copyright (c) 2001-2004 Twisted Matrix Laboratories.
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -53,6 +53,7 @@ log = Logger()
 
 defaultPortForScheme = {'http': 80, 'https': 443, 'ftp': 21}
 
+
 def splitHostPort(scheme, hostport):
     """Split the host in "host:port" format into host and port fields.
     If port was not specified, use the default for the given scheme, if
@@ -68,7 +69,6 @@ def splitHostPort(scheme, hostport):
     return hostport[0], defaultPortForScheme.get(scheme, 0)
 
 
-
 def parseVersion(strversion):
     """Parse version strings of the form Protocol '/' Major '.' Minor. E.g. 'HTTP/1.1'.
     Returns (protocol, major, minor).
@@ -82,7 +82,6 @@ def parseVersion(strversion):
     return (proto.lower(), major, minor)
 
 
-
 class HTTPError(Exception):
 
     def __init__(self, codeOrResponse):
@@ -95,12 +94,10 @@ class HTTPError(Exception):
         self.response = iweb.IResponse(codeOrResponse)
         Exception.__init__(self, str(self.response))
 
-
     def __repr__(self):
         return "<%s %s>" % (self.__class__.__name__, self.response)
 
 
-
 class Response(object):
     """An object representing an HTTP Response to be sent to the client.
     """
@@ -136,7 +133,6 @@ class Response(object):
         if stream is not None:
             self.stream = IByteStream(stream)
 
-
     def __repr__(self):
         if self.stream is None:
             streamlen = None
@@ -146,7 +142,6 @@ class Response(object):
         return "<%s.%s code=%d, streamlen=%s>" % (self.__module__, self.__class__.__name__, self.code, streamlen)
 
 
-
 class StatusResponseElement(Element):
     """
     Render the HTML for a L{StatusResponse}
@@ -164,7 +159,6 @@ class StatusResponseElement(Element):
         self.title = title
         self.description = description
 
-
     @renderer
     def response(self, request, tag):
         """
@@ -173,7 +167,6 @@ class StatusResponseElement(Element):
         return tag.fillSlots(title=self.title, description=self.description)
 
 
-
 class StatusResponse (Response):
     """
     A L{Response} object which simply contains a status code and a description
@@ -203,16 +196,15 @@ class StatusResponse (Response):
 
         self.description = description
 
-
     def __repr__(self):
         return "<%s %s %s>" % (self.__class__.__name__, self.code, self.description)
 
 
-
 class RedirectResponse (StatusResponse):
     """
     A L{Response} object that contains a redirect to another network location.
     """
+
     def __init__(self, location, temporary=False):
         """
         @param location: the URI to redirect to.
@@ -227,7 +219,6 @@ class RedirectResponse (StatusResponse):
         self.headers.setHeader("location", location)
 
 
-
 def NotModifiedResponse(oldResponse=None):
     if oldResponse is not None:
         headers = http_headers.Headers()
@@ -246,7 +237,6 @@ def NotModifiedResponse(oldResponse=None):
     return Response(code=responsecode.NOT_MODIFIED, headers=headers)
 
 
-
 def checkPreconditions(request, response=None, entityExists=True, etag=None, lastModified=None):
     """Check to see if this request passes the conditional checks specified
     by the client. May raise an HTTPError with result codes L{NOT_MODIFIED}
@@ -288,13 +278,12 @@ def checkPreconditions(request, response=None, entityExists=True, etag=None, las
     if response:
         assert etag is None and lastModified is None
         # if the code is some sort of error code, don't do anything
-        if not ((response.code >= 200 and response.code <= 299)
-                or response.code == responsecode.PRECONDITION_FAILED):
+        if not ((response.code >= 200 and response.code <= 299) or
+                response.code == responsecode.PRECONDITION_FAILED):
             return False
         etag = response.headers.getHeader("etag")
         lastModified = response.headers.getHeader("last-modified")
 
-
     def matchETag(tags, allowWeak):
         if entityExists and '*' in tags:
             return True
@@ -348,7 +337,6 @@ def checkPreconditions(request, response=None, entityExists=True, etag=None, las
             raise HTTPError(NotModifiedResponse(response))
 
 
-
 def checkIfRange(request, response):
     """Checks for the If-Range header, and if it exists, checks if the
     test passes. Returns true if the server should return partial data."""
@@ -363,7 +351,6 @@ def checkIfRange(request, response):
         return ifrange == response.headers.getHeader("last-modified")
 
 
-
 class _NotifyingProducerStream(stream.ProducerStream):
     doStartReading = None
 
@@ -371,7 +358,6 @@ class _NotifyingProducerStream(stream.ProducerStream):
         stream.ProducerStream.__init__(self, length=length)
         self.doStartReading = doStartReading
 
-
     def read(self):
         if self.doStartReading is not None:
             doStartReading = self.doStartReading
@@ -380,12 +366,10 @@ class _NotifyingProducerStream(stream.ProducerStream):
 
         return stream.ProducerStream.read(self)
 
-
     def write(self, data):
         self.doStartReading = None
         stream.ProducerStream.write(self, data)
 
-
     def finish(self):
         self.doStartReading = None
         stream.ProducerStream.finish(self)
@@ -394,6 +378,7 @@ class _NotifyingProducerStream(stream.ProducerStream):
 # response codes that must have empty bodies
 NO_BODY_CODES = (responsecode.NO_CONTENT, responsecode.NOT_MODIFIED)
 
+
 class Request(object):
     """A HTTP request.
 
@@ -429,7 +414,6 @@ class Request(object):
         self.stream = _NotifyingProducerStream(contentLength, doStartReading)
         self.stream.registerProducer(self.chanRequest, True)
 
-
     def checkExpect(self):
         """Ensure there are no expectations that cannot be met.
         Checks Expect header against self.known_expects."""
@@ -438,44 +422,36 @@ class Request(object):
             if expect not in self.known_expects:
                 raise HTTPError(responsecode.EXPECTATION_FAILED)
 
-
     def process(self):
         """Called by channel to let you process the request.
 
         Can be overridden by a subclass to do something useful."""
         pass
 
-
     def handleContentChunk(self, data):
         """Callback from channel when a piece of data has been received.
         Puts the data in .stream"""
         self.stream.write(data)
 
-
     def handleContentComplete(self):
         """Callback from channel when all data has been received. """
         self.stream.unregisterProducer()
         self.stream.finish()
 
-
     def connectionLost(self, reason):
         """connection was lost"""
         pass
 
-
     def __repr__(self):
         return '<%s %s %s>' % (self.method, self.uri, self.clientproto)
 
-
     def _sendContinue(self):
         self.chanRequest.writeIntermediateResponse(responsecode.CONTINUE)
 
-
     def _reallyFinished(self, x):
         """We are finished writing data."""
         self.chanRequest.finish()
 
-
     def _finished(self, x):
         """
         We are finished writing data.
@@ -489,16 +465,14 @@ class Request(object):
         else:
             self._reallyFinished(x)
 
-
     def _error(self, reason):
         if reason.check(error.ConnectionLost):
-            log.info("Request error: {message}", message=reason.getErrorMessage())
+            log.info("Request error: {msg}", msg=reason.getErrorMessage())
         else:
             log.failure("Request error", reason)
             # Only bother with cleanup on errors other than lost connection.
             self.chanRequest.abortConnection()
 
-
     def writeResponse(self, response):
         """
         Write a response.
@@ -537,12 +511,12 @@ class Request(object):
         d.addCallback(self._finished).addErrback(self._error)
 
 
-
 class XMLResponse (Response):
     """
     XML L{Response} object.
     Renders itself as an XML document.
     """
+
     def __init__(self, code, element):
         """
         @param xml_responses: an iterable of davxml.Response objects.
@@ -551,12 +525,12 @@ class XMLResponse (Response):
         self.headers.setHeader("content-type", http_headers.MimeType("text", "xml"))
 
 
-
 class JSONResponse (Response):
     """
     JSON L{Response} object.
     Renders itself as an JSON document.
     """
+
     def __init__(self, code, jobj, contentType="application/json", pretty=False):
         """
         @param jobj: a Python object that can be serialized to JSON.
