@@ -1,7 +1,7 @@
 # -*- test-case-name: txweb2.test.test_http_headers -*-
 ##
 # Copyright (c) 2008 Twisted Matrix Laboratories.
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -33,6 +33,7 @@ from calendar import timegm
 import base64
 import re
 
+
 def dashCapitalize(s):
     ''' Capitalize a string, making sure to treat - as a word separator '''
     return '-'.join([x.capitalize() for x in s.split('-')])
@@ -49,18 +50,17 @@ monthname_lower = [name and name.lower() for name in monthname]
 
 header_case_mapping = {}
 
+
 def casemappingify(d):
     global header_case_mapping
     newd = dict([(key.lower(), key) for key in d.keys()])
     header_case_mapping.update(newd)
 
 
-
 def lowerify(d):
     return dict([(key.lower(), value) for key, value in d.items()])
 
 
-
 class HeaderHandler(object):
     """HeaderHandler manages header generating and parsing functions.
     """
@@ -81,7 +81,6 @@ class HeaderHandler(object):
         if generators:
             self.HTTPGenerators.update(generators)
 
-
     def parse(self, name, header):
         """
         Parse the given header based on its given name.
@@ -110,7 +109,6 @@ class HeaderHandler(object):
 
         return header
 
-
     def generate(self, name, header):
         """
         Generate the given header based on its given name.
@@ -135,7 +133,6 @@ class HeaderHandler(object):
         # self._raw_headers[name] = h
         return header
 
-
     def updateParsers(self, parsers):
         """Update en masse the parser maps.
 
@@ -145,7 +142,6 @@ class HeaderHandler(object):
         casemappingify(parsers)
         self.HTTPParsers.update(lowerify(parsers))
 
-
     def addParser(self, name, value):
         """Add an individual parser chain for the given header.
 
@@ -157,7 +153,6 @@ class HeaderHandler(object):
         """
         self.updateParsers({name: value})
 
-
     def updateGenerators(self, generators):
         """Update en masse the generator maps.
 
@@ -167,7 +162,6 @@ class HeaderHandler(object):
         casemappingify(generators)
         self.HTTPGenerators.update(lowerify(generators))
 
-
     def addGenerators(self, name, value):
         """Add an individual generator chain for the given header.
 
@@ -179,7 +173,6 @@ class HeaderHandler(object):
         """
         self.updateGenerators({name: value})
 
-
     def update(self, parsers, generators):
         """Conveniently update parsers and generators all at once.
         """
@@ -242,18 +235,17 @@ def parseDateTime(dateString):
     return int(timegm((year, month, day, hour, min, sec)))
 
 
-
 # HTTP tokenizer
 class Token(str):
     __slots__ = []
     tokens = {}
+
     def __new__(self, char):
         token = Token.tokens.get(char)
         if token is None:
             Token.tokens[char] = token = str.__new__(self, char)
         return token
 
-
     def __repr__(self):
         return "Token(%s)" % str.__repr__(self)
 
@@ -262,6 +254,7 @@ class Token(str):
 http_tokens = " \t\"()<>@,;:\\/[]?={}"
 http_ctls = "\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f\x7f"
 
+
 def tokenize(header, foldCase=True):
     """Tokenize a string according to normal HTTP header parsing rules.
 
@@ -342,7 +335,6 @@ def tokenize(header, foldCase=True):
             yield string[start:cur]
 
 
-
 def split(seq, delim):
     """The same as str.split but works on arbitrary sequences.
     Too bad it's not builtin to python!"""
@@ -365,7 +357,6 @@ def split(seq, delim):
 #         return -1
 
 
-
 def filterTokens(seq):
     """Filter out instances of Token, leaving only a list of strings.
 
@@ -384,7 +375,6 @@ def filterTokens(seq):
     return l
 
 
-
 # parser utilities:
 def checkSingleToken(tokens):
     if len(tokens) != 1:
@@ -392,7 +382,6 @@ def checkSingleToken(tokens):
     return tokens[0]
 
 
-
 def parseKeyValue(val):
     if len(val) == 1:
         return val[0], None
@@ -401,7 +390,6 @@ def parseKeyValue(val):
     raise ValueError("Expected key or key=value, but got %s." % (val,))
 
 
-
 def parseArgs(field):
     args = split(field, Token(';'))
     val = args.next()
@@ -409,7 +397,6 @@ def parseArgs(field):
     return val, args
 
 
-
 def listParser(fun):
     """Return a function which applies 'fun' to every element in the
     comma-separated list"""
@@ -422,14 +409,12 @@ def listParser(fun):
     return listParserHelper
 
 
-
 def last(seq):
     """Return seq[-1]"""
 
     return seq[-1]
 
 
-
 # Generation utilities
 def quoteString(s):
     """
@@ -442,7 +427,6 @@ def quoteString(s):
     return '"%s"' % s.replace('\\', '\\\\').replace('"', '\\"')
 
 
-
 def listGenerator(fun):
     """Return a function which applies 'fun' to every element in
     the given list, then joins the result with generateList"""
@@ -452,17 +436,16 @@ def listGenerator(fun):
     return listGeneratorHelper
 
 
-
 def generateList(seq):
     return ", ".join(seq)
 
 
-
 def singleHeader(item):
     return [item]
 
 _seperators = re.compile('[' + re.escape(http_tokens) + ']')
 
+
 def generateKeyValues(parameters):
     """
     Format an iterable of key/value pairs.
@@ -490,7 +473,6 @@ def generateKeyValues(parameters):
     return ";".join(l)
 
 
-
 class MimeType(object):
 
     def fromString(cls, mimeTypeString):
@@ -519,7 +501,6 @@ class MimeType(object):
         if kwargs:
             self.params.update(kwargs)
 
-
     def __eq__(self, other):
         if not isinstance(other, MimeType):
             return NotImplemented
@@ -527,20 +508,16 @@ class MimeType(object):
                 self.mediaSubtype == other.mediaSubtype and
                 self.params == other.params)
 
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
     def __repr__(self):
         return "MimeType(%r, %r, %r)" % (self.mediaType, self.mediaSubtype, self.params)
 
-
     def __hash__(self):
         return hash(self.mediaType) ^ hash(self.mediaSubtype) ^ hash(tuple(self.params.iteritems()))
 
 
-
 class MimeDisposition(object):
 
     def fromString(cls, dispositionString):
@@ -568,27 +545,22 @@ class MimeDisposition(object):
         if kwargs:
             self.params.update(kwargs)
 
-
     def __eq__(self, other):
         if not isinstance(other, MimeDisposition):
             return NotImplemented
         return (self.dispositionType == other.dispositionType and
                 self.params == other.params)
 
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
     def __repr__(self):
         return "MimeDisposition(%r, %r)" % (self.dispositionType, self.params)
 
-
     def __hash__(self):
         return hash(self.dispositionType) ^ hash(tuple(self.params.iteritems()))
 
 
-
 # Specific header parsers.
 def parseAccept(field):
     atype, args = parseArgs(field)
@@ -626,27 +598,24 @@ def parseAccept(field):
     return ret
 
 
-
 def parseAcceptQvalue(field):
     atype, args = parseArgs(field)
 
     atype = checkSingleToken(atype)
 
-    qvalue = 1.0 # Default qvalue is 1
+    qvalue = 1.0  # Default qvalue is 1
     for arg in args:
         if arg[0] == 'q':
             qvalue = float(arg[1])
     return atype, qvalue
 
 
-
 def addDefaultCharset(charsets):
     if charsets.get('*') is None and charsets.get('iso-8859-1') is None:
         charsets['iso-8859-1'] = 1.0
     return charsets
 
 
-
 def addDefaultEncoding(encodings):
     if encodings.get('*') is None and encodings.get('identity') is None:
         # RFC doesn't specify a default value for identity, only that it
@@ -655,7 +624,6 @@ def addDefaultEncoding(encodings):
     return encodings
 
 
-
 def parseContentType(header):
     # Case folding is disabled for this header, because of use of
     # Content-Type: multipart/form-data; boundary=CaSeFuLsTuFf
@@ -671,7 +639,6 @@ def parseContentType(header):
     return MimeType(ctype[0].lower(), ctype[2].lower(), tuple(args))
 
 
-
 def parseContentDisposition(header):
     # Case folding is disabled for this header, because of use of
     # So, we need to explicitly .lower() the dtype and arg keys.
@@ -686,7 +653,6 @@ def parseContentDisposition(header):
     return MimeDisposition(dtype[0].lower(), tuple(args))
 
 
-
 def parseContentMD5(header):
     try:
         return base64.decodestring(header)
@@ -694,7 +660,6 @@ def parseContentMD5(header):
         raise ValueError(e)
 
 
-
 def parseContentRange(header):
     """Parse a content-range header into (kind, start, end, realLength).
 
@@ -716,7 +681,6 @@ def parseContentRange(header):
     return (kind, start, end, realLength)
 
 
-
 def parseExpect(field):
     etype, args = parseArgs(field)
 
@@ -724,7 +688,6 @@ def parseExpect(field):
     return (etype[0], (lambda *args: args)(etype[1], *args))
 
 
-
 def parseExpires(header):
     # """HTTP/1.1 clients and caches MUST treat other invalid date formats,
     #    especially including the value 0, as in the past (i.e., "already expired")."""
@@ -735,7 +698,6 @@ def parseExpires(header):
         return 0
 
 
-
 def parseIfModifiedSince(header):
     # Ancient versions of netscape and *current* versions of MSIE send
     #   If-Modified-Since: Thu, 05 Aug 2004 12:57:27 GMT; length=123
@@ -746,7 +708,6 @@ def parseIfModifiedSince(header):
     return parseDateTime(header.split(';', 1)[0])
 
 
-
 def parseIfRange(headers):
     try:
         return ETag.parse(tokenize(headers))
@@ -754,7 +715,6 @@ def parseIfRange(headers):
         return parseDateTime(last(headers))
 
 
-
 def parseRange(crange):
     crange = list(crange)
     if len(crange) < 3 or crange[1] != Token('='):
@@ -790,7 +750,6 @@ def parseRange(crange):
     return rtype, ranges
 
 
-
 def parseRetryAfter(header):
     try:
         # delta seconds
@@ -802,7 +761,6 @@ def parseRetryAfter(header):
 # WWW-Authenticate and Authorization
 
 
-
 def parseWWWAuthenticate(tokenized):
     headers = []
 
@@ -847,7 +805,6 @@ def parseWWWAuthenticate(tokenized):
     return headers
 
 
-
 def parseAuthorization(header):
     scheme, rest = header.split(' ', 1)
     # this header isn't tokenized because it may eat characters
@@ -855,14 +812,12 @@ def parseAuthorization(header):
     return scheme.lower(), rest
 
 
-
 def parsePrefer(field):
     etype, args = parseArgs(field)
     etype = parseKeyValue(etype)
     return (etype[0], etype[1], args)
 
 
-
 # Header generators
 def generateAccept(accept):
     mimeType, q = accept
@@ -877,14 +832,12 @@ def generateAccept(accept):
     return out
 
 
-
 def removeDefaultEncoding(seq):
     for item in seq:
         if item[0] != 'identity' or item[1] != .0001:
             yield item
 
 
-
 def generateAcceptQvalue(keyvalue):
     if keyvalue[1] == 1.0:
         return "%s" % keyvalue[0:1]
@@ -892,7 +845,6 @@ def generateAcceptQvalue(keyvalue):
         return ("%s;q=%.3f" % keyvalue).rstrip('0').rstrip('.')
 
 
-
 def parseCacheControl(kv):
     k, v = parseKeyValue(kv)
     if k == 'max-age' or k == 'min-fresh' or k == 's-maxage':
@@ -912,7 +864,6 @@ def parseCacheControl(kv):
     return k, v
 
 
-
 def generateCacheControl((k, v)):
     if v is None:
         return str(k)
@@ -924,7 +875,6 @@ def generateCacheControl((k, v)):
         return '%s=%s' % (k, v)
 
 
-
 def generateContentRange(tup):
     """tup is (rtype, start, end, rlen)
     rlen can be None.
@@ -942,7 +892,6 @@ def generateContentRange(tup):
     return '%s %s/%s' % (rtype, startend, rlen)
 
 
-
 def generateDateTime(secSinceEpoch):
     """Convert seconds since epoch to HTTP datetime string."""
     year, month, day, hh, mm, ss, wd, _ignore_y, _ignore_z = time.gmtime(secSinceEpoch)
@@ -953,7 +902,6 @@ def generateDateTime(secSinceEpoch):
     return s
 
 
-
 def generateExpect(item):
     if item[1][0] is None:
         out = '%s' % (item[0],)
@@ -964,7 +912,6 @@ def generateExpect(item):
     return out
 
 
-
 def generateRange(crange):
     def noneOr(s):
         if s is None:
@@ -981,13 +928,11 @@ def generateRange(crange):
                       for startend in ranges]))
 
 
-
 def generateRetryAfter(when):
     # always generate delta seconds format
     return str(int(when - time.time()))
 
 
-
 def generateContentType(mimeType):
     out = "%s/%s" % (mimeType.mediaType, mimeType.mediaSubtype)
     if mimeType.params:
@@ -995,7 +940,6 @@ def generateContentType(mimeType):
     return out
 
 
-
 def generateContentDisposition(disposition):
     out = disposition.dispositionType
     if disposition.params:
@@ -1003,7 +947,6 @@ def generateContentDisposition(disposition):
     return out
 
 
-
 def generateIfRange(dateOrETag):
     if isinstance(dateOrETag, ETag):
         return dateOrETag.generate()
@@ -1013,7 +956,6 @@ def generateIfRange(dateOrETag):
 # WWW-Authenticate and Authorization
 
 
-
 def generateWWWAuthenticate(headers):
     _generated = []
     for seq in headers:
@@ -1034,12 +976,10 @@ def generateWWWAuthenticate(headers):
     return _generated
 
 
-
 def generateAuthorization(seq):
     return [' '.join(seq)]
 
 
-
 def generatePrefer(items):
     key, value, args = items
     if value is None:
@@ -1051,14 +991,13 @@ def generatePrefer(items):
     return out
 
 
-
 ####
 class ETag(object):
+
     def __init__(self, tag, weak=False):
         self.tag = str(tag)
         self.weak = weak
 
-
     def match(self, other, strongCompare):
         # Sec 13.3.
         # The strong comparison function: in order to be considered equal, both
@@ -1075,26 +1014,22 @@ class ETag(object):
             return False
         return True
 
-
     def __eq__(self, other):
         return isinstance(other, ETag) and other.tag == self.tag and other.weak == self.weak
 
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
     def __repr__(self):
         return "Etag(%r, weak=%r)" % (self.tag, self.weak)
 
-
     def parse(tokens):
         tokens = tuple(tokens)
         if len(tokens) == 1 and not isinstance(tokens[0], Token):
             return ETag(tokens[0])
 
-        if(len(tokens) == 3 and tokens[0] == "w"
-           and tokens[1] == Token('/')):
+        if(len(tokens) == 3 and tokens[0] == "w" and
+           tokens[1] == Token('/')):
             return ETag(tokens[2], weak=True)
 
         raise ValueError("Invalid ETag.")
@@ -1108,7 +1043,6 @@ class ETag(object):
             return quoteString(self.tag)
 
 
-
 def parseStarOrETag(tokens):
     tokens = tuple(tokens)
     if tokens == ('*',):
@@ -1117,7 +1051,6 @@ def parseStarOrETag(tokens):
         return ETag.parse(tokens)
 
 
-
 def generateStarOrETag(etag):
     if etag == '*':
         return etag
@@ -1125,7 +1058,6 @@ def generateStarOrETag(etag):
         return etag.generate()
 
 
-
 # Cookies. Blech!
 class Cookie(object):
     # __slots__ = ['name', 'value', 'path', 'domain', 'ports', 'expires', 'discard', 'secure', 'comment', 'commenturl', 'version']
@@ -1143,7 +1075,6 @@ class Cookie(object):
         self.commenturl = commenturl
         self.version = version
 
-
     def __repr__(self):
         s = "Cookie(%r=%r" % (self.name, self.value)
         if self.path is not None:
@@ -1165,7 +1096,6 @@ class Cookie(object):
         s += ")"
         return s
 
-
     def __eq__(self, other):
         return (isinstance(other, Cookie) and
                 other.path == self.path and
@@ -1177,12 +1107,10 @@ class Cookie(object):
                 other.commenturl == self.commenturl and
                 other.version == self.version)
 
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
 
-
 def parseCookie(headers):
     """Bleargh, the cookie spec sucks.
     This surely needs interoperability testing.
@@ -1250,7 +1178,6 @@ cookie_validvalue = cookie_validname + '|"([^"]|\\\\")*"$'
 cookie_validvalue_re = re.compile(cookie_validvalue)
 
 
-
 def generateCookie(cookies):
     # There's a fundamental problem with the two cookie specifications.
     # They both use the "Cookie" header, and the RFC Cookie header only allows
@@ -1329,7 +1256,6 @@ def generateCookie(cookies):
         return ';'.join(str_cookies)
 
 
-
 def parseSetCookie(headers):
     setCookies = []
     for header in headers:
@@ -1358,7 +1284,6 @@ def parseSetCookie(headers):
     return setCookies
 
 
-
 def parseSetCookie2(toks):
     outCookies = []
     for cookie in [[parseKeyValue(x) for x in split(y, Token(';'))]
@@ -1371,7 +1296,6 @@ def parseSetCookie2(toks):
     return outCookies
 
 
-
 def makeCookieFromList(tup, netscapeFormat):
     name, value = tup[0]
     if name is None or value is None:
@@ -1415,7 +1339,6 @@ def makeCookieFromList(tup, netscapeFormat):
     return cookie
 
 
-
 def generateSetCookie(cookies):
     setCookies = []
     for cookie in cookies:
@@ -1433,7 +1356,6 @@ def generateSetCookie(cookies):
     return setCookies
 
 
-
 def generateSetCookie2(cookies):
     setCookies = []
     for cookie in cookies:
@@ -1462,14 +1384,12 @@ def generateSetCookie2(cookies):
     return setCookies
 
 
-
 def parseDepth(depth):
     if depth not in ("0", "1", "infinity"):
         raise ValueError("Invalid depth header value: %s" % (depth,))
     return depth
 
 
-
 def parseOverWrite(overwrite):
     if overwrite == "F":
         return False
@@ -1478,7 +1398,6 @@ def parseOverWrite(overwrite):
     raise ValueError("Invalid overwrite header value: %s" % (overwrite,))
 
 
-
 def generateOverWrite(overwrite):
     if overwrite:
         return "T"
@@ -1486,7 +1405,6 @@ def generateOverWrite(overwrite):
         return "F"
 
 
-
 def parseBrief(brief):
     # We accept upper or lower case
     if brief.upper() == "F":
@@ -1496,7 +1414,6 @@ def parseBrief(brief):
     raise ValueError("Invalid brief header value: %s" % (brief,))
 
 
-
 def generateBrief(brief):
     # MS definition uses lower case
     return "t" if brief else "f"
@@ -1542,9 +1459,9 @@ def generateBrief(brief):
 #     return accepts.get('*')
 
 
-
 # Headers object
 class __RecalcNeeded(object):
+
     def __repr__(self):
         return "<RecalcNeeded>"
 
@@ -1569,12 +1486,10 @@ class Headers(object):
             for key, value in rawHeaders.iteritems():
                 self.setRawHeaders(key, value)
 
-
     def _setRawHeaders(self, headers):
         self._raw_headers = headers
         self._headers = {}
 
-
     def _toParsed(self, name):
         r = self._raw_headers.get(name, None)
         h = self.handler.parse(name, r)
@@ -1582,7 +1497,6 @@ class Headers(object):
             self._headers[name] = h
         return h
 
-
     def _toRaw(self, name):
         h = self._headers.get(name, None)
         r = self.handler.generate(name, h)
@@ -1590,13 +1504,11 @@ class Headers(object):
             self._raw_headers[name] = r
         return r
 
-
     def hasHeader(self, name):
         """Does a header with the given name exist?"""
         name = name.lower()
         return name in self._raw_headers
 
-
     def getRawHeaders(self, name, default=None):
         """Returns a list of headers matching the given name as the raw string given."""
 
@@ -1607,7 +1519,6 @@ class Headers(object):
 
         return self._toRaw(name)
 
-
     def getHeader(self, name, default=None):
         """Ret9urns the parsed representation of the given header.
         The exact form of the return value depends on the header in question.
@@ -1622,7 +1533,6 @@ class Headers(object):
             return parsed
         return self._toParsed(name)
 
-
     def setRawHeaders(self, name, value):
         """Sets the raw representation of the given header.
         Value should be a list of strings, each being one header of the
@@ -1632,7 +1542,6 @@ class Headers(object):
         self._raw_headers[name] = value
         self._headers[name] = _RecalcNeeded
 
-
     def setHeader(self, name, value):
         """Sets the parsed representation of the given header.
         Value should be a list of objects whose exact form depends
@@ -1642,7 +1551,6 @@ class Headers(object):
         self._raw_headers[name] = _RecalcNeeded
         self._headers[name] = value
 
-
     def addRawHeader(self, name, value):
         """
         Add a raw value to a header that may or may not already exist.
@@ -1661,7 +1569,6 @@ class Headers(object):
         raw_header.append(value)
         self._headers[name] = _RecalcNeeded
 
-
     def removeHeader(self, name):
         """Removes the header named."""
 
@@ -1670,17 +1577,14 @@ class Headers(object):
             del self._raw_headers[name]
             del self._headers[name]
 
-
     def __repr__(self):
         return '<Headers: Raw: %s Parsed: %s>' % (self._raw_headers, self._headers)
 
-
     def canonicalNameCaps(self, name):
         """Return the name with the canonical capitalization, if known,
         otherwise, Caps-After-Dashes"""
         return header_case_mapping.get(name) or dashCapitalize(name)
 
-
     def getAllRawHeaders(self):
         """Return an iterator of key,value pairs of all headers
         contained in this object, as strings. The keys are capitalized
@@ -1690,13 +1594,11 @@ class Headers(object):
                 v = self._toRaw(k)
             yield self.canonicalNameCaps(k), v
 
-
     def makeImmutable(self):
         """Make this header set immutable. All mutating operations will
         raise an exception."""
         self.setHeader = self.setRawHeaders = self.removeHeader = self._mutateRaise
 
-
     def _mutateRaise(self, *args):
         raise AttributeError("This header object is immutable as the headers have already been sent.")
 
@@ -1755,7 +1657,7 @@ parser_request_headers = {
     'Prefer': (tokenize, listParser(parsePrefer), list),
     # 'Proxy-Authorization': str, # what is "credentials"
     'Range': (tokenize, parseRange),
-    'Referer': (last, str), # TODO: URI object?
+    'Referer': (last, str),  # TODO: URI object?
     'TE': (tokenize, listParser(parseAcceptQvalue), dict),
     'User-Agent': (last, str),
 }
@@ -1765,7 +1667,7 @@ generator_request_headers = {
     'Accept-Charset': (iteritems, listGenerator(generateAcceptQvalue), singleHeader),
     'Accept-Encoding': (iteritems, removeDefaultEncoding, listGenerator(generateAcceptQvalue), singleHeader),
     'Accept-Language': (iteritems, listGenerator(generateAcceptQvalue), singleHeader),
-    'Authorization': (generateAuthorization,), # what is "credentials"
+    'Authorization': (generateAuthorization,),  # what is "credentials"
     'Cookie': (generateCookie, singleHeader),
     'Expect': (iteritems, listGenerator(generateExpect), singleHeader),
     'From': (str, singleHeader),
@@ -1788,7 +1690,7 @@ parser_response_headers = {
     'Accept-Ranges': (tokenize, filterTokens),
     'Age': (last, int),
     'ETag': (tokenize, ETag.parse),
-    'Location': (last,), # TODO: URI object?
+    'Location': (last,),  # TODO: URI object?
     # 'Proxy-Authenticate'
     'Retry-After': (last, parseRetryAfter),
     'Server': (last,),
@@ -1819,7 +1721,7 @@ parser_entity_headers = {
     'Content-Encoding': (tokenize, filterTokens),
     'Content-Language': (tokenize, filterTokens),
     'Content-Length': (last, int),
-    'Content-Location': (last,), # TODO: URI object?
+    'Content-Location': (last,),  # TODO: URI object?
     'Content-MD5': (last, parseContentMD5),
     'Content-Range': (last, parseContentRange),
     'Content-Type': (lambda hdr: tokenize(hdr, foldCase=False), parseContentType),
@@ -1842,25 +1744,25 @@ generator_entity_headers = {
 }
 
 parser_dav_headers = {
-    'Brief'       : (last, parseBrief),
-    'DAV'         : (tokenize, list),
-    'Depth'       : (last, parseDepth),
-    'Destination' : (last,), # TODO: URI object?
+    'Brief': (last, parseBrief),
+    'DAV': (tokenize, list),
+    'Depth': (last, parseDepth),
+    'Destination': (last,),  # TODO: URI object?
     # 'If'          : (),
     # 'Lock-Token'  : (),
-    'Overwrite'   : (last, parseOverWrite),
+    'Overwrite': (last, parseOverWrite),
     # 'Status-URI'  : (),
     # 'Timeout'     : (),
 }
 
 generator_dav_headers = {
-    'Brief'       : (),
-    'DAV'         : (generateList, singleHeader),
-    'Depth'       : (singleHeader),
-    'Destination' : (singleHeader),
+    'Brief': (),
+    'DAV': (generateList, singleHeader),
+    'Depth': (singleHeader),
+    'Destination': (singleHeader),
     # 'If'          : (),
     # 'Lock-Token'  : (),
-    'Overwrite'   : (),
+    'Overwrite': (),
     # 'Status-URI'  : (),
     # 'Timeout'     : (),
 }
