@@ -1,6 +1,6 @@
 # -*- test-case-name: txweb2.test.test_metafd -*-
 ##
-# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2017 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -40,7 +40,6 @@ from twext.internet.socketfile import MaxAcceptSocketFileServer
 log = Logger()
 
 
-
 class JustEnoughLikeAPort(object):
     """
     Fake out just enough of L{tcp.Port} to be acceptable to
@@ -49,7 +48,6 @@ class JustEnoughLikeAPort(object):
     _realPortNumber = 'inherited'
 
 
-
 class ReportingHTTPService(Service, object):
     """
     Service which starts up an HTTP server that can report back to its parent
@@ -84,7 +82,6 @@ class ReportingHTTPService(Service, object):
         # actually doing startTLS ourselves.
         self.usingSocketFile = usingSocketFile
 
-
     def startService(self):
         """
         Start reading on the inherited port.
@@ -97,7 +94,6 @@ class ReportingHTTPService(Service, object):
         inheritedPort.startReading()
         inheritedPort.reportStatus("0")
 
-
     def stopService(self):
         """
         Stop reading on the inherited port.
@@ -113,7 +109,6 @@ class ReportingHTTPService(Service, object):
         # Let any outstanding requests finish
         return self.reportingFactory.allConnectionsClosed()
 
-
     def createTransport(self, skt, peer, data, protocol):
         """
         Create a TCP transport, from a socket object passed by the parent.
@@ -125,7 +120,7 @@ class ReportingHTTPService(Service, object):
             if self.usingSocketFile:
                 # Mark the transport as "secure", enough for getHostInfo() to
                 # think so
-                transport.getPeerCertificate = lambda _ : None
+                transport.getPeerCertificate = lambda _: None
                 directlyProvides(transport, ISSLTransport)
             else:
                 transport.startTLS(self.contextFactory)
@@ -133,7 +128,6 @@ class ReportingHTTPService(Service, object):
         return transport
 
 
-
 class ReportingHTTPFactory(HTTPFactory):
     """
     An L{HTTPFactory} which reports its status to a
@@ -156,7 +150,6 @@ class ReportingHTTPFactory(HTTPFactory):
         """
         self.inheritedPort.reportStatus(message)
 
-
     def addConnectedChannel(self, channel):
         """
         Add the connected channel, and report the current number of open
@@ -165,7 +158,6 @@ class ReportingHTTPFactory(HTTPFactory):
         HTTPFactory.addConnectedChannel(self, channel)
         self._report("+")
 
-
     def removeConnectedChannel(self, channel):
         """
         Remove the connected channel, and report the current number of open
@@ -175,7 +167,6 @@ class ReportingHTTPFactory(HTTPFactory):
         self._report("-")
 
 
-
 @implementer(IStatus)
 class WorkerStatus(FancyStrMixin, object):
     """
@@ -235,18 +226,15 @@ class WorkerStatus(FancyStrMixin, object):
         self.starting = starting
         self.stopped = stopped
 
-
     def items(self):
         return dict([(attr, getattr(self, attr)) for attr in self.showAttributes])
 
-
     def effective(self):
         """
         The current effective load.
         """
         return self.acknowledged + self.unacknowledged
 
-
     def active(self):
         """
         Is the subprocess associated with this socket available to dispatch to.
@@ -254,7 +242,6 @@ class WorkerStatus(FancyStrMixin, object):
         """
         return self.starting == 0 and self.stopped == 0
 
-
     def start(self):
         """
         The child process for this L{WorkerStatus} is about to (re)start. Reset the status to indicate it
@@ -265,7 +252,6 @@ class WorkerStatus(FancyStrMixin, object):
             stopped=0,
         )
 
-
     def restarted(self):
         """
         The child process for this L{WorkerStatus} has indicated it is now available to accept
@@ -276,7 +262,6 @@ class WorkerStatus(FancyStrMixin, object):
             starting=0,
         )
 
-
     def stop(self):
         """
         The child process for this L{WorkerStatus} has stopped. Stop the socket and clear out
@@ -290,7 +275,6 @@ class WorkerStatus(FancyStrMixin, object):
             stopped=1,
         )
 
-
     def adjust(self, **kwargs):
         """
         Update the L{WorkerStatus} by adding the supplied values to the specified attributes.
@@ -300,7 +284,6 @@ class WorkerStatus(FancyStrMixin, object):
             setattr(self, k, max(newval, 0))
         return self
 
-
     def reset(self, **kwargs):
         """
         Reset the L{WorkerStatus} by setting the supplied values in the specified attributes.
@@ -310,7 +293,6 @@ class WorkerStatus(FancyStrMixin, object):
         return self
 
 
-
 @implementer(IStatusWatcher)
 class ConnectionLimiter(MultiService, object):
     """
@@ -320,6 +302,7 @@ class ConnectionLimiter(MultiService, object):
     """
 
     _outstandingRequests = 0
+    _maxOutstandingRequests = 0
 
     def __init__(self, maxAccepts, maxRequests):
         """
@@ -335,7 +318,6 @@ class ConnectionLimiter(MultiService, object):
         self.maxRequests = maxRequests
         self.overloaded = False
 
-
     def startService(self):
         """
         Start up multiservice, then start up the dispatcher.
@@ -343,7 +325,6 @@ class ConnectionLimiter(MultiService, object):
         super(ConnectionLimiter, self).startService()
         self.dispatcher.startDispatching()
 
-
     def addPortService(self, description, port, interface, backlog,
                        serverServiceMaker=MaxAcceptTCPServer):
         """
@@ -357,7 +338,6 @@ class ConnectionLimiter(MultiService, object):
             backlog=backlog
         ).setServiceParent(self)
 
-
     def addSocketFileService(self, description, address, backlog=None):
         lipf = LimitingInheritingProtocolFactory(self, description)
         self.factories.append(lipf)
@@ -365,7 +345,6 @@ class ConnectionLimiter(MultiService, object):
             lipf, address, backlog=backlog
         ).setServiceParent(self)
 
-
     # IStatusWatcher
 
     def initialStatus(self):
@@ -374,7 +353,6 @@ class ConnectionLimiter(MultiService, object):
         """
         return WorkerStatus()
 
-
     def statusFromMessage(self, previousStatus, message):
         """
         Determine a subprocess socket's status from its previous status and a
@@ -399,7 +377,6 @@ class ConnectionLimiter(MultiService, object):
                 unclosed=1,
             )
 
-
     def closeCountFromStatus(self, status):
         """
         Determine the number of sockets to close from the current status.
@@ -407,14 +384,12 @@ class ConnectionLimiter(MultiService, object):
         toClose = status.unclosed
         return (toClose, status.adjust(unclosed=-toClose))
 
-
     def newConnectionStatus(self, previousStatus):
         """
         A connection was just sent to the process, but not yet acknowledged.
         """
         return previousStatus.adjust(unacknowledged=1)
 
-
     def statusesChanged(self, statuses):
         """
         The L{InheritedSocketDispatcher} is reporting that the list of
@@ -428,7 +403,8 @@ class ConnectionLimiter(MultiService, object):
         """
         current = sum(status.effective()
                       for status in self.dispatcher.statuses)
-        self._outstandingRequests = current # preserve for or= field in log
+        self._outstandingRequests = current  # preserve for or= field in log
+        self._maxOutstandingRequests = max(self._maxOutstandingRequests, self._outstandingRequests)
         maximum = self.maxRequests
         overloaded = (current >= maximum)
         available = len(filter(lambda x: x.active(), self.dispatcher.statuses))
@@ -439,11 +415,19 @@ class ConnectionLimiter(MultiService, object):
             else:
                 f.loadNominal()
 
-
-    @property # make read-only
+    @property  # make read-only
     def outstandingRequests(self):
         return self._outstandingRequests
 
+    @property  # make read-only
+    def maxOutstandingRequests(self):
+        """
+        Reset the max value to the current outstanding value every time the max is read. It
+        is up to the caller to track the overall max value.
+        """
+        temp = self._maxOutstandingRequests
+        self._maxOutstandingRequests = self._outstandingRequests
+        return temp
 
 
 class LimitingInheritingProtocolFactory(InheritingProtocolFactory):
@@ -473,7 +457,6 @@ class LimitingInheritingProtocolFactory(InheritingProtocolFactory):
         self.maxRequests = limiter.maxRequests
         self.stopping = False
 
-
     def stopFactory(self):
         """
         Mark this factory as being stopped to prevent attempts to start reading on its
@@ -482,14 +465,12 @@ class LimitingInheritingProtocolFactory(InheritingProtocolFactory):
         super(LimitingInheritingProtocolFactory, self).stopFactory()
         self.stopping = True
 
-
     def loadAboveMaximum(self):
         """
         The current server load has exceeded the maximum allowable.
         """
         self.myServer.myPort.stopReading()
 
-
     def loadNominal(self):
         """
         The current server load is nominal; proceed with reading requests (but only if
@@ -498,7 +479,6 @@ class LimitingInheritingProtocolFactory(InheritingProtocolFactory):
         if not self.stopping:
             self.myServer.myPort.startReading()
 
-
     @property
     def outstandingRequests(self):
         return self.limiter.outstandingRequests
