@@ -1,7 +1,7 @@
 # -*- test-case-name: txweb2.test.test_server -*-
 ##
 # Copyright (c) 2001-2008 Twisted Matrix Laboratories.
-# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2017 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -64,13 +64,13 @@ def defaultHeadersFilter(request, response):
     return response
 defaultHeadersFilter.handleErrors = True
 
+
 def preconditionfilter(request, response):
     if request.method in ("GET", "HEAD"):
         http.checkPreconditions(request, response)
     return response
 
 
-
 def doTrace(request):
     request = iweb.IRequest(request)
     txt = "%s %s HTTP/%d.%d\r\n" % (request.method, request.uri,
@@ -88,7 +88,6 @@ def doTrace(request):
         txt)
 
 
-
 def parsePOSTData(request, maxMem=100 * 1024, maxFields=1024,
                   maxSize=10 * 1024 * 1024):
     """
@@ -115,32 +114,29 @@ def parsePOSTData(request, maxMem=100 * 1024, maxFields=1024,
     if ctype is None:
         return defer.succeed(None)
 
-
     def updateArgs(data):
         args = data
         request.args.update(args)
 
-
     def updateArgsAndFiles(data):
         args, files = data
         request.args.update(args)
         request.files.update(files)
 
-
     def error(f):
         f.trap(fileupload.MimeFormatError)
         raise http.HTTPError(
             http.StatusResponse(responsecode.BAD_REQUEST, str(f.value)))
 
     if (
-        ctype.mediaType == 'application'
-        and ctype.mediaSubtype == 'x-www-form-urlencoded'
+        ctype.mediaType == 'application' and
+        ctype.mediaSubtype == 'x-www-form-urlencoded'
     ):
         d = fileupload.parse_urlencoded(request.stream)
         d.addCallbacks(updateArgs, error)
         return d
-    elif (ctype.mediaType == 'multipart'
-          and ctype.mediaSubtype == 'form-data'):
+    elif (ctype.mediaType == 'multipart' and
+          ctype.mediaSubtype == 'form-data'):
         boundary = ctype.params.get('boundary')
         if boundary is None:
             return defer.fail(http.HTTPError(
@@ -159,7 +155,6 @@ def parsePOSTData(request, maxMem=100 * 1024, maxFields=1024,
                     ctype.mediaType, ctype.mediaSubtype))))
 
 
-
 class StopTraversal(object):
     """
     Indicates to Request._handleSegment that it should stop handling
@@ -168,7 +163,6 @@ class StopTraversal(object):
     pass
 
 
-
 class Request(http.Request):
     """
     vars:
@@ -227,18 +221,15 @@ class Request(http.Request):
         except AttributeError:
             self.serverInstance = "Unknown"
 
-
     def timeStamp(self, tag):
         self.timeStamps.append((tag, time.time(),))
 
-
     def clientCredentials(self):
         try:
             return self.chanRequest.channel.peerCredentials
         except AttributeError:
             return None
 
-
     def addResponseFilter(self, filter, atEnd=False, onlyOnce=False):
         """
         Add a response filter to this request.
@@ -257,7 +248,6 @@ class Request(http.Request):
         else:
             self.responseFilters.insert(0, filter)
 
-
     def unparseURL(self, scheme=None, host=None, port=None,
                    path=None, params=None, querystring=None, fragment=None):
         """Turn the request path into a url string. For any pieces of
@@ -289,7 +279,6 @@ class Request(http.Request):
             scheme, hostport, path,
             params, querystring, fragment))
 
-
     def _parseURL(self):
         if self.uri[0] == '/':
             # Can't use urlparse for request_uri because urlparse
@@ -330,7 +319,6 @@ class Request(http.Request):
             self.postpath = path
         # print("_parseURL", self.uri, (self.uri, self.scheme, self.host, self.path, self.params, self.querystring))
 
-
     def _schemeFromPort(self, port):
         """
         Try to determine the scheme matching the supplied server port. This is needed in case
@@ -347,8 +335,10 @@ class Request(http.Request):
         @rtype: C{bool}
         """
 
-        # from twistedcaldav.config import config
-        if hasattr(self.site, "EnableSSL") and self.site.EnableSSL:
+        if (
+            (hasattr(self.site, "EnableSSL") and self.site.EnableSSL) or
+            (hasattr(self.site, "BehindTLSProxy") and self.site.BehindTLSProxy)
+        ):
             if port == self.site.SSLPort:
                 return True
             elif port in self.site.BindSSLPorts:
@@ -356,9 +346,8 @@ class Request(http.Request):
 
         return False
 
-
     def _fixupURLParts(self):
-        hostaddr, secure = self.chanRequest.getHostInfo()
+        _ignore_hostaddr, secure = self.chanRequest.getHostInfo()
         if not self.scheme:
             self.scheme = ('http', 'https')[secure]
 
@@ -377,9 +366,9 @@ class Request(http.Request):
                 # protocol version
                 if self.clientproto >= (1, 1):
                     raise http.HTTPError(responsecode.BAD_REQUEST)
-                self.host = hostaddr.host
-                self.port = hostaddr.port
 
+                # Always require host header even for HTTP/1.0
+                raise http.HTTPError(responsecode.BAD_REQUEST)
 
     def process(self):
         "Process a request."
@@ -412,18 +401,20 @@ class Request(http.Request):
         d.callback(None)
         return d
 
-
     def _processTimeStamp(self, res):
         self.timeStamp("t-req-proc")
         return res
 
-
     def preprocessRequest(self):
         """Do any request processing that doesn't follow the normal
         resource lookup procedure. "OPTIONS *" is handled here, for
         example. This would also be the place to do any CONNECT
         processing."""
 
+        # Always reject HTTP/1.0 - no clients should use it
+        if self.clientproto == (1, 0):
+            raise http.HTTPError(responsecode.BAD_REQUEST)
+
         if self.method == "OPTIONS" and self.uri == "*":
             response = http.Response(responsecode.OK)
             response.headers.setHeader('allow', ('GET', 'HEAD', 'OPTIONS', 'TRACE'))
@@ -441,7 +432,6 @@ class Request(http.Request):
         # This is where CONNECT would go if we wanted it
         return None
 
-
     def _getChild(self, _, res, path, updatepaths=True):
         """Call res.locateChild, and pass the result on to _handleSegment."""
 
@@ -456,7 +446,6 @@ class Request(http.Request):
         else:
             return self._handleSegment(result, res, path, updatepaths)
 
-
     def _handleSegment(self, result, res, path, updatepaths):
         """Handle the result of a locateChild call done in _getChild."""
 
@@ -481,7 +470,7 @@ class Request(http.Request):
         if newpath is StopTraversal:
             # We need to rethink how to do this.
             # if newres is res:
-                return res
+            return res
             # else:
             #    raise ValueError("locateChild must not return StopTraversal with a resource other than self.")
 
@@ -510,7 +499,6 @@ class Request(http.Request):
 
     _urlsByResource = weakref.WeakKeyDictionary()
 
-
     def _rememberResource(self, resource, url):
         """
         Remember the URL of a visited resource.
@@ -519,7 +507,6 @@ class Request(http.Request):
         self._urlsByResource[resource] = url
         return resource
 
-
     def _forgetResource(self, resource, url):
         """
         Remember the URL of a visited resource.
@@ -527,7 +514,6 @@ class Request(http.Request):
         del self._resourcesByURL[url]
         del self._urlsByResource[resource]
 
-
     def urlForResource(self, resource):
         """
         Looks up the URL of the given resource if this resource was found while
@@ -551,7 +537,6 @@ class Request(http.Request):
             raise NoURLForResourceError(resource)
         return url
 
-
     def locateResource(self, url):
         """
         Looks up the resource with the given URL.
@@ -614,7 +599,6 @@ class Request(http.Request):
         d.addErrback(notFound)
         return d
 
-
     def locateChildResource(self, parent, childName):
         """
         Looks up the child resource with the given name given the parent
@@ -653,7 +637,6 @@ class Request(http.Request):
         d.addErrback(notFound)
         return d
 
-
     def _processingFailed(self, reason):
         if reason.check(http.HTTPError) is not None:
             # If the exception was an HTTPError, leave it alone
@@ -670,7 +653,6 @@ class Request(http.Request):
         d.addErrback(self._processingReallyFailed, reason)
         return d
 
-
     def _processingReallyFailed(self, reason, origReason):
         """
         An error occurred when attempting to report an error to the HTTP
@@ -703,7 +685,6 @@ class Request(http.Request):
             )
             self.chanRequest.abortConnection()
 
-
     def _cbFinishRender(self, result):
         def filterit(response, f):
             if (
@@ -732,7 +713,6 @@ class Request(http.Request):
 
         raise TypeError("html is not a resource or a response")
 
-
     def renderHTTP_exception(self, req, reason):
         log.failure("Exception rendering request: {request}", reason, request=req)
 
@@ -745,20 +725,19 @@ class Request(http.Request):
             body)
 
 
-
 class Site(object):
+
     def __init__(self, resource):
         """Initialize.
         """
         self.resource = iweb.IResource(resource)
 
-
     def __call__(self, *args, **kwargs):
         return Request(site=self, *args, **kwargs)
 
 
-
 class NoURLForResourceError(RuntimeError):
+
     def __init__(self, resource):
         RuntimeError.__init__(self, "Resource %r has no URL in this request." % (resource,))
         self.resource = resource
