@@ -5,6 +5,7 @@
 Tests for the stream implementations in L{txweb2}.
 """
 
+from hashlib import md5
 import os
 import sys
 import tempfile
@@ -12,8 +13,7 @@ import tempfile
 from zope.interface import implements
 
 from twisted.python.util import sibpath
-sibpath # sibpath is *not* unused - the doctests use it.
-from twisted.python.hashlib import md5
+sibpath  # sibpath is *not* unused - the doctests use it.
 from twisted.internet import reactor, defer, interfaces
 from twisted.trial import unittest
 from txweb2 import stream
@@ -26,9 +26,9 @@ def bufstr(data):
         raise TypeError("%s doesn't conform to the buffer interface" % (data,))
 
 
-
 class SimpleStreamTests:
     text = '1234567890'
+
     def test_split(self):
         for point in range(10):
             s = self.makeStream(0)
@@ -51,7 +51,6 @@ class SimpleStreamTests:
                 self.assertEquals(bufstr(b.read()), self.text[point + 2:8])
             self.assertEquals(b.read(), None)
 
-
     def test_read(self):
         s = self.makeStream()
         self.assertEquals(s.length, len(self.text))
@@ -71,12 +70,11 @@ class SimpleStreamTests:
         self.assertEquals(s.length, 0)
 
 
-
 class FileStreamTest(SimpleStreamTests, unittest.TestCase):
+
     def makeStream(self, *args, **kw):
         return stream.FileStream(self.f, *args, **kw)
 
-
     def setUp(self):
         """
         Create a file containing C{self.text} to be streamed.
@@ -86,7 +84,6 @@ class FileStreamTest(SimpleStreamTests, unittest.TestCase):
         f.seek(0, 0)
         self.f = f
 
-
     def test_close(self):
         s = self.makeStream()
         s.close()
@@ -96,7 +93,6 @@ class FileStreamTest(SimpleStreamTests, unittest.TestCase):
         # would raise exception if f is closed
         self.f.seek(0, 0)
 
-
     def test_read2(self):
         s = self.makeStream(0)
         s.CHUNK_SIZE = 6
@@ -115,8 +111,7 @@ class FileStreamTest(SimpleStreamTests, unittest.TestCase):
         s = self.makeStream(0, 20)
         self.assertEquals(s.length, 20)
         self.assertEquals(bufstr(s.read()), self.text)
-        self.assertRaises(RuntimeError, s.read) # ran out of data
-
+        self.assertRaises(RuntimeError, s.read)  # ran out of data
 
 
 class MMapFileStreamTest(SimpleStreamTests, unittest.TestCase):
@@ -126,7 +121,6 @@ class MMapFileStreamTest(SimpleStreamTests, unittest.TestCase):
     def makeStream(self, *args, **kw):
         return stream.FileStream(self.f, *args, **kw)
 
-
     def setUp(self):
         """
         Create a file containing C{self.text}, which should be long enough to
@@ -137,7 +131,6 @@ class MMapFileStreamTest(SimpleStreamTests, unittest.TestCase):
         f.seek(0, 0)
         self.f = f
 
-
     def test_mmapwrapper(self):
         self.assertRaises(TypeError, stream.mmapwrapper)
         self.assertRaises(TypeError, stream.mmapwrapper, offset=0)
@@ -147,18 +140,16 @@ class MMapFileStreamTest(SimpleStreamTests, unittest.TestCase):
         test_mmapwrapper.skip = 'mmap not supported here'
 
 
-
 class MemoryStreamTest(SimpleStreamTests, unittest.TestCase):
+
     def makeStream(self, *args, **kw):
         return stream.MemoryStream(self.text, *args, **kw)
 
-
     def test_close(self):
         s = self.makeStream()
         s.close()
         self.assertEquals(s.length, 0)
 
-
     def test_read2(self):
         self.assertRaises(ValueError, self.makeStream, 0, 20)
 
@@ -191,39 +182,33 @@ class TestBufferedStream(unittest.TestCase):
         s = stream.MemoryStream(self.data)
         self.s = stream.BufferedStream(s)
 
-
     def _cbGotData(self, data, expected):
         self.assertEqual(data, expected)
 
-
     def test_readline(self):
         """Test that readline reads a line."""
         d = self.s.readline()
         d.addCallback(self._cbGotData, 'I was angry with my friend:\r\n')
         return d
 
-
     def test_readlineWithSize(self):
         """Test the size argument to readline"""
         d = self.s.readline(size=5)
         d.addCallback(self._cbGotData, 'I was')
         return d
 
-
     def test_readlineWithBigSize(self):
         """Test the size argument when it's bigger than the length of the line."""
         d = self.s.readline(size=40)
         d.addCallback(self._cbGotData, 'I was angry with my friend:\r\n')
         return d
 
-
     def test_readlineWithZero(self):
         """Test readline with size = 0."""
         d = self.s.readline(size=0)
         d.addCallback(self._cbGotData, '')
         return d
 
-
     def test_readlineFinished(self):
         """Test readline on a finished stream."""
         nolines = len(self.data.split('\r\n'))
@@ -233,12 +218,10 @@ class TestBufferedStream(unittest.TestCase):
         d.addCallback(self._cbGotData, '')
         return d
 
-
     def test_readlineNegSize(self):
         """Ensure that readline with a negative size raises an exception."""
         self.assertRaises(ValueError, self.s.readline, size=-1)
 
-
     def test_readlineSizeInDelimiter(self):
         """
         Test behavior of readline when size falls inside the
@@ -249,14 +232,12 @@ class TestBufferedStream(unittest.TestCase):
         d.addCallback(lambda _: self.s.readline())
         d.addCallback(self._cbGotData, "\nI told my wrath, my wrath did end.\r\n")
 
-
     def test_readExactly(self):
         """Make sure readExactly with no arg reads all the data."""
         d = self.s.readExactly()
         d.addCallback(self._cbGotData, self.data)
         return d
 
-
     def test_readExactlyLimited(self):
         """
         Test readExactly with a number.
@@ -265,7 +246,6 @@ class TestBufferedStream(unittest.TestCase):
         d.addCallback(self._cbGotData, self.data[:10])
         return d
 
-
     def test_readExactlyBig(self):
         """
         Test readExactly with a number larger than the size of the
@@ -275,7 +255,6 @@ class TestBufferedStream(unittest.TestCase):
         d.addCallback(self._cbGotData, self.data)
         return d
 
-
     def test_read(self):
         """
         Make sure read() also functions. (note that this test uses
@@ -285,7 +264,6 @@ class TestBufferedStream(unittest.TestCase):
         self.assertEqual(str(self.s.read()), self.data)
 
 
-
 class TestStreamer:
     implements(stream.IStream, stream.IByteStream)
 
@@ -297,20 +275,17 @@ class TestStreamer:
     def __init__(self, list):
         self.list = list
 
-
     def read(self):
         self.readCalled += 1
         if self.list:
             return self.list.pop(0)
         return None
 
-
     def close(self):
         self.closeCalled += 1
         self.list = []
 
 
-
 class FallbackSplitTest(unittest.TestCase):
 
     def test_split(self):
@@ -323,7 +298,6 @@ class FallbackSplitTest(unittest.TestCase):
         d.addCallback(self._cbSplit, left, right)
         return d
 
-
     def _cbSplit(self, result, left, right):
         self.assertEquals(bufstr(result), 'e')
         self.assertEquals(left.read(), None)
@@ -332,7 +306,6 @@ class FallbackSplitTest(unittest.TestCase):
         self.assertEquals(bufstr(right.read()), 'ijkl')
         self.assertEquals(right.read(), None)
 
-
     def test_split2(self):
         s = TestStreamer(['abcd', defer.succeed('efgh'), 'ijkl'])
         left, right = stream.fallbackSplit(s, 4)
@@ -347,7 +320,6 @@ class FallbackSplitTest(unittest.TestCase):
         self.assertEquals(bufstr(right.read()), 'ijkl')
         self.assertEquals(right.read(), None)
 
-
     def test_splitsplit(self):
         s = TestStreamer(['abcd', defer.succeed('efgh'), 'ijkl'])
         left, right = stream.fallbackSplit(s, 5)
@@ -368,7 +340,6 @@ class FallbackSplitTest(unittest.TestCase):
         self.assertEquals(bufstr(right.read()), 'ijkl')
         self.assertEquals(right.read(), None)
 
-
     def test_closeboth(self):
         s = TestStreamer(['abcd', defer.succeed('efgh'), 'ijkl'])
         left, right = stream.fallbackSplit(s, 5)
@@ -380,7 +351,6 @@ class FallbackSplitTest(unittest.TestCase):
         self.assertEquals(s.readCalled, 0)
         self.assertEquals(s.closeCalled, 1)
 
-
     def test_closeboth_rev(self):
         s = TestStreamer(['abcd', defer.succeed('efgh'), 'ijkl'])
         left, right = stream.fallbackSplit(s, 5)
@@ -392,7 +362,6 @@ class FallbackSplitTest(unittest.TestCase):
         self.assertEquals(s.readCalled, 0)
         self.assertEquals(s.closeCalled, 1)
 
-
     def test_closeleft(self):
         s = TestStreamer(['abcd', defer.succeed('efgh'), 'ijkl'])
         left, right = stream.fallbackSplit(s, 5)
@@ -401,13 +370,11 @@ class FallbackSplitTest(unittest.TestCase):
         d.addCallback(self._cbCloseleft, right)
         return d
 
-
     def _cbCloseleft(self, result, right):
         self.assertEquals(bufstr(result), 'fgh')
         self.assertEquals(bufstr(right.read()), 'ijkl')
         self.assertEquals(right.read(), None)
 
-
     def test_closeright(self):
         s = TestStreamer(['abcd', defer.succeed('efgh'), 'ijkl'])
         left, right = stream.fallbackSplit(s, 3)
@@ -419,13 +386,11 @@ class FallbackSplitTest(unittest.TestCase):
         self.assertEquals(s.closeCalled, 1)
 
 
-
 class ProcessStreamerTest(unittest.TestCase):
 
     if interfaces.IReactorProcess(reactor, None) is None:
         skip = "Platform lacks spawnProcess support, can't test process streaming."
 
-
     def runCode(self, code, inputStream=None):
         if inputStream is None:
             inputStream = stream.MemoryStream("")
@@ -433,78 +398,78 @@ class ProcessStreamerTest(unittest.TestCase):
                                       [sys.executable, "-u", "-c", code],
                                       os.environ)
 
-
     def test_output(self):
         p = self.runCode("import sys\nfor i in range(100): sys.stdout.write('x' * 1000)")
         l = []
         d = stream.readStream(p.outStream, l.append)
+
         def verify(_):
             self.assertEquals("".join(l), ("x" * 1000) * 100)
         d2 = p.run()
         return d.addCallback(verify).addCallback(lambda _: d2)
 
-
     def test_errouput(self):
         p = self.runCode("import sys\nfor i in range(100): sys.stderr.write('x' * 1000)")
         l = []
         d = stream.readStream(p.errStream, l.append)
+
         def verify(_):
             self.assertEquals("".join(l), ("x" * 1000) * 100)
         p.run()
         return d.addCallback(verify)
 
-
     def test_input(self):
         p = self.runCode("import sys\nsys.stdout.write(sys.stdin.read())",
                          "hello world")
         l = []
         d = stream.readStream(p.outStream, l.append)
         d2 = p.run()
+
         def verify(_):
             self.assertEquals("".join(l), "hello world")
             return d2
         return d.addCallback(verify)
 
-
     def test_badexit(self):
         p = self.runCode("raise ValueError")
         l = []
         from twisted.internet.error import ProcessTerminated
+
         def verify(_):
             self.assertEquals(l, [1])
             self.assert_(p.outStream.closed)
             self.assert_(p.errStream.closed)
         return p.run().addErrback(lambda _: _.trap(ProcessTerminated) and l.append(1)).addCallback(verify)
 
-
     def test_inputerror(self):
         p = self.runCode("import sys\nsys.stdout.write(sys.stdin.read())",
                          TestStreamer(["hello", defer.fail(ZeroDivisionError())]))
         l = []
         d = stream.readStream(p.outStream, l.append)
         d2 = p.run()
+
         def verify(_):
             self.assertEquals("".join(l), "hello")
             return d2
+
         def cbVerified(ignored):
             excs = self.flushLoggedErrors(ZeroDivisionError)
             self.assertEqual(len(excs), 1)
         return d.addCallback(verify).addCallback(cbVerified)
 
-
     def test_processclosedinput(self):
         p = self.runCode("import sys; sys.stdout.write(sys.stdin.read(3));" +
                          "sys.stdin.close(); sys.stdout.write('def')",
                          "abc123")
         l = []
         d = stream.readStream(p.outStream, l.append)
+
         def verify(_):
             self.assertEquals("".join(l), "abcdef")
         d2 = p.run()
         return d.addCallback(verify).addCallback(lambda _: d2)
 
 
-
 class AdapterTestCase(unittest.TestCase):
 
     def test_adapt(self):
@@ -518,41 +483,37 @@ class AdapterTestCase(unittest.TestCase):
             self.assertEquals(s.read(), None)
 
 
-
 class ReadStreamTestCase(unittest.TestCase):
 
-
     def test_pull(self):
         l = []
         s = TestStreamer(['abcd', defer.succeed('efgh'), 'ijkl'])
         return stream.readStream(s, l.append).addCallback(
             lambda _: self.assertEquals(l, ["abcd", "efgh", "ijkl"]))
 
-
     def test_pullFailure(self):
         l = []
         s = TestStreamer(['abcd', defer.fail(RuntimeError()), 'ijkl'])
+
         def test(result):
             result.trap(RuntimeError)
             self.assertEquals(l, ["abcd"])
         return stream.readStream(s, l.append).addErrback(test)
 
-
     def test_pullException(self):
         class Failer:
+
             def read(self):
                 raise RuntimeError
         return stream.readStream(Failer(), lambda _: None).addErrback(
             lambda _: _.trap(RuntimeError))
 
-
     def test_processingException(self):
         s = TestStreamer(['abcd', defer.succeed('efgh'), 'ijkl'])
         return stream.readStream(s, lambda x: 1 / 0).addErrback(
             lambda _: _.trap(ZeroDivisionError))
 
 
-
 class ProducerStreamTestCase(unittest.TestCase):
 
     def test_failfinish(self):
@@ -567,7 +528,6 @@ class ProducerStreamTestCase(unittest.TestCase):
         return d
 
 
-
 class CompoundStreamTest:
     """
     CompoundStream lets you combine many streams into one continuous stream.
@@ -643,28 +603,25 @@ class CompoundStreamTest:
     """
 
 
-
 class AsynchronousDummyStream(object):
     """
     An L{IByteStream} implementation which always returns a
     L{defer.Deferred} from C{read} and lets an external driver fire
     them.
     """
+
     def __init__(self):
         self._readResults = []
 
-
     def read(self):
         result = defer.Deferred()
         self._readResults.append(result)
         return result
 
-
     def _write(self, bytes):
         self._readResults.pop(0).callback(bytes)
 
 
-
 class MD5StreamTest(unittest.TestCase):
     """
     Tests for L{stream.MD5Stream}.
@@ -687,7 +644,6 @@ class MD5StreamTest(unittest.TestCase):
 
         self.assertEquals(self.digest, md5Stream.getMD5())
 
-
     def test_asynchronous(self):
         """
         L{stream.MD5Stream} also supports L{IByteStream} providers which return
@@ -714,7 +670,6 @@ class MD5StreamTest(unittest.TestCase):
 
         return result
 
-
     def test_getMD5FailsBeforeClose(self):
         """
         L{stream.MD5Stream.getMD5} raises L{RuntimeError} if called before
@@ -724,7 +679,6 @@ class MD5StreamTest(unittest.TestCase):
         md5Stream = stream.MD5Stream(dataStream)
         self.assertRaises(RuntimeError, md5Stream.getMD5)
 
-
     def test_initializationFailsWithoutStream(self):
         """
         L{stream.MD5Stream.__init__} raises L{ValueError} if passed C{None} as
@@ -732,7 +686,6 @@ class MD5StreamTest(unittest.TestCase):
         """
         self.assertRaises(ValueError, stream.MD5Stream, None)
 
-
     def test_readAfterClose(self):
         """
         L{stream.MD5Stream.read} raises L{RuntimeError} if called after
@@ -744,7 +697,6 @@ class MD5StreamTest(unittest.TestCase):
         self.assertRaises(RuntimeError, md5Stream.read)
 
 
-
 __doctests__ = ['txweb2.test.test_stream']
 # TODO:
 # CompoundStreamTest
